/*
 * Copyright (C) 2001-2002 Peter J Jones (pjones@pmade.org)
 * All Rights Reserved
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/** @file
 * This file contains the implementation of the
 * sord::directory::const_iterator class.
**/

// sord includes
#include "sord/directory.h"

// load the right pimpl
#ifdef SORD_UNIX
#   include "directory_unix.h"
#elif  SORD_WIN32
#   include "directory_win32.h"
#else
#   error "sord does not know about your platform"
#endif

// standard includes
#include <stdexcept>

namespace {
    const char const_bad_iterator_message[] = "sord: use of invaild (null) directory iterator";
}

//####################################################################
sord::directory::const_iterator::const_iterator (dimpl *parent) 
    : parent_(parent)
{ }
//####################################################################
void sord::directory::const_iterator::set_parent (dimpl *parent) {
    parent_ = parent;
}
//####################################################################
const sord::directory::const_iterator::reference
    sord::directory::const_iterator::operator* (void) const
{
    if (!parent_) throw std::runtime_error(const_bad_iterator_message);
    return parent_->get_entry();
}
//####################################################################
const sord::directory::const_iterator::pointer
    sord::directory::const_iterator::operator-> (void) const
{
    if (!parent_) throw std::runtime_error(const_bad_iterator_message);
    return &(parent_->get_entry());
}
//####################################################################
sord::directory::const_iterator&
    sord::directory::const_iterator::operator++ (void)
{
    if (!parent_) throw std::runtime_error(const_bad_iterator_message);
    if (!parent_->next()) parent_ = 0;
    return *this;
}

namespace sord {
    //####################################################################
    bool operator== (const directory::const_iterator &lhs,
	const directory::const_iterator &rhs)
    {
	return lhs.parent_ == rhs.parent_;
    }
    //####################################################################
    bool operator!= (const directory::const_iterator &lhs,
	const directory::const_iterator &rhs) 
    {
	return !(lhs == rhs);
    }
}
