/*
 * Copyright (C) 2001-2002 Peter J Jones (pjones@pmade.org)
 * All Rights Reserved
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/** @file
 * This file contains the definition of the win32 directory impl class.
**/

#ifndef __sord_directory_win32_h__
#define __sord_directory_win32_h__

// sord includes
#include "sord/directory.h"
#include "sord/entry.h"

// standard includes
#include <string>
#include <vector>

// system includes
#include <windows.h>

namespace sord {

    class dimpl {
    public:
	explicit dimpl (const char *path);
	~dimpl (void);

	directory::const_iterator& begin (void);
	directory::const_iterator& end (void);
	const char* get_name (void) const;
	bool is_vaild (void) const;
	bool next (void);
	entry& get_entry (void);
    private:
	void close (void);
	bool reset (void);

	std::string path_;
	bool vector_iterator_ready_;
	HANDLE dir_pointer_;
	directory::const_iterator dir_iterator_;
	directory::const_iterator end_iterator_;
	bool moved_past_begin_;

	bool is_vaild_;
	bool last_entry_read_;
	WIN32_FIND_DATA finddata_;
	std::vector<entry> dirs_;
	std::vector<entry>::const_iterator vector_iterator_;
	entry entry_;
    };

}
#endif
