/*
 * Copyright (C) 2001-2002 Peter J Jones (pjones@pmade.org)
 * All Rights Reserved
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/** @file
 * This file contains the implementation of the sord::entry class.
**/

// sord includes
#include "sord/entry.h"

// load the correct pimpl
#ifdef SORD_UNIX
#   include "entry_unix.h"
#elif  SORD_WIN32
#   include "entry_win32.h"
#else
#   error "sord does not know your platform"
#endif

// standard includes
#include <algorithm>

/*
 * Construction and Destruction Functions
 */

//####################################################################
sord::entry::entry (const char *name) {
    pimpl_ = new pimpl(name);
}
//####################################################################
sord::entry::entry (const entry &other) {
    pimpl_ = new pimpl(*(other.pimpl_));
}
//####################################################################
sord::entry::entry (void) {
    pimpl_ = new pimpl(0);
}
//####################################################################
sord::entry& sord::entry::operator= (const entry &other) {
    sord::entry tmp(other);
    swap(tmp);
    return *this;
}
//####################################################################
void sord::entry::swap (entry &other) {
    std::swap(pimpl_, other.pimpl_);
}
//####################################################################
sord::entry::~entry (void) {
    delete pimpl_;
}


/*
 * Accessor Functions
 */

//####################################################################
const char* sord::entry::get_name (void) const {
    if (!pimpl_->prepare(&(pimpl_->common_.name_))) return 0;
    return pimpl_->common_.name_.c_str();
}
//####################################################################
const char* sord::entry::get_path (void) const {
    if (!pimpl_->prepare(&(pimpl_->common_.path_))) return 0;
    return pimpl_->common_.path_.c_str();
}
//####################################################################
sord::entry::type sord::entry::get_type (void) const {
    if (!pimpl_->prepare(&(pimpl_->common_.type_))) return type_regular;
    return pimpl_->common_.type_;
}
//####################################################################
sord::entry::mode_type sord::entry::get_mode (void) const {
    if (!pimpl_->prepare(&(pimpl_->common_.mode_))) return 0;
    return pimpl_->common_.mode_;
}
//####################################################################
sord::entry::time_type sord::entry::get_mtime (void) const {
    if (!pimpl_->prepare(&(pimpl_->common_.mtime_))) return 0;
    return pimpl_->common_.mtime_;
}
//####################################################################
sord::entry::time_type sord::entry::get_atime (void) const {
    if (!pimpl_->prepare(&(pimpl_->common_.atime_))) return 0;
    return pimpl_->common_.atime_;
}
//####################################################################
sord::entry::size_type sord::entry::get_size (void) const {
    if (!pimpl_->prepare(&(pimpl_->common_.size_))) return 0;
    return pimpl_->common_.size_;
}
//####################################################################
bool sord::entry::operator! (void) const {
    return !pimpl_->prepare(&(pimpl_->common_.is_valid_));
}

/*
 * Private Functions
 */

//####################################################################
void sord::entry::set (const void *data, const char *path) {
    pimpl_->set(data, path);
}
