/*
 * Copyright (C) 2001-2002 Peter J Jones (pjones@pmade.org)
 * All Rights Reserved
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/** @file
 * This file contains some hacks to make sord portable to diffierent
 * flavors of Unix
**/

#ifndef __sord_unixhacks_h__
#define __sord_unixhacks_h__

// standard includes
#include <cstdlib>
#include <cstring>
#include <new>

// system includes
#include <sys/types.h>
#include <dirent.h>
#include <unistd.h>

// what a mess ;-p
namespace {
    inline dirent* create_dirent (const char *path);
    inline void destroy_dirent (dirent *heap_dirent);
    inline unsigned long get_dirent_size (const char *path);

    dirent* create_dirent (const char *path)
    {
#	if defined(SORD_USE_READDIR_R)
	    dirent* new_dirent = reinterpret_cast<dirent*>
		(std::malloc(get_dirent_size(path)));

	    if (!new_dirent) throw std::bad_alloc();
	    std::memset(new_dirent, 0, get_dirent_size(path));

	    return new_dirent;

#	else
	    return 0;

#	endif
    }

    void destroy_dirent (dirent *heap_dirent)
    {
#	if defined(SORD_USE_READDIR_R)
	    std::free(heap_dirent);
#	endif
    }

    unsigned long get_dirent_size (const char *path)
    {
#	if defined(__FreeBSD__)
	    path = 0; // kill unused warning
	    return sizeof(dirent);

#	elif defined(sun)
	    return sizeof(dirent) + pathconf(path, _PC_NAME_MAX) + 1;

#	elif defined(__CYGWIN32__)
	    return sizeof(dirent) + 257;
#	else
	    path = 0; // kill unused warning
	    return sizeof(dirent) + NAME_MAX + 1;

#	endif
    }

} // end anonymous namespace

#endif
