/*
 * Copyright (C) 2001-2002 Peter J Jones (pjones@pmade.org)
 * All Rights Reserved
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sord/entry.h>
#include <iostream>

int main (int argc, char *argv[]) {
    if (argc < 2) {
	std::cerr << "Usage: " << argv[0] << " file [file ...]\n";
	return 1;
    }

    for (int i=1; i<argc; ++i) {
	std::cout << argv[i] << " ";
	sord::entry e(argv[i]);

	if (!e) {
	    std::cout << "valid=0" << std::endl;
	    continue;
	} else {
	    std::cout << "valid=1 ";
	}

	std::cout << "name="	<< e.get_name()	    << " ";
	std::cout << "path="	<< e.get_path()	    << " ";
	std::cout << "type="	<< e.get_type()	    << " ";
	std::cout << "mtime="	<< e.get_mtime()    << " ";
	std::cout << "atime="	<< e.get_atime()    << " ";
	std::cout << "size="	<< e.get_size()	    << " ";

	sord::entry::mode_type mode = e.get_mode();
	std::cout << "mode=";
	std::cout << ((mode & sord::entry::mode_user_read)	? 'r' : '-');
	std::cout << ((mode & sord::entry::mode_user_write)	? 'w' : '-');
	std::cout << ((mode & sord::entry::mode_user_execute)	? 'x' : '-');
	std::cout << ((mode & sord::entry::mode_group_read)	? 'r' : '-');
	std::cout << ((mode & sord::entry::mode_group_write)	? 'w' : '-');
	std::cout << ((mode & sord::entry::mode_group_execute)	? 'x' : '-');
	std::cout << ((mode & sord::entry::mode_other_read)	? 'r' : '-');
	std::cout << ((mode & sord::entry::mode_other_write)	? 'w' : '-');
	std::cout << ((mode & sord::entry::mode_other_execute)	? 'x' : '-');
	std::cout << " ";

	std::cout << std::endl;
    }

    return 0;
}
