#! /usr/bin/perl -w
######################################################################
# Copyright (C) 2001-2002 Peter J Jones (pjones@pmade.org)
# All Rights Reserved
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 3. Neither the name of the Author nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS''
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
# USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
# AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
######################################################################

require 'test_harness.pl';
my $program = "./entry_test";

$program = "$program.exe" if ($^O =~ /win/i && $^O !~ /darwin/i);
$program = "$program.app" if ($^O =~ /darwin/i); 

my @filenames = qw(test_a test_b test_c test_d);
my @dirnames  = qw(test_dir_a test_dir_b);

srand;

foreach (@filenames) {
    open(FILE, ">$_") or next;
    print FILE "$_\n", time, localtime, "\n";
    print FILE $_ foreach (1..rand(100));
    close(FILE);
}

mkdir $_, 0777 foreach (@dirnames);

chomp(@output = `$program @filenames @dirnames`);
@all = (@filenames, @dirnames);
for ($i=0; $i<@all; ++$i) {
    check_output($output[$i], $all[$i]);
}

unlink(@filenames);
rmdir($_) foreach (@dirnames);

sub check_output {
    my ($output, $entry) = @_;
    test_start($entry);

    unless ($output =~ s/^$entry\s+//) {
	test_failed();
	return;
    } else {
	test_passed();
    }

    my %attr = map{split(/=/)} split(/\s+/, $output);

    test_start("$entry valid");
    if ($attr{valid} == 1) {
	test_passed();
    } else {
	test_failed();
    }

    test_start("$entry size");
    if ($attr{size} == (stat($entry))[7]) {
	test_passed();
    } else {
	test_failed();
    }

    test_start("$entry mode");
    my $mode_str = '';
    my $mode = (stat(_))[2];

    $mode_str .= $mode & 0000700 & 0000400 ? "r" : "-";
    $mode_str .= $mode & 0000700 & 0000200 ? "w" : "-";
    $mode_str .= $mode & 0000700 & 0000100 ? "x" : "-";
    $mode_str .= $mode & 0000070 & 0000040 ? "r" : "-";
    $mode_str .= $mode & 0000070 & 0000020 ? "w" : "-";
    $mode_str .= $mode & 0000070 & 0000010 ? "x" : "-";
    $mode_str .= $mode & 0000007 & 0000004 ? "r" : "-";
    $mode_str .= $mode & 0000007 & 0000002 ? "w" : "-";
    $mode_str .= $mode & 0000007 & 0000001 ? "x" : "-";
    
    if ($attr{mode} eq $mode_str) {
	test_passed();
    } else {
	test_failed();
    }

    test_start("$entry time");
    if ($attr{mtime} == (stat(_))[9] and $attr{atime} == (stat(_))[8]) {
	test_passed();
    } else {
	test_failed();
    }
}
