#! /bin/sh

PROJECT=sord
VERSION_FILE=docs/VERSION
VERSION=`head -1 $VERSION_FILE | perl -pe 's/\s+.*//'`
CVSTAG=`head -1 $VERSION_FILE  | perl -pe 's/^[\d.]+\s+\d+\s+([^\s]+).*$/$1/'`
DESTDIR="$PROJECT-$VERSION"
DOCBOOK_XSL=/home/pjones/develop/pmade.org/config/readme2html.xsl
DEFAULT_FDP="${HOME}/develop/freebsd-doc-project"
W3M="w3m -cols 76 -dump"
XSLT="xsltproc --param heading-size \"'+1'\" $DOCBOOK_XSL"

echo "===> Making $PROJECT release (Version == $VERSION) (CVSTAG == $CVSTAG)"

# checkout the correct tag/branch from CVS
cvs -q checkout -r $CVSTAG -d $DESTDIR $PROJECT

# remove the CVS directories
find $DESTDIR -name CVS -type d | xargs rm -rf

# build the doxygen reference manual
perl -i -pe "s/^\s*(PROJECT_NUMBER\s*=).*\$/\${1}$VERSION/" $DESTDIR/docs/doxygen/doxyfile
(cd $DESTDIR/docs/doxygen; doxygen doxyfile)

# build the programming manual
if test "${FREEBSDDOC}x" = "x" -a -d $DEFAULT_FDP ; then
    export FREEBSDDOC=$DEFAULT_FDP
fi
(
    cd $DESTDIR/docs/manual
    make html
    $W3M one-html/manual.html > ../manual.txt
    make clean
)

# create the REAME, INSTALL and like files
TMPFILE=tmp.html
(
    cd $DESTDIR

    $XSLT docs/manual/readme.xml > $TMPFILE
    $W3M $TMPFILE > README

    $XSLT docs/manual/build.xml > $TMPFILE
    $W3M $TMPFILE > INSTALL

    $XSLT docs/manual/todo.xml > $TMPFILE
    $W3M $TMPFILE > docs/TODO

    $XSLT docs/manual/credits.xml > $TMPFILE
    $W3M $TMPFILE > docs/CREDITS

    rm $TMPFILE
)

# create a tarball
tar czvf ${DESTDIR}.tar.gz $DESTDIR

# remove the directory that CVS created
#rm -rf $DESTDIR
