#include "pch.h"

extern const GUID CLSID_XVID;

CUnknown *WINAPI CreateXviDCodec(LPUNKNOWN pUnk, HRESULT *phr);

//----------------------------
                              //Major/Minor type
static const AMOVIESETUP_MEDIATYPE pin_types_xvid_in = {
   &MEDIATYPE_Video, &MEDIASUBTYPE_NULL
}, pin_types_xvid_out = {
   &MEDIATYPE_Video, &CLSID_XVID
};

//----------------------------

static const AMOVIESETUP_PIN pins_xvid[] = {
   {
      L"Input",           // String pin name
      false,              // Is it rendered
      false,              // Is it an output
      false,              // Allowed none
      false,              // Allowed many
      &CLSID_NULL,        // Connects to filter
      L"Output",          // Connects to pin
      1,                  // Number of types
      &pin_types_xvid_in // The pin details
   },
   { 
      L"Output",          // String pin name
      false,              // Is it rendered
      true,               // Is it an output
      false,              // Allowed none
      false,              // Allowed many
      &CLSID_NULL,        // Connects to filter
      L"Input",           // Connects to pin
      1,                  // Number of types
      &pin_types_xvid_out // The pin details
   }
};

static const AMOVIESETUP_FILTER fil_xvid = {
   &CLSID_XVID,         // Filter CLSID
   L"XviD video encoder",         // Filter name
   MERIT_DO_NOT_USE,     // Its merit
   2,  // Number of pins
   pins_xvid          // Pin details
};

CFactoryTemplate g_Templates[] = {
   {
      L"XviD video encoder",
      &CLSID_XVID,
      CreateXviDCodec,
      NULL,
      &fil_xvid
   },
};

int g_cTemplates = sizeof(g_Templates) / sizeof(g_Templates[0]);

//----------------------------

static const REGFILTERPINS sudPinReg[] = {
   // Input pin.
   { 
      0,
      false, // Rendered?
      false, // Output?
      false, // Zero?
      false, // Many?
      0, 0, 
      1, &pin_types_xvid_in  // Media types.
   },
      // Output pin.
   {
      0,
      false, // Rendered?
      true, // Output?
      false, // Zero?
      false, // Many?
      0, 0, 
      1, &pin_types_xvid_out      // Media types.
   }
};


static const REGFILTER2 rf2FilterReg = {
   1, MERIT_DO_NOT_USE, 2, sudPinReg
};

//----------------------------
// Handles filter registry and unregistry
STDAPI DllRegisterServer(){
   HRESULT hr = AMovieDllRegisterServer2(true);
   if(FAILED(hr))
      return hr;

   IFilterMapper2 *pFM2 = NULL;
   hr = CoCreateInstance(CLSID_FilterMapper2, NULL, CLSCTX_INPROC_SERVER, IID_IFilterMapper2, (void**)&pFM2);
   if(SUCCEEDED(hr)){
      hr = pFM2->RegisterFilter(
         CLSID_XVID,                // Filter CLSID.
         L"XviD encoder",                       // Filter name.
         NULL,                            // Device moniker. 
         &CLSID_VideoCompressorCategory,  // Video compressor category.
         L"XviD encoder",                       // Instance data.
         &rf2FilterReg                    // Filter information.
         );
      pFM2->Release();
   }
   return hr;
}

//----------------------------

STDAPI DllUnregisterServer(){
   HRESULT hr = AMovieDllRegisterServer2(false);
   if(FAILED(hr))
      return hr;
   IFilterMapper2 *pFM2 = NULL;
   hr = CoCreateInstance(CLSID_FilterMapper2, NULL, CLSCTX_INPROC_SERVER, IID_IFilterMapper2, (void**)&pFM2);
   if(SUCCEEDED(hr)){
      hr = pFM2->UnregisterFilter(&CLSID_VideoCompressorCategory, L"XviD encoder", CLSID_XVID);
      pFM2->Release();
   }
   return hr;
}

//----------------------------

extern"C"
BOOL WINAPI DllEntryPoint(HINSTANCE, ULONG, void*);

//----------------------------

BOOL APIENTRY DllMain(HANDLE hModule, DWORD dwReason, LPVOID lpReserved){
   return DllEntryPoint((HINSTANCE)(hModule), dwReason, lpReserved);
}

//----------------------------
