#include "pch.h"

#include "..\src\xvid.h"             //XviD API

typedef unsigned long dword;

//----------------------------

#define QUALITY 6
#define MAX_KEYFRAME_INTERVAL 5 //maximum distance between keyframes (in seconds)
#define INIT_QUANTIZIER 4     //initial wuantizier value
#define MAX_QUANTIZIER 256
//#define USE_QUARTER_PEL       //use quarter pixel ME (frames will encoded as quarterpel)
//#define USE_GMC               //use global motion compensation (frames will be checked for gmc suitability)

//#define OWN_QAULITY

//#define DEBUG_PRINT

//#define DEBUG_NO_ENCODE

//----------------------------

/*
   DECLARE_IUNKNOWN;

   STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void **ppv){
      if(riid == IID_ISpecifyPropertyPages)
         return GetInterface(static_cast<ISpecifyPropertyPages*>(this), ppv);
      if(riid == CLSID_AudioSettings)
         return GetInterface(static_cast<C_audio_settings*>(this), ppv);
      return CTransformFilter::NonDelegatingQueryInterface(riid, ppv);
   }
*/
//----------------------------

extern const GUID CLSID_XVID;
const GUID CLSID_XVID = {
   mmioFOURCC('X','V','I','D'), 0x0000, 0x0010, { 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71 }
};

//----------------------------
                              //specialized output pin, supporting video compression quality setting
class C_xvid_out_pin: public CTransformOutputPin, public IAMVideoCompression{
public:
   C_xvid_out_pin(CTransformFilter *fil, HRESULT *phr):
      CTransformOutputPin("Transform output pin", fil, phr, L"XForm Out")
   {}

   DECLARE_IUNKNOWN;

   STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void **ppv){
      if(riid == IID_IAMVideoCompression)
         return GetInterface(static_cast<IAMVideoCompression*>(this), ppv);
      return CTransformOutputPin::NonDelegatingQueryInterface(riid, ppv);
   }

//----------------------------

   STDMETHODIMP put_KeyFrameRate(long KeyFrameRate){ return E_NOTIMPL; }
   STDMETHODIMP get_KeyFrameRate(long *pKeyFrameRate){ return E_NOTIMPL; }
   STDMETHODIMP put_PFramesPerKeyFrame(long PFramesPerKeyFrame){ return E_NOTIMPL; }
   STDMETHODIMP get_PFramesPerKeyFrame(long *pPFramesPerKeyFrame){ return E_NOTIMPL; }
   STDMETHODIMP put_Quality(double Quality){ return E_NOTIMPL; }
   STDMETHODIMP get_Quality(double *Quality){ return E_NOTIMPL; }
   STDMETHODIMP put_WindowSize(DWORDLONG WindowSize){ return E_NOTIMPL; }
   STDMETHODIMP get_WindowSize(DWORDLONG *WindowSize){ return E_NOTIMPL; }
   STDMETHODIMP OverrideKeyFrame(long FrameNumber){ return E_NOTIMPL; }
   STDMETHODIMP OverrideFrameSize(long FrameNumber, long Size){ return E_NOTIMPL; }

   STDMETHODIMP GetInfo(WCHAR *pszVersion, int *pcbVersion, LPWSTR pszDescription, int *pcbDescription, long *pDefaultKeyFrameRate, long *pDefaultPFramesPerKey,
      double *pDefaultQuality, long *pCapabilities){

      //pszVersion = L"Version 2.0";
      if(pcbVersion) *pcbVersion = 0;
      if(pcbDescription) *pcbDescription = 0;
      if(pDefaultKeyFrameRate) *pDefaultKeyFrameRate = -1;
      if(pDefaultPFramesPerKey) *pDefaultKeyFrameRate = -1;
      if(pDefaultQuality) *pDefaultQuality = -1;
      if(pCapabilities)
         *pCapabilities = CompressionCaps_CanCrunch;

      return S_OK;
   }
};

//----------------------------

class CXvidEncoder: public CTransformFilter{
   xvid_enc_create_t enc_create;
   xvid_enc_frame_t m_frame;
   xvid_enc_stats_t enc_stats;
#ifndef OWN_QAULITY
   xvid_plugin_single_t single;
   xvid_enc_plugin_t plugins[7];
#endif
   int frame_index;
   int total_size;            //size of output data (bytes)
   int curr_quant;

   //dword bitrate;             //requested bitrate
   float fps;                 //original fps

   int EncodeMain(bool &is_keyframe, dword frm_time);

//----------------------------
// Fully override getting pin, create our specialized output pin.
   CBasePin *GetPin(int n){

                              //create pins if necessary
      if(!m_pInput){
         HRESULT hr = S_OK;
         m_pInput = new CTransformInputPin(NAME("Transform input pin"), this, &hr, L"XForm In"); 
         
         //  Can't fail
         ASSERT(SUCCEEDED(hr));
         if (m_pInput == NULL) {
            return NULL;
         }
         //m_pOutput = new CTransformOutputPin(NAME("Transform output pin"), this, &hr, L"XForm Out");
         m_pOutput = new C_xvid_out_pin(this, &hr);
         
         // Can't fail
         ASSERT(SUCCEEDED(hr));
         if(m_pOutput == NULL){
            delete m_pInput;
            m_pInput = NULL;
         }
      }
      
      // Return the appropriate pin
      
      if(n == 0)
         return m_pInput;
      if (n == 1)
         return m_pOutput;
      return NULL;
   }

//----------------------------

   virtual HRESULT StartStreaming(){
                              //clear the destination in 1st transform
      frame_index = 0;
      total_size = 0;
      curr_quant = INIT_QUANTIZIER;
      return CTransformFilter::StartStreaming();
   }

//----------------------------

   virtual HRESULT StopStreaming(){
      frame_index = -1;
      total_size = 0;
      return CTransformFilter::StopStreaming();
   }

//----------------------------

public:

//----------------------------

   CXvidEncoder(LPUNKNOWN punk):
      CTransformFilter(NAME("! CXvidEncoder"), punk, CLSID_XVID),
      frame_index(-1),
      total_size(0),
      fps(0)
   {
      xvid_gbl_init_t xvid_gbl_init;
      memset(&xvid_gbl_init, 0, sizeof(xvid_gbl_init));
      xvid_gbl_init.version = XVID_VERSION;
      if(xvid_global(0, XVID_GBL_INIT, &xvid_gbl_init, NULL) < 0){
         MessageBox(0, "xvid_global() failed", "Error", 0);
         return;
      }
      memset(&enc_create, 0, sizeof(enc_create));
      enc_create.version = XVID_VERSION;
      enc_create.handle = NULL;
      enc_create.profile = XVID_PROFILE_AS_L4;

                              //Bframes settings
	   enc_create.max_bframes = 4;
                              //bframe quantizer multipier/offeset; used to decide bframes quant when bquant==-1
	   enc_create.bquant_ratio = 150;
                              //(avg(past_ref_quant,future_ref_quant)*bquant_ratio + bquant_offset) / 100
	   enc_create.bquant_offset = 100;

#ifndef OWN_QAULITY
      enc_create.plugins = plugins;
      enc_create.num_plugins = 0;
      {
         memset(&single, 0, sizeof(single));
         single.version = XVID_VERSION;
         //single.bitrate = bitrate*1000;

         plugins[enc_create.num_plugins].func = xvid_plugin_single;
         plugins[enc_create.num_plugins].param = &single;
         enc_create.num_plugins++;
      }
#endif
                              //setup frame
      memset(&m_frame, 0, sizeof(m_frame));
      m_frame.version = XVID_VERSION;
      m_frame.input.csp = XVID_CSP_BGR | XVID_CSP_VFLIP;

#ifdef USE_QUARTER_PEL
      m_frame.vol_flags |= XVID_VOL_QUARTERPEL;
      m_frame.motion |= XVID_ME_QUARTERPELREFINE16;
#endif
#ifdef USE_GMC
      m_frame.vol_flags |= XVID_VOL_GMC;
      m_frame.motion |= XVID_ME_GME_REFINE;
#endif
      //m_frame.vol_flags |= XVID_VOL_MPEGQUANT;   //enable MPEG type quantization
      //m_frame.vol_flags |= XVID_VOL_REDUCED_ENABLE;   //enable reduced resolution vops: frames will be checked for rrv suitability

                              //this makes bugs in video
      //if(QUALITY >= 2)
         //m_frame.vop_flags |= XVID_VOP_HALFPEL; //use halfpel interpolation
      if(QUALITY >= 3)
         m_frame.vop_flags |= XVID_VOP_INTER4V;
      if(QUALITY >= 4)
         m_frame.vop_flags |= XVID_VOP_TRELLISQUANT;  //use trellis based R-D "optimal" quantization
      if(QUALITY >= 6)
         m_frame.vop_flags |= XVID_VOP_HQACPRED;  //high quality ac prediction
      //m_frame.vop_flags |= XVID_VOP_CHROMAOPT;  //enable chroma optimization pre-filter
      //m_frame.vop_flags |= XVID_VOP_CARTOON;    //use 'cartoon mode'
      //m_frame.vop_flags |= XVID_VOP_GREYSCALE;
      //m_frame.vop_flags |= XVID_VOP_MODEDECISION_RD;  //enable DCT-ME and use it for mode decision
      //m_frame.vop_flags |= XVID_VOP_TOPFIELDFIRST;  //set top-field-first flag
      //m_frame.vop_flags |= XVID_VOP_REDUCED;  //reduced resolution vop
      //m_frame.vop_flags |= XVID_VOP_FAST_MODEDECISION_RD;  //use simplified R-D mode decision

#ifdef USE_QUARTER_PEL
      if(m_frame.vop_flags&XVID_VOP_INTER4V)
         m_frame.motion |= XVID_ME_QUARTERPELREFINE8;
#endif

      if(QUALITY >= 1)
         m_frame.motion |= XVID_ME_ADVANCEDDIAMOND16; //use advdiamonds instead of diamonds as search pattern
      if(QUALITY >= 2)
         m_frame.motion |= XVID_ME_HALFPELREFINE16;
      if(QUALITY >= 3)
         m_frame.motion |= XVID_ME_ADVANCEDDIAMOND8;  //use advdiamond for XVID_ME_EXTSEARCH8
      if(QUALITY >= 3)
         m_frame.motion |= XVID_ME_HALFPELREFINE8;
      if(QUALITY >= 4){
         m_frame.motion |= XVID_ME_CHROMA_PVOP; //also use chroma for P_VOP/S_VOP ME
         m_frame.motion |= XVID_ME_CHROMA_BVOP; //also use chroma for B_VOP ME
      }
      if(QUALITY >= 6){
         m_frame.motion |= XVID_ME_EXTSEARCH16; //extend PMV by more searches
         m_frame.motion |= XVID_ME_EXTSEARCH8; //use diamond/square for extended 8x8 search
      }
      //m_frame.motion |= XVID_ME_USESQUARES16; //use squares instead of diamonds as search pattern
      m_frame.motion |= XVID_ME_USESQUARES8;
      m_frame.motion |= XVID_ME_DETECT_STATIC_MOTION;
      m_frame.motion |= XVID_ME_FASTREFINE16;


      memset(&enc_stats, 0, sizeof(enc_stats));
      enc_stats.version = XVID_VERSION;
   }

//----------------------------

   ~CXvidEncoder(){
      if(enc_create.handle){
         xvid_encore(enc_create.handle, XVID_ENC_DESTROY, 0, 0);
         enc_create.handle = NULL;
      }
   }

//----------------------------

   HRESULT CheckInputType(const CMediaType *mtIn){
   
      CAutoLock lock_it(m_pLock);
      if(*mtIn->Type() != MEDIATYPE_Video){
         return VFW_E_TYPE_NOT_ACCEPTED;
      }
      BITMAPINFOHEADER *hdr;
      if(*mtIn->FormatType() == FORMAT_VideoInfo){
         VIDEOINFOHEADER *vih = (VIDEOINFOHEADER*) mtIn->Format();
         hdr = &vih->bmiHeader;
      }else
      if(*mtIn->FormatType() == FORMAT_VideoInfo2){
         VIDEOINFOHEADER2 *vih = (VIDEOINFOHEADER2*) mtIn->Format();
         hdr = &vih->bmiHeader;
      }else{
         return VFW_E_TYPE_NOT_ACCEPTED;
      }
      /*
      if(hdr->biHeight < 0){
         DPRINTF("colorspace: inverted input format not supported");
      }
      */
      if(hdr->biBitCount != 24 || hdr->biCompression!=BI_RGB)
         return VFW_E_TYPE_NOT_ACCEPTED;

      enc_create.width = hdr->biWidth;
      enc_create.height = hdr->biHeight;
      return S_OK;
   }

//----------------------------

   HRESULT GetMediaType(int iPosition, CMediaType *mtOut){
      
      CAutoLock lock_it(m_pLock);
      if(!m_pInput->IsConnected())
         return E_UNEXPECTED;
      
      VIDEOINFOHEADER *vih = (VIDEOINFOHEADER*)mtOut->ReallocFormatBuffer(sizeof(VIDEOINFOHEADER));
      if(!vih)
         return E_OUTOFMEMORY;
      
      memset(vih, 0, sizeof(VIDEOINFOHEADER));
      vih->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
      vih->bmiHeader.biWidth  = enc_create.width;
      vih->bmiHeader.biHeight = enc_create.height;
      vih->bmiHeader.biPlanes = 1;
      if(fps!=0.0f)
         vih->AvgTimePerFrame = int(10000000.0f/fps);
      
      if(iPosition < 0)
         return E_INVALIDARG;

      if(iPosition > 0)
         return VFW_S_NO_MORE_ITEMS;

      mtOut->SetType(&MEDIATYPE_Video);
      mtOut->SetSubtype(&CLSID_XVID);
      
      //vih->bmiHeader.biSizeImage = GetBitmapSize(&vih->bmiHeader);
      
      mtOut->SetFormatType(&FORMAT_VideoInfo);
      mtOut->SetTemporalCompression(true);
      mtOut->SetVariableSize();
      //mtOut->SetSampleSize(vih->bmiHeader.biSizeImage);

      vih->bmiHeader.biCompression = mmioFOURCC('x','v','i','d');
      
      return S_OK;
   }

//----------------------------

   HRESULT SetMediaType(PIN_DIRECTION direction, const CMediaType *mt){

      CAutoLock lock_it(m_pLock);
      if(direction==PINDIR_INPUT){
         if(*mt->FormatType() != FORMAT_VideoInfo)
            return E_INVALIDARG;
         if(!IsEqualGUID(*mt->Subtype(), MEDIASUBTYPE_RGB24))
            return E_INVALIDARG;
         const VIDEOINFOHEADER *pvi = (VIDEOINFOHEADER*)mt->Format();
         single.bitrate = pvi->dwBitRate;
         if(pvi->AvgTimePerFrame)
            fps = 10000000.0f / float(pvi->AvgTimePerFrame);
         return S_OK;
      }
      return CTransformFilter::SetMediaType(direction, mt);
   }
   
//----------------------------

   HRESULT CheckTransform(const CMediaType *mtIn, const CMediaType *mtOut){
      return S_OK;
   }

//----------------------------

   HRESULT DecideBufferSize(IMemAllocator *pima, ALLOCATOR_PROPERTIES *pProperties);

   HRESULT Transform(IMediaSample *pIn, IMediaSample *pOut);

//----------------------------
};

//----------------------------

HRESULT CXvidEncoder::DecideBufferSize(IMemAllocator *pAlloc, ALLOCATOR_PROPERTIES *ppropInputRequest){

   CAutoLock lock_it(m_pLock);
   HRESULT result;
   ALLOCATOR_PROPERTIES ppropActual;

   if(!m_pInput->IsConnected())
      return E_UNEXPECTED;

   ppropInputRequest->cBuffers = 128;
   ppropInputRequest->cbBuffer = enc_create.width * enc_create.height * 4;
   // cbAlign causes problems with the resize filter
   // ppropInputRequest->cbAlign = 16; 
   ppropInputRequest->cbPrefix = 0;
      
   result = pAlloc->SetProperties(ppropInputRequest, &ppropActual);
   if(result != S_OK)
      return result;

   if(ppropActual.cbBuffer < ppropInputRequest->cbBuffer)
      return E_FAIL;

   return S_OK;
}

//----------------------------

int CXvidEncoder::EncodeMain(bool &is_keyframe, dword frm_time){

                              //set up core's general features
   //m_frame.vop_flags |= XVID_VOP_DEBUG;

#ifdef OWN_QAULITY
                              //compute quality (quantizier value) depending on current output size and requested bitrate
   if(frame_index){
      //int want_bytes = bitrate*
                     //  convert ms to seconds
                     //   |           convert bytes to bits
                     //   |              |         convert bps to kbps
                     //   |              |            |
      dword curr_bitrate = dword(1000*__int64(total_size)*8/__int64(frm_time)/1024);
      dword thresh = curr_bitrate/10;
      if(curr_bitrate+thresh < bitrate){
                              //increase quality
         curr_quant -= Max(1, curr_quant/8);
         curr_quant = Max(0, curr_quant);
      }else
      if(curr_bitrate-thresh > bitrate){
         if(curr_bitrate > bitrate*2)
            curr_quant = Max(1, curr_quant*2);
         else
            ++curr_quant;
         curr_quant = Min(MAX_QUANTIZIER, curr_quant);
      }
   }
   m_frame.quant = curr_quant;
#else
   m_frame.quant = 0;
#endif

                              //we don't use special matrices
   //m_frame.quant_intra_matrix = NULL;
   //m_frame.quant_inter_matrix = NULL;

   //m_frame.type = !(frame_index&7) ? XVID_TYPE_IVOP : !(frame_index&3) ? XVID_TYPE_PVOP : XVID_TYPE_BVOP;
   //m_frame.type = !(frame_index&7) ? XVID_TYPE_IVOP : XVID_TYPE_BVOP;
   enc_stats.version = XVID_VERSION;
                              //encode the frame
   int sz = xvid_encore(enc_create.handle, XVID_ENC_ENCODE, &m_frame, &enc_stats);
   ASSERT(sz >= 0);
   is_keyframe = ((m_frame.out_flags & XVID_KEYFRAME) != 0);
#if defined _DEBUG && 0
   dword tp = enc_stats.type;
   PRINT(C_xstr(
      "#5%: %, sz: #7% type: %"
      )
      %frame_index
      %(is_keyframe ? "key" : "   ")
      %sz
      %(tp==XVID_TYPE_IVOP ? 'I' : tp==XVID_TYPE_PVOP ? 'P' : tp==XVID_TYPE_BVOP ? 'B' : tp==XVID_TYPE_SVOP ? 'S' : 'U')
      );
#endif
   //*stats_type = xvid_enc_stats.type;
   //*stats_quant = xvid_enc_stats.quant;
   //*stats_length = xvid_enc_stats.length;
   /*
   sse[0] = xvid_enc_stats.sse_y;
   sse[1] = xvid_enc_stats.sse_u;
   sse[2] = xvid_enc_stats.sse_v;
   */
   total_size += sz;
   ++frame_index;
   return sz;
}

//----------------------------

HRESULT CXvidEncoder::Transform(IMediaSample *pIn, IMediaSample *pOut){

   CAutoLock lock_it(m_pLock);

   HRESULT hr;

                              //don't encode pre-roll samples
   if(pIn->IsPreroll() == S_OK)
      return S_FALSE;

   REFERENCE_TIME TimeStart, TimeEnd;
   hr = pIn->GetTime(&TimeStart, &TimeEnd);
   ASSERT(hr==S_OK);
   if(!enc_create.handle){
      {
         if(fps==0.0f){
            int tm_delta = int(TimeEnd - TimeStart);
            fps = float((100000+(tm_delta/2)) / tm_delta);
                              //still zero? probably 1st frame wrong, then just guess fps
            if(fps<1.0f)
               fps = 12.5f;
         }
         enc_create.fbase = int(fps*1000.0f);
         enc_create.fincr = 1000;
                              //maximum interval between key frames
         enc_create.max_key_interval = int(fps ? (fps * MAX_KEYFRAME_INTERVAL / 1000) : (MAX_KEYFRAME_INTERVAL*15));
      }

      if(xvid_encore(NULL, XVID_ENC_CREATE, &enc_create, NULL) < 0){
         return S_FALSE;
      }
   }
   hr = pOut->SetTime(&TimeStart, &TimeEnd);
   ASSERT(hr==S_OK);

   byte *src;
   if(pIn->GetPointer(&src) != S_OK)
      return S_FALSE;

   if(pOut->GetPointer((byte**)&m_frame.bitstream) != S_OK)
      return S_FALSE; 

   m_frame.input.plane[0] = src;
   m_frame.length = -1;

   m_frame.input.stride[0] = enc_create.width*3;

   bool is_keyframe;
#ifndef DEBUG_NO_ENCODE
   int m4v_size = EncodeMain(is_keyframe, dword(TimeStart/10000));
#else
   int m4v_size = 400; is_keyframe = (frame_index==1);
#endif

#ifdef DEBUG_PRINT
   {
      PRINT(C_xstr(
         "VID #4%, time: #.3% (#.3%) sz: %"
         )
         %(frame_index-1)
         %(float(TimeStart)/10000000.0f)
         %(float(TimeEnd-TimeStart)/10000000.0f)
         %m4v_size
         );
      /*
      static double time_sum;
      double frm_time = double(TimeEnd-TimeStart)/10000000.0;
      time_sum += frm_time;
      PRINT(C_xstr("#5% % (avg: %)") %frame_index %float(frm_time) %float(time_sum/double(frame_index+1)));
      */
   }
#endif

   hr = pOut->SetActualDataLength(m4v_size);
   ASSERT(hr==S_OK);
   
   hr = pOut->SetSyncPoint(is_keyframe);
   ASSERT(hr==S_OK);
   hr = pOut->SetDiscontinuity(true);
   ASSERT(hr==S_OK);

#ifdef _DEBUG
   //Sleep(100);
#endif
//----------------------------
   /*
   m_frame.general = XVID_LOWDELAY;
   if(pIn->IsDiscontinuity() == S_OK)
      m_frame.general = XVID_DISCONTINUITY;

repeat:
   if(stats.type == XVID_TYPE_VOL){
      if(stats.data.vol.width != enc_create.width || stats.data.vol.height != enc_create.height){
         DPRINTF("TODO: auto-resize");
         return S_FALSE;
      }
      m_frame.bitstream = (BYTE*)m_frame.bitstream + length;
      m_frame.length -= length;
      goto repeat;
   }
   */
   return S_OK;
}

//----------------------------

CUnknown *WINAPI CreateXviDCodec(LPUNKNOWN pUnk, HRESULT *phr){
   *phr = S_OK;
   return new CXvidEncoder(pUnk);
}

//----------------------------

