 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: adaptationenginedialog.cpp                                    *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "adaptationenginedialog.hpp"
#ifndef WIN32
//#include "moc_adaptationenginedialog.cpp"
#endif
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qfiledialog.h>
#include <qprogressbar.h>

#include "adaptors/Forwarder.hpp"
#include "ContainerInfo.hpp"
#include "container/ContainerFile.hpp"
#include "VideoESInfo.hpp"
#include "Adaptor.hpp"
#include "adaptors/TemporalAdaptor.hpp"
#include "adaptors/SpatialReductionAdaptor.hpp"
#include "adaptors/QualityReductionAdaptor.hpp"
#include "adaptors/ColorReductionAdaptor.hpp"
#include "container/ContainerDemux.hpp"
#include "metadata/MP7File.hpp"
#include "io/IOCreator.hpp"

#ifndef WIN32
char* itoa(int i, char* dest,int b) {
	sprintf(dest,"%i",i);
	return dest;
};
#endif

AdaptationEngineDialog::AdaptationEngineDialog( QWidget* parent, const char* name, bool modal, WFlags f )
	: AdaptationEngineDialogBase( parent, name, modal, f )
{
//	QString *mp4file;

	// Add your code
	
	currentVideo=NULL;
	framePatternDest->insertItem("IPB");
	framePatternDest->insertItem("IPP");
	framePatternDest->insertItem("III");
	
	codecChooser->insertItem("FFMPEG");
	codecChooser->insertItem("XVID");
	
	mp7=NULL;
	showAdaptation->setChecked(true);

	disableDest();
	
	// chosenFileName->setText("Z:/streams/coastguard_9_6.mp4");
	// xmlFileOutput->setText("Z:/streams/coastguard_9_6.xml");
	// enableDest();
	// mp4file = new QString("Z:/streams/coastguard_9_6.mp4");
//	loadMP4(*mp4file);
	resetDest();

}

AdaptationEngineDialog::~AdaptationEngineDialog() { 	
		if(currentVideo) {
			delete currentVideo;
			currentVideo=NULL;
		}
		closeMP7();
	};

void AdaptationEngineDialog::closeMP7() {


	if(mp7) {
		MP7File::closeMP7File(mp7);
		mp7=NULL;
		if(showXML->isChecked()) {
			
			char syscall[1024];
			strcpy(syscall,"start iexplore.exe ");
			strcat(syscall,xmlFileOutput->text().latin1());
			system(syscall);
		}
	}
};

void AdaptationEngineDialog::fileChoose()
{
	QFileDialog f;

    QString fn = QFileDialog::getOpenFileName( QString::null, "Videos (*.mp4 *.avi *.mpg)", this );
	if ( !fn.isEmpty() ) {
		enableDest();
        chosenFileName->setText(fn);
		QString xml=fn;
		xml.truncate(fn.length()-3);
		xml+="xml";
		xmlFileOutput->setText(xml);
		loadMP4(fn);
    } 

};

void AdaptationEngineDialog::loadMP4(QString &file) {
	if(currentVideo) {
		delete currentVideo;
	}

	//const char *t = file.latin1();
	currentVideo=ContainerFile::loadContainerInfo(file.latin1());
	if(currentVideo && currentVideo->getFirstVisualES()) {
		// ContainerDemux::demultiplexAndUpdateIO(currentVideo);
		MP4Decoder* dec=new MP4Decoder(currentVideo->getFirstVisualES(),true);
		dec->initialize();
		delete dec;
		this->MultiLineEdit1->clear();
		this->MultiLineEdit1->insertLine("Succeeded in loading video",0);
		FILE* f=fopen(currentVideo->getLocalFile(),"rb");
		if(f) {
			fseek(f,0,SEEK_END);
			totalSizeSrc=ftell(f);
			fclose(f);
		}
		closeMP7();
		mp7=MP7File::openMP7File(xmlFileOutput->text().latin1());
		resetDest();
	}
	else {
		this->MultiLineEdit1->clear();
		if(currentVideo && currentVideo->getFirstVisualES()==NULL) {
			MultiLineEdit1->insertLine("File contains no visual stream",0);
			delete currentVideo;
			currentVideo=NULL;
		}
		else
			MultiLineEdit1->insertLine("Failed in loading video",0);

	}
	
}


void AdaptationEngineDialog::doAdaptation() {
	if(!currentVideo) {
		this->MultiLineEdit1->clear();
		MultiLineEdit1->insertLine("Choose a video first",0);
		return;
	}
	checkDest(); // now entrys are valid
	if(!mp7)
	{
		mp7=MP7File::openMP7File(xmlFileOutput->text().latin1());
	}
	// first get the wished properties and compare them to the src properties
	unsigned int dimX=0;
	unsigned int dimY=0;
	int bitRateDest=0;
	int adaptorCount=0;
	
	ContainerInfo* clone=currentVideo->clone();
	VideoESInfo* es=clone->getFirstVisualES();

	MP4Decoder::meCoderType codec = MP4Decoder::FFMPEG;
	if(codecChooser->currentText().compare("XVID")==0) {
		codec=MP4Decoder::XVID;
		dprintf_full("Using XVID %i\n",codec);
	}
	dprintf_full("Using Codec %i\n",codec);
	
	sscanf(dimensionDestX->text().latin1(),"%i", &dimX);
	sscanf(dimensionDestY->text().latin1(),"%i", &dimY);
	sscanf(avgBitrateDest->text().latin1(),"%i", &bitRateDest);

	this->MultiLineEdit1->clear();
	Adaptor* a[20];

	this->MultiLineEdit1->insertLine("Adaptors needed:",0);
	int bitrate=(int)(es->getAvgBandwidth()/1024);
	// check framerate first
	if(framePatternDest->currentText().compare("IPP")==0) {
		// select TemporalReduction
		a[adaptorCount]=new TemporalAdaptor(es);
		adaptorCount++;
		bitrate= (int)(bitrate*0.8);
		this->MultiLineEdit1->insertLine("+ TemporalReductionAdaptor",adaptorCount);
	}
	else if(framePatternDest->currentText().compare("III")==0) {
		// select StrongTemporalReduction
		a[adaptorCount]=new StrongTemporalAdaptor(es);
		adaptorCount++;
		bitrate= (int)(bitrate*0.2);
		this->MultiLineEdit1->insertLine("+ StrongTemporalReductionAdaptor",adaptorCount);
	}	
	// then geyscale
	if(greyScaleDest->isChecked()) {
		// ColorReduction
		bitrate= (int)(bitrate*0.8);
		a[adaptorCount]=new ColorReductionAdaptor(es,30,bitrate,codec);
		adaptorCount++;
		
		this->MultiLineEdit1->insertLine("+ ColorReductionAdaptor",adaptorCount);
	}

	// then check dimension	
	if(dimX!=es->getWidth() || dimY!=es->getHeight() ) {
		
		a[adaptorCount]=new SpatialReductionAdaptor(es,&dimX, &dimY, bitRateDest*1024,30,codec);
		adaptorCount++;
		this->MultiLineEdit1->insertLine("+ SpatialReductionAdaptor",adaptorCount);
	}
	else {
		// no dim change but maybe bitrate?
		dprintf_full("Dest %i Grey %i Codec %i",bitRateDest, bitrate,codec);
		if(bitRateDest< bitrate ) {
			a[adaptorCount]=new QualityReductionAdaptor(es,bitRateDest*1024,30,codec);
			adaptorCount++;
			this->MultiLineEdit1->insertLine("+ QualityReductionAdaptor",adaptorCount);
		}
	}
	if(adaptorCount>0) {
		// now perform the adaptation
		if(!showAdaptation->isChecked()) {
			this->MultiLineEdit1->insertLine("WORKING",adaptorCount+1);
		}
		if(mp7) {
			MP7File::writeVariationSet(mp7,a,adaptorCount,clone,showAdaptation->isChecked());
		}
		
		VideoESInfo* es2=clone->getFirstVisualES();
		IO* inp=IOCreator::createInput(es2);
		const char* io=inp->getURL();
		if(inp) {
			FILE* f=fopen(io,"rb");
			if(f) {
				fseek(f,0,SEEK_END);
				long size=ftell(f);
				// FIXME: now add 1000 byte for BIFS+OD
				size+=1000;
				size/=1024; // kByte
				bBitrateDest=size*8000/es2->getDurationInMs();
				char t[30];
				avgBitrateDest->setText(itoa(bBitrateDest,t,10));
				// now update the GUI to reflect the changes
				fileSizeD->setText("New FileSize: ");
				percentGainedDest->setProgress(bBitrateDest*100/bBitrateSrc);
				fileSizeDest->setText( itoa(size,t,10));
				fclose(f);
			}
			delete[] inp;
		}
	}
	else {
		this->MultiLineEdit1->insertLine("+ No adaptation needed",1);
		if(showAdaptation->isChecked()) {
			Forwarder *f=new Forwarder();
			ContainerDemux::demultiplexAndUpdateIO(clone,NULL,f,false,false,true);
		}
	}
	this->MultiLineEdit1->removeLine(adaptorCount+1);
	this->MultiLineEdit1->insertLine("DONE",adaptorCount+1);
	delete clone;
}

void AdaptationEngineDialog::checkDest() {
	if(!currentVideo)
		return;

	int dimX;
	int dimY;
	int bitRateDest;
	char temp[60];
	temp[0]='\0';

	VideoESInfo* es=currentVideo->getFirstVisualES();
	assert(es);
	//check dimx
	if(1!=sscanf(dimensionDestX->text().latin1(),"%i", &dimX)) {
		//failed -> set to src
		dimensionDestX->setText(dimensionSrcX->text());
	}
	else {
		// check if valid
		dimX>>=4;
		dimX<<=4;	// MOD 16 TRUE
		if(dimX<128) {
			dimX=128;
		}
		if(dimX>(int)es->getWidth()) {
			dimX=es->getWidth();
		}
		
		dimensionDestX->setText(itoa(dimX,temp,10));
	}

	//check dimy
	if(1!=sscanf(dimensionDestY->text().latin1(),"%i", &dimY)) {
		//failed -> set to src
		dimensionDestY->setText(dimensionSrcY->text());
	}
	else {
		// check if valid
		dimY>>=3;	// MOD 8 TRUE
		dimY<<=3;
		if(dimY<64) {
			dimY=64;
		}
		if(dimY>(int)es->getHeight()) {
			dimY=es->getHeight();
		}
		dimensionDestY->setText(itoa(dimY,temp,10));
	}

	//check bitrate
	if(1!=sscanf(avgBitrateDest->text().latin1(),"%i", &bitRateDest)) {
		//failed -> set to src
		avgBitrateDest->setText(avgBitrateSrc->text());
	}
	else {
		// check if valid
		if(bitRateDest<64)
			bitRateDest=64;
		int difference=bitRateDest-es->getAvgBandwidth();
		
		if(difference>-1)
			bitRateDest=es->getAvgBandwidth();

		avgBitrateDest->setText(itoa(bitRateDest,temp,10));
	}

};

void AdaptationEngineDialog::resetDest() {

	if(currentVideo) {
		// set dimension
		char t[55];
		strcpy(t,"0 X 0");
		VideoESInfo* es=currentVideo->getFirstVisualES();
		if(es) {
			// set bitrate
			sprintf(t,"%i",es->getAvgBandwidth()/1024);
			bBitrateSrc=es->getAvgBandwidth()/1024;
			bBitrateDest=es->getAvgBandwidth()/1024;
			avgBitrateSrc->setText(t);
			avgBitrateDest->setText(t);
			percentGainedDest->setProgress(100);

			dimensionSrcX->setText(itoa(es->getWidth(),t,10));
			dimensionSrcY->setText(itoa(es->getHeight(),t,10));
			dimensionDestX->setText(itoa(es->getWidth(),t,10));
			dimensionDestY->setText(itoa(es->getHeight(),t,10));

			// set framerate
			sprintf(t,"%.2f",es->getFPS());
			frameRateSrc->setText(t);
			framePatternDest->setCurrentItem(0);

		
		
		}
		else {
			this->MultiLineEdit1->clear();
			this->MultiLineEdit1->insertLine("Video contains no visual stream?",0);
		}
		//set fileSize
		fileSizeSrc->setText(itoa(totalSizeSrc/1024,t,10));
		fileSizeDest->setText(itoa(totalSizeSrc/1024,t,10));
		greyScaleDest->setChecked(false);

	}
	else {
		this->MultiLineEdit1->clear();
		dimensionDestX->setText("");
		dimensionDestY->setText("");
		framePatternDest->setCurrentItem(0);
		avgBitrateDest->setText("");
	}
};

void AdaptationEngineDialog::updateBitrate() {
	if(!currentVideo)
		return;
	unsigned int dimX;
	unsigned int dimY;
	VideoESInfo* es=currentVideo->getFirstVisualES();
	if(!es)
		return;

	if(1!=sscanf(dimensionDestX->text().latin1(),"%i", &dimX))
		return;

	if(1!=sscanf(dimensionDestY->text().latin1(),"%i", &dimY))
		return;
	fileSizeD->setText("New FileSize (estimated): ");	
	bBitrateDest=bBitrateSrc;
	// check framerate first
	if(framePatternDest->currentText().compare("IPP")==0) {
		// select TemporalReduction
		bBitrateDest=(int)(bBitrateDest*0.8);
			
	}
	else if(framePatternDest->currentText().compare("III")==0) {
		// select StrongTemporalReduction
		bBitrateDest=(int)(bBitrateDest*0.2);
	}	

	// then greyscale
	if(greyScaleDest->isChecked()) {
		// ColorReduction
		bBitrateDest=(int)(bBitrateDest*0.8);
	}

	// then check dimension	
	if(dimX!=es->getWidth() || dimY!=es->getHeight() ) {
		
		int sq=dimX*dimY;
		int sq2=es->getWidth()*es->getHeight();
		float diff=((float)sq/sq2)*1.5; // if we scale a video by a factor of 2, we don't estimate a 75% reduction
		if(diff>1.0)
			diff=1.0;
		bBitrateDest=(int)(bBitrateDest*diff);
		if(bBitrateDest<64)
			bBitrateDest=64;
	}
	char t[30];
	avgBitrateDest->setText(itoa(bBitrateDest,t,10));
	percentGainedDest->setProgress(bBitrateDest*100/bBitrateSrc);
	fileSizeDest->setText(itoa(totalSizeSrc*bBitrateDest/bBitrateSrc/1024,t,10));
};

void AdaptationEngineDialog::bitrateChange() {

	fileSizeD->setText("New FileSize (estimated): ");
	int bitRateDest=0;

	if(1!=sscanf(avgBitrateDest->text().latin1(),"%i", &bitRateDest))
		return;
	if(bitRateDest<64)
		bitRateDest=64;
	if(bitRateDest>(int)bBitrateSrc) 
		bitRateDest=(int)bBitrateSrc;

	char t[30];
	
	percentGainedDest->setProgress(bitRateDest*100/bBitrateSrc);
	fileSizeDest->setText(itoa(totalSizeSrc*bitRateDest/bBitrateSrc/1024,t,10));

}

void AdaptationEngineDialog::enableDest() {

	dimensionDestX->setDisabled(false);
	dimensionDestY->setDisabled(false);
	framePatternDest->setDisabled(false);
	avgBitrateDest->setDisabled(false);
	greyScaleDest->setDisabled(false);
	showAdaptation->setDisabled(false);
	showXML->setDisabled(false);
	codecChooser->setDisabled(false);
}

void AdaptationEngineDialog::disableDest() {

	dimensionDestX->setDisabled(true);
	dimensionDestY->setDisabled(true);
	framePatternDest->setDisabled(true);
	codecChooser->setDisabled(true);
	avgBitrateDest->setDisabled(true);
	greyScaleDest->setDisabled(true);
	showAdaptation->setDisabled(true);
#ifdef WIN32
	showXML->setDisabled(true);
#endif
};
