#include "RequestIterator.hpp"


#define REQUESTITERATOR_MAX_PARALLEL_SESSIONS 1024

extern int GLOBAL_TIME_SCALE;

RequestIterator::RequestIterator(const char* tle, const char* outFile, 
								 const char* DescFile,const char* jlFile, UserPrefGenerator* up, 
								 const char* prx)
{
	title=new char[strlen(tle)+1];
	strcpy(title,tle);
	outFileName=new char[strlen(outFile)+1];
	strcpy(outFileName,outFile);
	memset(videos,0,sizeof(Feature*)*MAX_VIDEOS);
	nrVideos=0;
	assert(up);
	pUserPref=up;
	proxy=NULL;
	if(prx) {
		proxy=new char[strlen(prx)+1];
		strcpy(proxy,prx);
	}
	assert(DescFile);
	readDescriptions( DescFile,jlFile );

}


RequestIterator::~RequestIterator()
{
	delete pUserPref;
	int i=0;
	for(i=0;i<nrVideos;i++)
	{
		if(videos[i])
			delete videos[i];
	}
	delete[] title;
	delete[] outFileName;
	if(proxy)
		delete[] proxy;
}

void RequestIterator::readDescriptions( const char* descfile, const char* jlFile )
{
	assert(descfile);
	/* Read the whole file sorted into the entries vector
	 */
	std::vector<datentry> entries;
	std::ifstream input (descfile);
	if (!input.is_open())
	{
		std::cout << "Couldn't open input file" << std::endl;
		::exit(1);
	}	
	std::string tmp;
	datentry tmpEntry;
	
	printf("Reading %s...\n",descfile);
	
	while(!input.eof())
	{
		getline(input, tmp, '\n');
		if(tmp.size() > 0)
		{
			sscanf(tmp.c_str(),"%u %u %u",&tmpEntry.id,&tmpEntry.occurrence,&tmpEntry.size);
			entries.push_back(tmpEntry);
		}
	}
	input.close();
	/* Sort */
	std::sort(entries.begin(),entries.end());

	printf("Reading %s...\n",jlFile);

	BMJobList* joblist = NULL;
	joblist = new BMJobList();
	joblist->fromConfig(jlFile);
	joblist->sort();
	
	printf ( "\033[32m OK\033[0m\n");

	BMJob* nextjob = NULL;
	
	printf("\033[0mdescribing sources...\033[31m");
	fflush(stdout);
		
	int j = 0;
	
	char* temp = new char[10240];

	for(std::vector<datentry>::iterator i = entries.begin() ; i!=entries.end() ;i++)
	{
		// get videoinfos from VideoEsInfo and create net Video = Feature opbject
		
		nextjob = joblist->getJob(j);
		
		if (nextjob == NULL) {
			delete[] temp;
			return;
		}

		if ( nextjob->getVideoWidth() == 0 ||  nextjob->getVideoHeight() == 0 )
		{
			printf( nextjob->toString(temp) );

			printf("!!! droped video !!!\n");
		}
		else
		{
			Feature* vid = new Feature ( nextjob->getURL()->toString(), nextjob->getVideoWidth() , nextjob->getVideoHeight(), 32, nextjob->getVideoBitrate(), 0, 1, 25.0, nextjob->getDuration()/1000 );
			videos[j++]=vid;
			nrVideos++;
		}
	}
	delete[] temp;
	
	return;

}

	int round2bin( int input )
	{
		int val=1;
		while ( val < input )
			val=val*2;

		printf("round bandwith to %i\n", val);
		return val;
	}

 bool RequestIterator::makeRequest(FILE* fp, UserPreferences* &pUp, Feature* &vid, double &timep )
{
	dprintf_full(">> RequestIterator(%p)::makeRequest(%p) >>\n", this, fp);
	if(nrVideos<1)
		return false;
	if(!fp)
		return false;
	
	dprintf_full("pUp = %p, pUserPref = %p\n", pUp, pUserPref);
	
	int id;
	int size;
	
	if(!readJobEntry(fp,&id,&timep,&size))
	{
		return false;
	}
	
	pUp=pUserPref->makeUserPreferences( videos[id%nrVideos]->dimX, videos[id%nrVideos]->dimY );
	vid=new Feature(videos[id%nrVideos]);
	
	FILE* fpl = fopen( "userprefs.log", "a" );
	if(fpl) {
		char* pTmp=vid->toString();
		fprintf( fpl, "ORIG %s\n", pTmp );
		delete[] pTmp;
		
		if(!pUp) {
			fprintf( fpl, "NEW  see ORIG\n");
			fclose(fpl);
			return true;
		}

		
		pTmp=pUp->toString();
		fprintf( fpl, "NEW  %s\n",pTmp);
		delete[] pTmp;
		fclose ( fpl );
	}		

	dprintf_full(" NrVideos = %i, ID = %i\n", nrVideos, id);

	dprintf_full("get video# %i\n", id%nrVideos);
	
	
	
	return true;
}

bool RequestIterator::readJobEntry(FILE* job, int* id, double* timePoint, int* size)
{
	assert(id && timePoint && size);
	if(!job)
		return false;
	
	if ( feof(job) )
		return false;
	
	char temp[1024];
	
	if(fgets(temp,1024,job))
	{
		if(3==sscanf(temp,"%lf %i %i",timePoint,id,size))
			return true;
	}
	return false;
}

void RequestIterator::doAll(FILE* jobs, bool rtx)
{
	dprintf_full(">> RequestIterator::doAll >>\n");

	assert(jobs);
	
	UserPreferences* pUp = NULL;
	
	Feature* vid = NULL;

	int m=0;

	ClientSession* session[REQUESTITERATOR_MAX_PARALLEL_SESSIONS];
	double timePoint=0.0;
	long timeSleepInMs=0;
	struct timeval benchStartTime;
	struct timeval theTimeIsNow;
	long timeDiffInMs=0;
	int i=0;
	gettimeofday(&benchStartTime,NULL);

	
	for(i=0;i<REQUESTITERATOR_MAX_PARALLEL_SESSIONS;i++)
		session[i]=NULL;

	while( makeRequest(jobs, pUp, vid,timePoint) )
	{
	//	char* rs=req->up.toString();
	//	char* rv=req->sourceVideo.toString();
	//	dprintf_full("Working on Video %s with request %s\n",rv,rs);

		if(session[m]) // even if thread is running --> kill
			delete session[m];
				
		gettimeofday(&theTimeIsNow,NULL);
		timeDiffInMs=(theTimeIsNow.tv_sec-benchStartTime.tv_sec)*1000+
		(theTimeIsNow.tv_usec-benchStartTime.tv_usec)/1000;
		timeSleepInMs=(long)timePoint*1000*GLOBAL_TIME_SCALE;
		timeSleepInMs-=timeDiffInMs;
		
		printf("\033[0mstarting benchmark in %li ms ... #%i\033[31m\n", timeSleepInMs, m); 
		if(timeSleepInMs>0) {
#ifdef WIN32
			msleep(timeSleepInMs);
#else
			struct timespec tm;
  			tm.tv_sec = timeSleepInMs/1000;
			tm.tv_nsec=(timeSleepInMs%1000)*1000*1000;
			nanosleep(&tm,NULL);
#endif
		}
		if(proxy==NULL)
			session[m] = new ClientSession( (const char*)vid->url, (IO*) new DevNull(), (IO*) new DevNull() , pUp, rtx);
		else
			session[m] = new ClientSession( (const char*)vid->url, proxy,(IO*) new DevNull(), (IO*) new DevNull() , pUp, rtx);
		session[m]->start();
		fflush(stdout);
/*		// delete as soon as possible!!!!
		for(int k=m-1;k>=0;k--) {
			if(session[k]) {
				if( !session[k]->running() || session[k]->finished()) {
					delete session[k];
					session[k]=NULL;
				}
			}
		} */
		m++;
		m%=REQUESTITERATOR_MAX_PARALLEL_SESSIONS;
		delete vid;vid=NULL;
		
	}
	
	for(i=0;i<REQUESTITERATOR_MAX_PARALLEL_SESSIONS;i++) {
		if(session[i]) {
			if( session[i]->running() || !session[i]->finished())
				session[i]->wait();
			delete session[i];session[i]=NULL;
		}
	}
}
