/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   UPContainer.cpp                                                      *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /**********************************************************************
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "UPContainer.hpp"
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#ifndef WIN32
#include <unistd.h>
#endif
UPContainer::UPContainer()
{
	vidWidth_min = 0;
	vidWidth_max = 0;
	framerate_min = 0;
	framerate_max = 0;
	bitrate_min = 0;
	bitrate_max = 0;
	retransmission = false;
	num_probs = 0;
	non_adaptation_prob = 0;
	probabilities = NULL; 
}
UPContainer::UPContainer(int wmin,int wmax,int fmin,int fmax,int bmin,int bmax,bool retr,int numProbs,int* probs,int nap)
{
	this->vidWidth_min = wmin;
	this->vidWidth_max = wmax;
	this->framerate_min = fmin;
	this->framerate_max = fmax;
	this->bitrate_min = bmin;
	this->bitrate_max = bmax;
	this->retransmission = retr;
	this->num_probs = numProbs;
	this->probabilities = probs;
	this->non_adaptation_prob = nap;
}

UPContainer::~UPContainer()
{
	free(probabilities);
}

void UPContainer::fromFile(FILE* fp)
{
	char* line = (char*)malloc(sizeof(char)*1024);
	char* field;
	char* value;
	int tmpInt = 0;
	char* tmpString = NULL;
	int i;
	while(fgets(line,1024,fp) != NULL)
	{
		field = strtok(line,"=");
		field = strip_spaces(field);
		value = strtok(NULL,"\n");
		if (value)
		{
			value = strip_spaces(value);
			if (strcmp(field,"VideoWidth-min") == 0) {
				tmpInt =  atoi(value);
				this->vidWidth_min = tmpInt;
				printf("VideoWidth-min read\n");
			} else if (strcmp(field,"VideoWidth-max") == 0) {
				tmpInt = atoi(value);
				this->vidWidth_max = tmpInt;
				printf("VideoWidth-max read\n");
			} else if (strcmp(field,"Framerate-min") == 0) {
				tmpInt = atoi(value);
				this->framerate_min = tmpInt;
				printf("Framerate-min read\n");
			} else if (strcmp(field,"Framerate-max") == 0) {
				tmpInt = atoi(value);
				this->framerate_max = tmpInt;
				printf("Framerate-max read\n");
			} else if (strcmp(field,"Bitrate-min") == 0) {
				tmpInt = atoi(value);
				this->bitrate_min = tmpInt;
				printf("Bitrate-min read\n");
			} else if (strcmp(field,"Bitrate-max") == 0) {
				tmpInt = atoi(value);
				this->bitrate_max = tmpInt;
				printf("Bitrate-max read\n");
			} else if (strcmp(field,"NonAdaptationProbability") == 0) {
				tmpInt = atoi(value);
				this->non_adaptation_prob = tmpInt;
				printf("NonAdaptationProbability read\n");
			} else if (strcmp(field,"Retransmission") == 0) {
				if (strcmp(value,"true") == 0) {
					this->retransmission = true;
				} else if (strcmp(value,"false") == 0) {
					this->retransmission = false;
				} else {
					tmpInt = atoi(value);
					this->retransmission = (tmpInt!=0);
				}
				printf("Retransmission read\n");
			} else if (strcmp(field,"NumProbabilities") == 0) {
				tmpInt = atoi(value);
				this->num_probs = tmpInt;
				printf("NumProbs read\n");
			} else if (strcmp(field,"Probabilities") == 0) {
				tmpString = (char*)malloc(sizeof(char)*(strlen(value)+1));
				strcpy(tmpString,value);
				printf("Probabilities read\n");
			}
		}
		
	}
	rewind(fp);
	free(line);
	field = NULL;
	value = NULL;
	
	/* If probabilities have been set, read the tmpString for the values */
	printf("NumProbs = %i\n",this->num_probs);
	if (this->num_probs > 0) 
	{
		if (tmpString && strlen(tmpString) > 0){
			this->probabilities = (int*)malloc(sizeof(int)*this->num_probs);
			if (this->num_probs == 1) {
				tmpInt = atoi(tmpString);
				this->probabilities[0] = tmpInt;
			}
			/* More than one entry so it's strtok time ^_^ */
			else {
				value = strtok(tmpString,",");
				value = strip_spaces(value);
				tmpInt = atoi(value);
				this->probabilities[0] = tmpInt;
				for(i=1 ; value != NULL ; i++) {
					value = strtok(NULL,",");
					if (!value) break;
					tmpInt = atoi(value);
					this->probabilities[i] = tmpInt;
				}
			}
			free(tmpString);
			for (i=0; i < this->num_probs ; i++){
				printf("%i ",this->probabilities[i]);
			}
			printf("\n");
		}
		else {
			printf("Uhmm...%i\n",strlen(tmpString));
		}
	}
}

bool UPContainer::fromFile(const char* fname)
{
	FILE* fp = fopen(fname,"r");
	if (fp != NULL) {
		this->fromFile(fp);
		fclose(fp);
//		close(fileno(fp));
		fp = NULL;
		return true;
	}
	else {
		printf("Failed to open %s for reading\n",fname);
		return false;
	}
}

void UPContainer::toFile(FILE* fp)
{
	int sum = 0;
	sum+=fprintf(fp,"VideoWidth-min=%i\n",this->vidWidth_min);
	sum+=fprintf(fp,"VideoWidth-max=%i\n",this->vidWidth_max);
	sum+=fprintf(fp,"Framerate-min=%i\n",this->framerate_min);
	sum+=fprintf(fp,"Framerate-max=%i\n",this->framerate_max);
	sum+=fprintf(fp,"Bitrate-min=%i\n",this->bitrate_min);
	sum+=fprintf(fp,"Bitrate-max=%i\n",this->bitrate_max);
	sum+=fprintf(fp,"Retransmission=%i\n",this->retransmission);
	sum+=fprintf(fp,"NumProbabilities=%i\n",this->num_probs);
	sum+=fprintf(fp,"Probabilities=");
	for (int i = 0 ; i < this->num_probs ; i++)
	{
		printf("Writing %i\n",probabilities[i]);
		if (i == this->num_probs-1){
			sum+=fprintf(fp,"%i",this->probabilities[i]);
		}
		else {
			sum+=fprintf(fp,"%i,",this->probabilities[i]);
		}
	}
	sum+=fprintf(fp,"\n");
	sum+=fprintf(fp,"NonAdaptationProbability=%i\n",this->non_adaptation_prob);
// 	printf("%i bytes written\n",sum);
}

bool UPContainer::toFile(char* fname)
{
	FILE* fp = fopen(fname,"w+");
	if (fp != NULL){
// 		printf("File opened\n");
		this->toFile(fp);
// 		printf("About to close file\n");
		fclose(fp);
//		close(fileno(fp));
 		printf("File closed\n");
		return true;
	}
	else {
		printf("Failed to open '%s' for writing\n",fname);
		return false;
	}
}

char* UPContainer::strip_spaces(char* string)
{
	int len = strlen(string);
	while(len > 0 && string[0] == ' ')
	{
		string = &string[1];
		len--;
	}
	while(len > 0 && string[len-1] == ' ')
	{
		string[len-1] = '\0';
		len--;
	}
	return string;
}
