#include "UserPrefGenerator.hpp"

/** @param users: array of single features for groups
  * @param users: array of possibility in promille for each group
  * @param nrProbClasses: number of different groups, defines the size of the arrays
  * @param seed: starting seed for pseudonumber generator
  */
UserPrefGenerator::UserPrefGenerator(UserPreferences** groups, const int* p, unsigned int nrProbClasses, 
				int seed,int proOrig)
{
	srand(seed);
	promilleOriginal=proOrig;
	probClasses=nrProbClasses;
	probabilities=new int[probClasses];
	// convert prob to upper limit of range
	unsigned int i=0;
	if(nrProbClasses>0)
		probabilities[0]=p[0];
	// test probab:
	for(i=1;i<nrProbClasses;i++) {
		assert(probabilities[i-1]>=0);
		probabilities[i]=(probabilities[i-1]+p[i]);
	}
	assert(probabilities[nrProbClasses-1]==1000-promilleOriginal);
	ppClasses=groups;

};
UserPrefGenerator::~UserPrefGenerator()
{
	delete probabilities;
	for(int i=0;i<probClasses;i++)
		delete ppClasses[i];
	delete[] ppClasses;
};

/** returns array of pre-generated user classes */
UserPreferences** UserPrefGenerator::generateClasses(const Feature& min, const Feature& max, int maxClasses)
{
	// generate feature ranges
	
	FeatureRange* dx=NULL;
	FeatureRange* br=NULL;
	dFeatureRange* fr=NULL;
	FeaturePoint* color=NULL;
	FeaturePoint* delay=NULL;
	UserPreferences ** groups=new UserPreferences*[maxClasses];
	double curFR=min.frameRate;
	double dxStep=((double)(max.dimX-min.dimX))/maxClasses;

	printf ( "dx-step: %f \n" , dxStep );
	
	double brStep=((double)(max.bitrate-min.bitrate))/maxClasses;

	printf ( "br-step: %f \n" , brStep );

	int dxmin=0, brmin=0, dxmax=0, brmax=0, dxmed=0,brmed=0,imed=0;
	dxmin=min.dimX;
	brmin=min.bitrate;
	int i;
	for(i=0;i<maxClasses;i++)
	{

		// the last range is made larger by one pixel
		// so that we include the max feature value into the last
		// range
		if(i==maxClasses-1)
		{
			dxmax=max.dimX;
			brmax=max.bitrate;
		}
		else
		{
			dxmax=(int)(dxmin+dxStep)-1;
			brmax=(int)(brmin+brStep)-1;
		}	
		dxmed=(dxmin+dxmax)/2;
		brmed=(brmin+brmax)/2;
		
		dx=new FeatureRange(dxmin, dxmed,dxmax,0.2);
		br=new FeatureRange(brmin,brmed,brmax,0.2);
		fr=new dFeatureRange(min.frameRate,curFR,max.frameRate,0.2);
		dxmin=dxmax+1;
		brmin=brmax+1;
		curFR+=5.0;
		if(curFR>max.frameRate)
			curFR=max.frameRate;
		if(i%3==0)
		{
			imed=0;
		}
		else
			imed=1;
		if(maxClasses==1)
			imed=1;
		color=new FeaturePoint(imed,0.2);
		delay=new FeaturePoint(MAXIMUM_ACCEPTABLE_DELAY,0.2);
		groups[i]=new UserPreferences(dx,br,delay,color,fr,10000);
		delete dx;delete br; delete delay; delete color; delete fr;
	}
	return groups;
};

UserPreferences* UserPrefGenerator::makeUserPref()
{
	int val=rand()%1000;
	int genClass=findClassFor(val);
	if(genClass==-1) // no adaptation wanted!
		return NULL;
	UserPreferences* up=new UserPreferences(ppClasses[genClass]);
	return up;
};

int UserPrefGenerator::findClassFor(int val)
{
	int genClass;
	for(genClass=0;genClass<probClasses;genClass++)
	{
		if(probabilities[genClass]>=val)
			return genClass;
	}
	return -1;
};

UserPreferences* UserPrefGenerator::makeUserPreferences(int dimX,int dimY) { 
      UserPreferences* up=makeUserPref(); // may return NULL
      if(up) {
      	up->correctDimension(); 
      	up->correctBitRate(dimX,dimY); 
      }
      return up; 
 } 
  
void UserPrefGenerator::doTest()
{
	Feature minF(150, 130, 0 , 64000, 0, 0, 15.0, 0.0);
	Feature maxF(400, 300, 1 , 640000, 0, 0, 30.0, 0.0);
	static const int prob[]={200,200,200,200,200};
	static const int nrProb=sizeof(prob)/sizeof(int);
	UserPreferences** ppUp=UserPrefGenerator::generateClasses(minF,maxF,nrProb);
	int i=0;
	for(i=0;i<nrProb;i++) {
		char* tmp=ppUp[i]->toString();
		printf("%i:%s\n",i,tmp);
		delete[] tmp;
	}
 	UserPrefGenerator* pUpGen=new UserPrefGenerator(ppUp,prob,nrProb,1377,0);
	for(i=0;i<1000;i++) {
		UserPreferences* pUp=pUpGen->makeUserPref();
		if(pUp) {
			char* tmp=pUp->toString();
			printf("%i:%s\n",i,tmp);
			delete[] tmp;
        	delete pUp;
		}
		else
			printf("%i:NULL",i);
	}
	delete pUpGen;
 	
};
