/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:  bm_interface.ui.h                                            *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "markglobal.hpp"
#include "bm_list.hpp"
#include "cg_list.hpp"
#include "vitooki.hpp"
#include "bmjoblist.hpp"

// screenshot_version

void bm_interface::init()
{
	dprintf_full("\033[32mbm_interface::start init\033[0m\n");

	vitooki_init_all();
	
	this->bml = (void*) new BenchmarkList();
	this->cgl = (void*) new ClientGroupList();
	this->jobl = (void*) new BMJobList();
	
	BMJob* tempj;
	int index=0;
	
	dprintf_full("\033[32mbm_interface::BMJobList->fromConfig()\033[0m\n");
        ((BMJobList*)this->jobl)->fromConfig();
	while ( NULL!= (tempj = ((BMJobList*)this->jobl)->getJob(index)) )
	{
		this->jobBox->insertItem( QString(tempj->getURL()->toString() ) );
		index++;
	}

	ClientGroup* tempg;
	index=0;

	dprintf_full("\033[32mbm_interface::ClientGroupList->fromConfig()\033[0m\n");
        ((ClientGroupList*)this->cgl)->fromConfig();
	while ( NULL!=(tempg = ((ClientGroupList*)this->cgl)->getGroup(index)) )
	{
		this->cgBox->insertItem( QString(tempg->getName() ) );
	        index++;
	}

	Benchmark* tempb;
	index=0;

	dprintf_full("\033[32mbm_interface::BenchmarkList->fromConfig()\033[0m\n");
        ((BenchmarkList*)this->bml)->fromConfig();
	while ( NULL!=(tempb = ((BenchmarkList*)this->bml)->getBenchmark(index)) )
	{
		this->numList->insertItem( QString::number( tempb->getNumClients() ) );
		this->typeList->insertItem( QString( tempb->getGroup()->getName() ) );
		this->urlList->insertItem( QString( tempb->getJob()->getURL()->toString()) );
		index++;

		//TODO add Clients and Url to cg and joblist
	    
	}

	dprintf_full("\033[32mbm_interface::finished init\033[0m\n");
}

void bm_interface::destroy()
{
	dprintf_full("\033[32mbm_interface::destroy\n");
	
	delete ((BenchmarkList*)this->bml);
	delete ((ClientGroupList*)this->cgl);
	delete ((BMJobList*)this->jobl);
	
	dprintf_full("\033[32mbm_interface::finished\n");
}

void bm_interface::job_clicked( int index )
{
	dprintf_full("\033[32mbm_interface::job_clicked %i\033[0m\n", index);
}


void bm_interface::cg_clicked( int index )
{
	dprintf_full("\033[32mbm_interface::cg_clicked %i\033[0m\n", index);
}


void bm_interface::bm_clickend( int index )
{
	dprintf_full("\033[32mbm_interface::bm_clickend %i\033[0m\n",index);
	Benchmark* actbm;
	
	// TODO check for inconsistency in lists !!!
	
	actbm = ((BenchmarkList*) this->bml)->getBenchmark( this->urlList->currentItem() );
	
	this->numclients->setValue( actbm->getNumClients() );
	this->startdelay->setValue( actbm->getDelayInMs() );
	this->parallelity->setValue( actbm->getParallelity() );

}


void bm_interface::add_clicked()
{
	dprintf_full("\033[32mbm_interface::add_clicked\033[0m\n");
	Benchmark* newbm = new Benchmark();
	
	newbm->setNumClients( this->numclients->value() );
	newbm->setDelayInMs( this->startdelay->value() );
	newbm->setParallelity( this->parallelity->value() );
	
	newbm->setGroup( ((ClientGroupList*)this->cgl)->getGroup( this->cgBox->currentItem() ));
	newbm->setJob( ((BMJobList*)this->jobl)->getJob( this->jobBox->currentItem() ));
	((BenchmarkList*) this->bml)->addBenchmark(newbm);
	
	this->numList->insertItem( QString::number( newbm->getNumClients() ) );
	this->typeList->insertItem( QString( newbm->getGroup()->getName() ) );
	this->urlList->insertItem( QString( newbm->getJob()->getURL()->toString()) );
	
	((BenchmarkList*)this->bml)->toConfig();
}


void bm_interface::del_clicked()
{
	dprintf_full("\033[32mbm_interface::del_clicked\033[0m\n");
	int cur = this->urlList->currentItem();

	if ( cur>=0 )
	{
		((BenchmarkList*) this->bml)->removeBenchmark(  ((BenchmarkList*) this->bml)->getBenchmark( cur )  );

		this->numList->removeItem( cur );
		this->typeList->removeItem( cur );
		this->urlList->removeItem( cur );
	
		((BenchmarkList*)this->bml)->toConfig();
	}

}


void bm_interface::start_benchmark()
{
	dprintf_full("\033[32m> bm_interface::start_benchmark\033[0m\n");
	int cur = getCurrentBenchmark();

	dprintf_full("\033[32m- bm_interface::create_benchmark\033[0m\n");
	
	Benchmark* bm1 = ((BenchmarkList*)this->bml)->getBenchmark( cur );
//	Benchmark* bm2 = ((BenchmarkList*)this->bml)->getBenchmark( cur+1 );

	bm1->init();
	
	bm1->run();
	
	bm1->wait();

//	delete(bm1);

	printf("\n\033[31mXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\033[0mn");
	fflush(stdout);

	delete(bm1);
}


void bm_interface::call_cg()
{
	dprintf_full("\033[32mbm_interface::call_cg\033[0m\n");
}


void bm_interface::call_job()
{
	dprintf_full("\033[32mbm_interface::call_job\033[0m\n");
}


void bm_interface::bm_up()
{
	dprintf_full("\033[32mbm_interface::bm_up\033[0m\n");

	int cur = getCurrentBenchmark();
	
	if ( !((BenchmarkList*)this->bml)->moveUp( cur ) )
		return;
	
	this->numList->clear();
	this->typeList->clear();
	this->urlList->clear();

	Benchmark* tempb = NULL;
        int index=0;

	while ( NULL!=(tempb = ((BenchmarkList*)this->bml)->getBenchmark(index)) )
	{
		this->numList->insertItem( QString::number( tempb->getNumClients() ) );
		this->typeList->insertItem( QString( tempb->getGroup()->getName() ) );
		this->urlList->insertItem( QString( tempb->getJob()->getURL()->toString()) );
		index++;
	}
	
	this->urlList->setCurrentItem(cur-1 );
	((BenchmarkList*)this->bml)->toConfig();
	
}


void bm_interface::bm_down()
{
	dprintf_full("\033[32mbm_interface::bm_down\033[0m\n");
	int cur = getCurrentBenchmark();
	
	if ( !((BenchmarkList*)this->bml)->moveDown( cur ) )
		return;
		
	this->numList->clear();
	this->typeList->clear();
	this->urlList->clear();

	Benchmark* tempb = NULL;
        int index=0;

	while ( NULL!=(tempb = ((BenchmarkList*)this->bml)->getBenchmark(index)) )
	{
		this->numList->insertItem( QString::number( tempb->getNumClients() ) );
		this->typeList->insertItem( QString( tempb->getGroup()->getName() ) );
		this->urlList->insertItem( QString( tempb->getJob()->getURL()->toString()) );
		index++;
	}
	
	this->urlList->setCurrentItem(cur+1);
	((BenchmarkList*)this->bml)->toConfig();
}


void bm_interface::clearOfflineSources()
{
	dprintf_full("\033[32mbm_interface::clearOfflineSources\033[0m\n");
	// TODO do something ;)
	((BenchmarkList*)this->bml)->toConfig();
}


void bm_interface::update_lists( int index )
{
	dprintf_full("\033[32mbm_interface::update_start\033[0m\n");
	this->numList->setCurrentItem(index);
	this->typeList->setCurrentItem(index);
	this->urlList->setCurrentItem(index);

//	this->bm_clickend( index );
	dprintf_full("\033[32mbm_interface::update_fin\033[0m\n");
}


int bm_interface::getCurrentBenchmark()
{
	dprintf_full("\033[32mbm_interface::getCurrentBenchmark\033[0m\n");
	//TODO do something ;)
	
	return this->numList->currentItem();
}
