/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   bg_list.cpp                                                 *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "bm_list.hpp"
#include "bm.hpp"

BenchmarkList::BenchmarkList()
{

}

BenchmarkList::~BenchmarkList()
{
	dprintf_full("\033[31mBenchmarkList::destructor\033[0m\n");
	list<Benchmark*>::iterator next = this->bmlist.begin();
	while( next!=this->bmlist.end() )
	{
		next = this->bmlist.begin();
		if ( *next == NULL )
			return;
			
		this->bmlist.erase(next);
		delete(*next);
		dprintf_full("\033[31mBenchmarkList::removed element\033[0m\n");
	}

}


bool BenchmarkList::moveDown( int id)
{
	dprintf_full("\033[31mBenchmarkList::moveDown %i\033[0m\n", id);
	list<Benchmark*>::iterator next = this->bmlist.begin();
	
	while( next!=this->bmlist.end() && id>0 )
	{
		next++;
		id--;
	}
	
	if ( id == 0 && next!=this->bmlist.end())
	{
		dprintf_full("\033[31mBenchmarkList::found element\033[0m\n");
		if( ++next == this->bmlist.end() )
			return false;
		else
		{
		--next;
		Benchmark* todo = *next;
		*next = (*(++next));
		*next = todo;
		return true;
		}
	}
	else
	{
		dprintf_err("\033[31mBenchmarkList::End of list\033[0m\n");
		return false;
	}
}


bool BenchmarkList::moveUp( int index)
{
	dprintf_full("\033[31mBenchmarkList::moveUp\033[0m\n");
	return this->moveDown(index-1);
}


bool BenchmarkList::addBenchmark(Benchmark* new_bm)
{
	dprintf_full("\033[31mBenchmarkList::addBenchmark\033[0m\n");
	assert(new_bm);
	this->bmlist.push_back(new_bm);
	return true;
}

bool BenchmarkList::removeBenchmark(Benchmark* to_remove)
{
	dprintf_full("\033[31mBenchmarkList::removeBenchmark\033[0m\n");
	assert(to_remove);
	list<Benchmark*>::iterator next = this->bmlist.begin();
	while( next!=this->bmlist.end() )
	{
		if ( (*next) == to_remove )
		{
			delete(*next);
			this->bmlist.erase(next);
			dprintf_full("\033[31mBenchmarkList::removed element\033[0m\n");
			return true;
		}
		else
		{
			next++;
		}
	}
	dprintf_err("\033[31mBenchmarkList::removed nothing\033[0m\n");
	return false;
}

Benchmark* BenchmarkList::getBenchmark(int id)
{
	dprintf_full("\033[31mBenchmarkList::getBenchmark\033[0m\n");
	list<Benchmark*>::iterator next = this->bmlist.begin();
	while( next!=this->bmlist.end() && id>0 )
	{
		next++;
		id--;
	}
	if ( id == 0 && next!=this->bmlist.end())
	{
		dprintf_full("\033[31mBenchmarkList::found element\033[0m\n");
		return *next;
	}
	else
	{
		dprintf_err("\033[31mBenchmarkList::End of list\033[0m\n");
		return NULL;
	}
}

bool BenchmarkList::toConfig()
{
	dprintf_full("\033[31mBenchmarkList::toConfig\033[0m\n");
/*
	char* bmbuffer = (char*) malloc(MAX_XML_LEN);
	list<Benchmark*>::iterator next = this->bmlist.begin();
	FILE* cfg = fopen( BML_FILE , "w+" );
	assert(cfg);
	while( next!=this->bmlist.end() )
	{
		fputs( (*next)->toString(bmbuffer, MAX_XML_LEN), cfg );
		next++;
	}
	fclose(cfg);
*/
	dprintf_full("\033[31mBenchmarkList::configed\033[0m\n");
	return true;
}

bool BenchmarkList::fromConfig()
{
	dprintf_full("\033[31mBenchmarkList::fromConfig\033[0m\n");
/*
	FILE* cfg_file = fopen( BML_FILE, "r" );		// hardcoded ! may not ok ?
	if ( cfg_file != NULL ) 
	{
		char* buffer = (char*) malloc(MAX_CFG_LEN);
	//	while( !feof(cfg_file) )
	//	{
			fread( (void*)buffer, 1, MAX_CFG_LEN, cfg_file );
			char* next = buffer;
			
			while ( NULL != (next=strstr(next, BM_START)) )
			{
				Benchmark* newbm = new Benchmark();
				newbm->fromString( next );
				dprintf_full("\033[31mBenchmarkList->addGroup\033[0m\n");
				this->addBenchmark( newbm );
				next = next+=5;
			}

	//	}
		fclose ( cfg_file );
		free(buffer);
		dprintf_full("\033[31mBenchmarkList::read ok\033[0m\n");
		return true;
	}
	else
	{
		dprintf_err("\033[31mBenchmarkList::read error\033[0m\n");
		return false;
	}
	*/
	return true;
}

