/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   bm_session.cpp                                              *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "bm_session.hpp"

BenchmarkSession::BenchmarkSession()	// just for testing
{

}


BenchmarkSession::BenchmarkSession(URL* new_url, int new_localport, int new_remoteport)
{

	this->url = new_url;
	this->remoteport = new_remoteport;
	this->localport = new_localport;

	struct hostent* hp = NULL;
	char* rtspbuf = NULL;
	char* recbuf = NULL;

	char* tmpcp = (char*)malloc(MAX_STR_LEN);
	char** tmpcpp = &tmpcp;

	hp = gethostbyname ( this->url->toString() );

	assert(hp);	// FIXME just for testing

	memset ( (char*)&(this->sin),0,sizeof(struct sockaddr_in) );
	this->sin.sin_family = AF_INET;
	bcopy ( hp->h_addr,(char*)&(this->sin.sin_addr),hp->h_length );
	this->sin.sin_port = htons (this->remoteport);

	this->sock = socket(PF_INET, SOCK_STREAM, 0);

	connect ( this->sock, (struct sockaddr*)&(this->sin), sizeof(struct sockaddr) );

	this->rtsp->generateDescribeForUrl ( this->url->toString(), PLAYER_ID );

	rtspbuf = (char*)rtsp->getBuffer();

	send ( this->sock, rtspbuf, strlen(rtspbuf) );

	recbuf = malloc(MSG_BUFFER_SIZE);

	memset ( recbuf, 0, MSG_BUFFER_SIZE );

	recv ( this->sock, recbuf, MSG_BUFFER_SIZE, 0 );

	this->strc = this->rtsp->parseResponseDescribe( recbuf, this->url->toString(), tmpcpp );

	this->esi = this->strc->getFirstVisualES();

	this->pl = new RFC3016();

	this->gt = new GlobalTimer();

	this->stats = new Statistics(esi, false);

	this->rtp = new Rtp ( this->url->toString(), this->remoteport, this->url->getServerName(), this->localport, this->esi, this->pl, false, this->tc, this->gt, this->stats );

	this->rtp->open();

	this->dc = new DataChannel( this->rtp, this->esi );

	this->mp4d = new MP4Decoder(this->esi, true);

	this->rtsp->generateSetup ( this->url->toString(), this->strc->getFirstVisualES()->getStreamId(), this->localport, true, this->localport);

	rtspbuf = (char*)rtsp->getBuffer();

	send ( this->sock, rtspbuf, strlen(rtspbuf) );

	memset ( recbuf, 0, MSG_BUFFER_SIZE );

	recv ( this->sock, recbuf, MSG_BUFFER_SIZE, 0 );

	bool switching;
	this->rtsp->parseResponseSetup ( recbuf, &this->sID, &(this->localport), &(this->remoteport), &switching, tmpcpp);
	this->OK = true;
}

bool BenchmarkSession::exec()
{
	if (!this->OK)
		return false;
	
	this->rtsp->generatePlay ( this->url->toString(), this->sID, 0, 0, 1);

	pbuf = (char*)rtsp->getBuffer();

	send ( this->sock, rtspbuf, strlen(rtspbuf) );

	this->rtp->play(0);

	memset ( recbuf, 0, MSG_BUFFER_SIZE );

	recv ( this->sock, recbuf, MSG_BUFFER_SIZE, 0 );

	free(recbuf); recbuf = NULL;

	this->dc->enableCaching(true);

	this->dc->start();
	
	return true;
}

BenchmarkSession::~BenchmarkSession()
{
	this->dc->wait();

	if (!this->mp4d) delete this->mp4d;
	if (!this->dc) delete this->dc;
	if (!this->rtp) delete this->rtp;
	if (!this->stats) delete this->stats;
	if (!this->gt) delete this->gt;
	if (!this->pl) delete this->pl;
}
