 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	bmjob.cpp                                                    *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "bmjob.hpp"
#define STD_PLAYERID "dude"

/** Default constructure with rtsp://localhost/ as destination */
BMJob::BMJob()
{
	dprintf_full("\033[31mJob::Job started\033[0m\n");
	this->audioSampleRate = 0;
	this->audio_channels = 0;
	this->videoWidth = 0;
	this->videoHeight = 0;
	this->videoBitrate = 0;
	this->audioBitrate = 0;
	this->url = new Url("rtsp://localhost/");
	if (!this->url){
		dprintf_full("URL not set\n");
		this->available = false;
	}
	else {
		dprintf_full("URL set\n");
		if (this->url->getFileName() == NULL){
			dprintf_err("Specified URL has no filename\n");
			this->available = false;
		}
		else {
			dprintf_err("Specified URL has a filename\n");
		}
	}
	this->described = false;
	this->customValues = false;
	/* quite optimistic but anyway... */
	this->available = false;
	this->hasAudio = false;
	this->hasVideo = false;
}

/** Custom-URL-Constructor */
BMJob::BMJob(const char* dest)
{
	dprintf_full("\033[31mJob::Job started\033[0m\n");
	this->audioSampleRate = 0;
	this->audio_channels = 0;
	this->videoWidth = 0;
	this->videoHeight = 0;
	this->videoBitrate = 0;
	this->audioBitrate = 0;
	this->duration = 0;
	this->url = new Url(dest);
	if (!this->url){
		dprintf_full("URL not set\n");
		this->available = false;
	}
	else {
		dprintf_full("URL set\n");
		if (this->url->getFileName() == NULL){
			dprintf_err("Specified URL has no filename\n");
			this->available = false;
		}
		else {
			dprintf_err("Specified URL has a filename\n");
		}
	}
	this->described = false;
	this->customValues = false;
	this->hasAudio = false;
	this->hasVideo = false;
}

/** Basically only erases the URL */
BMJob::~BMJob()
{
	dprintf_full("\033[31mJob::~Job destroying\033[0m\n");
	if(url)
		delete this->url;
	this->url = NULL;
	dprintf_full("\033[31mJob::~Job destroyed\033[0m\n");
}

/** Sorting jobs by width*height */
bool BMJob::operator<( BMJob other)
{
	return this->getDuration()*(this->getVideoBitrate()+this->getAudioBitrate()) < other.getDuration()*(other.getVideoBitrate()+other.getAudioBitrate());
}
/** Returns the URL of the Job

   \return URL of the Job
   */
Url* BMJob::getURL()
{
	dprintf_full("\033[31mJob::getURL\033[0m\n");
	return this->url;
}

bool BMJob::setURL(const char* dest)
{
	Url newUrl(dest);
	if (newUrl.getFileName() == NULL)
		return false;
	if(url)
		delete (this->url);
	this->url = new Url(dest);
	this->described = false;
	return true;
}

/**
  \return true if the stream has an audio component
  \return false if not
  */
bool BMJob::hasAudioComponent()
{
	dprintf_full("\033[31mJob::hasAudioComponent\033[0m\n");
	if (!this->described){
		dprintf_full("\033[31mJob::hasAudioComponent DESCRIBE necessary\033[0m\n");
		this->describe();
	}
	return this->hasAudio;
}

/**
  \return true if the stream has a video component
  \return false if not
  */
bool BMJob::hasVideoComponent()
{
	dprintf_full("\033[31mJob::hasVideoComponent\033[0m\n");
	if (!this->described){
		dprintf_full("\033[31mJob::hasVideoComponent DESCRIBE necessary\033[0m\n");
		this->describe();
	}
	return this->hasVideo;
}

u32 BMJob::getVideoWidth()
{
	dprintf_full("\033[31mJob::getVideoWidth\033[0m\n");
	if (!this->described){
		dprintf_full("\033[31mJob::getVideoWidth DESCRIBE necessary\033[0m\n");
		this->describe();
	}
	return this->videoWidth;
}

u32 BMJob::getVideoHeight()
{
	dprintf_full("\033[31mJob::getVideoHeight\033[0m\n");
	if (!this->described){
		dprintf_full("\033[31mJob::getVideoHeight DESCRIBE necessary\033[0m\n");
		this->describe();
	}
	return this->videoHeight;
}

u32 BMJob::getAudioSampleRate()
{
	dprintf_full("\033[31mJob::getAudioSampleRate\033[0m\n");
	if (!this->described){
		dprintf_full("\033[31mJob::getAudioSampleRate DESCRIBE necessary\033[0m\n");
		this->describe();
	}
	return this->audioSampleRate;
}

u8 BMJob::getAudioChannels()
{
	dprintf_full("\033[31mJob::getAudioChannels\033[0m\n");
	if (!this->described)
		dprintf_full("\033[31mJob::getAudioChannels DESCRIBE necessary\033[0m\n");
		this->describe();
	return this->audio_channels;
}

u32 BMJob::getAudioBitrate()
{
	if (!this->described)
		this->describe();
	return this->audioBitrate;
}

u32 BMJob::getVideoBitrate()
{
	if (!this->described)
		this->describe();
	return this->videoBitrate;
}

u64 BMJob::getDuration(){
	if (!this->described)
		this->describe();
	return this->duration;
}
/*void BMJob::setVideoWidth(u32 value)
{
}

void BMJob::setVideoHeight(u32 value)
{
}

void BMJob::setAudioSampleRate(u32 value)
{
}

void BMJob::setAudioChannels(u32 value)
{
}*/

/** returns a string incl. all import data of a job
  \todo xml
  */
char* BMJob::toString(char* buffer)	// TODO numAudioChannels, etc.
{
	dprintf_full("\033[31mJob::toString(char*)\033[0m\n");
	return toString(buffer,MAX_XML_LEN);
}

char* BMJob::toString(char* buffer, int max_len)
{
	dprintf_full("\033[31mJob::toString( %p, %i )\033[0m\n", buffer, max_len);
	this->describe();
	char* format = (char*)malloc(max_len);
	assert(format);
	memset(format, 0, max_len);
	if ( max_len > snprintf(format, max_len, "%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s%s",
		JOB_START, URL_print , WIDTH , HEIGHT ,  AUDIO_FREQ , AUDIO_CHANN , DURATION , 
		VIDEO_BITRATE , AUDIO_BITRATE , JOB_END) )
	{
		memset(buffer, 0, max_len);
		if ( max_len > snprintf(buffer, max_len, format, this->url->toString() , this->videoWidth , this->videoHeight , this->audioSampleRate , this->audio_channels , 0 , this->videoBitrate , this->audioBitrate) )		// HACK !!!!!!!!!!!!!!! HACK !!!!!!!!!!!!!!!!!
		{
			free(format);
			dprintf_full("\033[31mJob:: snprintf ok\033[0m\n");
			return buffer;
		}
		else	
		{
			dprintf_err("\033[31mJob::toString\033[0m\n");
			return NULL;
		}
	}
	else
	{
		free(format);
		dprintf_err("\033[31mJob::toString\033[0m\n");
		return NULL;
	}
	
}

bool BMJob::fromString(char* buffer)	// FIXME change to "%u %u %u %u..."
{
	
	dprintf_full("\033[31mJob::fromString\033[0m\n");
	const char* next = buffer;
	char* urlbuf = (char*)malloc(1024*sizeof(char));
	assert(urlbuf);
	char* end = NULL;
	if ( next == strstr(next, JOB_START) )
	{
		next = (const char*)next+strlen(JOB_START);
	}
	else
	{
		dprintf_err("\033[31mJob::fromString1\033[0m\n");
		free(urlbuf);
		return false;
	}

	if ( (next = strstr(next, URL_scan)) )
	{
                next = (const char*)next+strlen(URL_scan);
	}
	else
	{
		dprintf_err("\033[31mJob::fromString2\033[0m\n");
		free(urlbuf);
		return false;
	}

	if ( NULL != (end=strchr(next, '\n')) )
	{
		memset(urlbuf,0,1024);
		if ( NULL == memcpy(urlbuf, next, end-next) )
		{
			dprintf_err("\033[31mJob::fromString3\033[0m\n");
			free(urlbuf);
			return false;
		}
		else
		{
			if(url)
				delete url;
			this->url = new Url((const char*)urlbuf);
			next = (char*)end+1;
		}
	}
	else
	{
		dprintf_err("\033[31mJob::fromString4\033[0m\n");
		free(urlbuf);
		return false;
	}
	free(urlbuf);
	//we have a valid url!
	char* format = (char*)malloc(MAX_XML_LEN);
	assert(format);
        memset(format, 0, MAX_XML_LEN);

	if ( 0 < snprintf(format, MAX_XML_LEN, "%s%s%s%s%s%s%s", WIDTH, HEIGHT , AUDIO_FREQ , AUDIO_CHANN , DURATION,  VIDEO_BITRATE , AUDIO_BITRATE ) )
	{
		
		u32 b,c,e,f , vb, ab;
		u64 g;
		
		printf("Scanned: %i\n", sscanf( next, format, &b, &c, &e, &f , &g , &vb , &ab) );
		
		if ( 7 == sscanf( next, format, &b, &c, &e, &f , &g , &vb , &ab))
		{
			//this->url = new Url(a);
			printf("------------------------read videoWidth %i, videoHeight %i----------------------------------\n", b, c);
			this->videoWidth = b;
			this->videoHeight = c;
			this->audioSampleRate = e;
			this->audio_channels = f;
			this->duration = g;
			this->audioBitrate = ab;
			this->videoBitrate = vb;
			
			free(format);
			dprintf_full("\033[31mJob:: sscanf\033[0m\n");
			//this->described = true;
			return true;
		}
		else
		{
			free(format);
			dprintf_err("\033[31mJob::fromString5\033[0m\n");
			return true; //we have a valid url!
		}
	}
	else
	{
		dprintf_err("\033[31mJob::fromString6\033[0m\n");
		return false;
	}
}

bool BMJob::isAvailable()
{
	if(!this->described){
		this->describe();
	}
	return this->available;
}

/** Tries to execute a describe for the target file
  \param force Enforces a new describe even if a DESCRIBE has already been requested
  \return true Describe succeeded or has already succeeded
  \return false Describe failed or custom values are set and describe is not enforced
  */
bool BMJob::describe(bool force)
{
	dprintf_full("\033[31mJob::describe\033[0m\n");
	/* If a describe is not forced and the file has already been
	   described, return true */
	if (this->described && this->available && force==false){
		dprintf_full("\033[31mJob::describe already described and re-describe not enforced\033[0m\n");
		return true;
	}
	if (this->customValues && force==false){
		dprintf_full("\033[31mJob::describe job has custom values and describe is not enforced\033[0m\n");
		return false;
	}
	if (this->described && !this->available && force==false){
		dprintf_full("\033[31mJob::describe target is not available and re-describe not enforced\033[0m\n");
		return false;
	}
	VideoESInfo* videoESInfo = NULL;
	AudioESInfo* audioESInfo = NULL;
	bool retVal = true;
	//RTSP* rtsp = new RTSP();
	ContainerInfo* container = NULL;
	ClientSession* session = new ClientSession(this->url->toString(),NULL,NULL,(TerminalCapabilities* )NULL,false);
	dprintf_full("Client session created\n");	
	if (session->connect()){
		dprintf_full("\033[31mJob::describe connection successful\033[0m\n");
		/* Read ContainerInfo and store the resulting data
		   into the local variables */
		if(session->getContainerInfo()) {
		 	container=((ContainerInfo*)session->getContainerInfo())->clone();
		}
		else {
		   	delete session;
		   	return false;	
		}
		videoESInfo = container->getFirstVisualES();
		audioESInfo = container->getFirstAudioES();
		if (videoESInfo != NULL){
			dprintf_full("\033[31mJob::describe job has video component\033[0m\n");
			this->hasVideo = true;
			/* Video components: */
			dprintf_full("\033[31mJob::describe getHeight\033[0m\n");
			this->videoHeight = videoESInfo->getHeight();
			dprintf_full("\033[31mJob::describe getWidth\033[0m\n");
			this->videoWidth = videoESInfo->getWidth();
			this->videoBitrate = videoESInfo->getAvgBandwidth();
			this->duration = videoESInfo->getDuration() / videoESInfo->getMediaTimeScale();
		}
		else {
			this->videoWidth = 0;
			this->videoHeight = 0;
			this->videoBitrate = 0;
		}
		
		if (audioESInfo != NULL){
			dprintf_full("\033[31mJob::describe job has audio component\033[0m\n");
			this->hasAudio = true;
			/* Audio components: */
			dprintf_full("\033[31mJob::describe getAudioChannels\033[0m\n");
			this->audio_channels = audioESInfo->getAudioChannels();
			dprintf_full("\033[31mJob::describe getSampleRate\033[0m\n");
			this->audioSampleRate = audioESInfo->getSampleRate();
			this->audioBitrate = audioESInfo->getAvgBandwidth();
			this->duration = audioESInfo->getDuration() / audioESInfo->getMediaTimeScale();
		}
		else {
			this->audio_channels = 0;
			this->audioSampleRate = 0;
			this->audioBitrate = 0;
		}
		if (audioESInfo == NULL && videoESInfo == NULL)
			duration = 0;
		this->described = true;
		this->available = true;		
		this->customValues = false;
		delete session;
	}
	else{
		delete session;
		dprintf_err("\033[31mJob::describe Connection/Describe failed\033[0m\n");
		retVal = false;
		this->described = true;
		this->available = false;
	}

	return retVal;
}
