 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	bmjob.hpp                                                    *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/
#ifndef MUVIMARK_BMJOB_HPP
#define MUVIMARK_BMJOB_HPP


#include "markglobal.hpp"
#include "net/Url.hpp"
#include "net/RTSP.hpp"
#include "ContainerInfo.hpp"
#include "VideoESInfo.hpp"
#include "AudioESInfo.hpp"
#include "net/ClientSession.hpp"

/*********************************************************************
 Available (at least the IMO important ones ;-) ) public functions:
 =====================================================================
	char* toString()
	bool fromString(char*)
	int getVideoHeight()
	int getVideoWidth()
	u32 getAudioSampleRate()
	--------------------------------------------------------------
	URL* getURL()
	u64 getDuration()
	u64 getDurationInMs ()
**********************************************************************/
//class Job : public ContainerInfo

class BMJob
{
	protected:
		u32 videoHeight;
		u32 videoWidth;
		u32 audioSampleRate;
		u8  audio_channels;
		u32 videoBitrate;
		u32 audioBitrate;
		Url* url;
		bool hasAudio;
		bool hasVideo;
		bool described;
		/** is TRUE when user set a custom value and FALSE if this values is overwritten by a DESCRIBE */
		bool customValues;
		bool available;
		u64 duration;
	public:
		BMJob();
		BMJob(const char* dest);
		~BMJob();
		/** Returns the Url pointer */
		Url* getURL();
		bool setURL(const char* dest);
		bool hasAudioComponent();
		bool hasVideoComponent();
		bool isAvailable();
		u32 getVideoWidth();
		u32 getVideoHeight();
		u32 getAudioSampleRate();
		u8 getAudioChannels();
		u32 getAudioBitrate();
		u32 getVideoBitrate();
		u64 getDuration();
		/*void setVideoWidth(u32 value);
		void setVideoHeight(u32 value);
		void setAudioSampleRate(u32 value);
		void setAudioChannels(u32 value);*/
		/** Connects to the target server and tries to
		  execute a describe for the target file */
		bool describe(bool force=false);
		char* toString( char* buffer);
		char* toString( char* buffer , int max_len );
		/** returns true as long asa valid url is found, which can be used
		  * to do a  describe */
		bool fromString(char* buffer);
		bool operator<(const BMJob other);
};

#endif
