 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	joblist.cpp                                                    *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "bmjoblist.hpp"
#include <fstream>

BMJobList::BMJobList()
{
}

/** removes every element from the list and finally the list itself */
BMJobList::~BMJobList()
{
	/* Delete all jobs in and from the list */
	while(this->joblist.size() > 0){
		delete(*joblist.begin());
		joblist.pop_front();
	}
}

/** returns the number of elements in the internal list
  \return number of elements in the list
  */
u32 BMJobList::size()
{
	return this->getList()->size();
}

/** add a new job to the list
  Doesn't add a job if its URL doesn't include a filename
  \return true always
  */
bool BMJobList::addJob(BMJob* job)
{
	if(job->getURL()->getFileName() == NULL){
		dprintf_err("\033[33mBMJobList::addJob URL of the job has no filename. No job added\033[0m\n");		
		return false;
	}
	dprintf_full("\033[33mBMJobList::addJob Adding job (%s)\033[0m\n",job->getURL()->toString());
	this->joblist.push_back(job);
	dprintf_full("\033[33m[BMJobList:%p] Current size: %i\033[0m\n",this,this->joblist.size());
	return true;
}

/** currently removes the first matching job it finds
  \return true if a job matched and was removed
  \return false if no matching job was found
  */
bool BMJobList::removeJob(BMJob* job)
{
	bool found = false;
	std::list<BMJob*>::iterator currentElement;
	currentElement = this->joblist.begin();
	while(currentElement != joblist.end()){
		if(job == *currentElement){
			found = true;
			dprintf_full("\033[33m[JobList:%p] JobToBeRemoved found\033[0m\n",this); 
			break;
		}
		currentElement++;
	}
	if(found){
		this->joblist.erase(currentElement);
			dprintf_full("\033[33m[JobList:%p] JobToBeRemoved removed\033[0m\n",this); 
		return true;
	}
	else{
		return false;
	}
}

/** Returns pointer to the internal std::list<Job*>
  \return std__list<Job*> joblist
  */
std::list<BMJob*>* BMJobList::getList()
{
	return &(this->joblist);
}

/** Sorts the internal list according to the bmjob->operator<
 */
void BMJobList::sort(){
	joblist.sort();
}
/** Imports to config file (JL_FILE)
  \return true if import was succesful
  \return false if not
  */
bool BMJobList::fromConfig(const char* jlFile)
{
	/* Delete all elements from the list */
	dprintf_full("Joblist::fromConfig %s\n",jlFile);
	
	while(this->joblist.size() > 0)
	{
		this->joblist.pop_back();
	}
	/* Refill */
	FILE* cfg_file = fopen( jlFile, "r" );
	if ( cfg_file != NULL ) 
	{
		char* buffer = (char*) malloc(MAX_CFG_LEN);
	//	while( !feof(cfg_file) )
	//	{
			fread( (void*)buffer, 1, MAX_CFG_LEN, cfg_file );
			char* next = buffer;
			
			while ( NULL != (next=strstr(next, JOB_START)) )
			{
				BMJob* newjob = new BMJob();
				if(newjob->fromString( next )) {
					dprintf_full("JobList->addGroup\n");
					this->addJob( newjob );
				}
				else
					delete newjob;	
				next+=5;
			}

	//	}
		fclose ( cfg_file );
		free(buffer);
		dprintf_full("Joblist::fromConfig (%i)ok\n",joblist.size()) ;
		return true;
	}
	else
	{
		dprintf_err("Joblist::fromConfig error\n");
		return false;
	}
}

/** Exports joblist to config file (JL_FILE)
  \return true if export is succesful
  \return false if not
  */
bool BMJobList::toConfig()
{
	char cbuf[1024];
	if(joblist.empty())
		return false;
	ofstream fout(JL_FILE);
	if(!fout){
		dprintf_full("Failed to open %s for writing\n",JL_FILE);
		return false;
	}
	std::list<BMJob*>::iterator currentElement;
	currentElement = this->joblist.begin();
	while(currentElement != this->joblist.end()){
		memset(cbuf,0,1024);
		fout << ((BMJob*)*currentElement)->toString(cbuf,1024);
		currentElement++;
	}
	fout.close();
	return true;
}

/** returns the num'th job from the list */
BMJob* BMJobList::getJob(u32 num)
{
	if(num >= joblist.size()) return NULL;
	std::list<BMJob*>::iterator currentElement;
	currentElement = this->joblist.begin();
	u32 i = 0;
	while(currentElement != joblist.end() && i <= num){
		if(i == num) return *currentElement;
		i++;
		currentElement++;
	}
	return NULL;
}

