/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   cg.cpp                                                      *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

//#include <stdlib.h>
//#include <string.h>
#include "cg.hpp"

ClientGroup::ClientGroup():TerminalCapabilities()
{
	dprintf_full("\033[32mClientGroup::construct\033[0m\n");
	this->name=(char*)malloc(MAX_STR_LEN);
	assert(name);
}

ClientGroup::~ClientGroup()
{
	dprintf_full("\033[32mClientGroup::destruct\033[0m\n");
	
	if (!this->name)
		free(this->name);
	this->name=NULL;
}

void ClientGroup::setName(const char* name)
{
	dprintf_full("\033[32mClientGroup::setName\033[0m\n");
	assert(name);
	assert(this->name);
	memset( this->name, 0, MAX_STR_LEN);
	memcpy( this->name, name, strlen(name) );	// FIXME change to strncp
	dprintf_full("\033[32mClientGroup::finished\033[0m\n");
}

char* ClientGroup::getName()
{
	dprintf_full("\033[32mClientGroup::getName\033[0m\n");
	return this->name;
}
/*
void ClientGroup::setNumber(int number)
{
	dprintf_full("\033[32mClientGroup::setNumber\033[0m\n");
	this->number = number;
}

int ClientGroup::getNumber()
{
	dprintf_full("\033[32mClientGroup::getNumber\033[0m\n");
	return this->number;
}
*/

char* ClientGroup::toString(char* buffer, int max_len)	// TODO test
{
	dprintf_full("\033[32mClientGroup::toString\033[0m\n");
	assert(buffer);
	char* format = (char*)malloc(max_len);
	assert(format);
	memset(format, 0, max_len);
	if ( max_len > snprintf(format, max_len, "%s%s%s%s%s%s%s%s%s%s%s%s%s",CG_START, NAME_print, MAX_DEC_BUFF_SIZE, MAX_BITRATE, BITS_PER_PIXEL, 
								COLOR, WIDTH, HEIGHT, VIDEO_FPS, AUDIO_CHANN, AUDIO_BPS, AUDIO_FREQ, CG_END ) )
	{
		memset(buffer, 0, max_len);
		if ( max_len > snprintf(buffer, max_len, format, this->name, this->maxDecoderBufferSizeInByte, this->maxDecoderBitRateInBit,
				this->displayBitsPerPixel, this->colorDisplay, this->displayWidth, this->displayHeight, this->displayRefreshRate,
				this->numAudioChannels, this->audioBitsPerSample, this->audioSamplingFrequency ) )
		{
			free(format);
			dprintf_full("\033[32mClientGroup:: snprintf ok\033[0m\n");
			return buffer;
		}
		else	
		{
			dprintf_err("\033[32mClientGroup::toString\033[0m\n");
			return NULL;
		}
	}
	else
	{
		free(format);
		dprintf_err("\033[32mClientGroup::toString\033[0m\n");
		return NULL;
	}
}

bool ClientGroup::fromString(const char* buffer)	// TODO test
{
	dprintf_full("\033[32mClientGroup::fromString\033[0m\n");
	assert(buffer);
	const char* next = buffer;
	char* end = NULL;
	if ( next == strstr(next, CG_START) )
	{
		next = (const char*)(unsigned int)next+strlen(CG_START);
	}
	else
	{
		dprintf_err("\033[32mClientGroup::toString\033[0m\n");
		return false;
	}

	if ( next == strstr(next, NAME_scan) )
	{
                next = (const char*)(unsigned int)next+strlen(NAME_scan);
	}
	else
	{
		dprintf_err("\033[32mClientGroup::toString\033[0m\n");
		return false;
	}

	if ( NULL != (end=strchr(next, '\n')) )
	{
		memset(this->name,0,1024);
		if ( NULL == memcpy(this->name, next, end-next) )
		{
			dprintf_err("\033[32mClientGroup::toString\033[0m\n");
			return false;
		}
		else
		{
			next = (char*)(unsigned int)end+1;
		}
	}
	else
	{
		dprintf_err("\033[32mClientGroup::toString\033[0m\n");
		return false;
	}

	char* format = (char*)malloc(MAX_XML_LEN);
	assert(format);
        memset(format, 0, MAX_XML_LEN);

	if ( 0 < snprintf(format, MAX_XML_LEN, "%s%s%s%s%s%s%s%s%s%s%s", MAX_DEC_BUFF_SIZE, MAX_BITRATE, BITS_PER_PIXEL, COLOR, WIDTH, HEIGHT, VIDEO_FPS, AUDIO_CHANN, AUDIO_BPS, AUDIO_FREQ, CG_END ) )
	{
		
		u32 a,b,c,d,e,f,g,h,i,j;

		if ( 10 == sscanf( next, format, &a, &b, &c, &d, &e, &f, &g, &h, &i, &j))

		{
			this->maxDecoderBufferSizeInByte = a;
			this->maxDecoderBitRateInBit = b;
			this->networkCapacityInByte = b;
			this->minGuaranteedNetworkCapacityInByte = b;
			this->displayBitsPerPixel = c;
			this->colorDisplay = d;
			this->displayWidth = e;
			this->displayHeight = f;
			this->displayRefreshRate = g;
			this->numAudioChannels = h;
			this->audioBitsPerSample = i;
			this->audioSamplingFrequency = j;
			
			free(format);
			dprintf_full("\033[32mClientGroup:: sscanf\033[0m\n");
			return true;
		}
		else
		{
			free(format);
			dprintf_err("\033[32mClientGroup::toString\033[0m\n");
			return false;
		}
	}
	else
	{
		dprintf_err("\033[32mClientGroup::toString\033[0m\n");
		return false;
	}
}


