/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   cg_interface.ui.h                                           *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/


#include <qfiledialog.h>
#include <qfile.h>
#include <qtextstream.h>

#include "cg_list.hpp"

ClientGroup* actgrp;

void mycg::init()
{
	dprintf_full("\033[34mcg_interface::start init\033[0m\n");
	int i;
	
	resolutionbox->insertItem( "[ ??? x ??? ]" );

	for ( i=0; i < (int)(sizeof(res)/sizeof(res[0]))/2; i++ )
	resolutionbox->insertItem( "[ " + QString::number(res[i*2]) + " x " + QString::number(res[i*2+1]) + " ]" );
    
	for ( i=0; i<7; i++)
		sampleFreq->insertItem( QString::number( freq[i] ) );
	
//	actgrp = new ClientGroup();
	
	this->cgl = new ClientGroupList();
	((ClientGroupList*)cgl)->fromConfig();

	int index=0;
	ClientGroup* temp = NULL;
	while ( NULL!=(temp = ((ClientGroupList*)cgl)->getGroup(index)) )
	{
		this->grouplist->insertItem( QString(temp->getName() ) );
		index++;
	}
	dprintf_full("\033[34mcg_interface::init finished\033[0m\n");
}

void mycg::resolution_update(int i )
{
	dprintf_full("\033[34mcg_interface::resolution_update\033[0m\n");
	if ( i > 0 )
	{
		i--;
		this->xBox->setValue( res[i*2] );
		this->yBox->setValue( res[i*2+1] );
	}
	else
	{
		this->xBox->setValue( 0 );
		this->yBox->setValue( 0 );
	}
}

void mycg::resbox_update()
{
	dprintf_full("\033[34mcg_interface::resbox_update\033[0m\n");
	int i;

	for( i=0; i < (int)(sizeof(res)/sizeof(res[0]))/2; i++ )
	{
		if ( (this->xBox->value() == res[i*2]) && (this->yBox->value() == res[i*2+1]) )
		{
			this->resolutionbox->setCurrentItem(++i);
			return;
		}
	}
	this->resolutionbox->setCurrentItem(0);
}

void mycg::add_group()
{
	dprintf_full("\033[34mcg_interface::start add_group\033[0m\n");

	actgrp = new ClientGroup();

	assert(actgrp);
	
	update_gui2class();

	 ( (ClientGroupList*)cgl )->addGroup( actgrp );
	
	char* buffer = (char*)malloc(MAX_XML_LEN);
    
    	this->grouplist->insertItem( QString(actgrp->getName()) );
	FILE* cfg_file = fopen( ".clientgroup.conf", "a+" );
	fputs( actgrp->toString(buffer , MAX_XML_LEN), cfg_file );
	fclose ( cfg_file );
	free (buffer);
	dprintf_full("\033[34mcg_interface::add_group finished\033[0m\n");
}

void mycg::del_group()
{
	dprintf_full("\033[34mcg_interface::del_group\033[0m\n");
	( (ClientGroupList*)cgl )->removeGroup(  ( (ClientGroupList*)cgl )->getGroup( this->grouplist->currentItem() )  ); 
	this->grouplist->removeItem( this->grouplist->currentItem() );
	((ClientGroupList*)cgl)->toConfig();
}

void mycg::modify_group()
{
	dprintf_full("\033[34mcg_interface::update_group\033[0m\n");
	this->grouplist->changeItem( this->groupname->text(), this->grouplist->currentItem() );
	actgrp = ((ClientGroupList*)cgl)->getGroup( this->grouplist->currentItem() );
	this->update_gui2class();
	 ((ClientGroupList*)cgl)->toConfig();
}

void mycg::update_gui2class()
{
	dprintf_full("\033[34mcg_interface::start update_gui2class\033[0m\n");
	
	if ( !actgrp )
	{
		dprintf_err("\033[34mcg_interface::nothing to update\033[0m\n");
		return;
	}
 	actgrp->setName( (this->groupname->text()).ascii());
	
	actgrp->setMaxDecoderBufferSizeInByte ( (const u32) this->maxDecBufSize->value() );
	actgrp->setNetworkCapacityInByte(  (const u32) this->maxDecBufSize->value()*2 );	// FIXME HACK !
	actgrp->setMinGuaranteedNetworkCapacityInByte( (const u32) this->maxDecBufSize->value() ); // FIXME  HACK !
	actgrp->setMaxDecoderBitRateInBit ( (const u32) this->maxDecBitrate->value() );
	actgrp->setDisplayBitsPerPixel ( (const u32) this->displayBits->value() );
	actgrp->setColorDisplay ( this->colorcb->isChecked() );
	actgrp->setDisplayWidth ( this->xBox->value() );
	actgrp->setDisplayHeight ( this->yBox->value() );
	actgrp->setDisplayRefreshRate ( this->fps->value() );
	actgrp->setNumAudioChannels ( this->audioChann->value() );
	actgrp->setAudioBitsPerSample ( this->audioBits->value() );
	actgrp->setAudioSamplingFrequency( freq[ this->sampleFreq->currentItem() ] );
	dprintf_full("\033[34mcg_interface::update_gui2class finished\033[0m\n");
}

void mycg::update_class2gui(int index)
{
	dprintf_full("\033[34mcg_interface::start update_class2gui\033[0m\n");
	// create list in Qt-GUI-class
	// read all groups from file to the list
	// work on the list, write / read only on request.
	// use file only for persistent storage

	assert(cgl);

	actgrp = ((ClientGroupList*)cgl)->getGroup(index);

	assert(actgrp);

	this->groupname->setText( QString( actgrp->getName() ) );
	this->maxDecBufSize->setValue ( actgrp->getMaxDecoderBufferSizeInByte () );
	this->maxDecBitrate->setValue ( actgrp->getMaxDecoderBitRateInBit() );
	this->displayBits->setValue ( actgrp->getDisplayBitsPerPixel() );
	
	this->colorcb->setChecked ( (bool) actgrp->getColorDisplay() );
	
	this->colorcb->setChecked( (bool)actgrp->getColorDisplay() );

	this->xBox->setValue ( actgrp->getDisplayWidth() );
	this->yBox->setValue ( actgrp->getDisplayHeight() );
	this->fps->setValue ( actgrp->getDisplayRefreshRate() );
	this->audioChann->setValue ( actgrp->getNumAudioChannels() );
	this->audioBits->setValue ( actgrp->getAudioBitsPerSample() );
	this->sampleFreq->setCurrentText ( QString::number( actgrp->getAudioSamplingFrequency() ) );

	dprintf_full("\033[34mcg_interface::update_class2gui finished\033[0m\n");
}


