/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   cg_list.cpp                                                 *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "cg_list.hpp"
#include "cg.hpp"

ClientGroupList::ClientGroupList()
{
	dprintf_full("\033[33mClientGroupList::constructor\033[0m\n");
}

ClientGroupList::~ClientGroupList()
{
	dprintf_full("\033[33mClientGroupList::destructor\033[0m\n");
	list<ClientGroup*>::iterator next = this->cglist.begin();
	while( next!=this->cglist.end() )
	{

		next = this->cglist.begin();
		if ( *next == NULL )
			return;

		this->cglist.erase(next);
		delete(*next);
		dprintf_full("\033[33mClientGroupList::removeGroup\033[0m\n");
	}
}

ClientGroup* ClientGroupList::getGroup(int id)
{
	dprintf_full("\033[33mClientGroupList::getGroup\033[0m\n");
	list<ClientGroup*>::iterator next = this->cglist.begin();
	while( next!=this->cglist.end() && id>0 )
	{
		next++;
		id--;
	}
	if ( id == 0 && next!=this->cglist.end())
	{
		dprintf_full("\033[33mClientGroupList::found element\033[0m\n");
		return *next;
	}
	else
	{
		dprintf_err("\033[33mClientGroupList::End of list\033[0m\n");
		return NULL;
	}
}

bool ClientGroupList::addGroup(ClientGroup* new_cg)
{
	dprintf_full("\033[33mClientGroupList::addGroup\033[0m\n");
	assert(new_cg);
	this->cglist.push_back(new_cg);
	return true;				// TODO check this
}
	
bool ClientGroupList::removeGroup(ClientGroup* to_remove)
{
	dprintf_full("\033[33mClientGroupList::removeGroup\033[0m\n");
//	assert(to_remove);
	if (!to_remove)
	{
		dprintf_err("\033[33mClientGroupList::nothing to remove\033[0m\n");
		return false;
	}

	list<ClientGroup*>::iterator next = this->cglist.begin();
	while( next!=this->cglist.end() )
	{
		if ( (*next) == to_remove )
		{
			delete(*next);
			this->cglist.erase(next);
			dprintf_full("\033[33mClientGroupList::removed element\033[0m\n");
			return true;		// TODO check this
		}
		else
		{
			next++;
		}
	}
	dprintf_err("\033[33mClientGroupList::removed nothing\033[0m\n");
	return false;				// TODO check this
}

bool ClientGroupList::toConfig()
{
	dprintf_full("\033[33mClientGroupList::write2Config\033[0m\n");
	char* cgbuffer = (char*) malloc(MAX_XML_LEN);
	assert(cgbuffer);
	list<ClientGroup*>::iterator next = this->cglist.begin();
	FILE* cfg = fopen( CGL_FILE , "w+" );
	assert(cfg);
	while( next!=this->cglist.end() )
	{
		fputs( (*next)->toString(cgbuffer, MAX_XML_LEN), cfg );
		next++;
	}
	fclose(cfg);
	dprintf_full("\033[33mClientGroupList::configed\033[0m\n");
	return true;
}

bool ClientGroupList::fromConfig()
{
	dprintf_full("\033[33mClientGroupList::fromConfig\033[0m\n");
	FILE* cfg_file = fopen( CGL_FILE, "r+" );		// hardcoded ! may not ok ?
	if ( cfg_file != NULL ) 
	{
		char* buffer = (char*) malloc(MAX_CFG_LEN);
		assert(buffer);
	//	while( !feof(cfg_file) )
	//	{
			fread( (void*)buffer, 1, MAX_CFG_LEN, cfg_file );
			char* next = buffer;
			
			while ( NULL != (next=strstr(next, CG_START)) )
			{
				ClientGroup* newgrp = new ClientGroup();
				newgrp->fromString( next );
				dprintf_full("\033[33mClientGroupList->addGroup\033[0m\n");
				this->addGroup( newgrp );
				next = next+=5;
			}

	//	}
		fclose ( cfg_file );
		free(buffer);
		dprintf_full("\033[33mClientGroupList::read ok\033[0m\n");
		return true;
	}
	else
	{
		dprintf_err("\033[33mClientGroupList::read error\033[0m\n");
		return false;
	}
	
}


