/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   jl_interface.ui.h                                           *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "bmjoblist.hpp"
//#include <qlistbox.h>
void jl_interface::init()
{
    this->joblist = (void*)new BMJobList();
    this->pb_describe->setEnabled(false);
    this->pb_describeAll->setEnabled(false);
    /* Get BMJobList from file */
    ((BMJobList*)(this->joblist))->fromConfig();
    /* Now fill the listbox */
    this->lb_joblist->clear();
    for (unsigned int i = 0 ; i < ((BMJobList*)(this->joblist))->size() ; i++) {
	this->lb_joblist->insertItem(((BMJobList*)(this->joblist))->getJob(i)->getURL()->toString());
    }
    this->pb_describeAll->setEnabled(true);
    this->pb_update->setEnabled(false);
    this->pb_remove->setEnabled(false);
}



void jl_interface::pb_describe_clicked()
{
    dprintf("\033[33mjl_inteface::pb_describe_clicked\033[0m\n");    
    int currentItem = this->lb_joblist->currentItem();
    if (currentItem == -1){
	dprintf_err("\033[33mjl_inteface::pb_describe_clicked WOW, you were able to click that button without having a selected item in the list\033[0m\n");
	return;
    }
    BMJob* currentJob = ((BMJobList*)(this->joblist))->getJob(currentItem);
    if (currentJob->describe(true)){
	/* Update element in the list */
	/* no need as the URL shouldn't be changed by a DESCRIBE 
	   but update the other GUI elements */
	this->le_videoWidth->setText(QString::number(currentJob->getVideoWidth()));
	this->le_videoHeight->setText(QString::number(currentJob->getVideoHeight()));
	this->le_audioSampleRate->setText(QString::number(currentJob->getAudioSampleRate()));
	this->le_audioChannels->setText(QString::number(currentJob->getAudioChannels()));
    }
    else {
	dprintf_err("jl_interface::pb_describe_clicked DESCRIBE failed\n");
	/* Paint the list element red ... ok, perhaps adding a small pixmap to that
element would be simpler :S *... ok, for the time being I will simply append a [!] in front 
of the link :S */
	this->lb_joblist->changeItem("[!] "+QString(currentJob->getURL()->toString()),currentItem);
    }
    /* Keep selection */
    this->lb_joblist->setSelected(this->lb_joblist->item(currentItem),true);
}


void jl_interface::pb_describeAll_clicked()
{
    dprintf_full("\033[33mjl_inteface::pb_describeAll_clicked\033[0m\n");
    u32 i = 0;
    while ( i <((BMJobList*)(this->joblist))->size())
    {
	((BMJobList*)(this->joblist))->getJob(i)->describe(true);
	i++;
    }
    /* Keep selection */
    this->lb_joblist->setSelected(this->lb_joblist->item(this->lb_joblist->currentItem()),true);
}


void jl_interface::pb_add_clicked()
{
    /* Create a new job from the URL in the le_url field and append it to the 
       list */
    if (this->le_url->text().isEmpty())
	return;
    BMJob* newJob = new BMJob(le_url->text().ascii());
    if(newJob->getURL()->getFileName() == NULL){
	dprintf("jl_interface::pb_add_clicked URL has no filename. No job added\n");
	return;
    }
    dprintf_full("jl_interface::pb_add_clicked Trying to add %s as new Job\n",le_url->text().ascii());
    ((BMJobList*)this->joblist)->addJob(newJob);
    this->lb_joblist->insertItem(le_url->text());
    /* Update listening */
    char buffer[1024];
    //this->lb_joblist->clear();
    for (unsigned int i = 0 ; i < ((BMJobList*)(this->joblist))->size() ; i++) {
	memset(buffer,0,1024);
	printf("[GUI] %s\n",((BMJobList*)(this->joblist))->getJob(i)->toString(buffer,1024));
	//this->lb_joblist->insertItem(((BMJobList*)(this->joblist))->getJob(i)->getURL()->toString());
    }
    /* Focus last list element (the one that was added) */
    this->lb_joblist->setSelected(this->lb_joblist->item(this->lb_joblist->numRows()-1),true);
    this->lb_joblist->setFocus();
}


void jl_interface::lb_joblist_selectionChanged()
{
    dprintf_full("[GUI] New item selected (%i)\n",this->lb_joblist->currentItem());
    this->pb_describe->setEnabled(true);
    this->pb_update->setEnabled(true);
    this->pb_remove->setEnabled(true);
    /* Update gui elements */
    int currentItem = this->lb_joblist->currentItem();
    if (currentItem == -1){
	this->pb_describe->setEnabled(false);
	this->pb_update->setEnabled(false);
	this->pb_remove->setEnabled(false);	
	dprintf_err("\033[33mjl_inteface::pb_describe_clicked WOW, you were able to click that button without having a selected item in the list\033[0m\n");
	return;
    }
    BMJob* currentJob = ((BMJobList*)(this->joblist))->getJob(currentItem);
    dprintf_full("\033[33m[jl_interface::lb_joblist_selectionChanged] Pointer: %p , Width: %i , Height: %i\033[0m\n",currentJob,currentJob->getVideoWidth(),currentJob->getVideoHeight());
    /* Take the url from the joblist itself and not from the display list:
    this->le_url->setText(this->lb_joblist->item(currentItem)->text());
    */
    this->le_url->setText(currentJob->getURL()->toString());
    this->le_videoWidth->setText(QString::number(currentJob->getVideoWidth()));
    this->le_videoHeight->setText(QString::number(currentJob->getVideoHeight()));    
    this->le_audioSampleRate->setText(QString::number(currentJob->getAudioSampleRate()));
    this->le_audioChannels->setText(QString::number(currentJob->getAudioChannels()));
}


void jl_interface::pb_remove_clicked()
{
    int currentItem = this->lb_joblist->currentItem();
    if (currentItem < 0){
	dprintf_err("[jl_interface::pb_remove_clicked] No job selected... how where you able to click this button...?\n");
	return;
    }
    ((BMJobList*)this->joblist)->removeJob(((BMJobList*)this->joblist)->getJob(currentItem));
    this->lb_joblist->removeItem(currentItem);
    /* Hightlight new currentItem */
    this->lb_joblist->setSelected(this->lb_joblist->item(this->lb_joblist->currentItem()),true);
}
void jl_interface::pb_reset_clicked()
{
    ((BMJobList*)this->joblist)->fromConfig();
    this->lb_joblist->clear();
    for (unsigned int i = 0 ; i < ((BMJobList*)(this->joblist))->size() ; i++) {
	this->lb_joblist->insertItem(((BMJobList*)(this->joblist))->getJob(i)->getURL()->toString());
    }    
}


void jl_interface::pb_update_clicked()
{
    /* Replace the URL in the selected job */
    int currentItem = this->lb_joblist->currentItem();
    if (currentItem < -1){
	dprintf_err("[jl_interface::pb_update_clicked] No item selected for update ...\n");
	return;
    }
    if(((BMJobList*)this->joblist)->getJob(currentItem)->setURL(le_url->text().ascii())){
	this->lb_joblist->changeItem(le_url->text(),currentItem);
    }
    else {
	dprintf_full("[jl_interface::pb_update_clicked] Invalid URL\n");
    }
}


void jl_interface::pb_close_clicked()
{
    ((BMJobList*)this->joblist)->toConfig();
    this->close();
}
