#include <stdio.h>
#include <qapplication.h>
#include <qdialog.h>
#include <qtextedit.h>
#include "joblist.hpp"
#include "job.hpp"
#include <unistd.h>
#include "VThread.hpp"
/** TestJob thread class 

  Basically simple executes the test job in a different thread 
  and prints it's output into the text field of the gui
  */
class JobThread : public VThread
{
	private:
		Job* job;
		QTextEdit* textedit;
	public:
		JobThread(Job* job,QTextEdit* textedit,unsigned int stackSize=0);
		void run();
		
};

JobThread::JobThread(Job* job,QTextEdit* textedit,unsigned int stackSize):QThread(stackSize)
{
	this->job = job;
	this->textedit = textedit;
}

void JobThread::run()
{
	char* buffer = (char*)malloc(sizeof(char)*1024);
	job->describe();
	printf("[TEST] Describe sent\n");
	if (!job->hasAudioComponent())
		this->textedit->setText(this->textedit->text()+"{"+job->getURL()->toString()+"}[!] No audio component\n");
	else
		this->textedit->setText(this->textedit->text()+"{"+job->getURL()->toString()+"}[+] Audio component found\n");
	if (!job->hasVideoComponent())
		this->textedit->setText(this->textedit->text()+"{"+job->getURL()->toString()+"}[!] No video component\n");
	else
		this->textedit->setText(this->textedit->text()+"{"+job->getURL()->toString()+"}[+] Video component found\n");
	printf("\033[31mDESCRIBE\033[0m\n");
	this->textedit->setText(this->textedit->text()+"VideoWidth: "+QString::number(job->getVideoWidth())+"\nVideoHeight: "+QString::number(job->getVideoHeight()));
	//this->textedit->setText(this->textedit->text()+"\ntoString: "+job->toString(buffer));
	this->exit();
}

void usage()
{
	printf("Usage: jobtest <URL>\n\n");
}

int main(int argc,char** argv)
{
	if (argc < 2){
		usage();
		exit(0);
	}
	QApplication app(argc,argv);
	QDialog mainWidget;
	QTextEdit textedit((QWidget*)&mainWidget);
	textedit.resize(900,300);
	app.setMainWidget((QWidget*)&mainWidget);
	Job job1();
	Job job2();
	JobList* list = new JobList();
	if(list->addJob(&job1)){
		printf("\033[33m[TEST] Job1 added to list\033[0m\n");
	}
	else{
		printf("\033[33m[TEST] Job1 _not_ added to list\033[0m\n");
	}
	if(list->addJob(&job2)){
		printf("\033[33m[TEST] Job2 added to list\033[0m\n");
	}
	else{
		printf("\033[33m[TEST] Job2 _not_ added to list\033[0m\n");
	}
	list->fromConfig();
	if(list->removeJob(&job1)){
		printf("\033[33m[TEST] Job1 removed\033[0m\n");
	}
	else{
		printf("\033[33m[TEST] Job1 _not_ removed\033[0m\n");
	}
	JobThread thread1(&job1,&textedit);
	JobThread thread2(&job2,&textedit);
	printf("\033[31mStarting thread 1\033[0m\n");
	thread1.start();
	printf("\033[31mStarting thread 2\033[0m\n");
	thread2.start();
	printf("\033[31mStarting gui\033[0m\n");
	mainWidget.setCaption("JobText output");
	//mainWidget.show();
	//return app.exec();
	return 0;
}
