#include "markglobal.hpp"
#include "vitooki.hpp"
#include "bm.hpp"
#include "SimpleStatistics.hpp"
#ifndef WIN32
#include <netdb.h>
#endif

int GLOBAL_TIME_SCALE=20;

 void usage()
{
	printf("./muvimark [--vids <video.dat>] [--req <requests.dat>] [--up <.up.conf>] [--proxy <proxy> | --merge-logs]\n");
	printf("   --vids       ... Override the default video file video.dat\n");
	printf("   --req        ... Override the default request file requests.dat\n");
	printf("   --ts         ... Override the default timescale of %i\n",GLOBAL_TIME_SCALE);
	printf("   --job        ... Override the default joblist file %s\n",JL_FILE);
	printf("   --up         ... Set the new UserPreferences template. Default is %s\n",USERPREFERENCES_FILE);
	printf("   --proxy      ... Set a proxy server (protocol://hostname:port)\n");
	printf("   --merge-logs ... Merges log files into a global logfile. Ignores other params.\n");
	exit(0);
}

int main (int argc,char** argv)
{
	int i = 0;
	vitooki_init_all();
	char req[MAX_STR_LEN];
	char vid[MAX_STR_LEN];
	char up[MAX_STR_LEN];
	char jobFile[MAX_STR_LEN];
	char* proxy=NULL;
	strcpy(req,"requests.dat");
	strcpy(vid,"video.dat");
	strcpy(up,USERPREFERENCES_FILE);
	strcpy(jobFile,JL_FILE);
	for(i=0 ; i<argc ; i++)
	{
		if( (strcmp(argv[i],"--help") == 0) || (strcmp(argv[i],"-h") == 0) ){
			usage();
		}
		else if( strcmp(argv[i],"--proxy") == 0){
			proxy = argv[i+1];
			i++;
		}
		else if (strcmp(argv[i],"--merge-logs") == 0){
			SimpleStatistics::createGlobalLog("global.log","./","_client.in");
			return 0;
		}
		else if (strcmp(argv[i],"--req") == 0) {
			strcpy(req,argv[i+1]);i++;
		}
		else if (strcmp(argv[i],"--vid") == 0) {
			strcpy(vid,argv[i+1]);i++;
		}
		else if (strcmp(argv[i],"--up") == 0) {
			strcpy(up,argv[i+1]);i++;
		}
		else if (strcmp(argv[i],"--ts") == 0) {
			int tmp=atoi(argv[i+1]);i++;
			if(tmp>0)
				GLOBAL_TIME_SCALE=tmp;
			else
				printf("Ignoring incorrect set param --ts\n");
		}
		else if (strcmp(argv[i],"--job") == 0) {
			strcpy(jobFile,argv[i+1]);i++;
		}
	}


	Benchmark* bm = new Benchmark(req, vid,up,jobFile);
	
	bm->init(proxy);
	
	delete (bm);
	return 1;
}
