 /***********************************************************************
 *                                                                      *
 * MuViNo - ViTooKi Video Annotation Tool                               *
 *                                                                      *
 * title:	MP7ListViewItem.cpp                                     *
 *                                                                      *
 * $Id: MP7ListViewItem.cpp,v 1.4 2004/08/17 13:42:05 mtaschwer Exp $
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

#include "MP7ListViewItem.hpp"

#include <debug.hpp>

MP7ListViewItem::MP7ListViewItem(QListView *parent)
  : QListViewItem(parent)
{
  /* nop */
}

MP7ListViewItem::MP7ListViewItem(MP7ListViewItem *parent, MP7ListViewItem *after)
  : QListViewItem(parent)
{
  if (after)
    moveItem(after);
}

MP7ListViewItem::MP7ListViewItem(MP7ListViewItem *parent, QDomElement elem,
				 MP7ListViewItem *after)
  : QListViewItem(parent)
{
  if (after) {
    moveItem(after);
    if (!elem.isNull())
      parent->domElement().insertAfter(elem, after->domElement());
  } else if (!elem.isNull()) {
    parent->domElement().insertBefore(elem, QDomNode());
  }
  domElem = elem;
}

MP7ListViewItem::MP7ListViewItem(MP7ListViewItem *parent, int pos, QDomElement elem)
  : QListViewItem(parent)
{
  insert(parent, pos, elem);
}

MP7ListViewItem::~MP7ListViewItem()
{
  if (domElem.isNull())
    return;
  domElem.parentNode().removeChild(domElem);
}

void MP7ListViewItem::insert(MP7ListViewItem *parent, int pos, QDomElement &elem)
{
  if (!parent) {
    dprintf_err("MP7ListViewItem::insert() parent == NULL!\n");
    return;
  }
  domElem = elem;
  if (pos == 0) {
    /* insert at the beginning - already done by constructor of super class */
    if (!elem.isNull()) {
      parent->domElement().insertBefore(elem, QDomNode());
    }
  } else {
    int i = 0;
    if (pos < 0 || pos >= parent->childCount())
      pos = parent->childCount() - 1; // last position; <this> is included in childCount()
    for (MP7ListViewItem *child = (MP7ListViewItem *) parent->firstChild();
	 child; child = (MP7ListViewItem *) child->nextSibling()) {
      if (i == pos) {
	if (pos == 0) {
	  /* <this> is the only child */
	  if (!elem.isNull()) {
	    parent->domElement().appendChild(elem);
	  }
	} else {
	  moveItem(child);		// move <this> after <child>
	  if (!elem.isNull()) {
	    parent->domElement().insertAfter(elem, child->domElement());
	  }
	}
	break;
      }
      i++;
    }
  }
}

void MP7ListViewItem::setDomElement(const QDomElement &elem)
{
  domElem = elem;
}

QDomElement MP7ListViewItem::domElement() const
{
  return domElem;
}

MP7ListViewItem *MP7ListViewItem::prevSibling() const
{
  QListViewItem *pItem = parent();
  if (!pItem)
    return NULL;
  QListViewItem *sib = pItem->firstChild();
  if (sib == (QListViewItem *)this)
    return NULL;
  for ( ; sib ; sib = sib->nextSibling()) {
    if (sib->nextSibling() == (QListViewItem *) this)
      break;
  }
  return (MP7ListViewItem *) sib;
}
