 /***********************************************************************
 *                                                                      *
 * MuViNo - ViTooKi Video Annotation Tool                               *
 *                                                                      *
 * title:	MP7ListViewItem.hpp                                     *
 *                                                                      *
 * $Id: MP7ListViewItem.hpp,v 1.4 2004/09/24 13:15:09 mtaschwer Exp $
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

#ifndef _MP7LISTVIEWITEM_HPP_
#define _MP7LISTVIEWITEM_HPP_

/** @file
 * Declaration of the MP7ListViewItem class.
 */

#include <qdom.h>
#include <qlistview.h>

/** Instances of this class represent derived QListViewItem objects
 * representing XML elements within an MPEG-7 document.
 * Each instance of this class is associated with an XML element in a
 * QDom tree.
 * <p>
 * <b>Please note:</b> The QListView object containing instances of this class
 * <em>must not</em> allow sorting of items (see <code>QListView::setSorting()</code>),
 * because this class assumes a 1-1 correspondence between the child node order
 * in the list view and QDom trees.
 * @author Mario Taschwer
 * @version $Id: MP7ListViewItem.hpp,v 1.4 2004/09/24 13:15:09 mtaschwer Exp $
 */
class MP7ListViewItem : public QListViewItem
{
public:
  /** @see QListViewItem::QListViewItem(QListView *) */
  MP7ListViewItem(QListView *parent);

  /** Insert this item after <code>after</code> in <code>parent</code>'s
   * list of children. If <code>after</code> is <code>NULL</code>, this
   * item will be <code>parent</code>'s first child.
   * @see QListViewItem::QListViewItem(QListViewItem *, QListViewItem *after) 
   */
  MP7ListViewItem(MP7ListViewItem *parent, MP7ListViewItem *after = NULL);

  /** Insert this item after <code>after</code> in <code>parent</code>'s
   * list of children and create a corresponding element <code>elem</code>
   * in the QDom tree. 
   * If <code>after</code> is <code>NULL</code>, this
   * item will be <code>parent</code>'s first child.
   */
  MP7ListViewItem(MP7ListViewItem *parent, QDomElement elem,
		  MP7ListViewItem *after = NULL);

  /** Construct this item in position <code>pos</code> in the 
   * <code>parent</code>'s list of children.
   * @param parent    the parent item.
   * @param pos       the position of this item in the parent's list of children.
   *                  If <code>pos</code> is -1, this item will be inserted
   *                  as the last child.
   * @param elem      if <code>!elem.isNull()</code>, <code>elem</code> will
   *                  be inserted into the corresponding QDom tree.
   */
  MP7ListViewItem(MP7ListViewItem *parent, int pos, QDomElement elem);

  /** Destructor. Removes the associated QDomElement, too. */
  virtual ~MP7ListViewItem();

  /** Set the XML element corresponding to this item.
   * The element must already be a node of the XML DOM tree.
   */
  void setDomElement(const QDomElement &elem);

  /** Return the XML element corresponding to this item. */
  QDomElement domElement() const;

  /** Return the previous sibling of this item.
   * @return <code>NULL</code> if there is no such sibling.
   */
  MP7ListViewItem *prevSibling() const;

private:
  /** Insert this item in position <code>pos</code>
   * in the <code>parent</code>'s list of children.
   * This node must have been inserted as the first child of its parent
   * using e.g. the {@link QListViewItem::QListViewItem(QListViewItem *)}
   * constructor prior to calling this function.
   * @param parent     the parent item.
   * @param pos        if -1, this item will be the last child.
   * @param elem      if <code>!elem.isNull()</code>, also <code>elem</code> will
   *                  be inserted into the corresponding QDom tree.
   */
  void insert(MP7ListViewItem *parent, int pos, QDomElement &elem);

  /** The corresponding XML element node. */
  QDomElement domElem;

};

#endif /* _MP7LISTVIEWITEM_HPP_ */
