 /***********************************************************************
 *                                                                      *
 * MuViNo - ViTooKi Video Annotation Tool                               *
 *                                                                      *
 * title:	MP7VideoSegment.cpp                                     *
 *                                                                      *
 * $Id: MP7VideoSegment.cpp,v 1.16 2005/03/01 15:04:42 amueller_itec Exp $
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

#include <qcheckbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qregexp.h>
#include <qspinbox.h>
#include <qtextedit.h>
#include <qdatetimeedit.h>

#include <debug.hpp>

#include "MP7ListViewItem.hpp"
#include "MP7VideoSegment.hpp"

MP7VideoSegment::MP7VideoSegment()
  : MP7View()
{
  /* nop */
}

MP7VideoSegment::MP7VideoSegment(MP7Doc *doc, MP7ListViewItem *root)
  : MP7View()
{
  this->doc = doc;
  this->root = root;
}

MP7ListViewItem *MP7VideoSegment::parent(MP7ListViewItem *root) const
{
  if (!root)
    root = this->root;
  if (!doc || !root || !root->parent())
    return NULL;
  QStringList tags;
  tags.append("VideoSegment");
  tags.append("Video");
  return doc->nearestElement((MP7ListViewItem *) root->parent(), tags, true);
}

MP7ListViewItem *MP7VideoSegment::setRoot(MP7ListViewItem *root, bool searchUp)
{
  if (!doc || !root)
    return NULL;
  QStringList tags;
  tags.append("VideoSegment");
  tags.append("Video");
  MP7ListViewItem *newRoot = doc->nearestElement(root, tags, searchUp);
  if (newRoot)
    this->root = newRoot;
  return newRoot;
}

void MP7VideoSegment::setEmpty()
{
  VideoSegment dlg;
  if (!updateMP7DocFromDlg(dlg)) {
    dprintf_err("MP7VideoSegment::setEmpty() updateMP7DocFromDlg() FAILED!\n");
  }
}

bool MP7VideoSegment::edit()
{
  VideoSegment dlg(NULL, "video segment dialog", true); // modal dialog
  if (!initDlgFromMP7Doc(dlg)) {
    dprintf_err("MP7VideoSegment::edit() initDlgFromMP7Doc() FAILED!\n");
    return false;
  }
  bool rval = dlg.exec();
  if (rval) {
    if (!updateMP7DocFromDlg(dlg)) {
      dprintf_err("MP7VideoSegment::edit() updateMP7DocFromDlg() FAILED!\n");
      return false;
    }
  }
  return rval;
}

bool MP7VideoSegment::editCreation()
{
  
  CreationInformation dlg(NULL, "creation information dialog", true); // modal dialog
  
  if (!initCreationDlgFromMP7Doc(dlg)) {
    dprintf_err("MP7VideoSegment::editCreation() initCreationDlgFromMP7Doc() FAILED!\n");
    return false;
  }
  bool rval = dlg.exec();
  if (rval) {
    if (!updateCreationMP7DocFromDlg(dlg)) {
      dprintf_err("MP7VideoSegment::editCreation() updateCreationMP7DocFromDlg() FAILED!\n");
      return false;
    }
  }
  return rval;
}


bool MP7VideoSegment::editUri()
{
  
  Uri dlg(NULL, "Uri editing dialog", true); // modal dialog
  
  if (!initUriDlgFromMP7Doc(dlg)) {
    dprintf_err("MP7VideoSegment::editUri() initUriDlgFromMP7Doc() FAILED!\n");
    return false;
  }
  bool rval = dlg.exec();
  if (rval) {
    if (!updateUriMP7DocFromDlg(dlg)) {
      dprintf_err("MP7VideoSegment::editUri() updateUriMP7DocFromDlg() FAILED!\n");
      return false;
    }
  }
  return rval;
}

bool MP7VideoSegment::editVariation()
{
  
  Variation dlg(NULL, "Variation editing dialog", true); // modal dialog
  
  if (!initVariationDlgFromMP7Doc(dlg)) {
    dprintf_err("MP7VideoSegment::editVariation() initVariationDlgFromMP7Doc() FAILED!\n");
    return false;
  }
  bool rval = dlg.exec();
  if (rval) {
    if (!updateVariationMP7DocFromDlg(dlg)) {
      dprintf_err("MP7VideoSegment::editVariation() updateVariationMP7DocFromDlg() FAILED!\n");
      return false;
    }
  }
  return rval;
  
}


/*
 * Set the Content Variation Descriptors according to the values chosen from the UI dialog
 */
bool MP7VideoSegment::initVariationDlgFromMP7Doc(Variation &dlg)
{
  if (!doc || !root)
    return false;
  /* media time */
  MP7Time time;
  MP7ListViewItem *item;
  MP7ListViewItem *variationRoot;
  //MP7ListViewItem *tmpItem;
   
  /* get document root */
  
  MP7ListViewItem *documentRoot;
  QStringList tags;
  /* variation root element */
  tags.append("VariationSet");
  if(!(documentRoot = doc->nearestElement(root, tags, true))){
  	return false;
  }
  
  /*
   * Try to get the BitrateReduced Variation
   */ 
  variationRoot = doc->nodeByPath("Variation[@id=BitrateReduced]", documentRoot);
  
  // If one exists, the enable the dialog settings
  if (variationRoot) {
    dlg.bitrateCheckBox->toggle();
  }
  
  // Now search for the corresponding Descriptors and fill in the dialog
  item = doc->nodeByPath("Content/Video/MediaInformation/MediaProfile/MediaInstance/MediaLocator/MediaUri", variationRoot);
  if (item) {
    dlg.bitrateUriCheckBox->toggle();
    dlg.bitrateUriLineEdit->setText(item->text(1));
  }
  
  //Bitrate
  item = doc->nodeByPath("Content/Video/MediaInformation/MediaProfile/MediaFormat/BitRate", variationRoot);
  if (item) {
    dlg.bitrateBitrateSpinBox->setValue(item->text(1).toInt()/1024);
       
  }
  // End of BitrateReduced Variation
  
  /*
   * Try to get the BitrateResolutionReduced Variation
   */ 
  variationRoot = doc->nodeByPath("Variation[@id=BitrateResolutionReduced]", documentRoot);
  
  // If one exists, the enable the dialog settings
  if (variationRoot) {
    dlg.resolutionCheckBox->toggle();
  }
   
  // Now search for the corresponding Descriptors and fill in the dialog
  // URI
  item = doc->nodeByPath("Content/Video/MediaInformation/MediaProfile/MediaInstance/MediaLocator/MediaUri", variationRoot);
  if (item) {
    dlg.resolutionUriCheckBox->toggle();
    dlg.resolutionUriLineEdit->setText(item->text(1));
  }
  
  // Bitrate
  //Bitrate
  item = doc->nodeByPath("Content/Video/MediaInformation/MediaProfile/MediaFormat/BitRate", variationRoot);
  if (item) {
    dlg.resolutionBitrateSpinBox->setValue(item->text(1).toInt()/1024);   
  }
  
  // resolution data
  item = doc->nodeByPath("Content/Video/MediaInformation/MediaProfile/ComponentMediaProfile/MediaFormat/VisualCoding/Frame", variationRoot);
  
  if (item) {
    dlg.resolutionWidthSpinBox->setValue(doc->getAttribute(item, "width").toInt());
    dlg.resolutionHeightSpinBox->setValue(doc->getAttribute(item, "height").toInt());  
  }
   // End of BitrateResolutionReduced Variation
   
   
   /*
   * Try to get the ResolutionColorspaceSoundReduced Variation
   */ 
  variationRoot = doc->nodeByPath("Variation[@id=ResolutionColorspaceSoundReduced]", documentRoot);
  
  // If one exists, the enable the dialog settings
  if (variationRoot) {
    dlg.colorCheckBox->toggle();
  }
   
  // Now search for the corresponding Descriptors and fill in the dialog
  item = doc->nodeByPath("Content/Video/MediaInformation/MediaProfile/MediaInstance/MediaLocator/MediaUri", variationRoot);
  if (item) {
    dlg.colorUriCheckBox->toggle();
    dlg.colorUriLineEdit->setText(item->text(1));
  }
  
  // resolution data
  item = doc->nodeByPath("Content/Video/MediaInformation/MediaProfile/ComponentMediaProfile/MediaFormat/VisualCoding/Frame", variationRoot);
  
  if (item) {
    dlg.colorWidthSpinBox->setValue(doc->getAttribute(item, "width").toInt());
    dlg.colorHeightSpinBox->setValue(doc->getAttribute(item, "height").toInt());  
  }
   // End of ResolutionColorspaceSoundReduced Variation
    
  return true;
}


bool MP7VideoSegment::initUriDlgFromMP7Doc(Uri &dlg)
{
  if (!doc || !root)
    return false;
  /* media time */
  MP7Time time;
  MP7ListViewItem *item;
   
  /* get document root */
  
  MP7ListViewItem *documentRoot;
  QStringList tags;
  tags.append("VariationSet");
  if(!(documentRoot = doc->nearestElement(root, tags, true))){
  	return false;
  }
   
  /* video uri */
  item = doc->nodeByPath("Source/Video/MediaInformation/MediaProfile/MediaInstance/MediaLocator/MediaUri", documentRoot);
  //item = doc->nodeByPath("MediaLocator/MediaUri", root);
  if (item) {
    dlg.videoUriEdit->setText(item->text(1));
  }
  
  /* audio uri */
  
  
  item = doc->nodeByPath("Variation/Content/Audio/MediaLocator/MediaUri", documentRoot);
  if (item) {
    dlg.audioUriEdit->setText(item->text(1));
  }
    
  return true;
}



bool MP7VideoSegment::initCreationDlgFromMP7Doc(CreationInformation &dlg)
{
  if (!doc || !root)
    return false;
  /* media time */
  MP7Time time;
  MP7ListViewItem *item;
    
  /* creator */
  
  item = doc->nodeByPath("CreationInformation/Creation/Creator/Agent[@xsi:type=PersonType]/Name/GivenName", root);
  if (item) {
    dlg.givenNameEdit->setText(item->text(1));
  }
  
  item = doc->nodeByPath("CreationInformation/Creation/Creator/Agent[@xsi:type=PersonType]/Name/FamilyName", root);
  if (item) {
    dlg.familyNameEdit->setText(item->text(1));
  }
  
  /* creation coordinates */
  /* place */
  item = doc->nodeByPath("CreationInformation/Creation/CreationCoordinates/Location", root);
  if (item) {
    MP7ListViewItem *subitem = doc->nodeByPath("Name", item);
    if (subitem) {
      dlg.creationPlaceEdit->setText(subitem->text(1));
    }
    subitem = doc->nodeByPath("Region", item);
    if (subitem) {
      if (!dlg.setCreationRegion(subitem->text(1)))
	return false;
    }
  }
   
  /* time */
  item = doc->nodeByPath("CreationInformation/Creation/CreationCoordinates/Date", root);
  if (item) {
    MP7ListViewItem *subitem = doc->nodeByPath("TimePoint", item);
    if (subitem) {
      QString mp7Date = subitem->text(1);
      QRegExp rx("T(\\d{2}(:\\d{2}(:\\d{2})?)?)");
      if (rx.search(mp7Date) >= 0) {
	QTime time = QTime::fromString(rx.cap(1));
	if (!time.isValid())
	  return false;
	dlg.creationTimeCheckBox->setChecked(true);
	dlg.creationTimeEdit->setTime(time);
	mp7Date.remove(rx);
      }
      dlg.creationDateEdit->setText(mp7Date);
    }
    
  } 
  
  /* copyrightstring */
  item = doc->nodeByPath("CreationInformation/Creation/CopyrightString", root);
  if (item) {
    dlg.creationCopyRightEdit->setText(item->text(1));
  }
  
  return true;
}


bool MP7VideoSegment::initDlgFromMP7Doc(VideoSegment &dlg)
{
  if (!doc || !root)
    return false;
  /* media time */
  MP7Time time;
  MP7ListViewItem *item = doc->nodeByPath("MediaTime/MediaTimePoint", root);
  if (item) {
    time.setTimePoint(item->text(1));
    dlg.mediaTimeStartValue->setText(time.timePointPretty());
  }
  item = doc->nodeByPath("MediaTime/MediaDuration", root);
  if (item) {
    time.setDuration(item->text(1));
    dlg.mediaTimeDurValue->setText(time.durationPretty());
    dlg.mediaTimeEndValue->setText(time.endTimePointPretty());
  }
  /* title */
  item = doc->nodeByPath("CreationInformation/Creation/Title", root);
  if (item) {
    dlg.titleEdit->setText(item->text(1));
  }
  /* determine base path for text annotation, set abstractCheckBox */
  QString path("CreationInformation/Creation/Abstract");
  item = doc->nodeByPath(path, root);
  dlg.abstractCheckBox->setChecked(item != NULL);
  if (!item)
    path = "TextAnnotation";
  /* free text annotation */
  item = doc->nodeByPath(path + "/FreeTextAnnotation", root);
  if (item) {
    dlg.textAnnotationEdit->setText(item->text(1));
  }
  /* keywords annotation */
  item = doc->nodeByPath(path + "/KeywordAnnotation", root);
  if (item) {
    QPtrList<MP7ListViewItem> list = doc->children(item, "Keyword");
    for (MP7ListViewItem *keyword = list.first(); keyword; keyword = list.next()) {
      dlg.keywordsEdit->append(keyword->text(1));
    }
  }
  /* semantic place/region */
  item = doc->nodeByPath("Semantic/SemanticBase[@xsi:type=SemanticPlaceType]/Place", root);
  if (item) {
    MP7ListViewItem *subitem = doc->nodeByPath("Name", item);
    if (subitem) {
      dlg.semanticPlaceEdit->setText(subitem->text(1));
    }
    subitem = doc->nodeByPath("Region", item);
    if (subitem) {
      if (!dlg.setSemanticRegion(subitem->text(1)))
	return false;
    }
  }
  /* semantic time */
  item = doc->nodeByPath("Semantic/SemanticBase[@xsi:type=SemanticTimeType]/Time", root);
  if (item) {
    MP7ListViewItem *subitem = doc->nodeByPath("TimePoint", item);
    if (subitem) {
      QString mp7Date = subitem->text(1);
      QRegExp rx("T(\\d{2}(:\\d{2}(:\\d{2})?)?)");
      if (rx.search(mp7Date) >= 0) {
	QTime time = QTime::fromString(rx.cap(1));
	if (!time.isValid())
	  return false;
	dlg.semanticTimeCheckBox->setChecked(true);
	dlg.semanticTimeEdit->setTime(time);
	mp7Date.remove(rx);
      }
      dlg.semanticDateEdit->setText(mp7Date);
    }
    subitem = doc->nodeByPath("IncrDuration", item);
    if (subitem) {
      dlg.semanticDurationSpinBox->setValue(subitem->text(1).toInt());
      QString timeUnit = subitem->domElement().attribute("timeUnit");
      if (timeUnit != QString::null) {
	dlg.setSemanticDurationUnit(timeUnit); // ignore error
      }
    }
  }
  return true;
}


bool MP7VideoSegment::updateVariationMP7DocFromDlg(const Variation &dlg)
{
  if (!doc || !root)
    return false;
    
  //bool hasVideoUri = !dlg.videoUriEdit->text().stripWhiteSpace().isEmpty();
  //bool hasAudioUri = !dlg.audioUriEdit->text().stripWhiteSpace().isEmpty();
  /* get document root */
  MP7ListViewItem *documentRoot;
  MP7ListViewItem *variationRoot;
  MP7ListViewItem *tmpRoot;
  char string[1024];
  QStringList tags;
  tags.append("VariationSet");
  if(!(documentRoot = doc->nearestElement(root, tags, true))){
  	return false;
  }
  
  // Variation 1: Bitrate Reduced Version
  bool hasVideoUri = !dlg.bitrateUriLineEdit->text().stripWhiteSpace().isEmpty();
  bool isChecked = dlg.bitrateCheckBox->isChecked();
  
  
  MP7ListViewItem *item;
  item = doc->nodeByPath("Variation[@id=BitrateReduced]", documentRoot, false);
  //item = doc->nodeByPath("MediaLocator/MediaUri", root, false);
  if (item && !isChecked) {
    doc->removeElementDeep(item);
    item = NULL;
  } else if (isChecked) {
    variationRoot = doc->nodeByPath("Variation[@id=BitrateReduced]", documentRoot, true);
    item = doc->nodeByPath("Content", variationRoot, true);
    doc->setAttribute(item, "xsi:type", "VideoType");
    
    item = doc->nodeByPath("Video", item, true);
    doc->setAttribute(item, "id", "Variation1_BitrateReduced");
    
    item = doc->nodeByPath("MediaInformation", item, true);
    
    item = doc->nodeByPath("MediaProfile", item, true);
    
    // Video ES
    item = doc->nodeByPath("ComponentMediaProfile", item, true);
    doc->setAttribute(item, "id", "Var1_ES1");
    
    tmpRoot = doc->nodeByPath("MediaFormat", item, true);
    
    item = doc->nodeByPath("Content", tmpRoot, true);
    doc->setAttribute(item, "href", "MPEG7ContentCS");
    
    item = doc->nodeByPath("Name", item, true);
    doc->setElementValue(item, "visual");
    
    item = doc->nodeByPath("VisualCoding", tmpRoot, true);
    item = doc->nodeByPath("Format", item, true);
    doc->setAttribute(item, "href", "urn:mpeg:mpeg7:cs:VisualCodingFormatCS:2001:3");
    
    item = doc->nodeByPath("Name", item, true);
    doc->setAttribute(item, "xml:lang", "en");
    doc->setElementValue(item, "MPEG-4 Visual");
    
    item = doc->nodeByPath("VisualCoding/Frame", tmpRoot, true);
    
    tmpRoot = doc->nodeByPath("Source/Video/MediaInformation/MediaProfile/ComponentMediaProfile/MediaFormat/VisualCoding/Frame", documentRoot);
    doc->setAttribute(item, "height", doc->getAttribute(tmpRoot, "height"));
    doc->setAttribute(item, "width", doc->getAttribute(tmpRoot, "width"));
    doc->setAttribute(item, "rate", doc->getAttribute(tmpRoot, "rate"));
    
    // Audio ES
    item = doc->nodeByPath("Content/Video/MediaInformation/MediaProfile/ComponentMediaProfile", variationRoot, true);
    doc->setAttribute(item, "id", "Var1_ES2");
    
    tmpRoot = doc->nodeByPath("MediaFormat", item, true);
    
    item = doc->nodeByPath("Content", tmpRoot, true);
    doc->setAttribute(item, "href", "MPEG7ContentCS");
    
    item = doc->nodeByPath("Name", item, true);
    doc->setElementValue(item, "audio");
    
    item = doc->nodeByPath("AudioCoding", tmpRoot, true);
    item = doc->nodeByPath("Format", item, true);
    doc->setAttribute(item, "href", "urn:mpeg:mpeg7:cs:VisualCodingFormatCS:2001:3");
    
    item = doc->nodeByPath("Name", item, true);
    doc->setAttribute(item, "xml:lang", "en");
    doc->setElementValue(item, "MPEG-1 Audio Layer III");
    
    // Format
    tmpRoot = doc->nodeByPath("Content/Video/MediaInformation/MediaProfile/MediaFormat", variationRoot, true);
    item = doc->nodeByPath("Content", tmpRoot, true);
    doc->setAttribute(item, "href", "MPEG7ContentCS");
    item = doc->nodeByPath("Name", item, true);
    doc->setElementValue(item, "audiovisual");
    
    item = doc->nodeByPath("FileFormat", tmpRoot, true);
    doc->setAttribute(item, "href", "urn:mpeg:mpeg7:cs:FileFormatCS:2001:5");
    item = doc->nodeByPath("Name", item, true);
    doc->setElementValue(item, "mp4");
        
    
    
    item = doc->nodeByPath("BitRate", tmpRoot, true);
    sprintf(string,"%i",dlg.bitrateBitrateSpinBox->value());
    doc->setElementValue(item, string);
    
    // Instance
    tmpRoot = doc->nodeByPath("Content/Video/MediaInformation/MediaProfile/MediaInstance", variationRoot, true);
    item = doc->nodeByPath("InstanceIdentifier", tmpRoot, true);
    doc->setAttribute(item, "organization", "MPEG");
    doc->setAttribute(item, "type", "MPEG7ContentSetOnLineId");
    doc->setElementValue(item, "mpeg7");
    
    item = doc->nodeByPath("MediaLocator", tmpRoot, true);
    item = doc->nodeByPath("MediaUri", item, true);
    doc->setElementValue(item, dlg.bitrateUriLineEdit->text());
    
    // VariationRelationship
    item = doc->nodeByPath("VariationRelationship", variationRoot, true);
    doc->setElementValue(item, "QualityReductionAdaptor");
    
    //doc->setElementValue(item, "mpeg7");
    //doc->setAttribute(item, "organization", "MPEG");
    //doc->setAttribute(item, "type", "MPEG7ContentSetOnLineId");
   
    //item = doc->nodeByPath("Source/Video/MediaInformation/MediaProfile/MediaInstance/MediaLocator/MediaUri", documentRoot, true);
   }
  
  // Variation 2: Bitrate and Resolution Reduced Version
  hasVideoUri = !dlg.resolutionUriLineEdit->text().stripWhiteSpace().isEmpty();
  isChecked = dlg.resolutionCheckBox->isChecked();
  
  item = doc->nodeByPath("Variation[@id=BitrateResolutionReduced]", documentRoot, false);
  //item = doc->nodeByPath("MediaLocator/MediaUri", root, false);
  if (item && !isChecked) {
    doc->removeElementDeep(item);
    item = NULL;
  } else if (isChecked) {
    variationRoot = doc->nodeByPath("Variation[@id=BitrateResolutionReduced]", documentRoot, true);
    item = doc->nodeByPath("Content", variationRoot, true);
    doc->setAttribute(item, "xsi:type", "VideoType");
    
    item = doc->nodeByPath("Video", item, true);
    doc->setAttribute(item, "id", "Variation2_BitrateAndResolutionReduced");
    
    item = doc->nodeByPath("MediaInformation", item, true);
    
    item = doc->nodeByPath("MediaProfile", item, true);
    
    // Video ES
    item = doc->nodeByPath("ComponentMediaProfile", item, true);
    doc->setAttribute(item, "id", "Var2_ES1");
    
    tmpRoot = doc->nodeByPath("MediaFormat", item, true);
    
    item = doc->nodeByPath("Content", tmpRoot, true);
    doc->setAttribute(item, "href", "MPEG7ContentCS");
    
    item = doc->nodeByPath("Name", item, true);
    doc->setElementValue(item, "visual");
    
    item = doc->nodeByPath("VisualCoding", tmpRoot, true);
    item = doc->nodeByPath("Format", item, true);
    doc->setAttribute(item, "href", "urn:mpeg:mpeg7:cs:VisualCodingFormatCS:2001:3");
    
    item = doc->nodeByPath("Name", item, true);
    doc->setAttribute(item, "xml:lang", "en");
    doc->setElementValue(item, "MPEG-4 Visual");
    
    item = doc->nodeByPath("VisualCoding/Frame", tmpRoot, true);
    
    tmpRoot = doc->nodeByPath("Source/Video/MediaInformation/MediaProfile/ComponentMediaProfile/MediaFormat/VisualCoding/Frame", documentRoot);
    sprintf(string,"%d",dlg.resolutionHeightSpinBox->value());
    doc->setAttribute(item, "height", string);
    sprintf(string,"%d",dlg.resolutionWidthSpinBox->value());
    doc->setAttribute(item, "width", string);
    doc->setAttribute(item, "rate", doc->getAttribute(tmpRoot, "rate"));
    
    // Audio ES
    item = doc->nodeByPath("Content/Video/MediaInformation/MediaProfile/ComponentMediaProfile", variationRoot, true);
    doc->setAttribute(item, "id", "Var2_ES2");
    
    tmpRoot = doc->nodeByPath("MediaFormat", item, true);
    
    item = doc->nodeByPath("Content", tmpRoot, true);
    doc->setAttribute(item, "href", "MPEG7ContentCS");
    
    item = doc->nodeByPath("Name", item, true);
    doc->setElementValue(item, "audio");
    
    item = doc->nodeByPath("AudioCoding", tmpRoot, true);
    item = doc->nodeByPath("Format", item, true);
    doc->setAttribute(item, "href", "urn:mpeg:mpeg7:cs:VisualCodingFormatCS:2001:3");
    
    item = doc->nodeByPath("Name", item, true);
    doc->setAttribute(item, "xml:lang", "en");
    doc->setElementValue(item, "MPEG-1 Audio Layer III");
    
    // Format
    tmpRoot = doc->nodeByPath("Content/Video/MediaInformation/MediaProfile/MediaFormat", variationRoot, true);
    item = doc->nodeByPath("Content", tmpRoot, true);
    doc->setAttribute(item, "href", "MPEG7ContentCS");
    item = doc->nodeByPath("Name", item, true);
    doc->setElementValue(item, "audiovisual");
    
    item = doc->nodeByPath("FileFormat", tmpRoot, true);
    doc->setAttribute(item, "href", "urn:mpeg:mpeg7:cs:FileFormatCS:2001:5");
    item = doc->nodeByPath("Name", item, true);
    doc->setElementValue(item, "mp4");
        
        
    item = doc->nodeByPath("BitRate", tmpRoot, true);
    sprintf(string,"%i",dlg.resolutionBitrateSpinBox->value());
    doc->setElementValue(item, string);
    
    // Instance
    tmpRoot = doc->nodeByPath("Content/Video/MediaInformation/MediaProfile/MediaInstance", variationRoot, true);
    item = doc->nodeByPath("InstanceIdentifier", tmpRoot, true);
    doc->setAttribute(item, "organization", "MPEG");
    doc->setAttribute(item, "type", "MPEG7ContentSetOnLineId");
    doc->setElementValue(item, "mpeg7");
    
    item = doc->nodeByPath("MediaLocator", tmpRoot, true);
    item = doc->nodeByPath("MediaUri", item, true);
    doc->setElementValue(item, dlg.resolutionUriLineEdit->text());
    
    // VariationRelationship
    item = doc->addElement(variationRoot, "VariationRelationship");
    doc->setElementValue(item, "QualityReductionAdaptor");
    
    item = doc->addElement(variationRoot, "VariationRelationship");
    doc->setElementValue(item, "SpatialReductionAdaptor");
      
   }
  // Variation 3: Resolution and Colorspace Reduced Version
  hasVideoUri = !dlg.colorUriLineEdit->text().stripWhiteSpace().isEmpty();
  isChecked = dlg.colorCheckBox->isChecked();
  
  item = doc->nodeByPath("Variation[@id=ResolutionColorspaceSoundReduced]", documentRoot, false);
  //item = doc->nodeByPath("MediaLocator/MediaUri", root, false);
  if (item && !isChecked) {
    doc->removeElementDeep(item);
    item = NULL;
  } else if (isChecked) {
    variationRoot = doc->nodeByPath("Variation[@id=ResolutionColorspaceSoundReduced]", documentRoot, true);
    item = doc->nodeByPath("Content", variationRoot, true);
    doc->setAttribute(item, "xsi:type", "VideoType");
    
    item = doc->nodeByPath("Video", item, true);
    doc->setAttribute(item, "id", "Variation3_ColorAndResolutionReduced");
    
    item = doc->nodeByPath("MediaInformation", item, true);
    
    item = doc->nodeByPath("MediaProfile", item, true);
    
    // Video ES
    item = doc->nodeByPath("ComponentMediaProfile", item, true);
    doc->setAttribute(item, "id", "Var3_ES1");
    
    tmpRoot = doc->nodeByPath("MediaFormat", item, true);
    
    item = doc->nodeByPath("Content", tmpRoot, true);
    doc->setAttribute(item, "href", "MPEG7ContentCS");
    
    item = doc->nodeByPath("Name", item, true);
    doc->setElementValue(item, "visual");
    
    item = doc->nodeByPath("VisualCoding", tmpRoot, true);
    item = doc->nodeByPath("Format", item, true);
    doc->setAttribute(item, "href", "urn:mpeg:mpeg7:cs:VisualCodingFormatCS:2001:3");
    
    item = doc->nodeByPath("Name", item, true);
    doc->setAttribute(item, "xml:lang", "en");
    doc->setElementValue(item, "MPEG-4 Visual");
    
    item = doc->nodeByPath("VisualCoding/Frame", tmpRoot, true);
    
    tmpRoot = doc->nodeByPath("Source/Video/MediaInformation/MediaProfile/ComponentMediaProfile/MediaFormat/VisualCoding/Frame", documentRoot);
    sprintf(string,"%d",dlg.colorHeightSpinBox->value());
    doc->setAttribute(item, "height", string);
    sprintf(string,"%d",dlg.colorWidthSpinBox->value());
    doc->setAttribute(item, "width", string);
    doc->setAttribute(item, "rate", doc->getAttribute(tmpRoot, "rate"));
    
    // Audio ES
    item = doc->nodeByPath("Content/Video/MediaInformation/MediaProfile/ComponentMediaProfile", variationRoot, true);
    doc->setAttribute(item, "id", "Var3_ES2");
    
    tmpRoot = doc->nodeByPath("MediaFormat", item, true);
    
    item = doc->nodeByPath("Content", tmpRoot, true);
    doc->setAttribute(item, "href", "MPEG7ContentCS");
    
    item = doc->nodeByPath("Name", item, true);
    doc->setElementValue(item, "audio");
    
    item = doc->nodeByPath("AudioCoding", tmpRoot, true);
    item = doc->nodeByPath("Format", item, true);
    doc->setAttribute(item, "href", "urn:mpeg:mpeg7:cs:VisualCodingFormatCS:2001:3");
    
    item = doc->nodeByPath("Name", item, true);
    doc->setAttribute(item, "xml:lang", "en");
    doc->setElementValue(item, "MPEG-1 Audio Layer III");
    
    // Format
    tmpRoot = doc->nodeByPath("Content/Video/MediaInformation/MediaProfile/MediaFormat", variationRoot, true);
    item = doc->nodeByPath("Content", tmpRoot, true);
    doc->setAttribute(item, "href", "MPEG7ContentCS");
    item = doc->nodeByPath("Name", item, true);
    doc->setElementValue(item, "audiovisual");
    
    item = doc->nodeByPath("FileFormat", tmpRoot, true);
    doc->setAttribute(item, "href", "urn:mpeg:mpeg7:cs:FileFormatCS:2001:5");
    item = doc->nodeByPath("Name", item, true);
    doc->setElementValue(item, "mp4");
        
       
    // Instance
    tmpRoot = doc->nodeByPath("Content/Video/MediaInformation/MediaProfile/MediaInstance", variationRoot, true);
    item = doc->nodeByPath("InstanceIdentifier", tmpRoot, true);
    doc->setAttribute(item, "organization", "MPEG");
    doc->setAttribute(item, "type", "MPEG7ContentSetOnLineId");
    doc->setElementValue(item, "mpeg7");
    
    item = doc->nodeByPath("MediaLocator", tmpRoot, true);
    item = doc->nodeByPath("MediaUri", item, true);
    doc->setElementValue(item, dlg.colorUriLineEdit->text());
    
    // VariationRelationship
    item = doc->addElement(variationRoot, "VariationRelationship");
    doc->setElementValue(item, "ColorReductionAdaptor");
    
    item = doc->addElement(variationRoot, "VariationRelationship");
    doc->setElementValue(item, "SpatialReductionAdaptor");
    
   }
    
  return true;
}


bool MP7VideoSegment::updateUriMP7DocFromDlg(const Uri &dlg)
{
  if (!doc || !root)
    return false;
    
  bool hasVideoUri = !dlg.videoUriEdit->text().stripWhiteSpace().isEmpty();
  bool hasAudioUri = !dlg.audioUriEdit->text().stripWhiteSpace().isEmpty();
  
  /* get document root */
  MP7ListViewItem *documentRoot;
  QStringList tags;
  tags.append("VariationSet");
  if(!(documentRoot = doc->nearestElement(root, tags, true))){
  	return false;
  }
  
  MP7ListViewItem *item;
  item = doc->nodeByPath("Source/Video/MediaInformation/MediaProfile/MediaInstance/MediaLocator/MediaUri", documentRoot, false);
  //item = doc->nodeByPath("MediaLocator/MediaUri", root, false);
  if (item && !hasVideoUri) {
    doc->removeElementDeep(item);
    item = NULL;
  } else if (hasVideoUri) {
    item = doc->nodeByPath("Source/Video/MediaInformation/MediaProfile/MediaInstance/InstanceIdentifier", documentRoot, true);
    doc->setElementValue(item, "mpeg7");
    doc->setAttribute(item, "organization", "MPEG");
    doc->setAttribute(item, "type", "MPEG7ContentSetOnLineId");
   
    item = doc->nodeByPath("Source/Video/MediaInformation/MediaProfile/MediaInstance/MediaLocator/MediaUri", documentRoot, true);
    
    //item = doc->nodeByPath("MediaLocator/MediaUri", root, true);
    if (!item)
      return false;
    if (doc->setElementValue(item, dlg.videoUriEdit->text()) < 0)
      return false;
  }
  
      
  item = doc->nodeByPath("Variation/Content/Audio/MediaLocator/MediaUri", documentRoot, false);
  if (item && !hasAudioUri) {
    doc->removeElementDeep(item);
    item = NULL;
  } else if (hasAudioUri) {
    item = doc->nodeByPath("Variation/Content/Audio/MediaLocator/MediaUri", documentRoot, true);
    if (!item)
      return false;
    if (doc->setElementValue(item, dlg.audioUriEdit->text()) < 0)
      return false;
  }
  
  
  return true;
}



bool MP7VideoSegment::updateCreationMP7DocFromDlg(const CreationInformation &dlg)
{
  if (!doc || !root)
    return false;
  bool hasGivenName = !dlg.givenNameEdit->text().stripWhiteSpace().isEmpty();
  bool hasFamilyName = !dlg.familyNameEdit->text().stripWhiteSpace().isEmpty();
  //bool hasPlace = !dlg.creationPlaceEdit->text().stripWhiteSpace().isEmpty();
  //bool hasCopyRightString = !dlg.creationCopyRightEdit->text().stripWhiteSpace().isEmpty();
  MP7ListViewItem *item;
  /* mandatory role*/
  item = doc->nodeByPath("CreationInformation/Creation/Creator/Role", root, false);
  if (item && !hasGivenName) {
    doc->removeElementDeep(item);
    item = NULL;
  } else if (hasGivenName) {
    item = doc->nodeByPath("CreationInformation/Creation/Creator/Role", root, true);
    if (!item)
      return false;
    doc->setAttribute(item,"href","urn:mpeg:mpeg7:cs:RoleCS:2001:AUTHOR");
    
    
  }
  /* given name */
  
  item = doc->nodeByPath("CreationInformation/Creation/Creator/Agent[@xsi:type=PersonType]/Name/GivenName", root, false);
  if (item && !hasGivenName) {
    doc->removeElementDeep(item);
    item = NULL;
  } else if (hasGivenName) {
    item = doc->nodeByPath("CreationInformation/Creation/Creator/Agent[@xsi:type=PersonType]/Name/GivenName", root, true);
    if (!item)
      return false;
    if (doc->setElementValue(item, dlg.givenNameEdit->text()) < 0)
      return false;
  }
  
  /* family name */
  
  item = doc->nodeByPath("CreationInformation/Creation/Creator/Agent[@xsi:type=PersonType]/Name/FamilyName", root, false);
  if (item && !hasFamilyName) {
    doc->removeElementDeep(item);
    item = NULL;
  } else if (hasGivenName) {
    item = doc->nodeByPath("CreationInformation/Creation/Creator/Agent[@xsi:type=PersonType]/Name/FamilyName", root, true);
    if (!item)
      return false;
    if (doc->setElementValue(item, dlg.familyNameEdit->text()) < 0)
      return false;
  }
  
  /* creation coordinates */
  bool hasCreationPlaceName = !dlg.creationPlaceEdit->text().stripWhiteSpace().isEmpty();
  bool hasCreationRegion = dlg.creationRegionIsValid();
  bool hasCreationPlace = hasCreationPlaceName || hasCreationRegion;
  bool hasCreationDate = !dlg.creationDateEdit->text().isEmpty();
  bool hasCreationTime = dlg.creationTimeCheckBox->isChecked();
  bool hasCreationTimeDesc = hasCreationDate || hasCreationTime;
  bool hasCreation = hasCreationPlace || hasCreationTimeDesc;
  
  if (item && !hasCreation) {
    doc->removeElementDeep(item);
    item = NULL;
  } else if (hasCreation) {
    
    /* creation place/region */
    item = doc->nodeByPath("CreationInformation/Creation/CreationCoordinates", root,
			   hasCreationPlace);
    if (item && !hasCreationPlace) {
      doc->removeElementDeep(item);
      item = NULL;
    } else if (hasCreationPlace) {
      if (!item)
	return false;
      MP7ListViewItem *subitem;
      subitem = doc->nodeByPath("Location/Name", item, hasCreationPlaceName);
      if (subitem && !hasCreationPlaceName) {
	doc->removeElementDeep(subitem);
	subitem = NULL;
      } else if (hasCreationPlaceName) {
	if (!subitem)
	  return false;
	doc->setElementValue(subitem, dlg.creationPlaceEdit->text().stripWhiteSpace());
      }
      subitem = doc->nodeByPath("Location/Region", item, hasCreationRegion);
      if (subitem && !hasCreationRegion) {
	doc->removeElementDeep(subitem);
	subitem = NULL;
      } else if (hasCreationRegion) {
	if (!subitem)
	  return false;
	doc->setElementValue(subitem, dlg.creationRegion());
      }
    }
    
    
    /* creation time */
    item = doc->nodeByPath("CreationInformation/Creation/CreationCoordinates", root,
			   hasCreationTimeDesc);
    if (item && !hasCreationTimeDesc) {
      doc->removeElementDeep(item);
      item = NULL;
    } else if (hasCreationTimeDesc) {
      if (!item)
	return false;
      MP7ListViewItem *subitem;
      QString mp7Time;
      if (hasCreationDate)
	mp7Time += dlg.creationDateEdit->text();
      if (hasCreationTime)
	mp7Time += "T" + dlg.creationTimeEdit->time().toString("hh:mm:ss");
      subitem = doc->nodeByPath("Date/TimePoint", item, true);
      if (!subitem)
	return false;
      doc->setElementValue(subitem, mp7Time);
      
      }
       
    
   }/*hasCreation*/
  
   
   
   /* copyrightstring */
  bool hasCopyRightString = !dlg.creationCopyRightEdit->text().stripWhiteSpace().isEmpty();
   
  item = doc->nodeByPath("CreationInformation/Creation/CopyrightString", root, false);
  if (item && !hasCopyRightString) {
    doc->removeElementDeep(item);
    item = NULL;
  } else if (hasCopyRightString) {
    item = doc->nodeByPath("CreationInformation/Creation/CopyrightString", root, true);
    if (!item)
      return false;
    if (doc->setElementValue(item, dlg.creationCopyRightEdit->text()) < 0)
      return false;
  }
  
  
  return true;
}



bool MP7VideoSegment::updateMP7DocFromDlg(const VideoSegment &dlg)
{
  if (!doc || !root)
    return false;
  bool hasTitle = !dlg.titleEdit->text().stripWhiteSpace().isEmpty();
  bool hasTextAnnotation = !dlg.textAnnotationEdit->text().stripWhiteSpace().isEmpty();
  bool hasKeywords = !dlg.keywordsEdit->text().stripWhiteSpace().isEmpty();
  /* title */
  MP7ListViewItem *item = doc->nodeByPath("CreationInformation/Creation/Title",
					  root, false);
  if (item && !hasTitle) {
    doc->removeElementDeep(item);
    item = NULL;
  } else if (hasTitle) {
    item = doc->nodeByPath("CreationInformation/Creation/Title", root, true);
    if (!item)
      return false;
    if (doc->setElementValue(item, dlg.titleEdit->text()) < 0)
      return false;
  }
  /* set base path for text annotation */
  QString path, removepath;
  if (dlg.abstractCheckBox->isChecked()) {
    path = "CreationInformation/Creation/Abstract";
    removepath = "TextAnnotation";
  } else {
    path = "TextAnnotation";
    removepath = "CreationInformation/Creation/Abstract";
  }
  /* remove non-selected text annotation */
  item = doc->nodeByPath(removepath, root);
  if (item)
    doc->removeElementDeep(item);
  /* free text annotation */
  item = doc->nodeByPath(path + "/FreeTextAnnotation", root, false);
  if (item && !hasTextAnnotation) {
    doc->removeElementDeep(item);
    item = NULL;
  } else if (hasTextAnnotation) {
    item = doc->nodeByPath(path + "/FreeTextAnnotation", root, true);
    if (!item)
      return false;
    if (doc->setElementValue(item, dlg.textAnnotationEdit->text()) < 0)
      return false;
  }
  /* keywords annotation */
  item = doc->nodeByPath(path + "/KeywordAnnotation", root, false);
  if (item && !hasKeywords) {
    doc->removeElementDeep(item);
    item = NULL;
  } else if (hasKeywords) {
    item = doc->nodeByPath(path + "/KeywordAnnotation", root, true);
    if (!item)
      return false;
    doc->removeChildrenDeep(item, "Keyword");
    QStringList strlist = QStringList::split("\n", dlg.keywordsEdit->text());
    for (QStringList::Iterator it = strlist.begin(); it != strlist.end(); it++) {
      MP7ListViewItem *child = doc->addElement(item, "Keyword");
      if (!child)
	return false;
      if (doc->setElementValue(child, *it) < 0)
	return false;
    }
  }
  /* semantic descriptors */
  bool hasSemanticPlaceName = !dlg.semanticPlaceEdit->text().stripWhiteSpace().isEmpty();
  bool hasSemanticRegion = dlg.semanticRegionIsValid();
  bool hasSemanticPlace = hasSemanticPlaceName || hasSemanticRegion;
  bool hasSemanticDate = !dlg.semanticDateEdit->text().isEmpty();
  bool hasSemanticTime = dlg.semanticTimeCheckBox->isChecked();
  bool hasSemanticDuration = dlg.semanticDurationSpinBox->value() != 0;
  bool hasSemanticTimeDesc = hasSemanticDate || hasSemanticTime;
  bool hasSemantic = hasSemanticPlace || hasSemanticTimeDesc;
  item = doc->nodeByPath("Semantic", root);
  if (item && !hasSemantic) {
    doc->removeElementDeep(item);
    item = NULL;
  } else if (hasSemantic) {
    /* set label */
    item = doc->nodeByPath("Semantic/Label/Name", root, true);
    if (!item)
      return false;
    doc->setElementValue(item, "Semantic Place and Time");
    /* semantic place/region */
    item = doc->nodeByPath("Semantic/SemanticBase[@xsi:type=SemanticPlaceType]", root,
			   hasSemanticPlace);
    if (item && !hasSemanticPlace) {
      doc->removeElementDeep(item);
      item = NULL;
    } else if (hasSemanticPlace) {
      if (!item)
	return false;
      MP7ListViewItem *subitem = doc->nodeByPath("Label/Name", item, true);
      if (!subitem)
	return false;
      doc->setElementValue(subitem, "Semantic Place");
      subitem = doc->nodeByPath("Place/Name", item, hasSemanticPlaceName);
      if (subitem && !hasSemanticPlaceName) {
	doc->removeElementDeep(subitem);
	subitem = NULL;
      } else if (hasSemanticPlaceName) {
	if (!subitem)
	  return false;
	doc->setElementValue(subitem, dlg.semanticPlaceEdit->text().stripWhiteSpace());
      }
      subitem = doc->nodeByPath("Place/Region", item, hasSemanticRegion);
      if (subitem && !hasSemanticRegion) {
	doc->removeElementDeep(subitem);
	subitem = NULL;
      } else if (hasSemanticRegion) {
	if (!subitem)
	  return false;
	doc->setElementValue(subitem, dlg.semanticRegion());
      }
    }
    /* semantic time */
    item = doc->nodeByPath("Semantic/SemanticBase[@xsi:type=SemanticTimeType]", root,
			   hasSemanticTimeDesc);
    if (item && !hasSemanticTimeDesc) {
      doc->removeElementDeep(item);
      item = NULL;
    } else if (hasSemanticTimeDesc) {
      if (!item)
	return false;
      MP7ListViewItem *subitem = doc->nodeByPath("Label/Name", item, true);
      if (!subitem)
	return false;
      doc->setElementValue(subitem, "Semantic Time");
      QString mp7Time;
      if (hasSemanticDate)
	mp7Time += dlg.semanticDateEdit->text();
      if (hasSemanticTime)
	mp7Time += "T" + dlg.semanticTimeEdit->time().toString("hh:mm:ss");
      subitem = doc->nodeByPath("Time/TimePoint", item, true);
      if (!subitem)
	return false;
      doc->setElementValue(subitem, mp7Time);
      subitem = doc->nodeByPath("Time/IncrDuration", item, hasSemanticDuration);
      if (subitem && !hasSemanticDuration) {
	doc->removeElementDeep(subitem);
	subitem = NULL;
      } else if (hasSemanticDuration) {
	if (!subitem)
	  return false;
	QString durNum;
	durNum.setNum(dlg.semanticDurationSpinBox->value());
	doc->setElementValue(subitem, durNum);
	doc->setAttribute(subitem, "timeUnit", dlg.semanticDurationUnit());
      }
    }
  } /* hasSemantic */
  return true;
}

int MP7VideoSegment::childSegments(std::vector<MP7VideoSegment> &result) const
{
  result.clear();
  if (!doc || !root)
    return 0;
  MP7ListViewItem *item = doc->nodeByPath("TemporalDecomposition", root);
  if (item) {
    QPtrList<MP7ListViewItem> children = doc->children(item, "VideoSegment");
    result.reserve(children.count());
    for (MP7ListViewItem *child = children.first(); child; child = children.next()) {
      result.push_back(MP7VideoSegment(doc, child));
    }
  }
  return result.size();
}

MP7Time MP7VideoSegment::mediaTime() const
{
  MP7Time time;
  if (!doc || !root)
    return time;
  MP7ListViewItem *item = doc->nodeByPath("MediaTime/MediaTimePoint", root);
  if (item) {
    time.setTimePoint(item->text(1));
  }
  item = doc->nodeByPath("MediaTime/MediaDuration", root);
  if (item) {
    time.setDuration(item->text(1));
  }
  return time;
}

bool MP7VideoSegment::setMediaTime(const MP7Time &time)
{
  if (!doc || !root)
    return false;
  MP7ListViewItem *item;
  if (time.timePointIsValid()) {
    item = doc->nodeByPath("MediaTime/MediaTimePoint", root, true);
    if (!item)
      return false;
    doc->setElementValue(item, time.timePointMP7());
  }
  if (!time.durationIsNull()) {
    item = doc->nodeByPath("MediaTime/MediaDuration", root, true);
    if (!item)
      return false;
    doc->setElementValue(item, time.durationMP7());
  }
  return true;
}

bool MP7VideoSegment::modifyMediaStartTime(const MP7Time &time, bool recursive)
{
  MP7Time thisTime = mediaTime();
  MP7Time endTime = thisTime.endTimePoint();
  if (!thisTime.setTimePoint(time))
    return false;
  if (!thisTime.setDurationFromEnd(endTime))
    return false;
  if (!setMediaTime(thisTime))
    return false;
  bool ok = true;
  if (recursive) {
    std::vector<MP7VideoSegment> children;
    if (childSegments(children) > 0) {
      ok = children.front().modifyMediaStartTime(time);
    }
  }
  return ok;
}

bool MP7VideoSegment::modifyMediaEndTime(const MP7Time &endTime, bool recursive)
{
  MP7Time thisTime = mediaTime();
  if (!thisTime.setDurationFromEnd(endTime))
    return false;
  if (!setMediaTime(thisTime))
    return false;
  bool ok = true;
  if (recursive) {
    std::vector<MP7VideoSegment> children;
    if (childSegments(children) > 0) {
      ok = children.back().modifyMediaEndTime(endTime);
    }
  }
  return ok;
}

QString MP7VideoSegment::title() const
{
  QString null;
  if (!doc || !root)
    return null;
  MP7ListViewItem *item = doc->nodeByPath("CreationInformation/Creation/Title", root);
  if (item) {
    return item->text(1);
  }
  return null;
}

bool MP7VideoSegment::ancestor(MP7VideoSegment &result, int level) const
{
  int lev = this->level();
  if (level < -1 || level >= lev)
    return false;
  MP7ListViewItem *p = parent();
  if (level >= 0) {
    for (lev--; p && lev > level; lev--)
      p = parent(p);
  }
  if (!p)
    return false;
  result.setMP7Doc(doc);
  result.setRoot(p);
  return true;
}

int MP7VideoSegment::level() const
{
  int level = 0;
  for (MP7ListViewItem *p = parent(); p; p = parent(p))
    level++;
  return level;
}

bool MP7VideoSegment::addChildSegment(MP7VideoSegment &result)
{
  MP7ListViewItem *tempDecomp = doc->nodeByPath("TemporalDecomposition", root, true);
  if (!tempDecomp)
    return false;
  MP7ListViewItem *childRoot = doc->addElement(tempDecomp, "VideoSegment");
  if (!childRoot)
    return false;
  result.setMP7Doc(doc);
  result.setRoot(childRoot);
  return true;
}

bool MP7VideoSegment::move(MP7VideoSegment *newParent, MP7VideoSegment *newSibling)
{
  if (!newParent)
    return false;
  MP7ListViewItem *parentItem = (MP7ListViewItem *) root->parent();  // TemporalDecomposition
  MP7ListViewItem *newPItem = doc->nodeByPath("TemporalDecomposition",
					      newParent->rootItem(), true);
  if (!parentItem || !newPItem)
    return false;
  MP7ListViewItem *after = (newSibling && newSibling->isValid()) ?
    newSibling->root : NULL;
  if (!doc->moveElement(root, newPItem, after))
    return false;
  /* remove TemporalDecomposition of old parent segment, if there is
   * no child segment left.
   */
  if (!parentItem->firstChild()) {
    if (!doc->removeElementDeep(parentItem))
      return false;
  }
  return true;
}

bool MP7VideoSegment::split(MP7VideoSegment &result, const MP7Time &time)
{
  MP7Time thisTime = mediaTime();
  if (!thisTime.contains(time))
    return false;
  /* find subsegment containing split time point */
  int thisLevel = level();
  MP7VideoSegment subSeg;
  if (!searchByMediaTime(subSeg, time, thisLevel+1)) {
    return false;
  }
  /* split this segment */
  MP7ListViewItem *parentItem = (MP7ListViewItem *) root->parent(); // TemporalDecomposition
  if (!parentItem) {
    return false;
  }
  MP7ListViewItem *newSibling = doc->addElement(parentItem, root, "VideoSegment");
  if (!newSibling) {
    return false;
  }
  MP7Time siblingTime;
  siblingTime.setTimePoint(time);
  siblingTime.setDurationFromEnd(thisTime.endTimePoint());
  thisTime.setDurationFromEnd(time);
  result.setMP7Doc(doc);
  result.setRoot(newSibling);
  result.setMediaTime(siblingTime);
  setMediaTime(thisTime);
  /* split subsegment recursively */
  if (subSeg.level() > thisLevel) {
    MP7VideoSegment subSibling;
    if (!subSeg.split(subSibling, time)) {
      delete newSibling;
      return false;
    }
    /* move appropriate subsegments to result */
    std::vector<MP7VideoSegment> subSegments;
    for (int i = childSegments(subSegments)-1; i >=0; i--) {
      if (time.compareTimePoint(subSegments.at(i).mediaTime()) <= 0) {
	if (!subSegments.at(i).move(&result)) {
	  delete newSibling;
	  return false;
	}
      }
    }
  }
  return true;
}

bool MP7VideoSegment::remove(bool recursive)
{
  bool ok = true;
  MP7VideoSegment parent;
  if (!doc || !root || !ancestor(parent))
    return false;
  MP7ListViewItem *parentItem = (MP7ListViewItem *) root->parent(); // TemporalDecomposition node
  if (!parentItem) {
    dprintf_err("MP7VideoSegment::remove() this = %s, parent MP7ListViewItem not found!\n",
		root->text(0).latin1());
    return false;
  }
  /* if there are no child segments, force <recursive> to true
   * in order to update sibling's children correctly
   */
  std::vector<MP7VideoSegment> children;
  int childCount = childSegments(children);
  if (childCount == 0)
    recursive = true;
  /* adapt sibling's media time */
  MP7VideoSegment sibSeg;
  MP7Time sibSegTime;		// original media time of sibSeg
  sibSeg.setMP7Doc(doc);
  /* where to move this segment's children: 
   * -1 previous sibling
   * 0  parent
   * 1  next sibling
   */
  int moveTo = 0;
  for (MP7ListViewItem *sib = (MP7ListViewItem *) parentItem->firstChild();
       sib; sib = (MP7ListViewItem *) sib->nextSibling()) {
    /* if this segment has a previous sibling, increase its duration
     * to include this segment.
     */
    if ((MP7ListViewItem *) sib->nextSibling() == root) {
      sibSeg.setRoot(sib);
      sibSegTime = sibSeg.mediaTime();
      ok = sibSeg.modifyMediaEndTime(mediaTime().endTimePoint(), recursive);
      moveTo = -1;
      break;
    }
    /* if this segment has a next sibling, modify its start time and duration
     * to include this segment.
     */
    if (sib == root && sib->nextSibling()) {
      sibSeg.setRoot((MP7ListViewItem *) sib->nextSibling());
      sibSegTime = sibSeg.mediaTime();
      ok = sibSeg.modifyMediaStartTime(mediaTime(), recursive);
      moveTo = 1;
      break;
    }
  }
  if (ok) {
    /* move child segments */
    if (!recursive && childCount > 0) {
      // sibling's child after which this segment's children should be inserted
      MP7VideoSegment sibChildPos;
      /* if there is no sibling, use the parent segment */
      if (moveTo == 0) {
	sibSeg = parent;
	sibChildPos = *this;	// insert after this segment
      }
      /* if <sibSeg> has no child segments, create a
       * child segment of <sibSeg> with <sibSegTime> before
       * moving the current child segments to <sibSeg>.
       */
      std::vector<MP7VideoSegment> sibChildren;
      if (sibSeg.childSegments(sibChildren) == 0) {
	MP7VideoSegment newChild;
	sibSeg.addChildSegment(newChild);
	newChild.setMediaTime(sibSegTime);
      }
      if (moveTo == -1) {
	sibChildPos = sibChildren.back();
      }
      /* if (moveTo == 1) leave sibChildPos invalid */

      /* move every child segment to <sibSeg>, as next sibling of <sibChildPos>;
       * as this loop will invert the order of the operands,
       * we start with moving the last child segment.
       */
      for (int i = (int) children.size() - 1; i >= 0; i--) {
	if (!children.at(i).move(&sibSeg, &sibChildPos)) {
	  dprintf_err("MP7VideoSegment::remove() this = %s, moving child segment %s "
		      "to new parent %s FAILED!\n",
		      mediaTime().toString().latin1(),
		      children.at(i).mediaTime().toString().latin1(),
		      sibSeg.mediaTime().toString().latin1());
	  ok = false;
	  break;
	}
      }
    }
  }
  if (ok) {
    /* remove this segment from the MPEG-7 document */
    if ((ok = doc->removeElementDeep(root))) {
      /* remove TemporalDecomposition node if this segment was the only child */
      if (!parentItem->firstChild()) {
	if (!doc->removeElementDeep(parentItem)) {
	  dprintf_err("MP7VideoSegment::remove() removing parent "
		      "TemporalDecomposition node FAILED!\n");
	  ok = false;
	}
      }
    }
  }
  if (!ok) {
    dprintf_err("MP7VideoSegment::remove() FAILED - this segment: %s (%s)\n",
		title().latin1(), mediaTime().toString().latin1());
    return false;
  }
  return true;
}

bool MP7VideoSegment::searchByMediaTime(MP7VideoSegment &result,
					const MP7Time &time, int level) const
{
  int lev = this->level();
  if (level < lev || !mediaTime().contains(time))
    return false;
  if (lev == level) {
    result = *this;
    return true;
  }
  std::vector<MP7VideoSegment> children;
  int size = childSegments(children);
  for (int i=0; i < size; i++) {
    if (children.at(i).searchByMediaTime(result, time, level))
      return true;
  }
  /* there is no segment at <level>, but <this> contains <time> */
  result = *this;
  return true;
}

bool MP7VideoSegment::prevGenSibling(MP7VideoSegment &result) const
{
  if (!isValid())
    return false;
  /* find previous sibling of (this segment or its nearest ancestor) */
  MP7ListViewItem *current = root;
  MP7ListViewItem *ancestorSib = NULL;
  while (current && !ancestorSib) {
    ancestorSib = current->prevSibling();
    current = MP7VideoSegment(doc, current).parent();
  }
  if (!ancestorSib)
    return false;		// generalized sibling does not exist
  /* descend to right-most successor of ancestorSib at this->level() */
  result.setMP7Doc(doc);
  result.setRoot(ancestorSib);
  int thisLevel = this->level();
  int level = result.level();
  for ( ; level < thisLevel; level++) {
    std::vector<MP7VideoSegment> children;
    if (result.childSegments(children) == 0)
      break;
    result = children.back();
  }
  return level == thisLevel;
}

bool MP7VideoSegment::nextGenSibling(MP7VideoSegment &result) const
{
  if (!isValid())
    return false;
  /* find next sibling of (this segment or its nearest ancestor) */
  MP7ListViewItem *current = root;
  MP7ListViewItem *ancestorSib = NULL;
  while (current && !ancestorSib) {
    ancestorSib = (MP7ListViewItem *) current->nextSibling();
    current = MP7VideoSegment(doc, current).parent();
  }
  if (!ancestorSib)
    return false;		// generalized sibling does not exist
  /* descend to left-most successor of ancestorSib at this->level() */
  result.setMP7Doc(doc);
  result.setRoot(ancestorSib);
  int thisLevel = this->level();
  int level = result.level();
  for ( ; level < thisLevel; level++) {
    std::vector<MP7VideoSegment> children;
    if (result.childSegments(children) == 0)
      break;
    result = children.front();
  }
  return level == thisLevel;
}

MP7ListViewItem *MP7VideoSegment::setTitleImage(const MP7Time &time)
{
  if (!isValid() || !mediaTime().contains(time))
    return NULL;
  MP7ListViewItem *item =
    doc->nodeByPath("CreationInformation/Creation/TitleMedia/TitleImage/MediaTimePoint",
		    root, true);
  if (!item)
    return NULL;
  doc->setElementValue(item, time.timePointMP7());
  return item;
}
