/***********************************************************************
 *                                                                      *
 * MuViNo - ViTooKi Video Annotation Tool                               *
 *                                                                      *
 * title:	cretioninformation.cpp                                  *
 *                                                                      *
 * $Id: creationinformation.cpp,v 1.2 2006/03/08 08:17:30 mkropfbe Exp $
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

#include <qcombobox.h>
#include <qlineedit.h>
#include <qvalidator.h>

#include "creationinformation.hpp"

#include "debug.hpp"

struct country {
  char *iso;			// ISO 2-letter country code
  char *name;			// country name
};

/* countries displayed by semanticRegionComboBox
 * sorted by <name>, terminated by NULL entry 
 */
static const struct country countries[] = {
  { "AF", "Afghanistan" },
  { "AL", "Albania" },
  { "DZ", "Algeria" },
  { "AS", "American Samoa" },
  { "AD", "Andorra" },
  { "AO", "Angola" },
  { "AI", "Anguilla" },
  { "AQ", "Antarctica" },
  { "AG", "Antigua and Barbuda" },
  { "AR", "Argentina" },
  { "AM", "Armenia" },
  { "AW", "Aruba" },
  { "AU", "Australia" },
  { "AT", "Austria" },
  { "AZ", "Azerbaijan" },
  { "BS", "Bahamas" },
  { "BH", "Bahrain" },
  { "BD", "Bangladesh" },
  { "BB", "Barbados" },
  { "BY", "Belarus" },
  { "BE", "Belgium" },
  { "BZ", "Belize" },
  { "BJ", "Benin" },
  { "BM", "Bermuda" },
  { "BT", "Bhutan" },
  { "BO", "Bolivia" },
  { "BA", "Bosnia and Herzegovina" },
  { "BW", "Botswana" },
  { "BV", "Bouvet Island" },
  { "BR", "Brazil" },
  { "BN", "Brunei" },
  { "BG", "Bulgaria" },
  { "BF", "Burkina Faso" },
  { "MM", "Burma" },
  { "BI", "Burundi" },
  { "KH", "Cambodia" },
  { "CM", "Cameroon" },
  { "CA", "Canada" },
  { "CV", "Cape Verde" },
  { "KY", "Cayman Islands" },
  { "CF", "Central Africa" },
  { "TD", "Chad" },
  { "CL", "Chile" },
  { "CN", "China" },
  { "CX", "Christmas Island (Australia)" },
  { "CC", "Cocos (Keeling) Islands (Australia)" },
  { "CO", "Colombia" },
  { "KM", "Comores" },
  { "CG", "Congo" },
  { "CK", "Cook Islands" },
  { "CR", "Costa Rica" },
  { "CI", "Cote d'Ivoire" },
  { "HR", "Croatia" },
  { "CU", "Cuba" },
  { "CY", "Cyprus" },
  { "CZ", "Czech Republic" },
  { "CS", "Czechoslovakia" },
  { "DK", "Denmark" },
  { "DJ", "Djibouti" },
  { "DM", "Dominica" },
  { "DO", "Dominican Republic" },
  { "EC", "Ecuador" },
  { "EG", "Egypt" },
  { "SV", "El Salvador" },
  { "GQ", "Equatorial Guinea" },
  { "ER", "Eritrea" },
  { "EE", "Estonia" },
  { "ET", "Ethiopia" },
  { "FK", "Falkland Islands" },
  { "FO", "Faroe Islands" },
  { "FJ", "Fiji" },
  { "FI", "Finland" },
  { "FR", "France" },
  { "GF", "French Guiana" },
  { "PF", "French Polynesia" },
  { "GA", "Gabon" },
  { "GE", "Georgia" },
  { "DE", "Germany" },
  { "GH", "Ghana" },
  { "GI", "Gibraltar (UK)" },
  { "GR", "Greece" },
  { "GL", "Greenland" },
  { "GD", "Grenada" },
  { "GP", "Guadaloupe (France)" },
  { "GU", "Guam" },
  { "GT", "Guatemala" },
  { "GN", "Guinea" },
  { "GW", "Guinea-Bissau" },
  { "GY", "Guyana" },
  { "HT", "Haiti" },
  { "HM", "Heard Island and McDonald Islands (Australia)" },
  { "VA", "Holy See" },
  { "HN", "Honduras" },
  { "HK", "Hong Kong (UK)" },
  { "HU", "Hungary" },
  { "IN", "India" },
  { "IO", "Indian Ocean Territory (UK)" },
  { "ID", "Indonesia" },
  { "IR", "Iran" },
  { "IQ", "Iraq" },
  { "IE", "Ireland" },
  { "IS", "Island" },
  { "IL", "Israel" },
  { "IT", "Italy" },
  { "JM", "Jamaica" },
  { "JP", "Japan" },
  { "JO", "Jordan" },
  { "KZ", "Kazakhstan" },
  { "KE", "Kenya" },
  { "KI", "Kiribati" },
  { "KP", "Korea" },
  { "KR", "Korea" },
  { "KW", "Kuwait" },
  { "KG", "Kyrgyzstan" },
  { "LA", "Laos" },
  { "LV", "Latvia" },
  { "LB", "Lebanon" },
  { "LS", "Lesotho" },
  { "LR", "Liberia" },
  { "LY", "Libya" },
  { "LI", "Liechtenstein" },
  { "LT", "Lietuva" },
  { "LU", "Luxembourg" },
  { "MO", "Macau" },
  { "MK", "Macedonia" },
  { "MG", "Madagascar" },
  { "MW", "Malawi" },
  { "MY", "Malaysia" },
  { "MV", "Maldives" },
  { "ML", "Mali" },
  { "MT", "Malta" },
  { "MH", "Marshall Islands" },
  { "MQ", "Martinique (France)" },
  { "MR", "Mauritania" },
  { "MU", "Mauritius" },
  { "YT", "Mayotte (France)" },
  { "FX", "Metropolitan France" },
  { "MX", "Mexico" },
  { "FM", "Micronesia" },
  { "MD", "Moldova" },
  { "MC", "Monaco" },
  { "MN", "Mongolia" },
  { "MS", "Montserrat" },
  { "MA", "Morocco" },
  { "MZ", "Mozambique" },
  { "NA", "Namibia" },
  { "NR", "Nauru" },
  { "NP", "Nepal" },
  { "NL", "Netherlands" },
  { "AN", "Netherlands Antilles" },
  { "NC", "New Caledonia" },
  { "NZ", "New Zealand" },
  { "NI", "Nicaragua" },
  { "NE", "Niger" },
  { "NG", "Nigeria" },
  { "NU", "Niue" },
  { "NF", "Norfolk Island (Australia)" },
  { "MP", "Northern Mariana Islands" },
  { "NO", "Norway" },
  { "OM", "Oman" },
  { "PK", "Pakistan" },
  { "PW", "Palau" },
  { "PA", "Panama" },
  { "PG", "Papua New Guinea" },
  { "PY", "Paraguay" },
  { "PE", "Peru" },
  { "PH", "Philippines" },
  { "PN", "Pitcairn Islands" },
  { "PL", "Poland" },
  { "PT", "Portugal" },
  { "PR", "Puerto Rico" },
  { "QA", "Qatar" },
  { "RE", "Reunion" },
  { "RO", "Romania" },
  { "RU", "Russia" },
  { "RW", "Rwanda" },
  { "SH", "Saint Helena (UK)" },
  { "KN", "Saint Kitts and Nevis" },
  { "LC", "Saint Lucia" },
  { "VC", "Saint Vincent and the Grenadines" },
  { "PM", "Saint-Pierre et Miquelon" },
  { "WS", "Samoa" },
  { "SM", "San Marino" },
  { "ST", "Sao Tome e Principe" },
  { "SA", "Saudi Arabia" },
  { "SN", "Senegal" },
  { "SC", "Seychelles" },
  { "SL", "Sierra Leone" },
  { "SG", "Singapore" },
  { "SK", "Slovakia" },
  { "SI", "Slovenia" },
  { "SB", "Solomon Islands" },
  { "SO", "Somalia" },
  { "ZA", "South Africa" },
  { "GS", "South Georgia" },
  { "TF", "Southern and Antarctic Lands" },
  { "SU", "Soviet Union" },
  { "ES", "Spain" },
  { "LK", "Sri Lanka" },
  { "SD", "Sudan" },
  { "SR", "Suriname" },
  { "SJ", "Svalbard og Jan Mayen" },
  { "SZ", "Swaziland" },
  { "SE", "Sweden" },
  { "CH", "Switzerland" },
  { "SY", "Syria" },
  { "TW", "Taiwan" },
  { "TJ", "Tajikistan" },
  { "TZ", "Tanzania" },
  { "TH", "Thailand" },
  { "GM", "The Gambia" },
  { "TP", "Timor" },
  { "TG", "Togo" },
  { "TK", "Tokelau (New Zealand)" },
  { "TO", "Tonga" },
  { "TT", "Trinidad and Tobago" },
  { "TN", "Tunisia" },
  { "TR", "Turkey" },
  { "TM", "Turkmenistan" },
  { "TC", "Turks and Caicos Islands" },
  { "TV", "Tuvalu" },
  { "UG", "Uganda" },
  { "UA", "Ukraine" },
  { "AE", "United Arab Emirates" },
  { "GB", "United Kingdom" },
  { "UM", "United States Minor Outlying Islands" },
  { "US", "United States of America" },
  { "UY", "Uruguay" },
  { "UZ", "Uzbekistan" },
  { "VU", "Vanuatu" },
  { "VE", "Venezuela" },
  { "VN", "Vietnam" },
  { "VG", "Virgin Islands (UK)" },
  { "VI", "Virgin Islands (USA)" },
  { "WF", "Wallis et Futuna" },
  { "EH", "Western Sahara" },
  { "YD", "Yemen" },
  { "YE", "Yemen" },
  { "YU", "Yugoslavia" },
  { "ZR", "Zaire" },
  { "ZM", "Zambia" },
  { "ZW", "Zimbabwe" },
  { 0, 0 }
};

CreationInformation::CreationInformation(QWidget *parent, const char *name, bool modal, WFlags f)
  : CreationInformationDlg(parent, name, modal, f)
{
/* initialize creationRegionComboBox */
  creationRegionComboBox->insertItem("(no region)");
  for (int i=0; countries[i].iso; i++) {
    creationRegionComboBox->insertItem(QString("%1 (%2)")
				       .arg(countries[i].name)
				       .arg(countries[i].iso));
  }
  /* set validator of creationDateEdit */
  QRegExp rx("\\d+(-\\d{2}(-\\d{2})?)?");
  QRegExpValidator *v = new QRegExpValidator(rx, creationDateEdit);
  creationDateEdit->setValidator(v);
}

bool CreationInformation::creationRegionIsValid() const
{
  return creationRegionComboBox->currentItem() != 0;
}

QString CreationInformation::creationRegion() const
{
  int index = creationRegionComboBox->currentItem();
  if (index == 0)
    return QString();
  return QString(countries[index-1].iso);
}

bool CreationInformation::setCreationRegion(const QString &countryCode)
{
  for (int i=0; countries[i].iso; i++) {
    if (countryCode == countries[i].iso) {
      creationRegionComboBox->setCurrentItem(i+1);
      return true;
    }
  }
  dprintf_err("CreationInformation::setCreationRegion() countryCode = %s NOT FOUND!\n",
	      countryCode.latin1());
  return false;
}



