/***********************************************************************
 *                                                                      *
 * MuViNo - ViTooKi Video Annotation Tool                               *
 *                                                                      *
 * title:	cretioninformation.hpp                                  *
 *                                                                      *
 * $Id: creationinformation.hpp,v 1.3 2006/03/08 08:17:30 mkropfbe Exp $
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/
 
#ifndef _CREATIONINFORMATION_HPP_
#define _CREATIONINFORMATION_HPP_

/** @file
 * Declaration of the creationinformation class.
 */

#include "creationinformationdlg.hpp"

/** This class implements the (simple) application logic of CreationInformationDlg,
 * the GUI dialog for editing the MPEG-7 descriptors of the creationinformation of a video.
 * @author Armin Mueller
 * @version $Id: creationinformation.hpp,v 1.3 2006/03/08 08:17:30 mkropfbe Exp $
 */

class CreationInformation : public CreationInformationDlg
{
public:
  /** Constructor. */
  CreationInformation(QWidget *parent = 0, const char *name = 0,
	       bool modal = false, WFlags f = 0);

  /** Return <code>true</code> if the current value of <code>creationRegionComboBox</code>
   * denotes a valid country.
   * @return <code>false</code> if the current value of <code>creationRegionComboBox</code>
   * is not a valid country code.
   */
  bool creationRegionIsValid() const;

  /** Return the 2-letter ISO country code of the current 
   * <code>creationRegionComboBox</code> entry.
   */
  QString creationRegion() const;

  /** Set the current item of <code>creationRegionComboBox</code>
   * according to the given 2-letter ISO country code.
   * @return <code>false</code> if the given country code could not be found.
   */
  bool setCreationRegion(const QString &countryCode);
  
};

 
#endif

