 /***********************************************************************
 *                                                                      *
 * MuViNo - ViTooKi Video Annotation Tool                               *
 *                                                                      *
 * title: main.cpp                                                      *
 *                                                                      *
 * $Id: main.cpp,v 1.4 2005/01/20 09:47:08 mkropfbe Exp $
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <qapplication.h>
#include <qfont.h>
#include <qstring.h>
#include <qtextcodec.h>
#include <qtranslator.h>

#include "muvinomain.hpp"

void printUsage(char *progname)
{
  fprintf(stderr, "Usage: %s [-h] [--video <mp4file> [--gray]] [<mp7file>]\n"
	  "If <mp7file> is given, open it as MPEG-7 document.\n"
	  "--video     load first visual stream of <mp4file> as video\n"
	  "--gray      render to gray-scale color space\n"
	  "Demuxed MP4 elementary streams will be stored in the current directory.\n",
	  progname);
}

int main(int argc, char *argv[])
{
  char *progname = argv[0];
  bool arg_invalid = false;
  QString mp7file;
  char *video_file = NULL;
  const char *curr_dir = ".";
  bool opt_gray = false;
  QApplication a(argc, argv);

  /* parse command-line arguments */
  while (!arg_invalid && --argc > 0) {
    argv++;
    if (strcmp(*argv, "-h") == 0) {
      printUsage(progname);
      return 0;
    } else if (strcmp(*argv, "--video") == 0) {
      if (--argc <= 0) {
	arg_invalid = true;
	break;
      }
      video_file = *++argv;
    } else if (strcmp(*argv, "--gray") == 0) {
      opt_gray = true;
    } else if (mp7file.isEmpty() && (*argv)[0] != '-') {
      mp7file = *argv;
    } else {
      arg_invalid = true;
      break;
    }
  }
  if (arg_invalid) {
    printUsage(progname);
    return 1;
  }

//  a.setFont(QFont("helvetica", 12));
  MuvinoMain *muvino=new MuvinoMain();
  a.setMainWidget(muvino);

  /* open MPEG-7 document */
  if (!mp7file.isEmpty()) {
    if (!muvino->loadMP7Doc(mp7file)) {
      fprintf(stderr, "Loading MPEG-7 file %s FAILED!\n",
	      mp7file.latin1());
      delete muvino;
      return 1;
    }
  }

  /* load video */
  if (video_file) {
    if (!muvino->loadVideo(video_file, curr_dir,
			   opt_gray ? UncompressedVideoFrame::ColorSpaceGRAY8
			   : UncompressedVideoFrame::ColorSpaceRGB24)) {
      fprintf(stderr, "Loading video %s FAILED!\n", video_file);
      delete muvino;
      return 1;
    }
    muvino->playerPlay();
  }

  muvino->show();
  return a.exec();
}


