 /***********************************************************************
 *                                                                      *
 * MuViNo - ViTooKi Video Annotation Tool                               *
 *                                                                      *
 * title: muvinomain.cpp                                                *
 *                                                                      *
 * $Id: muvinomain.cpp,v 1.31 2005/11/17 16:55:34 mkropfbe Exp $
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <cmath>

#include <qaction.h>
#include <qapplication.h>
#include <qcheckbox.h>
#include <qdir.h>
#include <qfiledialog.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlistview.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qspinbox.h>
#include <qstring.h>
#include <qtextedit.h>
#include <qtimer.h>

#include <vitooki.hpp>
#include <DataChannel.hpp>
#include <DataSink.hpp>
#include <ContainerInfo.hpp>
#include <ESInfo.hpp>
#include "VideoESInfo.hpp"
#include "adaptors/MP4Decoder.hpp"
#include <AudioAdaptorChain.hpp>
#include <VideoAdaptorChain.hpp>
#include <adaptors/MP4audioDecoder.hpp>
#include <adaptors/MP4Decoder.hpp>
#include <adaptors/ESSynchronizer.hpp>
#include <adaptors/GlobalTimer.hpp>
#include <adaptors/SharedAdaptor.hpp>
#include "adaptors/FramePatternDetector.hpp"
#include <container/ContainerDemux.hpp>
#include <io/SDLaudioIO.hpp>
#include <io/IOCreator.hpp>
#include <io/MPGStreamIO.hpp>
#include <io/QLabelRenderer.hpp>
#include "net/Url.hpp"

#ifdef WIN32
#include <crtdbg.h>
#endif
#include <container/ContainerFile.hpp>
#ifndef ISOMP4
#include <io/ffMP4IO.hpp>
#else
#include <io/isoMP4IO.hpp>
#endif

#include "muvinomain.hpp"
#include "MP7ListViewItem.hpp"
#include "SegmentSlider.hpp"


/* mediaTimeUpdater timer interval in milliseconds */
#define MEDIA_TIME_UPDATER_INTERVAL_MS   250

/* tolerance in milliseconds by which the durations of
 * the MPEG-7 document and the video may differ
 */
#define MP7DOC_VIDEO_DURATION_TOLERANCE_MS  2000

const QString MuvinoMain::caption("Muvino 0.12");

MuvinoMain::MuvinoMain()
{
  setCaptionFromFile();
  videoDC = NULL;
  audioDC = NULL;
  globalTimer = NULL;
  essync = NULL;
  mediaTimeUpdater = new QTimer(this, "mediaTimeUpdater");
  // mediaTimeUpdater will be automatically deleted by QObject destructor
  doc = NULL;
  currentMP7Item = NULL;
  resetVideoDisplay();
  vitooki_init_codecs();
  initMP7TreeView();
  resetMP7ElementValue();
  initSegmentNav();

  // signals and slots connections
  connect(qApp, SIGNAL(aboutToQuit()), this, SLOT(closeVideo()));
  connect(qApp, SIGNAL(lastWindowClosed()), qApp, SLOT(quit()));
  connect(mediaTimeUpdater, SIGNAL(timeout()), this, SLOT(displayCurrentMediaTime()));
  
}

MuvinoMain::~MuvinoMain()
{
  closeVideo();
  resetSegmentNav();
  if (doc)
    delete doc;
}

void MuvinoMain::setCaptionFromFile(const QString &file, bool modified)
{
  QString s;
  if (modified)
    s.append("* ");
  if (!file.isEmpty()) {
    QDir path(file);
    s.append(path.dirName());
    s.append(" - ");
  }
  s.append(caption);
  setCaption(s);
}

/* ------------------ Event Handlers -------------------------------------- */

void MuvinoMain::closeEvent(QCloseEvent *e)
{
  if (warnAndSaveMP7Doc())
    e->accept();
  else
    e->ignore();
}

/* ------------------ Video Player Functions ------------------------------ */

bool MuvinoMain::loadVideo(const char *filename, const char *demuxDir,
			   UncompressedVideoFrame::ColorSpace colorSpace)
{
  closeVideo();
  // now check for config file
	char confFile[4096];
	ContainerInfo *mp4 = NULL;
	//dprintf_full("MuvinoMain::loadVideo relative name X%sX\r\n",&t[offset]);
	// build url
	char baseDir[4096];
	
	if(demuxDir)
		sprintf(baseDir,"%s",demuxDir);
	else
		sprintf(baseDir,".");
	
	
	//Url* url=new Url(pTemp);
	Url* url=new Url(filename);
	
	
	char fileOnDisk[1024];
	//char demuxDir[1024];
	sprintf(fileOnDisk,"%s",filename);
	char* dummyStr=Url::transformPathToLocalFile(url);
	sprintf(confFile,"%s/%s.conf",baseDir, url->getFileName());
	//sprintf(demuxDir,"%s",baseDir);
	delete[] dummyStr;
#ifdef WIN32
	unsigned int i=0;
	for(i=0;i<strlen(fileOnDisk);i++) {
		if(fileOnDisk[i]=='/')
			fileOnDisk[i]='\\';
	}
	for(i=0;i<strlen(confFile);i++) {
		if(confFile[i]=='/')
			confFile[i]='\\';
	}
	for(i=0;i<strlen(baseDir);i++) {
		if(baseDir[i]=='/')
			baseDir[i]='\\';
	}
#endif
	FILE *tmpFp=fopen(confFile,"rb");
	if(tmpFp) {
		fclose(tmpFp);
		// we have already the demultiplexed es 
		mp4=new ContainerInfo(filename, filename, ContainerInfo::SDP);
		if(!mp4->loadFromCfgFile(confFile)) {
			delete mp4; 
			mp4 = ContainerFile::loadContainerInfo( fileOnDisk,url );	
			if(mp4) {
				FramePatternDetector* fpd=new FramePatternDetector(mp4->getFirstVisualES(),true);
				ContainerDemux::demultiplexAndUpdateIO(mp4,baseDir,fpd);
				delete fpd;
				mp4->saveToCfgFile(confFile);
			}
		}
	}
	else {
		mp4 = ContainerFile::loadContainerInfo(fileOnDisk,url );
		
		if(mp4) {
			dprintf_full("MuvinoMain::loadVideo Before demultiplex\r\n");
			FramePatternDetector* fpd=new FramePatternDetector(mp4->getFirstVisualES(),true);
			ContainerDemux::demultiplexAndUpdateIO(mp4,baseDir,fpd);
			delete fpd;
			if(mp4->saveToCfgFile(confFile))
				fprintf(stderr, "Saved to confFile %s\n",confFile);
			else
				fprintf(stderr, "COULD NOT Save to confFile <%s>\n",confFile);
			dprintf_full("MuvinoMain::loadVideo After demultiplex %s\r\n",url->toString());
		}
	}
	mp4->setUrl(url);
  
  //ContainerDemux::demultiplexAndUpdateIO(mp4, demuxDir);
  
  
  list < ESInfo * >* streams = mp4->getESList();
  /* setup video stream first */
  list < ESInfo * >::const_iterator li = streams->begin();
  while (li != streams->end()) {
    ESInfo* es=(*li);
    if (!videoDC && es->isVisualStream()) {
      dprintf_full("MuvinoMain::loadVideo(): ES %llu, visual = %d, input = %s\n",
									 es->getStreamId(), es->isVisualStream(), es->getInput());
      VideoESInfo *esVideo = (VideoESInfo *) es;
      QLabelRenderer *renderer = new QLabelRenderer(esVideo, colorSpace,
																										videoWidget, false);
      if (!renderer) {
				dprintf_err("MuvinoMain::loadVideo(): QLabelRenderer constructor FAILED!\n");
				delete mp4;
				return false;
      }
      IO *input = IOCreator::createInput(es);
      if (!input) {
				dprintf_err("MuvinoMain::loadVideo(): IOCreator::createInput(esVideo) FAILED!\n");
				delete renderer;
				delete mp4;
				return false;
      }
      MP4Decoder *decoder = new MP4Decoder((VideoESInfo *) es, true, 
																					 UncompressedVideoFrame::ColorSpaceRGB24);
      if (!decoder) {
				dprintf_err("MuvinoMain::loadVideo(): MP4Decoder constructor FAILED!\n");
				delete input;
				delete renderer;
				delete mp4;
				return false;
      }
      if (!globalTimer) {
				globalTimer = new GlobalTimer();
				globalTimerRefCnt = new ReferenceCounter();
      }
      if (!essync)
				essync = new ESSynchronizer(es, globalTimer);
      VideoAdaptorChain *chain = new VideoAdaptorChain();
      chain->setESInfo(es);
      Adaptor* tmp=(Adaptor *) globalTimer; // necessary for VC .NET
      chain->addAdaptor(new SharedAdaptor(globalTimerRefCnt, tmp));
      chain->addAdaptor(essync);
      chain->addAdaptor(decoder);
      videoDC = new DataChannel(input, es, chain);
      if (!videoDC) {
				dprintf_err("MuvinoMain::loadVideo(): DataChannel constructor FAILED!\n");
				delete chain;
				delete input;
				delete renderer;
				delete mp4;
				delete globalTimer;
				globalTimer = NULL;
				return false;
      }
      videoDC->insert(new DataSink(0, renderer, NULL, 0));
      videoDC->play(0, 0.0);
      break;
    }
    ++li;
  }
  /* setup audio stream after video stream */
  li = streams->begin();
  while (li != streams->end()) {
    ESInfo* es=(*li);
    if (!audioDC && es->isAudioStream()) {
      dprintf_full("MuvinoMain::loadVideo(): ES %llu, visual = %d, input = %s\n",
									 es->getStreamId(), es->isVisualStream(), es->getInput());
      AudioESInfo *esAudio = (AudioESInfo *) es;
      IO *input = IOCreator::createInput(es);
      if (!input) {
				dprintf_err("MuvinoMain::loadVideo(): IOCreator::createInput(esAudio) FAILED!\n");
				delete mp4;
				return false;
      }
      if (!globalTimer) {
				globalTimer = new GlobalTimer();
				globalTimerRefCnt = new ReferenceCounter();
      }
      MP4audioDecoder *decoder = new MP4audioDecoder(esAudio, true);
      SDLaudioIO *audio_io = new SDLaudioIO(esAudio, essync);
      AudioAdaptorChain *chain = new AudioAdaptorChain();
      chain->setESInfo(es);
      chain->addAdaptor(decoder);
      Adaptor* tmp=(Adaptor *) globalTimer; // for VC .NET
      chain->addAdaptor(new SharedAdaptor(globalTimerRefCnt, tmp));
      audioDC = new DataChannel(input, es, chain);
      audioDC->insert(new DataSink(0, audio_io, NULL, 0));
      audioDC->play(0, 0.0);
      break;
    }
    ++li;
  }
  /* initialize player controls */
  setDisplayRateSlider(1.0);
  setCurrentVideoTimeLabel(MP7Time());
  audioCheckBox->setChecked(audioDC != NULL);
  audioCheckBox->setDisabled(audioDC == NULL);
  /* signal - slot connections */
  connect(currentSlider, SIGNAL(sliderPressed()),
					this, SLOT(playerStop()));
  connect(currentSlider, SIGNAL(sliderMoved(const MP7Time &)),
					this, SLOT(playVideo(const MP7Time &)));
  connect(totalSlider, SIGNAL(sliderPressed()),
					this, SLOT(playerStop()));
  connect(totalSlider, SIGNAL(sliderMoved(const MP7Time &)),
					this, SLOT(playVideo(const MP7Time &)));
  return true;
}



void MuvinoMain::closeVideo()
{
  if (!videoDC)
    return;
  disconnect(currentSlider, SIGNAL(sliderPressed()),
	     this, SLOT(playerStop()));
  disconnect(currentSlider, SIGNAL(sliderMoved(const MP7Time &)),
	     this, SLOT(playVideo(const MP7Time &)));
  disconnect(totalSlider, SIGNAL(sliderPressed()),
	     this, SLOT(playerStop()));
  disconnect(totalSlider, SIGNAL(sliderMoved(const MP7Time &)),
	     this, SLOT(playVideo(const MP7Time &)));
  mediaTimeUpdater->stop();
  globalTimer->setPaused(false); // de-block before closing
  stopVideoAudio();
  if (videoDC)
    delete videoDC;		// deletes all Adaptors and DataSinks as well
  videoDC = NULL;
  if (audioDC)
    delete audioDC;
  audioDC = NULL;
  essync = NULL;
  globalTimer = NULL;
  globalTimerRefCnt = NULL;	// deleted by ~SharedAdaptor()
}


void MuvinoMain::stopAudio()
{
  if (!audioDC)
    return;
  if (audioDC->running()) {
    audioDC->pause(0);
    audioDC->initiateThreadStop(true);
  }
}

void MuvinoMain::stopVideoAudio()
{
  if (videoDC && videoDC->running()) {
      videoDC->pause(0);
  }
  if (audioDC && audioDC->running()) {
      audioDC->pause(0);
  }
  if (videoDC && videoDC->running()) {
    videoDC->initiateThreadStop(true);
  }
  if (audioDC && audioDC->running()) {
    audioDC->initiateThreadStop(true);
  }
}

void MuvinoMain::pauseVideoAudio()
{
  if (videoDC && videoDC->running()) {
      videoDC->pause(0);
  }
  if (audioDC && audioDC->running()) {
      audioDC->pause(0);
  }
}

void MuvinoMain::resetVideoDisplay()
{
  QPixmap pixmap(videoWidget->width(), videoWidget->height());
  pixmap.fill(Qt::black);
  videoWidget->setPixmap(pixmap);
  mediaTimeUpdater->stop();
  setCurrentVideoTimeLabel(MP7Time());
}

bool MuvinoMain::playVideo(const MP7Time &time)
{
  MP7Time starttime(time);
  u32 frameNo;
  long iframeNo;
  const ESInfo *es, *audio_es;
  bool enableAudio;
  if (!videoDC)			// no video loaded
    return false;
  es = videoDC->getESInfo();
  enableAudio = audioCheckBox->isChecked() &&
    audioDC != NULL && essync->getOutputRateFactor() == 1.0;
  if (!time.timePointIsValid()) {
    /* avoid re-positioning audioDC if DC threads already running */
    if (enableAudio && videoDC->running() && audioDC->running())
      return true;
    starttime = currentVideoTime();	// needed for re-positioning audioDC
  } else {
    MP7Time videoDur = videoDuration();
    if (!videoDur.contains(starttime)) {
      dprintf_small("MuvinoMain::playVideo(time = %s) WARNING: starttime out of range, "
		    "starting at 0!\n",
		    time.timePointPretty().latin1());
      starttime.setTimePoint(0);
    }
    if (videoDC->running())
      playerStop();
    /* clear adaptor buffers */
    globalTimer->setPaused(false); // de-block before closing
    videoDC->getAdaptor()->close();
    /* re-position videoDC */
    frameNo = es->getFrameNumber(starttime.timeToSec());
    iframeNo = videoDC->getInput()->setToClosestIFrame(frameNo);
    if (iframeNo < 0) {
      dprintf_err("MuvinoMain::playVideo(time = %s) ERROR: setToClosestIFrame(%u) FAILED!\n",
		  time.timePointPretty().latin1(), frameNo);
      return false;
    }
    starttime = currentVideoTime();
    dprintf_full("MuvinoMain::playVideo(): adjusting to I-Frame at %s (frameNo %ld)\n",
		 starttime.timePointPretty().latin1(), iframeNo);
    essync->adjust(starttime.timeToSec());
  }
  /* start DataChannel threads */
  if (!globalTimer->isPaused())
    globalTimer->setPaused(true);  // block frame processing in DataChannels
  if (enableAudio) {
    /* clear audioDC buffers */
    if (audioDC->getDataSink(0))
      ((SDLaudioIO *) audioDC->getDataSink(0)->getOutput())->clearBuffer();
    audioDC->getAdaptor()->close();
    /* re-position audioDC  and start audioDC thread */
    audio_es = audioDC->getESInfo();
    frameNo = audio_es->getFrameNumber(starttime.timeToSec());
    audioDC->getInput()->setToFrameNumber(frameNo);
    audioDC->play(0, 0.0);
    if (!audioDC->running())
      audioDC->start();
  }
  videoDC->play(0, 0.0);
  if (!videoDC->running())
    videoDC->start();		   // start videoDC thread

  mediaTimeUpdater->start(MEDIA_TIME_UPDATER_INTERVAL_MS);
  displayCurrentMediaTime();	   // remove sub-seconds from displayed media time
  globalTimer->setPaused(false);   // de-block DataChannels
  return true;
}

MP7Time MuvinoMain::videoDuration() const
{
  MP7Time time;
  if (!videoDC)
    return time;
  const ESInfo *es = videoDC->getESInfo();
  if (!es) {
    dprintf_err("MuvinoMain::videoDuration() ESInfo == NULL!\n");
    return time;
  }
  time.setTimePoint(0);
  // set duration in frames/second units
  uint vopIncr = (uint) es->getVOPTimeIncrement();
  time.setDuration((uint) es->getDuration() / vopIncr,
		   (uint) es->getMediaTimeScale() / vopIncr);
  return time;
}

MP7Time MuvinoMain::currentVideoTime() const
{
  MP7Time time;
  if (!videoDC)
    return time;
  const ESInfo *es = videoDC->getESInfo();
  time.setTimePoint((uint) videoDC->getInput()->getCurrentFrameNumber(),
		    (uint) (es->getMediaTimeScale() / es->getVOPTimeIncrement()));
  return time;
}

void MuvinoMain::setCurrentVideoTimeLabel(const MP7Time &time, bool showSubSec)
{
  QString label(time.timePointPretty(true)); // includes 2 sub-second digits
  if (!showSubSec)
    label = label.left(label.length() - 3);
  currentMediaTimeLabel->setText(label);
}

/* value = round(2 + log2(rate)) */
void MuvinoMain::setDisplayRateSlider(double rate)
{
  if (rate < 0.25 || rate > 4.0) {
    dprintf_err("MuvinoMain::setDisplayRateSlider(rate = %5.2f): rate out of range!\n",
		rate);
    return;
  }
  displayRateSlider->setValue((int)(2.5 + (log10(rate)/log10(2.0))));
}

/* ------------------------ Video Player Slot Implementations ---------------------- */

/* slot: "Start" button clicked */
void MuvinoMain::playerStart()
{
  dprintf_full("MuvinoMain::playerStart(): entered\n");
  MP7Time time;
  time.setTimePoint(0);
  playVideo(time);
}

/* slot: Play button was clicked */
void MuvinoMain::playerPlay()
{
  dprintf_full("MuvinoMain::playerPlay(): entered\n");
  playVideo();
}

/* slot: Stop button was clicked */
void MuvinoMain::playerStop()
{
  dprintf_full("MuvinoMain::playerStop(): entered\n");
  if (!videoDC)
    return;
  mediaTimeUpdater->stop();
  pauseVideoAudio();
  /* avoid double pausing */
  if (!globalTimer->isPaused())
    globalTimer->setPaused(true);
  displayCurrentMediaTime(true);
}

/* slot: "-3s" button clicked */
void MuvinoMain::playerBack3s()
{
  dprintf_full("MuvinoMain::playerBack3s(): entered\n");
  MP7Time time = currentVideoTime();
  time.moveTimePoint(-3, 1);	// ignore error status
  playVideo(time);
}

/* slot: update current media time label  and emit currentMediaTimeEvent() signal */
void MuvinoMain::displayCurrentMediaTime(bool showSubSec)
{
  MP7Time currTime = currentVideoTime();
  setCurrentVideoTimeLabel(currTime, showSubSec);
  emit currentMediaTimeEvent(currTime);
}

/* slot: set display frame rate factor
 * meaning of value passed from displayRateSlider:
 * factor = 2 ^ (value - 2)
 */
void MuvinoMain::setDisplayRateFromSlider(int value)
{
  double factor = pow(2.0,(double) (value - 2));
  if (!essync)
    return;
  dprintf_full("MuvinoMain::setDisplayRateFromSlider() value = %d, factor = %4.2f\n",
	       value, factor);
  if (audioDC && factor != 1.0)
    stopAudio();
  essync->setOutputRateFactor(factor);
  audioCheckBox->setDisabled(!audioDC || factor != 1.0);
  // may cause playerAudioStateChange() to be called
  audioCheckBox->setChecked(audioDC && factor == 1.0);
}

/* slot: audioCheckBox changed its state */
void MuvinoMain::playerAudioStateChange(int state)
{
  dprintf_full("MuvinoMain::playerAudioStateChange(): audioCheckBox->isChecked() == %d\n",
	       audioCheckBox->isChecked());
  if (audioCheckBox->isChecked()) {
    playVideo();
  } else {
    stopAudio();
  }
}

/* ------------------- MP7Doc/MP7TreeView functions ---------------------------- */

void MuvinoMain::initMP7TreeView()
{
  mp7DocTreeView->clear();
  for (int i = mp7DocTreeView->columns()-1; i >= 0; i--)
    mp7DocTreeView->removeColumn(i);
  mp7DocTreeView->addColumn("MPEG-7 Element");
  mp7DocTreeView->addColumn("Value");
}

bool MuvinoMain::warnAndSaveMP7Doc()
{
  bool rval = true;
  if (!doc || !doc->isModified())
    return true;
  switch (QMessageBox::warning( this, "Muvino",
				"The current MPEG-7 document has not been saved.\n"
				"Do you want to save it?\n\n",
				"Save",
				"Discard",
				"Cancel",
				0, 2 ) ) {
  case 0:			// Save button was clicked
    fileSave();
    break;
  case 1:			// Discard button was clicked
    // nop
    break;
  case 2:			// Cancel button was clicked
    rval = false;
    break;
  }
  return rval;
}

void MuvinoMain::createMP7Doc(const MP7Time &duration)
{
  if (doc) {
    delete doc;
    doc = NULL;
  }
  resetMP7ElementValue();

  
   


  doc = new MP7Doc(mp7DocTreeView); // creates Mpeg7 descriptor
  connect(doc, SIGNAL(modificationChanged(bool)),
	  this, SLOT(mp7DocModificationChange(bool)));
  setCaptionFromFile();
  /* create an empty VariationSet description */
  MP7ListViewItem *item = doc->nodeByPath("Description", NULL, true);
  MP7ListViewItem *tmpItem = NULL;
  MP7ListViewItem *localRoot = NULL;
  
  
  doc->setAttribute(item, "xsi:type", "VariationDescriptionType");
  item = doc->nodeByPath("VariationSet/Source", item, true);
  doc->setAttribute(item, "xsi:type", "VideoType");
  item = doc->addElement(item, "Video");
  tmpItem = item;
  
  item = doc->addElement(item, "MediaInformation");
  item = doc->addElement(item, "MediaProfile");
  
  /* Insertion of MediaInformation  */
  
  /* Information about the visual stream */
   VideoESInfo* ves = NULL;
   double fps = 0;
   int width = 0;
   int height = 0;
   bool color = true;
   int bitspersecond = 0;
   double quality = 0;
   int mediaSize = 0;
   int frameCount = 0;
   CodecID codec;
   const char* videoCodecName;
   const ESInfo *es = NULL;
   int streamID = 0;
   char string[1024];

   if(videoDC){
	es = videoDC->getESInfo();
	if(es->isVisualStream()){
		
		ves = (VideoESInfo *) es;
		streamID = ves->getStreamId();	
		fps = ves->getFPS();
		width = ves->getWidth();
		height = ves->getHeight();
		color = ves->getHasColor();
		bitspersecond = ves->getAvgBandwidth();
		quality = ves->getStreamQuality();
		mediaSize = ves->getOrigMediaSize();
		frameCount = ves->getNumberOfMediaSamples();
		codec = ves->getCodecID();
		videoCodecName = MP4Decoder::codecToString(codec);
	
		printf("VideoInfo->FPS: %f, width: %i, height: %i, color: %i, Bitrate: %i, quality: %f, Size: %i, Frames: %i, Codec: %s\n",fps,width,height, color, bitspersecond, quality, mediaSize, frameCount, videoCodecName);	
						
		item = doc->addElement(item, "ComponentMediaProfile");
		sprintf(string,"id_%d",streamID);
		//strcpy(string, "myID");
		doc->setAttribute(item, "id", string);
		localRoot = item;
		item = doc->addElement(item, "MediaFormat");
		item = doc->addElement(item, "Content");
		doc->setAttribute(item, "href", "MPEG7ContentCS");
		item = doc->addElement(item, "Name");
		doc->setElementValue(item, "visual");
		item = doc->nodeByPath("MediaFormat", localRoot, false);
		//item = doc->nodeByPath("Description", NULL, false);
		//item = doc->nodeByPath("VariationSet/Source/Video/MediaInformation/MediaProfile/ComponentMediaProfile/MediaFormat", item, false);
		//item = doc->nearestElement(item, "MediaFormat", true);
		item = doc->addElement(item, "VisualCoding");
		item = doc->addElement(item, "Format");
		doc->setAttribute(item, "href", "urn:mpeg:mpeg7:cs:VisualCodingFormatCS:2001:3");
		item = doc->addElement(item, "Name");
		doc->setAttribute(item, "xml:lang", "en");
		if(strcmp(videoCodecName, "MPEG4")== 0)
			doc->setElementValue(item, "MPEG-4 Visual");
		else
			doc->setElementValue(item, videoCodecName);
			
		item = doc->nodeByPath("MediaFormat/VisualCoding", localRoot, false);
		item = doc->addElement(item, "Frame");
										
		sprintf(string,"%d",width);
		doc->setAttribute(item, "width", string);
		
		sprintf(string,"%d",height);
		doc->setAttribute(item, "height", string);
		
		sprintf(string,"%.2f",fps);
		doc->setAttribute(item, "rate", string);
	
	}
   }
   else
   	printf("VideoInfo->No Video!Is not recommended!\n");
  
  /* Information about the audio stream */
  AudioESInfo* aes = NULL;
  int bitrate = 0;
  const char* audioCodecName;
  
  if(audioDC){
    es = audioDC->getESInfo();
    if(es->isAudioStream()){
	aes = (AudioESInfo *) es;
	streamID = aes->getStreamId();	
	bitrate = aes->getAvgBandwidth();
	codec = aes->getCodecID();
	audioCodecName = MP4Decoder::codecToString(codec);

	printf("AudioInfo->Bitrate: %i, Codec: %s\n", bitrate, audioCodecName);
	
	item = doc->nodeByPath("Description", NULL, false);
	item = doc->nodeByPath("VariationSet/Source/Video/MediaInformation/MediaProfile", item, false);
	item = doc->addElement(item, "ComponentMediaProfile");
	sprintf(string,"id_%d",streamID);
	//strcpy(string, "myID");
	doc->setAttribute(item, "id", string);
	localRoot = item;
	item = doc->addElement(item, "MediaFormat");
	item = doc->addElement(item, "Content");
	doc->setAttribute(item, "href", "MPEG7ContentCS");
	item = doc->addElement(item, "Name");
	doc->setElementValue(item, "audio");
	
	item = doc->nodeByPath("MediaFormat", localRoot, false);
	item = doc->addElement(item, "AudioCoding");
	item = doc->addElement(item, "Format");
	doc->setAttribute(item, "href", "urn:mpeg:mpeg7:cs:VisualCodingFormatCS:2001:1");
	item = doc->addElement(item, "Name");
	doc->setAttribute(item, "xml:lang", "en");
	
	if(strcmp(audioCodecName, "MPEG4AAC")== 0)
			doc->setElementValue(item, "MPEG-1 Audio Layer III");
		else
			doc->setElementValue(item, audioCodecName);
		
    }
  }
  else
	printf("AudioInfo->No Audio!\n");
  
  
  item = doc->nodeByPath("Description", NULL, false);
  item = doc->nodeByPath("VariationSet/Source/Video/MediaInformation/MediaProfile", item, false);
  item = doc->addElement(item, "MediaFormat");
  localRoot = item;
  item = doc->addElement(item, "Content");
  doc->setAttribute(item, "href", "MPEG7ContentCS");
  item = doc->addElement(item, "Name");
  doc->setElementValue(item, "audiovisual");
  
  item = doc->addElement(localRoot, "FileFormat");
  doc->setAttribute(item, "href", "urn:mpeg:mpeg7:cs:FileFormatCS:2001:5");
  item = doc->addElement(item, "Name");
  doc->setAttribute(item, "xml:lang", "en");
  doc->setElementValue(item, "mp4");
  
  item = doc->addElement(localRoot, "FileSize");
  sprintf(string, "%d", mediaSize);
  doc->setElementValue(item, string);
  
  item = doc->addElement(localRoot, "BitRate");
  sprintf(string, "%d", bitspersecond);
  doc->setElementValue(item, string);
  
  //item = tmpItem;
  item = doc->nodeByPath("Description", NULL, false);
  item = doc->nodeByPath("VariationSet/Source/Video", item, false);
  tmpItem = NULL; 
  MP7VideoSegment segment(doc, item);
  if (!duration.durationIsNull())
    segment.setMediaTime(duration);
  resetSegmentNav();
  
  item = doc->nodeByPath("Description", NULL, false);
  /* minimal requirements for variation, mandatory*/
  item = doc->nodeByPath("VariationSet/Variation/Content",item,true);
  doc->setAttribute(item, "xsi:type", "AudioType");
  item = doc->addElement(item, "Audio");
  
  item = doc->nodeByPath("Description", NULL, false);
  //item = doc->nodeByPath("VariationSet/Variation/VariationRelationship",item,true);
  item = doc->nodeByPath("VariationSet/Variation",item,true);
  item = doc->addElement(item, "VariationRelationship");
  doc->setElementValue(item, "extraction");
  item = doc->nodeByPath("Description", NULL, false);
  item = doc->nodeByPath("VariationSet/Variation",item,true);
  item = doc->addElement(item, "VariationRelationship");
  doc->setElementValue(item, "modalityTranslation");
  
    
  
}

bool MuvinoMain::loadMP7Doc(const QString &filename)
{
  dprintf_full("MuvinoMain::loadMP7Doc(filename = %s)\n",
	       filename.latin1());
  bool rval = false;
  if (doc) {
    delete doc;			// clears also mp7DocTreeView
    doc = NULL;
  }
  resetMP7ElementValue();
  doc = new MP7Doc(mp7DocTreeView);
  if (!doc) {
    dprintf_err("MuvinoMain::loadMP7Doc() MP7Doc constructor FAILED!\n");
    return false;
  }
  connect(doc, SIGNAL(modificationChanged(bool)),
	  this, SLOT(mp7DocModificationChange(bool)));
  rval = doc->parse(filename);
  if (rval)
    setCaptionFromFile(filename, false);
  resetSegmentNav();
  return rval;
}

int MuvinoMain::checkMP7DocWithVideo()
{
  if (!videoDC)
    return true;
  MP7Time videoDur = videoDuration();
  if (!doc) {
    createMP7Doc(videoDur);
    return 1;
  }
  MP7VideoSegment topSegment = totalSlider->getSegment();
  MP7Time segmentDur = topSegment.mediaTime();
  int diffms = videoDur.durationToMSec() - segmentDur.durationToMSec();
  if (diffms < 0)
    diffms = -diffms;
  if (diffms > MP7DOC_VIDEO_DURATION_TOLERANCE_MS) {
    switch (QMessageBox::warning(this, "MPEG-7 Document and Video Incompatible",
				 QString("The MPEG-7 document and the video differ "
					 "wrt. media time: %1 vs. %2!\n"
					 "Do you want to open another MPEG-7 document, "
					 "or create a new one?")
				 .arg(segmentDur.durationPretty())
				 .arg(videoDur.durationPretty()),
				 "&Open", "&New", "&Cancel",
				 0, 2)) {
    case 0:
      return 2;
    case 1:
      createMP7Doc(videoDur);
      return 1;
    case 2:
      return 3;
    }
  }
  return 0;
}

/* slot: mp7TreeView item was selected */
void MuvinoMain::treeViewSelectItem(QListViewItem *item)
{
  if (item) {
    QString value(item->text(1)); // column 1 contains the element values
    mp7ElementValueEdit->setText(value);
    disableMP7ElementValue(value.isNull());
    currentMP7Item = (MP7ListViewItem *) item;
  }
}

/* slot: modification status of MPEG-7 document has changed */
void MuvinoMain::mp7DocModificationChange(bool m)
{
  if (doc)
    setCaptionFromFile(doc->filePath(), m);
}

/* slot: set selected tree view item */
void MuvinoMain::treeViewSetSelected(MP7ListViewItem *item)
{
  QListViewItem *i = (QListViewItem *) item;
  mp7DocTreeView->setSelected(i, true);
  mp7DocTreeView->setCurrentItem(i);
  mp7DocTreeView->ensureItemVisible(i);
}

void MuvinoMain::treeViewClicked(QListViewItem *item)
{
  mp7DocTreeView->setSelected(item, true);
  mp7DocTreeView->setCurrentItem(item);
  /* make currentSlider show the segment containing <item> */
  MP7VideoSegment segment;
  segment.setMP7Doc(doc);
  if (currentSlider && segment.setRoot((MP7ListViewItem *) item, true)) {
    currentSlider->setSegment(segment, false); // don't emit a segmentChanged signal
  } else {
    dprintf_err("MuvinoMain::treeViewClicked() FAILED - currentSlider = %p, "
		"segment: %s (%s)\n", currentSlider, segment.title().latin1(),
		segment.mediaTime().toString().latin1());
  }
}

/* ------------------------- MP7ElementValue functions ----------------------------- */

/* slot: the text of mp7ElementValueEdit has changed */
void MuvinoMain::mp7ElementValueChange()
{
  if (!mp7ElementValueEdit->hasFocus())
    return;
  QString value(mp7ElementValueEdit->text());
  if (!value.isEmpty() && currentMP7Item) {
    if (doc->setElementValue(currentMP7Item, value) < 0) {
      dprintf_err("MuvinoMain::mp7ElementValueChange() ERROR: "
		  "doc->setElementValue(%p, %s) FAILED!\n",
		  currentMP7Item, value.latin1());
    }
  }
}

void MuvinoMain::resetMP7ElementValue()
{
  mp7ElementValueEdit->setTextFormat(Qt::PlainText);
  mp7ElementValueEdit->clear();
  disableMP7ElementValue(true);
}

void MuvinoMain::disableMP7ElementValue(bool disable)
{
  mp7ElementValueEdit->setDisabled(disable);
}

/* ------------------ Segment Navigation Methods --------------------------- */

void MuvinoMain::initSegmentNav()
{
  int pos;
  /* replace segmentNavCurrentSlider by a SegmentSlider widget */
  pos = segmentNavFrameLayout->findWidget(segmentNavCurrentSlider);
  if (pos >= 0) {
    currentSlider = new SegmentSlider(segmentNavFrame,
				      "segmentNavCurrentSlider");
    currentSlider->setTitleLabel(segmentNavCurrentTitle);
    currentSlider->setStartTimeLabel(segmentNavCurrentTimeStart);
    currentSlider->setEndTimeLabel(segmentNavCurrentTimeEnd);
    currentSlider->setLevelSpinBox(segmentNavLevelSpinbox);
    currentSlider->setPrevSiblingButton((QButton *) segmentNavPrevSiblingButton);
    currentSlider->setNextSiblingButton((QButton *) segmentNavNextSiblingButton);
    connect(this, SIGNAL(currentMediaTimeEvent(const MP7Time &)),
	    currentSlider, SLOT(setSliderFromMediaTime(const MP7Time &)));
    connect(segmentNavSplitButton, SIGNAL(clicked()), this, SLOT(splitCurrentSegment()));
    connect(segmentNavDeleteButton, SIGNAL(clicked()), currentSlider, SLOT(removeSegment()));
    connect(segmentNavEditButton, SIGNAL(clicked()), currentSlider, SLOT(editSegment()));
    connect(creationButton, SIGNAL(clicked()), currentSlider, SLOT(editCreation()));
    connect(urisButton, SIGNAL(clicked()), currentSlider, SLOT(editUri()));
    connect(variationButton, SIGNAL(clicked()), currentSlider, SLOT(editVariation()));
    connect(currentSlider, SIGNAL(segmentChanged(MP7ListViewItem *)),
	    this, SLOT(treeViewSetSelected(MP7ListViewItem *)));
    segmentNavFrameLayout->remove(segmentNavCurrentSlider);
    segmentNavFrameLayout->insertWidget(pos, currentSlider);
    delete segmentNavCurrentSlider;
    segmentNavCurrentSlider = NULL;
  }
  /* replace segmentNavTotalSlider by a SegmentSlider widget */
  pos = segmentNavFrameLayout->findWidget(segmentNavTotalSlider);
  if (pos >= 0) {
    totalSlider = new SegmentSlider(segmentNavFrame,
				    "segmentNavTotalSlider");
    totalSlider->enableSegmentZooming(false);
    totalSlider->setTitleLabel(segmentNavTotalTitle);
    totalSlider->setStartTimeLabel(segmentNavTotalTimeStart);
    totalSlider->setEndTimeLabel(segmentNavTotalTimeEnd);
    connect(this, SIGNAL(currentMediaTimeEvent(const MP7Time &)),
	    totalSlider, SLOT(setSliderFromMediaTime(const MP7Time &)));
    // update totalSlider if eg. the title of the top-level segment is modified
    connect(currentSlider, SIGNAL(segmentChanged(MP7ListViewItem *)),
	    totalSlider, SLOT(updateSegmentDisplay()));
    segmentNavFrameLayout->remove(segmentNavTotalSlider);
    segmentNavFrameLayout->insertWidget(pos, totalSlider);
    delete segmentNavTotalSlider;
    segmentNavTotalSlider = NULL;
  }
  resetSegmentNav();
}

void MuvinoMain::resetSegmentNav()
{
  dprintf_full("MuvinoMain::resetSegmentNav() entered\n");
  MP7Time time;			// initializes to 0
  MP7ListViewItem *root = (MP7ListViewItem *) mp7DocTreeView->firstChild();
  /* reset MP7VideoSegment objects */
  MP7VideoSegment currentSeg;
  if (doc && root) {
    currentSeg.setMP7Doc(doc);
    currentSeg.setRoot(root, false); // search top-down for segment root
  }
  /* reset SegmentSlider objects */
  /* since the segmentChanged() signal of currentSlider
   * is connected to the updateSegmentDisplay() slot of
   * totalSlider, the latter _must_ be reset first.
   */
  totalSlider->setValue(totalSlider->minValue());
  totalSlider->setSegment(currentSeg);
  currentSlider->setValue(currentSlider->minValue());
  currentSlider->setSegment(currentSeg);
}

void MuvinoMain::splitCurrentSegment()
{
  MP7VideoSegment currentSeg(currentSlider->getSegment());
  if (!currentSeg.isValid())
    return;
  MP7Time time(currentVideoTime());
  if (!time.timePointIsValid())
    return;
  MP7VideoSegment splitSeg;
  if (!currentSeg.searchByMediaTime(splitSeg, time, currentSeg.level()+1)) {
    dprintf_err("MuvinoMain::splitCurrentSegment() segment to be splitted not found "
		"(time = %s)\n", time.timePointPretty().latin1());
    return;
  }
  /* If there is no sub-segment, create one and split the sub-segment. */
  if (splitSeg.rootItem() == currentSeg.rootItem()) {
    if (!currentSeg.addChildSegment(splitSeg)) {
      dprintf_err("MuvinoMain::splitCurrentSegment() adding child segment FAILED "
		"(time = %s)\n", time.timePointPretty().latin1());
      return;
    }
    splitSeg.setMediaTime(currentSeg.mediaTime());
  }
  MP7VideoSegment siblingSeg;
  if (!splitSeg.split(siblingSeg, time)) {
    dprintf_err("MuvinoMain::splitCurrentSegment() splitting segment FAILED "
		"(time = %s)\n", time.timePointPretty().latin1());
    return;
  }
  currentSlider->updateSegmentDisplay();
}

void MuvinoMain::setTitleImage()
{
  MP7VideoSegment segment = currentSlider->getSegment();
  MP7Time time = currentVideoTime();
  if (segment.isValid() && time.timePointIsValid()) {
    MP7ListViewItem *item = segment.setTitleImage(time);
    if (!item) {
      dprintf_err("MuvinoMain::setTitleImage() segment.setTitleImage(time = %s) FAILED!\n",
		  time.toString().latin1());
      return;
    }
    treeViewSetSelected(item);
    // make sure that mp7ElementValue is updated even
    // if the current item of mp7TreeView does not change
    treeViewSelectItem(item);
  }
}

/* ------------------------- Menu Bar Functions ---------------------------------------- */

void MuvinoMain::initMenuBar()
{
  fileSaveAction->setEnabled(false);
  fileSaveAsAction->setEnabled(false);
}

void MuvinoMain::fileNew()
{
  if (!warnAndSaveMP7Doc())
    return;
  MP7Time duration = videoDuration();
  createMP7Doc(duration);
}

void MuvinoMain::fileOpen()
{
  fileOpen(QString::null);
}

void MuvinoMain::fileOpen(const QString &path)
{
  if (!warnAndSaveMP7Doc())
    return;
  bool doRepeat;
  do {
    doRepeat = false;
    QString s(path);
    if (s.isEmpty())
      s = QFileDialog::getOpenFileName(".", "MPEG-7 documents (*.mp7 *.xml)",
				       this, "load MPEG-7 file dialog",
				       "Load MPEG-7 file");
    if (!s.isEmpty()) {
      loadMP7Doc(s);
      switch (checkMP7DocWithVideo()) {
      case 2:
	doRepeat = true;
	break;
      case 3:
	return;
      }
    }
  } while (doRepeat);
  fileSaveAction->setEnabled(true);
  fileSaveAsAction->setEnabled(true);
}

void MuvinoMain::fileLoadVideo()
{
  fileLoadVideo(QString::null);
}

void MuvinoMain::fileLoadVideo(const QString &path)
{
  QString s(path);
  if (s.isEmpty())
    s = QFileDialog::getOpenFileName(".", "Videos (*.mp4 *.avi)",
				     this, "load video file dialog",
				     "Load video file");
  if (!s.isEmpty()) {
    loadVideo(s.latin1(), ".", UncompressedVideoFrame::ColorSpaceRGB24);
    switch (checkMP7DocWithVideo()) {
    case 2:
      fileOpen();
      break;
    case 3:
      return;
    }
    playerPlay();
  }
}

void MuvinoMain::fileExit()
{
  if (warnAndSaveMP7Doc()) {
    closeVideo();
    qApp->quit();
  }
}

void MuvinoMain::fileSave()
{
  if (!doc)
    return;
  QString file(doc->filePath());
  if (file.isEmpty())
    fileSaveAs();
  else {
    doc->save(file);
  }
}

void MuvinoMain::fileSaveAs()
{
  if (!doc)
    return;
  QString s = QFileDialog::getSaveFileName(".", "MPEG-7 documents (*.mp7 *.xml)",
					   this, "save MPEG-7 file dialog",
					   "Save MPEG-7 file");
  if (!s.isEmpty()) {
    doc->save(s);
  }
}
