 /***********************************************************************
 *                                                                      *
 * MuViNo - ViTooKi Video Annotation Tool                               *
 *                                                                      *
 * title:	videosegment.hpp                                        *
 *                                                                      *
 * $Id: videosegment.hpp,v 1.3 2004/09/24 13:15:08 mtaschwer Exp $
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

#ifndef _VIDEOSEGMENT_HPP_
#define _VIDEOSEGMENT_HPP_

/** @file
 * Declaration of the VideoSegment class.
 */

#include "videosegmentdlg.hpp"

/** This class implements the (simple) application logic of VideoSegmentDlg,
 * the GUI dialog for editing the MPEG-7 descriptors of a video segment.
 * @author Mario Taschwer
 * @version $Id: videosegment.hpp,v 1.3 2004/09/24 13:15:08 mtaschwer Exp $
 */
class VideoSegment : public VideoSegmentDlg
{
public:
  /** Constructor. */
  VideoSegment(QWidget *parent = 0, const char *name = 0,
	       bool modal = false, WFlags f = 0);

  /** Return <code>true</code> if the current value of <code>semanticRegionComboBox</code>
   * denotes a valid country.
   */
  bool semanticRegionIsValid() const;

  /** Return the 2-letter ISO country code of the current 
   * <code>semanticRegionComboBox</code> entry.
   */
  QString semanticRegion() const;

  /** Set the current item of <code>semanticRegionComboBox</code>
   * according to the given 2-letter ISO country code.
   * @return <code>false</code> if the given country code could not be found.
   */
  bool setSemanticRegion(const QString &countryCode);

  /** Return the MPEG-7 representation of the current
   * <code>semanticDurationComboBox</code> entry.
   */
  QString semanticDurationUnit() const;

  /** Set the current item of <code>semanticDurationComboBox</code>
   * according to the given MPEG-7 duration value.
   * @return <code>false</code> if the given duration value could not be found.
   */
  bool setSemanticDurationUnit(const QString &mp7Duration);
};

#endif /* _VIDEOSEGMENT_HPP_ */
