/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	LocalPresentation.cpp                                         *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include <qobject.h>
#include "LocalPresentation.hpp"
#include "DataChannel.hpp"
#include "DataSink.hpp"
#include "ContainerInfo.hpp"
#include "ESInfo.hpp"
#include "adaptors/MP4Decoder.hpp"
#include "adaptors/MP4audioDecoder.hpp"
#include "VideoAdaptorChain.hpp"
#include "AudioAdaptorChain.hpp"
#include "Adaptor.hpp"
#include "VideoDisplayIO.hpp"
#include "metadata/TerminalCapabilities.hpp"
#include "adaptors/ColorReductionAdaptor.hpp"
#include "adaptors/TemporalAdaptor.hpp"
#include "adaptors/StrongTemporalAdaptor.hpp"
#include "adaptors/QualityReductionAdaptor.hpp"
#include "adaptors/SpatialReductionAdaptor.hpp"
#include "ReferenceCounter.hpp"
#include "adaptors/SharedAdaptor.hpp"
#include "adaptors/GlobalTimer.hpp"
#include "adaptors/ESSynchronizer.hpp"
#include "adaptors/PSNR.hpp"
#include "adaptors/DataDump.hpp"
#include "io/MPGStreamIO.hpp"
#include "io/ffMP4IO.hpp"
#include "io/IOCreator.hpp"
#include "io/Rtp.hpp"
#include "Statistics.hpp"
#include "adaptors/FrameCounter.hpp"
#include "adaptors/FrameTypeCounter.hpp"
#include "adaptors/StatisticsUpdater.hpp"
#include "container/ContainerFile.hpp"

LocalPresentation::LocalPresentation(u32 x_inSDL,u32 y_inSDL,  ReferenceCounter * gtRef, GlobalTimer *gt) : Presentation(x_inSDL,y_inSDL, gtRef, gt) {

  localPres = true;
  firstTime = true;
  offlineChecked = false;
  presInform.localPresentation = true;
  needToMove = true;
  this->x_inSDL = x_inSDL;
  this->y_inSDL = y_inSDL;
  state = CLOSED;
  lastX = 0;
  lastY = 0;
  mp4Stream = NULL;
  pWidth = 0;
  pHeight = 0;
  origURI = NULL;
  duration = 0;
  //syncForLongestES = NULL;
  presState = PRES_INITIALIZED;
	gtRefCount = gtRef;
  globalTimer = gt;
}

LocalPresentation::~LocalPresentation() {
	cout << "LocalPresentation::~LocalPresentation() called" << endl;
}

int LocalPresentation::initalizeURI(const char * URI) {
  pWidth = 0;
  pHeight = 0;
  longestESSynchronizerId = -1;
	
  assert(globalTimer);
    //globalTimer = new GlobalTimer();
  if(state == CLOSED) {
    uri = URI;
    sessionKey = TEARDOWN_ALL;
	
    if(!mp4Stream)
      mp4Stream = ContainerFile::loadContainerInfo(uri);
    //#else	

    /*	 if (mp4Stream)  {
				 delete mp4Stream;
				 mp4Stream = NULL;
		 
				 }
				 mp4Stream = ContainerFile::loadMP4Info(uri);
    */
    //#endif
	 
    if (!mp4Stream) {
	
      dprintf_full("LocalPresentation::initializeURI -> Failed to load MP4 File\n");
      return -1;
    }
    else {
		
      dprintf_full("LocalPresentation::initializeURI -> MP4 File loaded...\n");
      if(firstTime) {
				origURI = URI;
#ifndef ISOMP4
				//			ContainerFile::demultiplexAndUpdateIO(mp4Stream);
#endif
				firstTime = false;
      }
      state = INITIALIZED;
    }
    presInform.URI = (char *) uri;
    return 0;
	 
	 
  }
  return -1;
}

void LocalPresentation::setupURI() {
  int j = 0;
  int count = 0;
  int nrAudioES = 0;
  int nrVideoES = 0;
	bool hasAdaptedInput=false;
	list < ESInfo * >::iterator eI;
  duration = 0;
  if(state == INITIALIZED) {
    list <ESInfo *> *esInf = mp4Stream->clone()->getESList();
    ESSynchronizer *ess = NULL;
    /* setup video stream first */
		
    for(eI = esInf->begin(); eI != esInf->end(); eI++) {
      if((*eI)->isVisualStream()) {
				dprintf_full("Found VideoStream with id = %lld\n",(*eI)->getStreamId());
				nrVideoES++;
				presM = new presentationManagement();
				presM->adaptorChain = new VideoAdaptorChain();
				origESInfo.push_back((*eI)->clone((*eI)->getContainerInfo()));
				actESInfo.push_back((*eI)->clone((*eI)->getContainerInfo()));
				presM->esInfo = (*eI)->clone((*eI)->getContainerInfo());
				presM->stats = new Statistics(presM->esInfo, false); //input stats
				if (!ess)
					ess = presM->esSync = new ESSynchronizer(*eI,globalTimer);
				duration = MAX((unsigned)duration,(*eI)->getDurationInMs());
				
				unsigned int adaptorWidth = 0;
				unsigned int adaptorHeight = 0;
				
				SharedAdaptor * sharedAdapt = new SharedAdaptor(gtRefCount, (Adaptor*&)globalTimer);
				presM->adaptorChain->addAdaptor(sharedAdapt);
				presM->adaptorChain->addAdaptor(presM->esSync);
				presM->adaptorChain->addAdaptor(new FrameTypeCounter(*eI,presM->stats));
				
				if(!offlineChecked) {	
					dprintf_full("LocalPresentation::setupURI -> before iteration\n");
					presInform.usedAdaptors.clear();
					for(list <QString>::iterator aI = adaptorsL.begin(); aI != adaptorsL.end(); aI++) {
						presInform.usedAdaptors.push_back(*aI);
						hasAdaptedInput=true;
						dprintf_full("LocalPresentation::setupURI -> inside iteration\n");
						if ((*aI).compare("Color Reduction") == 0) {
							dprintf_full("LocalPresentation::setupURI -> ColorReduction\n");
							if(!presM->decoderSet) {
								presM->adaptorChain->addAdaptor( new MP4Decoder((VideoESInfo*)(*eI),false));
								presM->decoderSet = true;
							}
							presM->adaptorChain->addAdaptor(new YUVColorReductionAdaptor((VideoESInfo*)(*eI)));
						}
						if ((*aI).compare("Temporal Adaptation") == 0) {
							dprintf_full("LocalPresentation::setupURI -> TemporalAdaptation\n");
							presM->adaptorChain->addAdaptor(new TemporalAdaptor((VideoESInfo*)(*eI)));
						}
						if ((*aI).compare("Strong Temporal Adaptation") == 0) {
							dprintf_full("LocalPresentation::setupURI -> Strong Temporal Adaptation\n");
							presM->adaptorChain->addAdaptor(new StrongTemporalAdaptor((VideoESInfo*)(*eI)));
						}
						if ((*aI).compare("Quality Reduction") == 0) {
							dprintf_full("LocalPresentation::setupURI -> Quality Reduction\n");
							if(!presM->decoderSet) {
								presM->adaptorChain->addAdaptor(new MP4Decoder((VideoESInfo*)(*eI),false));
								
							}
							presM->adaptorChain->addAdaptor(new MP4Encoder((VideoESInfo*)(*eI),(int)(((VideoESInfo *) (*eI))->getAvgBandwidth()*0.5), 30));
							
							presM->decoderSet = false;
							actESInfo.back()->setAvgBandwidth((int)(((VideoESInfo *) (*eI))->getAvgBandwidth()*0.5));
						}
						if ((*aI).compare("Spatial Adaptation") == 0) {
							dprintf_full("LocalPresentation::setupURI -> Spatial Adaptation\n");
							aI++;
							adaptorWidth = (*aI).toInt();
							aI++;
							adaptorHeight = (*aI).toInt();
							dprintf_full("LocalPresentation::setupURI -> ADAPTORWIDTH = %i\n",adaptorWidth);
							dprintf_full("LocalPresentation::setupURI -> ADAPTORWIDTH = %i\n",adaptorHeight);
							if((adaptorWidth < ((VideoESInfo*)(*eI))->getWidth()) && 
								 (adaptorHeight < ((VideoESInfo*)(*eI))->getHeight())) {
								if(!presM->decoderSet) {
									presM->adaptorChain->addAdaptor(new MP4Decoder((VideoESInfo*)(*eI),false));
									presM->decoderSet = true;
								}
								presM->adaptorChain->addAdaptor(new YUVSpatialReductionAdaptor((VideoESInfo*)(*eI),&adaptorWidth, &adaptorHeight));
								//	((VideoESInfo*)(*eI))->setWidth(adaptorWidth);
								//	((VideoESInfo*)(*eI))->setHeight(adaptorHeight);
							}
							
						}
					}
				} //!offlineChecked
				else {
					presInform.usedAdaptors.clear();
					for(list <QString>::iterator aI = adaptorsL.begin(); aI != adaptorsL.end(); aI++) {
						presInform.usedAdaptors.push_back(*aI);
						if ((*aI).compare("Spatial Adaptation") == 0) {
							aI++;
							aI++;
						}
					}
				}
				if (adaptorWidth > 0) {
					if( ((VideoESInfo*)(*eI))->getWidth() > adaptorWidth)
						((VideoESInfo*)(*eI))->setWidth(adaptorWidth);
					if(((VideoESInfo*)(*eI))->getHeight() > adaptorHeight)
						((VideoESInfo*)(*eI))->setHeight(adaptorHeight);
					count = x_inSDL+pWidth;
					((VideoESInfo*)actESInfo.back())->setWidth(adaptorWidth);
					((VideoESInfo*)actESInfo.back())->setHeight(adaptorHeight);
					presM->videoOutput = new VideoDisplayIO(NULL,PG_Rect(x_inSDL+lastX,y_inSDL,adaptorWidth+4,adaptorHeight+4)
																									,false);
					lastX = adaptorWidth+4;
					pWidth += lastX;
					lastY = adaptorHeight+4;
					pHeight = MAX(pHeight,(u32) lastY);
				}
				else {
					int realWidth = 0;
					int realHeight = 0;
					
					realWidth =  ((VideoESInfo*)(*eI))->getWidth();
					realHeight = (int)(floor((float)realWidth / ((VideoESInfo*)(*eI))->getAspectRatio()));
					//adjust to even boundaries (241-->242, 179-->180)
					realHeight = realHeight + ((realHeight % 2));
					
					count = x_inSDL+pWidth;
					//	presM->videoOutput = new VideoDisplayIO(NULL,
					//											PG_Rect(x_inSDL+lastX,y_inSDL,((VideoESInfo*)(*eI))->getWidth()+4
					//											,((VideoESInfo*)(*eI))->getHeight()+4),false);
					presM->videoOutput = new VideoDisplayIO(NULL,	PG_Rect(x_inSDL+lastX,y_inSDL,realWidth+4,realHeight+4),false);
					//	lastX = ((VideoESInfo*)(*eI))->getWidth()+4;
					lastX = realWidth+4;
					this->pWidth += lastX;
					//	lastY = ((VideoESInfo*)(*eI))->getHeight()+4;
					lastY = realHeight+4;
					this->pHeight = MAX(pHeight,(u32) lastY);
					((VideoESInfo*)actESInfo.back())->setWidth(realWidth);
					((VideoESInfo*)actESInfo.back())->setHeight(realHeight);
				}
				presM->videoOutput->setESInfo((*eI));
				presM->videoOutput->open();
				/*	if(presM->decoderSet) {
					presM->adaptorChain->addAdaptor(new MP4Encoder((*eI),(*eI)->getAvgBandwidth()*8));
					presM->decoderSet = false;
					}
				*/
				addDecoderAdaptor(presM->videoOutput, *eI, presM->stats, j, hasAdaptedInput);
				
				presManage.push_back(presM);
				j++;
				
				//add StatsUpdater after local adaptation
				presM->adaptorChain->addAdaptor(new StatisticsUpdater(*eI,presM->stats));
      } // isVisualStream
    } // for
    /* setup audio stream after video stream */
    for(eI = esInf->begin(); eI != esInf->end(); eI++) {
      if((*eI)->isAudioStream()) {
				dprintf_full("Found AudioStream with id = %lld\n",(*eI)->getStreamId());
				nrAudioES++;
				presM = new presentationManagement();
				presM->adaptorChain = new AudioAdaptorChain();
				origESInfo.push_back((*eI)->clone((*eI)->getContainerInfo()));
				actESInfo.push_back((*eI)->clone((*eI)->getContainerInfo()));
				presM->esInfo = (*eI)->clone((*eI)->getContainerInfo());
				presM->stats = new Statistics(presM->esInfo, false); //input stats
				duration = MAX((unsigned)duration,(*eI)->getDurationInMs());
				// pass video ESSync to SDLaudioIO (may be NULL)
				SDLaudioIO * audioOut = new SDLaudioIO((AudioESInfo*)(*eI), ess);
				audioOut->open();
				addDecoderAdaptor(audioOut, *eI, presM->stats, j, false);
				presM->audioOutput = audioOut;
				presManage.push_back(presM);
				j++;
				presM->adaptorChain->addAdaptor(new StatisticsUpdater(*eI,presM->stats));
      }
    }
		
    presInform.actESInfo = actESInfo;
    presInform.origESInfo = origESInfo;
    presInform.numberOfAES = nrAudioES;
    presInform.numberOfVES = nrVideoES;
		
    /* Test OUPUT for presInform */
		
		/*  cout << "PRESENTATION INFORMATION\n\n" << endl;
				cout << "LOCAL/REMOTE: ";
				if (presInform.localPresentation) {
				cout << "local" << endl;
				}
				else {
				cout << "remote" << endl;
				}
				cout << "URI = " << presInform.URI << endl;
				cout << "Used Adaptors: " << endl;
				int k = 1;
				cout << "Nr. VES = " << presInform.numberOfVES << endl;
				cout << "Nr. AES = " << presInform.numberOfAES << endl;
				for (list <ESInfo *>::iterator oESI=origESInfo.begin(); oESI != origESInfo.end(); oESI++) {
				cout << " Original Video ES " << k << endl;
				cout << "--------------" << endl;
				cout << "Width = " << (*oESI)->getWidth() << endl;
				cout << "Height = " << (*oESI)->getHeight() << endl;
				int dur = ((*oESI)->getDurationInMs()/1000);
				cout << "Duration = " <<  dur << endl;
				cout << "Bandwidth = " << (*oESI)->getAvgBandwidth() << endl;
				cout << "------------------------------------------------ " << endl;
				k++;
				}
				k = 1;
				for (list <ESInfo *>::iterator aESI=actESInfo.begin(); aESI != actESInfo.end(); aESI++) {
				cout << " Actual Video ES " << k << endl;
				cout << "--------------" << endl;
				cout << "Width = " << (*aESI)->getWidth() << endl;
				cout << "Height = " << (*aESI)->getHeight() << endl;
				int dur = ((*aESI)->getDurationInMs()/1000);
				cout << "Duration = " <<  dur << endl;
				cout << "Bandwidth = " << (*aESI)->getAvgBandwidth() << endl;
				cout << "------------------------------------------------ " << endl;
				k++;
				}

		*/

    presState = PRES_READYTOPLAY;
    state = READYTOPLAY;
  }		
	

}

void LocalPresentation::addDecoderAdaptor(IO * output, ESInfo * esInfo, 
																					Statistics *stats, int id, bool hasAdaptedInput) {
  char localhost[10]="127.0.0.1";
  char yuvOrig[MAX_STR_LEN];
  char yuvDumpOut[MAX_STR_LEN];
  const char* path;
  
  QString * fullPath = new QString(this->uri);
  QString * fileName1 = new QString(fullPath->left(fullPath->length()-4));
 // int position = fileName1.findRev('/');
  //QString fileName2(fileName1.right(fileName1.length()-position-1));

  if (esInfo->isVisualStream()) {
	AdaptorChain *ach = new VideoAdaptorChain();
//	path=esInfo->getContainerInfo()->getUrl()->getFileName();
//	strcpy((char *) fileName1.latin1(),path);
	path = fileName1->latin1();
	cout << path << endl;
 //	sprintf(yuvOrig,"localfile_%s.video.%i.yuv",(path?path:"NULL"),	esInfo->getStreamId());
//	sprintf(yuvDumpOut,"localfile_%s.video.%i.dump.yuv",(path?path:"NULL"),	esInfo->getStreamId());
	sprintf(yuvOrig,"%s.yuv",(path?path:"NULL"));
	sprintf(yuvDumpOut,"%s.dump.yuv",(path?path:"NULL"));
    FrameCounter *frmCnt = new FrameCounter(esInfo,stats);
    if(!presM->decoderSet) { //we have to decode, since all adaptation was in compressed domain
      
			MP4Decoder *mp4dec = new MP4Decoder((VideoESInfo*)esInfo,false);

      ach->addAdaptor(mp4dec);
      ach->addAdaptor(frmCnt);
    //  presM->clientObj = new DataSink(i,output,localhost,0, ach);
    }
    else { //frames are already decoded by adaptors (eg ColorReduction)
		ach->addAdaptor(frmCnt);
     // presM->clientObj = new DataSink(i,output,localhost,0,frmCnt);
    }

	struct stat st;
	if (!hasAdaptedInput && (stat(yuvOrig,&st) != -1)) { //found orig YUV, so add PSNR adaptor
		YUVStreamIO *yuv = new YUVStreamIO((VideoESInfo*)esInfo,yuvOrig,false);
		yuv->open();
		PSNR *psnr = new PSNR((VideoESInfo*)esInfo,yuv,stats);
		ach->addAdaptor(psnr);
		dprintf_full("LocalPresentation::addDecoderAdaptor  found origYUV %s for PSNR calc adaptor\n",yuvOrig);

		YUVStreamIO *yuvOut = new YUVStreamIO((VideoESInfo*)esInfo,yuvDumpOut,true);
		DataDump *yuvdump = new DataDump(esInfo,yuvOut,true);
		yuvOut->open();
		ach->addAdaptor(yuvdump);
		dprintf_full("LocalPresentation::addDecoderAdaptor  YUV-DataDump to %s enabled\n",yuvDumpOut);
	} else {
		cout << "Could not find orig YUV file" << endl;
		dprintf_full("LocalPresentation::addDecoderAdaptor  could not find origYUV"
									 " %s for PSNR calc adaptor...\n",yuvOrig);
		}
	presM->clientObj = new DataSink(id,output,localhost,0,ach);

  }
  if(esInfo->isAudioStream()) {
    presM->clientObj = new DataSink(id,output,localhost,0,
																		new MP4audioDecoder(((AudioESInfo*)esInfo),true,MP4audioDecoder::FFMPEG));
	

  }
}




void LocalPresentation::setAdaptor(list <QString> adaptorList) {
  adaptorsL.clear();
  for(list <QString>::iterator aI = adaptorList.begin(); aI != adaptorList.end(); aI ++) {
    adaptorsL.push_back(*aI);
  }
}



bool LocalPresentation::play() {
		
  if(state == READYTOPLAY) {
    
    for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
      (*pm)->dChannel->insert((*pm)->clientObj);
       bool res = (*pm)->dChannel->play((*pm)->clientObj->getDataSinkId(),0.0);
      
	  if(!(*pm)->dChannel->running())				
      (*pm)->dChannel->start();
	  
	   if(!res) {
				return false;
	   }
    }
	globalTimer->setPaused(false);
    state = PLAYING;
    presState = PRES_PLAYING;
  }
  else {
    if (state == PAUSED) {
			struct timeval tv;
			double old_dtime = globalTimer->getActualSec();
			gettimeofday(&tv,NULL);
			globalTimer->setPaused(false);
      for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
				double now_dtime =  tv.tv_sec + tv.tv_usec/1000000.0 - (*pm)->stats->getFirstPacketTime();
				//adjust playout time after pausing
				(*pm)->stats->setFirstPacketTime((*pm)->stats->getFirstPacketTime() + (now_dtime - old_dtime));

				bool res = (*pm)->dChannel->play((*pm)->clientObj->getDataSinkId(),0.0);
		  	if(!(*pm)->dChannel->running())
					(*pm)->dChannel->start();
				
				if(!res) {
					return false;
				}
      }
	  	
      state = PLAYING;
      presState = PRES_PLAYING;
    }
  }

  return true;
}

bool LocalPresentation::isPaused() {
  return globalTimer->isPaused();
}

bool LocalPresentation::isPrefetching() {
  return globalTimer->isPrefetching();
}

bool LocalPresentation::pause(u32 sessionKey) {
  if(state == PLAYING) {
    globalTimer->setPaused(true);
    for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {

      (*pm)->dChannel->pause((*pm)->clientObj->getDataSinkId());
	}

    state = PAUSED;
    presState = PRES_PAUSED;
    return true;
  }
  return false;
}

bool LocalPresentation::mute(u32 sessionKey,bool muteAudio, bool muteVideo) {
  if(state == PLAYING) {
    dprintf_full("LocalPresentation::mute -> MUTE was called\n");
    for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
      if(muteAudio && muteVideo) {
				/*if((*pm)->esInfo->isAudioStream()) { //we do this in SDLaudioIO::mute 
					SDL_PauseAudio(1);
				}*/
				(*pm)->dChannel->mute((*pm)->clientObj->getDataSinkId());

      }
      else
		  if(muteAudio) {
				if((*pm)->dChannel->getESInfo()->isAudioStream()) {
					(*pm)->dChannel->mute((*pm)->clientObj->getDataSinkId());
				/*	SDL_PauseAudio(1);*/
				}
		  }
		  else
			  if(muteVideo) {
				  if((*pm)->dChannel->getESInfo()->isVisualStream()) {
					(*pm)->dChannel->mute((*pm)->clientObj->getDataSinkId());
				  }
			  }
    }
    presState = PRES_MUTED;
    return true;
  }
  return false;
}

bool LocalPresentation::unmute(u32 sessionKey,bool muteAudio, bool muteVideo) {
  if(state == PLAYING) {
    for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
      if(muteAudio && muteVideo) {
				/*if((*pm)->esInfo->isAudioStream()) { //we do this in SDLaudioIO class
					SDL_PauseAudio(0);
				}*/
				(*pm)->dChannel->unmute((*pm)->clientObj->getDataSinkId());
      }
      else 
		  if(muteAudio) {
		   		if((*pm)->dChannel->getESInfo()->isAudioStream()) {
					(*pm)->dChannel->unmute((*pm)->clientObj->getDataSinkId());
				/*	SDL_PauseAudio(0);*/
				}
		  }
		  else
			  if(muteVideo) {
				  if((*pm)->dChannel->getESInfo()->isVisualStream()) {
					(*pm)->dChannel->unmute((*pm)->clientObj->getDataSinkId());
					
				  }
			  }


	
    }
    presState = PRES_PLAYING;
    return true;
  }
  return false;
}

bool LocalPresentation::tearDown(u32 sessionKey) {
	
  if(state == PLAYING || state == PAUSED || state == READYTOPLAY) {
    list <presentationManagement *>::iterator pm;
	globalTimer->setPaused(false);
    while(!presManage.empty()) {
      pm = presManage.begin();
      if((*pm)->dChannel) {
				if(state != PAUSED) {
					(*pm)->dChannel->pause((*pm)->clientObj->getDataSinkId());
				}
				(*pm)->dChannel->teardown((*pm)->clientObj->getDataSinkId(),true);
				if((*pm)->dChannel->getNumberOfActiveDataSinks()==0) {
					(*pm)->dChannel->wait(); // wait for thread to finish
					
				}
				if(state != PAUSED) {
					delete (*pm)->dChannel;
				}
			//	if(!(*pm)->dChannel->running()) 
			//		delete (*pm)->dChannel;
			/*	while((*pm)->dChannel->running()) {}
							delete (*pm)->dChannel;
			*/		
				
      }
      else {
				if((*pm)->clientObj)
					delete (*pm)->clientObj; //also deletes video+audioOutput
				if ((*pm)->adaptorChain)
					delete (*pm)->adaptorChain;
			}
	/*	if((*pm)->esSync) {
			delete (*pm)->esSync;
		}
		if((*pm)->stats){
			delete (*pm)->stats;
		}
	*/

      if(*pm)
				delete (*pm);
      presManage.pop_front();
    }
    presManage.clear();
    origESInfo.clear();
    actESInfo.clear();
    lastX = 0;
    lastY = 0;
    state = CLOSED;
    return true;
  }
  return false;
}

void LocalPresentation::run() {
  MP4IO* input = NULL;
  if (state == READYTOPLAY) {
    for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
      input = (MP4IO*)IOCreator::createInput((*pm)->esInfo);
      (*pm)->dChannel = new DataChannel(input,(*pm)->esInfo);
      (*pm)->dChannel->setAdaptor((*pm)->adaptorChain);
			input->open();
    }
  }
  if ((state == READYTOPLAY) || (state == PAUSED))
    this->play();
	this->exit();

}

int LocalPresentation::getTotalAverageBandwidth(u32 interval) {
  long totavBW = 0;
  long avBWStream = 0;
  int num=0;
  int ret;
  u32 real_iv=1;
  u32 streamSec = 0;
  if(!presManage.empty()) {
    for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
      streamSec = (*pm)->stats->getStreamoutSec();
      if (streamSec > 0) {
				if (streamSec < interval)
					real_iv = streamSec;
				else
					real_iv = interval;
				for (u32 i=1; i < real_iv+1; i++) {
					ret = (*pm)->stats->getBWfromPlayoutSec(streamSec-i);
					if (ret >= 0) {
						num++;
						avBWStream += ret;
					}

				}
      }
      if (num > 0)
				avBWStream = avBWStream / num;
	  totavBW += avBWStream;
	  avBWStream = 0;
	  num = 0;
    }
    return totavBW;
  }
  else {
    return -1;
  }
}


float LocalPresentation::getActualSecOfLongestES() {
  return globalTimer->getActualSec();		
}

list <float> LocalPresentation::getFramesPerSecond(u32 interval) {
	
  list <float> fpsList;
  float fps = 0;
  u32 real_iv=1;
  u32 streamSec = 0;
  if(!presManage.empty()) {
    for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
      fps = 0;
      streamSec = (int)floor((*pm)->stats->getPlayoutSec());
      if (streamSec > 0) {
				if (streamSec < interval)
					real_iv = streamSec;
				else
					real_iv = interval;
				for (u32 i=1; i < real_iv+1; i++) {
					fps += (*pm)->stats->getPlayoutSecFPS((int)streamSec-i);
				}
      }
      fpsList.push_back(fps/real_iv);
    }
    return fpsList;
  }
  else {
    fpsList.clear();
    return fpsList;
  }
}


list <float> LocalPresentation::getOrigFramesPerSecond() {
	list <float> fpsList;
	float fps = 0;

	fpsList.clear();
  if(!presManage.empty()) {
    for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
			fps = (*pm)->stats->getOriginalFPS();
			fpsList.push_back(fps);
		}
	}
	return fpsList;
}

list <int> LocalPresentation::getAverageBandwidth(u32 interval) {

  list <int> aBWList;
  int num=0;
  int ret;
  long avBW = 0;
  u32 real_iv=1;
  u32 streamSec = 0;

  if(!presManage.empty()) {
    for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
      avBW = 0;
      streamSec = (*pm)->stats->getStreamoutSec();
      if (streamSec > 0) {
				if (streamSec < interval)
					real_iv = streamSec;
				else
					real_iv = interval;
				for (u32 i=1; i < real_iv+1; i++) {
					ret = (*pm)->stats->getBWfromPlayoutSec(streamSec-i);
					if (ret >= 0) {
						num++;
						avBW += ret;
					}
				}
				if (num > 0)
					avBW /= num;
				aBWList.push_back(avBW);
				num = 0;
      }
    }
    return aBWList;
  }
  else {
    aBWList.clear();
    return aBWList;
  }
}

list <float> LocalPresentation::getActualSec() {

  list <float> secList;
  if(!presManage.empty()) {
    for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
      secList.push_back(globalTimer->getActualSec());	
    }
    return secList;
  }
  else {
    secList.clear();
    return secList;
  }
}

const char * LocalPresentation::getPresentationName() {
  Url * url =new Url(uri);
  return (const char *) url->getPath();

}
