/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	LocalPresentation.hpp                                         *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _LOCALPRESENTATION_HPP_
#define _LOCALPRESENTATION_HPP_

#include <qlabel.h>
#include "Presentation.hpp"
#include <qstring.h>


class ContainerInfo;
class DataChannel;
class DataSink;
class ESInfo;
class AdaptorChain;
class Adaptor;
class MPGStreamIO;

/**
 *
 * <Class Description goes here>
 *
 *@author Ronald Sowa
 *
 *@version $Id: LocalPresentation.hpp,v 1.7 2004/03/12 22:49:46 coderun Exp $
 *
 *@short <short description>
 *
 */
class LocalPresentation : public Presentation {

public:
  /* states for LocalPresentation */
  enum LocalPresentationState { CLOSED, INITIALIZED, READYTOPLAY,PLAYING, PAUSED, TEARDOWN };


  LocalPresentation(u32 x_inSDL, u32 y_inSDL,  ReferenceCounter * gtRef, GlobalTimer *gt) ;
  ~LocalPresentation();

  /* loads mp4 file from disk, fetches the ContainerInfo
     returns 0 on success
  */
  int initalizeURI(const char * URI);

  /* parses ContainerInfo object, sets up VideoDisplays for each VES
     ToDo: implement audio part
  */
  void setupURI();

  const char * getPresentationName();

  const char * getOrigURI() {
    return origURI;
  }

  /* sets adaptors for LocalPresentation */
  void setAdaptor(list <QString> adaptorList);

  /* returns a list of set adaptors for LocalPresentation */
  list <QString> getAdaptors() {
    return adaptorsL;
  }
  /* see Presentation.hpp for further information */
  float getActualSecOfLongestES();
  int getTotalAverageBandwidth(u32 intervall);
  list <float> getFramesPerSecond(u32 intervall);
	list <float> getOrigFramesPerSecond();
  list <int> getAverageBandwidth(u32 intervall);
  list <float> getActualSec();
  bool play();
  bool pause(u32 sessionKey);
  bool isPaused();
  bool isPrefetching();
  bool mute(u32 sessionKey,bool muteAudio, bool muteVideo);
  bool unmute(u32 sessionKey,bool muteAudio, bool muteVideo);
  bool tearDown(u32 sessionKey);
  void run();

  bool offlineChecked;

private:

  list <IO *> input;
  LocalPresentationState state;
  MPGStreamIO * inputs;
  int longestESSynchronizerId;

	/* sets the decoder for IO output, if available and orig is not adapted, enables PSNR with orig */
  void addDecoderAdaptor(IO * output, ESInfo * esInfo, Statistics *stats, int id, bool hasAdaptedInput=false);
  char *url;
  const char *origURI;
  bool firstTime;
  bool inputSet;

  /* a list of selected adaptors
     if spatial adaption is selected, the next 2
     list entries after "Spatial Adaption" are values
     for width and height
  */
  list <QString> adaptorsL;
};
#endif
