/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	MultiVideoPlayerDialog.cpp                                    *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/
#include <fstream>
#include <time.h>

#include <qabstractlayout.h>
#include <qaction.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qcolor.h>
#include <qcombobox.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qgroupbox.h>
#include <qinputdialog.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlistbox.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qregexp.h>
#include <qslider.h>
#include <qspinbox.h>
#include <qstring.h>
#include <qtextstream.h>
#include <qtoolbutton.h>
#include <qtooltip.h>

#include "TermCapsImpl.hpp"
#include "TermCapsRemoteImpl.hpp"
#include "RemotePresentation.hpp"
#include "LocalPresentation.hpp"
#include "PresentationInfoBox.hpp"
#include "RemoteDialog.hpp"
#include "ProfileParser.hpp"
#include "IntroDialog.hpp"

#include <adaptors/TemporalAdaptor.hpp>
#include <net/SDP.hpp>

#include "MultiVideoPlayerDialog.hpp"

GlobSlider::GlobSlider(QWidget * parent,const char * name) :
  QSlider(parent,name) {
  setTracking(false);
}

GlobSlider::~GlobSlider() {
}


void GlobSlider::mousePressEvent(QMouseEvent *e) {}

MultiVideoPlayerDialog::MultiVideoPlayerDialog( QWidget* parent, const char* name, bool modal, WFlags f )
  : MultiVideoPlayerDialogBase( parent, name, modal, f ) {
  commandLine = false;
  profileLoading = false;
  firstTime = true;
  wasPrefetching=true; //inital prefetching
  loop = false; //initial there is no looping
  exitafterplay = false; //initial no automatic exiting

  windowWidth = 0;
  windowHeight = 288;
  oldWidth = width();
  totWidth = 0;
  firstTime = true;
  nrOfWidgets=0;
  presentationsPlaying = false;
  initMenuBar();


  boxScroll = new QScrollBar(this);
  boxScroll->setOrientation(QScrollBar::Horizontal);
  boxScroll->hide();

  QSpacerItem* sp = new QSpacerItem( 20, 16, QSizePolicy::Minimum, QSizePolicy::Fixed );
  Layout5->addItem(sp,0,1);
  Layout5->addWidget(boxScroll,1,1);

  glOpts = new GlobalOptions(0,0,true);

#ifdef WIN32
	theapp.InitScreen(1,1,0,  SDL_ANYFORMAT | SDL_HWSURFACE  |  SDL_NOFRAME);
#else
	//  theapp.InitScreen(352,288,16, SDL_ANYFORMAT | SDL_HWSURFACE  );
	theapp.InitScreen(1,1, 0, SDL_SWSURFACE | SDL_NOFRAME);
#endif

#ifdef WIN32
  QString str(getenv("USERPROFILE"));
  str.append("\\");
#else
  QString str(getenv("HOME"));
  str.append("/");
#endif
  str.append(GLOBALOPTIONSFILE);
  loadGlobalConfig(str.latin1());

  theapp.LoadTheme("default");
	char pixmap[MAX_STR_LEN];
	sprintf(pixmap,"%s/%s",glConfig->iconPath, "vitooki.png");
  theapp.SetCaption("muViPlayer - Vitooki",pixmap);
  setCaption("Multivideo Player");

  last_opened = NULL;
  activeVideo = NULL;
  activePresentation = NULL;
  longestPresentation = NULL;
  buttonStop->setEnabled(false);
  buttonPlayPause->setEnabled(false);

	sprintf(pixmap,"%s/%s",glConfig->iconPath, "play.bmp");
  if(!playPic.load(pixmap)) {
    dprintf_full("Could not load PlayPic!\n");
  }
	sprintf(pixmap,"%s/%s",glConfig->iconPath, "pause.bmp");
  pausePic.load(pixmap);
	sprintf(pixmap,"%s/%s",glConfig->iconPath, "stop.bmp");
  stopPic.load(pixmap);
  buttonPlayPause->setPixmap(playPic);
  buttonStop->setPixmap(stopPic);
  buttonPlayPause->show();
  buttonStop->show();
  boxValuesTimer = new QTimer(this);

  delete globalSlider;
  slid = new GlobSlider(gSelections);
  globalSlider = slid;
  globalSlider->setOrientation( QSlider::Horizontal );
  Layout4->addWidget( globalSlider, 1, 0 );
  buttonNewLocalWidget->setMinimumSize(QSize(45,42));
  buttonNewLocalWidget->setMaximumSize(QSize(45,42));
  buttonNewRemoteWidget->setMinimumSize(QSize(45,42));
  buttonNewRemoteWidget->setMaximumSize(QSize(45,42));
  buttonNewLocalWidget->setFocus();

  globalSlider->disconnect(SIGNAL(sliderPressed()),globalSlider);
  globalSlider->disconnect(SIGNAL(sliderMoved(int)),globalSlider);


  endOfPresentationTimer = new QTimer(this);
  connect(&theapp,SIGNAL(f_pressed()),this,SLOT(setWindowedMode()));
  connect(&theapp,SIGNAL(space_pressed()),this,SLOT(play_pause()));
  connect(&theapp,SIGNAL(signal_boxupdate()),this,SLOT(endOfPresentation()));
  connect(boxValuesTimer,SIGNAL(timeout()),this,SLOT(updateBoxValues()));
  connect(endOfPresentationTimer,SIGNAL(timeout()),this,SLOT(endOfPresentation()));
  connect(boxScroll,SIGNAL(sliderMoved(int)),this,SLOT(moveInfoBoxes()));
  connect(boxScroll,SIGNAL(nextLine()),this,SLOT(moveInfoBoxes()));
  connect(boxScroll,SIGNAL(prevLine()),this,SLOT(moveInfoBoxes()));
  connect(boxScroll,SIGNAL(valueChanged(int)),this,SLOT(moveInfoBoxes()));

	gtRefCount = new ReferenceCounter();
	globalTimer = new GlobalTimer();
	//this will prevent deletion on DataChannel closes:
	gtSharedAdapt = new SharedAdaptor(gtRefCount, (Adaptor*&)globalTimer);

  theapp.EnableAppIdleCalls(false);
}


MultiVideoPlayerDialog::~MultiVideoPlayerDialog() {
	if(presentationsPlaying)
		stop();
  if(activeVideo)
    delete activeVideo;
  if(activePresentation)
    delete activePresentation;
  videoBoxes.clear();
  presentations.clear();

	delete gtSharedAdapt; //this will also del gtRefCount and globalTimer

  theapp.LockScreen();
  theapp.Quit();
  theapp.UnlockScreen();

}


void MultiVideoPlayerDialog::initMenuBar() {

  QAction *fileLoadAction = new QAction( "Load", "&Load Configuration", QDialog::CTRL+QDialog::Key_P, this, "load_config()" );
  QAction *fileSaveAction = new QAction( "Save", "&Save Configuration", QDialog::CTRL+QDialog::Key_S, this, "save_config" );
  QAction *fileDeleteAllPresentationsAction = new QAction( "Delete all Presentations", "&Delete all Presentations", QDialog::CTRL+QDialog::Key_D, this, "deleteAllPresentations()" );
  QAction *fileQuitAction = new QAction( "Quit", "&Quit", QDialog::CTRL+QDialog::Key_Q, this, "quit" );
  QAction *fileOpenAction = new QAction( "Open Local Presentation", "Open &Local Presentation", QDialog::CTRL+QDialog::Key_L, this, "new_localWidget()" );
  QAction *urlOpenAction = new QAction( "Open Remote Presentation", "Open &Remote Presentation", QDialog::CTRL+QDialog::Key_R, this, "new_remoteWidget()" );
  QAction *optionsOpenAction = new QAction( "Global Options", "&Global Options", QDialog::CTRL+QDialog::Key_G, this, "globalOptionsDialog()" );
  QAction *about = new QAction("About","About", QDialog::CTRL+QDialog::Key_A,this,"about()");
  QAction *swfullscreen = new QAction("Fullscreen","Fullscreen", QDialog::Key_F,this,"fullscreen()");

  QMenuBar * menBar = new QMenuBar(this);
  fileMenu = new QPopupMenu( NULL );
  QPopupMenu *openMenu = new QPopupMenu(fileMenu);
  fileOpenAction->addTo(openMenu);
  urlOpenAction->addTo(openMenu);
  fileMenu->insertItem("Open..",openMenu);

  fileLoadAction->addTo( fileMenu );
  fileSaveAction->addTo( fileMenu );
  fileMenu->insertSeparator(3);
  fileDeleteAllPresentationsAction->addTo( fileMenu);
  fileQuitAction->addTo( fileMenu );

  menBar->insertItem( "&File", fileMenu );
  menBar->insertSeparator();

  QPopupMenu * optionsMenu = new QPopupMenu(NULL);
  optionsOpenAction->addTo(optionsMenu);
  swfullscreen->addTo(optionsMenu);
  menBar->insertItem("&Options",optionsMenu);


  QPopupMenu * helpMenu = new QPopupMenu(NULL);
  about->addTo(helpMenu);

  menBar->insertItem("&Help",helpMenu);

  connect( fileQuitAction, SIGNAL( activated() ),this, SLOT( quit() ) );
  connect( fileLoadAction, SIGNAL( activated() ),this, SLOT( load_config() ) );
  connect( fileSaveAction, SIGNAL( activated() ),this, SLOT( save_config() ) );
  connect( fileOpenAction, SIGNAL( activated() ), this, SLOT( new_localWidget()));
  connect( urlOpenAction, SIGNAL( activated() ), this, SLOT( new_remoteWidget()));
  connect( fileDeleteAllPresentationsAction,SIGNAL( activated() ), this, SLOT( askIfDeleteAllPresentations()));
  connect( optionsOpenAction, SIGNAL( activated() ), this, SLOT( globalOptionsDialog()));
  connect( about, SIGNAL( activated() ), this, SLOT( about()));
  connect( swfullscreen, SIGNAL( activated() ), this, SLOT( fullScreen()));
  fileMenu->setItemEnabled(fileMenu->idAt(2),false);
  fileMenu->setItemEnabled(fileMenu->idAt(4),false);
}

void MultiVideoPlayerDialog::enableLooping(bool loop) {
	this->loop = loop;
}

void MultiVideoPlayerDialog::exitAfterPlay() {
	exitafterplay = true;
}

void MultiVideoPlayerDialog::quit() {

  // ToDo: call destructors
  exit(0);
}

void MultiVideoPlayerDialog::about() {
  About * ab = new About(0,0,true);
  ab->exec();
}


void MultiVideoPlayerDialog::loadGlobalConfig(const char * configFile) {
  glConfig = new GlobalConfig();
  FILE * fp = NULL;
  fp = fopen(configFile,"r");
  if(!fp) {
    fp = fopen(configFile,"w");
    fputs(".\n",fp);
    fputs("NULL\n",fp);
    fputs("0\n",fp);
#ifndef WIN32
		fputs("/usr/local/share/muviplayer",fp); //default path LINUX
#else
		fputs("./Icons/",fp); // default path WINDOWS
		strcpy(glConfig->iconPath,"./Icons/");
#endif
    fclose(fp);
    // add here default proxyname
    strcpy(glConfig->pathToContainerFiles,".");
    strcpy(glConfig->proxyName,"");
    // add here default proxyport
    glConfig->proxyPort = 0;

    glOpts->checkBoxProxy->setChecked(false);
    glOpts->lineEditProxyName->setEnabled(false);
    glOpts->spinBoxProxyPort->setEnabled(false);
    glOpts->lineEditLocalDirectory->setText(".");

    glOpts->lineEditProxyName->setText("");
    glOpts->spinBoxProxyPort->setValue(5554);


  }
  else
    {
      char * pport = new char[255];
      fgets(glConfig->pathToContainerFiles,MAX_STR_LEN,fp);
      glConfig->pathToContainerFiles[strlen(glConfig->pathToContainerFiles)-1] = '\0';
      fgets(glConfig->proxyName,MAX_STR_LEN,fp);
      glConfig->proxyName[strlen(glConfig->proxyName)-1] = '\0';
      if(strcmp(glConfig->proxyName,"NULL") == 0) {
				strcpy(glConfig->proxyName,"");
				glOpts->checkBoxProxy->setChecked(false);
				glOpts->lineEditProxyName->setEnabled(false);
				glOpts->spinBoxProxyPort->setEnabled(false);
      }
      else {


				glOpts->checkBoxProxy->setChecked(true);
				glOpts->lineEditProxyName->setEnabled(true);
				glOpts->spinBoxProxyPort->setEnabled(true);
      }
      fgets(pport,255,fp);
      glConfig->proxyPort = atoi(pport);

			if (!fgets(glConfig->iconPath,MAX_STR_LEN,fp))
#ifndef WIN32
				strcpy(glConfig->iconPath,"/usr/local/share/muviplayer");
#else
				strcpy(glConfig->iconPath,"./Icons/");
#endif
			else
				glConfig->iconPath[strlen(glConfig->iconPath)-1] = '\0';

      strcpy(glConfig->configFile,configFile);
      fclose(fp);
    }
  glOpts->lineEditLocalDirectory->setText(QString(glConfig->pathToContainerFiles));
  if(!glConfig->proxyName) {
    glOpts->lineEditProxyName->setText("");
  }
  else {
    glOpts->lineEditProxyName->setText(QString(glConfig->proxyName));
  }
  glOpts->spinBoxProxyPort->setValue(glConfig->proxyPort);
  dprintf_full("MultiVideoPlayerDialog::loadGlobalConfig -> ProxyName: %s\n",glConfig->proxyName);
  dprintf_full("MultiVideoPlayerDialog::loadGlobalConfig -> ProxyPort: %i\n",glConfig->proxyPort);
}

void MultiVideoPlayerDialog::save_globalConfig() {

  strcpy(glConfig->pathToContainerFiles,last_opened->latin1());
  if(glOpts->checkBoxProxy->isChecked()) {
    strcpy(glConfig->proxyName,glOpts->lineEditProxyName->displayText().latin1());
    glConfig->proxyPort = (glOpts->spinBoxProxyPort->text()).toInt();
  }
  else {
    strcpy(glConfig->proxyName,""); // = NULL;
    glConfig->proxyPort = 0;
  }
  ofstream outConfig;
  outConfig.open(glConfig->configFile,fstream::out | fstream::trunc);
  if(!outConfig.is_open()) {
    return;
  }
  outConfig << glConfig->pathToContainerFiles << endl;
  if(strcmp(glConfig->proxyName,"") == 0) {
    outConfig << "NULL" << endl;
  }
  else {
    outConfig << glConfig->proxyName << endl;
  }
  outConfig << glConfig->proxyPort << endl;

  outConfig << glConfig->iconPath << endl;
  outConfig.close();
}


void MultiVideoPlayerDialog::setWindowedMode() {

  list <Presentation *>::iterator pres;
  for(pres=presentations.begin();pres!=presentations.end();pres++) {
    if((*pres)->isActive()) {
      (*pres)->mute((*pres)->getSessionKey(),false,true);
      (*pres)->resizePresentation((*pres)->getPresentationWidth(), (*pres)->getPresentationHeight());
      break;
    }
  }
  reinitializeVideoWindow(false);
  list <PresentationInfoBox *>::iterator aV = videoBoxes.begin();
  for(pres=presentations.begin();pres!=presentations.end();pres++) {
    (*pres)->unmute((*pres)->getSessionKey(),true,true);
    if((*aV)->isMuted()) {
      (*pres)->mute((*pres)->getSessionKey(),true,false);
    }
    aV++;
  }

}

void MultiVideoPlayerDialog::fullScreen() {
  if(presentationsPlaying)
		theapp.switchToFullscreen();
}

void MultiVideoPlayerDialog::presentationProperties() {
  activeVideo->openProperties();
}

void MultiVideoPlayerDialog::globalOptionsDialog() {
  glOpts->lineEditLocalDirectory->setText(glConfig->pathToContainerFiles);
  glOpts->exec();
  if(glOpts->result() == Accepted) {
    last_opened = new QString(glOpts->lineEditLocalDirectory->displayText().latin1());
    save_globalConfig();
  }
}

void MultiVideoPlayerDialog::termCapsDialog() {
  bool dimensionChanged = false;
  bool dummyAdaptors = false;
  if(activeVideo) {

    TerminalCapabilities * terminalCaps = NULL;
    TermCapsRemote * TermCapsR = NULL;
    list <ESInfo *> presInfo;
    if(activeVideo->isLocal()) {
      TermCapsImpl * tcaps = new TermCapsImpl(NULL,"Terminal Capabilities",true);
      if(((LocalPresentation *) activePresentation)->offlineChecked) {
				tcaps->ofCheckBox->setChecked(true);

				dummyAdaptors = true;
      }
      //	if(dummyAdaptors)
      tcaps->setActiveAdaptors(((LocalPresentation *)activePresentation)->getAdaptors());
      ///	else
				//		tcaps->setActiveAdaptors(adaptorList);

				tcaps->exec();
				if (tcaps->result() != Accepted) {
					return;
				}
				if(tcaps->ofCheckBox->isChecked()) {
					dummyAdaptors = true;
					((LocalPresentation *) activePresentation)->offlineChecked = true;
				}
				else
					((LocalPresentation *) activePresentation)->offlineChecked = false;
				dimensionChanged = tcaps->mustWindowResize();
				dimensionChanged = true;
				QString adapt;
				int numAdaptors = tcaps->activeAdaptors->count();
				adaptorList.clear();
				for (int i = 0; i < numAdaptors; i++) {
					adapt = tcaps->activeAdaptors->text(i);
					adaptorList.push_back(adapt);
					if(adapt.compare("Spatial Adaptation") == 0) {
						QString str;
						adaptorList.push_back(QString(str.setNum(tcaps->getDimensionX())));
						adaptorList.push_back(QString(str.setNum(tcaps->getDimensionY())));

					}
				}
    }
    else {

      TermCapsR = new TermCapsRemote(this,"Terminal Capabilities",true);
      TermCapsR->setTermcapsOfPresentation(((RemotePresentation *)activePresentation)->getTermCaps(),
																					 ((RemotePresentation *)activePresentation)->isRtxEnabled(),
																					 ((RemotePresentation *)activePresentation)->isUpscaleSet(),
																					 ((RemotePresentation *)activePresentation)->isKeepAspectSet(),
																					 ((RemotePresentation *)activePresentation)->isSwitchingEnabled());
      TermCapsR->exec();
      if (TermCapsR->result() != Accepted) {
				return;
      }
      terminalCaps = new TerminalCapabilities();
      terminalCaps->setDisplayBitsPerPixel(16);
      terminalCaps->setColorDisplay(TermCapsR->getColorEnabled());
      terminalCaps->setDisplayWidth(TermCapsR->getVideoWidth());
      terminalCaps->setDisplayHeight(TermCapsR->getVideoHeight());
      terminalCaps->setDisplayRefreshRate(100);
      terminalCaps->setNetworkCapacityInByte(TermCapsR->bwvalue->text().toInt()*128);
      terminalCaps->setMinGuaranteedNetworkCapacityInByte((TermCapsR->bwvalue->text().toInt()-10)*128);
      terminalCaps->setMaxDecoderBitRateInBit(12345678);
      terminalCaps->setMaxDecoderBufferSizeInByte(12345678);
      terminalCaps->setNumAudioChannels(TermCapsR->getNumAudioChannels());
      dimensionChanged = true;
    }


    if(activePresentation->getState() == Presentation::PRES_READYTOPLAY) {
      activePresentation->tearDown(TEARDOWN_ALL);
      if(activePresentation->isLocal()) {
				((LocalPresentation *) activePresentation)->setAdaptor(adaptorList);
      }
      else {
				((RemotePresentation *) activePresentation)->setTermCaps(terminalCaps,TermCapsR->getRtxEnabled(),TermCapsR->checkUpscale->isChecked(),TermCapsR->checkAspect->isChecked(),TermCapsR->switchAllow->isChecked());
      }
      QString * URI = new QString();
      if((dummyAdaptors) && (activePresentation->isLocal())) {
				URI->append(getAdaptedFile(adaptorList,((LocalPresentation *) activePresentation)->getOrigURI()));
      }
      else {
				URI->append(activePresentation->getURI());
      }
      if(activePresentation->initalizeURI(URI->latin1()) == -1) {
				if(activePresentation->isLocal()) {
					QMessageBox mess( "Error loading MP4 File...",
														"Cannot load MP4 File. Maybe the URI is not correct, file was not found."
														" Should the original MP4 File be loaded?",
														QMessageBox::Information , QMessageBox::Yes
														, QMessageBox::No , QMessageBox::NoButton , this );
					mess.exec();
					list <QString> emptyList;
					emptyList.clear();
					((LocalPresentation *)activePresentation)->setAdaptor(emptyList);
					if(mess.result() == QMessageBox::Yes) {

						if(activePresentation->initalizeURI(((LocalPresentation *)activePresentation)->getOrigURI()) == -1) {
							QMessageBox mess( "Error loading MP4 File...", "Cannot load original MP4 File."
																" Aborting now.", QMessageBox::Critical , QMessageBox::Ok
																, QMessageBox::NoButton , QMessageBox::NoButton , this );
							mess.exec();
							deletePresentation();
						}
					}
					else {
						deletePresentation();
						return;
					}


				}
				else {
					QMessageBox mess( "Error loading MP4 File...",
														"Cannot load MP4 File. Maybe the URI is not correct, file was not found. "
														, QMessageBox::Information , QMessageBox::Ok
														, QMessageBox::NoButton , QMessageBox::NoButton , this );
					mess.exec();
					deletePresentation();
					return;
				}
      }
      activePresentation->setupURI();
    }

    presInfo = activePresentation->getActualPresentationInfo();
    activeVideo->pDialog->setProperties(activePresentation->presInform);
    activeVideo->setVideoInfo(presInfo);
    if(dimensionChanged) {
      reinitializeVideoWindow(false);
    }
  }
}

Presentation *MultiVideoPlayerDialog::setNewWidget(bool localWidget) {

  Presentation *newPres = NULL;
  buttonPlayPause->setEnabled(true);
  buttonPlayPause->setFocus();
  int lastX = 0;
  int xPos = 0;
  if(!presentations.empty()) {
    lastX = presentations.back()->getPresentationWidth();
    xPos = presentations.back()->getPresentationXpos();

  }
  if(!localWidget) {
    RemotePresentation * rPresentation = NULL;
    if(glConfig->proxyName) {
      dprintf_full("new RemotePres   proxy %s\n",glConfig->proxyName);

      rPresentation = new RemotePresentation(lastX+xPos,0, gtRefCount, globalTimer, glConfig->proxyName,
       this, glConfig->proxyPort);

    }
    else {

      rPresentation = new RemotePresentation(lastX+xPos,0, gtRefCount, globalTimer, NULL, this);

    }
    newPres = (Presentation *) rPresentation;
    rPresentation->setActive(true);
    for(list <Presentation *>::iterator w = presentations.begin();w !=presentations.end(); ++w) {
      if((*w)->isActive()) {
				(*w)->setActive(false);
      }
    }
    presentations.push_back(rPresentation);


    theapp.setPresentations(presentations);
    activePresentation = rPresentation;
  }
  else {
    LocalPresentation *lPresentation = NULL;

    lPresentation = new LocalPresentation(lastX+xPos,0, gtRefCount, globalTimer);
    newPres = (Presentation *) lPresentation;
    lPresentation->setActive(true);
    for(list <Presentation *>::iterator w = presentations.begin();w !=presentations.end(); ++w) {
      if((*w)->isActive()) {
				(*w)->setActive(false);
      }
    }

    presentations.push_back(lPresentation);
    theapp.setPresentations(presentations);
    activePresentation = lPresentation;
  }
  if(presentations.size() == 1)
	  (presentations.front())->theOnlyOnePresentation = true;
  else
	  (presentations.front())->theOnlyOnePresentation = false;
  lastActive =1;
  for(list <PresentationInfoBox *>::iterator v = videoBoxes.begin();v != videoBoxes.end(); ++v) {
    if((*v)->isActive()) {
      (*v)->setActive(false);
      (*v)->deleteButton->setEnabled(false);
    }
    lastActive++;
  }
  QString vidName("Presentation ");
  QString vidNumber;
  vidName.append(vidNumber.setNum(lastActive));
  if(localWidget) {
    vidName.append(" - LOCAL");
  }
  else {
    vidName.append(" - REMOTE");
  }
  PresentationInfoBox * box = NULL;
  box = new PresentationInfoBox(lastActive,localWidget,tr(vidName),Frame3,tr(vidName));


  if (!videoBoxes.empty()) {
    box->setGeometry( QRect((videoBoxes.back())->x()+VIDEOBOXWIDTH+1,0,VIDEOBOXWIDTH,Frame3->height()));
    box->initBoxElements(glConfig);

  }
  else {
    box->setGeometry( QRect(0,0,VIDEOBOXWIDTH,Frame3->height()));
    box->initBoxElements(glConfig);
  }

  //SDL_Delay(50);
  box->deleteButton->setEnabled(true);
  activeVideo = box;
  box->show();
  nrOfWidgets++;
  connect( box, SIGNAL( pressed() ), this, SLOT( change_activeWidget() ) );
  connect( box->deleteButton, SIGNAL( clicked() ), this, SLOT( askIfDeletePresentation() ) );
  connect( box->options,SIGNAL(activated(int) ), this, SLOT( handleVideoBoxPopup(int) ) );
  connect( box->speakerButton,SIGNAL(clicked()),this, SLOT(mute_unmuteAudio()));
  videoBoxes.push_back(box);

  if(!profileLoading) {
    open_input();
  }
  return newPres;
}

void MultiVideoPlayerDialog::handleVideoBoxPopup(int i) {
  int itemId = activeVideo->options->idAt(i);
  if(itemId==3) {
    askIfDeletePresentation();
  }
  if(itemId==0) {
    presentationProperties();
  }
  if(itemId==1) {
    termCapsDialog();
  }
  dprintf_full("MultiVideoPlayerDialog::handleVideoBoxPopup -> slot called\n");
}

void MultiVideoPlayerDialog::load_config() {

  QString * fn = new QString();
  if(!commandLine) {
		fn->append(QFileDialog::getOpenFileName("./Profiles","*.prof"));
		if(fn->isEmpty()) {
			return;
		}
		if(!presentations.empty()) {
			QMessageBox mess( "Question", "Do you want to delete existing presentations before "
												"adding presentations from the profile?", QMessageBox::Information,
												QMessageBox::Yes, QMessageBox::No , QMessageBox::NoButton , this );
			mess.exec();
			if(mess.result() == QMessageBox::Yes) {
				deleteAllPresentations();
			}
		}
  }
  else {
	  fn->append(*profileName);
  }
  ProfileParser * parse = new ProfileParser();
  if( parse->parseFromConfigFile(fn->latin1()) == -1) {
    QMessageBox mess( "Parse Error!", "Parse error in profile. See logfile for further information."
											" \nAborting now!", QMessageBox::Critical , QMessageBox::Ok
											, QMessageBox::NoButton , QMessageBox::NoButton , this );
    mess.exec();
    return;
  }

  list <Profile *> prof = parse->getProfile();
  profileLoading = true;
  for(list <Profile *>::iterator pI = prof.begin(); pI != prof.end(); pI++) {
    QString * str = new QString((*pI)->uri);
    fileName = new QString(*str);
    if((*pI)->localPres) {
      new_localWidget();
      ((LocalPresentation *) activePresentation)->setAdaptor((*pI)->adaptors);
      if((*pI)->dummyAdaptors) {
				((LocalPresentation *) activePresentation)->offlineChecked = true;
      }
    }
    else {
      new_remoteWidget();
      ((RemotePresentation *) activePresentation)->setTermCaps((*pI)->termCaps,(*pI)->rtxEnabled);
    }
    open_input();
    list <ESInfo *> presInfo;
    presInfo = activePresentation->getActualPresentationInfo();
    activeVideo->pDialog->setProperties(activePresentation->presInform);
    activeVideo->setVideoInfo(presInfo);
  }
  profileLoading = false;
  reinitializeVideoWindow(false);

}
void MultiVideoPlayerDialog::save_config() {
  QString * fn = new QString();
  fn->append(QFileDialog::getSaveFileName(".","*.prof"));
  list <Profile *> profiles;
  for(list <Presentation *>::iterator pI = presentations.begin(); pI != presentations.end(); pI++) {
    Profile * prof = new Profile();
    prof->uri = (*pI)->getURI();
    if((*pI)->isLocal()) {
      if(((LocalPresentation *) (*pI))->offlineChecked) {
				prof->dummyAdaptors = true;
      }
    }
    for(list <QString>::iterator aI = (*pI)->presInform.usedAdaptors.begin();
				aI != (*pI)->presInform.usedAdaptors.end();
				aI ++)
      {
				prof->adaptors.push_back(*aI);
				if((*aI).compare("Spatial Adaptation") == 0) {
					QString * dimX = new QString();
					dimX->setNum((int) (((VideoESInfo*)(*pI)->presInform.actESInfo.front()))->getWidth());
					prof->adaptors.push_back(*dimX);
					QString * dimY = new QString();
					dimY->setNum((int) (((VideoESInfo*)(*pI)->presInform.actESInfo.front()))->getHeight());
					prof->adaptors.push_back(*dimY);
				}
      }
    prof->termCaps = (*pI)->presInform.usedTermCaps;
    prof->rtxEnabled = (*pI)->presInform.rtxEnabled;
    profiles.push_back(prof);
  }
  ProfileParser *prof = new ProfileParser();
  prof->saveConfigToFile(fn->latin1(),profiles);
}
void MultiVideoPlayerDialog::new_localWidget() {
  setNewWidget(true);
}

void MultiVideoPlayerDialog::new_remoteWidget() {
  setNewWidget(false);
}

bool MultiVideoPlayerDialog::newPresentationFromSDP(const QString &file)
{
  QString fname = file;
  fname.replace(QRegExp("^file:(//)?"),"");

  /* read SDP file */
  QFile qfile(fname);
  if (!qfile.open(IO_ReadOnly))
    return false;
  QTextStream stream(&qfile);
  QString sdpBuf = stream.read();
  qfile.close();
  /* parse SDP file */
  SDP sdp;
  if (!sdp.parse(sdpBuf.latin1()))
    return false;
  const struct SDP::session *session = sdp.firstSession();
  if (!session)
    return false;
  /* setup new presentation */
  bool local = false;
  fileName = new QString(session->url.c_str());
  if (fileName->startsWith("file:") || fileName->startsWith("/"))
    local = true;
  Presentation *pres = setNewWidget(local);
  if (!pres)
    return false;
  pres->setTimeRange(session->nptStart, session->nptEnd);
  return true;
}

QString MultiVideoPlayerDialog::getAdaptedFile(list <QString> adaptors, const char * origURI) {

  const QString * saX = NULL;
  const QString * saY = NULL;
  bool cr = false;
  bool ta = false;
  bool sta = false;
  bool qr = false;
  bool sa = false;

  QString adaptedFileName(origURI);
  adaptedFileName.truncate(adaptedFileName.length()-4);
  for(list <QString>::iterator aI = adaptors.begin(); aI != adaptors.end(); aI++) {
    if ((*aI).compare("Color Reduction") == 0) {
      cr = true;
    }
    if ((*aI).compare("Temporal Adaptation") == 0) {
      ta = true;

    }
    if ((*aI).compare("Strong Temporal Adaptation") == 0) {
      sta = true;
    }
    if ((*aI).compare("Quality Reduction") == 0) {
      qr = true;
    }
    if ((*aI).compare("Spatial Adaptation") == 0) {
      sa = true;
      aI++;
      const QString xx((*aI).latin1());
      saX = new QString((*aI));
      aI++;
      saY = new QString((*aI));
    }
  }
  if(cr)
    adaptedFileName.append("_cr");
  if(ta)
    adaptedFileName.append("_ta");
  if(sta)
    adaptedFileName.append("_sta");
  if(qr)
    adaptedFileName.append("_qr_");
  if(sa) {
    adaptedFileName.append("_sa_");
		adaptedFileName.append(*saX);
    adaptedFileName.append("x");
    adaptedFileName.append(*saY);
  }
  adaptedFileName.append(".mp4");
  return adaptedFileName;
}

RemoteDialogBase * MultiVideoPlayerDialog::readURIsFromFile() {
  RemoteDialogBase * rDialog = new RemoteDialogBase(this,"Open Remote...",true);
  rDialog->ComboBox1->setDuplicatesEnabled(false);
  rDialog->ComboBox1->setAutoCompletion(true);

#ifdef WIN32
  QString str(getenv("USERPROFILE"));
  str.append("\\");
#else
  QString str(getenv("HOME"));
  str.append("/");
#endif
  str.append(URISFILE);
  //	 loadGlobalConfig(str.latin1());

  char * inItem;
  FILE * fp = NULL;
  fp = fopen(str.latin1(),"r");
  if(fp == NULL) {
    fp = fopen(str.latin1(),"w");
    fclose(fp);
    dprintf_full("MultiVideoPlayerDialog::readURIsFromFile -> Could not open File!\n");
    return rDialog;

  }
  /* dummy read first line because remoteURIs.txt starts with \n
   */
  if(!feof(fp)) {
    inItem = new char[100];
    fgets(inItem,100,fp);
  }
  while(!feof(fp)) {
    inItem = new char[100];
    fgets(inItem,100,fp);
    int i = strlen(inItem);
    if(inItem[i-1] == '\n') {
      inItem[i-1] = '\0';
    }
    rDialog->ComboBox1->insertItem(QString(inItem));

  }
  fclose(fp);
  return rDialog;
}

void MultiVideoPlayerDialog::writeURIsToFile(RemoteDialogBase *rDialog) {
  int maxItems = 10;
  maxItems = MIN(rDialog->ComboBox1->count(),maxItems);

#ifdef WIN32
  QString str(getenv("USERPROFILE"));
  str.append("\\");
#else
  QString str(getenv("HOME"));
  str.append("/");
#endif
  str.append(URISFILE);

  FILE * fp;
  fp  = fopen(str.latin1(),"w");
	if(fp) {
		for (int i=0; i< maxItems; i++) {
			rDialog->ComboBox1->setCurrentItem(i);
			QString itemTextNoNewLine = rDialog->ComboBox1->currentText();
			const char * itemText =  itemTextNoNewLine.latin1();
			fputc('\n',fp);
			fputs(itemText,fp);

		}
		fclose(fp);
	}
}

void MultiVideoPlayerDialog::open_input() {

  QString * fn = new QString();
  list <ESInfo *> info;

  bool localWidget = activeVideo->isLocal();

  /* Presentation is local
   */
#define FILE_FORMATS "*.mp4;*.avi;*.mpg;*.mpeg;*.mp3"
  if(localWidget) {
    if(!commandLine && !profileLoading) {
      if(last_opened)
				fn->append(QFileDialog::getOpenFileName(*last_opened, FILE_FORMATS));
      else {
				fn->append(QFileDialog::getOpenFileName(QString(glConfig->pathToContainerFiles),FILE_FORMATS));
				//last_opened->append(glConfig->pathToContainerFiles);
      }
    }
    else {
      fn->append(*fileName);
      if(commandLine) {
				commandLine = false;
      }
    }
  }

  /*Presentation is remote
   */
  else {
    if(!commandLine && !profileLoading) {
      RemoteDialogBase * remDialog = readURIsFromFile();
      remDialog->exec();
      if (remDialog->result() == Accepted) {
				fn->append(remDialog->ComboBox1->currentText());
				int c = remDialog->ComboBox1->count();
				for (int i = 0; i<c; i++) {
					if (fn->compare(remDialog->ComboBox1->text(i)) == 0) {
						remDialog->ComboBox1->removeItem(i);
					}
				}
				remDialog->ComboBox1->insertItem(*fn,0);
				writeURIsToFile(remDialog);

      }
      else {
				deletePresentation();
				return;
      }
      delete remDialog;
    }
    else {
      fn->append(*fileName);
      if (commandLine) {
				commandLine = false;
      }
    }

  }
  if(!fn->isEmpty()) {
    int PositionOfFirstSlash = fn->findRev('/')+1;
    QString * path = new QString(fn->left(PositionOfFirstSlash));
    if(last_opened) {
      if((strcmp(last_opened->latin1(),path->latin1())!= 0) && localWidget ) {
				last_opened = path;
				strcpy(glConfig->pathToContainerFiles,path->latin1()); //,fn->length());
				save_globalConfig();
      }
    }
    else {
      if(localWidget)
				last_opened = path;
      else
				last_opened = new QString(glConfig->pathToContainerFiles);
      save_globalConfig();
    }



    if(videoBoxes.size()*VIDEOBOXWIDTH > (unsigned)Frame3->width()) {

      boxScroll->setEnabled(true);

      boxScroll->setMaxValue(VIDEOBOXWIDTH*videoBoxes.size()+videoBoxes.size()-(int)Frame3->width());
    }
    else {
      boxScroll->setEnabled(false);
      boxScroll->setMaxValue(0);
    }
    if(videoBoxes.size()*VIDEOBOXWIDTH > (unsigned)Frame3->width()) {
      boxScroll->show();
    }
    for (list <Presentation *>::iterator w = presentations.begin();w !=presentations.end(); ++w) {
      if((*w)->isActive()) {
				if((*w)->getState() == Presentation::PRES_READYTOPLAY) {
					(*w)->tearDown(TEARDOWN_ALL);

				}
				if((*w)->initalizeURI(fn->latin1()) ==-1) {
					QMessageBox mess( "Cannot play...", "Presentation is not ready to play. "
														"You did not load a correct MP4 File!", QMessageBox::Information,
														QMessageBox::Ok, QMessageBox::NoButton , QMessageBox::NoButton , this );
					mess.exec();
					deletePresentation();
					return;
				}
				(*w)->setupURI();
      }
      info = (*w)->getActualPresentationInfo();
      dprintf_full("MultiVideoPlayerDialog::open_input -> Found active Widget\n");
    }
    if(!profileLoading) {
      reinitializeVideoWindow(false);
    }
    activeVideo->setVideoInfo(info);
    int positionOfFirstSlash = fn->findRev('/')+1;
    QString * boxTitle = new QString(fn->right(fn->length()-positionOfFirstSlash));
    if(boxTitle->length() > 20) {
      QString *cuttedBoxTitle = new QString(boxTitle->left(11));
      cuttedBoxTitle->append(tr("...."));
      cuttedBoxTitle->append(boxTitle->right(5));
      activeVideo->setTitle(*cuttedBoxTitle);
    }
    else
      activeVideo->setTitle(*boxTitle);
    activeVideo->pDialog->setProperties(activePresentation->presInform);
    fileMenu->setItemEnabled(fileMenu->idAt(2),true);
    fileMenu->setItemEnabled(fileMenu->idAt(4),true);
  }
  else {
    deletePresentation();
    return;
  }
}



void MultiVideoPlayerDialog::change_activeWidget() {

  for(list <PresentationInfoBox *>::iterator v = videoBoxes.begin();v != videoBoxes.end(); ++v) {
    if((*v)->isActive()) {
      (*v)->setActive(false);
      (*v)->deleteButton->setEnabled(false);

    }


  }
  int l = 0;

  for(list <PresentationInfoBox *>::iterator w = videoBoxes.begin();w != videoBoxes.end(); ++w) {
    if((*w)->newActive) {
      (*w)->setActive(true);
      (*w)->newActive = false;

      if(!presentationsPlaying) {
				(*w)->deleteButton->setEnabled(true);
      }

      activeVideo = (*w);
      break;
    }
    l++;
  }

  for (list <Presentation *>::iterator x = presentations.begin();x !=presentations.end(); ++x) {
    if((*x)->isActive()) {
      (*x)->setActive(false);
    }
  }
  list <Presentation *>::iterator y = presentations.begin();
  for (int j =0;j < l;j++) {
    y++;
  }
  (*y)->setActive(true);

  activePresentation = (*y);
}

void MultiVideoPlayerDialog::play_pause() {
  if(presentationsPlaying) {
    pause();
		QToolTip::add(  buttonPlayPause, tr( "Play" ) );
  }
  else {
    play();
		QToolTip::add(  buttonPlayPause, tr( "Pause" ) );
  }
}

void MultiVideoPlayerDialog::play(double nptStart, double nptEnd) {
  double longestDur = 0.0;

  dprintf_full("MultiVideoPlayerDialog::play()\n");
  wasPrefetching=true; //inital prefetching

  list <PresentationInfoBox *>::iterator v = videoBoxes.begin();
  list <Presentation *>::iterator r = presentations.begin();
  longestPresentation = (*r); //take first one as default (there might be wrong durations for MPEG1!)
  for(;r !=presentations.end(); ++r) {
    if (nptStart != 0.0 || nptEnd != 0.0)
      (*r)->setTimeRange(nptStart, nptEnd);
    double thisDur = (double)(*r)->getPresentationDurationMS() / 1000.0;
    if (nptEnd > 0.0 && thisDur > nptEnd - nptStart)
      thisDur = nptEnd - nptStart;
    if( longestDur < thisDur) {
      longestPresentation = (*r);
      longestDur = thisDur;
    }
    if(((*r)->getState() != Presentation::PRES_READYTOPLAY) &&
       ((*r)->getState() != Presentation::PRES_MUTED) &&
       ((*r)->getState() != Presentation::PRES_PAUSED)) {
      QMessageBox mess( "Cannot play...", "Presentation X is not ready to play, you did not load a file!"
			,QMessageBox::Information , QMessageBox::Ok
			,QMessageBox::NoButton , QMessageBox::NoButton , this );
      mess.show();
      return;
    }
  }
  globalSlider->setMaxValue((int)(longestDur * 1000.0));
  for(list <Presentation *>::iterator wp = presentations.begin();wp !=presentations.end(); wp++) {
    if((*wp)->getState() == Presentation::PRES_READYTOPLAY
       || (*wp)->getState() == Presentation::PRES_PAUSED) {
      (*wp)->start();
    }
    if((*v)->isMuted()) {
      dprintf_full("MultiVideoPlayerDialog::play -> Presentation is muted!\n");
      (*wp)->wait(200);
      (*wp)->mute((*wp)->getSessionKey(),true,false);
    }
    (*v)->deleteButton->setEnabled(false);
    (*v)->options->setItemEnabled(1,false);
    (*v)->options->setItemEnabled(3,false);
    v++;
  }
  boxValuesTimer->start(100);
  endOfPresentationTimer->start(450);
  presentationsPlaying = true;
  buttonStop->setEnabled(true);
  buttonPlayPause->setPixmap(pausePic);
  buttonNewRemoteWidget->setEnabled(false);
  buttonNewLocalWidget->setEnabled(false);
  fileMenu->setItemEnabled(fileMenu->idAt(0),false);
  fileMenu->setItemEnabled(fileMenu->idAt(1),false);
  fileMenu->setItemEnabled(fileMenu->idAt(2),false);
  fileMenu->setItemEnabled(fileMenu->idAt(4),false);
}

void MultiVideoPlayerDialog::pause() {
  boxValuesTimer->stop();
  endOfPresentationTimer->stop();
  for(list <Presentation *>::iterator w = presentations.begin();w !=presentations.end();++w) {
    (*w)->pause((*w)->getSessionKey());

  }

  presentationsPlaying = false;
  buttonPlayPause->setPixmap(playPic);

}

void MultiVideoPlayerDialog::stop() {
  dprintf_full("MultiVideoPlayerDialog::stop -> Stop was called\n");
  endOfPresentationTimer->stop();
  boxValuesTimer->stop();
  list <PresentationInfoBox *>::iterator v = videoBoxes.begin();
  for(list <Presentation *>::iterator w = presentations.begin();w !=presentations.end();++w) {
    if((*v)->isActive()) {
      (*v)->deleteButton->setEnabled(true);
    }
    (*w)->tearDown(TEARDOWN_ALL);
    (*v)->setVideoProgress(0);
    (*v)->setActualSecond(0);
    (*v)->resetABW();
    (*v)->resetFPS();
    (*v)->pDialog->resetABW();
    (*v)->pDialog->resetFPS();
    (*v)->pDialog->resetSeconds();
    (*v)->options->setItemEnabled(1,true);
    (*v)->options->setItemEnabled(3,true);
    v++;
  }
  /* wait until all data channels have exited, i.e. the globalTimer is used
   * only by this->gtSharedAdapt (gtRefCount->getUsage() == 1)
   */
  struct timespec tm;
  tm.tv_sec = 0;
  tm.tv_nsec = 100*1000*1000;	// 100 ms
  for (int i=0; i < 10; i++) {
    dprintf_full("MultiVideoPlayerDialog::stop gtRefCount %i\n",gtRefCount->getUsage());
    if (gtRefCount->getUsage() <= 1)
      break;
#ifdef WIN32
    msleep(tm.tv_nsec/1000000);
#else
    nanosleep(&tm, NULL);
#endif
  }
  globalTimer->reset();

  for(list <Presentation *>::iterator wI = presentations.begin();wI !=presentations.end();++wI) {
		if((*wI)->initalizeURI((*wI)->getURI()) ==-1) {
      QMessageBox mess( "Cannot play...",
												"Presentation X is not ready to play, you did not load a file!",
												QMessageBox::Information , QMessageBox::Ok,
												QMessageBox::NoButton , QMessageBox::NoButton , this );
      mess.exec();
      return;
    }
    (*wI)->setupURI();
  }

  globalSlider->setValue(0);
  presentationsPlaying = false;
  reinitializeVideoWindow(false);
  buttonNewRemoteWidget->setEnabled(true);
  buttonNewLocalWidget->setEnabled(true);
  buttonStop->setEnabled(false);
  buttonPlayPause->setEnabled(true);
  buttonPlayPause->setPixmap(playPic);
  //setFocus();
  buttonPlayPause->setFocus();
  QToolTip::add(  buttonPlayPause, tr( "Play" ) );
  fileMenu->setItemEnabled(fileMenu->idAt(0),true);
  fileMenu->setItemEnabled(fileMenu->idAt(1),true);
  fileMenu->setItemEnabled(fileMenu->idAt(2),true);
  fileMenu->setItemEnabled(fileMenu->idAt(4),true);

}


void MultiVideoPlayerDialog::mute_unmuteAudio() {
  if(activePresentation->getState() == Presentation::PRES_READYTOPLAY) {
    return;
  }
  if(activePresentation->getState() == Presentation::PRES_MUTED) {
    activePresentation->unmute(activePresentation->getSessionKey(),true,false);
  }
  else {
    activePresentation->mute(activePresentation->getSessionKey(),true,false);
  }
}


void MultiVideoPlayerDialog::moveInfoBoxes() {
  int wOfBoxBefore = 0;
  int i = 1;
  for (list <PresentationInfoBox *>::iterator v = videoBoxes.begin(); v != videoBoxes.end();v++) {
    (*v)->move(-boxScroll->value()+wOfBoxBefore,0);
    wOfBoxBefore = (*v)->x()+(*v)->width()+i+boxScroll->value();
    i++;
  }

  if((videoBoxes.size()*VIDEOBOXWIDTH <= (unsigned)Frame3->width()) && (boxScroll->value() == 0)) {
    //	boxScroll->setEnabled(false);

    boxScroll->hide();
  }
}
void MultiVideoPlayerDialog::updateBoxValues() {
  bool allStreamsPaused = true;


  list <PresentationInfoBox *>::iterator v = videoBoxes.begin();
  for(list <Presentation *>::iterator actPres = presentations.begin();actPres !=presentations.end();++actPres) {
    if((*actPres)->getState() != Presentation::PRES_READYTOPLAY) {
      (*v)->setFPS(	(*actPres)->getFramesPerSecond(1).front(),
										(*actPres)->getFramesPerSecond(5).front(),
										(*actPres)->getFramesPerSecond(10).front());
      (*v)->setABW( (*actPres)->getTotalAverageBandwidth(1),
										(*actPres)->getTotalAverageBandwidth(5),
										(*actPres)->getTotalAverageBandwidth(10));
      (*v)->pDialog->setFramesPerSecond( (*actPres)->getOrigFramesPerSecond(),
																				 (*actPres)->getFramesPerSecond(1),
																				 (*actPres)->getFramesPerSecond(5),
																				 (*actPres)->getFramesPerSecond(10));
      (*v)->pDialog->setAverageBandwidth( (*actPres)->getAverageBandwidth(1),
																					(*actPres)->getAverageBandwidth(5),
																					(*actPres)->getAverageBandwidth(10));


      if (!(*actPres)->isLocal() && !wasPrefetching && (*actPres)->isPrefetching()) {
				wasPrefetching = true;
				//inform DataChannel of prebuffering
				/*
				dprintf_full("MultiVideoPlayerDialog::updateBoxValues: inform DataChannel of prefetching\n");
				(*actPres)->pause(0);
				(*actPres)->play();
				*/
      }

      if (!(*actPres)->isLocal() && wasPrefetching && !(*actPres)->isPrefetching() ) {//((*actPres)->getState()==Presentation::PRES_PLAYING) ) {
				wasPrefetching = false;
				dprintf_full("MultiVideoPlayerDialog::updateBoxValues: DONE with prefetching\n");
      }

      int syncProgress;

      if ( ((*actPres)->isPrefetching()) &&
		    !( ((*actPres)->presInform.numberOfVES == 0 ) &&
		      ((*actPres)->presInform.numberOfAES == 0)) ) { //BUFFERING
				list <float> dummy; //fill array with -1
				for (uint i=0; i < (*actPres)->getActualPresentationInfo().size(); i++)
					dummy.push_front(-1);
				(*v)->pDialog->setActualSecond(dummy);
				syncProgress = -1;
				buttonPlayPause->setEnabled(false);
				buttonStop->setEnabled(false);
      } else { // NORMAL PLAYING
				(*v)->pDialog->setActualSecond((*actPres)->getActualSec());
				syncProgress = (int) (*actPres)->getActualSecOfLongestES()*1000;
				allStreamsPaused = false;
				buttonPlayPause->setEnabled(true);
				buttonStop->setEnabled(true);
      }

      if( ! (*actPres)->isLocal()) { // remote streaming
				(*v)->pDialog->setBufferFillLevel((*actPres)->getBufferFillLevel());
				(*v)->pDialog->setSecBufferFillLevel((*actPres)->getBufferFillLevelInSecs());
      }

      (*v)->setVideoProgress(syncProgress);
      (*v)->setActualSecond((int)floor(syncProgress/1000.0));

    }
    v++;
  }

  if (allStreamsPaused) {
    globalSlider->setValue(0);
    globalSlider->setEnabled(false);
  } else {

    globalSlider->setValue((int)(longestPresentation->getActualSecOfLongestES()*1000.0));
  }

}


void MultiVideoPlayerDialog::endOfPresentation() {
  if(!presentationsPlaying)
	  return;
  bool allPresentationsEnded = true;
  list <PresentationInfoBox *>::iterator v = videoBoxes.begin();
  for(list <Presentation *>::iterator p = presentations.begin(); p != presentations.end(); p++) {
    if(!(*p)->endOfPresentation()) {
      allPresentationsEnded = false;
    }
    else {
      if((*p)->getState() != Presentation::PRES_READYTOPLAY) {
			//	(*p)->clearPresentationLists();
				(*v)->setVideoProgress(0);
				(*v)->setActualSecond(0);
				(*v)->resetABW();
				(*v)->resetFPS();
				(*v)->pDialog->resetABW();
				(*v)->pDialog->resetFPS();
				(*v)->pDialog->resetSeconds();
				(*p)->tearDown(TEARDOWN_ALL);
				//#ifndef MEASUREMENT
				if(-1 == (*p)->initalizeURI( (*p)->getURI() )) {
				/*	deletePresentation();
					QMessageBox mess( "Error -1", "Connection to server/proxy lost.",
														QMessageBox::Information , QMessageBox::Ok
														, QMessageBox::NoButton , QMessageBox::NoButton , this );
					mess.exec();
					break;
				*/

				}
				else {
				(*p)->setupURI();
				(*p)->redrawPresentation((*v)->getBoxNumber());
				}
				//#endif
      }
    }

    v++;

  }
  if(allPresentationsEnded) {

    endOfPresentationTimer->stop();
    boxValuesTimer->stop();
    globalTimer->reset();
	if(exitafterplay) {
		exit(0);
	}
#ifdef MEASUREMENT

		//	exit(0);
#endif
    buttonStop->setEnabled(false);
    buttonPlayPause->setPixmap(playPic);
		QToolTip::add(  buttonPlayPause, tr( "Play" ) );
    buttonNewRemoteWidget->setEnabled(true);
    buttonNewLocalWidget->setEnabled(true);
    activeVideo->deleteButton->setEnabled(true);

    globalSlider->setValue(0);
    globalSlider->setTracking(false);
    presentationsPlaying = false;
    for(list <PresentationInfoBox *>::iterator v = videoBoxes.begin(); v != videoBoxes.end();v++) {
      (*v)->options->setItemEnabled(1,true);
      (*v)->options->setItemEnabled(2,true);
      (*v)->options->setItemEnabled(3,true);
    }

    fileMenu->setItemEnabled(fileMenu->idAt(0),true);
    fileMenu->setItemEnabled(fileMenu->idAt(1),true);
    fileMenu->setItemEnabled(fileMenu->idAt(2),true);
    fileMenu->setItemEnabled(fileMenu->idAt(4),true);
		if(loop) {
			play();
		}
  }
}

void MultiVideoPlayerDialog::keyPressEvent ( QKeyEvent * e ) {
  if(tolower(e->ascii()) == 'f') {
	  cout << "f pressed" << endl;
		fullScreen();
  }
  if(tolower(e->ascii()) == 0x1000) { //ESC
	  cout << "ESC pressed" << endl;
		stop();
	  exit(0);
  }
}

void MultiVideoPlayerDialog::askIfDeleteAllPresentations() {
  QMessageBox mess( "Delete all Presentations",
										"Do you really want to delete all Presentations?", QMessageBox::Information , QMessageBox::Yes
										, QMessageBox::No , QMessageBox::NoButton , this );
  mess.exec();
  if (mess.result() == QMessageBox::Yes) {
    deleteAllPresentations();
  }

}
void MultiVideoPlayerDialog::askIfDeletePresentation() {
  dprintf_full("MultiVideoPlayerDialog::deletePresentation -> Delete was pressed\n");
  QString str("Do you really want to delete Presentation ");
  str.append(QString::number(activeVideo->getBoxNumber()));
  str.append("?");
  QMessageBox mess( "Delete Presentation",str, QMessageBox::Information , QMessageBox::Yes
										, QMessageBox::No , QMessageBox::NoButton , this );
  mess.exec();
  if (mess.result() == QMessageBox::Yes) {
    deletePresentation();
  }
}

void MultiVideoPlayerDialog::deleteAllPresentations() {
  if (videoBoxes.empty()) {
    return;
  }
  list <Presentation *>:: iterator presI;
  list <PresentationInfoBox *>::iterator v;
  while(!videoBoxes.empty()) {
    presI = presentations.begin();
    v = videoBoxes.begin();
    (*v)->hide();
    (*v)->pDialog->close();
    (*presI)->clearPresentationLists();
    (*presI)->tearDown(TEARDOWN_ALL);
    presentations.pop_front();
    videoBoxes.pop_front();
  }

  buttonPlayPause->setEnabled(false);
  fileMenu->setItemEnabled(fileMenu->idAt(2),false);
  fileMenu->setItemEnabled(fileMenu->idAt(4),false);
  reinitializeVideoWindow(true);

}


void MultiVideoPlayerDialog::deletePresentation() {

  QString title;

  list <Presentation *>:: iterator presI = presentations.begin();
  for(list <PresentationInfoBox *>::iterator v = videoBoxes.begin();v != videoBoxes.end(); v++) {
    if( *v && (*v)->isActive()) {
      (*v)->hide();
      (*v)->pDialog->close();
      (*presI)->tearDown(TEARDOWN_ALL);
			delete (*presI);
      presentations.erase(presI);

      videoBoxes.erase(v);
      break;
    }

    presI++;
  }
  if(presentations.empty()) {
    buttonPlayPause->setEnabled(false);
    fileMenu->setItemEnabled(fileMenu->idAt(2),false);
    fileMenu->setItemEnabled(fileMenu->idAt(4),false);
  }
  reinitializeVideoWindow(true);
}

void MultiVideoPlayerDialog::reinitializeVideoWindow(bool presDeleted) {

	int widthDW = 0;
	int heightDW = 0;
	int tmpWidthDW = 0;
	int	tmpHeightDW = 0;
	int heightRow = 0;
	int widthThisRow = 0;
	int xPos = 0;

	list <Presentation *>:: iterator pI = presentations.begin();
	if(!presentations.empty())
		if(presentations.size() == 1)
			presentations.front()->theOnlyOnePresentation = true;
		else
			presentations.front()->theOnlyOnePresentation = false;

	int boxNum = 1;
	for(list <PresentationInfoBox *>::iterator w = videoBoxes.begin();w != videoBoxes.end(); w++) {
		if( w == videoBoxes.begin()) {
			if(presDeleted) {
				(*w)->setActive(true);
				(*pI)->setActive(true);
				(*w)->deleteButton->setEnabled(true);
				activeVideo = (*w);
				activePresentation = (*pI);
			}
		}


		if( (widthThisRow + (*pI)->getPresentationWidth()) > 1024) {
			(*pI)->setPresentationPosition(0,heightDW);
			heightRow = heightDW;
			heightDW = heightDW + (*pI)->getPresentationHeight();
			widthThisRow = (*pI)->getPresentationWidth();
		}

		else {
			tmpWidthDW = widthThisRow;
			tmpHeightDW = heightDW;
			if ( (widthThisRow + (*pI)->getPresentationWidth()) > (unsigned)widthDW)
				widthDW = widthThisRow + (*pI)->getPresentationWidth();
			widthThisRow = widthThisRow + (*pI)->getPresentationWidth();
			heightDW = MAX((unsigned)heightDW,heightRow+(*pI)->getPresentationHeight());
			(*pI)->setPresentationPosition(tmpWidthDW,heightRow);
		}



		if(presDeleted) {

      (*w)->setGeometry( QRect(xPos,0,VIDEOBOXWIDTH,Frame3->height()));
      (*w)->setBoxNumber(boxNum);

      if(videoBoxes.size()*VIDEOBOXWIDTH > (unsigned)Frame3->width()) {
				boxScroll->setEnabled(true);
				boxScroll->setMaxValue(VIDEOBOXWIDTH*videoBoxes.size()+videoBoxes.size()+0-Frame3->width());
				boxScroll->setValue(0);
      }
      else {
				boxScroll->hide();
				boxScroll->setMaxValue(0);
				boxScroll->setValue(0);
      }
      xPos += VIDEOBOXWIDTH+1;
			boxNum++;
    }
    pI++;


  }
	if(theapp.GetScreen() && SDL_MUSTLOCK(theapp.GetScreen())) {
		theapp.LockScreen();
	}
	bool noPresentation = false;
	if( (widthDW == 0) && (heightDW == 0) ) {
		widthDW = 1;
		heightDW = 1;
		noPresentation = true;
	}
	dprintf_full("MultiVideoPlayerDialog::reinitializeVideoWindow to new SDL %ix%i\n",widthDW,heightDW);
	if(noPresentation) {
		buttonNewLocalWidget->setFocus();
#ifdef WIN32
		theapp.InitScreen(widthDW,heightDW,0,SDL_ANYFORMAT | SDL_HWSURFACE | SDL_NOFRAME);
#else
		//theapp.InitScreen(352,288,16, SDL_HWSURFACE | SDL_HWPALETTE);
		theapp.InitScreen(352,288);
#endif
	}
	else {
		//		theapp.InitScreen(widthDW,heightDW,16, SDL_ANYFORMAT | SDL_HWSURFACE );
		theapp.InitScreen(widthDW,heightDW, 0, SDL_SWSURFACE);
	}
	if(theapp.GetScreen() && SDL_MUSTLOCK(theapp.GetScreen())) {
		theapp.UnlockScreen();
	}

	int presNr = 1;

	for(list <Presentation *>::iterator pres = presentations.begin(); pres != presentations.end();pres++) {
		(*pres)->movePresentation();
		(*pres)->redrawPresentation(presNr);
		presNr++;
	}
}


void MultiVideoPlayerDialog::resizeEvent ( QResizeEvent * e) {
  MultiVideoPlayerDialogBase::resizeEvent(e);

  int val = 0;
  if( 0 < (int) (videoBoxes.size()*VIDEOBOXWIDTH-Frame3->width()+15)) {
    val = videoBoxes.size()*VIDEOBOXWIDTH-Frame3->width()+15;
  }
	//  int temp = boxScroll->maxValue();
  boxScroll->setMaxValue(val);
  if(videoBoxes.size()*VIDEOBOXWIDTH > (unsigned)Frame3->width()) {
    boxScroll->show();
    boxScroll->setEnabled(true);
  }

  else {
    int val = 0;
    if( 0 < (int) (videoBoxes.size()*VIDEOBOXWIDTH-Frame3->width()+15)) {
      val = videoBoxes.size()*VIDEOBOXWIDTH-Frame3->width()+15;
    }

    int temp = boxScroll->maxValue();
    boxScroll->setMaxValue(val);

    if(videoBoxes.size()*VIDEOBOXWIDTH > (unsigned)Frame3->width()) {
      boxScroll->show();
      boxScroll->setEnabled(true);
    }
    boxScroll->setValue(boxScroll->value()-(boxScroll->maxValue()-temp));
  }
  moveInfoBoxes();

}
