 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	MultiVideoPlayerDialog.hpp                                    *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef MultiVideoPlayerDialog_hpp
#define MultiVideoPlayerDialog_hpp

#include "MultiVideoPlayerDialogBase.hpp"
#include "GlobalOptionsImpl.hpp"

class PresentationInfoBox;
#include <qmainwindow.h>
#include <qslider.h>
#include <qscrollbar.h>
#include <qpixmap.h>


#include "Presentation.hpp"
#include "RemotePresentation.hpp"
#include "VideoWindow.hpp"
#include "RemoteDialog.hpp"
#include "ReferenceCounter.hpp"
#include "adaptors/SharedAdaptor.hpp"
#include "adaptors/GlobalTimer.hpp"


#define GLOBALOPTIONSFILE ".muViPlayerrc"
#define URISFILE ".muViPlayerURIrc"

#define VIDEOBOXWIDTH 180
#define MAXVIDEOWINDOWWIDTH 1000

typedef struct GlobalConfig {
	char pathToContainerFiles[MAX_STR_LEN];
	char proxyName[MAX_STR_LEN];
	char configFile[MAX_STR_LEN];
	int proxyPort;
	char iconPath[MAX_STR_LEN];
	GlobalConfig() {
		proxyPort = 0;
	}
} GlobalConfig;

class GlobSlider : public QSlider {
	Q_OBJECT
public:
	GlobSlider( QWidget * parent, const char * name=0 );
	~GlobSlider();

protected:
	virtual void mousePressEvent ( QMouseEvent * );
};



/**
 *
 * <Class Description goes here>
 *
 *@author Ronald Sowa
 *
 *@version $Id: MultiVideoPlayerDialog.hpp,v 1.13 2006/02/12 10:36:05 klschoef Exp $
 *
 *@short <short description>
 *
 */
class MultiVideoPlayerDialog : public MultiVideoPlayerDialogBase
{
	Q_OBJECT
public:
	MultiVideoPlayerDialog( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags f = 0 );
	~MultiVideoPlayerDialog();

	void loadGlobalConfig(const char * configFile);
	list <PresentationInfoBox *> videoBoxes;
	bool commandLine;
	QString * profileName;
	QString * fileName;
        /** Play all presentations within the given media time range.
	 * @param nptStart    presentation start time in seconds,
	 *                    0.0 means media start.
	 * @param nptEnd      presentation end time in seconds,
	 *                    0.0 means media end.
	 */
	void play(double nptStart = 0.0, double nptEnd = 0.0);
	void pause();
	void enableLooping(bool loop);
	void exitAfterPlay();

private:
	
	void open_input();
	RemoteDialogBase * readURIsFromFile();
	void writeURIsToFile(RemoteDialogBase * rDialog);
	QString getAdaptedFile(list <QString> adaptors, const char * origURI);

	void resizeEvent ( QResizeEvent * e);
	QString * last_opened;
	bool presentationsPlaying;
	PresentationInfoBox * activeVideo;
	Presentation * activePresentation;
	Presentation * longestPresentation;
	u32 windowWidth;
	u32 windowHeight;
	u32 totWidth;
	u32 totHeight;
	int oldWidth;
	bool loop;
	bool exitafterplay;
	QPixmap playPic;
	QPixmap pausePic;
	QPixmap stopPic;
	GlobalConfig * glConfig;
	ReferenceCounter * gtRefCount;
	SharedAdaptor *gtSharedAdapt;
	GlobalTimer *globalTimer;

	VideoWindow theapp;

public slots:

	void play_pause();
	void stop();
	void fullScreen();
	void load_config();
	void save_config();
	void new_localWidget();
	void new_remoteWidget();
	void change_activeWidget();
	void termCapsDialog();
	void presentationProperties();
	void globalOptionsDialog();
	void handleVideoBoxPopup(int i);
	void mute_unmuteAudio();
	void setWindowedMode();
	void moveInfoBoxes();
	void updateBoxValues();
	
	void endOfPresentation();

	void askIfDeleteAllPresentations();
	void askIfDeletePresentation();
	void about();
	void quit();

        /** Create a new presentation from a local SDP file.
	 * Recognized SDP fields:<br>
	 * <ul>
	 * <li><code>v=0</code> required to start a valid SDP description.</li>
	 * <li><code>s=_session_name_</code> session name; required to start a session description.</li>
	 * <li><code>a=control:_URL_</code> media URL.</li>
	 * <li><code>a=range:npt=_start_-_end_</code> media presentation time range in seconds
	 *     (floating point).</li>
	 * </ul>
	 * @return <code>false</code> if the given file does not contain a valid
	 *         SDP description.
	 */
        bool newPresentationFromSDP(const QString &file);  

protected:
	GlobSlider * slid;
	QPopupMenu *fileMenu;
	GlobalOptions * glOpts;
	void keyPressEvent ( QKeyEvent * e );
	list <Presentation *> presentations;
	list <QString> adaptorList;
	void initMenuBar();
        /** Create new presentation. */
	Presentation *setNewWidget(bool localWidget);
	void reinitializeVideoWindow(bool presDeleted);
	void deleteAllPresentations();
	void deletePresentation();
	void save_globalConfig();
	bool firstTime;
	
	int nrOfWidgets;
	int lastActive;
	QTimer *boxValuesTimer;
	QTimer *endOfPresentationTimer;
	bool profileLoading;
	bool wasPrefetching;
	QScrollBar * boxScroll;
};

#endif
