/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	Presentation.cpp                                              *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "Presentation.hpp"

#include "DataChannel.hpp"
#include "ReferenceCounter.hpp"
#include "adaptors/GlobalTimer.hpp"
#include "adaptors/ESSynchronizer.hpp"
#include "DataSink.hpp"
#include "VideoDisplayIO.hpp"


Presentation::Presentation(u32 x_inSDL, u32 y_inSDL, ReferenceCounter * gtRef, GlobalTimer * gt)

{
  activePresentation = true;
  presInform.localPresentation = true;
	gtRefCount = gtRef;
  globalTimer = gt;
  theOnlyOnePresentation = false;
  nptStart = 0.0;
  nptEnd = 0.0;
};

Presentation::~Presentation() {
  list <presentationManagement *>::iterator pI = presManage.begin();
  while(!presManage.empty()) {
    pI = presManage.begin();
    if(*pI)
      delete *pI;
    presManage.pop_front();
  }
};

int Presentation::initalizeURI(const char * URI) { 
  return 0;
};
void Presentation::setupURI() {};

const ContainerInfo * Presentation::getContainerInfo() {
  return mp4Stream;
}
const list <DataChannel *>  Presentation::getDataChannels() {
  list <DataChannel *> channels;
  for(list <presentationManagement *>::iterator pM = presManage.begin(); pM != presManage.end(); pM++) {
    channels.push_back((*pM)->dChannel);
  }
  return channels;
}
const char * Presentation::getURI() const{
  return uri;
}

const char * Presentation::getPresentationName() {
  return "implement in subclasses";
}

DataChannel * Presentation::getDataChannel (int esId) {
  return NULL;
}

void Presentation::setPresentationPosition(u32 xpos, u32 ypos) {
  x_inSDL = xpos;
  y_inSDL = ypos;
	
}


void Presentation::movePresentation() {
  u32 pos = x_inSDL;
  for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
    if((*pm)->videoOutput) {
      (*pm)->videoOutput->MoveWidget(pos,y_inSDL);
      pos += (*pm)->videoOutput->Width();
    }
  }
}

void Presentation::hidePresentation() {
  for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
    if((*pm)->videoOutput) {
      (*pm)->videoOutput->Hide();
    }
  }
		
		
	
}
void Presentation::showPresentation() {
  for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
    if((*pm)->videoOutput) {
      (*pm)->videoOutput->Show();
    }
  }
}

void Presentation::redrawPresentation(int presNr) {
	
	
  for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
    if((*pm)->videoOutput) {
      if(!(*pm)->videoOutput->IsVisible()) {
	(*pm)->videoOutput->Show();
      }
      QString presString("PRESENTATION #");
      QString presNumber;
      presString.append(presNumber.setNum(presNr));
      (*pm)->videoOutput->SetFontSize(12);
      (*pm)->videoOutput->DrawText(20,20,(const char *) presString.latin1());
	  if(this->isActive() && !theOnlyOnePresentation) {
		(*pm)->videoOutput->drawFrame(true);		
	  }
	  else
		(*pm)->videoOutput->drawFrame(false);	
      (*pm)->videoOutput->Update();
	  	

    }
  }

		
}
void Presentation::resizePresentation(int w, int h) {
  for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
	  if((*pm)->videoOutput) {
		(*pm)->videoOutput->setDimension(w,h);
		(*pm)->videoOutput->Update();
	  }
  }
		
}

int Presentation::keepAspectRatio(int height) {
  int realWidth = 0;
  for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
	  if((*pm)->videoOutput) {
		  float aRatio = ((VideoESInfo *) ((*pm)->esInfo))->getAspectRatio(); 
		realWidth = (int)(ceil((float)height * aRatio));  
	  }
  }
  return realWidth;		
}

void Presentation::clearPresentationLists() {
  list <presentationManagement *>::iterator pI;
  while(!presManage.empty()) {
    pI = presManage.begin();
    if((*pI)->dChannel) {
      delete (*pI)->dChannel;
    }
    if((*pI)->esInfo) {
      delete (*pI)->esInfo;
    }
    presManage.pop_front();
  }
  presManage.clear();
  list <ESInfo *>::iterator eI;
  while(!origESInfo.empty()) {
    eI = origESInfo.begin();
    if(*eI) {
      delete (*eI);
    }
    origESInfo.pop_front();
  }
  while(!actESInfo.empty()) {
    eI = actESInfo.begin();
    if(*eI) {
      delete (*eI);
    }
    actESInfo.pop_front();
  }
  origESInfo.clear();
  actESInfo.clear();
  lastX = 0;
  lastY = 0;
			
}

u32 Presentation::getPresentationWidth() {
  return pWidth;
}
u32 Presentation::getPresentationHeight() {
  return pHeight;
}

u32 Presentation::getPresentationXpos() {
  return x_inSDL;
}
u32 Presentation::getPresentationYpos() {
  return y_inSDL;
}
u32 Presentation::getSessionKey() {
  return sessionKey;
}

void Presentation::setActive(bool active) {
  bool act = active;
  activePresentation = active;
   for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
		  if((*pm)->videoOutput) {
				if(!theOnlyOnePresentation) {
					(*pm)->videoOutput->drawFrame(act);		
				}
				else
					(*pm)->videoOutput->drawFrame(false);
				(*pm)->videoOutput->Update();
		  }
	  }
}
	
bool Presentation::isActive() {
  return activePresentation;
}

bool Presentation::isLocal() {
  return localPres;
}

	
list <ESInfo *> Presentation::getOrigPresentationInfo() {
  return origESInfo;
}

list <ESInfo *> Presentation::getActualPresentationInfo() {
  return actESInfo;
}
int Presentation::getPresentationDurationMS() {

  return duration;
}

int Presentation::getTotalAverageBandwidth(u32 interval) {
  return -10;
}

list <float> Presentation::getFramesPerSecond(u32 interval) {
  list <float>  l;
  return l;
}

list <float> Presentation::getOrigFramesPerSecond() {
  list <float>  l;
  return l;
}

list <int> Presentation::getAverageBandwidth(u32 interval) {
  list <int>  l;
  return l;
};

list <float> Presentation::getActualSec() {
  list <float>  l;
  return l;
};

float Presentation::getActualSecOfLongestES() {
  return 0;
};

list <int> Presentation::getBufferFillLevel() {
  list <int> l;
  return l;
}

list <double> Presentation::getBufferFillLevelInSecs() {
  list <double> l;
  return l;
}

bool Presentation::isLoadCritical() {
  // just an experiment
  /*	if(!presManage.empty()) {
	for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
	if((*pm)->esSync->cpuLoadCritical)
	return true;
	}
	}
  */
  return false;
}


bool Presentation::endOfPresentation() {
  for(list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
    if(presState == PRES_READYTOPLAY) {
      return true;
    }
	if((*pm)->dChannel->running()) {
		return false;
	}
  /*  if((*pm)->audioOutput) {
      dprintf_full("Presentation::endOfPresentation checking audio state %i\n",(*pm)->audioOutput->getState());
      if((*pm)->audioOutput->getState() != IO::CLOSED) {
	return false;
      }
    }
		
    if((*pm)->videoOutput) {
      dprintf_full("Presentation::endOfPresentation checking video state %i\n",(*pm)->videoOutput->getState());
      if((*pm)->videoOutput->getState() != IO::CLOSED) {
	return false;
      }
    }
	*/
  }
	
  return true;
}

void Presentation::setTimeRange(double starttime, double endtime)
{
  nptStart = starttime;
  nptEnd = endtime;
}
