/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	PresentationInfoBox.cpp                                       *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/


#include <qpopupmenu.h>
#include <iostream>
#include "PresentationInfoBox.hpp"
#include "VideoESInfo.hpp"
#include <qcolor.h>
#include <qstring.h>
#include <qtooltip.h>
#include <qpixmap.h>
#include <qpoint.h>
#include <qsemimodal.h>

#include<qlayout.h>

ExtQLabel::ExtQLabel(const QString & text, QWidget * parent, const char * name, WFlags f ) : 
	QLabel(text, parent, name, f) {

}

ExtQLabel::~ExtQLabel() {
}

void ExtQLabel::mousePressEvent(QMouseEvent * e) {

	setBackgroundColor(QColor(128,128,128));
	emit pressed();
	if(e->button() == RightButton)
		emit rightPressed();
}

ExtQSlider::ExtQSlider(QWidget * parent,const char * name) : 
	QSlider(parent,name) {
	setTracking(false);
}

ExtQSlider::~ExtQSlider() {
}

void ExtQSlider::mousePressEvent(QMouseEvent * e) {

	setBackgroundColor(QColor(128,128,128));
	emit pressed();
	if(e->button() == RightButton)
		emit rightPressed();
}


PresentationInfoBox::PresentationInfoBox(int boxNumber,bool local,const QString & title,QWidget * parent, const char * name) :
	QGroupBox(title,parent, name) {
	this->boxNumber = boxNumber;
	this->localBox = local;
	setBackgroundColor(QColor(128,128,128));
	active = true;
	newActive = false;
 	muted = false;
 	speakerShowedFirstTime = true;
	unmutedPicLoaded = true;
	value_dimX = NULL;
	value_dimY = NULL;
	value_fps1 = NULL;
	value_fps5 = NULL;
	value_fps10 = NULL;
	value_durationMS = NULL;
	value_aBW1 = NULL;
	value_aBW5 = NULL;
	value_aBW10 = NULL;
	progress = 0;
	actualSecond = 0;
}

PresentationInfoBox::~PresentationInfoBox() {
	pDialog->hide();
	pDialog->close();
	delete pDialog;
}

void PresentationInfoBox::initBoxElements(GlobalConfig *glConfig) {

	options = new QPopupMenu(this,"Options");
	options->insertItem("Presentation Properties",0);
	if(isLocal()) {
		options->insertItem("Add/Remove Adaptors",1);
	}
	else {
		options->insertItem("Select Terminal Capabitlities",1);
	}
	options->insertSeparator(2);
	options->insertItem("Delete Presentation",3);
	options->setGeometry(parentWidget()->x()+x()+15,parentWidget()->y()+y()+20,100,80);
	setColumnLayout(0, Qt::Vertical );
	layout()->setSpacing( 0 );
	layout()->setMargin( 0 );
	GroupBox1Layout = new QGridLayout( this->layout() );
	GroupBox1Layout->setAlignment( Qt::AlignTop );
	GroupBox1Layout->setSpacing( 6 );
	GroupBox1Layout->setMargin( 11 );
	Layout8 = new QHBoxLayout; 
	Layout8->setSpacing( 6 );
	Layout8->setMargin( 0 );
	
	videoProgress = new ExtQSlider( this, "Slider1" );
	videoProgress->setOrientation( QSlider::Horizontal );
	Layout8->addWidget( videoProgress );
	
	speakerButton = new QPushButton( this, "speakerButton" );
	speakerButton->setMinimumSize( QSize( 20, 22 ) );
	speakerButton->setMaximumSize( QSize( 20, 22 ) );
	speakerButton->setText( tr( "SP" ) );
	char pixmap[MAX_STR_LEN];
	sprintf(pixmap,"%s/%s",glConfig->iconPath, "speaker.bmp");
	if(!unmutedPic.load(pixmap)) {
		
		dprintf_full("PresentationInfoBox::Constructor -> Failed to load unmuted BITMAP\n");
	}
	sprintf(pixmap,"%s/%s",glConfig->iconPath, "mutedsp.bmp");
	if(!mutedPic.load(pixmap)) {
		dprintf_full("PresentationInfoBox::Constructor -> Failed to load muted BITMAP\n");
	}
	speakerButton->setPixmap(unmutedPic);
	Layout8->addWidget( speakerButton );
	
	GroupBox1Layout->addLayout( Layout8, 6, 0 );
	QSpacerItem* spacer_2 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
	GroupBox1Layout->addItem( spacer_2, 4, 0 );
	
	Layout1 = new QHBoxLayout;
	Layout1->setSpacing( 6 );
	Layout1->setMargin( 0 );
	
	
	source = new ExtQLabel("bla",this,"sourceLabel");
	source->setText( tr( "Source:  N/A" ) );
	Layout1->addWidget( source );
	GroupBox1Layout->addLayout(Layout1,0,0);
	
	
	
	Layout10 = new QHBoxLayout; 
	Layout10->setSpacing( 6 );
	Layout10->setMargin( 0 );
	
	
	durationMS = new ExtQLabel( "bla",this, "durationLabel" );
	durationMS->setText( tr( "Duration:" ) );
	Layout10->addWidget( durationMS );
	
	value_durationMS = new ExtQLabel( "bla",this, "durationValue" );
	value_durationMS->setText( tr( "N/A" ) );
	value_durationMS->setAlignment( int( QLabel::AlignCenter ) );
	Layout10->addWidget( value_durationMS );

	GroupBox1Layout->addLayout( Layout10, 2, 0 );
	Layout9 = new QHBoxLayout; 
	Layout9->setSpacing( 6 );
	Layout9->setMargin( 0 );

	dimension = new ExtQLabel( "bla",this, "dimensionLabel" );
	dimension->setText( tr( "Dimension:" ) );
	Layout9->addWidget( dimension );

	value_dimX = new ExtQLabel( "bla",this, "dimensionValue" );
	value_dimX->setText( tr( "N/A" ) );
	value_dimX->setAlignment( int( QLabel::AlignCenter ) );
	Layout9->addWidget( value_dimX );

	GroupBox1Layout->addLayout( Layout9, 1, 0 );

	Layout19 = new QGridLayout; 
	Layout19->setSpacing( 6 );
	Layout19->setMargin( 0 );

	value_aBW1 = new ExtQLabel( "bla",this, "avbwValue5" );
	value_aBW1->setText( tr( "0" ) );
	value_aBW1->setAlignment( int( QLabel::AlignCenter ) );

	Layout19->addWidget( value_aBW1, 1, 1 );

	value_aBW10 = new ExtQLabel( "bla",this, "avbwValue10" );
	value_aBW10->setText( tr( "0" ) );
	value_aBW10->setAlignment( int( QLabel::AlignCenter ) );

	Layout19->addWidget( value_aBW10, 1, 3 );

	value_fps5 = new ExtQLabel( "bla",this, "fpsValue5" );
	value_fps5->setText( tr( "0" ) );
	value_fps5->setAlignment( int( QLabel::AlignCenter ) );

	Layout19->addWidget( value_fps5, 0, 2 );

	avBandwidth = new ExtQLabel( "bla",this, "avbwLabel" );
	avBandwidth->setText( tr( "avBW:" ) );

	Layout19->addWidget( avBandwidth, 1, 0 );

	value_fps10 = new ExtQLabel( "bla",this, "fpsValue10" );
	value_fps10->setText( tr( "0" ) );
	value_fps10->setAlignment( int( QLabel::AlignCenter ) );

	Layout19->addWidget( value_fps10, 0, 3 );

	value_fps1 = new ExtQLabel( "bla",this, "fpsValue1" );
	value_fps1->setText( tr( "0" ) );
	value_fps1->setAlignment( int( QLabel::AlignCenter ) );

	Layout19->addWidget( value_fps1, 0, 1 );

	framesPerSec = new ExtQLabel( "bla",this, "fpsLabel" );
	framesPerSec->setText( tr( "FPS:" ) );

	Layout19->addWidget( framesPerSec, 0, 0 );

	value_aBW5 = new ExtQLabel( "bla",this, "avbwValue1" );
	value_aBW5->setText( tr( "0" ) );
	value_aBW5->setAlignment( int( QLabel::AlignCenter ) );

	Layout19->addWidget( value_aBW5, 1, 2 );

	GroupBox1Layout->addLayout( Layout19, 3, 0 );
	deleteButton = new QPushButton(this,"Delete Presentation");
	deleteButton->setGeometry(165,0,16,16);
	deleteButton->setText("x");
	deleteButton->show();
	setActive(true);

	/************************************/
	connect(value_durationMS,SIGNAL(pressed()),this,SLOT(emitActive()));
	connect(value_durationMS,SIGNAL(rightPressed()),this,SLOT(showOptions()));
	connect(value_fps1,SIGNAL(pressed()),this,SLOT(emitActive()));
	connect(value_fps1,SIGNAL(rightPressed()),this,SLOT(showOptions()));
	connect(value_fps5,SIGNAL(pressed()),this,SLOT(emitActive()));
	connect(value_fps5,SIGNAL(rightPressed()),this,SLOT(showOptions()));
	connect(value_fps10,SIGNAL(pressed()),this,SLOT(emitActive()));
	connect(value_fps10,SIGNAL(rightPressed()),this,SLOT(showOptions()));
	connect(value_aBW1,SIGNAL(pressed()),this,SLOT(emitActive()));
	connect(value_aBW1,SIGNAL(rightPressed()),this,SLOT(showOptions()));
	connect(value_aBW5,SIGNAL(pressed()),this,SLOT(emitActive()));
	connect(value_aBW5,SIGNAL(rightPressed()),this,SLOT(showOptions()));
	connect(value_aBW10,SIGNAL(pressed()),this,SLOT(emitActive()));
	connect(value_aBW10,SIGNAL(rightPressed()),this,SLOT(showOptions()));
	connect(value_dimX,SIGNAL(pressed()),this,SLOT(emitActive()));
	connect(value_dimX,SIGNAL(rightPressed()),this,SLOT(showOptions()));
	connect(source,SIGNAL(pressed()),this,SLOT(emitActive()));
	connect(source,SIGNAL(rightPressed()),this,SLOT(showOptions()));


	pDialog = new PropertiesDialog(localBox,0,"Presentation Properties");
	pDialog->setGeneralProperties(boxNumber);
	connect(durationMS,SIGNAL(pressed()),this,SLOT(emitActive()));
	connect(durationMS,SIGNAL(rightPressed()),this,SLOT(showOptions()));
	connect(dimension,SIGNAL(pressed()),this,SLOT(emitActive()));
	connect(dimension,SIGNAL(rightPressed()),this,SLOT(showOptions()));
	connect(videoProgress,SIGNAL(pressed()),this,SLOT(emitActive()));
	connect(videoProgress,SIGNAL(rightPressed()),this,SLOT(showOptions()));
	connect(framesPerSec,SIGNAL(pressed()),this,SLOT(emitActive()));
	connect(framesPerSec,SIGNAL(rightPressed()),this,SLOT(showOptions()));
	connect(avBandwidth,SIGNAL(pressed()),this,SLOT(emitActive()));
	connect(avBandwidth,SIGNAL(rightPressed()),this,SLOT(showOptions()));
	connect(speakerButton,SIGNAL(clicked()),this,SLOT(mute_unmute()));
	connect(speakerButton,SIGNAL(clicked()),this,SLOT(emitActive()));
}

void PresentationInfoBox::emitActive() {
	setBackgroundColor(QColor(128,128,128));
	newActive = true;
	emit pressed();
}

void PresentationInfoBox::setBoxNumber(int num) {
	boxNumber = num;
}

int PresentationInfoBox::getBoxNumber() {
	return boxNumber;
}
void PresentationInfoBox::showOptions() {
	options->exec(this->mapToGlobal(QPoint(5,15)));
}

void PresentationInfoBox::openProperties() {
	pDialog->show();
}

void PresentationInfoBox::mousePressEvent(QMouseEvent * e) {

	setBackgroundColor(QColor(128,128,128));
	newActive = true;
	emit pressed();
	if(e->button() == RightButton) {
		showOptions();
	}
}

void PresentationInfoBox::setActive(bool act) {
	if(act) {
		setBackgroundColor(QColor(128,128,128));
		dimension->setBackgroundColor(QColor(128,128,128));
		framesPerSec->setBackgroundColor(QColor(128,128,128));
		durationMS->setBackgroundColor(QColor(128,128,128));
		videoProgress->setBackgroundColor(QColor(128,128,128));
		avBandwidth->setBackgroundColor(QColor(128,128,128));
		source->setBackgroundColor(QColor(128,128,128));
		if(value_dimX)
			value_dimX->setBackgroundColor(QColor(128,128,128));
		if(value_durationMS)
			value_durationMS->setBackgroundColor(QColor(128,128,128));
		if(value_fps1) {
			value_fps1->setBackgroundColor(QColor(128,128,128));
			value_fps5->setBackgroundColor(QColor(128,128,128));
			value_fps10->setBackgroundColor(QColor(128,128,128));
		}

		if(value_aBW1) {
			value_aBW1->setBackgroundColor(QColor(128,128,128));
			value_aBW5->setBackgroundColor(QColor(128,128,128));
			value_aBW10->setBackgroundColor(QColor(128,128,128));
		}
	}

	else {
		setBackgroundColor(parentWidget()->backgroundColor());
		dimension->setBackgroundColor(parentWidget()->backgroundColor());
		videoProgress->setBackgroundColor(parentWidget()->backgroundColor());
		framesPerSec->setBackgroundColor(parentWidget()->backgroundColor());
		avBandwidth->setBackgroundColor(parentWidget()->backgroundColor());
		durationMS->setBackgroundColor(parentWidget()->backgroundColor());
		source->setBackgroundColor(parentWidget()->backgroundColor());
		if(value_dimX)
			value_dimX->setBackgroundColor(parentWidget()->backgroundColor());
		if(value_dimY)
			value_dimY->setBackgroundColor(parentWidget()->backgroundColor());
		if(value_durationMS)
			value_durationMS->setBackgroundColor(parentWidget()->backgroundColor());
		if(value_fps1) {
			value_fps1->setBackgroundColor(parentWidget()->backgroundColor());
			value_fps5->setBackgroundColor(parentWidget()->backgroundColor());
			value_fps10->setBackgroundColor(parentWidget()->backgroundColor());
		}
		if(value_aBW1) {
			value_aBW1->setBackgroundColor(parentWidget()->backgroundColor());
			value_aBW5->setBackgroundColor(parentWidget()->backgroundColor());
			value_aBW10->setBackgroundColor(parentWidget()->backgroundColor());
		}
	}
	active = act;
}

void PresentationInfoBox::setVideoInfo(list <ESInfo *> info) {
	

	int pWidth = 0;
	int pHeight = 0;
	int nrVideos = 0;
	int nrAudios = 0;
	int durationMS = 0;
	int fps = 0;
	int aBW = 0;
	bool firstVES = true;
	if (localBox) {
		source->setText("Source: local");
	}
	else {
		source->setText("Source: network");
	}
	for(list <ESInfo *>::iterator esI = info.begin(); esI != info.end(); esI++) {
		if ((*esI)->isVisualStream()) {
			nrVideos++;
			pWidth += ((VideoESInfo*)(*esI))->getWidth();
			pHeight = MAX((u32)pHeight, ((VideoESInfo*)(*esI))->getHeight());
			if (firstVES) {
				
				fps = (int) (((double) ((*esI)->getNumberOfMediaSamples()) * 1000) /
										 (*esI)->getDurationInMs());
				aBW = (*esI)->getAvgBandwidth();
				firstVES = false;
			}
			durationMS = MAX(durationMS,(*esI)->getDurationInMs());
			
		}
		if ((*esI)->isAudioStream()) {
			nrAudios++;
			durationMS = MAX(durationMS,(*esI)->getDurationInMs());
		}
	}
	
	if(nrAudios == 0) {
		speakerButton->hide();
		
	}

	videoProgress->setMaxValue(durationMS);
	if(nrVideos == 0) {
		value_dimX->setText("N/A");
	} 
	else {
		QString * dimX = new QString();
		dimX->sprintf("%i  x  %i",pWidth,pHeight);
		value_dimX->setText(*dimX);
	}


}
bool PresentationInfoBox::isActive() {
	return active;
}

void PresentationInfoBox::setLocal(bool local) {
	localBox = local;
}

bool PresentationInfoBox::isLocal() {
	return localBox;
}


void PresentationInfoBox::setActualSecond(int second) {
	if(value_durationMS) {
		QString duration;
		int max_dur = videoProgress->maxValue();
		if (second >= 0) {
			duration.sprintf("%02i:%02i of %02i:%02i",second/60,second%60,max_dur/1000/60,max_dur/1000%60);
			value_durationMS->setText(duration);
		} else {
			duration = tr("BUFFERING");
			value_durationMS->setText(duration);
		}
		actualSecond = second;
	}
}
void PresentationInfoBox::setVideoProgress(int timeVal) {

	if (timeVal >= 0) {
		videoProgress->setEnabled(true);
	} else {
		videoProgress->setEnabled(false);
	}
	videoProgress->setValue(timeVal);
	progress = timeVal;
}

void PresentationInfoBox::resetFPS() {
	value_fps1->setNum(0);
	value_fps5->setNum(0);
	value_fps10->setNum(0);
}

void PresentationInfoBox::resetABW() {
	value_aBW1->setNum(0);
	value_aBW5->setNum(0); 
	value_aBW10->setNum(0); 
}

void PresentationInfoBox::setFPS(float fps1, float fps5, float fps10) {
  if(value_fps1) {
    QString qs;
    qs.sprintf("%2.1f",fps1);
    value_fps1->setText(qs);
    qs.sprintf("%2.1f",fps5);
    value_fps5->setText(qs);
    qs.sprintf("%2.1f",fps10);
    value_fps10->setText(qs);
    actualFPS = (int)fps1;
  }
}

void PresentationInfoBox::setABW(int aBW1, int aBW5, int aBW10) {
	if(value_aBW1) {
		value_aBW1->setNum(aBW1/128); // = * 8 / 1024
		value_aBW5->setNum(aBW5/128); // = * 8 / 1024
		value_aBW10->setNum(aBW10/128); // = * 8 / 1024

	}
}

void PresentationInfoBox::mute_unmute() {

	if(muted)
		muted = false;
	else
		muted = true;

	if(unmutedPicLoaded) {
		speakerButton->setPixmap(mutedPic);
		unmutedPicLoaded = false;
	}
	else {
		speakerButton->setPixmap(unmutedPic);
		unmutedPicLoaded = true;
	}
}

