 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	PresentationInfobox.hpp                                       *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef PresentationInfoBox_hpp
#define PresentationInfoBox_hpp

#include <qgroupbox.h>
#include <qtabwidget.h>
#include <qtabbar.h>
#include <qlabel.h>
#include <qslider.h>
#include <qpushbutton.h>
#include "ESInfo.hpp"
#include "PropertiesDialog.hpp"
#include "MultiVideoPlayerDialog.hpp"
class QVBoxLayout; 
class QHBoxLayout; 
class QGridLayout; 

/* Extended class to QLabel which receives a mousePressEvent
   and reacts by changing its background color
*/
class ExtQLabel : public QLabel {
	Q_OBJECT
public:
	ExtQLabel(const QString & text, QWidget * parent, const char * name=0, WFlags f=0 );
	~ExtQLabel();
signals:
	void pressed();
	void rightPressed();
protected:
	virtual void mousePressEvent ( QMouseEvent * );
};


/* Extended class to QSlider which receives a mousePressEvent
   and reacts by changing its background color
*/
class ExtQSlider : public QSlider {
	Q_OBJECT
public:
	ExtQSlider( QWidget * parent, const char * name=0 );
	~ExtQSlider();
signals:
	void pressed();
	void rightPressed();
protected:
	virtual void mousePressEvent ( QMouseEvent * );
};

/**
 *
 * Extended class to QGroupBox which receives a mousePressEvent
 * and reacts by changing its backgroundcolor
 *
 *@author Ronald Sowa
 *
 *@version $Id: PresentationInfoBox.hpp,v 1.5 2004/12/27 14:53:40 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class PresentationInfoBox : public QGroupBox {
	Q_OBJECT
public:
	
	/* Constructor 
	*/
	PresentationInfoBox(int boxNumber,bool local,const QString & title,QWidget * parent=0, const char * name=0  );

	/* Destructor
	*/
	~PresentationInfoBox();

	void initBoxElements(GlobalConfig *glConfig);
	
	int getBoxNumber();
	void setBoxNumber(int num);
	/* sets the PresentationInfoBox active/inactive */
	void setActive(bool act);

	/* returns true if PresentationInfoBox is active, otherwise false */
	bool isActive();

	/* used for adding a new PresentationInfoBox to a list of PresentationInfoBoxes */
	bool newActive;

	/* sets infos about the Video-ES of one presentation */
	void setVideoInfo(list <ESInfo *> info);

	/* set the ActiveVideBox local/remote */
	void setLocal(bool local);

	/* return true if PresentationInfoBox is local, otherwise false */
	bool isLocal();


	bool isMuted() {
		return muted;
	}
	/* sets the the number of seconds the Presentation has played */
	void setActualSecond(int second);

	/* readjusts the Slider depending on timeVal */
	void setVideoProgress(int timeVal); 

	/* resets all FPS output to zero */
	void resetFPS();

	/* resets all ABW output to zero */
	void resetABW();

	/* sets the fps values. there are three values set 
	   eg.: fps after 1 sec
			fps after 5 sec
			fps after 10 sec
	*/
	void setFPS(float fps1, float fps5, float fps10);


	/* sets the aBW values. there are three values set 
	   eg.: aBW after 1 sec
			aBW after 5 sec
			aBW after 10 sec
	*/
	void setABW(int aBW1, int aBW5, int aBW10);
	

	/* returns the slider of the videoProgress */
	ExtQSlider * getProgressSlider() {
		return videoProgress;
	}

	/* returns the value of slider's width */
	int getSliderValue() {
		return progress;
	};
	
	/* a button to delete the Presentation and the PresentationInfoBox */
	QPushButton * deleteButton;

	/* a button to enable/disable audio */
	QPushButton * speakerButton;

	/* picture for unmuted audio */
	QPixmap unmutedPic;

	/* picture for muted audio */
	QPixmap mutedPic;

	/*  PropertiesDialog belonging to the PresentationInfoBox.
	    Each PresentationInfoBox has one PropertiesDialog
	*/
	QPopupMenu * options;
	PropertiesDialog * pDialog;
signals:
	/* signal when mouse button was pressed */
	void pressed();
public slots:

	/* emits active when PresentationInfoBox was pressed by left mouse button */
	void emitActive();

	/* open when PresentationInfoBox was pressed by left mouse button */
	void openProperties();

	/* shows options for PresentationInfoBox when it was pressed by right mouse button 
	
	*/
	void showOptions();

	/* loads muted or unmuted picture for audio */
	void mute_unmute();
private:

	bool active;
	bool localBox;
	bool muted;
	bool speakerShowedFirstTime;
	bool unmutedPicLoaded;
	int boxNumber;

	/* Labels for status output */
	ExtQLabel * framesPerSec;
	ExtQLabel * nrOfAudioES;
	ExtQLabel * dimension;
	ExtQLabel * durationMS;
	ExtQLabel * value_dimX;
	ExtQLabel * value_dimY;


	ExtQLabel * avBandwidth;
	ExtQLabel * fixedValue_aBW;
	ExtQLabel * value_aBW1;
	ExtQLabel * value_aBW5;
	ExtQLabel * value_aBW10;

	ExtQLabel * value_nrVideos;
	ExtQLabel * value_nrAudios;

	/* slider for video progress */
	ExtQSlider * videoProgress;
	ExtQLabel * value_durationMS;
	
	ExtQLabel * fixedValue_fps;
	ExtQLabel * value_fps1;
	ExtQLabel * value_fps5;
	ExtQLabel * value_fps10;
	

	ExtQLabel * source;



	int progress;
	int actualSecond;
	int actualFPS;

protected:
    QGridLayout* GroupBox1Layout;
    QHBoxLayout* Layout8;
    QHBoxLayout* Layout10;
    QHBoxLayout* Layout9;
    QGridLayout* Layout19;
	QHBoxLayout* Layout1;
	/* waits for a mouse press */
	virtual void mousePressEvent ( QMouseEvent * );
	

};


#endif
