 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	ProfileParser.hpp                                             *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PROFILEPARSER_HPP_
#define _PROFILEPARSER_HPP_

#include "global.hpp"
#include "metadata/TerminalCapabilities.hpp"
#include <list>
#include <qstring.h>
struct Profile {
	bool localPres;
	const char * uri;
	bool dummyAdaptors;
	list <QString> adaptors;
	TerminalCapabilities * termCaps;
	bool rtxEnabled;
	Profile() {
		localPres = true;
		dummyAdaptors = false;
		uri = NULL;
		adaptors.clear();
		termCaps = new TerminalCapabilities();
		termCaps->setColorDisplay(true);
		termCaps->setDisplayBitsPerPixel(16);
		termCaps->setDisplayRefreshRate(100);
		termCaps->setDisplayWidth(352);
		termCaps->setDisplayHeight(288);
		termCaps->setNetworkCapacityInByte(262144);
		termCaps->setMinGuaranteedNetworkCapacityInByte(262000);
		termCaps->setMaxDecoderBitRateInBit(12345678);
		termCaps->setMaxDecoderBufferSizeInByte(12345678);
		termCaps->setNumAudioChannels(2);
		termCaps->setAudioBitsPerSample(16);
		termCaps->setAudioSamplingFrequency(44100);
		rtxEnabled = true;
	}
};

/**
 *
 * <Class Description goes here>
 *
 *@author Ronald Sowa
 *
 *@version $Id: ProfileParser.hpp,v 1.4 2004/08/26 09:09:41 mtaschwer Exp $
 *
 *@short <short description>
 *
 */
class ProfileParser {

public:

	ProfileParser();
	~ProfileParser();

	int parseFromConfigFile(const char * configFile);
	int saveConfigToFile(const char * configFile,list <Profile *> profiles);
	list <Profile *> getProfile();
private:
	char * parseNextLine(FILE * fp);
	Profile * profile;
	list <Profile *> profileList;

};
#endif
