 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	PorpertiesDialog.cpp                                          *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/


#include "PropertiesDialog.hpp"
#include "PropertiesTab.hpp"
#include "VideoESInfo.hpp"
#include "metadata/TerminalCapabilities.hpp"
#include <qgroupbox.h>
#include <qprogressbar.h>
#include "io/Rtp.hpp"

TabInformation::TabInformation() {
}

TabInformation::~TabInformation() {
}
PropertiesDialog::PropertiesDialog(bool local, QWidget* parent,  const char* name, bool modal, WFlags fl) :
	PropertiesBase( parent, name, modal, fl ){
	setCaption(tr(name));
	
	TabWidget->removePage(tab_2);
	termCapsBox->setGeometry(10,165,360,130);
	if(local) {
		termCapsBox->hide();
	}
	else {
		adaptorTermCapBox->hide();
	}
}



PropertiesDialog::~PropertiesDialog() {
}


void PropertiesDialog::setGeneralProperties(int presNum) {
	this->presNum->setNum(presNum);
	this->presNum->show();
}

void PropertiesDialog::setProperties(PresentationInfo inf) {
	if(inf.localPresentation)
		LabelLocalRemote->setText(tr("- LOCAL"));
	else
		LabelLocalRemote->setText(tr("- REMOTE"));
	urlText->setText(inf.URI);
	valueAES->setNum((int)inf.numberOfAES);
	valueVES->setNum((int)inf.numberOfVES);
	int origTotWidth =0;
	int origTotHeight = 0;
	int dispTotWidth = 0;
	int dispTotHeight = 0;
	list <ESInfo *>::iterator oeI = inf.origESInfo.begin();
	for(list <ESInfo *>::iterator eI = inf.actESInfo.begin(); eI != inf.actESInfo.end(); eI++) {
		if((*eI)->isVisualStream()) {
			dispTotWidth += ((VideoESInfo*)(*eI))->getWidth();
			dispTotHeight = MAX((u32) dispTotHeight,((VideoESInfo*)(*eI))->getHeight());
			origTotWidth += ((VideoESInfo*)(*oeI))->getWidth();
			origTotHeight = MAX((u32) origTotHeight,((VideoESInfo*)(*oeI))->getHeight());
			
		}
		oeI++;
	}
	QString origRes;
	QString dispRes;
	valueDispRes->setText(dispRes.sprintf("%i x %i",dispTotWidth,dispTotHeight));
	valueOrigRes->setText(origRes.sprintf("%i x %i",origTotWidth,origTotHeight));
	


	list <QString>::iterator uA = inf.usedAdaptors.begin();
	if(uA != inf.usedAdaptors.end()) {
		adaptorLabel1->setText(*uA);
		uA++;
	}
	else
		adaptorLabel1->setText(tr("N/A"));
	if(uA != inf.usedAdaptors.end()) {
		adaptorLabel2->setText(*uA);
		uA++;
	}
	else
		adaptorLabel2->setText(tr("N/A"));
	if(uA != inf.usedAdaptors.end()) {
		adaptorLabel3->setText(*uA);
		uA++;
	}
	else
		adaptorLabel3->setText(tr("N/A"));
	if(uA != inf.usedAdaptors.end()) {
		adaptorLabel4->setText(*uA);
		uA++;
	}
	else
		adaptorLabel4->setText(tr("N/A"));
	if(uA != inf.usedAdaptors.end()) {
		adaptorLabel5->setText(*uA);
		//uA++;
	}
	else
		adaptorLabel5->setText(tr("N/A"));

	/* Remote presentation Properties */
	if(inf.usedTermCaps) {
		if(inf.usedTermCaps->getColorDisplay())
			valueColorDisplay->setText(tr("enabled"));
		else 
			valueColorDisplay->setText(tr("disabled"));
		u32 w = inf.usedTermCaps->getDisplayWidth();
		u32 h = inf.usedTermCaps->getDisplayHeight();
		QString qstr;
		if((w==176) && (h==144)) {
			qstr = tr("QCIF (176 x 144)");
		} else
			if((w==352) && (h==288)) {
			  qstr = tr("CIF (352 x 288)");
			} else
				if((w==704) && (h == 576)) {
						qstr = tr("4CIF (704 x 576)");
				} else
					qstr.sprintf("%s (%i x %i)",tr("user defined").latin1(),w,h);
		valueDimension->setText(qstr.latin1());
		valueBandwidth->setNum((int)(inf.usedTermCaps->getNetworkCapacityInByte()/128));
		if (inf.rtxEnabled) {
			valueRtx->setText(tr("enabled"));
		}
		else {
			valueRtx->setText(tr("disabled"));
		}
	}


	/* Properties for each ES */
	if(tabs.empty()) {
		int i = 1;
		int j = 1;
		list <ESInfo *> ::iterator actESI = inf.actESInfo.begin();
		for(list <ESInfo *>::iterator esI = inf.origESInfo.begin(); esI != inf.origESInfo.end(); esI++) {
			QString * tabTitle = NULL;
		

			if((*esI)->isVisualStream()) {
					tabTitle = new QString("Video ES ");
					tabTitle->append(QString::number(i));
					i++;
			}
			if((*esI)->isAudioStream()) {
					tabTitle = new QString("Audio ES ");
					tabTitle->append(QString::number(j));
					j++;
			}
			PropertiesTab * nextTab = new PropertiesTab(TabWidget,"anotherTab");
			if(inf.localPresentation) {
				nextTab->bufferGroupBox->setEnabled(false);
				}
			QString timeInfo;
			timeInfo.sprintf("00:00 of %02i:%02i",(int)(*esI)->getDurationInMs()/1000/60,
											 (int)(*esI)->getDurationInMs()/1000%60);
			nextTab->dynDurationValue->setText(timeInfo);
			nextTab->durationMS = (int)(*esI)->getDurationInMs();

			/* set dimension for a VES 
			*/
			if((*esI)->isVisualStream()) {
				nextTab->origX->setNum((int)((VideoESInfo*)(*esI))->getWidth());
				QString * dimY = new QString();
				dimY->sprintf("x     %i",(int)((VideoESInfo*)(*esI))->getHeight());
				nextTab->origY->setText(*dimY);
			}
			if((*actESI)->isVisualStream()) {
				nextTab->actX->setNum((int)((VideoESInfo*)(*actESI))->getWidth());
				QString * dimY = new QString();
				dimY->sprintf("x     %i",(int)((VideoESInfo*)(*actESI))->getHeight());
				nextTab->actY->setText(*dimY);
			}
			
			/* no dimension for an AES
			*/
			if((*esI)->isAudioStream()) {
				nextTab->origX->setText(tr("N/A"));
				nextTab->origY->setText(tr(" "));
			}
			if((*actESI)->isAudioStream()) {
				nextTab->actX->setText(tr("N/A"));
				nextTab->actY->setText(tr(" "));
			}

			//fpsOrig has to be set after first packet arrived (new VOPTimeIncrement!)
			nextTab->abwOrig->setNum((int) ceil((double)(*esI)->getAvgBandwidth()/(double)1024));
			TabWidget->insertTab(nextTab,*tabTitle);
			tabs.push_back(nextTab);
			actESI++;
		}
	}
	else {
		list <ESInfo *> ::iterator mESI = inf.actESInfo.begin();
		for (list <PropertiesTab *>::iterator tabI = tabs.begin(); tabI != tabs.end(); tabI++) {
			if((*mESI)->isVisualStream()) {
				(*tabI)->actX->setNum((int)((VideoESInfo*)(*mESI))->getWidth());
				QString * dimY = new QString();
				dimY->sprintf("x     %i",(int)((VideoESInfo*)(*mESI))->getHeight());
				(*tabI)->actY->setText(*dimY);
			}
		}
	}
}

void PropertiesDialog::setModifiedProperties(PresentationInfo inf) {

	if(!tabs.empty()) {
	
	}
}
		

void PropertiesDialog::setFramesPerSecond(list <float> fps_orig, list <float> fps_1,list <float> fps_2, list <float> fps_3) {
	if(!tabs.empty()) {

		if (fps_1.size() != tabs.size())
			return;

		list <float>::iterator forig = fps_orig.begin();
		list <float>::iterator f1 = fps_1.begin();
		list <float>::iterator f2 = fps_2.begin();
		list <float>::iterator f3 = fps_3.begin();

		for(list <PropertiesTab *>::iterator tabI = tabs.begin(); tabI != tabs.end(); tabI++) {
		  QString qs;
		  qs.sprintf("%2.1f",(*forig));
			(*tabI)->fpsOrig->setText(qs);
		  qs.sprintf("%2.1f",(*f1));
		  (*tabI)->fps1->setText(qs);
		  qs.sprintf("%2.1f",(*f2));
		  (*tabI)->fps2->setText(qs);
		  qs.sprintf("%2.1f",(*f3));
		  (*tabI)->fps3->setText(qs);
			forig++;
		  f1++;
		  f2++;
		  f3++;
		}
	}
}

void PropertiesDialog::setAverageBandwidth(list <int> abw_1, list <int> abw_2, list <int> abw_3)  {
	if(!tabs.empty()) {

		if (abw_1.size() != tabs.size())
			return;

		list <int>::iterator a1 = abw_1.begin();
		list <int>::iterator a2 = abw_2.begin();
		list <int>::iterator a3 = abw_3.begin();

		for(list <PropertiesTab *>::iterator tabI = tabs.begin(); tabI != tabs.end(); tabI++) {
			(*tabI)->abw1->setNum((*a1)/128); // == * 8 / 1024 
			(*tabI)->abw2->setNum((*a2)/128);
			(*tabI)->abw3->setNum((*a3)/128);
			a1++;
			a2++;
			a3++;
		}
	}
}		

void PropertiesDialog::setActualSecond(list <float> actualSecond) {
	if(!tabs.empty()) {
		list <float>::iterator aSec = actualSecond.begin();
		for(list <PropertiesTab *>::iterator tabI = tabs.begin(); tabI != tabs.end(); tabI++) {
			QString timeInfo;
			if ((int)*aSec >= 0) {
				timeInfo.sprintf("%02i:%02i of %02i:%02i",((int)*aSec)/60,((int)*aSec)%60,
						 (int)(*tabI)->durationMS/1000/60,(int)(*tabI)->durationMS/1000%60);
				(*tabI)->dynDurationValue->setText(timeInfo);
			} else {
				timeInfo = tr("BUFFERING");
				(*tabI)->dynDurationValue->setText(timeInfo);
			}
			aSec++;
		}
	}
}


void PropertiesDialog::setBufferFillLevel(list <int> bufferFillLevel) {
	QString tmp;
	if(!tabs.empty()) {
		list <int>::iterator bfl = bufferFillLevel.begin();
		for(list <PropertiesTab *>::iterator tabI = tabs.begin(); tabI != tabs.end(); tabI++) {
			tmp.sprintf("(%6i KB)",*bfl/1024);
			(*tabI)->bufferBytesLabel->setText(tmp);
			bfl++;
		}
	}
}

void PropertiesDialog::setSecBufferFillLevel(list <double> secBufferFillLevel){
	QString tmp;
	if(!tabs.empty()) {
		list <double>::iterator sbfl = secBufferFillLevel.begin();
		for(list <PropertiesTab *>::iterator tabI = tabs.begin(); tabI != tabs.end(); tabI++) {	
			(*tabI)->bufferProgressBar->setProgress((int)(*sbfl*100+0.5));
			tmp.sprintf("%03.2f secs",*sbfl);
			(*tabI)->bufferSecsLabel->setText(tmp);

			sbfl++;
		}
	}
}

void PropertiesDialog::resetSeconds() {

	if(!tabs.empty()) {
		for(list <PropertiesTab *>::iterator tabI = tabs.begin(); tabI != tabs.end(); tabI++) {	
			(*tabI)->dynDurationValue->setText(tr("00:00"));
		}
	}
}

void PropertiesDialog::resetFPS() {

if(!tabs.empty()) {
	for(list <PropertiesTab *>::iterator tabI = tabs.begin(); tabI != tabs.end(); tabI++) {
			(*tabI)->fps1->setNum(0);
			(*tabI)->fps2->setNum(0);
			(*tabI)->fps3->setNum(0);
		
		}
	}
}

void PropertiesDialog::resetABW() {
	if(!tabs.empty()) {
		for(list <PropertiesTab *>::iterator tabI = tabs.begin(); tabI != tabs.end(); tabI++) {
			(*tabI)->abw1->setNum(0); 
			(*tabI)->abw2->setNum(0);
			(*tabI)->abw3->setNum(0);
			
		}
	}
}
