 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	Propertiesdialog.hpp                                          *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef PROPERTIES_DIALOG_HPP
#define PROPERTIES_DIALOG_HPP

#include "Properties.hpp"
#include "Presentation.hpp"
#include "qtabwidget.h"
#include "qlabel.h"
#include "PropertiesTab.hpp"

/**
 *
 * class which includes general information and
 * status information of each VES/AES of one Presentation
 *
 *@author Ronald Sowa
 *
 *@version $Id: PropertiesDialog.hpp,v 1.4 2004/02/25 16:02:56 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class TabInformation {
public:
	TabInformation();
	~TabInformation();
	QWidget * newTab;
	QLabel * duration;
	QLabel * dynDurationValue;
	QLabel * origDimension;
	QLabel * origX;
	QLabel * origY;
	QLabel * actDimension;
	QLabel * actX;
	QLabel * actY;
	QLabel * fpsTitle;
	QLabel * fps1;
	QLabel * fps2;
	QLabel * fps3;
	QLabel * fpsOrig;
	QLabel * abw1;
	QLabel * abw2;
	QLabel * abw3;

};


/* dialog, which shows all Properties of one Presentation */
class PropertiesDialog : public PropertiesBase {
public:
	/* Constructor */
	PropertiesDialog(bool local,QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0);

	/* Destructor */
	~PropertiesDialog();

	/* sets presentation number in PropertiesDialog */
	void setGeneralProperties(int presNum);

	/* sets presentation Properties for the first time */
	void setProperties(PresentationInfo inf);

	/* sets modified Properties (e.g: new adaptors have been selected / terminal capabilites have changed */
	void setModifiedProperties(PresentationInfo inf);

	/* periodically sets actual FPS values; 
	  the periodic update depends on the timer value set in VideoDemoDialog 
	*/
	void setFramesPerSecond(list <float> fps_orig, list <float> fps_1, list <float> fps_2,list <float> fps_3);

	/* periodically sets actual aBW values; 
	  the periodic update depends on the timer value set in VideoDemoDialog 
	*/
	void setAverageBandwidth(list <int> abw_1, list <int> abw_2,list <int> abw_3);

	/* periodically sets actual playing time; 
	  the periodic update depends on the timer value set in VideoDemoDialog 
	*/
	void setActualSecond(list <float> actualSecond);

	void setBufferFillLevel(list <int> bufferFillLevel);

	void setSecBufferFillLevel(list <double> secBufferFillLevel);

	/* sets all playing time values to zero */
	void resetSeconds();

	/* sets all FPS values to zero */
	void resetFPS();

	/* sets all aBW values to zero */
	void resetABW();

private:
	/* a list of Tabs, one for each elementary stream 
	   support for audio ES not implemented 
	*/

	list <PropertiesTab *> tabs;
};
#endif
