 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	PropertiesTab.cpp                                             *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "PropertiesTab.hpp"

#include <qgroupbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qprogressbar.h>
#include "io/Rtp.hpp"

PropertiesTab::PropertiesTab(QWidget * parent, const char * name, WFlags f) :
	QWidget(parent,name,f) {
    bufferGroupBox = new QGroupBox( this, "bufferGroupBox" );
    bufferGroupBox->setGeometry( QRect( 11, 215, 360, 96 ) ); 
    bufferGroupBox->setTitle( tr( "Client Buffer Fill Level" ) );
    bufferGroupBox->setColumnLayout(0, Qt::Vertical );
    bufferGroupBox->layout()->setSpacing( 0 );
    bufferGroupBox->layout()->setMargin( 0 );
    bufferGroupBoxLayout = new QGridLayout( bufferGroupBox->layout() );
    bufferGroupBoxLayout->setAlignment( Qt::AlignTop );
    bufferGroupBoxLayout->setSpacing( 6 );
    bufferGroupBoxLayout->setMargin( 11 );

    bufferBytesLabel = new QLabel( bufferGroupBox, "bufferBytesLabel" );
    bufferBytesLabel->setText( tr( "(N/A KB)" ) );

    bufferGroupBoxLayout->addWidget( bufferBytesLabel, 0, 3 );

    bufferSecsLabel = new QLabel( bufferGroupBox, "bufferSecsLabel" );
    bufferSecsLabel->setText( tr( "N/A secs" ) );
    bufferGroupBoxLayout->addWidget( bufferSecsLabel, 0, 0 );

    QSpacerItem* spacer_8 = new QSpacerItem( 160, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    bufferGroupBoxLayout->addItem( spacer_8, 0, 1 );

    bufferProgressBar = new QProgressBar( bufferGroupBox, "bufferProgressBar" );
#if ((WIN32) && ((QT_VERSION >> 16) < 3))
		bufferProgressBar->setCenterIndicator(false);
#else
		bufferProgressBar->setPercentageVisible(false);
#endif
		bufferProgressBar->setTotalSteps (RTP_CLIENT_PreQ_MAX_SECS * 100); //*100: give finer granularity

    bufferGroupBoxLayout->addMultiCellWidget( bufferProgressBar, 1, 1, 0, 3 );

    QWidget* privateLayoutWidget_8 = new QWidget( this, "Layout21" );
    privateLayoutWidget_8->setGeometry( QRect( 11, 11, 360, 200 ) ); 
    Layout21_2 = new QVBoxLayout( privateLayoutWidget_8 ); 
    Layout21_2->setSpacing( 6 );
    Layout21_2->setMargin( 0 );

    Layout37 = new QGridLayout; 
    Layout37->setSpacing( 0 );
    Layout37->setMargin( 0 );
    QSpacerItem* spacer_9 = new QSpacerItem( 16, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout37->addItem( spacer_9, 1, 4 );

    origDimension = new QLabel( privateLayoutWidget_8, "origDimension" );
    origDimension->setText( tr( "Original Dimension:" ) );

    Layout37->addMultiCellWidget( origDimension, 1, 1, 0, 3 );
    QSpacerItem* spacer_10 = new QSpacerItem( 20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout37->addItem( spacer_10, 1, 6 );

    actDimension = new QLabel( privateLayoutWidget_8, "actDimension" );
    actDimension->setText( tr( "Actual Dimension:" ) );

    Layout37->addMultiCellWidget( actDimension, 2, 2, 0, 3 );

    origY = new QLabel( privateLayoutWidget_8, "origY" );
    origY->setText( tr( "N/A" ) );

    Layout37->addWidget( origY, 1, 7 );

    origX = new QLabel( privateLayoutWidget_8, "origX" );
    origX->setText( tr( "N/A" ) );

    Layout37->addWidget( origX, 1, 5 );
    QSpacerItem* spacer_11 = new QSpacerItem( 157, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout37->addItem( spacer_11, 1, 8 );
    QSpacerItem* spacer_12 = new QSpacerItem( 20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout37->addItem( spacer_12, 2, 6 );

    actY = new QLabel( privateLayoutWidget_8, "actY" );
    actY->setText( tr( "N/A" ) );

    Layout37->addWidget( actY, 2, 7 );
    QSpacerItem* spacer_13 = new QSpacerItem( 30, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout37->addItem( spacer_13, 2, 4 );
    QSpacerItem* spacer_14 = new QSpacerItem( 157, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout37->addItem( spacer_14, 2, 8 );
    QSpacerItem* spacer_15 = new QSpacerItem( 16, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout37->addItem( spacer_15, 0, 1 );


    dynDurationValue = new QLabel( privateLayoutWidget_8, "dynDurationValue" );
    dynDurationValue->setText( tr( "0" ) );

    Layout37->addWidget( dynDurationValue, 0, 2 );
    QSpacerItem* spacer_16 = new QSpacerItem( 157, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout37->addItem( spacer_16, 0, 8 );

    duration = new QLabel( privateLayoutWidget_8, "duration" );
    duration->setText( tr( "Duration:" ) );

    Layout37->addWidget( duration, 0, 0 );

    actX = new QLabel( privateLayoutWidget_8, "actX" );
    actX->setText( tr( "N/A" ) );

    Layout37->addWidget( actX, 2, 5 );
    QSpacerItem* spacer_17 = new QSpacerItem( 20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout37->addItem( spacer_17, 0, 3 );
    Layout21_2->addLayout( Layout37 );

    Layout61 = new QGridLayout; 

    Layout61->setSpacing( 0 );
    Layout61->setMargin( 0 );

    abw1 = new QLabel( privateLayoutWidget_8, "abw1" );
    abw1->setText( tr( "N/A" ) );

    Layout61->addWidget( abw1, 2, 1 );

    fiveSec = new QLabel( privateLayoutWidget_8, "fiveSec" );
    fiveSec->setText( tr( "(5 sec)" ) );

    Layout61->addWidget( fiveSec, 0, 3 );

    abw2 = new QLabel( privateLayoutWidget_8, "abw2" );
    abw2->setText( tr( "N/A" ) );

    Layout61->addWidget( abw2, 2, 3 );
    QSpacerItem* spacer_18 = new QSpacerItem( 20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout61->addItem( spacer_18, 1, 4 );

    abw3 = new QLabel( privateLayoutWidget_8, "abw3" );
    abw3->setText( tr( "N/A" ) );

    Layout61->addWidget( abw3, 2, 5 );

    origLabel = new QLabel( privateLayoutWidget_8, "origLabel" );
    origLabel->setText( tr( "(orig)" ) );

    Layout61->addWidget( origLabel, 0, 7 );

    oneSec = new QLabel( privateLayoutWidget_8, "oneSec" );
    oneSec->setText( tr( "(1 sec)" ) );

    Layout61->addWidget( oneSec, 0, 1 );
    QSpacerItem* spacer_19 = new QSpacerItem( 20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout61->addItem( spacer_19, 2, 2 );

    abwOrig = new QLabel( privateLayoutWidget_8, "abwOrig" );
    abwOrig->setText( tr( "N/A" ) );

    Layout61->addWidget( abwOrig, 2, 7 );

    abwTitle = new QLabel( privateLayoutWidget_8, "abwTitle" );
    abwTitle->setText( tr( "Actual aBW:" ) );

    Layout61->addWidget( abwTitle, 2, 0 );
    QSpacerItem* spacer_20 = new QSpacerItem( 20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout61->addItem( spacer_20, 1, 2 );

    fps3 = new QLabel( privateLayoutWidget_8, "fps3" );
    fps3->setText( tr( "N/A" ) );

    Layout61->addWidget( fps3, 1, 5 );
    QSpacerItem* spacer_21 = new QSpacerItem( 26, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout61->addItem( spacer_21, 0, 4 );
    QSpacerItem* spacer_22 = new QSpacerItem( 60, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout61->addItem( spacer_22, 0, 0 );
    QSpacerItem* spacer_23 = new QSpacerItem( 20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout61->addItem( spacer_23, 2, 4 );

    fps1 = new QLabel( privateLayoutWidget_8, "fps1" );
    fps1->setText( tr( "N/A" ) );

    Layout61->addWidget( fps1, 1, 1 );
    QSpacerItem* spacer_24 = new QSpacerItem( 26, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout61->addItem( spacer_24, 0, 2 );
    QSpacerItem* spacer_25 = new QSpacerItem( 21, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout61->addItem( spacer_25, 0, 6 );

    fpsOrig = new QLabel( privateLayoutWidget_8, "fpsOrig" );
    fpsOrig->setText( tr( "N/A" ) );

    Layout61->addWidget( fpsOrig, 1, 7 );

    fpsTitle = new QLabel( privateLayoutWidget_8, "fpsTitle" );
    fpsTitle->setText( tr( "Actual FPS:" ) );

    Layout61->addWidget( fpsTitle, 1, 0 );
    QSpacerItem* spacer_26 = new QSpacerItem( 20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout61->addItem( spacer_26, 2, 6 );

    fps2 = new QLabel( privateLayoutWidget_8, "fps2" );
    fps2->setText( tr( "N/A" ) );

    Layout61->addWidget( fps2, 1, 3 );

    tenSec = new QLabel( privateLayoutWidget_8, "tenSec" );
    tenSec->setText( tr( "(10 sec)" ) );

    Layout61->addWidget( tenSec, 0, 5 );
    QSpacerItem* spacer_27 = new QSpacerItem( 20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    Layout61->addItem( spacer_27, 1, 6 );
    Layout21_2->addLayout( Layout61 );
}



PropertiesTab::~PropertiesTab() {
}
