 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	RemotePresentation.cpp                                        *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/


#include "RemotePresentation.hpp"

#include "net/RTSP.hpp"
#include "DataChannel.hpp"
#include "DataSink.hpp"
#include "cache/MetaObject.hpp"
#include "ContainerInfo.hpp"
#include "net/PacketizationLayer.hpp"
#include "VideoESInfo.hpp"
#include "AudioESInfo.hpp"
#include "ReferenceCounter.hpp"
#include "adaptors/SharedAdaptor.hpp"
#include "adaptors/GlobalTimer.hpp"
#include "adaptors/ESSynchronizer.hpp"
#include "VideoAdaptorChain.hpp"
#include "AudioAdaptorChain.hpp"
#include "net/RFC3016.hpp"
#include "net/RFC2250.hpp"
#include "adaptors/MP4Decoder.hpp"
#include "adaptors/MP4audioDecoder.hpp"
#include "VideoDisplayIO.hpp"
#include "adaptors/YUVScalingAdaptor.hpp"
#include "adaptors/PSNR.hpp"
#include "adaptors/DataDump.hpp"
#include "io/Rtp.hpp"
#include "Statistics.hpp"
#include "adaptors/FrameCounter.hpp"
#include "adaptors/StatisticsUpdater.hpp"

#ifdef WIN32
#include <winsock2.h>
#else
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <netinet/in.h>
#include <libgen.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#endif

/** creates a socket to the given server. If the url is invalid or anyhing
 * goes wrong, the state is set to SESSION_ERR. Otherwise the state is set
 * to SESSION_CLOSED */


RemotePresentation::RemotePresentation(u32 x_inSDL, u32 y_inSDL,  ReferenceCounter * gtRef, GlobalTimer *gt, const char* proxyName,
		MultiVideoPlayerDialog *mvpd, int proxyPort) :
	Presentation(x_inSDL,y_inSDL, gtRef, gt)
{

	if(proxyName) {
		strcpy(this->proxyName,proxyName);
	}
	else {
		this->proxyName[0] = '\0';
	}
	this->proxyPort = proxyPort;
	localPres = false;
	this->x_inSDL = x_inSDL;
	this->y_inSDL = y_inSDL;
	pWidth = 0;
	pHeight = 0;
	lastX = 0;
	lastY = 0;

	rtxEnabled = true;
	presInform.rtxEnabled = true;
	presInform.localPresentation = false;
	modifiedTermCap = NULL;
	termCap = NULL;
	presState = PRES_INITIALIZED;

	//default values for TermCapsRemote
	upscale = false;
	keepAspectRatio = true;
	enableSwitching = true;

	modifiedTermCap = new TerminalCapabilities();
	modifiedTermCap->setColorDisplay(true);
	modifiedTermCap->setDisplayBitsPerPixel(16);
	modifiedTermCap->setDisplayRefreshRate(100);
	modifiedTermCap->setDisplayWidth(800);
	modifiedTermCap->setDisplayHeight(600);
	modifiedTermCap->setNetworkCapacityInByte(131077);
	modifiedTermCap->setMinGuaranteedNetworkCapacityInByte(0);
	modifiedTermCap->setMaxDecoderBitRateInBit(12345678);
	modifiedTermCap->setMaxDecoderBufferSizeInByte(12345678);
	modifiedTermCap->setNumAudioChannels(2);
	modifiedTermCap->setAudioBitsPerSample(16);
	modifiedTermCap->setAudioSamplingFrequency(44100);
	gtRefCount = gtRef;
	globalTimer = gt;
	mp4Stream=NULL;
	presM=NULL;
	uri=NULL;url=NULL;
	protocol=NULL;
	server=NULL;
	receiver=NULL;

	socket = 0;

	this->mvpd = mvpd;

	//(maximum) waiting time for select on receiving messages
	readtv.tv_sec = 0;
	readtv.tv_usec = 1000;
}
RemotePresentation::~RemotePresentation()
{
	if (receiver != NULL) {
		receiver->deactivate();
		delete receiver;
		receiver = NULL;
	}

	if (protocol) {
		delete protocol;
		protocol = NULL;
	}
	if (server) {
		delete server;
		server = NULL;
	}
	if (url) {
		delete url;
		url = NULL;
	}
}

int RemotePresentation::initalizeURI(const char * URI) {

	pWidth = 0;
	pHeight = 0;
	lastX = 0;
	longestESfromRtpId = -1;
	rtpChannelOfLongestES = NULL;
	assert(globalTimer);
	//		globalTimer = new GlobalTimer();
	termCap = new TerminalCapabilities();
	if(modifiedTermCap) {
		termCap = modifiedTermCap->clone();
		presInform.usedTermCaps = modifiedTermCap->clone();
	}


	mp4Stream=NULL;

	uri = URI;
	presInform.URI = (char *)uri;
	protocol = NULL;
	// establish a connection to the server
	if(!uri) {
		dprintf_err("RemotePresentation::initializeURI: NULL url\r\n");
		state=SESSION_ERR;
		return -1;
	}
	else {
		url=new char[strlen(uri)+1];
		strcpy(url,uri);
	}

	socket=::socket(AF_INET,SOCK_STREAM,0);
	if(socket<0) {
		dprintf_full("RemotePresentation::initializeURI: Couldn't create socket for url %s\r\n",url);
		state=SESSION_ERR;
		return -1;
	}

	QString temp(url);
	QUrl u(temp);

	char* pUrl=strstr(url,"rtsp://");
	if(!pUrl) {
		dprintf_err("RemotePresentation::initializeURI: invalid RTSP url %s\r\n",url);
		state=SESSION_ERR;
		return -1;
	}
	pUrl+=strlen("rtsp://");
	QString host=u.host();
	struct hostent *ipAddr;
	ipAddr=gethostbyname(host.latin1());
	if(!ipAddr) {
		dprintf_err("RemotePresentation::initializeURI: DNS resolution failed for URL %s\r\n",url);
		state=SESSION_ERR;
		return -1;
	}	else {
		dprintf_full("RemotePresentation::initializeURI: DNS returned %s for URL %s\r\n",
								 inet_ntoa( *((struct in_addr*)ipAddr->h_addr)),url);
	}
	if (!server)
		server = new sockaddr_in();

	if(strstr(pUrl,":")) {
		dprintf_full("RemotePresentation::initializeURI: URL %s contains port %i,  PROXY [%s] port %i\r\n",
								 url,u.port(),proxyName,proxyPort);
		if(strlen(proxyName) == 0)
			server->sin_port=htons(u.port());
	}	else {
		dprintf_full("RemotePresentation::initializeURI: URL %s PPROXY %s contains no port\r\n",
								 url,proxyName);
		if(strlen(proxyName) == 0)
			server->sin_port=htons(DEFAULT_RTSP_PORT);
	}
	if(strlen(proxyName) > 0) {
		server->sin_port=htons(proxyPort);
		struct hostent *prxIpAddr = gethostbyname(proxyName);
		if(!prxIpAddr) {
			dprintf_err("RemotePresentation::initializeURI: DNS resolution failed for PROXY %s\r\n",proxyName);
			state=SESSION_ERR;
			return -1;
		}	else {
			dprintf_full("RemotePresentation::initializeURI: DNS returned %s for PROXY %s\r\n",
									 inet_ntoa( *((struct in_addr*)prxIpAddr->h_addr)),proxyName);
		}
		server->sin_addr.s_addr=inet_addr(prxIpAddr->h_addr);
		server->sin_family=AF_INET;
		bcopy(prxIpAddr->h_addr, (char *)&server->sin_addr, prxIpAddr->h_length);
	}
	else {
		server->sin_addr.s_addr=inet_addr(ipAddr->h_addr);
		server->sin_family=AF_INET;
		bcopy(ipAddr->h_addr, (char *)&server->sin_addr, ipAddr->h_length);
	}

	if (receiver != NULL) {
		receiver->deactivate();
		delete receiver;
		receiver = NULL;
	}
	receiver = new ReceiverThread(this);
	receiver->start();


	if (::connect(socket,(struct sockaddr *)(server),sizeof(struct sockaddr)) < 0) {
		perror("RemotePresentation failed at connect:");

		if (receiver != NULL) {
			receiver->deactivate();
			delete receiver;
			receiver = NULL;
		}

		state=SESSION_ERR;
		return -1;
	}

	protocol = new RTSP();

	state = SESSION_CLOSED;
	if(!connect()) {
		if (receiver != NULL) {
			receiver->deactivate();
			delete receiver;
			receiver = NULL;
		}
		dprintf_full("RemotePresentation::initializeURI -> Not connected!\n");
		return -1;
	}
	else {
		return 0;
        }
}

void RemotePresentation::setupURI() {

	int nrAudioES = 0;
	int nrVideoES = 0;
	int i = 1;
	list < ESInfo * >::iterator eI;

	if(mp4Stream) {

		state=SESSION_CONNECTED;
		list <ESInfo *> *esInf = mp4Stream->getESList();
		ESSynchronizer *ess = NULL;
		/* setup video stream first */
		for(eI = esInf->begin(); eI != esInf->end(); eI++) {
		  if((*eI)->isVisualStream()) {
		    nrVideoES++;
		    dprintf_full("RemotePresentation::setupURI-> Found VideoStream with id = %i\n",(*eI)->getStreamId());
		    presM = new presentationManagement();
		    presM->adaptorChain = new VideoAdaptorChain();
		    origESInfo.push_back((*eI)->clone((*eI)->getContainerInfo()));
		    actESInfo.push_back((*eI)->clone((*eI)->getContainerInfo()));
		    presM->esInfo = (*eI)->clone((*eI)->getContainerInfo());

		    presM->stats = new Statistics(presM->esInfo, false);
		    ess = presM->esSync = new ESSynchronizer(*eI,globalTimer);
		    SharedAdaptor * sharedAdapt = new SharedAdaptor(gtRefCount, (Adaptor*&)globalTimer);
		    presM->adaptorChain->addAdaptor(sharedAdapt);
		    duration = MAX(duration,(*eI)->getDurationInMs());
		    //add StatsUpdater for video, but after adaptation
		    presM->adaptorChain->addAdaptor(new StatisticsUpdater(*eI,presM->stats));

		    if ((termCap->getDisplayWidth() <= ((VideoESInfo*)(*eI))->getWidth()) &&
			(termCap->getDisplayHeight() <= ((VideoESInfo*)(*eI))->getHeight()) ) {
		      ((VideoESInfo*)(*eI))->setWidth(termCap->getDisplayWidth());
		      ((VideoESInfo*)(*eI))->setHeight(termCap->getDisplayHeight());
		    }
		    int realWidth = 0;
		    int realHeight = 0;

		    if(upscale) {
		      realWidth = termCap->getDisplayWidth();
		      realHeight = termCap->getDisplayHeight();
		    }
		    else if (keepAspectRatio) {
		      realWidth =  ((VideoESInfo*)(*eI))->getWidth();
		      realHeight = (int)(floor((float)realWidth / ((VideoESInfo*)(*eI))->getAspectRatio()));
		      //adjust to even boundaries (241-->242, 179-->180)
		      realHeight = realHeight + ((realHeight % 2));

		    }
		    else {
		      realWidth = ((VideoESInfo*)(*eI))->getWidth();
		      realHeight = ((VideoESInfo*)(*eI))->getHeight();
		    }

		    presM->videoOutput = new VideoDisplayIO(NULL, PG_Rect(x_inSDL+lastX, y_inSDL, realWidth+4,realHeight+4), false);
		    presM->videoOutput->setESInfo((*eI));
		    presM->videoOutput->open();
		    ((VideoESInfo*)actESInfo.back())->setWidth(realWidth);
		    ((VideoESInfo*)actESInfo.back())->setHeight(realHeight);
		    addDecoderAdaptor(presM->videoOutput, *eI, presM->stats, i, ess);
		    lastX = realWidth+4;
		    this->pWidth += lastX;
		    lastY = realHeight+4;
		    this->pHeight = MAX(pHeight,(u32)lastY);
		    i++;
		    presManage.push_back(presM);
		    break;	// only 1 video stream
		  } //video
		}
		/* setup audio stream after video */
		for(eI = esInf->begin(); eI != esInf->end(); eI++) {
		  if((*eI)->isAudioStream()) {
		    nrAudioES++;
		    dprintf_full("RemotePresentation::setupURI-> Found AudioStream with id = %i\n",(*eI)->getStreamId());
		    presM = new presentationManagement();
		    presM->adaptorChain = new AudioAdaptorChain();
		    origESInfo.push_back((*eI)->clone((*eI)->getContainerInfo()));
		    actESInfo.push_back((*eI)->clone((*eI)->getContainerInfo()));
		    presM->esInfo = (*eI)->clone((*eI)->getContainerInfo());
		    presM->stats = new Statistics(presM->esInfo, false);
		    SharedAdaptor * sharedAdapt = new SharedAdaptor(gtRefCount, (Adaptor*&)globalTimer);
		    presM->adaptorChain->addAdaptor(sharedAdapt);
		    duration = MAX(duration,(*eI)->getDurationInMs());
		    presM->adaptorChain->addAdaptor(new StatisticsUpdater(*eI,presM->stats));
		    // pass video ESSync to SDLaudioIO (may be NULL)
		    presM->audioOutput = new SDLaudioIO((AudioESInfo*)(*eI), ess);
		    presM->audioOutput->open();

		    addDecoderAdaptor(presM->audioOutput, *eI, presM->stats, i);
		    i++;
		    presManage.push_back(presM);
		    break;	// only 1 audio stream
		  } //audio
		}

		presInform.actESInfo = actESInfo;
		presInform.origESInfo = origESInfo;
		presInform.numberOfAES = nrAudioES;
		presInform.numberOfVES = nrVideoES;
		presState = PRES_READYTOPLAY;
	}
	else
		dprintf_full("RemotePresentation::setupURI-> NO MP4STREAM OBJECT! \n");
}


void RemotePresentation::addDecoderAdaptor(IO * output, ESInfo * esInfo, Statistics *stats, int id,
					   ESSynchronizer *ess) {

	char localhost[10]="127.0.0.1";
	char yuvOrig[MAX_STR_LEN];
	char yuvDumpOut[MAX_STR_LEN];
	const char* path;

	dprintf_full("RemotePresentation::addDecoderAdaptor with clientId %i\n",id);
	if (esInfo->isVisualStream()) {
		path=esInfo->getContainerInfo()->getUrl()->getFileName();
		sprintf(yuvOrig,"localfile_%s.video.%i.yuv",(path?path:"NULL"),	esInfo->getStreamId());
		sprintf(yuvDumpOut,"localfile_%s.video.%i.dump.yuv",(path?path:"NULL"),	esInfo->getStreamId());

		AdaptorChain *ach = new VideoAdaptorChain();
		//RemotePresentation is created before switching might be chosen, so we enable it always...
		MP4Decoder *mp4dec = new MP4Decoder((VideoESInfo*)esInfo,true, 
																				UncompressedVideoFrame::ColorSpaceYV12, MP4Decoder::FFMPEG);

		ach->addAdaptor(mp4dec);
		if (ess)
		  ach->addAdaptor(ess);

		ach->addAdaptor(new FrameCounter(esInfo,stats));

		struct stat st;
		//FIXME: if video is adapted on server, it might not fit to local orig stream!
		if (stat(yuvOrig,&st) != -1) { //could find orig YUV, so add PSNR adaptor

			// upscaling of frame for correct PSNR and YUV-DataDump.... else not needed and done in SDL display
		// this is only necessary for Stream Switching, but RemotePresentation is created before this might be
		// chosen in TermCapsRemote, so accept some overhead :(
		//		if (enableSwitching)
		ach->addAdaptor(new YUVScalingAdaptor((VideoESInfo*)esInfo,	((VideoESInfo*)esInfo)->getWidth(),
																					((VideoESInfo*)esInfo)->getHeight()) );

		YUVStreamIO *yuv = new YUVStreamIO((VideoESInfo*)esInfo,yuvOrig,false);
			PSNR *psnr = new PSNR((VideoESInfo*)esInfo,yuv,stats);
			ach->addAdaptor(psnr);
			dprintf_full("RemotePresentation::addDecoderAdaptor  found origYUV %s for PSNR calc adaptor\n",yuvOrig);

			YUVStreamIO *yuvOut = new YUVStreamIO((VideoESInfo*)esInfo,yuvDumpOut,true);

			DataDump *yuvdump = new DataDump(esInfo,yuvOut,true);
			ach->addAdaptor(yuvdump);
			dprintf_full("RemotePresentation::addDecoderAdaptor  YUV-DataDump to %s enabled\n",yuvDumpOut);
		} else
			dprintf_full("RemotePresentation::addDecoderAdaptor  could not find origYUV"
									 " %s for PSNR calc adaptor...\n",yuvOrig);

		presM->clientObj = new DataSink(id,output,localhost,0,ach);

	} else //is Audio
		if(esInfo->isAudioStream()) {
			presM->clientObj = new DataSink(id,output,localhost,0,
							new MP4audioDecoder((AudioESInfo*)esInfo,true,MP4audioDecoder::FFMPEG));
		}
		else {
			dprintf_err("RemotePresentation::addDecoderAdaptor  neither video nor audio!\n");
			_exit(1);
		}

}


/** writes the string to the session control channel */
bool RemotePresentation::sendResponse(const char *str,int bytes)
{
	if (socket) {
		#ifdef WIN32
		send(socket,str, bytes,0);
		#else
		send(socket,str, bytes,MSG_NOSIGNAL);
		#endif
		dprintf_full("\nRemotePresentation::sendResponse\n%s",str);
		protocol->incSeqNr();
		return true;
	} else {
		return false;
	}

};

bool RemotePresentation::setTermCaps(TerminalCapabilities *tc, bool rtxEnabled,
																		 bool upscale, bool keepAscpectRatio, bool enableSwitching) {

	pWidth = 0;
	pHeight = 0;
	lastX = 0;
	lastY = 0;
	modifiedTermCap = tc->clone();
	this->rtxEnabled = rtxEnabled;
	this->upscale = upscale;
	this->keepAspectRatio = keepAscpectRatio;
	this->enableSwitching = enableSwitching;
	presInform.rtxEnabled = rtxEnabled;
	return true;
}

void RemotePresentation::setRtxEnabled(bool enabled) {
	rtxEnabled = enabled;
}

/** actually does a describe */
bool RemotePresentation::connect()
{
	if(!protocol || !url)
		return false;
	protocol->generateDescribeForUrl(url,PLAYER_ID,this->termCap);
	const char* req=protocol->getBuffer();

	if(!sendResponse(req,strlen(req))) {
		return false;
	}
	char buffer[MSG_BUFFER_SIZE];
//	buffer[0]=0;
//	char **buffer = (char**) malloc(sizeof(char));

	int bytesRead = -1;
	// read answer to describe
	waitForMessage(socket, bytesRead, buffer);
	if(bytesRead<=0)
		return false;
	// FIXME: check RTSP 404
	char* dum=NULL;
	ContainerInfo* mp4=protocol->parseResponseDescribe(buffer,url,&dum,&rtxInfo);

	if(dum)
		delete dum;
	if(this->mp4Stream)
		delete mp4Stream;
	if(mp4) {
		state=SESSION_CONNECTED;
		mp4Stream=mp4;
	}
	else return false;

	return (state==SESSION_CONNECTED);
};

bool RemotePresentation::play() {
	if (mp4Stream) {
		presState = PRES_PLAYING;
		int sessionId=0;
		if ((state == SESSION_INITIALIZED) || (state == SESSION_PAUSED)) {
                        double starttime = -1.0;
			for( list <presentationManagement *>::iterator pm = presManage.begin();
					 pm != presManage.end(); pm++) {
				sessionId=(*pm)->dChannel->getSessionId();
				/* use a common start time for all DataChannels */
				if (starttime < 0.0) {
				  starttime = ((Rtp*)(*pm)->dChannel->getInput())->getStatistics()->getPlayoutSec();
				  if (nptStart > 0.0 && starttime < nptStart)
				    starttime = nptStart;
				}
				double endtime = (double)(*pm)->esInfo->getDurationInMs()/1000.0;
				if (nptEnd > 0.0 && nptEnd < endtime)
				  endtime = nptEnd;
				/* adjust ESSynchronizer and GlobalTimer */
				if ((*pm)->esSync)
				  (*pm)->esSync->adjust(starttime);
				/* send PLAY request */
				protocol->generatePlay(this->url,sessionId, starttime, endtime, RTP_PREFETCH_SECS);
				const char* req=protocol->getBuffer();
				if (!this->sendResponse(req,strlen(req)))
					return false;

				char buffer[MSG_BUFFER_SIZE];
//				char **buffer = (char**) malloc(sizeof(char));
				int len;
				waitForMessage(socket, len, buffer);
				if (len<=0)
					return false;

				if (!strstr(buffer,"200")) { // status code for OK
					state=SESSION_ERR;
					return false;
				}

				// insert client
				if (state == SESSION_INITIALIZED)
					(*pm)->dChannel->insert((*pm)->clientObj);

				if ( (*pm)->clientObj ) {
					bool res = (*pm)->dChannel->play((*pm)->clientObj->getDataSinkId(), RTP_PREFETCH_SECS);
					dprintf_full("RemotePresentation::play() adding client %s - %s\r\n",
						     (*pm)->clientObj->getAddress(),
						     res ? "success" : "FAILED");
				} else {
					dprintf_err("RemotePresentation::play Failed to add NULL client???\r\n");
				}

				// if not yet started -> start it
				if (!(*pm)->dChannel->running()) {
					dprintf_full("RemotePresentation::play -> Thread not running...starting now!\n");
					(*pm)->dChannel->start();
				}
				else
					dprintf_full("RemotePresentation::play -> Thread already running... fine...\n");
			} //for
			if (state == SESSION_PAUSED)
			  globalTimer->setPaused(false);
			state=SESSION_PLAYING;
		} //if
	} //mp4Stream
	else {
	  dprintf_err("RemotePresentation::play: FAILED. No ContainerInfo object\r\n");

	  return false;
	}

	return state == SESSION_PLAYING;
};

bool RemotePresentation::pause(u32 sessionId)
{
	// FIXME: search for the appropriate session
	dprintf_full("RemotePresentation::pause sessionId %i state %i\n",sessionId,state);
	// pause it
	if(state == SESSION_PLAYING) {
		for( list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
		        /* send PAUSE request */
			protocol->generatePause(this->url,(*pm)->dChannel->getSessionId());
			const char* req=protocol->getBuffer();

			if(!this->sendResponse(req,strlen(req))) {
				return false;
			}
			char buffer[MSG_BUFFER_SIZE];
//			char **buffer = (char**) malloc(sizeof(char));
			int len;
			waitForMessage(socket, len, buffer);
			if(len<0)
				return false;
//			buffer[len]=0;
			if(!strstr(buffer,"200")) // status code for OK
				state=SESSION_ERR;
			else
				state=SESSION_PAUSED;

			/* pause DataChannel */
			bool res = (*pm)->dChannel->pause((*pm)->clientObj->getDataSinkId());
			if(!res) {
			  state = SESSION_ERR;
			}

		        /* clear Adaptor and DataSink buffers */
		        (*pm)->dChannel->getAdaptor()->close();
			if ((*pm)->audioOutput)
			  (*pm)->audioOutput->clearBuffer();
		}
		globalTimer->setPaused(true);
		presState = PRES_PAUSED;
	}
	return state==SESSION_PAUSED;
};

bool RemotePresentation::isPaused() {
	return globalTimer->isPaused();
}

bool RemotePresentation::isPrefetching() {
	return globalTimer->isPrefetching();
}

bool RemotePresentation::mute(u32 sessionKey,bool muteAudio, bool muteVideo) {
	if(state == SESSION_PLAYING) {
		for( list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
			if(muteAudio && muteVideo) {
				(*pm)->dChannel->mute((*pm)->clientObj->getDataSinkId());
			}
			else
				if(muteAudio) {
					if((*pm)->esInfo->isAudioStream()) {
						(*pm)->dChannel->mute((*pm)->clientObj->getDataSinkId());
					}
				}
				else {
					if(muteVideo)
						if((*pm)->esInfo->isVisualStream()) {
							(*pm)->dChannel->mute((*pm)->clientObj->getDataSinkId());
						}
				}
		}
		presState = PRES_MUTED;
		return true;

	}
	return false;
}
bool RemotePresentation::unmute(u32 sessionKey,bool muteAudio, bool muteVideo) {
	if(state == SESSION_PLAYING) {
		for( list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
			if(muteAudio && muteVideo) {
				(*pm)->dChannel->unmute((*pm)->clientObj->getDataSinkId());

			}
			else
				if(muteAudio) {
					if((*pm)->esInfo->isAudioStream()) {
						(*pm)->dChannel->unmute((*pm)->clientObj->getDataSinkId());
					}
				}
				else {
					if(muteVideo)
						if((*pm)->esInfo->isVisualStream()) {
							(*pm)->dChannel->unmute((*pm)->clientObj->getDataSinkId());
						}
				}
		}
		presState = PRES_PLAYING;
		return true;
	}
	return false;
}
bool RemotePresentation::tearDown(u32 sessionId) {

	if(sessionId==TEARDOWN_ALL) {
		// teardown all
		state = SESSION_CLOSED;
		list < Rtp *>::iterator rtpChan;

		list <presentationManagement *>::iterator pm;
		while(!presManage.empty()) {
			pm = presManage.begin();
			rtpChan = rtp_channels.begin();
			if((*pm)->dChannel) {
				dprintf_full("RemotePresentation::tearDown -> TEARDOWN 0\n");
				protocol->generateTeardown(this->url,(*pm)->dChannel->getSessionId());
				const char* req=protocol->getBuffer();
				dprintf_full("RemotePresentation::tearDown -> TEARDOWN 1\n");
				if(req) {
					this->sendResponse(req,strlen(req));
					dprintf_full("RemotePresentation::tearDown -> TEARDOWN 2\n");
				}
				(*pm)->dChannel->teardown((*pm)->clientObj->getDataSinkId(),true);
				dprintf_full("RemotePresentation::tearDown -> TEARDOWN 3\n");
				(*rtpChan)->close(true);
				dprintf_full("RemotePresentation::tearDown -> TEARDOWN 4\n");

				// check if we have to delete it, might be shared with other sessions
				if((*pm)->dChannel->getNumberOfActiveDataSinks()==0) {
					(*pm)->dChannel->wait(); // wait for thread to finish
					delete (*pm)->dChannel;
				//	(*pm)->dChannel = NULL;
				}
				dprintf_full("RemotePresentation::tearDown -> TEARDOWN 5\n");

			//	*rtpChan = NULL;
				rtp_channels.pop_front();

			} else { // no dChannel yet
				if((*pm)->clientObj)
					delete (*pm)->clientObj; //also deletes video+audioOutput
				if ((*pm)->adaptorChain)
					delete (*pm)->adaptorChain;
			}
			if(*pm)
				delete (*pm);
			presManage.pop_front();
		}

		if (receiver != NULL) {
			receiver->deactivate();
			delete receiver;
			receiver = NULL;
		}

		#ifdef WIN32
		closesocket(socket);
		#else
		close(socket);
		#endif


		if (protocol) {
			delete protocol;
			protocol = NULL;
		}
		if (server) {
			delete server;
			server = NULL;
		}


		if (url) {
			delete url;
			url = NULL;
		}
		presManage.clear();
		origESInfo.clear();
		actESInfo.clear();

		rtp_channels.clear();
		lastX = 0;
		lastY = 0;
		state = SESSION_CLOSED;
	}


	/** very old code here! has to be reimplemented */

	/*	else {
	// teardown one session
	list < DataChannel * >::iterator dcI;
	if (!channels.empty()) {
	dcI = channels.begin();
	while (dcI != channels.end()) {
	if ((*dcI)->getSessionId()== sessionId) {
	char* req=protocol->generateTeardown(this->url,sessionId);
	if(req) {
	this->sendResponse(req,strlen(req));
	delete req;
	}
	(*dcI)->teardown(clientObj.front()->getDataSinkId(),true);
	// check if we have to delete it, might be shared with other sessions
	if((*dcI)->getNumberOfActiveDataSinks()==0) {
	(*dcI)->wait(); // wait for thread to finish
	delete (*dcI);
	}
	channels.erase(dcI);
	if(channels.empty())
	state = SESSION_CLOSED;
	return true;
	}
	else {
	++dcI;
	}
	}
	}

	*/
	//	return false; // delete on empty session
	//}

	return false;
};

void RemotePresentation::run() {
	int dummyDur = 0;
	if(!url) {
		dprintf_err("RemotePresentation::run(): No URL set. Can't continue\r\n");
		this->exit();
		return;
	}
	// url is set, check the session state
	if(this->state==SESSION_CLOSED) {
		// do a describe
		if(!this->connect())
			state=SESSION_ERR;
	}
	if(this->state==SESSION_CONNECTED) {
		// do setup
		if(mp4Stream==NULL) {
			state=SESSION_ERR;
			this->exit();
			return;
		}
		for( list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
			int clientPort=0;
			int serverPort=0;
			u32 streamId = (*pm)->esInfo->getStreamId();
			dprintf_full("RemotePresentation::run(): Visual streamID is %i\n",streamId);
			const char* buffer;
			int localPort = portGen.getNextFreePortPair();
			int localRtxPort=0;
			if(rtxEnabled) {
				localRtxPort = portGen.getNextFreePortPair();
				protocol->generateSetup(url,streamId,localPort,true,
																localRtxPort);
			} else
				protocol->generateSetup(url,streamId,localPort,true);
			buffer=protocol->getBuffer();
//			bool err=false;
			if(buffer) {
				sendResponse(buffer, strlen(buffer));
//				err= (sessionControlChannel->writeBlock(buffer,strlen(buffer)) != (int) strlen(buffer));
			}
/*			if(err) {
				portGen.closePortPair(localPort);
				if (localRtxPort)
					portGen.closePortPair(localRtxPort);
				state=SESSION_ERR;
			}
			else*/
				state=SESSION_INITIALIZED;

			if(state==SESSION_INITIALIZED) {
				// parse the response and create the RTP class
				char response[MSG_BUFFER_SIZE];
//				char **response = (char**) malloc(sizeof(char));
				int len;
				waitForMessage(socket, len, response);
				if(len<=0) {
					state=SESSION_ERR;
					this->exit();
					return;
				}
				//rtxEnabled = false;
				bool unicast=true;
				char* ssrc=NULL;
				int sessionID;
				rtx_info* rtx=NULL;
				if(!rtxInfo.empty()) {
					list<rtx_group*>::iterator li=rtxInfo.begin();
					while(li!=rtxInfo.end()) {
						if( (*li)->es && (*li)->es->getStreamId()==streamId) {
							rtx=(*li)->rtx;
							break;
						}
						else
							++li;
					}
				}
				if(protocol->parseResponseSetup(response,&sessionID,
																				&clientPort,&serverPort,&unicast,&ssrc,rtx)<4) {
					state=SESSION_ERR;
					dprintf_err("parseResponseSetup failed\r\n");
					if(ssrc)
						delete ssrc;
					this->exit();
					return;
				}

				sessionKey = sessionID;
				dprintf_full("Session is %i\n",sessionID);

				char localhost[10]="127.0.0.1";
				PacketizationLayer * p;
				if ((*pm)->esInfo->isAudioStream())
			                  p = new RFC2250();
			        else
			                  p = new RFC3016();

				Rtp* input=new Rtp(localhost,serverPort,
													 inet_ntoa(server->sin_addr),
													 clientPort,
													 (*pm)->esInfo,
													 p,
													 false, this->termCap,globalTimer,
													 new Statistics((*pm)->esInfo,false));
				if(rtx) {
					input->setRtxInfo(rtx);
				}
				if(dummyDur < (*pm)->esInfo->getDurationInMs()) {
					rtpChannelOfLongestES = input;
				}
				dummyDur = MAX(dummyDur,(*pm)->esInfo->getDurationInMs());
				dprintf_full("RemotePresentation::run -> Duration: %i\n",(int)(*pm)->esInfo->getDurationInMs());
				duration = dummyDur;
				rtp_channels.push_back(input);
				(*pm)->dChannel = new DataChannel(input,(*pm)->esInfo);
				(*pm)->dChannel->setSessionId(sessionID);
				(*pm)->dChannel->setAdaptor((*pm)->adaptorChain);
				if(ssrc)
					delete ssrc;
			}
		}
	}

	if(this->state==SESSION_INITIALIZED || this->state==SESSION_PAUSED) {

		// do a play
		if(!this->play()) {
			dprintf_full("RemotePresentation::run -> UNABLE to play\n");
			state=SESSION_ERR;
		}
	}

	/*	if(this->state==SESSION_PLAYING) {
	// we have tried to play
	// but we have to check the datachannels to be sure
	for (list < DataChannel * >::iterator i = this->channels.begin();
	i != this->channels.end(); i++) {
	if( (*i)->running())
	(*i)->wait();
	}
	state=SESSION_PAUSED;
	}

	*/
	this->exit();
}


/** creates a rtx_group for the given input params */
bool RemotePresentation::setRtxInfoForES(u32 esId, rtx_info* rtx)
{
	const ESInfo* es=this->mp4Stream->getES(esId);
	if(!es) {
		dprintf("RemotePresentation::setRtxInfoForES: invalid esId as input\r\n");
		return false;
	}
	rtx_group* grp=new rtx_group(ACK,3000);
	grp->es=es;
	grp->rtx=rtx;
	rtx->mId=rtxInfo.size()+1;
	grp->midESInfo=esId;
	rtxInfo.push_back(grp);
	return true;
};


/** returns an rtx_group for the esId, or NULL */
rtx_group* RemotePresentation::getRtxGroup(u32 esId)
{
	for(list<rtx_group*>::iterator li=rtxInfo.begin(); li!=rtxInfo.end(); li++) {
		if(esId==(*li)->es->getStreamId())
			return *li;
	}
	return NULL;

};

list <float> RemotePresentation::getFramesPerSecond(u32 interval) {
	list <float> fpsList;
	float fps = 0;
	u32 real_iv=0;
	u32 streamSec;

	fpsList.clear();

	for( list <presentationManagement *>::iterator pm = presManage.begin(); pm != presManage.end(); pm++) {
		fps=0;
		streamSec = (u32)floor(globalTimer->getActualSec());
		if (streamSec > 0) {
			if (streamSec < interval)
				real_iv = streamSec;
			else
				real_iv = interval;

			for (u32 i=1; i < real_iv+1; i++) {
				fps += (*pm)->stats->getPlayoutSecFPS((int)streamSec-i);

			}
			fps = fps / real_iv;
		}
		fpsList.push_back(fps);
	}
	return fpsList;
}

list <float> RemotePresentation::getOrigFramesPerSecond() {
	list <float> fpsList;
	float fps = 0;

	fpsList.clear();

	if (rtp_channels.empty())
		return fpsList;

	for (list <Rtp *>::iterator rtpChan = rtp_channels.begin();
			 rtpChan != rtp_channels.end();
			 rtpChan++) {
		fps = (*rtpChan)->getStatistics()->getOriginalFPS();
		fpsList.push_back(fps);
	}
	return fpsList;
}

float RemotePresentation::getActualSecOfLongestES() {
	if(rtpChannelOfLongestES) {
		return rtpChannelOfLongestES->getStatistics()->getPlayoutSec();
	}
	else return 0;
}

int RemotePresentation::getTotalAverageBandwidth(u32 interval) {
	long avBW = 0;
	long totavBW = 0;
	u32 real_iv=0;
	u32 streamoutSec;
	Statistics *stats;

	if (rtp_channels.empty())
		return -1;

	for (list <Rtp *>::iterator rtpChan = rtp_channels.begin();
			 rtpChan != rtp_channels.end();
			 rtpChan++) {
		stats = (*rtpChan)->getStatistics();
		avBW = 0;
		streamoutSec = stats->getStreamoutSec();
		if (streamoutSec > 0) {
			if (streamoutSec < interval)
				real_iv =  streamoutSec;
			else
				real_iv = interval;

			for (u32 i=1; i < real_iv+1; i++) {
				avBW += stats->getBWfromStreamoutSec(streamoutSec-i);
			}
			avBW = avBW / real_iv;
			totavBW += avBW;
		}
	}

	return totavBW;
}

list <int> RemotePresentation::getAverageBandwidth(u32 interval) {
	list <int> aBWList;
	long avBW;
	u32 real_iv=0;
	Statistics *stats;

	aBWList.clear();

	if (rtp_channels.empty())
		return aBWList;

	for (list <Rtp *>::iterator rtpChan = rtp_channels.begin();
			 rtpChan != rtp_channels.end();
			 rtpChan++) {
		stats = (*rtpChan)->getStatistics();
		avBW = 0;
		if (stats->getStreamoutSec() > 0) {
			if (stats->getStreamoutSec() < interval)
				real_iv =  stats->getStreamoutSec();
			else
				real_iv = interval;

			for (u32 i=1; i < real_iv+1; i++) {
				avBW += stats->getBWfromStreamoutSec(stats->getStreamoutSec()-i);
			}
			avBW = avBW / real_iv;
		}
		aBWList.push_back(avBW);
	}

	return aBWList;
}

list <float> RemotePresentation::getActualSec() {
	list <float> secList;
	if (!rtp_channels.empty()) {
		for(list <Rtp *>::iterator rtpChan = rtp_channels.begin(); rtpChan != rtp_channels.end(); rtpChan++) {
			secList.push_back((*rtpChan)->getStatistics()->getPlayoutSec());
		}
		return secList;
	}
	else {
		secList.clear();
		return secList;
	}
}

list <int> RemotePresentation::getBufferFillLevel() {
	list <int> fillLevelList;
	if (!rtp_channels.empty()) {
		for(list <Rtp *>::iterator rtpChan = rtp_channels.begin(); rtpChan != rtp_channels.end(); rtpChan++) {
			fillLevelList.push_back((*rtpChan)->getBufferFillLevelInBytes());
		}
		return fillLevelList;
	}
	else {
		fillLevelList.clear();
		return fillLevelList;
	}
}

list <double> RemotePresentation::getBufferFillLevelInSecs() {
	list <double> secFillLevelList;
	if (!rtp_channels.empty()) {
		for(list <Rtp *>::iterator rtpChan = rtp_channels.begin(); rtpChan != rtp_channels.end(); rtpChan++) {
			secFillLevelList.push_back((*rtpChan)->getStatistics()->getBufferFillLevelInSecs());
		}
		return secFillLevelList;
	}
	else {
		secFillLevelList.clear();
		return secFillLevelList;
	}

}

const char * RemotePresentation::getPresentationName() {
	Url url(uri);
	return (const char *) url.getPath();
}

/***********************************************************************/
bool RemotePresentation::lookForMessage(char *name, char *buffer, int &bytesRead)
{
	assert(name);
	if (recmessages.size() > 0) {
		vector<char*>::iterator iter = recmessages.begin();
		while (iter != recmessages.end()) {
			char  *msg = *iter;
			if (strstr(msg, name) != NULL) {
	//			*buffer = msg; //return pointer to that element
				memcpy(buffer, msg, strlen(msg)+1);
				bytesRead = strlen(msg);

				recmessages.erase(iter);
				delete msg;

				return true;
			}
			iter++;
		}
	}
	return false;
}

/***********************************************************************/
void RemotePresentation::receiveMessage()
{
	if (socket) {
		fd_set rfds;
		FD_ZERO(&rfds);
		select(socket+1, &rfds, NULL, NULL, &readtv);
		if (!FD_ISSET(socket, &rfds)) {
			uint n = 0;
			int bytesRead = 0;
			bool abortReading = false;
			while (abortReading == false) {
				assert(socket != 0);
				n = recv(socket, readBuffer + bytesRead, MSG_BUFFER_SIZE, 0);
				bytesRead += n;

				FD_ZERO(&rfds);
				select(socket+1, &rfds, NULL, NULL, &readtv);
				if (!FD_ISSET(socket, &rfds)) abortReading = true;
			}

			if (bytesRead > 0) {
				assert(readBuffer);
				char *newmsg = new char[bytesRead+1];
				memcpy(newmsg, readBuffer, bytesRead);
				newmsg[bytesRead] = 0;
				dprintf_full("RemotePresentation::receiveMessage: %s (%d/%d)\n",newmsg, bytesRead, strlen(newmsg));
				recmessages.push_back(newmsg);
			}
		}
	}
}


/***********************************************************************/
void RemotePresentation::waitForMessage(int sock, int &bytesRead, char *mybuffer)
{
	if (socket) {
		char name[11];
		sprintf(name, "CSeq: %i", protocol->getSeqNr()-1);

		bool abort = false;
		do {
			if (lookForMessage(name, mybuffer, bytesRead) == true) {
				abort = true;
			} else {
				msleep(WAIT_MSECS_ON_RECEIVE);
			}
		} while (abort == false);
	} else {
		bytesRead = -1;
	}
}


/***********************************************************************/
void RemotePresentation::handleAnnounce(char *buffer)
{
	protocol->generateNotImplemented();
	const char *msg = protocol->getBuffer();
	sendResponse(msg,strlen(msg));
}

/***********************************************************************/
void RemotePresentation::handleGetParameter(char *buffer)
{
	const char *msg;

	if (strstr(buffer, "Content-Length: 12\r\n\r\nposition\r\n") != NULL) {
		//only look on the first stream for gathering the playout time
		presentationManagement *pm = presManage.front();
		if (pm != NULL) {
			u32 rtptimems = (u32) (((Rtp*)(pm)->dChannel->getInput())->getStatistics()->getPlayoutSec() * 1000);

			//add result into map and generate response msg
			std::map<char*, char*, struct stringCompare>* param=new std::map<char*, char*, stringCompare>();
			char rtptimesStr[11];
			sprintf(rtptimesStr,"%d", rtptimems);
			std::pair<char*, char*> pos("position", rtptimesStr);
			param->insert(param->begin(), pos);

			protocol->generateResponseGetParameter(param);
			msg=protocol->getBuffer();
			dprintf("\nRemotePresentation::handleGetParameter: sent position: %s", rtptimesStr);
		} else {
			protocol->generateNotImplemented();
			msg = protocol->getBuffer();
		}
	} else {
		protocol->generateNotImplemented();
		msg = protocol->getBuffer();
	}
	sendResponse(msg,strlen(msg));
}

/***********************************************************************/
void RemotePresentation::handleOptions(char *buffer)
{
	protocol->generateNotImplemented();
	const char *msg = protocol->getBuffer();
	sendResponse(msg,strlen(msg));
}

/***********************************************************************/
void RemotePresentation::handleRedirect(char *buffer)
{
	protocol->generateNotImplemented();
	const char *msg = protocol->getBuffer();
	sendResponse(msg,strlen(msg));
}

/***********************************************************************/
void RemotePresentation::handleSetParameter(char *buffer)
{
	protocol->generateNotImplemented();
	const char *msg = protocol->getBuffer();
	sendResponse(msg,strlen(msg));
}

/***********************************************************************/
void RemotePresentation::handleTeardown(char *buffer)
{
	if (this->mvpd) this->mvpd->stop();
//	protocol->generateNotImplemented();
//	const char *msg = protocol->getBuffer();
//	sendResponse(msg,strlen(msg));
}
