 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	TermCapsRemoteImpl.cpp                                        *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "TermCapsRemoteImpl.hpp"

#include <qlabel.h>
#include <qaction.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qslider.h>
#include <iostream>
using namespace std;
/* 
 *  Constructs a TermCapsRemote which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */


TermCapsRemote::TermCapsRemote( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : TermCapsRemoteBase( parent, name, modal, fl )
{
	videoWidth = 352;
	videoHeight = 288;
	rtxEnabled = true;
	colorEnabled = true;
	setCaption("Terminal Capabilities");
	dimX->setEnabled(false);
	dimY->setEnabled(false);
	// not implemented
	//initMenuBar();
}

/*  
 *  Destroys the object and frees any allocated resources
 */
TermCapsRemote::~TermCapsRemote()
{
    // no need to delete child widgets, Qt does it all for us
}

void TermCapsRemote::initMenuBar() {
	QAction *fileNewAction = new QAction( "New", "&New Configuration", CTRL+Key_N, this, "new_config" );
	QAction *fileLoadAction = new QAction( "Load", "&Load Configuration", CTRL+Key_L, this, "load_config" );
	QAction *fileSaveAction = new QAction( "Save", "&Save Configuration", CTRL+Key_S, this, "save_config" );
   
	QPopupMenu *fileMenu = new QPopupMenu( NULL );
	QMenuBar * menBar = new QMenuBar(this);
	menBar->insertItem( "&File", fileMenu );
	menBar->insertSeparator();
	fileNewAction->addTo( fileMenu );
	fileLoadAction->addTo( fileMenu );
	fileSaveAction->addTo( fileMenu );

	
	connect( fileNewAction, SIGNAL( activated() ),this, SLOT( new_config() ) );
	connect( fileLoadAction, SIGNAL( activated() ),this, SLOT( load_config() ) );
	connect( fileSaveAction, SIGNAL( activated() ),this, SLOT( save_config() ) );

}

void TermCapsRemote::setTermcapsOfPresentation(TerminalCapabilities * termCaps, bool rtxEnabled, bool upscale, bool keepAspect, bool enableSwitching) {
	if((termCaps->getDisplayWidth() == 176) && (termCaps->getDisplayHeight() == 144)) {
		setQCIF();
		qcifRadioButton->setChecked(true);
	} else
		if((termCaps->getDisplayWidth() == 352) && (termCaps->getDisplayHeight() == 288)) {
			setCIF();
			cifRadioButton->setChecked(true);
		} else
			if((termCaps->getDisplayWidth() == 704) && (termCaps->getDisplayHeight() == 576)) {
				setPAL();
				palRadioButton->setChecked(true);
			} else {
					userDimRadioButton->setChecked(true);
					dimX->setValue(termCaps->getDisplayWidth());
					dimY->setValue(termCaps->getDisplayHeight());
					dimX->setEnabled(true);
					dimY->setEnabled(true);
			}

	videoWidth = termCaps->getDisplayWidth();
	videoHeight = termCaps->getDisplayHeight();

	if(termCaps->getColorDisplay() )
		enableColorDisplay->setChecked( true );
	else
		enableColorDisplay->setChecked( false );

	if(rtxEnabled) 
		enableRTX->setChecked(true);
	else
		enableRTX->setChecked(false);

	checkUpscale->setChecked(upscale);
	checkAspect->setChecked(keepAspect);
	switchAllow->setChecked(enableSwitching);
	/* max value is 1 Mbit/s */
	Slider2->setMaxValue(1024);
	Slider2->setValue(termCaps->getNetworkCapacityInByte()/128); // == / 1024 *8
	
	
	bwvalue->setNum((int)termCaps->getNetworkCapacityInByte()/128);
}


void TermCapsRemote::set_val(int num) {
	if(num == 0)
		bwvalue->setNum(10);
	else
		bwvalue->setNum(num);
	bwvalue->show();
}

void TermCapsRemote::setPAL() {
	videoWidth = 704;
	videoHeight = 576;
	dimX->setValue(videoWidth);
	dimY->setValue(videoHeight);
	dimX->setEnabled(false);
	dimY->setEnabled(false);
}

void TermCapsRemote::setCIF() {
	videoWidth = 352;
	videoHeight = 288;
	dimX->setValue(videoWidth);
	dimY->setValue(videoHeight);
	dimX->setEnabled(false);
	dimY->setEnabled(false);
}

void TermCapsRemote::setQCIF() {
	videoWidth = 176;
	videoHeight = 144;
	dimX->setValue(videoWidth);
	dimY->setValue(videoHeight);
	dimX->setEnabled(false);
	dimY->setEnabled(false);
}

void TermCapsRemote::setUserDefinedX() {
	videoWidth = (dimX->text()).toInt();
}

void TermCapsRemote::setUserDefinedY() {
	videoHeight = (dimY->text()).toInt();
}

void TermCapsRemote::setRTX() {
	if(enableRTX->isChecked()) {
		rtxEnabled = true;
	}
	else 
		rtxEnabled = false;
}

void TermCapsRemote::setColorDisplay() {
	if(enableColorDisplay->isChecked()) {
		colorEnabled = true;
	}
	else
		colorEnabled = false;
}

void TermCapsRemote::enableDimXY() {
	dimX->setEnabled(true);
	dimY->setEnabled(true);
}
