 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	VideoDisplayIO.cpp                                            *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "io/Rtp.hpp"
#include "VideoDisplayIO.hpp"
#include "Statistics.hpp"

#include "pgdraw.h"


using namespace PG_Draw;


VideoDisplayIO::VideoDisplayIO(PG_ThemeWidget* parent, PG_Rect r,bool RGB32_Output)
	         : PG_ThemeWidget(parent,r,true)  {

	num_frames = 0;
	muiHeight = 0;
	muiWidth = 0;
	muiTicksPerSecond = 0;
	sec = 0;
	usec = 0;
	mbOpen = false;
	state = CLOSED;
	surf = NULL;
	firstFrame = true;
	wasPaused = false;
	pause_dtime = 0.0;
	now_dtime = 0.0;
	start_dtime = 0.0;
	rgb32 = RGB32_Output;
	fFrame = NULL;
	rect.x = 2;
	rect.y = 2;
	rect.w = Width()-4;
	rect.h = Height()-4;
}


VideoDisplayIO::~VideoDisplayIO()
{
	mbOpen = false;
	state = CLOSED;
	if (rgb32) {
		surf = NULL;
	}
	else {
		SDL_FreeYUVOverlay(yuvOverlay);

		yuvOverlay = NULL;

		dprintf_full("VideoDisplayIO::~VideoDisplayIO -> Delete SURFACE\n");

	}
}

void VideoDisplayIO::setESInfo(ESInfo * esi)
{
	this->es = (VideoESInfo*)esi;
	muiHeight = es->getHeight();
	muiWidth = es->getWidth();
	ySize = muiWidth * muiHeight;
	uvSize = ySize / 4;

	muiTicksPerSecond=es->getMediaTimeScale();
	if(muiTicksPerSecond==0)
		muiTicksPerSecond=1;
}



void VideoDisplayIO::setTicksPerSecond(u32 uiTicksPerSecond)
{
	muiTicksPerSecond = uiTicksPerSecond;
}

void VideoDisplayIO::initialize()
{
}

Frame * VideoDisplayIO::getFrame()
{
	return NULL;
}

/***************************************************************/
IO::State VideoDisplayIO::play(double prefetchTime) {

	dprintf_err("VideoDisplayIO::play prefetch %f (old state %i)\n",prefetchTime,state);

	if (state==CLOSED) {
		open();
		return state;
	} else
		if ( (state==PAUSED) || (state==MUTED) ) 
			setState(OPEN);
		else {
			dprintf_err("VideoDisplayIO::play tried to OPEN from state %i\n",state);
			return state;
		}
	return state;
}


int VideoDisplayIO::writeFrame(Frame * frm, ESInfo *out_es)
{	

	if (SDL_PollEvent( &event )) {
		switch( event.type ){
		case SDL_KEYUP:{
			SDL_KeyboardEvent *key = &event.key;
			char *buf = SDL_GetKeyName( key->keysym.sym );
			char c;
			if (buf[1]=='\0') //only single character (a-zA-Z...)
				c = toupper(buf[0]);
			if ( ((key->keysym.mod & KMOD_CTRL)  && (c == 'C')) ||
					 (strstr(buf, "escape") != NULL)) {
				dprintf_err("VideoDisplayIO::writeFrame   STREAMERR because of CTRL-C/ESC!\n");
				state = STREAMERR;
				return 0;
			}
			break;
		}
		case SDL_KEYDOWN:
			break;
			
			/* SDL_QUIT event (window close) */
		case SDL_QUIT:
			dprintf_err("VideoDisplayIO::writeFrame   STREAMERR because of window close!\n");		
			state = STREAMERR;
			return 0;
			break;
			
		default:
			break;
    }
	}
	
	//just an experiment
	if(firstFrame) {
		//	*fFrame = frm;
		if(frm->getType() == Frame::I_VOP) {
			dprintf_full("VideoDisplayIO::writeFrame: First Frame is IVOP\n");
		}
		firstFrame = false;
	}
	
	assert(frm->getAU()->payload != NULL);
	
	//frame size might change after switching
	if ((muiWidth != (unsigned)es->getWidth()) || (muiHeight != (unsigned)es->getHeight())) {
		muiWidth = es->getWidth();
		muiHeight = es->getHeight();
		SDL_FreeYUVOverlay(yuvOverlay);
		dprintf_full("VideoDisplayIO::writeFrame detected Switching, reinit of SDL to YV12! \n");
		//	yuvOverlay = SDL_CreateYUVOverlay(muiWidth,muiHeight,SDL_IYUV_OVERLAY,GetWidgetSurface());
		yuvOverlay = SDL_CreateYUVOverlay(muiWidth,muiHeight,SDL_YV12_OVERLAY,GetWidgetSurface());
		ySize = muiWidth * muiHeight;
		uvSize = ySize / 4;
	}
	
	dprintf_full("VideoDisplayIO::writeFrame:  writing No %li  size: %i bytes (YUV check %i), %ix%i\n",
							 num_frames, frm->getAU()->size,ySize+uvSize+uvSize,muiWidth,muiHeight);
	assert(frm->getAU()->size == (unsigned)(ySize + uvSize + uvSize));
	
	if(IO::getState() != IO:: MUTED) {
		if (SDL_MUSTLOCK(GetWidgetSurface())) {
			SDL_LockSurface(GetWidgetSurface());
		}
		
		if (rgb32) {
			surf->pixels = NULL;
			SDL_LockSurface(surf);
			surf->pixels = frm->getAU()->payload;
			SDL_UnlockSurface(surf);
			BlitScale(surf,GetWidgetSurface());
			dprintf_full("VideoDisplayIO::writeFrame -> Displaying RGB Frame \n");
		}
		else {
			dprintf_full("VideoDisplayIO::writeFrame -> Displaying YUV Frame %ix%i\n",muiWidth,muiHeight);
			
			if(SDL_LockYUVOverlay(yuvOverlay) !=0) {
				return -1;
			}
			memset(yuvOverlay->pixels[0],0,ySize);
			memset(yuvOverlay->pixels[1],0,uvSize);
			memset(yuvOverlay->pixels[2],0,uvSize);
			
			//Y part
			memcpy(yuvOverlay->pixels[0], frm->getAU()->payload, ySize);

			//for (int i=0; i < ySize; i++)
			//	printf("%3i:",frm->getAU()->payload[i]);
			//printf("\n");

			//U Part
			memcpy(yuvOverlay->pixels[2], frm->getAU()->payload + ySize, uvSize);

			/*
			for (int i=0; i < uvSize; i++) {
				if (i % (muiWidth) == 0)
					frm->getAU()->payload[ySize+uvSize+i]=128;
				else
					frm->getAU()->payload[ySize+uvSize+i]=0;

				printf("%3i:",frm->getAU()->payload[ySize+uvSize+i]);
			}
			printf("\n");
			*/

			//V Part
			memcpy(yuvOverlay->pixels[1], frm->getAU()->payload + ySize + uvSize, uvSize);



			SDL_DisplayYUVOverlay(yuvOverlay, &rect);
			SDL_UnlockYUVOverlay(yuvOverlay);
			dprintf_full("VideoDisplayIO::writeFrame -> Displaying YUV Frame \n");
		}
		
		num_frames++;
		Update();
		if (SDL_MUSTLOCK(GetWidgetSurface())) {
			SDL_UnlockSurface(GetWidgetSurface());
		}		
 	}
#ifdef _POSIX_PRIORITY_SCHEDULING
  sched_yield(); //this is necessary to give parallel getFrames a chance
#endif
	
	return 1;
}


bool VideoDisplayIO::open()
{
	if(!mbOpen) {
		if(rgb32) {
			dprintf_full("VideoDisplayIO::open  Display RGB32\n");
			Uint32 rmask, gmask, bmask, amask;
			gmask = 0xff000000;
			rmask = 0x00ff0000;
			bmask = 0x0000ff00;
			amask = 0x00000000;
			surf = NULL;
			surf = SDL_CreateRGBSurface(SDL_SWSURFACE, muiWidth, muiHeight, 32,
						                          rmask, gmask, bmask, amask);
	
			if (surf == NULL) {
				cerr << "Couldn't initialize blitbuffer with dimension " << muiWidth <<
				"x" << muiHeight << "!\n";
				return false;		
			}
		}
		else {
			dprintf_full("VideoDisplayIO::open SDL Display with YV12\n");
			SDL_Surface *surface = SDL_SetVideoMode(muiWidth, muiHeight, 0, SDL_SWSURFACE);

			//yuvOverlay = SDL_CreateYUVOverlay(muiWidth,muiHeight,SDL_IYUV_OVERLAY,GetWidgetSurface());
			//yuvOverlay = SDL_CreateYUVOverlay(muiWidth,muiHeight,SDL_YV12_OVERLAY,surface);
			yuvOverlay = SDL_CreateYUVOverlay(muiWidth,muiHeight,SDL_YV12_OVERLAY,GetWidgetSurface());
			ySize = muiWidth * muiHeight;
			uvSize = ySize / 4;
		
		}
   		firstFrame = true; 
		state = OPEN;
	}
	return true; 
}

bool VideoDisplayIO::close(bool cl)
{
	destroy();
	state = CLOSED;
	mbOpen = false;
	return true;
}


bool VideoDisplayIO::destroy()
{
	return false;
}


int VideoDisplayIO::getBufferFillLevel() const
{
	if (state == OPEN)
	return 1;
    else if (state == STREAMEOF)
	return -1;
    else
	return 0;
}


const char *VideoDisplayIO::getURL() const
{
	return NULL;
}

void VideoDisplayIO::setDelay(int delay) {

}

void VideoDisplayIO::setDimension(int width, int height) {

		
		SizeWidget(width,height);
		Redraw();
		Update();
		rect.x = 2;
		rect.y = 2;
		rect.w = width-2;
		rect.h = height-2;
}

void VideoDisplayIO::run() 
{
}

 
void VideoDisplayIO::drawFrame(bool active) {
	SDL_Color col;
	if(active) {
		col.r = 230;
		col.g = 0;
		col.b = 0;
	}
	else {
		col.r = 0;
		col.g = 0;
		col.b = 0;
	}
	int drawW = rect.w+2;
	int drawH = rect.h+2;
	DrawLine(0,0,drawW,0,col,2);
	DrawLine(0,drawH,drawW,drawH,col,2);
	DrawLine(0,0,0,drawH,col,2);
	DrawLine(drawW,0,drawW,drawH,col,2);
	Update();
}
