 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	VideoDisplayIO.hpp                                            *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _VIDEODISPLAY_HPP
#define _VIDEODISPLAY_HPP

#include "pgthemewidget.h"
#include "VideoESInfo.hpp"
#include "Frame.hpp"
#include "Adaptor.hpp"
#include "ContainerInfo.hpp"

#ifdef WIN32
#include <sys/timeb.h>
#else
#include <sys/time.h>
#endif
#include <time.h>

class Statistics;

/**
 *
 * <Class Description goes here>
 *
 *@author Ronald Sowa
 *
 *@version $Id: VideoDisplayIO.hpp,v 1.10 2006/01/26 11:41:54 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class VideoDisplayIO : public PG_ThemeWidget, public IO {

public:

	VideoDisplayIO(PG_ThemeWidget* parent, PG_Rect r,bool RGB32_Output);
	~VideoDisplayIO();

	void initialize();
	void setESInfo(ESInfo * es);
	ESInfo* getESInfo() { return (ESInfo*) es;};	
	void setTicksPerSecond(u32 uiTicksPerSeconds);
	void setDelay(int delay);
	void setDimension(int width, int height);

	// IO Methods
	Frame * getFrame() ;
    /** returns the number of packets sent. Returns 0 on error */
	int writeFrame(Frame * frm, ESInfo *out_es=NULL);

    /** opens the IO connection. State is set to OPENING. Depending on the
        underlying QIODevice, a network connection or a file connection
	is established. When the connection is ready for use, State is OPEN */
    bool open();
    bool close(bool cl);
    /** deletes the data source for this IO class. Will only work with
      * local files, remote files can not be deleted and will always return true.
      */
    bool destroy();
	IO::State play(double prefetchTime);

  //  State getState() const;
    /** returns a value  from 0..100 indicating buffer usage */
    int getBufferFillLevel() const;
    /** returns a pointer to the local file name or an URL */
    const char *getURL() const;
    /** repositions the IO class to the given frame.
      * Will return false in the following cases:
      *  - an illegal frame number (too large) was specified
      *  - the stream is not seekable, because the underlying 
      *    device is a network device and the requested frame 
      *    is already out of range: FIXME: implement
      */
    bool setToFrameNumber(u32 frameNumber) {
		return false;
    };

	Frame * getFirstFrame() {
		return fFrame;
	}

    void run();

	void setFirstFrame(bool firstFrame) {
		wasPaused = firstFrame;
	};

	void drawFrame(bool active);



private:
		
	u32 muiWidth;
    u32 muiHeight;
	SDL_Surface * surf;
	SDL_Overlay * yuvOverlay;
	SDL_Rect rect;
	SDL_Event event;
	bool rgb32;

	int  ySize;
    int  uvSize;

	bool mbOpen;

    const VideoESInfo *es;
    
	Frame * fFrame;
    u32 muiTicksPerSecond;
	bool firstFrame;
	bool wasPaused;
	double start_dtime;
	double pause_dtime;
	double now_dtime;
	long num_frames;
    struct timeval tv;
	double sec;
	double usec;

};
#endif
