 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: main.cpp                                                      *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#define NOMINMAX

/*
#ifdef WIN32
# include <windows.h>
#endif
*/


#include <qapplication.h>
#include <qfont.h>
#include <qpushbutton.h>
#include <qstring.h>
#include <qdialog.h>
#include <signal.h>
#include "IntroDialog.hpp"
#include "SDL.h"
#include "vitooki.hpp"
#include "MultiVideoPlayerDialog.hpp"


MultiVideoPlayerDialog * dialog = NULL;



// destructing sig or quit
void sighandler(int sig)
{
  switch(sig) {
  case SIGINT:
    dprintf_full("handling SIGINT, bye!\n");
	if(dialog) {
		dialog->close();
		delete dialog;
	}
	exit(0);
	break;
  default:
    dprintf("got unhandled signal...\n");
  }
}


void Usage() {
	cout << endl;
	cout << endl;
	cout << endl;
	cout << "Vitooki muViPlayer" << endl;
	cout << "------------------" << endl;
	cout << endl;
	cout << "Usage:  muViPlayer --help  | [Options] <media> " << endl;
	cout << "	--help | -h		show these usage hints" << endl;
	cout << "	<media>			input media that should be played. This can be: "<< endl;
	cout << "				[file://]path+fileName (from local disk) or" << endl;
	cout << "				rtsp://URL (from the network) or" <<endl;
	cout << "				prof://path+profileName (a profile)" << endl;
	cout << "Options: " << endl;
	cout << "	--immediatePlay		| -i	start to play immediately"  << endl;
	cout << "	--fullscreen		| -fs	play in fullscreen" << endl;
	cout << "	--noGUI			| -n	only show videowindow without control GUI" << endl;
	cout << "	--loop			| -l	play in a loop" << endl;
	cout << "	--exit-after-play	| -e	exit after playing one time " << endl;
	cout << "					has no effect when -l is selected" << endl;
	cout << "       --sdp                   interpret <media> as a local file containing" << endl;
	cout << "                               an SDP description referencing the media to be played." << endl;
	cout << "                               Recognized SDP fields:" << endl;
	cout << "                                 v=0                       SDP version (required)" << endl;
	cout << "                                 s=<session name>          start of session description" << endl;
	cout << "                                 a=control:<uri>           fully-qualified media URI" << endl;
	cout << "                                 a=range:npt=<start>-<end> media time range to be played" << endl;
	cout << "                                                           (in seconds, floating point)" << endl;
	cout << "                               Only rtsp: URIs are supported for <uri>." << endl;
}


void startSDLEventLoop() {

	SDL_Event event;
   // SDL_EventState (SDL_ACTIVEEVENT, SDL_IGNORE);
   // SDL_EventState (SDL_MOUSEMOTION, SDL_IGNORE);
	bool exitApp = false;
	while(!exitApp) {
	while (SDL_PollEvent (&event) && !exitApp) {
			switch (event.type) {
				case SDL_QUIT:
					exit(0);
					break;
				case SDL_KEYDOWN:
					switch(event.key.keysym.sym) {
						case SDLK_SPACE:
							dialog->play_pause();
							break;
						case SDLK_ESCAPE: {
							dialog->stop();
							exitApp = true;
										  }
							break;

						case SDLK_f:
							dialog->fullScreen();
						default:
							break;
					}
					default: break;
			}


	}
	SDL_Delay(250);
	dialog->endOfPresentation();
	}
}


int main( int argc, char** argv )
{



/*
	About * intro = new About(0,0,false,Qt::WStyle_Customize | Qt::WStyle_NoBorder | Qt::WStyle_Tool);
	intro->okButton->hide();
	intro->show();
	SDL_Delay(3000);

	intro->hide();
	intro->close();
	delete intro;
*/

	QString * media=NULL;
	int mediaType = -1;  // 0 --> local, 1 --> remote, 2 --> profile, 3--> http
	bool immediateplay = false;
	bool fullscreen = false;
	bool nogui = false;
	bool startQtLoop = true;
	bool loop = false;
	bool exitafterplay = false;
	bool loadsdp = false;
	int i=1;


	if (argc > 7) {
		cout << "Too many parameters! " << endl;
		cout << endl;
		Usage();
		return 1;
	}
	while (i < argc) {


		if (argv[i][0] == '-') { //options
			bool wrongArg = true;
			if ((strcmp(argv[i],"--help") == 0) || (strcmp(argv[i],"-h") == 0)) {
				Usage();
				return 0;
			}
			if((strcmp(argv[i],"--immediatePlay") == 0) || (strcmp(argv[i],"-i") == 0)) {
				//	cout << "ImmediatePlay selected" << endl;
				immediateplay = true;
				wrongArg = false;
			}
			if( (strcmp(argv[i],"--fullScreen") == 0) || (strcmp(argv[i],"-fs") == 0))  {
				//		cout << "Fullscreen selected" << endl;
				fullscreen = true;
				wrongArg = false;
			}
			if((strcmp(argv[i],"--noGUI") == 0) || (strcmp(argv[i],"-n") == 0)) {
				//	cout << "noGUI selected" << endl;
				nogui = true;
				wrongArg = false;
			}
			if((strcmp(argv[i],"--loop") == 0) || (strcmp(argv[i],"-l") == 0)) {
				loop = true;
				wrongArg = false;
			}
			if((strcmp(argv[i],"--exit-after-play") == 0) || (strcmp(argv[i],"-e") == 0)) {
				exitafterplay = true;
				wrongArg = false;
			}
			if (strcmp(argv[i], "--sdp") == 0) {
			        loadsdp = true;
			        wrongArg = false;
			}
			if(wrongArg) {
				Usage();
				cout << endl << "Wrong parameter: " << argv[i] << endl;
				return 1;
			}
		} else { // URIs
			media = new QString(argv[i]);
			//	cout << "Media: " << media->latin1() << endl;
			if( (media->left(7)).compare("file://") == 0) {
				mediaType = 0;
			}
			if((media->left(7)).compare("rtsp://") == 0) {
				mediaType = 1;
			}
			if((media->left(7)).compare("prof://") == 0) {
				mediaType = 2;
			}
			if((media->left(7)).compare("http://") == 0) {
				mediaType = 3;
			}

			if ((media->right(4)).compare(".sdp") == 0) {
				loadsdp = true;
				immediateplay = true;
			}
			if(mediaType == -1) {
				FILE *fp = fopen(media->latin1(),"r");
				if (!fp) {
					Usage();
					cout << endl << "Unrecognized URI format or file not found: " << argv[i] << endl;
					return 1;
				} else { //assuming local file
					fclose(fp);
					mediaType = 0;
					media->prepend("file://");
				}
			}
		}
		i++;
	} //while

	if (loadsdp && mediaType != 0) {
	  cout << "ERROR: --sdp requires local file!" << endl << endl;
	  Usage();
	  return 1;
	}

	vitooki_init_all();

	QApplication qapp( argc, argv );

	QFont font;
 	font.setPixelSize(14);
	qapp.setFont(font,true);
	dialog = new MultiVideoPlayerDialog ( 0, 0, false ,Qt::WStyle_Title |Qt::WStyle_SysMenu |Qt::WStyle_MinMax);
	dialog->enableLooping(loop);
	if((exitafterplay && !loop) || (nogui && !loop)) {
		dialog->exitAfterPlay();
	}
	qapp.setMainWidget(dialog);

	if(nogui) {
		dialog->hide();
	}
	else {
		dialog->show();
	}

	if (loadsdp) {
	  dialog->commandLine = true;
	  if (!dialog->newPresentationFromSDP(*media)) {
	    cout << "ERROR parsing SDP file!" << endl;
	    return 1;
	  }
	} else if (mediaType==0) {
		dialog->commandLine = true;
		dialog->fileName = new QString(media->remove(0,7));
		dialog->new_localWidget();
	} else if (mediaType==1) {
		dialog->commandLine = true;
		dialog->fileName = new QString(*media);
		dialog->new_remoteWidget();
	} else if (mediaType==2) {
		dialog->commandLine = true;
		dialog->profileName = new QString(media->remove(0,7));
		dialog->load_config();
	} else if (mediaType==3) {
		dialog->commandLine = true;
		dialog->fileName = new QString(*media);
		dialog->new_localWidget();    //fixme: http needs local widget because of rtsp handling in remotewidget...
		//dialog->new_remoteWidget();
	}

	if(nogui) {
		dialog->play();
		if(fullscreen)
			dialog->fullScreen();
		startSDLEventLoop();
		startQtLoop = false;
		immediateplay = false;
	} else if (fullscreen) {
		dialog->play();
		dialog->fullScreen();
		immediateplay = false;
	}

	if(immediateplay) {
		dialog->play();
	}

	atexit(SDL_Quit);
	signal(SIGINT,sighandler);
	if(startQtLoop)
		qapp.exec();
	return 0;
}

