 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	MP4Proxy.cpp                                                  *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

/** Proxy Test File */

#ifdef WIN32
#undef BOOL
#include <winsock2.h>
#include <crtdbg.h>
#endif

#include "sord/directory.h"
#include "sord/entry.h"
#include <assert.h>


#include "vitooki.hpp"
#include "../muviserver/muviserver/Server.hpp"
#include "Proxy.hpp"
#include "ContainerInfo.hpp"
#include "ESInfo.hpp"
#include "container/ContainerDemux.hpp"
#include "container/ContainerFile.hpp"
#include "net/Url.hpp"
#include "cache/CacheManagerLRU.hpp"
#include "cache/AdmissionControl.hpp"

void saveDefaultServerConfig(struct ProxyConfig* s) {
	strcpy(s->homeDir,".");
	s->recurseDirectories=false;
	s->port=3128;
	s->maxCacheSize=DEFAULTCACHESIZE;
#ifdef WIN32
	s->enableRtx=false;
#else
	s->enableRtx=true;
#endif
	Proxy::saveProxyConfig(s);
};

void loadRecursive(struct ProxyConfig* s, const char* subDirPrefix, CacheManager* cm, sord::directory &dir) {
	sord::directory::const_iterator &i = dir.begin();
	sord::directory::const_iterator &end = dir.end();

	for (; i != end; ++i) {
		dprintf_full("X%s\r\n",i->get_name());
		if (i->get_type() == sord::entry::type_directory) {
			if(s->recurseDirectories) {
				sord::directory subdir(*i);
				char *tmp=new char[strlen(subDirPrefix)+strlen(i->get_name())+2];
				sprintf(tmp,"%s/%s",subDirPrefix,i->get_name());
				loadRecursive(s,tmp,cm,subdir);
				delete tmp;
			}
		} 
		else if(i->get_type() == sord::entry::type_regular){
			const char* name=i->get_name();
			dprintf_full("MP4Server::loadRecursive %s\r\n",name);
			if(	strcmp( (name+strlen(name)-4), ".mp4")==0 ||
				strcmp( (name+strlen(name)-4), ".MP4")==0 ||
				strcmp( (name+strlen(name)-4), ".MPG")==0 ||
				strcmp( (name+strlen(name)-4), ".avi")==0 ||
				strcmp( (name+strlen(name)-4), ".AVI")==0 ||
				strcmp( (name+strlen(name)-4), ".mpg")==0) {
				// copy dir+name!
				char t[4096];
				strcpy(t,subDirPrefix);
				if(	t[strlen(t)-1]!='\\' || 
					t[strlen(t)-1]!='/') {
					strcat(t,"/");
				}
				strcat(t,i->get_name());
				dprintf_full("MP4Proxy::loadRecursive Full name: %s\r\n",t);
				// eliminate ./
				int offset=0;
				if(strncmp(t,
#ifdef WIN32
					".\\",
#else
					"./",
#endif
					2)==0) {
					offset=2;
				}
				// now check for config file
				char confFile[4096];
				ContainerInfo *mp4File = NULL;
				dprintf_full("MP4Proxy::loadRecursive relative name X%sX\r\n",&t[offset]);
				// build url
				char pTemp[4096];
				char* ptmp;
				ptmp=strcpy(pTemp,"rtsp://");
				
				gethostname(pTemp+strlen(pTemp),4096-strlen(pTemp));
#ifdef linux
        if (strstr(pTemp,".") == NULL) { //no domain added 
				   strcat(pTemp,".");
				   getdomainname(pTemp+strlen(pTemp),4096-strlen(pTemp));
				   dprintf_full("added FQHN %s\r\n",pTemp);
        }
#endif


				snprintf(pTemp+strlen(pTemp),4096-strlen(pTemp),"%s",t);
				dprintf_full("stored file as %s\r\n",pTemp);
				Url* url=new Url(pTemp);
				char fileOnDisk[1024];
				char demuxDir[1024];
				sprintf(fileOnDisk,"%s%s",s->homeDir, url->getPath());
				sprintf(confFile,"%s/%s%s.conf",s->homeDir,DEMUX_PATH,Url::transformPathToLocalFile(url));
				sprintf(demuxDir,"%s/%s",s->homeDir,DEMUX_PATH);
#ifdef WIN32
				unsigned int i=0;
				for(i=0;i<strlen(fileOnDisk);i++) {
					if(fileOnDisk[i]=='/')
						fileOnDisk[i]='\\';
				}
				for(i=0;i<strlen(confFile);i++) {
					if(confFile[i]=='/')
						confFile[i]='\\';
				}
				for(i=0;i<strlen(demuxDir);i++) {
					if(demuxDir[i]=='/')
						demuxDir[i]='\\';
				}
#endif

				FILE *tmpFp=fopen(confFile,"rb");
				if(tmpFp) {
					fclose(tmpFp);
					// we have already the demultiplexed es 
					mp4File=new ContainerInfo(pTemp,fileOnDisk,ContainerInfo::SDP);
					if(!mp4File->loadFromCfgFile(confFile)) {
						delete mp4File; 
						mp4File = ContainerFile::loadContainerInfo( fileOnDisk,url );	

						if(mp4File) {
							ContainerDemux::demultiplexAndUpdateIO(mp4File,demuxDir);
							mp4File->saveToCfgFile(confFile);
						}
					}
				}
				else {
					mp4File = ContainerFile::loadContainerInfo( fileOnDisk,url );
					
					if(mp4File) {
						dprintf_full("Before demultiplex\r\n");
					
						ContainerDemux::demultiplexAndUpdateIO(mp4File,demuxDir);
						
						if(mp4File->saveToCfgFile(confFile))
							fprintf(stderr, "Saved to confFile\n");
						else
							fprintf(stderr, "COULD NOT Save to confFile <%s>\n",confFile);
						dprintf_full("After demultiplex\r\n");
					}
				}
	
				delete url;
				if(mp4File) {
					// insert the video into the cache
					cm->putVideo(mp4File);
				}
			}

		}
	}
};


/** parses a directory for mp4 files */
bool loadServerDir(struct ProxyConfig* s, CacheManager* cm) {
	sord::directory dir(s->homeDir);
	if(!dir==true) {
		printf("Failed to open directory\n");
		return false;
	}
	char subDirPrefix='\0';
	loadRecursive(s, &subDirPrefix, cm, dir); 
	
	if(cm->getActSize()==0)
		return false;

	return true;
};



ResourceLimit* loadResourceLimit(struct ProxyConfig* s)
{
	char benchFile[MAX_STR_LEN];
	char tmpStr[MAX_STR_LEN];
	ResourceLimit* lim=new ResourceLimit();
#ifdef WIN32
	strcpy(benchFile,getenv("USERPROFILE"));
	strcat(benchFile,"\\");
#else
	strcpy(benchFile,getenv("HOME"));
	strcat(benchFile,"/");
#endif
	strcat(benchFile,PROXYBENCHMARKCONFIGFILE);
	
	FILE* fp=fopen(benchFile,"r");
	bool writeDefaultBench=false;
	if(fp) {
		//# networkInMBit diskInMBit cpuInDecodingPixels
		fgets(tmpStr,MAX_STR_LEN-1,fp);
		double t1,t2;
		if(3!=sscanf(tmpStr,"%lf %lf %i",&t1,&t2,&lim->cpu)) {
			dprintf_err("loadResourceLimit: invalid file\n");
			writeDefaultBench=true;
		}
		else {
			lim->network=(int)(t1*1024*1024);
			lim->disk=(int)(t2*1024*1024);
		}

		fclose(fp);
	}
	else
		writeDefaultBench=true;

	if(writeDefaultBench) {

		fp=fopen(benchFile,"wb");
		fprintf(stderr,"WARNING! No benchmark was run prior!\n");
		fprintf(stderr,"AdmissionControl is TURNED OFF! Run muViServerBench to estimate the server performance.\n");

		if(fp) {
			fprintf(fp,"%.3lf %.3lf %i\n# networkInMBit diskInMBit cpuInDecodingPixels\n",
				100.0,200.0,400000000);
			lim->network = (int)(100.0*1024*1024);
			lim->disk = (int)(200.0*1024*1024);
			lim->cpu = 400000000;
			fclose(fp);
		}
	}
	return lim;
}

void saveDefaultServerConfig(struct ProxyConfig* s);
bool loadServerDir(struct ProxyConfig* s, CacheManager* cm);
void loadRecursive(struct ProxyConfig* s, const char* subDirPrefix, CacheManager* cm, sord::directory &dir); 
ResourceLimit* loadResourceLimit(void);

int main(int argc, char *argv[])
{

	vitooki_init_all();
  struct ProxyConfig servConf;

#ifdef WIN32
	strcpy(servConf.configfile,getenv("USERPROFILE"));
	strcat(servConf.configfile,"\\");
#else
	strcpy(servConf.configfile,getenv("HOME"));
	strcat(servConf.configfile,"/");
#endif
	strcat(servConf.configfile,PROXYCONFIGFILE);
	fprintf(stderr,">>>%s<<<",servConf.configfile);
	if(!Proxy::loadProxyConfig(&servConf)) {
		printf("No proxy config file or illegal homedir entry detected! ");
		printf("A default file is written to %s! Check the default settings!\n",PROXYCONFIGFILE);
		printf("Hint: You are NOT ALLOWED to paste absolute filenames as HOMEDIR.");
		printf("Use relative path names as .\\test\r\n");
		saveDefaultServerConfig(&servConf);
		exit(0);
	}

	// create the demux dir
	char t[256];

	strcpy(t,"mkdir ");
	strcat(t,servConf.homeDir);
#ifdef WIN32
	for(unsigned int i=0;i<strlen(t);i++) {
		if(t[i]=='/')
			t[i]='\\';
	}
	strcat(t,"\\");
#else
	strcat(t,"/");
#endif
	strcat(t,DEMUX_PATH);
	dprintf_full("Creating demux dir X%sX\n",t);
	system(t);

	// Object lru will be deleted by the Server object

	CacheManagerLRU *lru = new CacheManagerLRU(servConf.maxCacheSize,false);
	if(!loadServerDir(&servConf,lru)) {
		printf("No precached videos found\r\n");
	}

	
	dprintf_full("STARTING PROXY\r\n");
	if(argc==2)
		servConf.port=atoi(argv[1]);

	ResourceLimit* lim=loadResourceLimit(&servConf);
	ResourcePrices p(10,10,10);
	AdmissionControl* adm=new AdmissionControl(lim,&p);
	delete lim;
	TerminalCapabilities* tc=Proxy::loadTermCaps();
	Proxy proxy(servConf, lru, adm, tc);
	proxy.startup();
	return 0;
};
