 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: Prox.cpp                                                      *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/


#ifdef WIN32
#include <winsock2.h>
#endif

#include <signal.h>
#include <string.h>

#include "Proxy.hpp"
#include "net/ProxySession.hpp"
#include "net/RTSP.hpp"
#include "cache/AdmissionControl.hpp"
#include "../muviserver/muviserver/Server.hpp"
static const int SELECT_TIMEOUT = 100000;
static const int MAX_CONCURRENT_CONNECTIONS = 10;
static int finish=0;

extern ProxyStatistics prxStat;

// destructing sig or quit
void sighandler(int sig)
{
	FILE* fp=fopen("proxy.stat","wb");
	if(fp) {
		prxStat.saveStatisticTo(fp);
		fclose(fp);
	} 	
  switch(sig) {
  case SIGINT:
    dprintf_full("handling SIGINT\n");
    if (finish < 5)
      finish++;
    else {
      printf("Proxy repeatedly didn't respond to shutdown request. Terminating unfriendly...\n");
      exit(1);
    }
  
    break;
	case SIGSEGV:
		fprintf(stderr,"Received SIGSEGV\n");
		break;
  default:
    dprintf("got unhandled signal...\n");
  }
}

Proxy::Proxy(struct ProxyConfig &cfg,CacheManager* c,
			 AdmissionControl *adm,TerminalCapabilities* tc):prxCfg(cfg)
{
	cout << "Proxy is initializing\r\n" << flush;
	admControl=adm;
	signal(SIGINT, sighandler);
	signal(SIGSEGV,sighandler);
	cm = c;
	socket=0;
	defTermCap=tc;
	this->counter = new SessionCounter("active_sessions.dat");
};


Proxy::~Proxy()
{
	FILE* fp=fopen("proxy.stat","wb");
	if(fp) {
	  prxStat.saveStatisticTo(fp);
	  fclose(fp);
	} 

  if (socket>0) {
#ifdef WIN32
		closesocket(socket);
#else
		::close(socket);
#endif
		socket = 0;
  }
  // find finished Sessions and remove them from the list
  list < Session * >::iterator li;
  if (!sessionList.empty()) {
		li = sessionList.begin();
		while (!sessionList.empty()) {
	    (*li)->tearDown(-1);
	    delete(*li);
	    li = sessionList.begin();
		}
  }
  if (cm && prxCfg.homeDir && strlen(prxCfg.homeDir)>0) {
		// create the demux dir
		char t[MAX_STR_LEN];

		strcpy(t,"mkdir ");
		strcat(t,prxCfg.homeDir);
#ifdef WIN32
		strcat(t,"\\");
#else
		strcat(t,"/");
#endif
		strcat(t,DEMUX_PATH);
		dprintf_full("Proxy::~Proxy() Saving CacheManager to %s\r\n",t);
		cm->saveCacheIndexToDir(t);
	}
	if(cm)
		delete cm;
	if(admControl)
		delete admControl;
	if(defTermCap)
		delete defTermCap;
	if(counter)
		delete counter;

};

bool Proxy::createSocket()
{
	this->socket =::socket(AF_INET, SOCK_STREAM, 0);
	if (socket < 0) {
		perror("create socket");
		return false;
	}
	
	const int reuse = 1;
	if (setsockopt(socket, SOL_SOCKET, SO_REUSEADDR,
								 (const char *) &reuse, sizeof(reuse)) < 0) {
		perror("setsockopt");
#ifdef WIN32
		closesocket(socket);
#else
		close(socket);
#endif
		socket = 0;
		return false;
	}
	
	if (prxCfg.port != 0) {
		struct sockaddr_in name;
		name.sin_family = AF_INET;
		name.sin_port = htons(prxCfg.port);
		name.sin_addr.s_addr = htonl(INADDR_ANY);
		if (bind(socket, (struct sockaddr *) &name, sizeof name) != 0) {
			perror("bind socket");
#ifdef WIN32
	    closesocket(socket);
#else
	    ::close(socket);
#endif
	    socket = 0;
	    return false;
		}
	} else {
		dprintf_err("No RTSP port set\n");
#ifdef WIN32
		closesocket(socket);
#else
		close(socket);
#endif
		socket = 0;
		return false;
	};
	
	
	if (0 != listen(socket, MAX_CONCURRENT_CONNECTIONS)) {
		perror("listen");
#ifdef WIN32
		closesocket(socket);
#else
		close(socket);
#endif
		socket = 0;
		return false;
	};
	
	return true;
}

void Proxy::startup()
{
  fd_set rdfs;
  struct timeval tv;
  counter->start();
	if (false == createSocket()) {
		cerr << "Could not create socket for port " << prxCfg.port << endl; 	
		return;
	}
	int new_socket = -1;
	dprintf("Proxy::startup:  everything up and ready on PORT %i\r\n", prxCfg.port);
	while (!finish) {
		// listen for new connections
		int sin_size = sizeof(struct sockaddr_in);
		struct sockaddr_in caller;
		memset(&caller,0,sin_size);
		
		FD_ZERO(&rdfs);
		FD_SET(socket,&rdfs);
		tv.tv_sec=0;
		tv.tv_usec=500000;
		if ( (select(socket+1,&rdfs,NULL,NULL,&tv) != -1)
	     && (FD_ISSET(socket,&rdfs)) ) {
			
			if ( (new_socket = accept(socket, (struct sockaddr *) &caller,
#ifndef WIN32
																(unsigned int *)
#endif
																&sin_size)) <= 0) {
				perror("accept new client");
				::exit(1);
			}
			TerminalCapabilities* tc=NULL;
			if(defTermCap)
				delete defTermCap;
			defTermCap=Proxy::loadTermCaps();
			if(defTermCap)
				tc=defTermCap->clone();
			struct ProxyConfig dummyPrxCfg(prxCfg);
			if(Proxy::loadProxyConfig(&dummyPrxCfg)) {
				// only update enablertx, rest is not allowed to change!
				prxCfg.enableRtx=dummyPrxCfg.enableRtx;
			}
		 
			FILE* fp=fopen("proxy.stat","wb");
			if(fp) {
				prxStat.saveStatisticTo(fp);
				fclose(fp);
			}

			Session *se =	new ProxySession(new_socket, (struct sockaddr_in *) &caller, 
																		 cm, admControl, prxCfg.enableRtx, counter, tc);
			sessionList.push_back(se);
			se->start();
		}
		
		// find finished Sessions and remove them from the list
		list < Session * >::iterator li;
		if (!sessionList.empty()) {
	    li = sessionList.begin();
	    while (!sessionList.empty() && (li != sessionList.end())) {
				if ((*li)->finished()) {
					delete(*li);
					li = sessionList.erase(li);
					/*
						static int finishedRequests=0;
						finishedRequests++;
						if(finishedRequests>3)
						finish=true; //FIXME: just for mem leak detection
					*/
				} else
					++li;
	    }
		}
	} // !finish
  if (cm) {
		// create the demux dir
		char t[256];
		
		strcpy(t,"mkdir ");
		strcat(t,prxCfg.homeDir);
#ifdef WIN32
		strcat(t,"\\");
#else
		strcat(t,"/");
#endif
		strcat(t,DEMUX_PATH);
		dprintf_full("Proxy::startup() Saving CacheManager to %s\r\n",t);

		cm->saveCacheIndexToDir(t);
	}

// remove all sessions

    dprintf("Proxy::startup: Shutting down proxy. Closing open sessions. Please wait...\r\n");
    // find finished Sessions and remove them from the list
    list < Session * >::iterator li;
    if (!sessionList.empty()) {
			li = sessionList.begin();
			while (!sessionList.empty() && (li != sessionList.end())) {
				(*li)->tearDown(-1,true);
				if (!(*li)->finished()) {
					(*li)->exit();
				}
				delete(*li);
				li = sessionList.erase(li);
			}
    }
#ifdef WIN32
    closesocket(socket);
#else
    close(socket);
#endif
    socket = 0;
		delete cm;
		cm = NULL;
		counter->stop();
		counter->wait();
}

ContainerInfo *Proxy::find(const Url *fileName)
{
    if (cm) {
	return cm->getVideo(fileName);
    }
    return NULL;
}

/** returns false, if no serverConfigFile or homedir was found */
bool Proxy::loadProxyConfig(struct ProxyConfig* s) {
	RTSP t;
	bool retVal=false;
	FILE* fp=fopen(s->configfile,"rb");
	int tmpPort=0;
	unsigned int tmpCacheSize=0;
	s->recurseDirectories=false;
	s->homeDir[0]='.';
	s->homeDir[1]='\0';
	s->port=t.getDefaultPort();
	s->maxCacheSize=DEFAULTCACHESIZE;
	s->enableRtx=true;
	
	if(!fp) {
		return retVal;
	}

	char temp[1024];
	temp[1023]=0;
	char* tmp;

	while(fgets(temp,1023,fp)) {
		// fgets read one line
		tmp=strstr(temp,HOMEDIR);
		if(tmp) {
			tmp+=strlen(HOMEDIR);
			tmp=stripWhiteSpace(tmp);
			if(strlen(tmp))
				strcpy(s->homeDir,tmp);
			retVal=true;
		} 
		tmp=strstr(temp, RECURSE);
		if(tmp) {
			tmp+=strlen(RECURSE);
			if(strstr(tmp,YES)) {
				s->recurseDirectories=true;
			}			
		}
		tmp=strstr(temp, PORT);
		if(tmp) {
			tmp+=strlen(PORT);
			tmpPort=atoi(tmp);
			if(tmpPort>0)
				s->port=tmpPort;
		}
		tmp=strstr(temp, CACHESIZE);
		if(tmp) {
			tmp+=strlen(CACHESIZE);
			sscanf(tmp,"%u",&tmpCacheSize);
			if(tmpCacheSize)
				s->maxCacheSize=tmpCacheSize;
		}
		tmp=strstr(temp,CONFIG_RTX);
		if(tmp) {
			tmp+=strlen(CONFIG_RTX);
			if(strstr(tmp,NO)) {
				s->enableRtx=false;
			}
		}
	}
	fclose(fp);
	return retVal;
}

TerminalCapabilities* Proxy::loadTermCaps()
{
	char tcFile[MAX_STR_LEN];
#ifdef WIN32
	strcpy(tcFile,getenv("USERPROFILE"));
	strcat(tcFile,"\\");
#else
	strcpy(tcFile,getenv("HOME"));
	strcat(tcFile,"/");
#endif
	strcat(tcFile,PROXYTCFILE);
	TerminalCapabilities* tc=new TerminalCapabilities();
	
	if(tc->initFromFile(tcFile))
		return tc;
	delete tc;
	return NULL;
};

char* Proxy::stripWhiteSpace(char* url) {
	char* temp=url;
	while(	*temp==' '  || *temp=='\r' ||
			*temp=='\n' || *temp=='\t')
			temp++;
	int pos=strlen(url)-1;
	while(	(url+pos)>temp && pos>=0 &&
			(url[pos]==' ' || url[pos]=='\r' ||
			url[pos]=='\n' || url[pos]=='\t') ){
			url[pos]=0;
			pos--;
	}
	return temp;
};

bool Proxy::saveProxyConfig(const struct ProxyConfig* s)
{
	bool ret=false;
	FILE* fp=fopen(s->configfile,"wb");
	if(!fp) {
		fprintf(stderr,"Proxy::saveProxyConfig: Failed to open %s for writing\n",s->configfile);
	}
	else { //write defaults to config file
		char hostname[MAX_STR_LEN];
		gethostname(hostname,MAX_STR_LEN);
		fprintf(fp,"#path where all video files are stored (eg. /var/spool/muViProxy/data)\n");
		fprintf(fp,"%s%s\n",HOMEDIR,s->homeDir);
		fprintf(fp,"#search HOMEDIR recursively for prechached videos (%s/%s)?\n",YES,NO);
		fprintf(fp,"%s%s\n",RECURSE,(s->recurseDirectories?YES:NO));
		fprintf(fp,"#RTSP/TCP port (default: %i)\n",3128);
		fprintf(fp,"%s%i\n",PORT,s->port);
		fprintf(fp,"#should the proxy support retransmission of lost packets (%s/%s)\n",YES,NO);
		fprintf(fp,"%s %s\n",CONFIG_RTX,(s->enableRtx?YES:NO));
		fprintf(fp,"#how many bytes of the hard disc is the proxy allowed to use as cache\n");
		fprintf(fp,"%s %lu\n",CACHESIZE,s->maxCacheSize);
		fclose(fp);
		ret=true;
	}
	return ret;	
}
