 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: Proxy.hpp                                                     *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PS_PROXY_HPP
#define _PS_PROXY_HPP
#include <list>

#include "ContainerInfo.hpp"
#include "cache/CacheManager.hpp"
#include "net/Session.hpp"
#include "net/Url.hpp"
#include "SessionCounter.hpp"
#include "metadata/TerminalCapabilities.hpp"
#define PROXYBENCHMARKCONFIGFILE ".muViProxyBench"
#define PROXYTCFILE ".muViProxytc"
#define PROXYCONFIGFILE ".muViProxyrc"
#define CACHESIZE "MAXCACHESIZE="
#define DEFAULTCACHESIZE (1UL<<31)

struct ProxyConfig {
	char homeDir[MAX_STR_LEN];
	char configfile[MAX_STR_LEN];
	bool recurseDirectories;
	bool enableRtx;
	int port;
	unsigned long maxCacheSize;
	ProxyConfig() {
		homeDir[0]='\0';
		configfile[0]='\0';
		recurseDirectories=enableRtx=false;
		port=-1;
		maxCacheSize=0xfffffffful;
	};
};

class AdmissionControl;

/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: Proxy.hpp,v 1.13 2005/01/07 10:24:13 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class Proxy {
  protected:
    bool createSocket();
  public:
	Proxy(struct ProxyConfig &cfg,CacheManager* cm,
		AdmissionControl *adm,TerminalCapabilities* tc=NULL);
	
    virtual ~ Proxy();
    ContainerInfo *find(const Url *fileName);
    void startup();
		static bool loadProxyConfig(struct ProxyConfig* s);
		static char* stripWhiteSpace(char* url); 
		static TerminalCapabilities* loadTermCaps();
		static bool saveProxyConfig(const struct ProxyConfig* s);
  protected:

	int socket;
	list < Session * >sessionList;
	CacheManager *cm;
	AdmissionControl* admControl;
	TerminalCapabilities* defTermCap;
	SessionCounter* counter;
	struct ProxyConfig prxCfg;
};

#endif
