 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	MP4Server.cpp                                                 *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifdef WIN32
#undef BOOL
#include <winsock2.h>
#else
#include <unistd.h> 
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <netdb.h>
#endif

#include "sord/directory.h"
#include "sord/entry.h"
#include <assert.h>

#include "vitooki.hpp"
#include "Server.hpp"
#include "ESInfo.hpp"
#include "VideoESInfo.hpp"
#include "ContainerInfo.hpp"
#include "net/Url.hpp"
#include "net/RTSP.hpp"
#include "cache/CacheManagerLRU.hpp"
#include "container/ContainerFile.hpp"
#include "container/ContainerDemux.hpp"
#include "io/IOCreator.hpp"
#include "adaptors/FramePatternDetector.hpp"
#include "cache/AdmissionControl.hpp"
#include "BitField.hpp"
#include "metadata/TerminalCapabilities.hpp"
#include "MovieListCreation.hpp"
#ifdef WIN32
#include "SimpleStatistics.hpp"
#endif
bool saveDefaultServerConfig(struct ServerConfig* s) {
	bool ret=false;
	RTSP rt;
	strcpy(s->homeDir,".");
	strcpy(s->movielistXMLFile, "./metadata/MovieList.xml");
        strcpy(s->movielistHTMLFile, "./metadata/MovieList.html");
	strcpy(s->metadataDir, "./metadata");
	s->recurseDirectories=true;
	strcpy(s->switchDir,"./switch");
	s->port=rt.getDefaultPort();
	strcpy(s->bindToIP,SERVER_BIND_ALLDEV);
#ifdef WIN32
	s->enableRtx=false;
#else
	s->enableRtx=true;
#endif
	ret=Server::saveServerConfig(s);
	return ret;
};

/*****************************************************************/
void loadRecursive(struct ServerConfig* s, const char* basedir, const char* subDirPrefix, CacheManager* cm, sord::directory &dir, bool isLoadSwitchDir) {
	sord::directory::const_iterator &i = dir.begin();
	sord::directory::const_iterator &end = dir.end();

	for (; i != end; ++i) 
		if (i->get_name()) { // *WONDERS WHY THIS HAPPENS*
		dprintf_full("MP4Server::loadRecursive: checking on %s\r\n",i->get_name());
		if (i->get_type() == sord::entry::type_directory) {
			if(s->recurseDirectories || isLoadSwitchDir) {
				sord::directory subdir(*i);
				char *tmp=new char[strlen(subDirPrefix)+strlen(i->get_name())+2];
				sprintf(tmp,"%s/%s",subDirPrefix,i->get_name());
				loadRecursive(s,basedir,tmp,cm,subdir,isLoadSwitchDir);
				delete tmp;
			}
		} 
		else if(i->get_type() == sord::entry::type_regular) {
			// check if filetype has known extension
			const char* name=i->get_name();
			dprintf_full("MP4Server::loadRecursive %s\r\n",name);
			if(	IOCreator::verifyContainerExtension(name+strlen(name)-4)) {
				// copy dir+name!
				char t[4096];
				strcpy(t,subDirPrefix);
				if(	t[strlen(t)-1]!='\\' || 
					t[strlen(t)-1]!='/') {
					strcat(t,"/");
				}
				strcat(t,i->get_name());
				dprintf_full("MP4Server::loadRecursive Full name: %s\r\n",t);
				// eliminate ./
				int offset=0;
				if(strncmp(t,
#ifdef WIN32
					".\\",
#else
					"./",
#endif
					2)==0) {
					offset=2;
				}
				// now check for config file
				char confFile[4096];
				ContainerInfo *mp4File = NULL;
				dprintf_full("MP4Server::loadRecursive relative name X%sX\r\n",&t[offset]);
				// build url
				char pTemp[4096];
				char* ptmp;
				ptmp=strcpy(pTemp,"rtsp://");
				
				if (0 == strcmp(s->bindToIP,SERVER_BIND_ALLDEV))
					gethostname(pTemp+strlen(pTemp),4096-strlen(pTemp));
				else
					strncpy(pTemp+strlen(pTemp),s->bindToIP,4096-strlen(pTemp));
				//if no domainname added, just ignore it, it will be done in class Url anyway...

				snprintf(pTemp+strlen(pTemp),4096-strlen(pTemp),"%s",t); //relative path name
				dprintf_full("MP4Server: stored file as %s\r\n",pTemp);
				Url* url=new Url(pTemp);
				char fileOnDisk[1024];
				char demuxDir[1024];
				sprintf(fileOnDisk,"%s%s",basedir, url->getPath());
				char* dummyStr=Url::transformPathToLocalFile(url);
				sprintf(confFile,"%s/%s%s.conf",basedir,DEMUX_PATH,dummyStr);
				sprintf(demuxDir,"%s/%s",basedir,DEMUX_PATH);
				delete[] dummyStr;
#ifdef WIN32
				unsigned int i=0;
				for(i=0;i<strlen(fileOnDisk);i++) {
					if(fileOnDisk[i]=='/')
						fileOnDisk[i]='\\';
				}
				for(i=0;i<strlen(confFile);i++) {
					if(confFile[i]=='/')
						confFile[i]='\\';
				}
				for(i=0;i<strlen(demuxDir);i++) {
					if(demuxDir[i]=='/')
						demuxDir[i]='\\';
				}
#endif
				FILE *tmpFp=fopen(confFile,"rb");
				if(tmpFp) {
					fclose(tmpFp);
					// we have already the demultiplexed es 
					mp4File=new ContainerInfo(pTemp, &t[offset], ContainerInfo::SDP);
					if(!mp4File->loadFromCfgFile(confFile)) {
						delete mp4File; 
						mp4File = ContainerFile::loadContainerInfo( fileOnDisk,url );	
						if(mp4File) {
							FramePatternDetector* fpd=new FramePatternDetector(mp4File->getFirstVisualES(),true);
							ContainerDemux::demultiplexAndUpdateIO(mp4File,demuxDir,fpd);
							delete fpd;
							mp4File->saveToCfgFile(confFile);
						}
					}
				}
				else {
					mp4File = ContainerFile::loadContainerInfo(fileOnDisk,url );
					
					if(mp4File) {
						dprintf_full("MP4Server::loadRecursive Before demultiplex\r\n");
						FramePatternDetector* fpd=new FramePatternDetector(mp4File->getFirstVisualES(),true);
						ContainerDemux::demultiplexAndUpdateIO(mp4File,demuxDir,fpd);
						delete fpd;
						if(mp4File->saveToCfgFile(confFile))
							fprintf(stderr, "Saved to confFile %s\n",confFile);
						else
							fprintf(stderr, "COULD NOT Save to confFile <%s>\n",confFile);
						dprintf_full("MP4Server::loadRecursive After demultiplex %s\r\n",url->toString());
					}
				}
				mp4File->setUrl(url);

				if(mp4File) {
					if (isLoadSwitchDir) {
						char switchUrl[MAX_STR_LEN];
						sprintf(switchUrl,"%s://%s",url->getProtocol(),url->getServerName());
						if (url->getPort() != 0)
							sprintf(switchUrl,"%s:%i",switchUrl,url->getPort());

						//chop off last file name, make directory name to URL name
						char newpath[MAX_STR_LEN];
						strcpy(newpath,url->getPath());
#ifdef linux
						char *c = strrchr(newpath,'/');
#endif
#ifdef WIN32
						char *c = strrchr(newpath,'\\');
#endif
						if (c != NULL)
							*c = '\0';

						sprintf(switchUrl,"%s%s",switchUrl, newpath);
						dprintf_full("MP4server::loadRecursive SwitchDir: old URL: %s new URL: %s\n",url->toString(),switchUrl);
						mp4File->setUrl(new Url(switchUrl));
					}

					// insert the video into the cache
					cm->putVideo(mp4File);

					//local data on server has to have all bits set!!
					//if (mp4File->getFirstVisualES())
						//assert(mp4File->getFirstVisualES()->getFrameStatistic()->isAllSet());
				}
			}
		}
	}
};


/** parses a directory for mp4 files and for switchset-mp4 files */
bool loadServerDir(struct ServerConfig* s, CacheManager* cm) {
	char subDirPrefix;
	char t[256];
	sord::directory hdir(s->homeDir);
	sord::directory sdir(s->switchDir);
	if ( !hdir && !sdir) {
		printf("MP4Server: Failed to open directories %s and %s\n",s->homeDir,s->switchDir);
		return false;
	}

	if (!(!hdir)) { //homeDir demux
		strcpy(t,"mkdir ");
		strcat(t,s->homeDir);
#ifdef WIN32
		for(unsigned int i=0;i<strlen(t);i++) {
			if(t[i]=='/')
				t[i]='\\';
		}
		strcat(t,"\\");
#else
		strcat(t,"/");
#endif
		strcat(t,DEMUX_PATH);
		dprintf_full("MP4Server: Creating demux for homeDir >%s<\n",t);
		system(t);
	}

	if (!(!sdir)) {  //switchDir demux
		strcpy(t,"mkdir ");
		strcat(t,s->switchDir);
#ifdef WIN32
		for(unsigned int i=0;i<strlen(t);i++) {
			if(t[i]=='/')
				t[i]='\\';
		}
		strcat(t,"\\");
#else
		strcat(t,"/");
#endif
		strcat(t,DEMUX_PATH);
		dprintf_full("MP4Server: Creating demux for switchDir >%s<\n",t);
		system(t);
	}

	//homeDir
	subDirPrefix='\0'; // at highest level
	loadRecursive(s, hdir.get_name(), &subDirPrefix, cm, hdir, false);

	//switchDir
	subDirPrefix='\0'; // at highest level
	loadRecursive(s, sdir.get_name(), &subDirPrefix, cm, sdir, true);
	
	if(cm->getActSize()==0)
		return false;

	return true;
};


ResourceLimit* loadResourceLimit(struct ServerConfig* s)
{
	char benchFile[MAX_STR_LEN];
	char tmpStr[MAX_STR_LEN];
	ResourceLimit* lim=new ResourceLimit();
#ifdef WIN32
	strcpy(benchFile,getenv("USERPROFILE"));
	strcat(benchFile,"\\");
#else
	strcpy(benchFile,getenv("HOME"));
	strcat(benchFile,"/");
#endif
	strcat(benchFile,SERVERBENCHMARKCONFIGFILE);
	
	FILE* fp=fopen(benchFile,"r");
	bool writeDefaultBench=false;
	if(fp) {
		//# networkInMBit diskInMBit cpuInDecodingPixels
		fgets(tmpStr,MAX_STR_LEN-1,fp);
		double t1,t2;
		if(3!=sscanf(tmpStr,"%lf %lf %i",&t1,&t2,&lim->cpu)) {
			dprintf_err("loadResourceLimit: invalid file\n");
			writeDefaultBench=true;
		}
		else {
			lim->network=(int)t1*1024*1024;
			lim->disk=(int)t2*1024*1024;
		}

		fclose(fp);
	}
	else
		writeDefaultBench=true;

	if(writeDefaultBench) {

		fp=fopen(benchFile,"wb");
		fprintf(stderr,"WARNING! No benchmark was run prior!\n");
		fprintf(stderr,"AdmissionControl is TURNED OFF! Run muViServerBench to estimate the server performance.\n");

		if(fp) {
			fprintf(fp,"%.3lf %.3lf %i\n# networkInMBit diskInMBit cpuInDecodingPixels\n",
				100.0,200.0,400000000);
			lim->network=(int)(100.0*1024*1024);
			lim->disk=(int)(200.0*1024*1024);
			lim->cpu=400000000;
			fclose(fp);
		}
	}
	return lim;
};

bool saveDefaultServerConfig(struct ServerConfig* s);
bool loadServerDir(struct ServerConfig* s, CacheManager* cm);
void loadRecursive(struct ServerConfig* s, const char* subDirPrefix, CacheManager* cm, sord::directory &dir); 

ResourceLimit* loadResourceLimit(void);

/*****************************************************/
int main(int argc, char *argv[])
{
	vitooki_init_all();
	
  struct ServerConfig servConf;
#ifdef WIN32
	strcpy(servConf.configfile,getenv("USERPROFILE"));
	strcat(servConf.configfile,"\\");
#else
	strcpy(servConf.configfile,getenv("HOME"));
	strcat(servConf.configfile,"/");
#endif
	strcat(servConf.configfile,SERVERCONFIGFILE);
	printf("MP4Server: Loading serverConfigFile %s\n",servConf.configfile);

	if(!Server::loadServerConfig(&servConf)) {
		printf("MP4Server: No server config file or illegal homeDir/switchDir entry detected! ");
		printf("A default file is written to %s!\r\n"
					 "Check the default settings!\r\n",servConf.configfile);
		saveDefaultServerConfig(&servConf);
#ifdef WIN32
		printf("Press Enter to quit\n");
		getc(stdin);
#endif
		exit(0);
	}

    // make Cache very large, so that no replacement can happen
    // Object lru will be deleted by the Server object
    u32 size = (u32) -1;
    CacheManagerLRU *lru = new CacheManagerLRU(size,true);
	if(!loadServerDir(&servConf,lru)) {
		printf("MP4Server: Couldn't find directory or any .mp4 files in the directory you specified\r\n");
		printf("No videos, no server :-)\r\n");
		delete lru;
		exit(0);
	}

	
	dprintf_full("MP4Server: STARTING SERVER\r\n");
	if(argc==2)
		servConf.port=atoi(argv[1]);

	ResourceLimit* lim=loadResourceLimit(&servConf);
	ResourcePrices p(10,10,10);
	AdmissionControl* adm=new AdmissionControl(lim,&p);
	delete lim;
	TerminalCapabilities* tc=Server::loadTermCaps();
	Server server(servConf, lru,adm,tc);
	MovieListCreation *mlc = new MovieListCreation(&servConf);
	mlc->createMovieList();
        delete mlc;

#ifdef WIN32
	// only useful when retransmission switched off
	SimpleStatistics::createGlobalLog("log_servertotal.out", ".", "_server.out");
#endif		
	server.startup();

	return 0;
};
