 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: MovieListCreation.cpp                                         *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 * Video Session Migration System, 2004                                 *
 * Author: Klaus Schoeffmann                                            *
 ************************************************************************/


#include "MovieListCreation.hpp"

/***********************************************************************/
MovieListCreation::MovieListCreation(ServerConfig *sc)
{
  this->sc = sc;
  this->port = sc->port;
  this->movieId = 0;

  //supported filetypes
  knownFileTypes.push_back(".avi");
  knownFileTypes.push_back(".mpg");
  knownFileTypes.push_back(".mpeg");
  knownFileTypes.push_back(".mp3");
  knownFileTypes.push_back(".mp4");

  //test existence of metadata directory (otherwise create)
  sord::directory mdir(sc->metadataDir);
  if (!mdir) {
  	cout << "\ncreating metadata dir" << endl;
	char t[256];
	strcpy(t,"mkdir ");
	strcat(t,sc->metadataDir);
#ifdef WIN32
	for(unsigned int i=0;i<strlen(t);i++) {
		if(t[i]=='/')	t[i]='\\';
	}
#endif
	system(t);

	//create index.html
	strcpy(t, sc->metadataDir);
	strcat(t, "//index.html");
	ofstream index (t, ios::out);
	index << "<html><head>"
	      << "<meta http-equiv=\"refresh\" content=\"0; URL=./MovieList.html\"></head>"
              << "</head><body></body></html>";
	index.close();
  }

  //find out IP-Address of this host
  char name[255];
  if (gethostname(name, sizeof(name)) == 0) {
    hostent *host = gethostbyname(name);
		if (host == NULL) {
			dprintf_err("MovieListCreation: binding specific IP-addr, though no DNS-hostname resolution possible...\n");
			strcpy(this->address,name);
		} else
			strcpy(this->address,inet_ntoa(*(reinterpret_cast<in_addr*>(host->h_addr))) );
  } else {
    throw GetHostNameProblem();
  }
}

/***********************************************************************/
string MovieListCreation::createHeader()
{
  //read current time
  time_t rawtime;
  struct tm *timeinfo;
  time(&rawtime);
  timeinfo = localtime(&rawtime);

  ostringstream hdr;
  hdr <<
"<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl <<
"<!-- This file was created by muviserver on " << asctime(timeinfo) << " -->" << endl <<
"<?xml-stylesheet type=\"text/xsl\" href=\"MovieList.xsl\" ?>" << endl <<
"<DIDL xmlns=\"urn:mpeg:mpeg21:2002:01-DIDL-NS\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\""
" xsi:schemaLocation=\" urn:mpeg:mpeg21:2002:01-DIDL-NS Mpeg21DIDL-2002.xsd\">" << endl <<
"  <Container>" << endl <<
"    <Item>" << endl <<
"      <Choice minSelections=\"0\" maxSelections=\"1\">" << endl <<
"        <Descriptor>" << endl <<
"          <Statement mimeType=\"text/plain\">" << endl <<
"            If you want to start streaming for a video, please"
"            select one of the available streams: "
"        </Statement>" << endl <<
"        </Descriptor>" << endl;

  return hdr.str();
}


/***********************************************************************/
bool MovieListCreation::isKnownFiletype(char *fname)
{

  for (u32 i=0; i < knownFileTypes.size(); i++) {
    if (strlen(fname) > strlen(knownFileTypes[i])
        && strstr(fname+(strlen(fname)-strlen(knownFileTypes[i])), knownFileTypes[i]) != NULL)
          return true;
  }

  return false;
}

/***********************************************************************/
void MovieListCreation::analyzeDir(char *path, char *fnprefix, bool recurseDirectories)
{
  sord::directory dir(path);
  if (!dir) {
    throw DirNotFound();
  } else {
    sord::directory::const_iterator &i = dir.begin();
    sord::directory::const_iterator &end = dir.end();

    //iterate over files in this directory
    for (; i != end; ++i) {
      char *filename = (char*)i->get_name();
      if (filename && isKnownFiletype(filename)) {
        cout << "Known file: " << fnprefix << filename << endl;
        atLeastOneFile = true;
        movieId++;
        data += createSelection(sc->metadataDir, filename);
        resources += createResource(filename, fnprefix);
      }
      else if (filename && i->get_type() == sord::entry::type_directory
               && recurseDirectories) {
        //copy new file-name prefix
        int len = strlen(fnprefix) + strlen(filename) + 2;
        char *newprefix = new char[len];
        strcpy(newprefix, fnprefix);
        strcpy(newprefix+strlen(fnprefix), filename);
        newprefix[len-2] = '/';
        newprefix[len-1] = 0;

        //copy new path
        len = strlen(path) + 1 + strlen(i->get_name()) + 1;
        char *newpath = new char[len];
        strcpy(newpath, path);
        newpath[strlen(path)] = '/';
        strcpy(newpath + strlen(path) + 1, i->get_name());
        newpath[len-1] = 0;

        analyzeDir(newpath, newprefix, recurseDirectories);

        delete newprefix;
        delete newpath;
      }
    }
  }
}

/***********************************************************************/
void MovieListCreation::createMovieList()
{
  atLeastOneFile = false;

  data = createHeader();
  analyzeDir(sc->homeDir, "", sc->recurseDirectories);
  data += "</Choice>\n\n";
  data += resources;
  data += createFooter();

  if (atLeastOneFile) {
    //write to file
    string fout (sc->movielistXMLFile);
    ofstream foutput(fout.c_str());
    foutput << data;
    foutput.close();

    cout << "MovieListCreation: movielist with " << movieId <<" entries successfully written to:"
         << endl << "\t- " << sc->movielistXMLFile << endl;

    //************generate HTML file************
    MovieList *ml = new MovieList();
    ml->readMoviesFromFile(sc->movielistXMLFile);
    //write to file
    ofstream foutputh(sc->movielistHTMLFile);
    foutputh << getHTMLHeader();

    /*char *imgdir = new char[strlen(sc->metadataDir) + 2];
    strcpy(imgdir, sc->metadataDir);
    imgdir[strlen(sc->metadataDir)] = '/';
    imgdir[strlen(sc->metadataDir)+1] = 0;
    foutputh << ml->getHTMLRepresentation(imgdir);*/

    foutputh << ml->getHTMLRepresentation("");
    foutputh << getHTMLFooter();
    foutputh.close();
    cout << "\t- " << sc->movielistHTMLFile << endl;
    delete ml;

  } else {
    cout << "MovieListCreation: no known files were found! MovieList NOT created!" << endl;
  }
}


/***********************************************************************/
string MovieListCreation::createSelection(const char *metadatadir, const char *fname)
{
  string name (fname);
  string desc = "No description available";
  string imgname = "noimage.jpg";

  //get filename without extension
  string fullfname (fname);
  u32 dotpos = fullfname.find(".");
  if (dotpos != string::npos) {
    string leftfname = fullfname.substr(0,dotpos);

    //try to find image
    string imgtest (metadatadir);
    imgtest += "/" + leftfname + ".jpg";
    ifstream imgfilestream (imgtest.c_str());
    if (imgfilestream) {
      imgname = leftfname + ".jpg";
    }

    //try to find meta-data (textual description)
    string desctest (metadatadir);
    desctest += "/" + leftfname + ".txt";
    ifstream descfilestream (desctest.c_str());
    if (descfilestream) {
      char buf[MAX_DESC_SIZE];
      memset(buf,0, MAX_DESC_SIZE);
      descfilestream.read(buf, sizeof(buf));
      desc = string(buf);
      descfilestream.close();
    }
  }

  ostringstream sel;
  sel << endl <<
"<Selection select_id=\"stream" << movieId << "\">" << endl <<
"<Descriptor>" << endl <<
"<Statement mimeType=\"text/plain\">" << fname <<
"</Statement>" << endl <<
"</Descriptor>" << endl <<
"<Descriptor>" << endl <<
"<Statement mimeType=\"text/plain\">" << desc <<
"</Statement>" << endl <<
"</Descriptor>" << endl <<
"<Descriptor>" << endl <<
"  <Component>" << endl <<
"    <Resource mimeType=\"image/jpeg\" ref=\"" << imgname << "\"/>" << endl <<
"  </Component>" << endl <<
"</Descriptor>" << endl <<
"</Selection>" << endl;

  return sel.str();
}


/***********************************************************************/
string MovieListCreation::createResource(const char *fname, const char *fnprefix)
{
  ostringstream res;
  res <<  endl <<
"      <Component>" << endl <<
"        <Condition require=\"stream" << movieId << "\"/>" << endl <<
"        <Resource mimeType=\"video/mpeg\" ref=\"rtsp://"
        << address << ":" << port << "/" << fnprefix << fname << "\"/>" << endl <<
"      </Component>" << endl;

  return res.str();
}


/***********************************************************************/
string MovieListCreation::createFooter()
{
  ostringstream footer;
  footer <<
"    </Item>" << endl <<
"  </Container>" << endl <<
"</DIDL>" << endl << endl;

  return footer.str();
}

/***********************************************************************/
string MovieListCreation::getHTMLHeader()
{
  string hdr = "<html><head><title>ViTooKi MuViServer - Videolist</title>"
  "<style type=\"text/css\">"
  "table.background {"
  "        background-color:white;"
  "        border-style:solid;"
  "        border-width:1px;"
  "        border-color:#000000;"
  "        text-align:center;"
  "}"
  "body, td, .profileinfo, input, textarea, select, td {"
  "        font-family: Verdana;"
  "        font-size: 12px;"
  "}"
  "a, a:visited {"
  "       color: #0000FF;"
  "}"
  "input, textarea, select {"
  "        color: #002F68;"
  "}"
  "body {"
  "        scrollbar-arrow-color: #253BC0;"
  "        scrollbar-base-color: #B2C2D7;"
  "        scrollbar-track-color: #D0D7E1;"
  "        scrollbar-highlight-color: #002F68;"
  "        scrollbar-3dlight-color: #D0D7E1;"
  "        scrollbar-darkshadow-color: #D0D7E1;"
  "        scrollbar-shadow-color: #002F68;"
  "        background-color:#ECF1F7;"
  "}"
  "font.warning {"
  "        color: #FF0000;"
  "}"
  "font.error {"
  "        color: #FF0000;"
  "        font-weight: bold;"
  "}"
  ".moviehdr {"
  "        font-size: 12px;"
  "        background-color:#F7F7F7;"
  "        color: #000000;"
  "}"
  ".moviedesc {"
  "        color: #AAAAAA;"
  "        font-style: italic;"
  "}"
  ".darkblueborder {"
  "        scrollbar-arrow-color: #FFFFFF;"
  "        scrollbar-base-color: #B2C2D7;"
  "        scrollbar-track-color: #D0D7E1;"
  "        scrollbar-highlight-color: #002F68;"
  "        scrollbar-3dlight-color: #D0D7E1;"
  "        scrollbar-darkshadow-color: #D0D7E1;"
  "        scrollbar-shadow-color: #002F68;"
  "        border-style:solid;"
  "        border-color:#002F68;"
  "        border-width:1px;"
  "}"
  "</style>"
  "</head>"
  "<body><center>"
  "<h3>MuViServer - Videolist</h3><br>\n"
  "<table style=\"width:480px;\" cellspacing=\"5\" class=\"background\">";

  return hdr;
}


/***********************************************************************/
string MovieListCreation::getHTMLFooter()
{
  string ftr = "\n</table></center></body></html>";

  return ftr;
}
