 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: MovieListCreation.hpp                                         *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/
 
 /***********************************************************************
 * Video Session Migration System, 2004                                 *
 * Author: Klaus Schoeffmann                                            *
 ************************************************************************/
 
 
#ifndef __MovieListCreation_hpp__
#define __MovieListCreation_hpp__
#include "../../../src/global.hpp"

#ifdef WIN32
#include <winsock2.h>
#else
#include <unistd.h>
#include <time.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <unistd.h>
#include <arpa/inet.h>
#endif

#include <string>
#include <iostream>
#include <sstream>
#include <fstream>
#include <vector>
#include <time.h>

#include <sys/types.h>
#include "sord/directory.h"
#include "sord/entry.h"

#include "Server.hpp"
#include "MovieList.hpp"


//maximum number of characters for meta-data (description)
#define MAX_DESC_SIZE  255

using namespace std;

class MovieListCreation
{
  class DirNotFound { }; //exception
  class GetHostNameProblem { }; //exception
  
  vector<char*> knownFileTypes;
  
  /** structure holding the current server configuration */
  ServerConfig *sc;
  
  /** ip-address used for Resource-Tag */
  char address[MAX_STR_LEN];
  /** port used for Resource-Tag */
  u32 port;
  /** movie-id; starts at 1 and will be increased on every entry */
  u32 movieId;
  /** buffer for movielist */
  string data;
  /** buffer for related resources */
  string resources;
  /** flag: true if at least one file was found; otherwise false */
  bool atLeastOneFile;
  
  /** Returns the header for the DigitalItem. */
  string createHeader();
  /**Returns a Selection-Tag for the specified file @fname containing
        - Name
        - Description
        - Image
        
    To create image and description, this function looks for additional files.
    If in @metadatadir there exists a file with same filename as @fname
    but with extension '.jpg' (image) or '.txt' (description), this data
    will be used...*/
  string createSelection(const char *metadatadir, const char *fname);
  /** analyzes a directory in order to find files with known file types */
  void analyzeDir(char *path, char *fnprefix, bool recurseDirectories);
  /** Returns a Resource-Tag for the specified file @fname. */
  string createResource(const char *fname, const char *fnprefix);
  /** Returns the footer for the DigitalItem. */
  string createFooter();
  /** Checks if file is a known filetype. (e.g. "mp4") */
  bool isKnownFiletype(char *fname);
  /** Create the HTML Header */
  string getHTMLHeader();
  /** Create the HTML Footer which is used on every page. */
  string getHTMLFooter();
  

  public:
    /** default construtor; 
        Param @sc holds the current server configuration*/
    MovieListCreation(ServerConfig *sc);
    /** Core-function. Writes an XML-file (MPEG21-DigitalItem-conform)
       containing a list of available movies.*/
    void createMovieList();
};

#endif
