 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	Server.hpp                                                    *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PS_SERVER_HPP
#define _PS_SERVER_HPP

#include <list>
class AdmissionControl;

#include "ContainerInfo.hpp"
#include "cache/CacheManager.hpp"
#include "net/Session.hpp"
#include "net/Url.hpp"


#define SERVER_BIND_ALLDEV "ALLDEV"
#define SERVERBENCHMARKCONFIGFILE ".muViServerBench"
#define SERVERCONFIGFILE ".muViServerrc"
#define SERVERTCFILE ".muViServertc"

#define HOMEDIR "HOMEDIR="
#define MOVIELISTXMLFILE "MOVIELISTXMLFILE="
#define MOVIELISTHTMLFILE "MOVIELISTHTMLFILE="
#define METADATADIR "METADATADIR="
#define SWITCHDIR "SWITCHDIR="
#define RECURSE "RECURSIVE="
#define PORT "PORT="
#define BINDTO "BINDTO="
#define YES "YES"
#define NO "NO"
#define CONFIG_RTX "ENABLE_RTX="

typedef struct ServerConfig {
	char configfile[MAX_STR_LEN];
	char homeDir[MAX_STR_LEN];
	char movielistXMLFile[MAX_STR_LEN];
        char movielistHTMLFile[MAX_STR_LEN];
	char metadataDir[MAX_STR_LEN];
	char switchDir[MAX_STR_LEN];
	bool recurseDirectories;
	bool enableRtx;
	unsigned int port;
	char bindToIP[MAX_STR_LEN];
	ServerConfig() {
		strcpy(homeDir,".");
		strcpy(movielistXMLFile, "./metadata/MovieList.xml");
                strcpy(movielistHTMLFile, "./metadata/MovieList.html");
		strcpy(metadataDir, "./metadata");
		recurseDirectories=false;
		strcpy(switchDir,"./switch");
#ifdef WIN32
		port=554u;
		enableRtx=false;
#else
		port=5554u;
		enableRtx=true;
#endif
		strcpy(bindToIP,SERVER_BIND_ALLDEV);
	};
} ServerConfig;

/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: Server.hpp,v 1.1 2005/01/06 22:01:33 klschoef Exp $
 *
 *@short <short description>
 *
 */
class Server {
protected:
	bool createSocket();
public:
	/** creates a server, rootDir is deep-copied, cm is only shallow copied, also am */
	Server(const ServerConfig& conf, CacheManager * cm,AdmissionControl* am, TerminalCapabilities* tc=NULL);
	virtual ~ Server();
	ContainerInfo *find(const Url *fileName);
	void startup();
	static bool loadServerConfig(struct ServerConfig* s);
	static bool saveServerConfig(const struct ServerConfig* s);
	static char* stripWhiteSpace(char* url); 
	static TerminalCapabilities* loadTermCaps();
protected:
	
	ServerConfig srvConf;
	list < Session * >sessionList;
	CacheManager *cm;
	int socket;
	bool enableRtx;
	AdmissionControl* admControl;
	TerminalCapabilities* defaultTC;
};

#endif
