#include "../../apps/muviproxy/Proxy.hpp"
#include "../../apps/muviserver/muviserver/Server.hpp"
#include "vitooki.hpp"
#include "global.hpp"
#include "VideoAdaptorChain.hpp"
#include "container/ContainerFile.hpp"
#include "container/ContainerDemux.hpp"
#include "adaptors/MP4Decoder.hpp"
#include "adaptors/MP4Encoder.hpp"
#include "adaptors/YUVColorReductionAdaptor.hpp"
#include "adaptors/TimeMeasureAdaptor.hpp"
#include "DataChannel.hpp"
#ifdef WIN32
#include <sys/timeb.h>
#endif

#define NRDATACHANNELS 6

double readParameter(const char* text,FILE* fp,double minVal,double maxVal);
int readParameter(const char* text,FILE* fp,int minVal,int maxVal);
void generateConfigs(const char* video);
void printHelp();	
int benchmarkCPU(const char* video);
static long getdifftime(struct timeval *t1, struct timeval *t2);

double readParameter(const char* text,FILE* fp,double minVal,double maxVal)
{
	assert(minVal<=maxVal);
	char tmp[MAX_STR_LEN];
	double param=0.0;
	tmp[0]='\0';
	bool stop=false;
	while(!stop) {
		printf("%s",text);
		fgets(tmp,MAX_STR_LEN-1,fp);
		if(1==sscanf(tmp,"%lf",&param) && param>=minVal && param<=maxVal) {

			stop=true;
		}
		else {
			printf("Illegal value detected! Valid range is %.2lf--%.2lf\n",minVal,maxVal);
		}
	}
	return param;
}
int readParameter(const char* text,FILE* fp,int minVal,int maxVal)
{
	assert(minVal<=maxVal);
	char tmp[MAX_STR_LEN];
	int param=0;
	tmp[0]='\0';
	bool stop=false;
	while(!stop) {
		printf("%s",text);
		fgets(tmp,MAX_STR_LEN-1,fp);
		if(1==sscanf(tmp,"%i",&param) && param>=minVal && param<=maxVal) {

			stop=true;
		}
		else {
			printf("Illegal value detected! Valid range is %i--%i\n",minVal,maxVal);
		}
	}
	return param;
};

void generateConfigs(const char* video)	
{
	char serverCF[MAX_STR_LEN];
	char proxyCF[MAX_STR_LEN];
	
#ifdef WIN32
	strcpy(serverCF,getenv("USERPROFILE"));
	strcat(serverCF,"\\");
#else
	strcpy(serverCF,getenv("HOME"));
	strcat(serverCF,"/");
#endif
	strcpy(proxyCF,serverCF);
	strcat(serverCF,SERVERBENCHMARKCONFIGFILE);
	strcat(proxyCF,PROXYBENCHMARKCONFIGFILE);
	double disk=readParameter("Estimation of your AVERAGE hard disc speed in MByte (1.0-200.0): ",stdin,1.0,200.0);
	double diskUse=readParameter("How many percentage is the application allowed to use (10-100): ",stdin,10.0,100.0);
	diskUse/=100.0;
	double net=readParameter("Estimation of your network upload speed in Mbit (0.5-1000.0): ",stdin,0.5,1000.0);
	double netUse=readParameter("How many percentage is the application allowed to use (10-100): ",stdin,10.0,100.0);
	netUse/=100.0;
	double cpuUse=readParameter("How many CPU percentage is the application allowed to use (10-100): ",stdin,10.0,100.0);
	int CPU=benchmarkCPU(video);
	if(CPU<1) {
		printf("Failed to calculate CPU speed\n");
		CPU=readParameter("Estimation of your CPU decoding speed in pixels (40000000 for P4 2Ghz (>1000000): ",
		stdin,1000000,2147483647);
	}
	cpuUse/=100.0;
	cpuUse*=CPU;
	CPU=(int)cpuUse;

	FILE* serverFP=fopen(serverCF,"wb");
	FILE* proxyFP=fopen(proxyCF,"wb");
	if(serverFP) {

		fprintf(serverFP,"%.3lf %.3lf %i\n# networkInMBit diskInMByte cpuInDecodingPixels\n",
			net*netUse,disk/diskUse,CPU);
		fclose(serverFP);
		printf("Successfully written %s!\n",serverCF);
	}
	if(proxyFP){
		//disk needs Mbit not Mbyte but for users MByte is more convenient
		fprintf(proxyFP,"%.3lf %.3lf %i\n# networkInMBit diskInMByte cpuInDecodingPixels\n",
			net*netUse,disk/diskUse,CPU);
		fclose(proxyFP);
		printf("Successfully written %s!\n",proxyCF);
	}
};

/** returns negative number on error */
int benchmarkCPU(const char* video)
{
	ContainerInfo* file=ContainerFile::loadContainerInfo(video);
	if(!file)
		return -1;
	if(file->getFirstVisualES()==NULL)
		return -2;

	ContainerInfo* clones[NRDATACHANNELS];
	DataChannel* dc[NRDATACHANNELS];
	Adaptor* adaptor[NRDATACHANNELS];
	TimeMeasureAdaptor* tma[NRDATACHANNELS][2];
	int i=0;
	for(i=0;i<NRDATACHANNELS;i++) {
		clones[i]=file->clone();
		VideoESInfo* vid=clones[i]->getFirstVisualES();
		VideoAdaptorChain* a=new VideoAdaptorChain();
		tma[i][0]=new TimeMeasureAdaptor(new MP4Decoder(vid,false));
		a->addAdaptor(tma[i][0]);
		tma[i][1]=new TimeMeasureAdaptor(new MP4Encoder(vid,vid->getAvgBandwidth()*8/10,(int)vid->getFPS()));
		a->addAdaptor(tma[i][1]);
		adaptor[i]=a;
		a=NULL;
		dc[i]=ContainerDemux::createDataChannel(vid,NULL,adaptor[i]);
	}

	struct timeval t1;
	struct timeval t2;
	gettimeofday(&t1, NULL);
	for(i=0;i<NRDATACHANNELS;i++) {
		// start the threads
		dc[i]->start();
	}
	for(i=0;i<NRDATACHANNELS;i++) {
		// start the threads
		if(dc[i]->running())
			dc[i]->wait();
	}
	gettimeofday(&t2, NULL);
	// extract time values

	long timeElapsedMS=(getdifftime(&t1, &t2)/1000);
	u32 dur=(u32)file->getFirstVisualES()->getDurationInMs();
	float fps=file->getFirstVisualES()->getFPS();
	
	printf("Time Elapsed %ld for %i videos of duration %u\n",
		timeElapsedMS,NRDATACHANNELS,dur);

	
	u64 pixelsPerSecond=(u64)fps*file->getFirstVisualES()->getWidth()*file->getFirstVisualES()->getHeight()*NRDATACHANNELS;
	// now scale the pixelsPerSecond value to achieve real-time performance
	pixelsPerSecond*=dur;
	pixelsPerSecond/=timeElapsedMS;
	// one third of the time was used for decoding, 2/3 for encoding
	// we have to make this assumption because with more than one thread running, the measured time values are of no use
	// which we get from the TimeMeasureAdaptor
	// calc: multply two third of the pixels value with two to get decoding performance
	pixelsPerSecond+=(2*pixelsPerSecond/3);
	u32 decodingPixelsPerSecond=(u32)(pixelsPerSecond);
	if(pixelsPerSecond>0xffffffffu) {
		printf("WARNING: system too fast!Clipping performance value\n");
		decodingPixelsPerSecond=0xffffffffu;
	}
	printf("Decoding Speed in Pixels is %u\n",decodingPixelsPerSecond);

	//cleanup
	for(i=0;i<NRDATACHANNELS;i++) {
		delete dc[i];
		delete clones[i];
	}
	return decodingPixelsPerSecond;

}

void printHelp()
{
	printf("muviserverbench <video>\n\n Make sure that the video is as large as possible for accurate results (5min should be enough)!\n");
}


static long getdifftime(struct timeval *t1, struct timeval *t2)
{
	long ret = 0;

	ret = t2->tv_usec - t1->tv_usec;
	ret += (t2->tv_sec - t1->tv_sec) * 1000000;

	return ret;
};


int main(int argc, char *argv[])
{

	vitooki_init_all();
	if(argc!=2) {
		printHelp();
		exit(1);
	}
	generateConfigs(argv[1]);
	return 0;
};
	
	
