#include "MuviServerConfigImpl.hpp"
#include "../muviserver/Server.hpp"
#include "../../muviproxy/Proxy.hpp"
#include "metadata/TerminalCapabilities.hpp"
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qstring.h>
#include <qfiledialog.h>
#include <qmessagebox.h>

MuviServerConfig::MuviServerConfig( QWidget* parent, const char* name, bool modal, WFlags fl):
Configurator(parent,name,modal,fl)
{
	loadCurrentConfig(true);
};

void MuviServerConfig::reloadCurrentTab()
{
	loadCurrentConfig(false);
};

void MuviServerConfig::loadCurrentConfig(bool reloadAll)
{
	if( (tabWidget->currentPage() && 
			strcmp(tabWidget->currentPage()->name(), "Server")==0) ||
			reloadAll ) { // OR part is for startup when no tabwidget is set
		loadCurrentServerConfig();
		loadCurrentServerTermCaps();
	}

	if( (tabWidget->currentPage() && 
			strcmp(tabWidget->currentPage()->name(), "Proxy")==0) ||
			reloadAll ) { // OR part is for startup when no tabwidget is set
		loadCurrentProxyConfig();
		loadCurrentProxyTermCaps();
	}

};

void MuviServerConfig::loadCurrentServerConfig()
{
	struct ServerConfig servConf;
	char temp[MAX_STR_LEN];
#ifdef WIN32
  QString str(getenv("USERPROFILE"));
  str.append("\\");
#else
  QString str(getenv("HOME"));
  str.append("/");
#endif
  str.append(SERVERCONFIGFILE);
	strcpy(servConf.configfile,str.latin1());
	if(!Server::loadServerConfig(&servConf)) {
		// save defaults
		videoDirectory->setText(".");
		switchDirectory->setText("./switch");
		checkSearchRec->setChecked(false);
		serverPort->setText("554");
#ifdef WIN32
		checkEnableRTX->setChecked(false);
#else
		checkEnableRTX->setChecked(true);
#endif
	}
	else {
		videoDirectory->setText(servConf.homeDir);
		switchDirectory->setText(servConf.switchDir);
		checkSearchRec->setChecked(servConf.recurseDirectories);
		sprintf(temp,"%u",servConf.port);
		serverPort->setText(temp);
		checkEnableRTX->setChecked(servConf.enableRtx);
	}
};

void MuviServerConfig::loadCurrentServerTermCaps()
{
	TerminalCapabilities* tc=Server::loadTermCaps();
	if(tc) {
		checkEnableGateway->setChecked(true);
		
		dimX->setText(tr("%1").arg((int)tc->getDisplayWidth()));
		dimY->setText(tr("%1").arg((int)tc->getDisplayHeight()));
		frameRate->setText(tr("%1").arg((int)tc->getDisplayRefreshRate()));
		bitRate->setText(tr("%1").arg((int)tc->getMinGuaranteedNetworkCapacityInByte()*8/1024));
		checkEnableColor->setChecked(tc->getColorDisplay());
		checkAllowAudio->setChecked( (tc->getNumAudioChannels()>0));
	}
	else {
		checkEnableGateway->setChecked(false);
		// set defaults
		dimX->setText("352");
		dimY->setText("288");
		frameRate->setText("50");
		bitRate->setText("1024");
		checkEnableColor->setChecked(true);
		checkAllowAudio->setChecked(true);
	}
	termCapsServerClicked();
};

void MuviServerConfig::termCapsServerClicked()
{
	if(checkEnableGateway->isChecked()) {
		dimX->setEnabled(true);
		dimY->setEnabled(true);
		frameRate->setEnabled(true);
		bitRate->setEnabled(true);
		checkEnableColor->setEnabled(true);
		checkAllowAudio->setEnabled(true);
	}
	else {
		dimX->setEnabled(false);
		dimY->setEnabled(false);
		frameRate->setEnabled(false);
		bitRate->setEnabled(false);
		checkEnableColor->setEnabled(false);
		checkAllowAudio->setEnabled(false);
	}
}

void MuviServerConfig::loadCurrentProxyConfig()
{
	struct ProxyConfig servConf;
	char temp[MAX_STR_LEN];
#ifdef WIN32
  QString str(getenv("USERPROFILE"));
  str.append("\\");
#else
  QString str(getenv("HOME"));
  str.append("/");
#endif
  str.append(PROXYCONFIGFILE);
	strcpy(servConf.configfile,str.latin1());
	if(!Proxy::loadProxyConfig(&servConf)) {
		// save defaults
		videoDirectory_2->setText(".");
		
		checkSearchRec_2->setChecked(false);
		proxyPort->setText("3128");
		cacheSize->setText("1024");
#ifdef WIN32
		checkEnableRTX_2->setChecked(false);
#else
		checkEnableRTX_2->setChecked(true);
#endif
	}
	else {
		videoDirectory_2->setText(servConf.homeDir);
		sprintf(temp,"%u",(unsigned)servConf.maxCacheSize/(1024*1024));
		cacheSize->setText(temp);
		checkSearchRec_2->setChecked(servConf.recurseDirectories);
		sprintf(temp,"%u",servConf.port);
		proxyPort->setText(temp);
		checkEnableRTX_2->setChecked(servConf.enableRtx);
	}
};

void MuviServerConfig::loadCurrentProxyTermCaps()
{
	TerminalCapabilities* tc=Proxy::loadTermCaps();
	if(tc) {
		checkEnableGateway_2->setChecked(true);
		
		dimX_2->setText(tr("%1").arg((int)tc->getDisplayWidth()));
		dimY_2->setText(tr("%1").arg((int)tc->getDisplayHeight()));
		frameRate_2->setText(tr("%1").arg((int)tc->getDisplayRefreshRate()));
		bitRate_2->setText(tr("%1").arg((int)tc->getMinGuaranteedNetworkCapacityInByte()*8/1024));
		checkEnableColor_2->setChecked(tc->getColorDisplay());
		checkAllowAudio_2->setChecked( (tc->getNumAudioChannels()>0));
	}
	else {
		checkEnableGateway_2->setChecked(false);
		// set defaults
		dimX_2->setText("352");
		dimY_2->setText("288");
		frameRate_2->setText("50");
		bitRate_2->setText("1024");
		checkEnableColor_2->setChecked(true);
		checkAllowAudio_2->setChecked(true);
	}
	termCapsProxyClicked();
};

void MuviServerConfig::termCapsProxyClicked()
{
	if(checkEnableGateway_2->isChecked()) {
		dimX_2->setEnabled(true);
		dimY_2->setEnabled(true);
		frameRate_2->setEnabled(true);
		bitRate_2->setEnabled(true);
		checkEnableColor_2->setEnabled(true);
		checkAllowAudio_2->setEnabled(true);
	}
	else {
		dimX_2->setEnabled(false);
		dimY_2->setEnabled(false);
		frameRate_2->setEnabled(false);
		bitRate_2->setEnabled(false);
		checkEnableColor_2->setEnabled(false);
		checkAllowAudio_2->setEnabled(false);
	}
}
MuviServerConfig::~MuviServerConfig()
{
}

void MuviServerConfig::butSwitchDirClicked()
{
	QString currentDir=switchDirectory->text();
	QString caption("Choose the switch directory");
	QString newDir=QFileDialog::getExistingDirectory(currentDir,this,NULL,caption,true);
	if(!newDir.isEmpty()) {
		// a valid dir
		switchDirectory->setText(newDir);
	}
}

void MuviServerConfig::butVidDirProxyClicked()
{
	QString currentDir=videoDirectory_2->text();
	QString caption("Choose the directory for the proxy cache");
	QString newDir=QFileDialog::getExistingDirectory(currentDir,this,NULL,caption,true);
	if(!newDir.isEmpty()) {
		// a valid dir
		videoDirectory_2->setText(newDir);
	}
}

void MuviServerConfig::butVidDirServerClicked()
{
	QString currentDir=videoDirectory->text();
	QString caption("Choose the directory for the server to search for videos");
	QString newDir=QFileDialog::getExistingDirectory(currentDir,this,NULL,caption,true);
	if(!newDir.isEmpty()) {
		// a valid dir
		videoDirectory->setText(newDir);
	}
}

void MuviServerConfig::saveConfig()
{
	bool b1=false,b2=false,b3=false,b4=false;
	b1=saveServerConfig();
	if(b1)
		b2=saveProxyConfig();
	if(b2)
		b3=saveServerTermCaps();
	if(b3)
		b4=saveProxyTermCaps();
	if(b4)
		QMessageBox::information(this,"All Saved","Save successful for Server and Proxy");
	else {
		char info[MAX_STR_LEN];
		sprintf(info,"Server Config: %s\n Server Term Caps: %s\nProxy Config: %s\n Proxy Term Caps: %s\n",
			(b1?"SUCCESS":"FAILURE"),(b2?"SUCCESS":"FAILURE"),(b3?"SUCCESS":"FAILURE"),(b4?"SUCCESS":"FAILURE"));
		QMessageBox::information(this,"Error",info);
	}
}

void MuviServerConfig::proxyPortChanged()
{
	QString temp=proxyPort->text();
	bool ok=false;
	int port=0;
	if(!temp.isEmpty()) {
		port=temp.toInt(&ok);
		if(ok && port>65535)
			proxyPort->setText("65535");
		else if(ok && port<10)
			proxyPort->setText("10");
	}
}

void MuviServerConfig::serverPortChanged()
{
	QString temp=serverPort->text();
	bool ok=false;
	int port=0;
	if(!temp.isEmpty()) {
		port=temp.toInt(&ok);
		if(ok && port>65535)
			serverPort->setText("65535");
		else if(ok && port<0)
			serverPort->setText("0");
	}
}

bool MuviServerConfig::saveServerConfig()
{
	QString caption("Error!");
	
	// extract all params from ServerConfig
	// check them if they are valid!
	// save the file
	struct ServerConfig s;
#ifdef WIN32
  QString str(getenv("USERPROFILE"));
  str.append("\\");
#else
  QString str(getenv("HOME"));
  str.append("/");
#endif
  str.append(SERVERCONFIGFILE);
	strcpy(s.configfile,str.latin1());
	if(videoDirectory->text().stripWhiteSpace().isEmpty()) {
		QString msg("No Server Directory is set! Save aborted.");
		QMessageBox::warning(this,caption,msg);
		return false;
	}
	strcpy(s.homeDir,videoDirectory->text().latin1());
	strcpy(s.switchDir,switchDirectory->text().latin1());
	strcpy(s.bindToIP,SERVER_BIND_ALLDEV);
	bool ok=false;
	s.port=serverPort->text().toInt(&ok);
	if(!ok) {
		QMessageBox::warning(this,caption,"Server Port is Invalid or not set! Save aborted.");
		return false;
	}

	s.enableRtx=checkEnableRTX->isChecked();
	s.recurseDirectories=checkSearchRec->isChecked();
	return Server::saveServerConfig(&s);
};

bool MuviServerConfig::saveProxyConfig()
{
	QString caption("Error!");
	
	// extract all params from ServerConfig
	// check them if they are valid!
	// save the file
	struct ProxyConfig s;
#ifdef WIN32
  QString str(getenv("USERPROFILE"));
  str.append("\\");
#else
  QString str(getenv("HOME"));
  str.append("/");
#endif
  str.append(PROXYCONFIGFILE);
	strcpy(s.configfile,str.latin1());
	if(videoDirectory_2->text().stripWhiteSpace().isEmpty()) {
		QString msg("No Proxy Directory is set! Save aborted.");
		QMessageBox::warning(this,caption,msg);
		return false;
	}
	bool ok=false;
	s.port=proxyPort->text().toInt(&ok);
	if(!ok) {
		QMessageBox::warning(this,caption,"Proxy Port is Invalid or not set! Save aborted.");
		return false;
	}
	strcpy(s.homeDir,videoDirectory_2->text().latin1());
	s.enableRtx=checkEnableRTX_2->isChecked();
	s.recurseDirectories=checkSearchRec_2->isChecked();
	s.maxCacheSize=cacheSize->text().toInt(&ok);
	if(s.maxCacheSize<1)
		ok=false;
	if(!ok) {
		QMessageBox::warning(this,caption,"Proxy Size is Invalid or not set! Save aborted.");
		return false;
	}
	s.maxCacheSize*=1024; // MB-->KB
	s.maxCacheSize*=1024; // KB-->Byte
	return Proxy::saveProxyConfig(&s);
};

bool MuviServerConfig::saveServerTermCaps()
{
	char tcFile[MAX_STR_LEN];
#ifdef WIN32
	QString str(getenv("USERPROFILE"));
	str.append("\\");
#else
	QString str(getenv("HOME"));
	str.append("/");
#endif
	str.append(SERVERTCFILE);
	strcpy(tcFile,str.latin1());
	if(checkEnableGateway->isChecked()) {
		bool ok=false;
		int width=0;
		width=dimX->text().toInt(&ok);
		if(!ok) {
			QMessageBox::warning(this,"Error!","dimX in Server Gatewaying is wrong. Save aborted");
			return false;
		}
		int height=0;
		height=dimY->text().toInt(&ok);
		if(!ok) {
			QMessageBox::warning(this,"Error!","dimY in Server Gatewaying is wrong. Save aborted");
			return false;
		}
		int fps=frameRate->text().toInt(&ok);
		if(ok) {
			if(fps<1)
				ok=false;
		}
		if(!ok) {
			QMessageBox::warning(this,"Error!","FrameRate in Server Gatewaying is wrong. Must be a value greater 0. Save aborted");
			return false;
		}
		int bitrate=bitRate->text().toInt(&ok);
		if(ok) {
			if(bitrate<1)
				ok=false;
		}
		if(!ok) {
			QMessageBox::warning(this,"Error!","Bitrate in Server Gatewaying is wrong. Must be a value greater 0. Save aborted");
			return false;
		}
		u32 brInBit=bitrate*1024;
		TerminalCapabilities tc(brInBit/4,brInBit,32,checkEnableColor->isChecked(),
			width,height,(u32)fps,64.0,brInBit/8,brInBit/8,1000,1000,(checkAllowAudio->isChecked()?2:0),16,44100);

	
		return tc.saveToFile(tcFile,true);
	}
	else {
		// delete tc file
		FILE* fp=fopen(tcFile,"r");
		if(fp) {
			fclose(fp);
			remove(tcFile);
		}
	}
	return true;
};

bool MuviServerConfig::saveProxyTermCaps()
{
	char tcFile[MAX_STR_LEN];
#ifdef WIN32
	QString str(getenv("USERPROFILE"));
	str.append("\\");
#else
	QString str(getenv("HOME"));
	str.append("/");
#endif
	str.append(PROXYTCFILE);
	strcpy(tcFile,str.latin1());
	if(checkEnableGateway_2->isChecked()) {
		bool ok=false;
		int width=0;
		width=dimX_2->text().toInt(&ok);
		if(!ok) {
			QMessageBox::warning(this,"Error!","dimX in Proxy Gatewaying is wrong. Save aborted");
			return false;
		}
		int height=0;
		height=dimY_2->text().toInt(&ok);
		if(!ok) {
			QMessageBox::warning(this,"Error!","dimY in Proxy Gatewaying is wrong. Save aborted");
			return false;
		}
		int fps=frameRate_2->text().toInt(&ok);
		if(ok) {
			if(fps<1)
				ok=false;
		}
		if(!ok) {
			QMessageBox::warning(this,"Error!","FrameRate in Proxy Gatewaying is wrong. Must be a value greater 0. Save aborted");
			return false;
		}
		int bitrate=bitRate_2->text().toInt(&ok);
		if(ok) {
			if(bitrate<1)
				ok=false;
		}
		if(!ok) {
			QMessageBox::warning(this,"Error!","Bitrate in Proxy Gatewaying is wrong. Must be a value greater 0. Save aborted");
			return false;
		}
		u32 brInBit=bitrate*1024;
		TerminalCapabilities tc(brInBit/4,brInBit,32,checkEnableColor->isChecked(),
			width,height,(u32)fps,64.0,brInBit/8,brInBit/8,1000,1000,(checkAllowAudio->isChecked()?2:0),16,44100);
	
		return tc.saveToFile(tcFile,true);
	}
	else {
		// delete tc file
		FILE* fp=fopen(tcFile,"r");
		if(fp) {
			fclose(fp);
			remove(tcFile);
		}
	}
	return true;
};
