 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: SemiProxy.hpp                                                 *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 * Video Session Migration System, 2004                                 *
 * Author: Klaus Schoeffmann                                            *
 ************************************************************************/


#ifndef __SemiProxy_hpp__
#define __SemiProxy_hpp__

//#include <string>
#include <vector>
#include <signal.h>


#ifdef WIN32
#include <winsock2.h>
#include <ws2tcpip.h>
#endif

//#include "SSession.hpp"
#include "SemiProxySession.hpp"
#include "SessionMgmt.hpp"

#include "UserList.hpp"
#include "User.hpp"
#include "ProfileList.hpp"
#include "Profile.hpp"


/**
 * This class is the main-class of the semiproxy.
 * On startup it reads all users and profiles into
 * memory, creates a SessionManagement-object (which
 * listens for requests from the semiserver) and
 * creates another socket, on which incoming connections
 * will be handled (by SemiProxySession.cpp).
 *
 * @author Klaus Schoeffmann
 */

class SemiProxy
{

  private:
    /** class reference to the last created SemiProxy object */
    static SemiProxy *refLastCreated;
    /** reference to the session-management */
    class SessionMgmt *sessMgmt;
    /** reference to the user-list */
    UserList *ul;
    /** reference to the profile-list */
    ProfileList *prol;
    /** list of startet SemiProxySession objects */
    vector<class SemiProxySession*> spsList;

    /** port to use for semi-proxy */
    uint proxyPort;

    /** flag, if 'destination=<address>' should be added to Setup-Message */
    bool addDestination;
    /** flag, if mpeg21 terminal capabilities schould
        be added to Describe-Message */
    bool addTermCaps;
    /** flag, if GET_PARAMETER should be used to obain mediatime
        of session from mediaserver. */
    bool requestServerPosition;
    /** flag, if GET_PARAMETER should be used to obain mediatime
        of session from client. */
    bool requestClientPosition;

    /** flag which is true until semiproxy is running */
    static bool semiproxyRunning;

  public:
    /** default constructor.
        Starts Proxy (and Session-Management) and reads all users/profiles into memory.
        @param proxyPort port to use for starting the proxy
        @param responderPort port to use for starting the Session-Management
    */
    SemiProxy(uint proxyPort, uint responderPort);
    /** default destructor */
    ~SemiProxy();
    /** Create a socket for communication with the media-server.
        This function is called from SemiProxySession on demand
        (e.g. on Describe message). */
    int createRtspServerSocket(const char *host, int port);
    /** Open the socket for communication with the client (the player).
        Every incoming connection on this socket is furhter handled
        from SemiProxySession */
    int openClientSocket(int proxyport);
    /** Add an active session to the list in Session-Management */
    void addSession(SSession *asess);
    /** Remove a session from the list in Session-Management */
    void removeSession(SSession *asess);
    /** @return a reference to the active session in Session-Management,
        having the Rtsp-SessionId specified by @param RtspSessionId */
    SSession *getSession(uint RtspSessionId);
    /** @return a reference to the userlist */
    UserList *getUserList();
    /** @return a reference to the profilelist */
    ProfileList *getProfileList();
    /** @return port on which the semiproxy listens for new connections. */
    uint getProxyPort();

    /** set addDestination flag */
    void setAddDestination(bool shouldAddDestination);
    /** set addTermcaps flag */
    void setAddTermCaps(bool shouldAddTermCaps);
    /** set requestServerPosition flag */
    void setRequestServerPosition(bool shouldRequestServerPosition);
    /** set requestClientPosition flag */
    void setRequestClientPosition(bool shouldRequestClientPosition);
    /** @return addDestination flag */
    bool shouldAddDestination();
    /** @return addTermcaps flag */
    bool shouldAddTermCaps();
    /** @return requestServerPosition flag */
    bool shouldRequestServerPosition();
    /** @return requestClientPosition flag */
    bool shouldRequestClientPosition();

    /** @return true if semiproxy is running, otherwise false */
    static bool isSemiProxyRunning();
    /** set semiproxyRunning flag */
    static void setSemiProxyRunning(bool semiproxyRunning);
    /** @return reference to the last created object */
    static SemiProxy *getRefLastCreated();
    
    /** cleanup used memory spaced */
    void cleanup();

};

#endif
