/***********************************************************************
*                                                                      *
* ViTooKi                                                              *
*                                                                      *
* title: ServerCommunication.cpp                                       *
*                                                                      *
*                                                                      *
*                                                                      *
* ITEC institute of the University of Klagenfurt (Austria)             *
* http://www.itec.uni-klu.ac.at                                        *
*                                                                      *
*                                                                      *
* For more information visit the ViTooKi homepage:                     *
* http://ViTooKi.sourceforge.net                                       *
* vitooki-user@lists.sourceforge.net                                   *
* vitooki-devel@lists.sourceforge.net                                  *
*                                                                      *
* This file is part of ViTooKi, a free video toolkit.                  *
* ViTooKi is free software; you can redistribute it and/or             *
* modify it under the terms of the GNU General Public License          *
* as published by the Free Software Foundation; either version 2       *
* of the License, or (at your option) any later version.               *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
* MA  02111-1307, USA.                                                 *
*                                                                      *
***********************************************************************/

/***********************************************************************
*                                                                      *
*  REVISION HISTORY:                                                   *
*                                                                      *
*                                                                      *
*                                                                      *
***********************************************************************/

/***********************************************************************
* Video Session Migration System, 2004                                 *
* Author: Klaus Schoeffmann                                            *
************************************************************************/


#include "ServerCommunication.hpp"


/***********************************************************************/
ServerCommunication::ServerCommunication(int clientsock, struct sockaddr_in* caller, SessionMgmt *sMgmt)
{
  this->clientsock = clientsock;
  this->caller = caller;
  this->sMgmt = sMgmt;
}

/***********************************************************************/
ServerCommunication::~ServerCommunication()
{
}


/***********************************************************************/
void ServerCommunication::sendSessionsToServer()
{
  int entries = 0;
  uint i = 0;
  vector<SSession*> *list = sMgmt->getVector();

  Globals::sdebug << "\nSC::sendSessionsToServer(): listSize=" << list->size();

  for (i = 0; i < list->size(); i++)
  {
    SSession *sess = (*list)[i];

    ulong tsCreated = sess->getTimestampCreated();
    ulong tsClosed = sess->getTimestampClosed();

    if (tsClosed != 0)
    {
      uint remainTime = (tsClosed - tsCreated) / 3600000;

      Globals::sdebug << " remainTime=" << remainTime << "\n";

      if (Globals::getHoursToPreserveClosedSessions() > remainTime &&
          sess->getSemiMovieId() != NULL)
      {
        entries++;
      }
      else
      {
        sMgmt->removeSession(sess);
      }
    }
    else
    {
      entries++;
    }
  }



  //first: send number of entries in the list
#ifndef WIN32
  send(clientsock, &entries, sizeof(int), 0);
#else
  send(clientsock, (const char *)&entries, sizeof(int), 0);
#endif

  Globals::sdebug << "\nSC::sendSessionsToServer(): entries=" << entries;

  //then send whole vector
  list = sMgmt->getVector();
  for (i=0; i < list->size(); i++)
  {
    SSession *asess = (*list)[i];

    uint RtspSessionId = asess->getRtspSessionId();
    char *SemiMovieId = asess->getSemiMovieId();
    uint movieIdlen = strlen(SemiMovieId);
    char *URL = asess->getUrl();
    int URLlen = strlen(URL);
    char *clientAdr = asess->getClientAdr();
    uint clientAdrlen = strlen(clientAdr);
    uint RtspSessionTOId = asess->getOriginRtspSessionId();
    ulong secsElapsed = asess->getElapsedMsecs();
    ulong duration = asess->getDuration();
    ulong prevDuration = asess->getPreviousSessionDuration();
    uint sectionCount = asess->getTimeSectionCount();
    uint State = asess->getState();
    ulong tsCreation = asess->getTimestampCreated();
    ulong tsClosed = asess->getTimestampClosed();

#ifndef WIN32
    send(clientsock, (uint *) &RtspSessionId, sizeof(uint), 0);
    send(clientsock, (uint *) &URLlen, sizeof(uint), 0);
    send(clientsock, (char *) URL, sizeof(char) * (URLlen+1), 0);
    send(clientsock, (uint *) &clientAdrlen, sizeof(uint), 0);
    send(clientsock, (char *) clientAdr, sizeof(char) * (clientAdrlen+1), 0);
    send(clientsock, (uint *) &movieIdlen, sizeof(uint), 0);
    send(clientsock, (char *) SemiMovieId, sizeof(char) * (movieIdlen+1), 0);
    send(clientsock, (uint *) &RtspSessionTOId, sizeof(uint), 0);
    send(clientsock, (ulong *) &secsElapsed, sizeof(ulong), 0);
    send(clientsock, (ulong *) &duration, sizeof(ulong), 0);
    send(clientsock, (ulong *) &prevDuration, sizeof(ulong), 0);
    send(clientsock, (uint *) &sectionCount, sizeof(uint), 0);
    send(clientsock, (uint *) &State, sizeof(uint), 0);
    send(clientsock, (ulong *) &tsCreation, sizeof(ulong), 0);
    send(clientsock, (ulong *) &tsClosed, sizeof(ulong), 0);
#else
	send(clientsock, (const char *)&RtspSessionId, sizeof(uint), 0);
    send(clientsock, (const char *)&URLlen, sizeof(uint), 0);
    send(clientsock, URL, sizeof(char) * (URLlen+1), 0);
    send(clientsock, (const char *)&clientAdrlen, sizeof(uint), 0);
    send(clientsock, clientAdr, sizeof(char) * (clientAdrlen+1), 0);
    send(clientsock, (const char *)&movieIdlen, sizeof(uint), 0);
    send(clientsock, SemiMovieId, sizeof(char) * (movieIdlen+1), 0);
    send(clientsock, (const char *)&RtspSessionTOId, sizeof(uint), 0);
    send(clientsock, (const char *)&secsElapsed, sizeof(ulong), 0);
    send(clientsock, (const char *)&duration, sizeof(ulong), 0);
    send(clientsock, (const char *)&prevDuration, sizeof(ulong), 0);
    send(clientsock, (const char *)&sectionCount, sizeof(uint), 0);
    send(clientsock, (const char *)&State, sizeof(uint), 0);
    send(clientsock, (const char *)&tsCreation, sizeof(ulong), 0);
    send(clientsock, (const char *)&tsClosed, sizeof(ulong), 0);
#endif

    Globals::sdebug << "\nSC::sendSessionsToServer(): - SSession sent: " << RtspSessionId; cout.flush();
    Globals::sdebug << "\n            " << URL << "(" << URLlen << ")";
    Globals::sdebug << "\n            " << clientAdr << "(" << clientAdrlen << ")";
    Globals::sdebug << "\n            " << SemiMovieId << "(" << movieIdlen << ")";
    Globals::sdebug << "\n            " << RtspSessionTOId;
    Globals::sdebug << "\n            " << secsElapsed;
    Globals::sdebug << "\n            " << duration;
    Globals::sdebug << "\n            " << prevDuration;
    Globals::sdebug << "\n            " << sectionCount;
    Globals::sdebug << "\n            " << State;
    Globals::sdebug << "\n            " << tsCreation;
    Globals::sdebug << "\n            " << tsClosed;

  }
}

