/***********************************************************************
*                                                                      *
* ViTooKi                                                              *
*                                                                      *
* title: SessionMgmt.cpp                                               *
*                                                                      *
*                                                                      *
*                                                                      *
* ITEC institute of the University of Klagenfurt (Austria)             *
* http://www.itec.uni-klu.ac.at                                        *
*                                                                      *
*                                                                      *
* For more information visit the ViTooKi homepage:                     *
* http://ViTooKi.sourceforge.net                                       *
* vitooki-user@lists.sourceforge.net                                   *
* vitooki-devel@lists.sourceforge.net                                  *
*                                                                      *
* This file is part of ViTooKi, a free video toolkit.                  *
* ViTooKi is free software; you can redistribute it and/or             *
* modify it under the terms of the GNU General Public License          *
* as published by the Free Software Foundation; either version 2       *
* of the License, or (at your option) any later version.               *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
* MA  02111-1307, USA.                                                 *
*                                                                      *
***********************************************************************/

/***********************************************************************
*                                                                      *
*  REVISION HISTORY:                                                   *
*                                                                      *
*                                                                      *
*                                                                      *
***********************************************************************/

/***********************************************************************
* Video Session Migration System, 2004                                 *
* Author: Klaus Schoeffmann                                            *
************************************************************************/


#include "SessionMgmt.hpp"

vector<SSession*> SessionMgmt::list;

/***********************************************************************/
SessionMgmt::SessionMgmt(int port)
{
  this->port = port;
}

/***********************************************************************/
SessionMgmt::~SessionMgmt()
{
  Globals::sdebug << "\nSessionMgmt::~ SessionMgmt() cleanup...";
  vector<SSession*>::iterator iter = list.begin();
  while (iter != list.end())
  {
    SSession *sess = *iter;
    delete sess;
    iter++;
  }
  list.clear();
  Globals::sdebug << "\nSessionMgmt::~ SessionMgmt() done!";
}

/***********************************************************************/
void SessionMgmt::addSession(SSession *sess)
{
  list.push_back(sess);
}

/***********************************************************************/
void SessionMgmt::removeSession(SSession *remove)
{
  if (remove == NULL) return;

  vector<SSession*>::iterator iter = list.begin();
  while (iter != list.end())
  {
    SSession *sess = *iter;
    if (sess->getRtspSessionId() == remove->getRtspSessionId())
      {
        Globals::sdebug << "\nSM::removeSession: session " << sess->getRtspSessionId() << " REMOVED !!!";
        iter = list.erase(iter);
      }
    else
    {
      iter++;
    }
  }

}

/***********************************************************************/
SSession* SessionMgmt::getSession(uint RtspSessionId)
{

  vector<SSession*>::const_iterator iter = list.begin();
  while (iter != list.end())
  {
    SSession *sess = *iter;
    if (sess->getRtspSessionId() == RtspSessionId)
    {
      return sess;
    }
    iter++;
  }

  //not found
  return NULL;

}

/***********************************************************************/
void SessionMgmt::listSessions()
{

  Globals::sdebug << "\nSM::listSessions(): Active Sessions:";
  vector<SSession*>::const_iterator iter = list.begin();
  while (iter != list.end())
  {
    SSession *sess = *iter;

    cout << endl << " - session: " << sess->getRtspSessionId();
    cout << endl << "            " << sess->getUrl();
    cout << endl << "            " << sess->getClientAdr();
    cout << endl << "            " << sess->getSemiMovieId();
    cout << endl << "            " << sess->getOriginRtspSessionId();
    cout << endl << "            " << sess->getElapsedMsecs();
    cout << endl << "            " << sess->getBufferingDelay()<< endl;
    iter++;
  }

}

/***********************************************************************/
int SessionMgmt::listenForServerRequests(int port)
{
  struct sockaddr_in sin, caller;
  int sock, newsocket;
  socklen_t len;

  memset((char*) &sin, 0, sizeof(sin));
  sin.sin_family = AF_INET;
  sin.sin_addr.s_addr = INADDR_ANY;
  sin.sin_port = htons(port);

  if ((sock = socket(PF_INET, SOCK_STREAM, 0)) < 0)
  {
    cerr << "\nSM::openSocket(): Error: Socket-Create";
    return 1;
  }

  const int reuse = 1;
  if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (const char *) &reuse, sizeof(reuse)) < 0)
  {
    cerr << "\nSM::openSocket(): Error: Socket-SetSockOpt";
    return 1;
  }

  if (bind(sock, (struct sockaddr*) &sin, sizeof(sin)) < 0)
  {
    cerr << "\nSM::openSocket(): Error: Socket-Bind";
    return 1;
  }

  //listen for new connections on this socket..
  listen(sock, MAX_PENDING);

  cout << "\nSM::openSocket(): accepting connections on port " << port <<" ...";
  while (SemiProxy::isSemiProxyRunning())
  {
    //accept connections
    if ((newsocket = accept(sock, (struct sockaddr*) &caller, &len)) < 0)
    {
      cerr << "\nSM::openSocket(): Error: Socket-Accept";
      return 1;
    }

    Globals::sdebug << "\nSM::openSocket(): socket connected with " << inet_ntoa(caller.sin_addr) << " on port " << caller.sin_port;

    //create a server-communication object and send all active
    //streaming-sessions through this socket...
    ServerCommunication *serverComm = new ServerCommunication(newsocket, &caller, this);
    serverComm->sendSessionsToServer();
    delete serverComm;
    serverComm = NULL;

    //sleep 20 microseconds
#ifndef WIN32
    usleep(20);
#else
	Sleep(20);
#endif
  }

  return 0; //success;
}

/***********************************************************************/
int SessionMgmt::getListSize()
{
  return list.size();
}

/***********************************************************************/
vector<SSession*>* SessionMgmt::getVector()
{
  return &list;
}

/***********************************************************************/
void SessionMgmt::run()
{
  listenForServerRequests(port);
 	this->exit();

}


