 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: SessionMgmt.hpp                                               *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/
 
 /***********************************************************************
 * Video Session Migration System, 2004                                 *
 * Author: Klaus Schoeffmann                                            *
 ************************************************************************/
 

#ifndef __SessionMgmt_hpp__
#define __SessionMgmt_hpp__

#include <string>
#include <vector>
#include <iostream>

#include "VThread.hpp"
#include "SSession.hpp"
#include "ServerCommunication.hpp"
#include "SemiProxy.hpp"

using namespace std;

#define MAX_PENDING 1


/**
 * This class is used for management of 
 * currently active streaming sessions.
 * Furthermore it listens for incoming connections
 * (from semiserver) and replies the list of 
 * active streaming-sessions (throuhg ServerCommunication.cpp).
 *
 * @author Klaus Schoeffmann
 */


class SessionMgmt: public VThread
{
private:
  /** list of all active streaming sessions */
  static vector<SSession*> list;  
  /** port used to listen on */
  int port;

  /** accept incoming requests specified @param port.
      Each request is handled by ServerCommunication. */
  int listenForServerRequests(int port);

public:
  /** default constructor */
  SessionMgmt(int port);
  /** default destructor */
  ~SessionMgmt();

  /** start this thread */
  void run();  
  /** add a streaming session to the list */
  void addSession(SSession *sess);
  /** remove a streaming session from the list */
  void removeSession(SSession *sess);
  /** @return a reference to the session object, which
      has the Rtsp-SessionId specified by @param RtspSessionId */
  SSession *getSession(uint RtspSessionId);
  /** print the content of the session-list to standard-out */
  void listSessions();
  /** @return number of entries in session-list. */
  int getListSize();
  /** @return a reference to the begin of the session-list. */
  vector<SSession*>* getVector();
};

#endif
