 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: TrackList.cpp                                                 *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 * Video Session Migration System, 2004                                 *
 * Author: Klaus Schoeffmann                                            *
 ************************************************************************/


#include "TrackList.hpp"


/***********************************************************************/
TrackList::TrackList()
{
  last = NULL;
}

/***********************************************************************/
TrackList::~ TrackList()
{
  Globals::sdebug << "\nTrackList::~TrackList() cleanup...";
  vector<Track*>::const_iterator iter = tracks.begin();
  for (; iter != tracks.end(); iter++)
  {
    Track *trk = *iter;
    delete trk;
  }
  tracks.clear();
  Globals::sdebug << "\nTrackList::~TrackList() done!\n";
}

/***********************************************************************/
vector<Track*>::const_iterator TrackList::begin()
{
  return tracks.begin();
}

/***********************************************************************/
vector<Track*>::const_iterator TrackList::end()
{
  return tracks.end();
}

/***********************************************************************/
void TrackList::addTrack(uint number)
{
  last = new Track(number);
  tracks.push_back(last);
  Globals::sdebug << "\nTrackList::addTrack(): added track with number " << number << "\n";
}

/***********************************************************************/
void TrackList::setIdOfLastTrack(uint sessionID)
{
  last->setId(sessionID);
}

/***********************************************************************/
Track* TrackList::getTrackByNumber(uint trackId)
{
  vector<Track*>::const_iterator iter = tracks.begin();
  for (; iter != tracks.end(); iter++)
  {
    Track *trk = *iter;

    if (trk->getNumber() == trackId) return trk;
  }

  return NULL;
}

/***********************************************************************/
Track* TrackList::getTrackById(uint sessionID)
{
  vector<Track*>::const_iterator iter = tracks.begin();
  for (; iter != tracks.end(); iter++)
  {
    Track *trk = *iter;

    if (trk->getId() == sessionID) return trk;
  }

  return NULL;
}

/***********************************************************************/
uint TrackList::getTCHavingState(TrackState state)
{
  uint count = 0;

  //Globals::sdebug << "\ngetTCHavingState: searching for state " << state << "...";

  vector<Track*>::const_iterator iter = tracks.begin();
  for (; iter != tracks.end(); iter++)
  {
    Track *trk = *iter;

    if (trk->getState() == state) count++;
  }

  //Globals::sdebug << "found " << count << "\n";

  return count;
}

/***********************************************************************/
uint TrackList::getTC()
{
  return tracks.size();
}

