 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: TrackList.hpp                                                 *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 * Video Session Migration System, 2004                                 *
 * Author: Klaus Schoeffmann                                            *
 ************************************************************************/


#ifndef __TrackList_hpp__
#define __TrackList_hpp__

#include <string>
#include <vector>
#include <assert.h>
#include "SemiGlobals.hpp"
#include "Track.hpp"

/**
 * This class is used by SemiProxySession in order to distinct
 * different tracks of a session.
 *
 * @author Klaus Schoeffmann
 */

class TrackList {

private:
  vector<Track*> tracks;

  /** point to last element in vector */
  Track *last;

public:
  /** default constructor */
  TrackList();
  /** default destructor */
  ~TrackList();

  /** @returns begin of tracks vector */
  vector<Track*>::const_iterator begin();
  /** @returns end of tracks vector */
  vector<Track*>::const_iterator end();

  /** add a track with a specified number to the list */
  void addTrack(uint number);
  /** set sessionid of last track */
  void setIdOfLastTrack(uint sessionID);
  /** get track with specified number */
  Track *getTrackByNumber(uint trackID);
  /** get track with specified sessionID */
  Track *getTrackById(uint sessionID);
  /** @returns number of tracks with specified state */
  uint getTCHavingState(TrackState state);
  /** @returns number of tracks */
  uint getTC();
};

#endif
