 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: URIParameters.hpp                                             *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 * Video Session Migration System, 2004                                 *
 * Author: Klaus Schoeffmann                                            *
 ************************************************************************/


#ifndef __URIParameters_hpp__
#define __URIParameters_hpp__

#include <string>
#include "SemiGlobals.hpp"

/**
 * This class contains all parameters, which were specified
 * in the Rtsp-URI using the (semi-used) syntax "/<param>=<val>".
 *
 * @author Klaus Schoeffmann
 */

class URIParameters {

private:
  /** full URI, also containing specified params */
  char* fullURI;
  /** Semi: MovieId */
  char* movieId;
  /** Semi: UserId */
  uint userId;
  /** Semi: ProfileId */
  uint profileId;
  /** Semi: SessionId (on migration) */
  uint sessionId;
  /** Semi: Action (migration/copy/continue) */
  uint action;
  /** Semi: Section to use for continue */
  uint section;
  /** Semi: Seconds to subtract on migrate/copy/continue */
  int subtractSecs;

  /** Range (<fromVal>-<toVal>) to use for Play-Request */
  char* range;
  /** Host to use for all requests */
  char* host;
  /** Port to use for all requests */
  uint port;

public:
  /** default constructor */
  URIParameters();
  /** default destructor */
  ~URIParameters();
  /** set full-URI */
  void setFullURI(char *fullURI);
  /** set value of MovieId parameter */
  void setMovieId(char* movieId);
  /** set value of UserId parameter */
  void setUserId(uint userId);
  /** set value of ProfileId parameter */
  void setProfileId(uint profileId);
  /** set value of Range parameter */
  void setRange(char* range);
  /** set value of Host parameter */
  void setHost(char* host);
  /** set value of Port parameter */
  void setPort(uint port);
  /** set value of SessionId parameter */
  void setSessionId(uint sessionId);
  /** set value of Action parameter */
  void setAction(uint action);
  /** set value of Section parameter */
  void setSection(uint section);
  /** set value of subtractSecs parameter */
  void setSubtractSecs(int subtractSecs);

  /** @return full URI, also containing parameters */
  char *getFullURI();
  /** @return value of MovieId parameter */
  char* getMovieId();
  /** @return value of UserId parameter */
  uint getUserId();
  /** @return value of ProfileId parameter */
  uint getProfileId();
  /** @return value of Range parameter */
  char* getRange();
  /** @return value of Host parameter */
  char* getHost();
  /** @return value of Port parameter */
  uint getPort();
  /** @return value of SessionId parameter */
  uint getSessionId();
  /** @return value of Action parameter */
  uint getAction();
  /** @return value of Section parameter */
  uint getSection();
  /** @return value of subtractSecs parameter */
  int getSubtractSecs();
};

#endif
