/***********************************************************************
*                                                                      *
* ViTooKi                                                              *
*                                                                      *
* title: Logon.cpp                                                     *
*                                                                      *
*                                                                      *
*                                                                      *
* ITEC institute of the University of Klagenfurt (Austria)             *
* http://www.itec.uni-klu.ac.at                                        *
*                                                                      *
*                                                                      *
* For more information visit the ViTooKi homepage:                     *
* http://ViTooKi.sourceforge.net                                       *
* vitooki-user@lists.sourceforge.net                                   *
* vitooki-devel@lists.sourceforge.net                                  *
*                                                                      *
* This file is part of ViTooKi, a free video toolkit.                  *
* ViTooKi is free software; you can redistribute it and/or             *
* modify it under the terms of the GNU General Public License          *
* as published by the Free Software Foundation; either version 2       *
* of the License, or (at your option) any later version.               *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
* MA  02111-1307, USA.                                                 *
*                                                                      *
***********************************************************************/

/***********************************************************************
*                                                                      *
*  REVISION HISTORY:                                                   *
*                                                                      *
*                                                                      *
*                                                                      *
***********************************************************************/

/***********************************************************************
* Video Session Migration System, 2004                                 *
* Author: Klaus Schoeffmann                                            *
************************************************************************/


#include "logon.hpp"


/***********************************************************************/
int main (void)
{
  //read all users into memory
  UserList *ul = new UserList();
  ul->readUsersFromFile(Globals::getConfigFilePath());

  //read all profiles from file
  //(and add each profile to the corresponding user)
  ProfileList *profl = new ProfileList();
  profl->readProfilesFromFile(Globals::getConfigFilePath(), ul);

  //read form data
  string formdata;
  cin >> formdata;
  CGIParser *cgiparser = new CGIParser(formdata);
  uint userId = atoi(cgiparser->getFieldValue("userid").c_str());
  uint profileId = atoi(cgiparser->getFieldValue("proid").c_str());
  char *pass = semifunc::newStrCpy((char*)cgiparser->getFieldValue("password").c_str());
  uint linktype = atoi(cgiparser->getFieldValue("linktype").c_str());
  uint pageSize = atoi(cgiparser->getFieldValue("pagesize").c_str());
  if (pageSize == 0) pageSize = 1;
  bool showActiveSessions = true, showClosedSessions = false;
  if (cgiparser->getFieldValue("showAS") != "on") showActiveSessions = false;
  if (cgiparser->getFieldValue("showCS") == "on") showClosedSessions = true;
  bool fakeRtspServer = false;
  if (cgiparser->getFieldValue("proxysupported") != "on") fakeRtspServer = true;

  //output HTML header and the SeMi menu
  cout << html::getHTMLHeader();
  cout << html::getMenu("logon.cgi", pageSize);
  html::createPageSizeForm(cgiparser->createOpenForm("frmPageSize", "logon.cgi", "post", ""), pageSize);

  //username, profile and password must be specified!
  if (userId == 0 || profileId == 0 || pass == NULL || strcmp(pass,"") == 0)
  {
    cout << html::error("You must specify username, profile and password!");
    exit(1);
  }

  //check if user exists
  User *u = ul->getUserById(userId);
  if (u == NULL)
  {
    cout << html::error("User was not found!") << endl;
    exit(1);
  }

  //check password
  if (strcmp(u->getPassword(), pass) != 0)
  {
    cout << html::error("Wrong password!") << endl;
    exit(1);
  }
  else
  {

    //    cout << "<table width=\"100%\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\">" << endl;
    //    cout << "<tr><td width=\"50%\" align=\"center\">";


    //************generate selectbox for profile...************
    cout << "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\">" << endl;
    cout << "<tr><td valign=\"top\">Profile:</td>" << endl;
    cout << "<td>";
    cout << cgiparser->createOpenForm("frmProfSel", "logon.cgi", "post", "proid");
    cout << "<select size=\"1\" id=\"proid\""
    << " name=\"proid\" class=\"darkblueborder\""
    << " onChange=\"document.forms['frmProfSel'].submit();\">" << endl;
    //output all profiles for chosen user
    vector<Profile*> profiles = ul->getUserById(userId)->getProfiles();
    vector<Profile*>::const_iterator piter = profiles.begin();
    for (;piter != profiles.end(); piter++)
    {
      Profile *profile = *piter;
      cout << "<option ";
      if (profile->getProfileId() == profileId) cout << " selected ";
      cout << " value=\"" << profile->getProfileId() << "\">";
      cout << profile->getProfileName();
      cout << "</option>" << endl;
    }
    cout << "</select></form></td></tr>";


    //************generate selectbox for link-type selection************
    cout << "<tr><td valign=\"top\">Links as:</td>" << endl;
    cout << "<td>";
    cout << cgiparser->createOpenForm("frmLinktypeSel", "logon.cgi", "post", "linktype");
    cout << "<select name=\"linktype\" size=\"1\""
    << " onChange=\"document.forms['frmLinktypeSel'].submit();\">" << endl;

    cout << "<option ";
    if (linktype == LINK_RTSP) cout << " selected ";
    cout << "value=\"" << LINK_RTSP << "\">RTSP-URIs</option>" << endl;

    cout << "<option ";
    if (linktype == LINK_QTE) cout << " selected ";
    cout << "value=\"" << LINK_QTE << "\">Embedded QT</option>" << endl;

    cout << "<option ";
    if (linktype == LINK_SDP) cout << " selected ";
    cout << "value=\"" << LINK_SDP << "\">SDP files</option>" << endl;

    cout << "</select>" << endl;
    cout << "</form>" << endl;

    cout << "</td></tr></table>";



    //************read movies************
    MovieList *ml = new MovieList(profl->getProfileById(profileId));
    cout << "<!-- reading movies from " << Globals::getMoviesFileName() << " -->" << endl;
    ml->readMoviesFromFile( Globals::getMoviesFileName() );


    //************read active sessions************
    SessionList *sl = new SessionList(Globals::getProxyComHost(),
                                      Globals::getProxyComPort(), profl->getProfileById(profileId));
    sl->retrieveSessions();


    //************generate a list of active session and all videos************
    cout << "<table width=\"100%\" border=\"0\" cellpadding=\"4\" cellspacing=\"0\">" << endl;
    cout << "<tr><td colspan=\"2\" align=\"center\"" << ml->getDescription() << "</td></tr>" << endl;

    //list all active sessions
    sl->writeHTMLRepresentation(cgiparser, showActiveSessions, showClosedSessions, fakeRtspServer);
    //list all available movies
    cout << ml->getHTMLRepresentation(Globals::getRelativeImagesFilePath(), cgiparser, fakeRtspServer);

    cout << "</table>\n";
  }

  cout << html::getHTMLFooter(cgiparser->createRefreshForm("logon.cgi", "post"));

  delete cgiparser;
  delete ul;
  delete profl;
  return 0;
}
