/***********************************************************************
*                                                                      *
* ViTooKi                                                              *
*                                                                      *
* title: Readprofiles.cpp                                              *
*                                                                      *
*                                                                      *
*                                                                      *
* ITEC institute of the University of Klagenfurt (Austria)             *
* http://www.itec.uni-klu.ac.at                                        *
*                                                                      *
*                                                                      *
* For more information visit the ViTooKi homepage:                     *
* http://ViTooKi.sourceforge.net                                       *
* vitooki-user@lists.sourceforge.net                                   *
* vitooki-devel@lists.sourceforge.net                                  *
*                                                                      *
* This file is part of ViTooKi, a free video toolkit.                  *
* ViTooKi is free software; you can redistribute it and/or             *
* modify it under the terms of the GNU General Public License          *
* as published by the Free Software Foundation; either version 2       *
* of the License, or (at your option) any later version.               *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
* MA  02111-1307, USA.                                                 *
*                                                                      *
***********************************************************************/

/***********************************************************************
*                                                                      *
*  REVISION HISTORY:                                                   *
*                                                                      *
*                                                                      *
*                                                                      *
***********************************************************************/

/***********************************************************************
* Video Session Migration System, 2004                                 *
* Author: Klaus Schoeffmann                                            *
************************************************************************/


#include "readprofiles.hpp"

/***********************************************************************/
void createProfileSelection(UserList *ul, uint userId)
{
  cout << "<select tabindex=\"1\" size=\"3\" style=\"width:80%\" id=\"proid\" name=\"proid\" class=\"darkblueborder\" onChange=\"showProfileInfo(this.value);\">";
  //output all profiles for chosen user
  User* user = ul->getUserById(userId);
  vector<Profile*> profiles = user->getProfiles();
  vector<Profile*>::const_iterator piter = profiles.begin();
  for (;piter != profiles.end(); piter++)
  {
    Profile *profile = *piter;
    cout << "<option value=\"" << profile->getProfileId() << "\">";
    cout << profile->getProfileName();
    cout << "</option>" << endl;
  }
  cout << "</select>";
}

/***********************************************************************/
void createJavaScriptCode(UserList *ul, uint userId)
{
  cout << "<script language=\"javascript\">" << endl;

  //first create arrays containing profile information
  User* user = ul->getUserById(userId);
  vector<Profile*> profiles = user->getProfiles();
  if (profiles.size() > 0)
  {
    ostringstream str;
    cout << "profileInfo = new Array(" << profiles.size() << ");" << endl;
    vector<Profile*>::const_iterator piter = profiles.begin();
    for (uint i=0;piter != profiles.end(); piter++, i++)
    {
      Profile *profile = *piter;

      if (piter != profiles.begin()) str << ", ";

      cout << "profileInfo[" << i << "] = ";
      cout << "\"Width=" << profile->getWidth();
      cout << " Height=" << profile->getHeight();
      cout << " Color=" << (profile->isColorCapable() == true ? "yes" : "no");
      cout << " Bpp=" << profile->getBitsPerPixel();
      cout << " Bufferms=" << profile->getBufferingDelayms();
      cout << " NetkBps=" << profile->getNetkBps();
      cout << " refreshRate=" << profile->getRefreshRate() << "\";" << endl;
    }
  }

  cout << "document.getElementById('proid').focus();" << endl;

  cout << "function disableLogon() {" << endl;
  cout << "  document.getElementById('password').disabled = true;" << endl;
  cout << "  document.getElementById('submit').disabled = true;" << endl;
  cout << "}" << endl;

  cout << "function showProfileInfo(nr) {" << endl;
  cout << "  document.getElementById('info').value = profileInfo[nr-1];" << endl;
  cout << "  document.getElementById('password').focus();" << endl;
  cout << "}" << endl;

  cout << "function disableInputs() {" << endl;
  cout << "  document.getElementById('proid').disabled = true;" << endl;
  cout << "  disableLogon();" << endl;
  cout << "}" << endl;

  //if no profiles found, disable password-field and logon-button
  if (profiles.size() == 0)
  {
    cout << "disableLogon();" << endl;
  }
  cout << "</script>" << endl;
}

/***********************************************************************/
int main (void)
{

  //read all users
  UserList *ul = new UserList();
  ul->readUsersFromFile(Globals::getConfigFilePath());

  //read all profiles
  //(and add each profile to corresponding user)
  ProfileList *profl = new ProfileList();
  profl->readProfilesFromFile(Globals::getConfigFilePath(), ul);

  //read form data (userid)
  string formdata;
  cin >> formdata;
  CGIParser cgiparser (formdata);
  uint userId = atoi(cgiparser.getFieldValue("userid").c_str());
  uint pageSize = atoi(cgiparser.getFieldValue("pagesize").c_str());
  if (pageSize == 0) pageSize = 1;


  cout << html::getHTMLHeader();
  cout << html::getMenu("readprofiles.cgi", pageSize);
  html::createPageSizeForm(cgiparser.createOpenForm("frmPageSize", "readprofiles.cgi", "post", ""), pageSize);

  //userId must be specified!
  if (userId == 0)
  {
    cout << html::error("You must first select an user!") << endl;
    exit(1);
  }

  //create form and selection-box for profiles
  cout << "<form action=\"" << Globals::getCGIPath() << "logon.cgi\" method=\"post\">";
  cout << "<input type=\"hidden\" id=\"userid\" name=\"userid\" value=\"" << userId << "\">";
  cout << "<input type=\"hidden\" id=\"linktype\" name=\"linktype\" value=\"" << LINK_RTSP << "\">";
  cout << "<input type=\"hidden\" id=\"subtractSecs\" name=\"subtractSecs\" value=\"2\">";
  cout << "<input type=\"hidden\" id=\"pagesize\" name=\"pagesize\" value=\"" << pageSize << "\">";
  cout << "<input type=\"hidden\" id=\"showAS\" name=\"showAS\" value=\"on\">";
  cout << "<input type=\"hidden\" id=\"showMS\" name=\"showMS\" value=\"on\">";
  if (cgiparser.getFieldValue("proxysupported") == "on") {
    cout << "<input type=\"hidden\" id=\"proxysupported\" name=\"proxysupported\" value=\"on\">";
  } else {
    cout << "<input type=\"hidden\" id=\"sphost\" name=\"sphost\" value=\"" << cgiparser.getFieldValue("sphost") << "\">";
    cout << "<input type=\"hidden\" id=\"spport\" name=\"spport\" value=\"" << cgiparser.getFieldValue("spport") << "\">";
  }
  cout << "<table width=\"100%\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\">";
  cout << "<tr><td colspan=\"2\" align=\"left\">Hello <b>" << ul->getUserById(userId)->getUsername() << "</b><br><br></td></tr>" << endl;
  cout << "<tr><td colspan=\"2\" align=\"left\">Please choose a profile:</td></tr>" << endl;

  //create selectbox for profiles
  cout << "<tr><td colspan=\"2\" align=\"left\">" << endl;
  createProfileSelection(ul, userId);
  cout << "</td></tr>";

  //create input field for password
  cout << "<tr><td colspan=\"2\" align=\"left\">Input password:</td></tr>" << endl;
  cout << "<tr><td align=\"left\" width=\"50%\"><input tabindex=\"2\" "
  << "type=\"password\" id=\"password\" name=\"password\" class=\"darkblueborder\" "
  << "size=\"12\" style=\"width:80%\" maxlength=\"30\"></td>" << endl;
  cout << "<td align=\"left\" width=\"50%\"><input tabindex=\"5\" "
  << "type=\"submit\" id=\"submit\" name=\"submit\" value=\"Logon\">" << endl;
  cout << "</td></tr>" << endl;

  //create hidden layer used to display profile info
  //cout << "<br><div id=\"info\" class=\"profileinfo\"><br><br></div>" << endl;
  cout << "<tr><td colspan=\"2\"><br>Profile info:<br>"
  << "<textarea cols=20 rows=4 id=\"info\" class=\"profileinfo\"></textarea></td></tr>" << endl;

  cout << "</table></form>";

  //create scripting part, used to display profile info and for input-protection
  createJavaScriptCode(ul, userId);

  cout << html::getHTMLFooter(cgiparser.createRefreshForm("readprofiles.cgi", "post"));

  delete ul;
  delete profl;

  return 0;
}
