 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: MovieListRequest.hpp                                          *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 * Video Session Migration System, 2004                                 *
 * Author: Klaus Schoeffmann                                            *
 ************************************************************************/


#ifndef __MovieListRequest_hpp__
#define __MovieListRequest_hpp__

#ifdef WIN32
#include <winsock2.h>
#else
#include <unistd.h>
#include <time.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <unistd.h>
#include <arpa/inet.h>
#endif

#include <string>
#include <iostream>
#include <sstream>
#include <fstream>
#include <list>

#include "SemiGlobals.hpp"

#define MAX_RECBUFFER_SIZE 262144

using namespace std;

/**
 * This class is used to request to movielist from the
 * mediaserver (by HTTP) and write it to a file. Also the
 * descriptive images, which are specified in this file will be
 * requested by HTTP and stored in a file.
 *
 * @author Klaus Schoeffmann
 */

class MovieListRequest
{
  private:
  /** Create an active socket to specified host and port. */
  int createSocket(const char *host, int port);
  /** Send a HTTP request to the specified server and wait for the
      response.
      @return on success, a pointer to the start of the received content.
      otherwise NULL */
  char* makeHttpRequest(const char *hostname, int sock, string msg, char *recbuffer, uint &contentLength);
  /** Add the specified string to the specified list,
      but only if that string isn't already there. */
  void addIfNotExists(list<char*> &imgList, char *str);
  /** Write the characters from buffer with length len,
      to specified file. */
  void writeToFile(const char *filename, char *buffer, uint len);

  public:
    /** default constructor */
    MovieListRequest();
    /** default destructor */
    ~MovieListRequest();
    /** Requests the movielist from a specified server. Also looks for images
        in this file and requests them too. */
    bool requestMovieList(const char *hostname, int port = 80);
};



#endif
