/***********************************************************************
*                                                                      *
* ViTooKi                                                              *
*                                                                      *
* title: Readusers.cpp                                                 *
*                                                                      *
*                                                                      *
*                                                                      *
* ITEC institute of the University of Klagenfurt (Austria)             *
* http://www.itec.uni-klu.ac.at                                        *
*                                                                      *
*                                                                      *
* For more information visit the ViTooKi homepage:                     *
* http://ViTooKi.sourceforge.net                                       *
* vitooki-user@lists.sourceforge.net                                   *
* vitooki-devel@lists.sourceforge.net                                  *
*                                                                      *
* This file is part of ViTooKi, a free video toolkit.                  *
* ViTooKi is free software; you can redistribute it and/or             *
* modify it under the terms of the GNU General Public License          *
* as published by the Free Software Foundation; either version 2       *
* of the License, or (at your option) any later version.               *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
* MA  02111-1307, USA.                                                 *
*                                                                      *
***********************************************************************/

/***********************************************************************
*                                                                      *
*  REVISION HISTORY:                                                   *
*                                                                      *
*                                                                      *
*                                                                      *
***********************************************************************/

/***********************************************************************
* Video Session Migration System, 2004                                 *
* Author: Klaus Schoeffmann                                            *
************************************************************************/


#include "readusers.hpp"

/***********************************************************************/
void createUserSelection()
{
  //first read all users
  UserList *ul = new UserList();
  ul->readUsersFromFile(Globals::getConfigFilePath());

  //then create selection box
  cout << "<select style=\"width:100%\" size=\"3\" name=\"userid\" ";
  cout << " onChange=\"doLogon();\">" << endl;
  //output all usernames
  vector<User*> users = ul->getUsers();
  vector<User*>::const_iterator iter = users.begin();
  for (;  iter != users.end(); iter++)
  {
    User *user = *iter;
    cout << "<option value=\"" << user->getUserId() << "\">";
    cout << user->getUsername() << "</option>" << endl;
  }
  cout << "</select>"   << endl;

  delete ul;
}

/***********************************************************************/
int main (int argc, char *argv[])
{

  bool foundMovieList = false;

  string formdata;
  cin >> formdata;
  CGIParser cgiparser (formdata);
  uint pageSize = atoi(cgiparser.getFieldValue("pagesize").c_str());
  if (pageSize == 0) pageSize = 1;

  //create header and menu
  cout << html::getHTMLHeader();
  cout << html::getMenu("readusers.cgi", pageSize);
  html::createPageSizeForm(cgiparser.createOpenForm("frmPageSize", "readusers.cgi", "post", ""), pageSize);

  Globals::sdebug << "\nreadusers: request movielist";

  //first try to request movielist from mediaserver
  MovieListRequest *mlr = new MovieListRequest();
  if (mlr->requestMovieList(Globals::getMediaserverHttpHost(),Globals::getMediaserverHttpPort()))
  {
    foundMovieList = true;
  }
  else
  {
    //cannot request movielist
    //try to use an older version
    ifstream testoldfile (Globals::getMoviesFileName());
    if (testoldfile)
    {
      cout << html::warning("movielist-request failed!<br>Using cached version!") << endl;
      foundMovieList = true;
    }
    else
    {
      cout << html::error("Cannot request movielist from mediaserver!") << endl;
      exit(1);
    }
  }
  delete mlr;

  //if movielist were received/found, output all users
  if (foundMovieList)
  {

    //create form and selection-box
    cout << "<form name=\"logon\" action=\"" << Globals::getCGIPath() << "readprofiles.cgi\" method=\"post\">";
    cout << "<input type=\"hidden\" id=\"pagesize\" name=\"pagesize\" value=\"" << pageSize << "\">";

    cout << "<table width=\"100%\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\">";
    cout << "<tr><td width=\"50%\" align=\"left\">Choose user:</td>" << endl;
    cout << "<td with=\"50%\"  align=\"center\" rowspan=\"2\">" << endl;
    cout << "<input name=\"logon\" id=\"logon\" type=\"submit\" value=\"Go\"></td></tr>" << endl;
    cout << "<tr><td align=\"left\">" << endl;

    createUserSelection();

    cout << "</td></tr></table>" << endl;
    cout << "<br><input type=\"checkbox\" id=\"proxysupported\" name=\"proxysupported\""
         << " checked onChange=\"checkProxyFields();\" onClick=\"checkProxyFields();\">"
         << "Client supports RTSP proxy" << endl;
    cout << "<span id=\"proxyconf\">" << endl;
    cout << "<br>SeMiProxy<br>Address:<input type=\"text\" id=\"sphost\" name=\"sphost\""
         << " value=\"" << semifunc::getHostname() << "\" size=\"15\">" << endl;
    cout << "<br>Port:<input type=\"text\" id=\"spport\" name=\"spport\""
         << "value=\"6554\" size=\"6\">" << endl;
    cout << "</span>" << endl;
    cout << "</form>" << endl;

    cout << "<script language=\"javascript\">" << endl;
    cout << "function doLogon() {" << endl;
    cout << "  document.forms['logon'].submit();" << endl;
    cout << "  document.getElementById('logon').disabled = true;" << endl;
    cout << "}" << endl;
    cout << "function checkProxyFields() {" << endl;
    cout << "  if (document.getElementById('proxysupported').checked) {" << endl;
    cout << "    document.getElementById('proxyconf').style.display = \"none\";" << endl;
    cout << " } else {" << endl;
    cout << "    document.getElementById('proxyconf').style.display = \"inline\";" << endl;
    cout << " }" << endl;
    cout << "}" << endl;
    cout << "document.getElementById('proxyconf').style.display = \"none\";" << endl;
    cout << "</script>" << endl;

  }


  cout << html::getHTMLFooter(cgiparser.createRefreshForm("readusers.cgi", "post"));

  return 0;
}
