/***********************************************************************
*                                                                      *
* ViTooKi                                                              *
*                                                                      *
* title: CGIParser.cpp                                                 *
*                                                                      *
*                                                                      *
*                                                                      *
* ITEC institute of the University of Klagenfurt (Austria)             *
* http://www.itec.uni-klu.ac.at                                        *
*                                                                      *
*                                                                      *
* For more information visit the ViTooKi homepage:                     *
* http://ViTooKi.sourceforge.net                                       *
* vitooki-user@lists.sourceforge.net                                   *
* vitooki-devel@lists.sourceforge.net                                  *
*                                                                      *
* This file is part of ViTooKi, a free video toolkit.                  *
* ViTooKi is free software; you can redistribute it and/or             *
* modify it under the terms of the GNU General Public License          *
* as published by the Free Software Foundation; either version 2       *
* of the License, or (at your option) any later version.               *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
* MA  02111-1307, USA.                                                 *
*                                                                      *
***********************************************************************/

/***********************************************************************
*                                                                      *
*  REVISION HISTORY:                                                   *
*                                                                      *
*                                                                      *
*                                                                      *
***********************************************************************/

/***********************************************************************
* Video Session Migration System, 2004                                 *
* Author: Klaus Schoeffmann                                            *
************************************************************************/


#include "CGIParser.hpp"
#ifdef ENABLE_QT
#include <qstring.h>
#endif

#ifdef WIN32
#include <algorithm>
typedef unsigned int uint;
#endif

/***********************************************************************/
CGIParser::CGIParser(string data)
{
  parseData(data, '&', '=');
}

/***********************************************************************/
CGIParser::~CGIParser()
{
  formFields.clear();
}

/***********************************************************************/
void CGIParser::parseData(string data, const char sep1, const char sep2)
{
  list<string> fields = splitData(data, sep1);
  list<string>::const_iterator iter = fields.begin();
  for (;iter != fields.end(); iter++)
  {
    list<string> fieldData = splitData(*iter, sep2);
    if (fieldData.size() == 2)
    {
      //success
      formFields[fieldData.front()] = fieldData.back();
    }
    else
    {
      cerr << "\nCGIParser::parseData(): cannot read field: " << *iter << "(size=" << fieldData.size() << ")";
    }
  }
}

/***********************************************************************/
list<string> CGIParser::splitData(string data, const char sep)
{
  //list of elements
  list<string> elems;

#ifndef WIN32  //back_inserters cause C2039

  string::iterator pos1 = data.begin();
  //find returns an iterator to the end, if nothing is found
  string::iterator pos2 = find(pos1, data.end(), sep);
  while (pos2 != data.end())
  {
    string field;
    copy(pos1, pos2, back_inserter(field));
    elems.push_back(field);
    pos1 = pos2+1;
    pos2 = find(pos1, data.end(), sep);
  }

  string field;
  copy(pos1, pos2, back_inserter(field));
  elems.push_back(field);

#endif

  return elems;
}

/***********************************************************************/
int CGIParser::hex2dec(char c)
{
  if (c >= 'A' && c <= 'F')
  {
    c -= 'A';
    c += 10;
  }
  else if (c >= 'a' && c <= 'f')
  {
    c -= 'a';
    c += 10;
  }
  else
  {
    c -= '0';
  }

  return (int)c;
}

#ifndef WIN32
#define STRING_PUSH_BACK(str,ch) str.push_back(ch)
#else
//#define STRING_PUSH_BACK(str,ch) str.append(string(ch))
#define STRING_PUSH_BACK(str,ch) str+=ch;
#endif
/***********************************************************************/
string CGIParser::decode(string data)
{
  string decoded;

  string::iterator iter = data.begin();
  for (; iter != data.end(); iter++)
  {
    if (*iter == '+')
    { //space
      //decoded.push_back(' ');
      STRING_PUSH_BACK(decoded,' ');
    }
    else if (*iter == '%')
    { //special character starts
      uint special = hex2dec(*++iter) * 16 + hex2dec(*++iter);
#ifdef ENABLE_QT
      //convert from unicode to latin1
      QChar qchar (special);
      //decoded.push_back(qchar.latin1());
	  STRING_PUSH_BACK(decoded,qchar.latin1());
#else
			//FIXME: this _will_ result in wrong coding!!!
      //decoded.push_back(special);
	  STRING_PUSH_BACK(decoded,special);
#endif
    }
    else
    {
      //decoded.push_back(*iter);
      STRING_PUSH_BACK(decoded,*iter);
    }
  }

  return decoded;
}

/***********************************************************************/
string CGIParser::getFieldValue(string name)
{
  string decoded = decode(formFields[name]);
  return decoded;
}

/***********************************************************************/
string CGIParser::createRefreshForm(string action, string method)
{
  ostringstream str;

  str << createOpenForm("refreshForm", action, method, "");
  str << "</form>" << endl;

  return str.str();
}

/***********************************************************************/
string CGIParser::createOpenForm(string name, string action, string method, string ignoreFields)
{
  ostringstream str;

  str << "\n<form name=\"" << name
  << "\" action=\"" << action
  << "\" method=\"" << method << "\">" << endl;

  map<string, string>::iterator iter = formFields.begin();
  for (; iter != formFields.end(); iter++)
  {
    //output every received parameters (but no the submit parameter)
    if (iter->first != "submit")
    {

      //do only output field, if name is not in ignore-string
      if (ignoreFields.find(iter->first) == string::npos)
      {
        str << "<input type=\"hidden\" name=\""
        << iter->first << "\" value=\""
        << iter->second << "\">" << endl;
      }

    }
  }

  return str.str();
}



