 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: CGIParser.hpp                                                 *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 * Video Session Migration System, 2004                                 *
 * Author: Klaus Schoeffmann                                            *
 ************************************************************************/


#ifndef __CGIParser_hpp__
#define __CGIParser_hpp__

#include <string>
#include <iostream>
#include <sstream>
#include <list>
#include <map>
#include <vector>


using namespace std;

/**
 * This class is used to parse 'Post'-Parameters, submitted
 * by a HTML-form.
 *
 * @author Klaus Schoeffmann
 */

class CGIParser
{
  private:
    /** list of parsed fields with corresponding values */
    map<string, string> formFields;

    /** Parses CGI data received from the browser. The fields are
        splitted and added to the local list of fields. */
    void parseData(string data, const char sep1, const char sep2);
    /** Splits the passed string into serval parts,
        seperated through passed seperator. */
    list<string> splitData(string data, const char sep);
    /** @return the decimal value of a hexadecimal character. */
    int hex2dec(char c);
    /** Decodes the HTML FORM data string.
        The URI-code of special characters like +,&,= or öäü etc. will be
        back-transformed. Special characters (stored as hex-code
        with % as prefix) will also be back-transformed.
        @return decoded string */
    string decode(string data);

  public:
    /** default constructor */
    CGIParser(string data);
    /** default destructor */
    ~CGIParser();
    /** @return the value of a specified field,
        which was parsed before through the method parseData(). */
    string getFieldValue(string name);
    /** @return code of a HTML form, containing all passed form fields
        with corresponding values, which can be used for refreshing
        the current page. */
    string createRefreshForm(string action, string method);
    /** @return code of an open HTML form (no end tag), containing all
        passed form fields except the parameters specified in ignoreParams
    */
    string createOpenForm(string name, string action, string method, string ignoreFields);
};
#endif
