/***********************************************************************
*                                                                      *
* ViTooKi                                                              *
*                                                                      *
* title: Movie.cpp                                                     *
*                                                                      *
*                                                                      *
*                                                                      *
* ITEC institute of the University of Klagenfurt (Austria)             *
* http://www.itec.uni-klu.ac.at                                        *
*                                                                      *
*                                                                      *
* For more information visit the ViTooKi homepage:                     *
* http://ViTooKi.sourceforge.net                                       *
* vitooki-user@lists.sourceforge.net                                   *
* vitooki-devel@lists.sourceforge.net                                  *
*                                                                      *
* This file is part of ViTooKi, a free video toolkit.                  *
* ViTooKi is free software; you can redistribute it and/or             *
* modify it under the terms of the GNU General Public License          *
* as published by the Free Software Foundation; either version 2       *
* of the License, or (at your option) any later version.               *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
* MA  02111-1307, USA.                                                 *
*                                                                      *
***********************************************************************/

/***********************************************************************
*                                                                      *
*  REVISION HISTORY:                                                   *
*                                                                      *
*                                                                      *
*                                                                      *
***********************************************************************/

/***********************************************************************
* Video Session Migration System, 2004                                 *
* Author: Klaus Schoeffmann                                            *
************************************************************************/


#include "Movie.hpp"
#include "SemiGlobals.hpp"


/***********************************************************************/
Movie::Movie()
{}

/***********************************************************************/
Movie::~Movie()
{}

/***********************************************************************/
Movie::Movie(string name, string id, string description,
             string image, string url)
{
  this->name = name;
  this->id = id;
  this->description = description;
  this->image = image;
  this->url = url;
}

/***********************************************************************/
string Movie::getName()
{
  return name;
}

/***********************************************************************/
string Movie::getId()
{
  return id;
}

/***********************************************************************/
string Movie::getDescription()
{
  return description;
}

/***********************************************************************/
string Movie::getImage()
{
  return image;
}

/***********************************************************************/
string Movie::getUrl(uint userId, uint profileId)
{
  if (userId == 0 && profileId == 0)
  {
    return getUrl();
  }
  else
  {
//    ostringstream str;
    unsigned int hostStart = url.find("rtsp://") + 7;
    unsigned int hostEnd = url.find("/", hostStart);
//    str << url.substr(0, hostEnd);

    ostringstream strU;
    strU << "/uid=" << userId;
    url.insert(hostEnd, strU.str()); hostEnd += strU.str().length();
    ostringstream strP;
    strP << "/proid=" << profileId;
    url.insert(hostEnd, strP.str()); hostEnd += strP.str().length();
    ostringstream strI;
    strI << "/id=" << id;
    url.insert(hostEnd, strI.str()); hostEnd += strI.str().length();

    return url;

//    str << strU.str() << strP.str() << strI.str();

//    str << url.substr(hostStart);

/*
    //=
    ostringstream str;
    str << url << "?uid=" << userId << "&proid=" << profileId << "&id=" << id ;

    return str.str();
 */
  }
}

/***********************************************************************/
string Movie::getUrl()
{
  return this->url;
}

/***********************************************************************/
void Movie::setName(string n)
{
  this->name = n;
}

/***********************************************************************/
void Movie::setId(string id)
{
  this->id = id;
}

/***********************************************************************/
void Movie::setDescription(string desc)
{
  description = desc;
}

/***********************************************************************/
void Movie::setImage(string img)
{
  image = img;
}

/***********************************************************************/
void Movie::setUrl(string u)
{
  url.assign(u);
}


/***********************************************************************/
string Movie::getHTMLRepresentation(char *imgPath, uint userId, uint profileId, uint linktype,
                                    uint qtewidth, uint qteheight, bool fakeRtspServer, char *sphost, char *spport)
{
  ostringstream buf;

  //output image (must have a fixed size of 94 x 140 pixel)
  buf << "<tr>" << endl
  << "<td valign=\"top\">" << endl
  << "<img width=\"94\" height=\"140\" src=\"" << imgPath << this->getImage() << "\">"
  << "</td>" << endl;

  //output name of movie
  buf << "<td valign=\"top\">"
  << "<a class=\"moviehdr\" name=\"" << this->getId() << "\">" << this->getName() << "</a><br>" << endl;

  //output description
  buf << "<font class=\"moviedesc\">" << this->getDescription() << "</font><br><br>" << endl;


  string url = this->getUrl(userId, profileId);
  unsigned int hostStart = url.find("rtsp://") + 7;
  unsigned int hostEnd = url.find("/", hostStart);
  unsigned int filePathStart = hostEnd;
  unsigned int portStart = url.find(":", hostStart) + 1;
  if (portStart != string::npos && portStart < hostEnd) hostEnd = portStart-1;
  if (fakeRtspServer) {
      if (portStart != string::npos) {
        unsigned int slashPos = filePathStart;
        string portNr = url.substr(portStart, slashPos-portStart);
        url.insert(filePathStart,"/port=");
        url.insert(filePathStart+strlen("/port="),portNr);
        url.erase(portStart, slashPos-portStart);
        url.insert(portStart, spport);
      }

      //change host
      string hostAdr = url.substr(hostStart, hostEnd-hostStart);
      if (hostAdr.compare(sphost) != 0) {
        url.erase(hostStart, hostEnd-hostStart);
        url.insert(hostStart, sphost);
        filePathStart = url.find("/", hostStart); //may have changed
        url.insert(filePathStart, "/host=");
        url.insert(filePathStart+strlen("/host="), hostAdr);
      }
  }



  //output link
  if (linktype == LINK_RTSP)
  {
    //***common RTSP***
    buf << "<a href=\"" << url << "\">Start</a>";
  }
  else if (linktype == LINK_SDP)
  {
    //***for SDP files***
    ostringstream tmpName;
    //add user_id and profile_id (otherwise files from different users would be overwritten)
    tmpName << "movie_" << this->getId() << "_u" << userId << "_p" << profileId;
    buf << "<a href=\""
    << semifunc::createSDPFile(tmpName.str(),
                               url,
                               Globals::getRelativeSDPFilePath(),
                               Globals::getAbsoluteSDPFilePath())
    << "\">Start</a>" << endl;
  }
  else if (linktype == LINK_QTE)
  {
    //***for QuickTime embedded***
    buf << "<form name=\"frmStart" << this->getId() << "\""
    << " action=\"embedded.cgi\" method=\"post\" target=\"frmStart" << this->getId() << "\">" << endl
    << "<input type=\"hidden\" name=\"uri\" value=\"" << url << "\">" << endl
    << "<input type=\"hidden\" name=\"width\" value=\"" << qtewidth << "\">" << endl
    << "<input type=\"hidden\" name=\"height\" value=\"" << qteheight << "\">" << endl
    << "</form>" << endl
    << "<a href=\"javascript:submitFormInWindow('frmStart" << this->getId() << "');\">Start</a>";
  }

  buf << "</td></tr>" << endl;

  return buf.str();
}
