 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: Movie.hpp                                                     *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 * Video Session Migration System, 2004                                 *
 * Author: Klaus Schoeffmann                                            *
 ************************************************************************/


#ifndef __Movie_hpp__
#define __Movie_hpp__

#include <string>
#include <iostream>
#include <fstream>
#include <sstream>

using namespace std;

#ifdef WIN32
typedef unsigned int uint;
#endif

/**
 * This class is used to store a movie.
 *
 * @author Klaus Schoeffmann
 */

class Movie
{
  private:
    /** name of the movie */
    string name;
    /** internal id of the movie */
    string id;
    /** description of the movie */
    string description;
    /** image file-path of the movie */
    string image;
    /** Rtsp-URI of the movie */
    string url;

  public:
    /** default constructor */
    Movie ();
    /** default destructor */
    ~Movie ();
    /** constructor */
    Movie(string name, string id, string description, string image, string url);
    /** @return name of movie */
    string getName();
    /** @return internal id of movie */
    string getId();
    /** @return description of movie */
    string getDescription();
    /** @return image file-path of movie */
    string getImage();
    /** @return Rtsp-URI of movie with added user- and profile-Id (as parameters) */
    string getUrl(uint userId, uint profileId);
    /** @return Rtsp-URI of movie without any (semi-added) parameters */
    string getUrl();
    /** Get the HTML representation of a movie.
        This representation contains the name, the image, the description and the Rtsp-URI.
        @param userId user-id which should be added to the Rtsp-Link
        @param profileId profile-id which should be added to the Rtsp-Link
        @param linktype identifies, which link type should be used
        @param qtewidth when QTE is used as link type: width for the new window
        @param qteheight when QTE is used as link type: for the new window
        @param fakeRtspServer flag which must be true if client does not support an RTSP Proxy
        @param sphost address of semiproxy (if fakeRtspServer == true)
        @param spport port number of semiproxy (if fakeRtspServer == true)
    */
    string getHTMLRepresentation(char *imgPath, uint userId = 0, uint profileId = 0,
                uint linktype = 1, uint qtewidth = 0, uint qteheight = 0, bool fakeRtspServer = false,
                char *sphost = NULL, char *spport = NULL);
    /** set name of movie */
    void setName(string n);
    /** set internal id of movie */
    void setId(string id);
    /** set description of movie */
    void setDescription(string desc);
    /** set image file-path of movie */
    void setImage(string img);
    /** set Rtsp-URI of movie */
    void setUrl(string u);
};

#endif
