/***********************************************************************
*                                                                      *
* ViTooKi                                                              *
*                                                                      *
* title: MovieList.cpp                                                 *
*                                                                      *
*                                                                      *
*                                                                      *
* ITEC institute of the University of Klagenfurt (Austria)             *
* http://www.itec.uni-klu.ac.at                                        *
*                                                                      *
*                                                                      *
* For more information visit the ViTooKi homepage:                     *
* http://ViTooKi.sourceforge.net                                       *
* vitooki-user@lists.sourceforge.net                                   *
* vitooki-devel@lists.sourceforge.net                                  *
*                                                                      *
* This file is part of ViTooKi, a free video toolkit.                  *
* ViTooKi is free software; you can redistribute it and/or             *
* modify it under the terms of the GNU General Public License          *
* as published by the Free Software Foundation; either version 2       *
* of the License, or (at your option) any later version.               *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
* MA  02111-1307, USA.                                                 *
*                                                                      *
***********************************************************************/

/***********************************************************************
*                                                                      *
*  REVISION HISTORY:                                                   *
*                                                                      *
*                                                                      *
*                                                                      *
***********************************************************************/

/***********************************************************************
* Video Session Migration System, 2004                                 *
* Author: Klaus Schoeffmann                                            *
************************************************************************/


#include "MovieList.hpp"


using namespace std;

vector<Movie*> MovieList::list;


/*******************************************************/
MovieList::MovieList(Profile *profile)
{
  this->profile = profile;
}

/*******************************************************/
MovieList::~MovieList()
{
  list.clear();
}

/*******************************************************/
void MovieList::addMovie(Movie* m)
{
  list.push_back(m);
}

/*******************************************************/
Movie* MovieList::getMovieById(string movieId)
{
  vector<Movie*>::const_iterator iter = list.begin();
  for (;iter != list.end(); iter++)
  {
    Movie *movie = *iter;
    if (movieId == movie->getId()) return movie;
  }
  //not found
  return NULL;
}

/*******************************************************/
string MovieList::getDescription()
{
  return desc;
}


/*******************************************************/
string MovieList::getHTMLRepresentation(char *imgPath, CGIParser *cgiparser, bool fakeRtspServer)
{
  string data;
  uint userId = 0;
  uint linktype = 0;
  char *sphost = NULL;
  char *spport = NULL;
  if (cgiparser != NULL)
  {
    userId = atoi(cgiparser->getFieldValue("userid").c_str());
    linktype = atoi(cgiparser->getFieldValue("linktype").c_str());
    sphost = semifunc::newStrCpy((char*)cgiparser->getFieldValue("sphost").c_str());
    spport = semifunc::newStrCpy((char*)cgiparser->getFieldValue("spport").c_str());
  }

  //create html file of all movies
  vector<Movie*>::const_iterator iter = list.begin();
  for (;iter != list.end(); iter++)
  {
    Movie *movie = *iter;
    if (cgiparser != NULL)
    {
      data += movie->getHTMLRepresentation(imgPath,
                                           userId, profile->getProfileId(),
                                           linktype, profile->getWidth(), profile->getHeight(),
                                           fakeRtspServer, sphost, spport);
    }
    else
    {
      data += movie->getHTMLRepresentation(imgPath, 0, 0, 1, 0, 0, fakeRtspServer);
    }
  }

  return data;
}

/*******************************************************/
void MovieList::readMoviesFromFile(const char *filename)
{
  ifstream fp (filename, ios::in);
  if (! fp.is_open())
  {
    cerr << "Error opening file '" << filename << "'";
    exit (1);
  }

  //read file into vector, tag by tag
  vector<string> vec;
  char buffer[4095];

  uint pos = 0;
  char c;
  while(true)
  {
    if (fp.get(c))
    {
      //look if a tag starts
      if (c == '<' && pos != 0)
      {
        //push back old buffer
        buffer[pos] = '\0';
        pos = 0;
        vec.push_back(string(buffer));
        buffer[pos++] = c;
      }
      else if (c == '>')
      {
        buffer[pos++] = c;
        buffer[pos] = '\0';
        pos = 0;
        vec.push_back(string(buffer));
      }
      else
      {
        buffer[pos++] = c;
      }
    }
    else
    {
      break;
    }
  }
  fp.close();


  //parse vector
  string name, description, img, url;
  string lastId, lastCondId;
  Movie *movie;
  uint i, statement=0;
  for (i=0; i<vec.size(); i++)
  {

    //on selection-tags, store ID
    if (vec[i].find("<Selection") != string::npos)
    {
      unsigned int loc1 = vec[i].find("select_id=\"") + 11;
      unsigned int loc2 = vec[i].find("\"", loc1);
      lastId = vec[i].substr(loc1, loc2-loc1);
    }
    else if (vec[i].find("<Statement") != string::npos)
    {
      //read whole content until end of statement-tag
      string stmt = readStatement(i, vec);
      //cout << "\nread statement: " << stmt;

      //assumption: 1st statement = name of movie, 2nd = desc, and so on...
      if (statement++ == 0)
      {
        //cout << "\nassigning to desc = " << stmt;
        this->desc.assign(stmt);
      }
      else if ((statement-1) % 2 == 1)
      {
        //cout << "\nassigning to name = " << stmt;
        //name of movie
        name.assign(stmt);
      }
      else if ((statement-1) % 2 == 0)
      {
        //cout << "\nassigning to description = " << stmt;
        //description of movie
        description.assign(stmt);
      }
    }
    else if (vec[i].find("<Resource mimeType=\"image/jpeg\"") != string::npos)
    {
        //image of movie
        unsigned int loc1 = vec[i].find("ref=\"") + 5;
        unsigned int loc2 = vec[i].find("\"", loc1);
        img = vec[i].substr(loc1, loc2-loc1);

        //create movie and store in list
        movie = new Movie(name, lastId, description, img, "url");
        list.push_back(movie);
    }
    //on conditions: store id for later use on resource-tag
    else if (vec[i].find("<Condition") != string::npos)
    {
      unsigned int loc1 = vec[i].find("require=\"") + 9;
      unsigned int loc2 = vec[i].find("\"", loc1);
      lastCondId = vec[i].substr(loc1, loc2-loc1);
    }
    //on resource: add this url to corresponding movie
    else if (vec[i].find("<Resource") != string::npos)
    {
      if (vec[i].find("mimeType=\"video/mpeg\"") != string::npos)
      {
        unsigned int loc1 = vec[i].find("ref=\"") + 5;
        unsigned int loc2 = vec[i].find("\"", loc1);
        url = vec[i].substr(loc1, loc2-loc1);
        uint j;
        for (j=0; j<list.size(); j++)
        {
          Movie *m = list[j];
          if (m->getId().compare(lastCondId) == 0)
          {
            m->setUrl(url);
          }
        }
      }
    }
  }
}

/*******************************************************/
string MovieList::readStatement(uint& i, vector<string> vec)
{
  if (vec[i].find("/>") == string::npos)
  {

    //read content of statement (everything until "</Statement>"
    string str;
    i++;
    while (vec[i].find("</Statement>") == string::npos)
    {
      str.append(vec[i++]);
    }
    i--;
    return str;

  }
  else
  {

    return vec[i]; //return "<Statement .... />"

  }
}
