 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: MovieList.hpp                                                 *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 * Video Session Migration System, 2004                                 *
 * Author: Klaus Schoeffmann                                            *
 ************************************************************************/


#ifndef __MovieList_hpp__
#define __MovieList_hpp__

#include <string>
#include <iostream>
#include <fstream>
#include <vector>
#include <assert.h>
#include "Movie.hpp"
#include "Profile.hpp"
#include "CGIParser.hpp"
#include "SemiGlobals.hpp"

using namespace std;

/**
 * This class is used read movies into memory.
 *
 * @author Klaus Schoeffmann
 */

class MovieList
{
  private:
    /** vector containing all movies */
    static vector<Movie*> list;
    /** description of this movie-list */
    string desc;
    /** reference to the profile to use for HTML-representation.
        In this representation, the URI-Parameters of the Rtsp-URI
        will depend on this profile */
    Profile *profile;

    /** Read the whole content of a "<Statement>" tag
        and return it as a string. */
    string readStatement(uint& i, vector<string> vec);

  public:
    /** default constructor */
    MovieList(Profile *profile = NULL);
    /** default destructor */
    ~MovieList();
    /** Add a movie to the list. */
    void addMovie(Movie* m);
    /** @return reference to the movie in the list, which has the
        specified id */
    static Movie* getMovieById(string movieId);
    /** Get the description of the movie-list */
    string getDescription();
    /** Get an HTML-representation of the movie-list.
        @param imgPath specifies the path where referenced images are stored
    */
    string getHTMLRepresentation(char *imgPath, CGIParser *cgiparser = NULL, bool fakeRtspServer = false);
    /** Read movies from specified xml-file.
        This function is a very small implementation of
        a MPEG21-DigitalItem-Parser. The read movies
        will be stored in the vector 'list'. */
    void readMoviesFromFile(const char *filename);
};
#endif
