/***********************************************************************
*                                                                      *
* ViTooKi                                                              *
*                                                                      *
* title: Profile.cpp  (S H A R E D)                                    *
*                                                                      *
*                                                                      *
*                                                                      *
* ITEC institute of the University of Klagenfurt (Austria)             *
* http://www.itec.uni-klu.ac.at                                        *
*                                                                      *
*                                                                      *
* For more information visit the ViTooKi homepage:                     *
* http://ViTooKi.sourceforge.net                                       *
* vitooki-user@lists.sourceforge.net                                   *
* vitooki-devel@lists.sourceforge.net                                  *
*                                                                      *
* This file is part of ViTooKi, a free video toolkit.                  *
* ViTooKi is free software; you can redistribute it and/or             *
* modify it under the terms of the GNU General Public License          *
* as published by the Free Software Foundation; either version 2       *
* of the License, or (at your option) any later version.               *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
* MA  02111-1307, USA.                                                 *
*                                                                      *
***********************************************************************/

/***********************************************************************
*                                                                      *
*  REVISION HISTORY:                                                   *
*                                                                      *
*                                                                      *
*                                                                      *
***********************************************************************/

/***********************************************************************
* Video Session Migration System, 2004                                 *
* Author: Klaus Schoeffmann                                            *
************************************************************************/


#include "Profile.hpp"

/***********************************************************************/
Profile::Profile (uint profileId, uint uidCount, uint *userIds, char profilename[],
                  uint width, uint height, uint bitsPerPixel, uint netkBps,
                  uint bufferingDelay, bool color, uint refreshRate)
{
  if (strlen(profilename) > PROFILENAME_MAXLEN)
  {
    //cerr << "Profile::Profile(): Cannot create Profile: too long name\r\n";
    exit(1);
  }
  else
  {
    this->profileId = profileId;
    strcpy(this->profilename, profilename);
    this->uidCount = uidCount;
    this->userIds = new uint[uidCount];
    memcpy(this->userIds, userIds, sizeof(uint) * uidCount);
    this->width = width;
    this->height = height;
    this->bitsPerPixel = bitsPerPixel;
    this->bufferingDelay = bufferingDelay;
    this->color = color;
    this->refreshRate = refreshRate;
    this->netkBps = netkBps;
  }
}

/***********************************************************************/
Profile::~Profile()
{
  //cout << " p" << profileId;
  delete userIds;
}

/***********************************************************************/
uint Profile::getProfileId()
{
  return profileId;
}

/***********************************************************************/
uint* Profile::getUserIds()
{
  return userIds;
}

/***********************************************************************/
char* Profile::getProfileName()
{
  return profilename;
}

/***********************************************************************/
uint Profile::getWidth()
{
  return width;
}

/***********************************************************************/
uint Profile::getHeight()
{
  return height;
}

/***********************************************************************/
uint Profile::getBitsPerPixel()
{
  return bitsPerPixel;
}

/***********************************************************************/
uint Profile::getNetkBps()
{
  return netkBps;
}

/***********************************************************************/
uint Profile::getBufferingDelayms()
{
  return bufferingDelay;
}

/***********************************************************************/
bool Profile::isColorCapable()
{
  return color;
}

/***********************************************************************/
uint Profile::getRefreshRate()
{
  return refreshRate;
}

