/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: Profile.hpp  (S H A R E D)                                    *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 * Video Session Migration System, 2004                                 *
 * Author: Klaus Schoeffmann                                            *
 ************************************************************************/


#ifndef __Profile_hpp__
#define __Profile_hpp__

#include <string>

#ifdef WIN32
typedef unsigned int uint;
#endif


/** maximum length for the name of a profile */
#define PROFILENAME_MAXLEN  50


/**
 * This class is used to store user-profiles.
 *
 * @author Klaus Schoeffmann
 */

class Profile
{
private:
  /** id of the profile */
  uint profileId;
  // name of profile */
  char profilename[PROFILENAME_MAXLEN];
  /** number of users, to which this profile can be applied */
  uint uidCount;
  /** list of users, to which this profile can be applied */
  uint *userIds;
  /** preferred width of movie */
  uint width;
  /** preferred height of movie */
  uint height;
  /** preferred bits per pixel */
  uint bitsPerPixel;
  /** preferred network kilo bytes per second */
  uint netkBps;
  /** buffering-delay (prefetch-buffer) to subtract, when using this profile */
  uint bufferingDelay;
  /** should movie be colored or only grayscaled */
  bool color;
  /** preferred refresh-rate */
  uint refreshRate;

public:

  /** default constructor */
  Profile(uint profileId, uint uidCount, uint *userIds,
          char profilename[], uint width, uint height,
          uint bitsPerPixel, uint netkBps,
          uint bufferingDelay, bool color, uint refreshRate);
  /** default destructor */
  ~Profile();
  /** @return id of profile*/
  uint getProfileId();
  /** @return list of users, to which this profile can be applied */
  uint *getUserIds();
  /** @return name of profile */
  char *getProfileName();
  /** @return preferred width of movie */
  uint getWidth();
  /** @return preferred height of movie */
  uint getHeight();
  /** @return preferred bits per pixel */
  uint getBitsPerPixel();
  /** @return preferred network kilo bytes per second */
  uint getNetkBps();
  /** @return buffering-delay to subtract, when using this profile */
  uint getBufferingDelayms();
  /** @return flag, if movie should be colored or only grayscaled */
  bool isColorCapable();
  /** @return preferred refresh-rate */
  uint getRefreshRate();

};
#endif
