/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: ProfileList.cpp  (S H A R E D)                                *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
*                                                                      *
*  REVISION HISTORY:                                                   *
*                                                                      *
*                                                                      *
*                                                                      *
***********************************************************************/

/***********************************************************************
* Video Session Migration System, 2004                                 *
* Author: Klaus Schoeffmann                                            *
************************************************************************/


#include "ProfileList.hpp"

using namespace std;

/***********************************************************************/
ProfileList::ProfileList()
{}

/***********************************************************************/
ProfileList::~ProfileList()
{
  Globals::sdebug << "\nProfileList::~ProfileList cleanup...";
  vector<Profile*>::iterator iter = list.begin();
  while (iter != list.end())
  {
    Profile *profl = *iter;
    delete profl;
    iter++;
  }
  list.clear();
  Globals::sdebug << "\nProfileList::~ProfileList done!";
}

/***********************************************************************/
void ProfileList::addProfile(Profile* profile)
{
  list.push_back(profile);
}

/***********************************************************************/
void ProfileList::readProfilesFromFile(const char *filename, UserList *ul)
{
  char line[READLINE_MAXLEN], *buffer, *tmp;
  char *profileName;
  uint *userIds, uidCount;
  uint profileId, width, height, bitsPerPixel, netkBps, bufferingDelay, refreshRate;
  bool color, colorRead;
  int profilesCount = 0;

  //open file of profiles
  ifstream fp (filename, ios::in);
  if (! fp.is_open())
  {
    cerr << "\nProfileList::readProfilesFromFile(): Error opening file '" << filename << "'";
    exit (1);
  }

  //reset variables used for reading
  profileName = NULL;
  profileId = width = height = bufferingDelay = 0;
  bitsPerPixel = refreshRate = 0;
  colorRead = false;

  //go to start of profiles
  while (! fp.eof() )
  {
    fp.getline(line,READLINE_MAXLEN);
    if (line != NULL && strncmp(line, "############PROFILES start############", strlen("############PROFILES end############")) == 0) break;
  }

  //read line by line until end of file
  //Attention: maximum line characters is set in Globals.hpp
  while (! fp.eof() )
  {
    fp.getline (line, READLINE_MAXLEN);
    if (line != NULL)
    {
      if (strncmp(line, "############PROFILES end############", strlen("############PROFILES end############")) == 0) break;

      if (line[0] == '#')
      {
        //ignore comments
      }
      //look for profilename
      else if (line[0] == '[' && line[strlen(line) - 1] == ']')
      {
        profileName = new char[strlen(line) - 1];
        strncpy(profileName, line + 1, strlen(line) - 2);
        profileName[strlen(line) - 2] = '\0';
      }
      //look for profile-ID
      else if ((buffer = strstr(line, "profileId=")) != NULL)
      {
        profileId = atoi(buffer + 10);
      }
      //look for userids
      else if ((buffer = strstr(line, "uids=")) != NULL)
      {
        //exract user-IDs
        uidCount=0;
        userIds = (uint*) malloc(sizeof(uint));
        userIds[uidCount++] = atoi(buffer + 5);

        tmp = buffer + 5;
        while ((tmp = strstr(tmp, ",")) != NULL)
        {
          userIds = (uint*) realloc(userIds, sizeof(uint) * (uidCount + 1));
          userIds[uidCount++] = atoi(tmp+1);
          tmp = tmp + 1; //this crashes, if behind a komma there is nothing!
        }

      }
      //look for width
      else if ((buffer = strstr(line, "width=")) != NULL)
      {
        width = atoi(buffer + 6);
      }
      //look for height
      else if ((buffer = strstr(line, "height=")) != NULL)
      {
        height = atoi(buffer + 7);
      }
      //look for color
      else if ((buffer = strstr(line, "color=")) != NULL)
      {
        if (strncmp(buffer + 6, "yes", 3) == 0)
        {
          color = true;
          colorRead = true;
        }
        else if (strncmp(buffer + 6, "no", 5) == 0)
        {
          color = false;
          colorRead = true;
        }
        else
        {
          cerr << "\nProfileList::readProfilesFromFile(): Wrong parameter value for color: " << line << endl;
          exit(1);
        }
      }
      //look for bits-per-pixel
      else if ((buffer = strstr(line, "bpp=")) != NULL)
      {
        bitsPerPixel = atoi(buffer + 4);
      }
      //look for network bandwidth
      else if ((buffer =strstr(line, "netkBps=")) != NULL)
      {
        netkBps = atoi(buffer + 8);
      }
      //look for bufferingDelay
      else if ((buffer = strstr(line, "bufferingDelay=")) != NULL)
      {
        bufferingDelay = atoi(buffer + 15);
      }
      //look for refreshRate
      else if ((buffer = strstr(line, "refreshRate=")) != NULL)
      {
        refreshRate = atoi(buffer + 12);
      }
      else
      {
        //Unrecognized parameter ignored
      }
    }

    //if all necessary information was read, add user to list
    //and reset variables used for reading
    if (profileName != NULL && width != 0 && height != 0
        && bitsPerPixel != 0 && netkBps != 0
        && bufferingDelay != 0 && colorRead == true && refreshRate != 0)
    {

      Profile *profile = new Profile(profileId, uidCount, userIds, profileName, width, height, bitsPerPixel,
                                     netkBps, bufferingDelay, color, refreshRate);
      addProfile(profile);
      profilesCount++;

      //add reference for this profile to corresponding users
      for (uint i = 0; i < uidCount; i++)
      {
        ul->getUserById(userIds[i])->addProfile(profile);
      }

      profileName = NULL;
      profileId = width = height = bufferingDelay = 0;
      bitsPerPixel = netkBps = refreshRate = 0;
      colorRead = false;
    }

  }

  fp.close();

  Globals::sdebug << "\nProfileList::readProfilesFromFile(): read profiles successfully finished with " << profilesCount << "entries!";
}

/***********************************************************************/
Profile* ProfileList::getProfileById(const uint profileId)
{
  Globals::sdebug << "\nProfileList::getProfileById(): searching for profile " << profileId;
  uint i;
  for (i=0; i < list.size(); i++)
  {
    Profile *profile = list[i];
    if (profile->getProfileId() == profileId) return profile;
  }

  Globals::sdebug << "\nProfileList::getProfileById(): NOT FOUND!!!";
  return NULL;
}
