/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: SSession.cpp  (S H A R E D)                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
*                                                                      *
*  REVISION HISTORY:                                                   *
*                                                                      *
*                                                                      *
*                                                                      *
***********************************************************************/

/***********************************************************************
* Video Session Migration System, 2004                                 *
* Author: Klaus Schoeffmann                                            *
************************************************************************/


#include "SSession.hpp"

/***********************************************************************/
SSession::SSession(uint RtspSessionId, char *URL, char *clientAdr, char *SemiMovieId,
                   uint OriginRtspSessionId, ulong prevDuration,
                   uint bufferingDelayms, TimeMeasurement *timemsmnt)
{

  Globals::sdebug << "\nSSession::SSession(): SSession-Konstruktor1 ";

  assert(URL != NULL && clientAdr != NULL && timemsmnt != NULL);

  this->RtspSessionId = RtspSessionId;
  this->URL = new string(URL);
  this->clientAdr = new string(clientAdr);
  this->SemiMovieId = NULL;
  if (SemiMovieId != NULL) this->SemiMovieId = new string(SemiMovieId);
  this->OriginRtspSessionId = OriginRtspSessionId;
  this->prevDuration = prevDuration;
  this->estMsecsElapsed = 0;
  this->bufferingDelayms = bufferingDelayms;
  this->timemsmnt = timemsmnt;
  this->tsCount = 0;
  this->State = STATE_INIT;

  //get current timestamp
  struct timeval currTime;
  gettimeofday(&currTime, NULL);
  this->tsCreation = currTime.tv_sec;
  Globals::sdebug << "\nsession-creation:tv_sec = " << currTime.tv_sec;
  Globals::sdebug << "\n=>" << semifunc::newStrCpy(semifunc::formatTimeSecs(tsCreation));
  this->tsClosed = 0;

  Globals::sdebug << "\nSSession::SSession(): created session:";
  Globals::sdebug << "\n\t" << getRtspSessionId();
  Globals::sdebug << "\n\turl: " << URL;
  Globals::sdebug << "\n\tclient: " << clientAdr;
  Globals::sdebug << "\n\tmovid: " << SemiMovieId;
  Globals::sdebug << "\n\torigSessId: " << this->OriginRtspSessionId;
  Globals::sdebug << "\n\tdur: " << this->prevDuration;
  Globals::sdebug << "\n\testMsecs: " << this->estMsecsElapsed;
  Globals::sdebug << "\n\tbufferdelay: " << this->bufferingDelayms;
  Globals::sdebug << "\n\tstate: " << this->State;
  Globals::sdebug << "\n\tcreation: " << this->tsCreation;
  Globals::sdebug << "\n\tclosed: " << this->tsClosed;
}

/***********************************************************************/
SSession::SSession(uint RtspSessionId, char *URL, char *clientAdr, char *SemiMovieId, uint OriginRtspSessionId,
                   ulong estMsecsElapsed, uint State, ulong duration, ulong prevDuration, uint tsCount,
                   ulong tsCreation, ulong tsClosed)
{
  Globals::sdebug << "\nSSession::SSession2(): SSession-Konstruktor2 ";


  this->RtspSessionId = RtspSessionId;
  this->URL = new string(URL);
  this->clientAdr = new string(clientAdr);
  this->SemiMovieId = new string(SemiMovieId);
  this->OriginRtspSessionId = OriginRtspSessionId;
  this->estMsecsElapsed = estMsecsElapsed;
  this->bufferingDelayms = 0;
  this->timemsmnt = NULL;
  this->duration = duration;
  this->prevDuration = prevDuration;
  this->tsCount = tsCount;
  this->State = State;
  this->tsCreation = tsCreation;
  this->tsClosed = tsClosed;

  Globals::sdebug << "\nSSession::SSession(): created session:";
  Globals::sdebug << "\n\t" << getRtspSessionId();
  Globals::sdebug << "\n\t" << URL;
  Globals::sdebug << "\n\t" << clientAdr;
  Globals::sdebug << "\n\t" << SemiMovieId;
  Globals::sdebug << "\n\t" << this->OriginRtspSessionId;
  Globals::sdebug << "\n\t" << this->estMsecsElapsed;
  Globals::sdebug << "\n\t" << this->bufferingDelayms;
  Globals::sdebug << "\n\t" << this->duration;
  Globals::sdebug << "\n\t" << this->prevDuration;
  Globals::sdebug << "\n\t" << this->tsCount;
  Globals::sdebug << "\n\t" << this->State;
  Globals::sdebug << "\n\t" << this->tsCreation;
  Globals::sdebug << "\n\t" << this->tsClosed;
}

/***********************************************************************/
SSession::~SSession()
{
  Globals::sdebug << "\nSSession::~SSession(): session " << getRtspSessionId() << " deleted";
  if (URL != NULL) delete URL;
  if (clientAdr != NULL) delete clientAdr;
  if (SemiMovieId != NULL) delete SemiMovieId;
}

/***********************************************************************/
uint SSession::getRtspSessionId()
{
  return this->RtspSessionId;
}

/***********************************************************************/
void SSession::setRtspSessionId(uint SessionId)
{
  this->RtspSessionId = SessionId;
}

/***********************************************************************/
char* SSession::getUrl()
{
  return (char *)URL->c_str();
}

/***********************************************************************/
char* SSession::getClientAdr()
{
  return (char *)clientAdr->c_str();
}

/***********************************************************************/
char* SSession::getSemiMovieId()
{
  return (SemiMovieId == NULL) ? (char*)(new string(""))->c_str() : (char *)SemiMovieId->c_str();
}

/***********************************************************************/
uint SSession::getOriginRtspSessionId()
{
  return OriginRtspSessionId;
}

/***********************************************************************/
uint SSession::getBufferingDelay()
{
  return bufferingDelayms;
}

/***********************************************************************/
void SSession::setActive()
{
  State = STATE_RUNNING;
}

/***********************************************************************/
void SSession::setInactive()
{
  State = STATE_PAUSED;
}

/***********************************************************************/
ulong SSession::getElapsedMsecs(uint sectionNr, ulong serverPlayout, ulong clientPlayout)
{
  if (timemsmnt != NULL)
  {
    //called from semiproxy
    return timemsmnt->getElapsedTime(sectionNr, serverPlayout, clientPlayout);
  }
  else
  {
    //called from semiserver
    return estMsecsElapsed;
  }
}

/***********************************************************************/
ulong SSession::getDuration()
{
  if (timemsmnt != NULL)
  {
    //called from semiproxy
    return timemsmnt->getDuration();
  }
  else
  {
    //called from semiserver
    return duration;
  }
}

/***********************************************************************/
ulong SSession::getPreviousSessionDuration()
{
  if (timemsmnt != NULL)
  {
    //called from semiproxy
    return timemsmnt->getPreviousSessionDuration();
  }
  else
  {
    //called from semiserver
    return prevDuration;
  }
}

/***********************************************************************/
uint SSession::getTimeSectionCount()
{
  if (timemsmnt != NULL)
  {
    //called from semiproxy
    return timemsmnt->getTimeSectionCount();
  }
  else
  {
    //called from semiserver
    return tsCount;
  }
}

/***********************************************************************/
uint SSession::getState()
{
  return State;
}

/***********************************************************************/
void SSession::setClosed()
{
  State = STATE_CLOSED;

  //get current timestamp
  struct timeval currTime;
  gettimeofday(&currTime, NULL);
  //calculate to msecs
  this->tsClosed = currTime.tv_sec;
}

/***********************************************************************/
bool SSession::isClosed()
{
  return (State == STATE_CLOSED);
}

/***********************************************************************/
bool SSession::isMaster()
{
  return (OriginRtspSessionId == 0);
}

/***********************************************************************/
ulong SSession::getTimestampCreated()
{
  return tsCreation;
}

/***********************************************************************/
ulong SSession::getTimestampClosed()
{
  return tsClosed;
}
