/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: SSession.hpp  (S H A R E D)                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 * Video Session Migration System, 2004                                 *
 * Author: Klaus Schoeffmann                                            *
 ************************************************************************/


#ifndef __SSession_hpp__
#define __SSession_hpp__

#include <string>
#include <iostream>
#ifndef WIN32
  #include <sys/time.h>
#else
  #include <time.h>
#endif
#include "SemiGlobals.hpp"
#include "TimeMeasurement.hpp"

#include "global.hpp"

using namespace std;

/**
 * This class contains all information about
 * an active streaming session. It is used from
 * SessionMgmt.cpp (on proxy-side) and from
 * SessionList.cpp (on server-side).
 *
 * @author Klaus Schoeffmann
 */

class SSession
{

private:
  /** RtspSessionId of this session */
  uint RtspSessionId;
  /** full URI of this session*/
  string *URL;
  /** client address of this session*/
  string *clientAdr;
  /** semi-movie-Id of this session*/
  string *SemiMovieId;
  /** Rtsp Session-Id of origin session (on migration)*/
  uint OriginRtspSessionId;
  /** duration of previous session in milliseconds (on migration)*/
  ulong prevDuration;
  /** estimated elapsed time of this session in milliseconds*/
  ulong estMsecsElapsed;
  /** duration of this session in milliseconds*/
  ulong duration;
  /** buffering delay of used profile(device)*/
  uint bufferingDelayms;
  /** state of session*/
  uint State;
  /** time-measurement class (only used on proxy-use)*/
  TimeMeasurement *timemsmnt;
  /** current number of time-sections*/
  uint tsCount;
  /** timestamp of creation in seconds */
  ulong tsCreation;
  /** timestamp of termination in seconds */
  ulong tsClosed;


public:
  /** Constructor called from SemiProxy */
  SSession(uint RtspSessionId, char *URL, char *clientAdr,
            char *SemiMovieId, uint OriginRtspSessionId, ulong prevDuration,
            uint bufferingDelayms, TimeMeasurement *timemsmnt);
  /** Constructor called from SemiServer */
  SSession(uint RtspSessionId, char *URL, char *clientAdr,
            char *SemiMovieId, uint OriginRtspSessionId, ulong estMsecsElapsed,
            uint State, ulong duration, ulong prevDuration, uint tsCount,
            ulong tsCreation, ulong tsClosed);
  /** default destructor */
  ~SSession();
  /** @return Rtsp Session-Id of this session. */
  uint getRtspSessionId();
  /** set Rtsp Session-Id of this session */
  void setRtspSessionId(uint SessionId);
  /** @return URI of this session. */
  char *getUrl();
  /** @return client-address of this session. */
  char *getClientAdr();
  /** @return semi-movie-id of the session. */
  char *getSemiMovieId();
  /** @return Rtsp Session-ID of the origin session (on migration). */
  uint getOriginRtspSessionId();
  /** @return duration of the origin session (on migration) */
  ulong getPreviousSessionDuration();
  /** @return estimated elapsed time of the session. */
  ulong getElapsedMsecs(uint sectionNr = 0, ulong serverPlayout = 0, ulong clientPlayout = 0);
  /** @return duration of this session */
  ulong getDuration();
  /** @return number of time-sections in this session */
  uint getTimeSectionCount();
  /** @return buffering delay of the device(profile) used by this session. */
  uint getBufferingDelay();
  /** @return current state of session. */
  uint getState();
  /** @return timestamp of creation in secs */
  ulong getTimestampCreated();
  /** @return timestamp of termination in secs */
  ulong getTimestampClosed();
  /** Set current session state to STATE_RUNNING. */
  void setActive();
  /** Set current session state to STATE_PAUSED */
  void setInactive();
  /** mark this session as 'closed by the client' */
  void setClosed();
  /** @return true if this session has been closed by the client, otherwise false */
  bool isClosed();
  /** @return true if this session is a master session (a potential source of a migration)*/
  bool isMaster();
};

#endif
