/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: SemiGlobals.hpp  (S H A R E D)                                *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 * Video Session Migration System, 2004                                 *
 * Author: Klaus Schoeffmann                                            *
 ************************************************************************/


#ifndef __SemiGlobals_hpp__
#define __SemiGlobals_hpp__

#include <string>
#include <sstream>
#include <fstream>
#include <iostream>
#include <vector>
#include <map>
#include <math.h>

#ifdef WIN32
#include <winsock2.h>
#else
#include <sys/time.h>
#include <unistd.h>
#include <time.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <unistd.h>
#include <arpa/inet.h>
#endif


using namespace std;


/** default port used to connect to rtsp-servers */
#define DEFAULT_RTSPSERVER_PORT 554
/** maximum line-length used in .users/.profiles/.semirc file */
#define READLINE_MAXLEN 160
/** maximum file-length (used for config-file) */
#define CONFIGFILE_MAXLEN  5000
/** maximum length for the name of a user */
#define USERNAME_MAXLEN 30
/** maximum length for the password of a user */
#define PASSWORD_MAXLEN 30

/** last received rtsp-message: none */
#define RTSP_NONE     0
/** last received rtsp-message: OPTIONS */
#define RTSP_OPTIONS  1
/** last received rtsp-message: DESCRIBE */
#define RTSP_DESCRIBE 2
/** last received rtsp-message: SETUP */
#define RTSP_SETUP    3
/** last received rtsp-message: PLAY */
#define RTSP_PLAY     4
/** last received rtsp-message: PAUSE */
#define RTSP_PAUSE    5
/** last received rtsp-message: TEARDOWN */
#define RTSP_TEARDOWN 6
/** last received rtsp-message: GET_PARAMETER */
#define RTSP_GETPARAMETER 7
/** last received rtsp-message: SET_PARAMETER */
#define RTSP_SETPARAMETER 8

/** session state: initialized */
#define STATE_INIT    0
/** session state: running/active */
#define STATE_RUNNING 1
/** session state: paused */
#define STATE_PAUSED  2
/** session state: closed */
#define STATE_CLOSED  3

/** URI-Parameters action-value: migrate */
#define ACTION_MIGRATE   1
/** URI-Parameters action-value: copy */
#define ACTION_COPY      2
/** URI-Parameters action-value: continue */
#define ACTION_CONTINUE  3

/** link type used on semiserver: Rtsp-URI */
#define LINK_RTSP  1
/** link type used on semiserver: Quicktime embedded */
#define LINK_QTE   2
/** link type used on semiserver: SDP file */
#define LINK_SDP   3

/** time to sleep (in microseconds) used by the proxy after a loop run */
#define PROXY_SLEEP_USECS  20;

typedef unsigned int uint;
typedef unsigned long ulong;



/**
 * This namespace covers some useful functions, which are used
 * in several other classes.
 *
 * @author Klaus Schoeffmann
 */

namespace semifunc {
  /** Replace the occurence of an old-value with the new-value.
      @param str string to scan
      @param val old-value
      @param newval new value
      @param all controls, if all occurrences of substring1 will be replaced. */
  inline int replaceStr(std::string &str, char* val, char* newval, bool all = false)
  {
    //TODO: use replace() function !!!
    uint pos = str.find(val);
    uint lengthDiff = 0;
    while (pos != std::string::npos) {
      str.replace(pos, strlen(val), newval);
      lengthDiff += (strlen(val)-strlen(newval));
      pos = str.find(val);
      if (!all) break;
    }
    return lengthDiff;
  };

  /** Replace the characters starting at position-1 until position-2.
      @param str string to scan
      @param from position-1
      @param to position-2
      @param newval string to insert
  */
  inline void replaceStr(std::string &str, uint from, uint to, char *newval)
  {
    //TODO: use replace() function !!!
    str.replace(from, to-from, newval, strlen(newval));
  }

  /** Convert any object to a string */
  template <class Ch>
    inline std::string toString(Ch arg)
  {
    std::ostringstream os;
    os << arg;
    return os.str();
  }

  /** Create a copy of the specified string with a specified length of characters */
  inline char* newStrCpy(char *from, uint length = 0)
  {
    if (length == 0) length = strlen(from);

    char *tmp = new char[length+1];
    strncpy(tmp, from, length);
    tmp[length] = 0;
    return tmp;
  }

  /** Get name of current host */
  inline char* getHostname()
  {
    //find out IP-Address of this host
    char *address = new char[255];
    char name[255];
    if (gethostname(name, sizeof(name)) == 0) {
      hostent *host = gethostbyname(name);
      if (host == NULL) {
        cerr << "semifunc::getHostname(): binding specific IP-addr, though no DNS-hostname resolution possible...\n";
        strcpy(address,name);
      } else
        strcpy(address,inet_ntoa(*(reinterpret_cast<in_addr*>(host->h_addr))) );
    } else {
      cerr << "semifunc::getHostname(): cannot get hostname!!!\n";
      return NULL;
    }

    return address;
  }


  /** Convert time in milliseconds to the time-format: HH:MM:SS */
  inline char* formatTime(ulong msecs)
  {
    ostringstream timeFormat;
    msecs /= 1000;

    /*double secsD60 = ceil((double)msecs/(double)60);
    double secsD3600 = ceil((double)msecs/3600);*/
    uint secsD60 = msecs/60;
    uint secsD3600 = msecs/3600;


    uint hour = (uint)secsD3600 % 24;
    uint minute = (uint)secsD60 % 60;
    uint second = msecs % 60;

    if (hour < 10)timeFormat << "0" << hour << ":";
    else            timeFormat << hour << ":";
    if (minute < 10)timeFormat << "0" << minute << ":";
    else            timeFormat << minute << ":";
    if (second < 10)timeFormat << "0" << second;
    else            timeFormat << second;

    return (char*)(timeFormat.str().c_str());
  }

  /** Convert time in seconds to the time-format: HH:MM:SS */
  inline char* formatTimeSecs(ulong secs)
  {
    ostringstream timeFormat;

    double secsD60 = ceil((double)secs/(double)60);
    double secsD3600 = ceil((double)secs/3600);

    uint hour = (uint)secsD3600 % 24;
    uint minute = (uint)secsD60 % 60;
    uint second = secs % 60;

    if (hour < 10)timeFormat << "0" << hour << ":";
    else            timeFormat << hour << ":";
    if (minute < 10)timeFormat << "0" << minute << ":";
    else            timeFormat << minute << ":";
    if (second < 10)timeFormat << "0" << second;
    else            timeFormat << second;

    return (char*)(timeFormat.str().c_str());
  }

  /** Convert milliseconds to a string */
  inline char* convertToString(ulong msecs)
  {
    float fmsecs = (float)msecs/1000;
    char *smsecs = new char[15];
    sprintf(smsecs, "%.3f000", fmsecs);
    return smsecs;
  }


  /** Create a SDP file
    @param name name of SDP file (without '.sdp')
    @param url Rtsp-URI to use in this file
    @param relpath relative path (used to store SDPs)
    @param abspath absolute path (used to store SDPs)
    @return web-path for that file
  */
  inline string createSDPFile(string name, string url, string relpath, string abspath)
  {
    string fname (abspath + name + ".sdp");

    ofstream ofile (fname.c_str(), ios::out | ios::trunc);
    if (ofile)
    {
      ofile << "v=0" << "\r\n";
      ofile << "s=" << name << "\r\n";
      ofile << "a=control:" << url <<"\r\n";
      ofile.close();
    } else {
      cerr << "no success!!!" << endl;
    }

    return relpath + name + ".sdp";
  }

};




/**
 * This class is used to output debugging information either to
 * standard output or to a specified file.
 *
 * @author Klaus Schoeffmann
 */

class SemiDebug
{
  /** file-output-stream used to write debugging information */
  ofstream outfile;
  /** vector containing the last debugging informations, which were
      not already written to the output (file) */
  vector<string> log;
  /** name of logfile to use */
  char *logfile;
  /** flag: ouput every message also to standard-output? */
  bool stdOutEnabled;
  /** flag: immediately output every message? */
  bool shouldFlush;

  public:
    /** default constructor */
    SemiDebug(char *logfilename = NULL);
    /** default destructor */
    ~SemiDebug();

    /** enable output of messages to standard-output */
    void enableStdOut();
    /** enable immediate output of messages */
    void enableFlush();
    /** flush messages in output.
        Thereby all messages will be written immediately to the output-file (and to standard-output) */
    void doFlush();
    /** write all messages in vector to the output-file and close it */
    void writeLogfile();
    /** write the current timestamp to the output-file */
    void writeTimestamp(char *str);


    /** convert the parameter to a string and push it to the vector.
        if the flag stdOutEnabled is set, also output parameter to standard-out.
        @param n parameter to convert to a string
        @return a pointer to this object itself */
    SemiDebug& operator << (unsigned int n);
    /** convert the parameter to a string and push it to the vector.
        if the flag stdOutEnabled is set, also output parameter to standard-out.
        @param n parameter to convert to a string
        @return a pointer to this object itself */
    SemiDebug& operator << (int n);
    /** convert the parameter to a string and push it to the vector.
        if the flag stdOutEnabled is set, also output parameter to standard-out.
        @param n parameter to convert to a string
        @return a pointer to this object itself */
    SemiDebug& operator << (long n);
    /** convert the parameter to a string and push it to the vector.
        if the flag stdOutEnabled is set, also output parameter to standard-out.
        @param n parameter to convert to a string
        @return a pointer to this object itself */
    SemiDebug& operator << (unsigned long n);
    /** convert the parameter to a string and push it to the vector.
        if the flag stdOutEnabled is set, also output parameter to standard-out.
        @param f parameter to convert to a string
        @return a pointer to this object itself */
    SemiDebug& operator << (float f);
    /** convert the parameter to a string and push it to the vector.
        if the flag stdOutEnabled is set, also output parameter to standard-out.
        @param d parameter to convert to a string
        @return a pointer to this object itself */
    SemiDebug& operator << (double d);
    /** convert the parameter to a string and push it to the vector.
        if the flag stdOutEnabled is set, also output parameter to standard-out.
        @param c parameter to convert to a string
        @return a pointer to this object itself */
    SemiDebug& operator << (char c);
    /** convert the parameter to a string and push it to the vector.
        if the flag stdOutEnabled is set, also output parameter to standard-out.
        @param str parameter to convert to a string
        @return a pointer to this object itself */
    SemiDebug& operator << (char *str);
    /** convert the parameter to a string and push it to the vector.
        if the flag stdOutEnabled is set, also output parameter to standard-out.
        @param str parameter to convert to a string
        @return a pointer to this object itself */
    SemiDebug& operator << (const char *str);
    /** convert the parameter to a string and push it to the vector.
        if the flag stdOutEnabled is set, also output parameter to standard-out.
        @param str parameter to convert to a string
        @return a pointer to this object itself */
    SemiDebug& operator << (string &str);
};




/**
 * This class is used to provide some configuration values, which
 * will be read from a configuration-file (which is created on first
 * run and filled with default-values, if not already existing)
 *
 * @author Klaus Schoeffmann
 */

class Globals
{
  /** flag: has class already been initialized with values from logfile? */
  static bool initialized;
  /** map used to store values of configuration-fields */
  static map<char*,char*> config;
  /** map used to store descriptions of configuration-fields */
  static map<char*,char*> configDesc;

  public:
    /** default constructor */
    Globals () {};
    /** load all configuration values from file */
    static void initialize();
    /** @return the CGI-Path to use */
    static string getCGIPath();
    /** @return the host-address of the semiproxy (used from semiserver) */
    static char* getProxyComHost();
    /** @return the port of the semiproxy (used from semiserver) */
    static int getProxyComPort();
    /** @return file-path of config-file */
    static char *getConfigFilePath();
    /** @return filepath to movielist (which was requested from mediaserver) */
    static char* getMoviesFileName();
    /** @return absolute path of images used from semiserver */
    static char* getAbsoluteImagesFilePath();
    /** @return relative path of images used from semiserver */
    static char* getRelativeImagesFilePath();
    /** @return host running the http-server of the mediaserver */
    static char* getMediaserverHttpHost();
    /** @return port running the http-server of the mediaserver */
    static int getMediaserverHttpPort();
    /** @return relative path where SDP files are/should be stored */
    static char* getRelativeSDPFilePath();
    /** @return absolute path where SDP files are/should be stored */
    static char* getAbsoluteSDPFilePath();
    /** @return number of hours, a closed session should be remained in the list of started sessions */
    static uint getHoursToPreserveClosedSessions();
    /** @return maximum number of seconds to wait for the response of a position request */
    static uint getPRMaxWaitSecs();

    /** reference to debugging class */
    static SemiDebug sdebug;
    /** load configuration from file */
    static bool loadConfiguration(char *filename);
    /** read configuration-fields from buffer */
    static void readConfigValues(char *buffer);
    /** load default-configuration into map */
    static void loadDefaultConfiguration();
    /** write current configuration to file */
    static void writeConfiguration(char *filename);
    /** output current configuration to standard-out */
    static void showCurrentConfiguration();
};


/**
 * This namespace provides default html-texts, which are
 * used from the semiserver-programs.
 *
 * @author Klaus Schoeffmann
 */
namespace html {

  /**
    @return a HTML-formatted string, containing a warning,
    filled with value of the parameter */
  inline string warning(string val)
  {
    string str = "<br><font class=\"warning\">WARNING: ";
    str.append(val);
    str.append("</font>");

    return str;
  }


  /** Create the HTML Header which is used on every page. */
  inline string getHTMLHeader()
  {
    string hdr = "Content-type: text/html\n\n"
    "<html><head><title>Videostream Migration System, (c) by KS 2004</title>"
    "<style type=\"text/css\">"
    "table.background {"
    "        background-color:white;"
    "        border-style:solid;"
    "        border-width:1px;"
    "        border-color:#000000;"
    "        text-align:center;"
    "}"
    "body, td, .profileinfo, input, textarea, select, td {"
    "        font-family: Verdana;"
    "        font-size: 12px;"
    "}"
    "a, a:visited {"
    "       color: #0000FF;"
    "}"
    "input, textarea, select {"
    "        color: #002F68;"
    "}"
    "body {"
    "        scrollbar-arrow-color: #253BC0;"
    "        scrollbar-base-color: #B2C2D7;"
    "        scrollbar-track-color: #D0D7E1;"
    "        scrollbar-highlight-color: #002F68;"
    "        scrollbar-3dlight-color: #D0D7E1;"
    "        scrollbar-darkshadow-color: #D0D7E1;"
    "        scrollbar-shadow-color: #002F68;"
    "        background-color:#ECF1F7;"
    "}"
    "font.copyright {"
    "        font-size:10px;"
    "}"
    "font.warning {"
    "        color: #FF0000;"
    "}"
    "font.error {"
    "        color: #FF0000;"
    "        font-weight: bold;"
    "}"
    "td.menu {"
    "        background-color: #EAEAEA;"
    "}"
    "td.copyright {"
    "        background-color: #F7F7F7;"
    "        border-top-style: solid;"
    "        border-top-width: 1px;"
    "        border-top-color: #000000;"
    "}"
    ".profileinfo {"
    "        width:80%;"
    "        background-color:#FFFFCC;"
    "        border-style:solid;"
    "        border-width:1px;"
    "        border-color:#CCCCCC;"
    "}"
    ".moviehdr {"
    "        font-size: 12px;"
    "        background-color:#F7F7F7;"
    "        color: #000000;"
    "}"
    ".moviedesc {"
    "        color: #AAAAAA;"
    "        font-style: italic;"
    "}"
    ".activeSessions {"
    "        background-color:#F7F7F7;"
    "        border-style:solid;"
    "        border-width:1px;"
    "        border-color:#FF0000;"
    "}"
    ".darkblueborder {"
    "        scrollbar-arrow-color: #FFFFFF;"
    "        scrollbar-base-color: #B2C2D7;"
    "        scrollbar-track-color: #D0D7E1;"
    "        scrollbar-highlight-color: #002F68;"
    "        scrollbar-3dlight-color: #D0D7E1;"
    "        scrollbar-darkshadow-color: #D0D7E1;"
    "        scrollbar-shadow-color: #002F68;"
    "        border-style:solid;"
    "        border-color:#002F68;"
    "        border-width:1px;"
    "}"
    "</style>"
    "<script language=\"JavaScript\">"
    "  function submitFormInWindow(formname) {"
    "    form = document.forms[formname];"
    "    win = window.open(\"about:blank\", formname, \"width=300,height=200,resizeable=yes\");"
    "    form.submit();"
    "  }"
    "</script></head>"
    "<body><center>"
    "\n<h3>Session Migration System</h3><br>\n";

    //status=yes,dependend=yes,menubar=yes, resizeable=yes

    return hdr;
  }

  /** Create the top-menu which is used on every page. */
  inline string getMenu(char *currMenu, uint pageSize)
  {
    ostringstream str;
    if (pageSize == 1) str << "\n<table style=\"width:240px;\" cellspacing=\"5\" class=\"background\"><tr>";
    else if (pageSize == 2) str << "\n<table style=\"width:480px;\" cellspacing=\"5\" class=\"background\"><tr>";
    else if (pageSize == 3) str << "\n<table style=\"width:640px;\" cellspacing=\"5\" class=\"background\"><tr>";

    str << "\n   <td width=\"50%\" class=\"menu\"><a href=\"../index.html\">Home</a></td>";

    str << "\n  <td width=\"50%\" class=\"menu\"><a href=\"javascript:document.forms['refreshForm'].submit();\">Refresh</td>";

    str << "\n</tr><tr><td align=\"center\" colspan=\"4\">";

    return str.str();
  }


  /** Create the HTML Footer which is used on every page. */
  inline string getHTMLFooter(string refreshForm = "")
  {
    //string ftr = "\n<tr><td colspan=\"4\"><br><br></td></tr>"
    string ftr = "\n<tr><td class=\"copyright\" colspan=\"4\">"
    "<font size=\"1\">Created by Klaus Sch&ouml;ffmann,<br>University of Klagenfurt - ITEC, 2004</font>"
    "</td></tr></table>" + refreshForm +
    "</center></body></html>";


    return ftr;
  }

  /** Create a form which can be used to change the page size */
  inline void createPageSizeForm(string formheader, uint pageSize)
  {
    cout << formheader;
    cout << "Page size:<br>" << endl;
    cout << "<input type=\"radio\" name=\"pagesize\" value=\"1\"";
    if (pageSize == 1) cout << " checked ";
    cout << " onClick=\"document.forms['frmPageSize'].submit();\">" << endl;
    cout << "small";
    cout << "<input type=\"radio\" name=\"pagesize\" value=\"2\"";
    if (pageSize == 2) cout << " checked ";
    cout << " onClick=\"document.forms['frmPageSize'].submit();\">" << endl;
    cout << "medium";
    cout << "<input type=\"radio\" name=\"pagesize\" value=\"3\"";
    if (pageSize == 3) cout << " checked ";
    cout << " onClick=\"document.forms['frmPageSize'].submit();\">" << endl;
    cout << "large</form>";
  }

  /** Create a form which can be used to open a new window with the
      embedded quicktime control
  */
  inline void createQTEForm(string name, string uri, uint width, uint height)
  {
    cout << "\n<form name=\"" << name << "\" action=\"embedded.cgi\" method=\"post\" target=\"" << name << "\">" << endl;
    cout << "<input type=\"hidden\" name=\"uri\" value=\"" << uri << "\">" << endl;
    cout << "<input type=\"hidden\" name=\"width\" value=\"" << width << "\">" << endl;
    cout << "<input type=\"hidden\" name=\"height\" value=\"" << height << "\">" << endl;
    cout << "</form>";
  }

  /** Print a HTML-formatted error */
  inline string error(string val)
  {
    string str = "<br><font class=\"error\"><br>ERROR: ";
    str.append(val);
    str.append("</font>");
    str.append(getHTMLFooter());

    return str;
  }

};


#endif
