/***********************************************************************
*                                                                      *
* ViTooKi                                                              *
*                                                                      *
* title: SessionList.cpp                                               *
*                                                                      *
*                                                                      *
*                                                                      *
* ITEC institute of the University of Klagenfurt (Austria)             *
* http://www.itec.uni-klu.ac.at                                        *
*                                                                      *
*                                                                      *
* For more information visit the ViTooKi homepage:                     *
* http://ViTooKi.sourceforge.net                                       *
* vitooki-user@lists.sourceforge.net                                   *
* vitooki-devel@lists.sourceforge.net                                  *
*                                                                      *
* This file is part of ViTooKi, a free video toolkit.                  *
* ViTooKi is free software; you can redistribute it and/or             *
* modify it under the terms of the GNU General Public License          *
* as published by the Free Software Foundation; either version 2       *
* of the License, or (at your option) any later version.               *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
* MA  02111-1307, USA.                                                 *
*                                                                      *
***********************************************************************/

/***********************************************************************
*                                                                      *
*  REVISION HISTORY:                                                   *
*                                                                      *
*                                                                      *
*                                                                      *
***********************************************************************/

/***********************************************************************
* Video Session Migration System, 2004                                 *
* Author: Klaus Schoeffmann                                            *
************************************************************************/


#include "SessionList.hpp"


/***********************************************************************/
SessionList::SessionList(char *host, int port, Profile *profile)
{
  this->host = host;
  this->port = port;
  this->profile = profile;

  if (createResponderSocket() == 0)
  {
    proxyConnected = true;
  }
  else
  {
    proxyConnected = false;
  }
}

/***********************************************************************/
SessionList::~SessionList()
{
  list.clear();
}


/***********************************************************************/
int SessionList::createResponderSocket()
{
  struct hostent *hp;
  struct sockaddr_in sin;

  cout << "<!--";
  cout << "Trying to connect to " << host << " on port " << port << "...";

  if (!(hp = gethostbyname(host)))
  {
    cerr << "SessionList::createResponderSocket: Error: cannot resolve host \"" << host << "\"\r\n";
    return 1;
  }

  sin.sin_family = AF_INET;
  memcpy(hp->h_addr, (char *) &sin.sin_addr, hp->h_length);
  //bcopy (hp->h_addr, (char*) &sin.sin_addr, hp->h_length);
  sin.sin_port = htons(port);

  if ((sock = socket(PF_INET, SOCK_STREAM, 0)) < 0)
  {
    cerr << "SessionList::createResponderSocket: Error: ServerSocket-Create\r\n";
    return 1;
  }

  if (connect(sock, (struct sockaddr*) &sin, sizeof(sin)) < 0)
  {
    cerr << "SessionList::createResponderSocket: Error: ServerSocket-Connect\r\n";
#ifndef WIN32
    close(sock);
#else
	closesocket(sock);
#endif
    return 1;
  }

  cout << "success!\r\n";
  cout << "-->";
  return 0;
}

/***********************************************************************/
void SessionList::retrieveSessions()
{
  if (proxyConnected)
  {
    cout << "<!--";
    cout << endl << "readActiveSessions"; cout.flush();
    list.clear();

    int entries;
#ifndef WIN32
    recv(sock, (int *) &entries, sizeof(int), 0);
#else
	recv(sock, (char *) &entries, sizeof(int), 0);
#endif

    cout << "..." << entries << " item(s)"; cout.flush();

    for (int i=0; i < entries; i++)
    {
      uint sessionId, URLlen, clientAdrlen, movieIdlen;
      uint sessionTOId, state, tsCount;
      ulong msecsElapsed, duration, prevDuration, tsCreation, tsClosed;
      char *URL, *clientAdr, *movieId;

#ifndef WIN32
      recv(sock, (uint *) &sessionId, sizeof(uint), 0);
      recv(sock, (uint *) &URLlen, sizeof(uint), 0);
      URL = (char *) malloc(sizeof(char) * (URLlen + 1));
      recv(sock, URL, sizeof(char) * (URLlen + 1), 0);
      recv(sock, (uint *) &clientAdrlen, sizeof(uint), 0);
      clientAdr = (char *) malloc(sizeof(char) * (clientAdrlen + 1));
      recv(sock, clientAdr, sizeof(char) * (clientAdrlen + 1), 0);
      recv(sock, (uint *) &movieIdlen, sizeof(uint), 0);
      movieId = (char *) malloc(sizeof(char) * (movieIdlen + 1));
      recv(sock, (char *) movieId, sizeof(char) * (movieIdlen + 1), 0);
      recv(sock, (uint *) &sessionTOId, sizeof(uint), 0);
      recv(sock, (ulong *) &msecsElapsed, sizeof(ulong), 0);
      recv(sock, (ulong *) &duration, sizeof(ulong), 0);
      recv(sock, (ulong *) &prevDuration, sizeof(ulong), 0);
      recv(sock, (uint *) &tsCount, sizeof(uint), 0);
      recv(sock, (uint *) &state, sizeof(uint), 0);
      recv(sock, (ulong *) &tsCreation, sizeof(ulong), 0);
      recv(sock, (ulong *) &tsClosed, sizeof(ulong), 0);
#else
      recv(sock, (char *) &sessionId, sizeof(uint), 0);
      recv(sock, (char *) &URLlen, sizeof(uint), 0);
      URL = (char *) malloc(sizeof(char) * (URLlen + 1));
      recv(sock, URL, sizeof(char) * (URLlen + 1), 0);
      recv(sock, (char *) &clientAdrlen, sizeof(uint), 0);
      clientAdr = (char *) malloc(sizeof(char) * (clientAdrlen + 1));
      recv(sock, clientAdr, sizeof(char) * (clientAdrlen + 1), 0);
      recv(sock, (char *) &movieIdlen, sizeof(uint), 0);
      movieId = (char *) malloc(sizeof(char) * (movieIdlen + 1));
      recv(sock, (char *) movieId, sizeof(char) * (movieIdlen + 1), 0);
      recv(sock, (char *) &sessionTOId, sizeof(uint), 0);
      recv(sock, (char *) &msecsElapsed, sizeof(ulong), 0);
      recv(sock, (char *) &duration, sizeof(ulong), 0);
      recv(sock, (char *) &prevDuration, sizeof(ulong), 0);
      recv(sock, (char *) &tsCount, sizeof(uint), 0);
      recv(sock, (char *) &state, sizeof(uint), 0);
      recv(sock, (char *) &tsCreation, sizeof(ulong), 0);
      recv(sock, (char *) &tsClosed, sizeof(ulong), 0);

#endif

      SSession *asess = new SSession(sessionId, URL, clientAdr, movieId, sessionTOId,
                                     msecsElapsed, state, duration, prevDuration, tsCount, tsCreation, tsClosed);
      cout << "<br>" << " received SSession: " << asess->getRtspSessionId(); cout.flush();
      cout << "<br>" << "            " << asess->getUrl(); cout.flush();
      cout << "<br>" << "            " << asess->getClientAdr() << endl; cout.flush();
      cout << "<br>" << "            " << asess->getSemiMovieId(); cout.flush();
      cout << "<br>" << "            " << asess->getOriginRtspSessionId() << endl; cout.flush();
      cout << "<br>" << "            " << asess->getElapsedMsecs(0,0) << endl; cout.flush();
      cout << "<br>" << "            " << asess->getDuration() << endl; cout.flush();
      cout << "<br>" << "            " << asess->getPreviousSessionDuration() << endl; cout.flush();
      cout << "<br>" << "            " << asess->getTimeSectionCount() << endl; cout.flush();
      cout << "<br>" << "            " << asess->getState() << endl; cout.flush();
      cout << "<br>" << "            " << asess->getTimestampCreated() << endl; cout.flush();
      cout << "<br>" << "            " << asess->getTimestampClosed() << endl; cout.flush();

      list.push_back(asess);
    }

    cout << "-->";
  }
}

/***********************************************************************/
vector<SSession*>* SessionList::getList()
{
  return &list;
}

/***********************************************************************/
void SessionList::writeHTMLRepresentation(CGIParser *cgiparser, bool showActiveSessions,
    bool showClosedSessions, bool fakeRtspServer)
{
  cout << "<tr><td colspan=\"2\" align=\"center\">";


  if (list.size() == 0)
  {
    cout << "Currently, there are no existing sessions!<br><br>" << endl;
  }
  else
  {

    //generate checkbox which can be used to display closed sessions to
    cout << cgiparser->createOpenForm("frmShowParams", "logon.cgi", "post", "showAS,showCS,showMS");
    cout << "Show:";
    //param: active sessions
    cout << "<input type=\"checkbox\" name=\"showAS\" ";
    if (showActiveSessions) cout << " checked ";
    cout << "onClick=\"document.forms['frmShowParams'].submit();\">active sessions";
    //param: closed sessions
    cout << "<input type=\"checkbox\" name=\"showCS\" ";
    if (showClosedSessions) cout << " checked ";
    cout << "onClick=\"document.forms['frmShowParams'].submit();\">closed sessions";
    cout << "</form>" << endl;


    cout << "List of currently active sessions:" << endl;

    uint userId = atoi(cgiparser->getFieldValue("userid").c_str());
    uint profileId = atoi(cgiparser->getFieldValue("proid").c_str());
    uint linktype = atoi(cgiparser->getFieldValue("linktype").c_str());
    int subtractSecs = atoi(cgiparser->getFieldValue("subtractSecs").c_str());
    char *sphost = semifunc::newStrCpy((char*)cgiparser->getFieldValue("sphost").c_str());
    char *spport = semifunc::newStrCpy((char*)cgiparser->getFieldValue("spport").c_str());

    cout << "<table width=\"95%\" class=\"activeSessions\">" << endl;

    //iterate through session list
    vector<SSession*>::reverse_iterator iter = list.rbegin();
    for (;iter != list.rend(); iter++)
    {
      SSession *as = *iter;

      bool showSession = false;

      if (!as->isClosed() && showActiveSessions) showSession = true;
      if (as->isClosed() && (as->getTimeSectionCount() > 1) && showClosedSessions) showSession = true;

      if (showSession)
      {

        //output name of video
        cout << "<tr><td ";
        if (as->isClosed()) cout << " style=\"background-color:#C6C6C6;\" ";
        cout << "><a href=\"#" << as->getSemiMovieId() << "\">" << endl;
        //if program crashes here... restart proxy!
        //(this may happen, if old sessions from another server exists in proxy)
        assert(MovieList::getMovieById(as->getSemiMovieId()) != NULL);

        cout << MovieList::getMovieById(as->getSemiMovieId())->getName()
        << "</a></td>" << endl;

        //generate links for migrate or copy
        Movie *movie = MovieList::getMovieById(as->getSemiMovieId());
        if (movie != NULL)
        {
          //generate URI
          string movieUri = movie->getUrl(userId, profile->getProfileId());
          unsigned int hostStart = movieUri.find("rtsp://") + 7;
          unsigned int hostEnd = movieUri.find("/", hostStart);
          unsigned int filePathStart = hostEnd;
          unsigned int portStart = movieUri.find(":", hostStart) + 1;
          if (portStart != string::npos && portStart < hostEnd) hostEnd = portStart-1;
          if (fakeRtspServer) {
            if (portStart != string::npos) {
              unsigned int slashPos = filePathStart;
              string portNr = movieUri.substr(portStart, slashPos-portStart);
              movieUri.insert(filePathStart,"/port=");
              movieUri.insert(filePathStart+strlen("/port="),portNr);
              movieUri.erase(portStart, slashPos-portStart);
              movieUri.insert(portStart, spport);
            }

            //change host
            string hostAdr = movieUri.substr(hostStart, hostEnd-hostStart);
            if (hostAdr.compare(sphost) != 0) {
              movieUri.erase(hostStart, hostEnd-hostStart);
              movieUri.insert(hostStart, sphost);
              filePathStart = movieUri.find("/", hostStart); //may have changed
              movieUri.insert(filePathStart, "/host=");
              movieUri.insert(filePathStart+strlen("/host="), hostAdr);
            }
          }

          filePathStart = movieUri.find("/", hostStart);
          string movieUri1 = movieUri.substr(0, filePathStart);
          string movieUri2 = movieUri.substr(filePathStart+1);

          ostringstream rtspUriMigrate, rtspUriCopy;
          rtspUriMigrate << movieUri1
          << "/action=" << ACTION_MIGRATE
          << "/sub=" << (subtractSecs)
          << "/sid=" << as->getRtspSessionId() << "/" << movieUri2;
          rtspUriCopy << movieUri1
          << "/action=" << ACTION_COPY
          << "/sub=" << (subtractSecs)
          << "/sid=" << as->getRtspSessionId() << "/" << movieUri2;

          //************* output links for migrate ******************
          cout << "<td>" << endl;
          if (as->isClosed())
          {
            cout << "&nbsp;"; //do not show migrate links for closed sessions
          }
          else
          {
            if (linktype == LINK_QTE)
            {

              string frmName = "frmMigrate" + semifunc::toString(as->getRtspSessionId());
              html::createQTEForm(frmName, rtspUriMigrate.str(),
                                  profile->getWidth(), profile->getHeight());
              cout << "<a href=\"javascript:submitFormInWindow('"
              << frmName << "');\">Migrate</a>*</td>" << endl;

            }
            else if (linktype == LINK_RTSP)
            {

              cout << "<a href=\"" << rtspUriMigrate.str() << "\">Migrate</a>*" << endl;

            }
            else if (linktype == LINK_SDP)
            {

              ostringstream tmpName;
              //add user_id and profile_id (otherwise files from different users would be overwritten)
              tmpName << as->getRtspSessionId() << "_m" << "_u"
              << userId << "_p" << profileId;
              cout << "<a href=\""
              << semifunc::createSDPFile(tmpName.str(),
                                         rtspUriMigrate.str(),
                                         Globals::getRelativeSDPFilePath(),
                                         Globals::getAbsoluteSDPFilePath())
              << "\">Migrate</a>*" << endl;

            }
          }
          cout << "</td>" << endl;

          //************* output links for copy ******************
          cout << "<td>" << endl;
          if (as->isClosed())
          {
            cout << "&nbsp;"; //do not show copy links for closed sessions
          }
          else
          {
            if (linktype == LINK_QTE)
            {

              string frmName = "frmCopy" + semifunc::toString(as->getRtspSessionId());
              html::createQTEForm(frmName, rtspUriCopy.str(),
                                  profile->getWidth(), profile->getHeight());
              cout << "<a href=\"javascript:submitFormInWindow('"
              << frmName << "');\">Copy</a>*</td>" << endl;

            }
            else if (linktype == LINK_RTSP)
            {

              cout << "<a href=\"" << rtspUriCopy.str() << "\">Copy</a>*";

            }
            else if (linktype == LINK_SDP)
            {

              ostringstream tmpName;
              //add user_id and profile_id (otherwise files from different users would be overwritten)
              tmpName << as->getRtspSessionId() << "_c" << "_u"
              << userId << "_p" << profileId;
              cout << "<a href=\""
              << semifunc::createSDPFile(tmpName.str(),
                                         rtspUriCopy.str(),
                                         Globals::getRelativeSDPFilePath(),
                                         Globals::getAbsoluteSDPFilePath())
              << "\">Copy</a>*" << endl;

            }
          }
          cout << "</td>" << endl;
//        }
          cout << "</tr>" << endl;

          //************* output client address ******************
          cout << "<tr><td colspan=\"1\">Client:"
          << as->getClientAdr() << "</td>" << endl;

          //************* output links for section copy ******************
          cout << "<td colspan=\"2\" rowspan=\"3\" align=\"center\""
          << " valign=\"middle\">" << endl;
          if (as->getTimeSectionCount() > 1)
          {
            cout << "Copy from section*: ";

            ostringstream selectBox;

            selectBox << "<select size=\"1\" onchange=\""
            << " if (this.value != '') eval(this.value);\">" << endl
            << "<option value="">#</option>" << endl;

            ostringstream rtspUriContinue;
//            rtspUriContinue << movieUri1
            rtspUriContinue << "/action=" << ACTION_CONTINUE
            << "/sub=" << (subtractSecs)
            << "/sid=" << as->getRtspSessionId() << "/" << movieUri2;

            for (uint j=1; j < as->getTimeSectionCount(); j++)
            {
              if (linktype == LINK_QTE)
              {

                ostringstream frmName, tmpLink;
                frmName << "frmCont" << as->getRtspSessionId() << "_" << j;
                tmpLink << movieUri1 << "/section=" << j << rtspUriContinue.str();
                html::createQTEForm(frmName.str(), tmpLink.str(), profile->getWidth(), profile->getHeight());

                selectBox << "<option value=\"javascript:submitFormInWindow('"
                << frmName.str() << "');\">" << j <<"</option>" << endl;

              }
              else if (linktype == LINK_RTSP)
              {

                selectBox << "<option value=\"window.open('" << movieUri1
                << "/section=" << j << rtspUriContinue.str() << "')\">" << j << "</option>" << endl;

              }
              else if (linktype == LINK_SDP)
              {

                ostringstream tmpLink, tmpName;
                tmpLink << movieUri1 << "/section=" << j << rtspUriContinue.str();
                //add user_id and profile_id (otherwise files from different users would be overwritten)
                tmpName << as->getRtspSessionId() << "_" << j << "_u" << userId << "_p" << profileId;

                selectBox << "<option value=\"window.open('"
                << semifunc::createSDPFile(tmpName.str(),
                                          tmpLink.str(),
                                          Globals::getRelativeSDPFilePath(),
                                          Globals::getAbsoluteSDPFilePath())
                << "')\">" << j << "</option>" << endl;

              }
            }

            selectBox << "</select>" << endl;
            cout << selectBox.str();

          }
          else
          {
            cout << "&nbsp;";
          }
          cout << "</td></tr>" << endl;
        }

        //************* output state ******************
        cout << "<tr><td colspan=\"1\">State: " << endl;
        switch (as->getState())
        {
        case STATE_RUNNING:
          cout << "<font color=\"#00FF00\">active</font>";
          break;
        case STATE_PAUSED:
          cout << "<font color=\"#0000FF\">paused</font>";
          break;
        case STATE_CLOSED:
          cout << "<font color=\"#000000\">closed</font>";
          break;
        default:
          cout << "unknown";
        }
        cout << "</td><tr>" << endl;

        //************* output elapsed time and duration ******************
        cout << "<tr><td colspan=\"1\">Seconds: " << endl;
        char *msecs = semifunc::newStrCpy(semifunc::formatTime(as->getElapsedMsecs(0,0)));
        char *dur = semifunc::newStrCpy(semifunc::formatTime(as->getDuration()));
        cout << msecs << " / ";
        cout << dur << "</td></tr>" << endl;

        //************* output elapsed time of previous session(s) ******************
        if (as->getOriginRtspSessionId() != 0)
        {
          cout << "<tr><td colspan=\"3\"><font color=\"#979797\">";
          cout << "Elapsed time of prev. session(s): " << semifunc::newStrCpy(semifunc::formatTime(as->getPreviousSessionDuration()));
          cout << "</font></td></tr>" << endl;
        }

        //************* output timestamps ******************
        cout << "<tr><td colspan=\"3\"><font color=\"#979797\">";
        cout << "Created on: " << semifunc::newStrCpy(semifunc::formatTimeSecs(as->getTimestampCreated()));
        if (as->getState() == STATE_CLOSED)
        {
          cout << "&nbsp;&nbsp;&nbsp;&nbsp;";
          cout << "Closed on: " << semifunc::newStrCpy(semifunc::formatTimeSecs(as->getTimestampClosed()));
        }
        cout << "</font></td></tr>" << endl;

        cout << "<tr><td colspan=\"3\" style=\"border-bottom-style:solid;border-bottom-width:1px;border-bottom-color:#000000;\"><br></td></tr>";

      }

    }
    cout << "</table>" << endl;


    //************* output selection of subtract-seconds ******************
    cout << cgiparser->createOpenForm("subtractSelect", "logon.cgi", "post", "");
    cout << "<table width=\"100%\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\">" << endl;
    cout << "<tr><td align=\"center\">";
    cout << "*Start ";
    cout << "<select name=\"subtractSecs\" onchange=\"document.forms['subtractSelect'].submit();\" size=1>" << endl;
    int runTo = 20;
    int step = 2;
    for (int i=-runTo; i <= runTo; i+=step)
    {
      cout << "<option ";
      if (i == subtractSecs) cout << " selected ";
      cout << "value=\"" << i << "\">";
      if (i > 0)
      {
        if (i >= 0 && i < 10) cout << "&nbsp;&nbsp;" << i;
        else cout << "&nbsp;" << i;
        cout << " seconds earlier";
      }
      else if (i < 0)
      {
        if (i < 0 && i > -10) cout << "&nbsp;&nbsp;" << (i*-1);
        else cout << "&nbsp;" << (i*-1);
        cout << " seconds later";
      }
      else
      {
        cout << "on EXACT POSITION";
      }
      cout <<"</option>" << endl;
    }
    cout << "</select>" << endl;
    cout << "</form>" << endl;
    cout << "</td></tr></table>";

    cout << "<br><br>";
  }
  cout << "</td></tr>";

}
