 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: SessionList.hpp                                               *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 * Video Session Migration System, 2004                                 *
 * Author: Klaus Schoeffmann                                            *
 ************************************************************************/


#ifndef __SessionList_hpp__
#define __SessionList_hpp__

#ifdef WIN32
#include <winsock2.h>
#else
#include <unistd.h>
#include <time.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <unistd.h>
#include <arpa/inet.h>
#endif

#ifndef WIN32
#include <unistd.h>
#include <sys/types.h>
#endif

#include <string>
#include <iostream>
#include <fstream>
#include <vector>
#include <assert.h>

#include "SSession.hpp"
#include "Movie.hpp"
#include "SemiGlobals.hpp"
#include "MovieList.hpp"
#include "Profile.hpp"
#include "CGIParser.hpp"

using namespace std;

/**
 * This class is used to request and manage a list of currently active
 * sessions from the semiproxy.
 *
 * @author Klaus Schoeffmann
 */


class SessionList
{
  private:
    /** host-address to connect to */
    char *host;
    /** port to connect to */
    int port;
    /** socket of established connection */
    int sock;
    /** flag: connected with semiproxy? */
    bool proxyConnected;
    /** reference to the profile to use for HTML-representation.
        In this representation, the URI-Parameters of the Rtsp-URI
        will depend on this profile */
    Profile *profile;

    /** list of currently active sessions */
    vector<SSession*> list;

    /** opens a connection to specified host/port */
    int createResponderSocket();

  public:
    /** default constrctor */
    SessionList(char* host, int port, Profile *profile);
    /** default destructor */
    ~SessionList();
    /** retrieve list of currently active sessions from semiproxy
        and store them in the list */
    void retrieveSessions();
    /** @return a reference to the list of currently active sessions */
    vector<SSession*>* getList();
    /** Write a HTML-representation of the session-list to standard-out. */
    void writeHTMLRepresentation(CGIParser *cgiparser, bool showActiveSessions,
                                  bool showClosedSessions, bool fakeRtspServer);
};
#endif
