 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: TimeMeasurement.hpp (S H A R E D)                             *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 * Video Session Migration System, 2004                                 *
 * Author: Klaus Schoeffmann                                            *
 ************************************************************************/


#ifndef __TimeMeasurement_hpp__
#define __TimeMeasurement_hpp__

#include <iostream>
#include <string>
#include <vector>
#include <assert.h>
#include "TimeSection.hpp"

using namespace std;

/**
 * This class is used to measure the elapsed time of
 * a streaming session.
 *
 * @author Klaus Schoeffmann
 */

class TimeMeasurement {

private:

  /** list of time-sections within current section */
  vector<TimeSection*> *sections;
  /** on migration/copy: elapsed time of origin session in msecs */
  ulong previousSessionDuration;
  /** duration of current session
      (announced from mediaserver in SDP of Describe-Response) */
  ulong duration;

public:
  /** default constructor */
  TimeMeasurement();
  /** default destructor */
  ~TimeMeasurement();
  /** Add a new section to the list of sections. */
  void addNewSection(ulong prebuffertime);
  /** Set the endtime of previous session */
  void setPreviousSessionDuration(ulong previousSessionDuration);
  /** Set the duration of the movie. */
  void setDuration(ulong duration);
  /** @return endtime of previous session */
  ulong getPreviousSessionDuration();
  /** @return duration of current session */
  ulong getDuration();
  /** @return estimated elapsed time of current session.
      This value is either calculated as the difference
      between measured end-time/start-time and client-buffer-size
      or from retrieved server-estimation and client-buffer-size.
      Furthermore, if this session contains more than one
      time-sections, the range-values of the client-requests
      are considered, to minimize the deviation from the real mediatime*/
  ulong getElapsedTime(uint sectionNr = 0, ulong serverPlayout = 0, ulong clientPlayout = 0);
  /** @return current number of time-sections */
  uint getTimeSectionCount();
  /** Set start-timestamp of current time-section */
  void startTS();
  /** Set stop-timestamp of current time-section */
  void stopTS();
  /** Set requested range-from value of current time-section
      (and correct duration of previous time-section) */
  void setTSrequestedFrom(ulong reqfrom);
  /** Set client playout-time (retrieved from mediaserver)
      of current time-section */
  void setTSserverPlayout(ulong playout);
  /** Set client playout-time (retrieved from client)
      of current time-section */
  void setTSclientPlayout(ulong playout);
};

#endif
