/***********************************************************************
*                                                                      *
* ViTooKi                                                              *
*                                                                      *
* title: TimeSection.cpp (S H A R E D)                                 *
*                                                                      *
*                                                                      *
*                                                                      *
* ITEC institute of the University of Klagenfurt (Austria)             *
* http://www.itec.uni-klu.ac.at                                        *
*                                                                      *
*                                                                      *
* For more information visit the ViTooKi homepage:                     *
* http://ViTooKi.sourceforge.net                                       *
* vitooki-user@lists.sourceforge.net                                   *
* vitooki-devel@lists.sourceforge.net                                  *
*                                                                      *
* This file is part of ViTooKi, a free video toolkit.                  *
* ViTooKi is free software; you can redistribute it and/or             *
* modify it under the terms of the GNU General Public License          *
* as published by the Free Software Foundation; either version 2       *
* of the License, or (at your option) any later version.               *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
* MA  02111-1307, USA.                                                 *
*                                                                      *
***********************************************************************/

/***********************************************************************
*                                                                      *
*  REVISION HISTORY:                                                   *
*                                                                      *
*                                                                      *
*                                                                      *
***********************************************************************/

/***********************************************************************
* Video Session Migration System, 2004                                 *
* Author: Klaus Schoeffmann                                            *
************************************************************************/


#include "TimeSection.hpp"

/***********************************************************************/
TimeSection::TimeSection(int nr, ulong prebuffertime)
{
  Globals::sdebug << "\nTS: start";
  this->nr = nr;
  this->prebufferTime = prebuffertime;
  measuredStarttime = 0;
  measuredEndtime = 0;
  requestedFrom = 0;
  clientReceived = 0;
  serverPlayout = 0;
  clientPlayout = 0;
  Globals::sdebug << "\nTimeSection::TimeSection():" << nr << " created with prebuffer:" << prebuffertime;
}

/***********************************************************************/
TimeSection::~TimeSection()
{
  Globals::sdebug << " s" << nr;
}

/***********************************************************************/
ulong TimeSection::getCurrentMsecs()
{
  //get current timestamp
  struct timeval currTime;
  gettimeofday(&currTime, NULL);

  //calculate to msecs
  ulong currTimeMs = ((ulong)currTime.tv_sec*1000 + (ulong)currTime.tv_usec/1000);
  return currTimeMs;
}

/***********************************************************************/
void TimeSection::start()
{
  measuredStarttime = getCurrentMsecs();
  Globals::sdebug << "\nTimeSection::start():" << nr << " = " << measuredStarttime;
}

/***********************************************************************/
void TimeSection::stop()
{
  measuredEndtime = getCurrentMsecs();
  Globals::sdebug << "\nTimeSection::stop():" << nr << " = " << measuredEndtime;
}

/***********************************************************************/
ulong TimeSection::getMeasuredStarttime()
{
  return measuredStarttime;
}

/***********************************************************************/
ulong TimeSection::getMeasuredEndtime()
{
  return measuredEndtime;
}

/***********************************************************************/
ulong TimeSection::getPrebufferTime()
{
  return prebufferTime;
}

/***********************************************************************/
ulong TimeSection::getRequestedFrom()
{
  return requestedFrom;
}

/***********************************************************************/
ulong TimeSection::getClientReceived()
{
  return clientReceived;
}

/***********************************************************************/
ulong TimeSection::getServerPlayout()
{
  return serverPlayout;
}

/***********************************************************************/
ulong TimeSection::getClientPlayout()
{
  return clientPlayout;
}

/***********************************************************************/
void TimeSection::setMeasuredStarttime(ulong measuredStarttime)
{
  this->measuredStarttime = measuredStarttime;
  Globals::sdebug << "\nTimeSection::setMeasuredStarttime():" << nr << " = " << measuredStarttime;
}

/***********************************************************************/
void TimeSection::setMeasuredEndtime(ulong measuredEndtime)
{
  this->measuredEndtime = measuredEndtime;
  Globals::sdebug << "\nTimeSection::setMeasuredEndtime():" << nr << " = " << measuredEndtime;
}

/***********************************************************************/
void TimeSection::setRequestedFrom(ulong requestedFrom)
{
  this->requestedFrom = requestedFrom;
  Globals::sdebug << "\nTimeSection::setRequestedFrom():" << nr << " = " << requestedFrom;
}

/***********************************************************************/
void TimeSection::setClientReceived(ulong clientReceived)
{
  this->clientReceived = clientReceived;
  Globals::sdebug << "\nTimeSection::clientReceived():" << nr << " = " << clientReceived;
}

/***********************************************************************/
void TimeSection::setServerPlayout(ulong serverPlayout)
{
  this->serverPlayout = serverPlayout;
  Globals::sdebug << "\nTimeSection::serverPlayout():" << nr << " = " << serverPlayout;
}

/***********************************************************************/
void TimeSection::setClientPlayout(ulong clientPlayout)
{
  this->clientPlayout = clientPlayout;
  Globals::sdebug << "\nTimeSection::clientPlayout():" << nr << " = " << clientPlayout;
}
