 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: TimeSection.hpp (S H A R E D)                                 *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 * Video Session Migration System, 2004                                 *
 * Author: Klaus Schoeffmann                                            *
 ************************************************************************/


#ifndef __TimeSection_hpp__
#define __TimeSection_hpp__

#include <iostream>
#include <string>
#ifndef WIN32
  #include <sys/time.h>
#else
  #include <time.h>
#endif
#include "SemiGlobals.hpp"

#include "global.hpp"

using namespace std;

/**
 * This class is used to measure the elapsed time of
 * a section (from PLAY to PAUSE) of a streaming session.
 *
 * @author Klaus Schoeffmann
 */

class TimeSection {

private:
  /** sequence number defined from TimeMeasurement */
  int nr;
  /** measured start time of section in msecs */
  ulong measuredStarttime;
  /** measured end time of section in msecs; */
  ulong measuredEndtime;
  /** prebuffering time in msecs (either announced from player or otherwise from profile) */
  ulong prebufferTime;
  /** from-value of range in msecs, requested by the player */
  ulong requestedFrom;
  /** received msecs by the client until the end of this section (info from player on next timesection) */
  ulong clientReceived;
  /** estimated client playout-time until the end of this section (info from mediaserver) */
  ulong serverPlayout;
  /** playout-time until the end of this section (info from client) */
  ulong clientPlayout;


public:
  /** default constructor */
  TimeSection(int nr, ulong prebuffertime);
  /** default destructor */
  ~TimeSection();
  /** Set start-timestamp of this timesection */
  void start();
  /** Set stop-timestamp of this timesection */
  void stop();
  /** @return measured start-timestamp */
  ulong getMeasuredStarttime();
  /** @return measured stop-timestamp */
  ulong getMeasuredEndtime();
  /** @return prebuffering time */
  ulong getPrebufferTime();
  /** @return from-value, requested by the client */
  ulong getRequestedFrom();
  /** @return duration of section, received by the client */
  ulong getClientReceived();
  /** @return server-based estimation of elapsed time on client */
  ulong getServerPlayout();
  /** @return elapsed time on client requested from the client*/
  ulong getClientPlayout();
  /** @return current time in msecs */
  ulong getCurrentMsecs();
  /** set measured start-timestamp */
  void setMeasuredStarttime(ulong measuredStarttime);
  /** set measured stop-timestamp */
  void setMeasuredEndtime(ulong measuredEndtime);
  /** set from-value, requested by the client */
  void setRequestedFrom(ulong requestedFrom);
  /** set duration of section, received by the client */
  void setClientReceived(ulong clientReceived);
  /** set server-based estimation of elapsed time on client */
  void setServerPlayout(ulong serverPlayout);
  /** set elapsed time on client, requested from the client*/
  void setClientPlayout(ulong clientPlayout);
};

#endif
