/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: User.hpp  (S H A R E D)                                       *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 * Video Session Migration System, 2004                                 *
 * Author: Klaus Schoeffmann                                            *
 ************************************************************************/


#ifndef __User_hpp__
#define __User_hpp__

#include <iostream>
#include <string>
#include <vector>
#include "SemiGlobals.hpp"
#include "Profile.hpp"

/**
 * This class is used to store users.
 *
 * @author Klaus Schoeffmann
 */

class User
{
  private:
    /** name of user */
    char username[USERNAME_MAXLEN];
    /** password of user */
    char password[PASSWORD_MAXLEN];
    /** id of user */
    uint userId;
    /** allowed profiles of user */
    vector<Profile*> profiles;

  public:
    /** default constructor */
    User(uint userId, char username[], char password[]);
    /** default destructor */
    ~User();
    /** @return name of user */
    char *getUsername();
    /** @return password of user */
    char *getPassword();
    /** @return id of user */
    uint getUserId();
    /** add a reference to the specified profile */
    void addProfile(Profile *profile);
    /** @return a reference to the list of profiles for that user */
    vector<Profile*> getProfiles();

};
#endif

