/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: UserList.cpp  (S H A R E D)                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
*                                                                      *
*  REVISION HISTORY:                                                   *
*                                                                      *
*                                                                      *
*                                                                      *
***********************************************************************/

/***********************************************************************
* Video Session Migration System, 2004                                 *
* Author: Klaus Schoeffmann                                            *
************************************************************************/


#include "UserList.hpp"

using namespace std;


/***********************************************************************/
UserList::UserList()
{}

/***********************************************************************/
UserList::~UserList()
{
  Globals::sdebug << "\nUserList::~UserList() cleanup...";
  vector<User*>::iterator iter = list.begin();
  while (iter != list.end())
  {
    User *usr = *iter;
    delete usr;
    iter++;
  }
  list.clear();
  Globals::sdebug << "\nUserList::~UserList() done!";
}

/***********************************************************************/
void UserList::addUser(User* u)
{
  list.push_back(u);
}

/***********************************************************************/
User* UserList::getUserByName(const char *username)
{
  uint i;
  for (i=0; i < list.size(); i++)
  {
    User *user = list[i];
    if (strcmp(user->getUsername(), username) == 0) return user;
  }

  return NULL;
}

/***********************************************************************/
User* UserList::getUserById(const uint userId)
{
  uint i;
  for (i=0; i < list.size(); i++)
  {
    User *user = list[i];
    if (user->getUserId() == userId) return user;
  }

  return NULL;
}

/***********************************************************************/
vector<User*> UserList::getUsers()
{
  return list;
}

/***********************************************************************/
void UserList::readUsersFromFile(const char *filename)
{
  char line[READLINE_MAXLEN], *buffer;
  char *username, *password;
  uint userId;
  uint usersCount = 0;

  //open file of users
  ifstream fp (filename);
  if (!fp)
  {
    cerr << "\nUserList::readUsersFromFile(): Error opening file '" << filename << "'";
    exit (1);
  }

  //reset variables used for reading
  userId = 0;
  username = password = NULL;

  //go to start of users
  uint zaehler = 0;
  while (! fp.eof() )
  {
    zaehler++;
    fp.getline(line,READLINE_MAXLEN);
    if (line != NULL && strncmp(line, "############USERS start############", strlen("############USERS start############")) == 0) break;
  }

  //read line by line until end of file
  //Attention: maximum line characters is set in Globals.hpp
  while (! fp.eof() )
  {
    fp.getline (line, READLINE_MAXLEN);
    if (line != NULL)
    {
      if (strncmp(line, "############USERS end############", strlen("############USERS end############")) == 0) break;

      //Globals::sdebug << "\nUL::readUsersFromFile: read line:\n" << line;
      if (line[0] == '#')
      {
        //ignore comments
      }
      //look for username
      else if (line[0] == '[' && line[strlen(line) - 1] == ']')
      {
        username = new char[strlen(line) - 1];
        strncpy(username, line + 1, strlen(line) - 2);
        username[strlen(line) - 2] = 0;
      }
      //look for userid
      else if ((buffer = strstr(line, "uid=")) != NULL)
      {
        userId = atoi(buffer + 4);
      }
      //look for password
      else if ((buffer = strstr(line, "pwd=")) != NULL)
      {
        password = new char[strlen(buffer) - 3];
        strncpy(password, buffer + 4, strlen(buffer) - 4);
        password[strlen(buffer) - 4] = 0;
      }
      else
      {
        //Unrecognized parameter ignored
      }
    }

    //if all necessary information was read, add user to list
    //and reset variables used for reading
    if (userId != 0 && username != NULL && password != NULL)
    {

      Globals::sdebug << "\nUL::readUsersFromFile: create User: " << userId << " " << username << " " << password;
      addUser(new User(userId, username, password));
      usersCount++;

      userId = 0;
      username = password = NULL;
    }

  }

  fp.close();

  Globals::sdebug << "\nUserList::readUsersFromFile(): read users successfully finished with " << usersCount << "entries!";
}
