/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   AudioJob.hpp                                                *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "AudioJob.hpp"
#include "xml.hpp"

/** constructor */
AudioJob::AudioJob():Job()
{
	dprintf_full("\033[32mAudioJob::empty construct\033[0m\n");
	this->name[0] = '\0';
	this->in_es = NULL;
	this->outFile[0] = '\0';
	this->language[0] = '\0';
	this->is_video = false;
};

AudioJob::AudioJob(const char *name, Source* es, const char* outFile, u32 targetBitRate, int sampleRate, int numChannels, int bitsPerSample, CodecID codec, const char* language, float startPosition, float endPosition):Job(name, es, outFile, targetBitRate, startPosition, endPosition)
{
	dprintf_full("\033[32mAudioJob::construct\033[0m\n");
	assert(name);
	assert(outFile);
	assert(language);
	strncpy(this->name, name, MAX_STR_LEN-1);
	this->name[MAX_STR_LEN-1] = '\0';
	this->in_es = es;
	strncpy(this->outFile, outFile, MAX_STR_LEN-1);
	this->outFile[MAX_STR_LEN-1] = '\0';
	this->targetBitRate = targetBitRate;
	this->startPosition = startPosition;
	this->endPosition = endPosition;
	
	this->sampleRate = sampleRate;
	this->numChannels = numChannels;
	this->bitsPerSample = bitsPerSample;
	this->codec = codec;
	strncpy(this->language, language, MAX_STR_LEN-1);
	this->language[MAX_STR_LEN-1] = '\0';
	this->is_video = false;
};
	
/** destructor */
AudioJob::~AudioJob()
{
	dprintf_full("\033[32mAudioJob::destruct\033[0m\n");
	this->checkReferences();
	this->in_es=NULL;
}


int AudioJob::getSampleRate() const 
{
	dprintf_full("\033[32mAudioJob::getSampleRate\033[0m\n");
	this->checkReferences();
	return this->sampleRate;
};

int AudioJob::getNumChannels() const
{
	dprintf_full("\033[32mAudioJob::getNumChannels\033[0m\n");
	this->checkReferences();
	return this->numChannels;
};


int AudioJob::getBitsPerSample() const
{
	dprintf_full("\033[32mAudioJob::getBitsPerSample\033[0m\n");
	this->checkReferences();
	return this->bitsPerSample;
};

CodecID AudioJob::getCodec() const
{
	dprintf_full("\033[32mAudioJob::getCodec\033[0m\n");
	this->checkReferences();
	return this->codec;
};

const char* AudioJob::getLanguage() const 
{
	dprintf_full("\033[32mAudioJob::getLanguage\033[0m\n");
	this->checkReferences();
	return this->language;
};
	

/** Implementation of virtual methods of Job */
bool AudioJob::toString(char* output, int max_len)
{
	dprintf_full("\033[32m>> AudioJob::toString(buf, len)\033[0m\n");
	assert(output && max_len>0);
	this->checkReferences();
	char* format = (char*)malloc(max_len);

	memset(format, 0, max_len);
	memset(output, 0, max_len);
	
	if(max_len < snprintf(format, max_len, "%s%s%s%s%s%s%s%s%s%s%s%s",
	AJOB_START,JOBID, NAME_PRINT, FILENAME_PRINT, TARGET_BR, START_POS,
	END_POS, SAMPLERATE, NUMCHANNELS, BITSPERSAMPLE, LANGUAGE_PRINT, 
	AJOB_END))
	{
		free(format);
		dprintf_err("\033[31mAudioJob::toString(buf, len)\033[0m\n");
		return false;
	}
	if(max_len < snprintf(output, max_len, format, this->jobID, this->name,
	this->outFile, this->targetBitRate, this->startPosition, 
	this->endPosition, this->sampleRate, this->numChannels, 
	this->bitsPerSample, this->language))
	{
		free(format);
		dprintf_err("\033[31mAudioJob::toString(buf, len)\033[0m\n");
		return false;
	}
	free(format);
	dprintf_full("\033[32m<< AudioJob::toString(buf, len)\033[0m\n");
	return true;
}

char* AudioJob::toString()
{
	dprintf_full("\033[32mAudioJob::toString\033[0m\n");
	this->checkReferences();
	char* output = (char*)malloc(MAX_XML_LEN);
	if(toString(output, MAX_XML_LEN))
	{
		dprintf_full("\033[32mAudioJob::toString\033[0m\n");
		return output;
	}
	else
	{
		free(output);
		dprintf_err("\033[31mAudioJob::toString\033[0m\n");
		return NULL;
	}
}

bool AudioJob::fromString(const char* initString)
{
	dprintf_full("\033[32mAudioJob::fromString\033[0m\n");
	assert(initString);
	this->checkReferences();
	const char* next = initString;
	char* end = NULL;
		 
	if(next == strstr(next, AJOB_START))
	{
		next=(const char*)next+strlen(AJOB_START);
	}
	else
	{
		dprintf_err("\033[31mAudioJob::fromString\033[0m\n");
		return false;
	}
	if(1==sscanf(next, JOBID, &this->jobID))
	{
		next = (const char*)strchr(next, '\n')+1;
	}
	else
	{
		dprintf_err("\033[31mAudioJob::fromString\033[0m\n");
		return false;
	}
	if(next == strstr(next, NAME_SCAN))
	{
		if(NULL!=(end=strchr(next, '\n')))
		{
			next=(const char*)next+strlen(NAME_SCAN);
			memset(this->name, 0, MAX_STR_LEN);
			if(NULL==memcpy(this->name, next, end-next))
			{
				dprintf_err("\033[31mAudioJob::fromString\033[0m\n");
				return false;
	        	}
	        	else
	        	{
				next = (const char*)end+1;
			}
		}
		else
		{
			dprintf_err("\033[31mAudioJob::fromString\033[0m\n");
			return false;
		}
	}
	else
	{
		dprintf_err("\033[31mAudioJob::fromString\033[0m\n");
		return false;
	}
	if(next == strstr(next, FILENAME_SCAN))
	{
		if(NULL!=(end=strchr(next, '\n')))
	        {
		        memset(this->outFile, 0, MAX_STR_LEN);
		        next=(const char*)next+strlen(FILENAME_SCAN);
			if(NULL==memcpy(this->outFile, next, end-next))
		        {
				dprintf_err("\033[31mAudioJob::fromString\033[0m\n");
			        return false;
			}
			else
			{
				next = (const char*)end+1;
			}
		}
		else
		{
			dprintf_err("\033[31mAudioJob::fromString\033[0m\n");
			return false;
		}
	}
	else
	{
		dprintf_err("\033[31mAudioJob::fromString\033[0m\n");
		return false;
	}
	char* format = (char*)malloc(MAX_XML_LEN);
	memset(format, 0, MAX_XML_LEN);

	snprintf(format, MAX_XML_LEN, "%s%s%s%s%s%s", TARGET_BR, START_POS, 
	END_POS, SAMPLERATE, NUMCHANNELS, BITSPERSAMPLE);
	
	if(6 == sscanf(next, format, &this->targetBitRate, &this->startPosition,
	&this->endPosition, &this->sampleRate, &this->numChannels, 
	&this->bitsPerSample))
	{
        	next=strstr(next, LANGUAGE_SCAN);
		if(NULL!=(end=strchr(next, '\n')))
		{
		memset(this->language, 0, MAX_STR_LEN);
		next=(const char*)next+strlen(LANGUAGE_SCAN);
			if(NULL==memcpy(this->language, next, end-next))
			{
				free(format);
				dprintf_err("\033[31mAudioJob::fromString\033[0m\n");
				return false;
			}
			else
			{
				next = (const char*)end+1;
				if(0 == memcmp(next, AJOB_END, strlen(AJOB_END)))
			        {
				        free(format);
					dprintf_full("\033[32m<< AudioJob::fromString\033[0m\n");
					return true;
				}		
			
			}
		}
	}
	else
	{
		free(format);
		dprintf_err("\033[31mAudioJob::fromString\033[0m\n");
		return false;
	}
	
	free(format);
	dprintf_err("\033[31mAudioJob::fromString\033[0m\n");
	return false;

}

	
void AudioJob::setSampleRate(int sampleRate)
{
	dprintf_full("\033[32mAudioJob::setSampleRate\033[0m\n");
	this->checkReferences();
	this->sampleRate = sampleRate; 
}

void AudioJob::setNumChannels(int numChannels)
{
	dprintf_full("\033[32mAudioJob::setNumChannels\033[0m\n");
	this->checkReferences();
	this->numChannels = numChannels;
}


void AudioJob::setBitsPerSample(int b)
{
	dprintf_full("\033[32mAudioJob::setBitsPerSample\033[0m\n");
	this->checkReferences();
	this->bitsPerSample = b;
}

void AudioJob::setCodec(CodecID codec)
{
	dprintf_full("\033[32mAudioJob::setCodec\033[0m\n");
	this->checkReferences();
	this->codec = codec;
}

void AudioJob::setLanguage(const char* language)
{
	dprintf_full("\033[32mAudioJob::setLanguage\033[0m\n");
	assert(language);
	this->checkReferences();
	strncpy(this->language, language, MAX_STR_LEN-1);
}

Job* AudioJob::clone(Source* new_in_es)
{
	dprintf_full("\033[32m>> AudioJob::clone\033[0m\n");
	//assert(new_in_es);
	this->checkReferences();
	AudioJob* clone = new AudioJob(this->name, this->in_es, this->outFile,
	this->targetBitRate, this->sampleRate, this->numChannels, 
	this->bitsPerSample, this->codec, this->language, this->startPosition,
	this->endPosition);
	if(new_in_es != NULL)
	{
	clone->setSource(new_in_es);
	}
	dprintf_full("\033[32m<< AudioJob::clone\033[0m\n");
	return clone;
}


const char* AudioJob::getDescription()
{
	dprintf_full("\033[32mAudioJob::generateDescription\033[0m\n");
	char codecName[MAX_STR_LEN];
	strcpy(codecName,"Unknown");

#ifdef ENABLE_FFMPEG
	AVCodec* av=avcodec_find_decoder(this->codec);
	if(av)
		strcpy(codecName,av->name);
#endif

	snprintf((char*)description, MAX_STR_LEN, "%4ukBps; %5uHz; %3uBit; %1uChannel(s); Codec: %10s", 
					 this->targetBitRate, this->sampleRate,
					 this->bitsPerSample, this->numChannels, codecName);
	return description;
}
