/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   AudioJob.hpp                                                *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/


#ifndef __AUDIOJOB_HPP__
#define __AUDIOJOB_HPP__

#include "Job.hpp"
#include "adaptors/MP4Decoder.hpp"
#include "global.hpp"


class AudioJob:public Job {

protected:
	/** params of destination file */
	int sampleRate;
	int numChannels;
	int bitsPerSample;
	CodecID codec;
	char language[MAX_STR_LEN];

public:

	/** constructor */
	AudioJob();
	AudioJob(const char *name, Source* es, const char* outFile,
	u32 targetBitRate, int sampleRate, int numChannels, int bitsPerSample,
	CodecID codec, const char* language, float startPosition = 0, 
 	float endPosition = 0);
	
	/** destructor */
	virtual ~AudioJob();
	
	/** methods to get params of AudioJob */
	int getSampleRate() const;
	int getNumChannels() const;
	int getBitsPerSample() const;
	CodecID getCodec() const;
	const char* getLanguage() const;
	const char* getDescription();

	/** Implementation of virtual methods of Job */
	bool toString(char* buf, int max_len);
	char* toString();
	bool fromString(const char* initString);

	/** methods to set params of AudioJob */
	void setSampleRate(int sampleRate);
	void setNumChannels(int numChannels);
	void setBitsPerSample(int b);
	void setCodec(CodecID codec);
	void setLanguage(const char* language);
	
	/** clone job and set a new source */
	Job* clone(Source* new_in_es);
}; //class AudioJob
#endif
