#include "DestinationAudioImpl.hpp"

#include <stdio.h>
#include <stdlib.h>
#include <qwidget.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qspinbox.h>
#include <qtextedit.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qstring.h>
#include <qslider.h>
#include <qfiledialog.h>
#include <qlistview.h>
#include <qregexp.h>
#include <qnamespace.h>
#include <qcursor.h>

#include <qlineedit.h>
#include <qgroupbox.h>
#include <qapplication.h>
#include <qtable.h>
#include <iostream>



#include "avformat.h"
#include "global.hpp"
#include "JobList.hpp"
#include "Job.hpp"
#include "AudioJob.hpp"
#include "Project.hpp"
#include "Source.hpp"
#include "SourceList.hpp"
#include "SystemFile.hpp"
#include "VideoJob.hpp"
#include "ESInfo.hpp"
#include "VideoESInfo.hpp"
#include "adaptors/MP4Decoder.hpp"
#include "ContainerInfo.hpp"
#include "container/ffMP4ContainerFile.hpp"
#include "JobList.hpp"
#include "AudioESInfo.hpp"


/*****************************************************************************/
/*                                CODEC ARRAY                                */
/*****************************************************************************/

	static const char* codec_txt[]={ 				
	    "NONE", 
	    "MP2",		
	    "MP3",   			
	    "VORBIS",			
	    "AC3",			
	    "MPEG4",			
	    "MSMPEG4V1",		
	    "MSMPEG4V2",		
	    "MSMPEG4V3",		
	    "DVAUDIO",				
	    "WMAV1",				
	    "WMAV2",				
	    "MACE3",				
	    "MACE6",				
	    "AAC",				
	    "MPEG4AAC",				
	    "ROQ",				
	    "VMDAUDIO",				
	    "MSZH",			
	    "ZLIB",			
	    "PCM_S16LE",			
	    "PCM_S16BE",			
	    "PCM_U16LE",			
	    "PCM_U16BE",			
	    "PCM_S8",				
	    "PCM_U8",				
	    "PCM_MULAW",			
	    "PCM_ALAW",				
	    "ADPCM_IMA_QT",			
	    "ADPCM_IMA_WAV",			
	    "ADPCM_IMA_DK3",				
	    "ADPCM_IMA_DK4",			
	    "ADPCM_IMA_WS",			
	    "ADPCM_MS",				
	    "ADPCM_4XM",				
	    "ADPCM_XA",					
	    "ADPCM_ADX",			
	    "RA_144",				
	    "RA_288",				
	    "ROQ_DPCM",				
	    "INTERPLAY_DPCM",			
	    "XAN_DPCM",    			
	    "MPEG2TS",				
	    "UNKNOWN_CODEC"	
	  };


/*****************************************************************************/
/*                                KONSTRUCTOR                                */
/*****************************************************************************/

DestinationAudioImpl::DestinationAudioImpl(
                      AudioESInfo* esinfo,
		      JobList* globaljoblist,
		      bool newJob,
		      u32 job_id,
		      QWidget* parent,
		      const char* name,
		      bool modal,
		      WFlags fl)
:DestinationAudio(parent,name,modal,fl){
 	this->esinfo=esinfo;
	this->globaljoblist=globaljoblist;
	this->newJob=newJob;
	this->job_id=job_id;
	
	localjoblist=new JobList();
	init();
		
	//INFO OR EDIT
	if(!newJob){
	
		//EDIT
		if((int)job_id!=0){
			//EDIT audio job
			AudioJob* audiojob=(AudioJob*)localjoblist->getJob(job_id);
			if(audiojob!=NULL){
				const QString q_name(audiojob->getName());
				joblist->setCurrentItem(joblist->findItem (q_name));
			}//if  
		}//if

		else{
			//INFO source file
			set_disabled(true);			
		}//else
	}//if

	else{
		//NEW audio job	
		joblist->clearSelection();
	}//else

}//DestinationAudioImpl

/*****************************************************************************/
/*                                DESTRUCTOR                                 */
/*****************************************************************************/

DestinationAudioImpl::~DestinationAudioImpl(){

}//DestinationAudioImpl

/*****************************************************************************/
/*                               SET EN/DISABLED                             */
/*****************************************************************************/

void DestinationAudioImpl::set_disabled(bool ed){
	b_new_job->setDisabled(ed);
	b_update_job->setDisabled(ed);
	b_del_job->setDisabled(ed);
	b_save_joblist->setDisabled(ed);
	b_load_joblist->setDisabled(ed);	
	outputfile->setDisabled(ed);
	bitrate->setDisabled(ed);
	bitrate_kbit->setDisabled(ed);
	bitrate_mbit->setDisabled(ed);
	samplerate->setDisabled(ed);
	bpsamplerate->setDisabled(ed);
	channels->setDisabled(ed);
	codec->setDisabled(ed);
	language->setDisabled(ed);
	audio_s_e->setDisabled(ed);	
	b_reset->setDisabled(ed);
}//set_disabled

/*****************************************************************************/
/*                                INIT                                       */
/*****************************************************************************/

void DestinationAudioImpl::init(){
	int i_count;
	//const char* codec_name;


	if(esinfo->isAudioStream()){
		bitrate->setMinValue(1);
		bitrate->setMaxValue(65535);
		
		channels->setMinValue(1);
		channels->setMaxValue(65535);

	
		joblist->clear();
		codec->clear();
		samplerate->clear();
		bpsamplerate->clear();
		language->clear();
		

		//BITS PER SAMPLE
		memset(bpsamplearray,0,MAX_BPSAMPLE);
		
		bpsamplearray[0]=8;
		bpsamplearray[1]=16;
		bpsamplearray[2]=24;
		bpsamplearray[3]=32;


		for(i_count=0;i_count<MAX_BPSAMPLE;i_count++){
			bpsamplerate->insertItem((QString::number((long)bpsamplearray[i_count])),i_count);
		}//for


		//SAMPLERATE
		memset(samplearray,0,MAX_SAMPLE);
		
		samplearray[0]=8000;
		samplearray[1]=11025;
		samplearray[2]=12000;
		samplearray[3]=16000;
		samplearray[4]=22050;
		samplearray[5]=24000;
		samplearray[6]=32000;
		samplearray[7]=44100;
		samplearray[8]=48000;
		samplearray[9]=64000;
		samplearray[10]=88200;
		samplearray[11]=96000;
		
		for(i_count=0;i_count<MAX_SAMPLE;i_count++){
			samplerate->insertItem((QString::number((long)samplearray[i_count])),i_count);
		}//for
		
		
		//CODEC
		fill_in_codec();

		//LANGUAGE
		memset(languagearray,0,MAX_LANGUAGE);

		//Extension Language
		languagearray[0]="N/A";
		languagearray[1]="English";
		languagearray[2]="German";
			
		//allready changing the language is not provided
		//languagearray[1]="English";
		
		for(i_count=0;i_count<MAX_LANGUAGE;i_count++){
			QString q_language(languagearray[i_count]);
			language->insertItem(q_language,i_count);
		}//for

		u64 u64_duration=this->esinfo->getDurationInMs();
		
		audio_s_sec->setMinValue(0);
		audio_s_sec->setMaxValue((int)(u64_duration/1000));
		audio_s_ms->setMinValue(0);
		audio_s_ms->setMaxValue(999);
		
		audio_e_sec->setMinValue(0);
		audio_e_sec->setMaxValue((int)(u64_duration/1000));
		audio_e_ms->setMinValue(0);
		audio_e_ms->setMaxValue(999);

		//Initial values
		audio_s_sec->setValue(0);
		audio_s_ms->setValue(0);
		audio_e_sec->setValue((int)(u64_duration/1000));
		audio_e_ms->setValue(0);
		
		first_settings();
		write_jobs_to_joblist();
	}//if

	else{
		//No Audio Stream exit
		this->done(-1);
	}//else
	

}//init 

/*****************************************************************************/
/*                             FIRST SETTINGS                                */
/*****************************************************************************/

void DestinationAudioImpl::first_settings(){

	char* c_language;
	const char* c_codec_id;
	
	int i_bitsPerSecond;
	int i_channel;
	int i_samplerate;
	int i_bitsPerSample;
	int i_audio_e_sec;
	int i_audio_e_ms;

	QString q_file_name;
	CodecID codec_id;

	//NAME
	q_file_name=get_name_of_input();
		
	//BITRATE
	i_bitsPerSecond=(int)(esinfo->getAvgBandwidth());
		
	//SAMPLERATE
	//Read out from ESInfo currently not available
	//i_samplerate=(int)(esinfo->getSampleRate());
	i_samplerate=samplearray[0];

	//BITS PER SAMPLE
	//Read out from ESInfo currently not available
	//i_bitsPerSample=(int)(esinfo->getBitsPerSample());
	i_bitsPerSample=bpsamplearray[0];
	
	//CHANNELS
	i_channel=(int)(esinfo->getAudioChannels());

	//CODEC
	codec_id=(CodecID)(esinfo->getCodecID());
	c_codec_id=MP4Decoder::codecToString(codec_id);
	
	//LANGUAGE
	//esinfo does not allready provide "getLanguage()"
	c_language="N/A";

	//AUDIO START/END
	u64 u64_duration=this->esinfo->getDurationInMs();
	
	i_audio_e_sec=((int)(u64_duration/1000));
	i_audio_e_ms=(int)(((float)u64_duration-(float)i_audio_e_sec)*1000);

	//fill in the form
	fill_in_form(q_file_name.ascii(),
	             i_bitsPerSecond,
		     i_samplerate,
		     i_bitsPerSample,
		     i_channel,
		     c_codec_id,
		     c_language,
		     0,//i_audio_s_sec,
		     0,//i_audio_s_ms,
		     i_audio_e_sec,
		     i_audio_e_ms
		     );

}//first_settings

/*****************************************************************************/
/*                          WRITE JOBS TO JOBLIST                            */
/*****************************************************************************/
                     
void DestinationAudioImpl::write_jobs_to_joblist(){

	int i;
	AudioJob* audiojob;
	AudioESInfo* e_esinfo;
	

	
	for(i=0;(audiojob=(AudioJob*)globaljoblist->getJobByIndex(i))!=NULL;i++){
		Source* source;
		QString q_jobname(audiojob->getName());
		source=(Source*)(audiojob->getSource());
		if(source!=NULL){
			e_esinfo=(AudioESInfo*)source->getESInfo();
			if(memcmp(e_esinfo,esinfo,sizeof(e_esinfo))==0){
				joblist->insertItem(q_jobname,-1);
				localjoblist->addJob(audiojob);
			}//if
		
		}//if
	}//for
	if(audiojob!=NULL){
		audiojob->clean();
	}//if
	
}//write_jobs_to_joblist


/*****************************************************************************/
/*                                 NEW JOB                                   */
/*****************************************************************************/

void DestinationAudioImpl::new_job(){
	char n_job[MAX_STR_LEN];
	char nn_job[MAX_STR_LEN];
	int i_num=0;

	memset(n_job,0,MAX_STR_LEN);
	memset(nn_job,0,MAX_STR_LEN);
	if(strcmp(outputfile->text(),"")==0){
		strcpy(n_job,"NEWJOB");
		outputfile->setText(n_job);
	}//if
	else{
		strcpy(n_job,outputfile->text());
	}//else
	
	strcpy(nn_job,n_job);
	
	while(joblist->findItem(nn_job)!=0){
		i_num++;
		memset(nn_job,0,MAX_STR_LEN);
		sprintf(nn_job,"%s-%d",n_job,i_num);
	}//while
	

	joblist->insertItem (nn_job,-1);
	save_job(nn_job);
	joblist->setCurrentItem(joblist->findItem(nn_job));

	
}//new_job

/*****************************************************************************/
/*                                 SAVE JOB                                  */
/*****************************************************************************/

void DestinationAudioImpl::save_job(const char* c_outFile){
  	QString q_jobname;//= Name of the outputfile
	QString q2_jobname;
	QString q_samplerate;
	QString q_language;
	QString q_codec_id;
	
  	char* c_language;
	const char* c_codec_id;
	
	int i_channels;
	int i_samplerate;
	int i_bitspersample;
	int i_targetBitRate;
	int i_audio_s_sec;
	int i_audio_s_ms;
	int i_audio_e_sec;
	int i_audio_e_ms;

	float f_audio_s;
	float f_audio_e;

	
	CodecID codec_id;
  	AudioJob* audiojob;
    	Source* es;

 	//get ESINFO from GUI MultiTranscoder
  	es=new Source(esinfo);

	//BITRATE targetBitRate in bits per second
	if(bitrate_kbit->isOn()){
		i_targetBitRate=bitrate->value()*1024; 
 	}//if
	else{
		i_targetBitRate=bitrate->value()*1024*1024;
	}//else

	//SAMPLERATE
	i_samplerate=samplearray[samplerate->currentItem()];

	//BITS PER SAMPLE
	i_bitspersample=bpsamplearray[bpsamplerate->currentItem()];
	
	//CHANNELS
	i_channels=channels->value();


	//CODEC
	q_codec_id=codec->currentText();
	c_codec_id=(const char*)(q_codec_id.ascii());
	codec_id=(CodecID)(MP4Decoder::stringToCodec(c_codec_id));

	//LANGUAGE
	c_language=languagearray[language->currentItem()];

	//AUDIO START
	i_audio_s_sec=audio_s_sec->value();
	i_audio_s_ms=audio_s_ms->value();
	
	//AUDIO END
	i_audio_e_sec=audio_e_sec->value();
	i_audio_e_ms=audio_e_ms->value();

	f_audio_s=(float)i_audio_s_sec+(float)i_audio_s_ms/1000;
	f_audio_e=(float)i_audio_e_sec+(float)i_audio_e_ms/1000;

	//create the audio job
	audiojob=new AudioJob(c_outFile,
				        es,
					c_outFile,
					i_targetBitRate,
					i_samplerate,
					i_channels,
					i_bitspersample,
					codec_id,
					c_language,
					f_audio_s,
					f_audio_e
					);
					
	localjoblist->addJob(audiojob);

	
}//save_job

/*****************************************************************************/
/*                                UPDATE JOB                                 */
/*****************************************************************************/

void DestinationAudioImpl::update_job(){
 	QString q_jobname;//= Name of the outputfile
	QString q2_jobname;
	QString q_samplerate;
	QString q_language;
	QString q_codec_id;
	
  	char* c_language;
	const char* c_codec_id;
	
	int i_channels;
	int i_samplerate;
	int i_bitspersample;
	int i_targetBitRate;
	int i_audio_s_sec;
	int i_audio_s_ms;
	int i_audio_e_sec;
	int i_audio_e_ms;

	float f_audio_s;
	float f_audio_e;

	CodecID codec_id;
  	AudioJob* audiojob;

	//NAME
	q2_jobname=outputfile->text();
	q_jobname=joblist->currentText();

	if(q_jobname!=NULL){
	
		//BITRATE targetBitRate in bits per second
		if(bitrate_kbit->isOn()){
			i_targetBitRate=bitrate->value()*1024; 
	 	}//if
		else{
			i_targetBitRate=bitrate->value()*1024*1024;
		}//else

		//SAMPLERATE
		i_samplerate=samplearray[samplerate->currentItem()];
	
		//BITS PER SAMPLE
		i_bitspersample=bpsamplearray[bpsamplerate->currentItem()];
		
		//CHANNELS
		i_channels=channels->value();
	
		//CODEC
		q_codec_id=codec->currentText();
		c_codec_id=(const char*)(q_codec_id.ascii());
		codec_id=(CodecID)(MP4Decoder::stringToCodec(c_codec_id));
	
		//LANGUAGE
		c_language=languagearray[language->currentItem()];;

		//AUDIO START
		i_audio_s_sec=audio_s_sec->value();
		i_audio_s_ms=audio_s_ms->value();
	
		//AUDIO END
		i_audio_e_sec=audio_e_sec->value();
		i_audio_e_ms=audio_e_ms->value();

		f_audio_s=(float)i_audio_s_sec+(float)i_audio_s_ms/1000;
		f_audio_e=(float)i_audio_e_sec+(float)i_audio_e_ms/1000;


		audiojob=(AudioJob*)localjoblist->getJob(q_jobname);
		audiojob->setName(q2_jobname);
		joblist->changeItem(q2_jobname,joblist->index(joblist->selectedItem()));
	
		audiojob->setOutFileName(q2_jobname);
		audiojob->setBitRate(i_targetBitRate);

		audiojob->setSampleRate(i_samplerate);
		audiojob->setNumChannels(i_channels);
		audiojob->setBitsPerSample(i_bitspersample);
		audiojob->setCodec(codec_id);
		audiojob->setLanguage(c_language);

		audiojob->setStartPosition(f_audio_s);
		audiojob->setEndPosition(f_audio_e);

		joblist_selected();
	}//if

}//update_job

/*****************************************************************************/
/*                                 DEL JOB                                   */
/*****************************************************************************/

void DestinationAudioImpl::del_job(){
	QListBoxItem* boxitem;
	QString jobname;
	char* name;
	Job* job;
	unsigned int ui_number;

	ui_number=joblist->count();
	if(ui_number>0){

		if((boxitem=joblist->selectedItem())>0){
			jobname=boxitem->text();
			name=(char*)jobname.ascii();
				
			job=localjoblist->getJob(name);
			if(job!=NULL){
				localjoblist->delJob(job);
				joblist->removeItem(joblist->currentItem());
			}//if
			if(ui_number>1){
				joblist->setSelected(joblist->currentItem(),true);
			}//if
		}//if
			
	}//if
}//del_job

/*****************************************************************************/
/*                               SAVE JOBLIST                                */
/*****************************************************************************/

void DestinationAudioImpl::save_joblist(){
	char* c_jobs_in_list=(char*)malloc(MAX_STR_LEN);
	//malloc(c_jobs_in_list,0,MAX_STR_LEN);
	memset(c_jobs_in_list,0,MAX_STR_LEN);

	//FIXME WINDOWS!!
	QString name = QFileDialog::getSaveFileName("/home", 0,this,"save joblist","Write a joblistname" );
	
	if(name){
		assert(name);
		FILE* file = fopen(name,"w+");
		if(file){
			assert(file);
			c_jobs_in_list=localjoblist->toStringJobs();
			fputs(c_jobs_in_list,file);
			fclose(file);
		}//if
	}//if

	free(c_jobs_in_list);

}//save_joblist

/*****************************************************************************/
/*                               LOAD JOBLIST                                */
/*****************************************************************************/

void DestinationAudioImpl::load_joblist(){
	load_joblist_intern();
}//load_joblist

/*****************************************************************************/
/*                          LOAD JOBLIST INTERN                              */
/*****************************************************************************/

bool DestinationAudioImpl::load_joblist_intern(){
	AudioJob* audiojob;
	JobList* localloadjoblist;
	int i=0;
	
	localloadjoblist=new JobList();

	QString name = QFileDialog::getOpenFileName("/home", 0,this,"open file dialog","Choose a joblist" );

	if(!name){
		return false;
	}//if
	else{
		assert(name);
		FILE* file = fopen(name,"r");
		if(!file){
			
			return false;
		}//if
		else{
			assert(file);	
			char* buf = (char*)malloc(MAX_FILE_LEN);
			fread( (void*) buf, 1, MAX_FILE_LEN, file);
			if(localloadjoblist->fromStringJobs(buf)){
				
				for(i=0;(audiojob=(AudioJob*)localloadjoblist->getJobByIndex(i))!=NULL;i++){	
					//Rename job	
					QString q_jobname(audiojob->getName());
					QString add_jobname=get_name_of_input();
					q_jobname.insert(0,"-");
					q_jobname.insert(0,add_jobname);
					audiojob->setName(q_jobname);

					//Insert job to list and joblist
					joblist->insertItem(q_jobname,-1);
					localjoblist->addJob(audiojob);

				}//for
				fclose(file);				
				return true;
			}//if
			else{
				fclose(file);
				return false;
			}//else
		}//else
	}//else

 
	return false;

}//load_joblist_intern

/*****************************************************************************/
/*                             GET NAME OF INPUT                             */
/*****************************************************************************/

QString DestinationAudioImpl::get_name_of_input(){
	//NAME
	QString q_file_name;
	QString q2_file_name;
	int i_find_rev;
	int i2_find_rev;
	QString q_extention;
	const char* c_input;	
	
	c_input=esinfo->getInput();
	
	if(c_input!=NULL){
		QString q_input(c_input);
		i_find_rev=q_input.findRev('.');
		if(i_find_rev>0){
			q_extention=QString((char*)(q_input.ascii()+i_find_rev));
			
			*((char*)(q_input.ascii()+i_find_rev))=0;
			q_file_name=QString(q_input.ascii());
			
		}//if
		else{
			q_file_name=q_input;
		}//else
	}//if
	
	if(q_file_name!=NULL){
		i2_find_rev=q_file_name.findRev('/');
		while(i2_find_rev>0){
			q2_file_name=QString((char*)(q_file_name.ascii()+i2_find_rev+1));
			i2_find_rev=q2_file_name.findRev('/');
			
		}//while
	}//if
	q_file_name=q2_file_name;

	
	return q_file_name;

}//get_name_of_input

/*****************************************************************************/
/*                             JOBLIST SELECTED                              */
/*****************************************************************************/

void DestinationAudioImpl::joblist_selected(){
	QString jobname;

	int i_bitsPerSecond;
	int i_samplerate;
	int i_channels;
	int i_bitsPerSample;
	unsigned int ui_number;
	int i_audio_s_sec;
	int i_audio_s_ms;
	int i_audio_e_sec;
	int i_audio_e_ms;

	float f_audio_s;
	float f_audio_e;
		
	const char* c_language;
	const char* c_codec_id;
	const char* c_name;

	CodecID codec_id;	
	AudioJob* audiojob;
	
	ui_number=joblist->count();
	if(ui_number>0){
		jobname=joblist->currentText();
		c_name=(char*)jobname.ascii();
		audiojob=(AudioJob*)localjoblist->getJob(c_name);
		i_samplerate=audiojob->getSampleRate();
		
		i_channels=audiojob->getNumChannels();
		i_bitsPerSample=audiojob->getBitsPerSample();
		codec_id=audiojob->getCodec();
		c_codec_id=MP4Decoder::codecToString(codec_id);
		c_language=audiojob->getLanguage();
		i_bitsPerSecond=(int)audiojob->getBitRate();
		f_audio_s=audiojob->getStartPosition();
		f_audio_e=audiojob->getEndPosition();

		i_audio_s_sec=(int)f_audio_s;
		i_audio_e_sec=(int)f_audio_e;

		i_audio_s_ms=(int)(((float)i_audio_s_sec-f_audio_s)*1000);
		i_audio_e_ms=(int)(((float)i_audio_e_sec-f_audio_e)*1000);

		fill_in_form(c_name,
			     i_bitsPerSecond,
			     i_samplerate,
			     i_bitsPerSample,
			     i_channels,
			     c_codec_id,
			     c_language,
			     i_audio_s_sec,
			     i_audio_s_ms,
			     i_audio_e_sec,
			     i_audio_e_ms
			     );

		if(audiojob!=NULL){
			
			audiojob->clean();
			
		}//if

	}//if

}//joblist_selected

/*****************************************************************************/
/*                                  CANCEL                                   */
/*****************************************************************************/

void DestinationAudioImpl::cancel(){
	this->done(-1);
}//cancel

/*****************************************************************************/
/*                                   SAVE                                    */
/*****************************************************************************/

void DestinationAudioImpl::save(){

	int i;
	const char* c_outFile;
	QString q_outFile;
	AudioJob* audiojob;
	AudioESInfo* e_esinfo;

	q_outFile=outputfile->text();
	c_outFile=q_outFile.ascii();

	if(this->newJob){

		if((joblist->currentItem())>=0){
			update_job();
		}//if
		else{
			new_job();
		}//else

	
		for(i=0;(audiojob=(AudioJob*)globaljoblist->getJobByIndex(i))!=NULL;i++){
			Source* source;
			QString q_jobname(audiojob->getName());
			source=(Source*)(audiojob->getSource());
			if(source!=NULL){
				e_esinfo=(AudioESInfo*)source->getESInfo();
				if(memcmp(e_esinfo,this->esinfo,sizeof(e_esinfo))==0){
					
					globaljoblist->delJob(audiojob);
					i--;
				}//if		
			}//if
		}//for

		for(i=0;(audiojob=(AudioJob*)localjoblist->getJobByIndex(i))!=NULL;i++){
			globaljoblist->addJob((Job*)audiojob);
			
		}//for

		if(audiojob!=NULL){
			audiojob->clean();
			
		}//if		
	}//if

	this->done(0);

}//save

/*****************************************************************************/
/*                               FILL IN FORM                                */
/*****************************************************************************/

void DestinationAudioImpl::fill_in_form(const char* c_name,
					int i_bitsPerSecond,
					int i_samplerate,
					int i_bitsPerSample,
					int i_channels,
					const char* c_codec_id,
					const char* c_language,
					int i_audio_s_sec,
					int i_audio_s_ms,
					int i_audio_e_sec,
					int i_audio_e_ms
					){

					
		int i_targetBitRate;	
		int i;
		QString q_name(c_name);
		QString q_language(c_language);
				
		//NAME		
		outputfile->setText(q_name);
		
		//BITRATE
		if(i_bitsPerSecond<=(1024*1024)){
			bitrate_kbit->setChecked(true);
			i_targetBitRate=i_bitsPerSecond/1024;
		}//if
		else{
			bitrate_mbit->setChecked(true);
			i_targetBitRate=i_bitsPerSecond/1024/1024;
		}//else
		bitrate->setValue(i_targetBitRate);

		//SAMPLERATE
		for(i=0;i<MAX_SAMPLE;i++){
			if(i_samplerate==samplearray[i]){
				samplerate->setCurrentItem(i);
				i=MAX_SAMPLE;
			}//if
		}//for
	
		//BITS PER SAMPLE
		for(i=0;i<MAX_BPSAMPLE;i++){
			if(i_bitsPerSample==bpsamplearray[i]){
				bpsamplerate->setCurrentItem(i);
				i=MAX_BPSAMPLE;
			}//if
		}//for

		//CHANNELS
		channels->setValue(i_channels);

		//CODEC
		for(i=0;i<codec->count();i++){
			codec->setCurrentItem(i);
			if(strcmp((codec->currentText()).ascii(),c_codec_id)==0){
				i=codec->count();
			}//if

		}//if

				
		//LANGUAGE
		language->setCurrentItem(0);
		for(i=0;i<MAX_LANGUAGE;i++){
			if(strcmp(c_language,languagearray[i])==0){
				language->setCurrentItem(i);
				i=MAX_LANGUAGE;
			}//if
		}//for	

		//AUDIO START
		audio_s_sec->setValue(i_audio_s_sec);
		audio_s_ms->setValue(i_audio_s_ms);
				
		//AUDIO END
		audio_e_sec->setValue(i_audio_e_sec);
		audio_e_ms->setValue(i_audio_e_ms);
}//fill_in_form

/*****************************************************************************/
/*                                RESET FORM                                 */
/*****************************************************************************/

void DestinationAudioImpl::reset_form(){
	first_settings();
}//reset_form

/*****************************************************************************/
/*                        OUTPUTFILE RETURN PRESSED                          */
/*****************************************************************************/

void DestinationAudioImpl::outputfile_returnPressed(){
	outputfile->doKeyboardAction(QTextEdit::ActionBackspace);
	new_job();
}//outputfile_returnPressed

/*****************************************************************************/
/*                           OUTPUTFILE TEXT CHANGED                         */
/*****************************************************************************/

void DestinationAudioImpl::outputfile_textChanged(){
	int i=-1;

	QString q_file_name=outputfile->text();
	
	i=q_file_name.find(QRegExp("\t"), 0 );

	if(i>=0){
		q_file_name.remove(i,1);
		outputfile->setText(q_file_name);
		//outputfile->unsetCursor();
	}//if

	i=-1;
	i=q_file_name.find(QRegExp("/"), 0 );
	if(i==0){
		q_file_name.remove(i,1);
		outputfile->setText(q_file_name);
	}//if
	
	//bitrate->setCursor(ArrowCursor);
	
}//outputfile_textChanged


/*****************************************************************************/
/*                              FILL IN CODEC                                */
/*****************************************************************************/

void DestinationAudioImpl::fill_in_codec(){
	int i;	
	
	for(i=0;codec_txt[i]!="UNKNOWN_CODEC";i++){
		codec->insertItem(codec_txt[i],-1);
	}//for    
	//codec->insertItem(codec_txt[i]);
}//fill_in_codec

/*****************************************************************************/
/*                               SET CODEC ID                                */
/*****************************************************************************/

void DestinationAudioImpl::set_Codec_ID(CodecID cid){
	const char* codec_name;
	int i;
	
	codec_name=MP4Decoder::codecToString(cid);

	for(i=0;codec_txt[i]!="UNKNOWN_CODEC";i++){
		if(strcmp(codec_name,codec_txt[i])==0){
			codec->setCurrentItem(i);
		}//if
	}//for    
}//set_Codec_ID
