#include "DestinationVideoImpl.hpp"

#include <stdio.h>
#include <stdlib.h>
#include <qwidget.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qspinbox.h>
#include <qtextedit.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qstring.h>
#include <qslider.h>
#include <qfiledialog.h>
#include <qlistview.h>
#include <qregexp.h>
#include <qnamespace.h>
#include <qcursor.h>
#include <qlineedit.h>
#include <qgroupbox.h>
#include <qapplication.h>
#include <qtable.h>
#include <iostream>


#include "avformat.h"
#include "global.hpp"
#include "JobList.hpp"
#include "Job.hpp"
#include "Project.hpp"
#include "Source.hpp"
#include "SourceList.hpp"
#include "SystemFile.hpp"
#include "VideoJob.hpp"
#include "ESInfo.hpp"
#include "VideoESInfo.hpp"
#include "adaptors/MP4Decoder.hpp"
#include "ContainerInfo.hpp"
#include "container/ffMP4ContainerFile.hpp"
#include "UserDefineImpl.hpp"


/*****************************************************************************/
/*                               CODEC ARRAY                                 */
/*****************************************************************************/

//Extension Codec
	static const char* codec_txt[]={ 				
	    "NONE", 				
	    "MPEG1VIDEO",		
	    "MPEG2VIDEO",		
	    "MPEG2VIDEO_XVMC",			
	    "H263",			
	    "RV10",			
	    "RV20",			
	    "MJPEG",			
	    "MJPEGB",				
	    "LJPEG",			
	    "SP5X",			
	    "MPEG4",			
	    "RAWVIDEO",			
	    "MSMPEG4V1",		
	    "MSMPEG4V2",		
	    "MSMPEG4V3",		
	    "WMV1",			
	    "WMV2",			
	    "H263P",				
	    "H263I",			
	    "FLV1",			
	    "SVQ1",			
	    "SVQ3",			
	    "DVVIDEO",			
	    "HUFFYUV",			
	    "CYUV",			
	    "H264",			
	    "INDEO3",			
	    "VP3",			
	    "THEORA",			
	    "ASV1",			
	    "ASV2",			
	    "FFV1",			
	    "4XM",			
	    "VCR1",			
	    "CLJR",			
	    "MDEC",			
	    "INTERPLAY_VIDEO",		
	    "XAN_WC3",			
	    "RPZA",			
	    "CINEPAK",			
	    "MSRLE",			
	    "MSVIDEO1",			
	    "8BPS",			
	    "SMC",			
	    "TRUEMOTION1",		
	    "FLIC"			
	    "VMDVIDEO",			
	    "MSZH",			
	    "ZLIB",			
	    "AMR_NB",			
	    "AMR_WB",			
	    "MPEG2TS",				
	    "UNKNOWN_CODEC"	
	  };


/*****************************************************************************/
/*                                KONSTRUCTOR                                */
/*****************************************************************************/

DestinationVideoImpl::DestinationVideoImpl(
		      VideoESInfo* esinfo,
		      JobList* globaljoblist,
		      bool newJob,
		      u32 job_id,
		      QWidget* parent,
		      const char* name,
		      bool modal,
		      WFlags fl)
:DestinationVideo(parent,name,modal,fl){
 	this->esinfo=esinfo;
	this->globaljoblist=globaljoblist;
	this->newJob=newJob;
	this->job_id=job_id;
	this->current_index_size=0;
	this->current_aspect_ratio=this->esinfo->getAspectRatio();

	localjoblist=new JobList();
	init();
		
	//INFO OR EDIT
	if(!newJob){
	
		//EDIT
		if(job_id!=0){
			//EDIT video job	
			VideoJob* videojob=(VideoJob*)localjoblist->getJob(job_id);
			if(videojob!=NULL){
				const QString q_name(videojob->getName());
				joblist->setCurrentItem(joblist->findItem (q_name));
			}//if  
		}//if

		else{
			//INFO source file
			set_disabled(true);
			
		}//else
	}//if

	else{
		//NEW video job
		joblist->clearSelection();
	}//else

}//DestinationVideoImpl


/*****************************************************************************/
/*                                DESTRUCTOR                                 */
/*****************************************************************************/

DestinationVideoImpl::~DestinationVideoImpl(){

}//DestinationVideoImpl

/*****************************************************************************/
/*                              SET EN/DISABLED                              */
/*****************************************************************************/

void DestinationVideoImpl::set_disabled(bool ed){
	b_new_job->setDisabled(ed);
	b_update_job->setDisabled(ed);
	b_del_job->setDisabled(ed);
	b_save_list->setDisabled(ed);
	b_load_list->setDisabled(ed);
	outputfile->setDisabled(ed);
	bitrate->setDisabled(ed);
	bitrate_kbit->setDisabled(ed);
	bitrate_mbit->setDisabled(ed);
	bitrate_min->setDisabled(ed);
	bitrate_max->setDisabled(ed);
	size->setDisabled(ed);
	codec->setDisabled(ed);
	color->setDisabled(ed);
	b_reset->setDisabled(ed);
	quantization->setDisabled(ed);
	gop->setDisabled(ed);
	video_packets->setDisabled(ed);
	video_s_e->setDisabled(ed);
}//set_disabled

/*****************************************************************************/
/*                                   INIT                                    */
/*****************************************************************************/

void DestinationVideoImpl::init(){
	char c_size[MAX_STR_LEN]; 
	//const char* codec_name;
	int i_count;

	if(esinfo->isVisualStream()){
		bitrate->setMinValue(1);
		bitrate->setMaxValue(65535);
		
		bitrate_min->setMinValue(1);
		bitrate_min->setMaxValue(65535);
		
		bitrate_max->setMinValue(bitrate_min->value());
		bitrate_max->setMaxValue(65535); 
		
		macro_min->setMinValue(0);
		macro_min->setMaxValue(31);		
		macro_max->setMinValue(macro_min->value());
		macro_max->setMaxValue(31);
		
		quanti_min->setMinValue(0);
		quanti_min->setMaxValue(31);		
		quanti_max->setMinValue(quanti_min->value());
		quanti_max->setMaxValue(31);
		
		i_frame->setMinValue(0);
		i_frame->setMaxValue(31);
		p_frame->setMinValue(0);
		p_frame->setMaxValue(31);
		b_frame->setMinValue(0);
		b_frame->setMaxValue(31);
		
		gop_size->setMinValue(0);
		gop_size->setMaxValue(255);
		
		b_frame_slide->setMinValue(0);
		b_frame_slide->setMaxValue(8);	

		video_pac_size->setMinValue(0);
		video_pac_size->setMaxValue(255);
		dec_buffer_size->setMinValue(0);
		dec_buffer_size->setMaxValue(255);

		u64 u64_duration=this->esinfo->getDurationInMs();

		video_s_sec->setMinValue(0);
		video_s_sec->setMaxValue((int)(u64_duration/1000));
		video_s_ms->setMinValue(0);
		video_s_ms->setMaxValue(999);
		
		video_e_sec->setMinValue(0);
		video_e_sec->setMaxValue((int)(u64_duration/1000));
		video_e_ms->setMinValue(0);
		video_e_ms->setMaxValue(999);

		joblist->clear();
		size->clear();
		codec->clear();
		
		//SIZE
		memset(w_size_array,0,MAX_ARRAY);
		memset(h_size_array,0,MAX_ARRAY);
		memset(c_size,0,MAX_STR_LEN);

		//w_size_array[0] holds the index of the next free entry
		w_size_array[1]=176;
		w_size_array[2]=352;
		w_size_array[3]=512;
		w_size_array[4]=768;

		h_size_array[1]=144;
		h_size_array[2]=288;
		h_size_array[3]=384;
		h_size_array[4]=576;

	
		size->insertItem("USER DEFINE",0);
		for(i_count=1;(i_count<MAX_ARRAY)&&(w_size_array[i_count]!=0);i_count++){
			sprintf(c_size,"%i x %i",w_size_array[i_count],h_size_array[i_count]);
			size->insertItem((QString)c_size,i_count);
			w_size_array[0]=i_count+1;
		}//for


		//CODEC
   		fill_in_codec();

		first_settings();
		write_jobs_to_joblist();
		fixed_quantisation->setChecked(false);
		fixed_quantisation_clicked();
		//free(c_size);
	}//if

	else{
		//DestinationVideoImpl: No Video Stream
		this->done(-1);
	}//else
}//init

/*****************************************************************************/
/*                             GET NAME OF INPUT                             */
/*****************************************************************************/

QString DestinationVideoImpl::get_name_of_input(){
	//NAME
	QString q_file_name;
	QString q2_file_name;
	int i_find_rev;
	int i2_find_rev;
	QString q_extention;
	const char* c_input;	
	
	c_input=esinfo->getInput();
	
	if(c_input!=NULL){
		QString q_input(c_input);
		i_find_rev=q_input.findRev('.');
		if(i_find_rev>0){
			q_extention=QString((char*)(q_input.ascii()+i_find_rev));
			
			*((char*)(q_input.ascii()+i_find_rev))=0;
			q_file_name=QString(q_input.ascii());
			
		}//if
		else{
			q_file_name=q_input;
		}//else
	}//if
	
	if(q_file_name!=NULL){
		i2_find_rev=q_file_name.findRev('/');
		while(i2_find_rev>0){
			q2_file_name=QString((char*)(q_file_name.ascii()+i2_find_rev+1));
			i2_find_rev=q2_file_name.findRev('/');
			
		}//while
	}//if
	q_file_name=q2_file_name;
	
	
	return q_file_name;

}//get_name_of_input

/*****************************************************************************/
/*                              FIRST SETTINGS                               */
/*****************************************************************************/

void DestinationVideoImpl::first_settings(){

	int i_bitsPerSecond;
	int i_bitrate_min;
	int i_bitrate_max;
	//int i_streamid;
	int i_width;
	int i_height;
	int i_gop_size;
	int i_fps;
	int i_b_frame_slide;
	int i_video_e_sec;
	int i_video_e_ms;
	
	float f_aspect_ratio;
	bool b_color;	
	CodecID codec_id;
	
	QString q_file_name;

	//NAME
	q_file_name=get_name_of_input();

	
	//BITRATE
	i_bitsPerSecond=esinfo->getAvgBandwidth();
	
	//Extension Bitrate min max
	i_bitrate_min=i_bitsPerSecond;
	i_bitrate_max=i_bitsPerSecond;

	//SIZE
	i_width=esinfo->getWidth();
	i_height=esinfo->getHeight();

	//CODEC
	codec_id=esinfo->getCodecID();

	//COLOR
	b_color=esinfo->getHasColor();

	//GOP SIZE
	i_gop_size=(int)esinfo->getGOP_size();

	//FPS
	i_fps=(int)esinfo->getFPS();

	//ASPECT RATIO
	f_aspect_ratio=esinfo->getAspectRatio();
		
	//NUMBER OF B FRAMES
	i_b_frame_slide=esinfo->getNum_B_frames();
	
	//VIDEO START/END
	u64 u64_duration=this->esinfo->getDurationInMs();

	i_video_e_sec=(int)(u64_duration/1000);
	i_video_e_ms=(int)(((float)u64_duration-(float)i_video_e_sec)*1000);
		
	//fill in the form
	fill_in_form(q_file_name.ascii(),
                     i_bitsPerSecond,
		     i_bitrate_min, 
		     i_bitrate_max,
		     i_width,
		     i_height,
		     codec_id,
		     f_aspect_ratio,
		     b_color,
		     12,//int i_macro_min,
		     12,//int i_macro_max,
		     12,//int i_quanti_min,
		     12,//int i_quanti_max,
		     12,//int i_I_frame,
		     16,//int i_P_frame,
		     16,//int i_B_frame,
		     false,//bool b_var_gop,
		     false,//bool b_only_i_frame,
		     false,//bool b_var_frame_rate,
		     i_gop_size,
		     i_b_frame_slide,
		     i_fps, 	
		     0,//int i_dec_buffer_size,
		     false,//bool i_en_video_pac,
		     0, //int i_video_pac_size,
		     0, //int i_video_s_sec,
		     0, //int i_video_s_ms,
		     i_video_e_sec,
		     i_video_e_ms
		     );
}//first_settings

/*****************************************************************************/
/*                           WRITE JOBS TO JOBLIST                           */
/*****************************************************************************/

void DestinationVideoImpl::write_jobs_to_joblist(){

	int i;
	VideoJob* videojob;
	VideoESInfo* e_esinfo;
	
	for(i=0;(videojob=(VideoJob*)globaljoblist->getJobByIndex(i))!=NULL;i++){
		Source* source;
		QString q_jobname(videojob->getName());
		source=(Source*)(videojob->getSource());
		if(source!=NULL){
			e_esinfo=(VideoESInfo*)source->getESInfo();
			if(memcmp(e_esinfo,esinfo,sizeof(e_esinfo))==0){
				joblist->insertItem(q_jobname,-1);
				localjoblist->addJob(videojob);


			}//if
		
		}//if
	}//for
	if(videojob!=NULL){
		videojob->clean();
	}//if

}//write_jobs_to_joblist

/*****************************************************************************/
/*                                 NEW JOB                                   */
/*****************************************************************************/

void DestinationVideoImpl::new_job(){
	char n_job[MAX_STR_LEN];
	char nn_job[MAX_STR_LEN];
	int i_num=0;

	memset(n_job,0,MAX_STR_LEN);
	memset(nn_job,0,MAX_STR_LEN);
	if(strcmp(outputfile->text(),"")==0){
		strcpy(n_job,"NEWJOB");
		outputfile->setText(n_job);
	}//if
	else{
		strcpy(n_job,outputfile->text());
	}//else
	
	strcpy(nn_job,n_job);
		
	while(joblist->findItem(nn_job)!=0){		
		i_num++;
		memset(nn_job,0,MAX_STR_LEN);
		sprintf(nn_job,"%s-%d",n_job,i_num);
	}//while
	
	
	joblist->insertItem (nn_job,-1);
	save_job(nn_job);
	joblist->setCurrentItem(joblist->findItem(nn_job));

	//free(n_job);
	//free(nn_job);
	
}//new_job

/*****************************************************************************/
/*                                 SAVE JOB                                  */
/*****************************************************************************/

void DestinationVideoImpl::save_job(const char* c_outFile){
  	QString q_jobname;//= Name of the outputfile
	QString q2_jobname;
	QString q_language;
	QString q_size;
	QString q_aspect_ratio;
	QString q_codec_id;
	
	const char* c_codec_id;
	char* c_size;
	//char* c_aspect_ratio;

	unsigned int i_targetBitRate;
	unsigned int i_bitrate_range_min;
	unsigned int i_bitrate_range_max;
	
	int i_macro_min;
	int i_macro_max;
	int i_quanti_min;
	int i_quanti_max;
	int i_I_frame;
	int i_P_frame;
	int i_B_frame;
	int i_b_frame_slide;
	int i_gop_size;
	int i_width;
	int i_height;
	int i_dec_buffer_size;
	int i_video_pac_size;
	int i_video_s_sec;
	int i_video_s_ms;
	int i_video_e_sec;
	int i_video_e_ms;
	
	float f_video_s;
	float f_video_e;
	float f_fps;

	bool b_color=false;
	bool b_var_gop=false;
	bool b_only_I_frame=false;
	bool b_var_frame_rate=false;
	bool b_en_video_pac=false;

	//Size s_size;
	CodecID codec_id;
  	VideoJob* videojob;
    	Source* es;

	//get ESINFO from GUI MultiTranscoder
  	es=new Source(esinfo);
	
	
	//BITRATE + AVERAGE BITRATE targetBitRate in bits per second
	if(bitrate_kbit->isOn()){
		i_targetBitRate=bitrate->value()*1024; 
		i_bitrate_range_min=bitrate_min->value()*1024;
		i_bitrate_range_max=bitrate_max->value()*1024;
	}//if
	else{
		i_targetBitRate=bitrate->value()*1024*1024;
		i_bitrate_range_min=bitrate_min->value()*1024*1024;
		i_bitrate_range_max=bitrate_max->value()*1024*1024;
	}//else
	if(i_bitrate_range_min>i_targetBitRate){
		i_bitrate_range_min=i_targetBitRate;
	}//if
	if(i_bitrate_range_max<i_targetBitRate){
		i_bitrate_range_max=i_targetBitRate;
	}//if
	
	
	//SIZE	
	q_size=size->currentText();
	c_size=(char*)(q_size.ascii());
	sscanf(c_size,"%i x %i",&i_width,&i_height);
	
	//CODEC
	q_codec_id=codec->currentText();
	c_codec_id=(const char*)(q_codec_id.ascii());
	codec_id=(CodecID)(MP4Decoder::stringToCodec(c_codec_id));

	//COLOR
	if(color->isChecked()){
		b_color=true;
	}//if

	//QUANTIZATION
	i_macro_min=macro_min->value();
	i_macro_max=macro_max->value();
	i_quanti_min=quanti_min->value();
	i_quanti_max=quanti_max->value();
	i_I_frame=i_frame->value();
	i_P_frame=p_frame->value();
	i_B_frame=b_frame->value();
	
	//GOP
	if(var_gop->isChecked()){
		b_var_gop=true;
	}//if
	if(only_i_frame->isChecked()){
		b_only_I_frame=true;
	}//if
	if(var_frame_rate->isChecked()){
		b_var_frame_rate=true;
	}//if
	
	i_b_frame_slide=b_frame_slide->value();
	i_gop_size=gop_size->value();
	f_fps=(float)(fps->value());
	
	//VIDEO PACKET
	i_dec_buffer_size=dec_buffer_size->value();
	i_video_pac_size=video_pac_size->value();
	
	if(en_video_pac->isChecked()){
		b_en_video_pac=true;
	}//if	

	//VIDEO
	i_video_s_sec=video_s_sec->value();
	i_video_s_ms=video_s_ms->value();
	i_video_e_sec=video_e_sec->value();
	i_video_e_ms=video_e_ms->value();


	f_video_s=(float)i_video_s_sec+(float)i_video_s_ms/1000;
	f_video_e=(float)i_video_e_sec+(float)i_video_e_ms/1000;
	
	

	//create the videojob
	videojob=new VideoJob(c_outFile,
	                      es,
			      c_outFile,
			      i_targetBitRate,
			      i_bitrate_range_min,
			      i_bitrate_range_max,	  
			      i_width,
			      i_height,
			      f_fps,
			      b_var_frame_rate,
			      b_color,
			      codec_id,
			      this->current_aspect_ratio,
			      i_macro_min,
			      i_macro_max,
			      i_quanti_min,
			      i_quanti_max,
			      i_I_frame,
			      i_B_frame,
			      i_P_frame,
			      b_var_gop,
			      i_gop_size,
			      b_en_video_pac,
			      i_video_pac_size,
			      i_b_frame_slide,
			      b_only_I_frame,
			      i_dec_buffer_size,
			      f_video_s,
			      f_video_e
			      );

	localjoblist->addJob(videojob);
	
	//videojob->clean();

}//save_job

/*****************************************************************************/
/*                                UPDATE JOB                                 */
/*****************************************************************************/

void DestinationVideoImpl::update_job(){
  	QString q_jobname;//= Name of the outputfile
	QString q2_jobname;
	QString q_language;
	QString q_size;
	QString q_aspect_ratio;
	QString q_codec_id;
	
	const char* c_codec_id;
	char* c_size;

	unsigned int i_targetBitRate;
	unsigned int i_bitrate_range_min;
	unsigned int i_bitrate_range_max;
	
	int i_macro_min;
	int i_macro_max;
	int i_quanti_min;
	int i_quanti_max;
	int i_I_frame;
	int i_P_frame;
	int i_B_frame;
	int i_b_frame_slide;
	int i_gop_size;
	int i_width;
	int i_height;
	int i_dec_buffer_size;
	int i_video_pac_size;
	int i_video_s_sec;
	int i_video_s_ms;
	int i_video_e_sec;
	int i_video_e_ms;

	float f_video_s;
	float f_video_e;
	float f_fps;

	bool b_color=false;
	bool b_var_gop=false;
	bool b_only_I_frame=false;
	bool b_var_frame_rate=false;
	bool b_en_video_pac=false;

	//Size s_size;
	CodecID codec_id;
  	VideoJob* videojob;
    		

	
	//NAME
	q2_jobname=outputfile->text();
	q_jobname=joblist->currentText();

	if(q_jobname!=NULL){

	
		//BITRATE + AVERAGE BITRATE targetBitRate in bits per second
		if(bitrate_kbit->isOn()){
			i_targetBitRate=bitrate->value()*1024; 
			i_bitrate_range_min=bitrate_min->value()*1024;
			i_bitrate_range_max=bitrate_max->value()*1024;
		}//if
		else{
			i_targetBitRate=bitrate->value()*1024*1024;
			i_bitrate_range_min=bitrate_min->value()*1024*1024;
			i_bitrate_range_max=bitrate_max->value()*1024*1024;
		}//else
		
		if(i_bitrate_range_min>i_targetBitRate){
			i_bitrate_range_min=i_targetBitRate;
		}//if
		if(i_bitrate_range_max<i_targetBitRate){
			i_bitrate_range_max=i_targetBitRate;
		}//if
	
		//SIZE	
		q_size=size->currentText();
		c_size=(char*)(q_size.ascii());
		sscanf(c_size,"%i x %i",&i_width,&i_height);
		
		//CODEC
		q_codec_id=codec->currentText();
		c_codec_id=(const char*)(q_codec_id.ascii());
		codec_id=(CodecID)(MP4Decoder::stringToCodec(c_codec_id));
	
	
		//COLOR
		if(color->isChecked()){
			b_color=true;
		}//if
	
		//QUANTIZATION
		i_macro_min=macro_min->value();
		i_macro_max=macro_max->value();
		i_quanti_min=quanti_min->value();
		i_quanti_max=quanti_max->value();
		i_I_frame=i_frame->value();
		i_P_frame=p_frame->value();
		i_B_frame=b_frame->value();
		
		//GOP
		if(var_gop->isChecked()){
			b_var_gop=true;
		}//if
		if(only_i_frame->isChecked()){
			b_only_I_frame=true;
		}//if
		if(var_frame_rate->isChecked()){
			b_var_frame_rate=true;
		}//if
	
		i_b_frame_slide=b_frame_slide->value();
		i_gop_size=gop_size->value();
		f_fps=(float)(fps->value());
	
		//VIDEO PACKET
		i_dec_buffer_size=dec_buffer_size->value();
		i_video_pac_size=video_pac_size->value();
	
		if(en_video_pac->isChecked()){
			b_en_video_pac=true;
		}//if

		//VIDEO START
		i_video_s_sec=video_s_sec->value();
		i_video_s_ms=video_s_ms->value();

		//VIDEO END
		i_video_e_sec=video_e_sec->value();
		i_video_e_ms=video_e_ms->value();

		f_video_s=(float)i_video_s_sec+(float)i_video_s_ms/1000;
		f_video_e=(float)i_video_e_sec+(float)i_video_e_ms/1000;
	
		videojob=(VideoJob*)localjoblist->getJob(q_jobname);
		videojob->setName(q2_jobname);
		videojob->setAspectRatio(this->current_aspect_ratio);
		joblist->changeItem(q2_jobname,joblist->index(joblist->selectedItem()));

		videojob->setBitRate(i_targetBitRate); 	
		videojob->setVariableGop(b_var_gop); 
		videojob->setGopSize(i_gop_size);
	
		videojob->setMinBitrate(i_bitrate_range_min);
		videojob->setMaxBitrate(i_bitrate_range_max);
	
		//QString q_size
		videojob->setWidth(i_width);
		videojob->setHeight(i_height);

		videojob->setFPS(f_fps);
		videojob->setVariableFR(b_var_frame_rate);
		videojob->setColor(b_color);
		videojob->setCodec(codec_id);
		
		videojob->setMinMacroBlock(i_macro_min);
		videojob->setMaxMacroBlock(i_macro_max);
		videojob->setMinQuantization(i_quanti_min);
		videojob->setMaxQuantization(i_quanti_max);
		videojob->setQuantI(i_I_frame);
		videojob->setQuantB(i_B_frame);
		videojob->setQuantP(i_P_frame);
		videojob->setNumBFrames(i_b_frame_slide);
		videojob->setIFramesOnly(b_only_I_frame);
		videojob->setDecBufferSize(i_dec_buffer_size);
		videojob->setEnableVideoPackets(b_en_video_pac);
		videojob->setVideoPacketSize(i_video_pac_size);

		videojob->setStartPosition(f_video_s);
		videojob->setEndPosition(f_video_e);


		joblist_selected();

	}//if
	

}//update_job

/*****************************************************************************/
/*                                 DEL JOB                                   */
/*****************************************************************************/

void DestinationVideoImpl::del_job(){
	QListBoxItem* boxitem;
	QString jobname;
	char* name;
	Job* job;
	unsigned int ui_number;
	
	ui_number=joblist->count();
	if(ui_number>0){

		if((boxitem=joblist->selectedItem())>0){
			jobname=boxitem->text();
			name=(char*)jobname.ascii();
				
	
			job=localjoblist->getJob(name);
			if(job!=NULL){
				localjoblist->delJob(job);
				joblist->removeItem(joblist->currentItem());
			}//if

			if(ui_number>1){
				joblist->setSelected(joblist->currentItem(),true);
			}//if
		}//if
		
	}//if
}//del_job

/*****************************************************************************/
/*                                SAVE JOBLIST                               */
/*****************************************************************************/

void DestinationVideoImpl::save_joblist(){
	char* c_jobs_in_list=(char*)malloc(MAX_STR_LEN);
	//malloc(c_jobs_in_list,0,MAX_STR_LEN);
	memset(c_jobs_in_list,0,MAX_STR_LEN);

	//FIXME WINDOWS!!
	QString name = QFileDialog::getSaveFileName("/home", 0,this,"save joblist","Write a joblistname" );
	if(name){
		assert(name);
		FILE* file = fopen(name,"w+");
		if(file){
			assert(file);
			c_jobs_in_list=localjoblist->toStringJobs();
			fputs(c_jobs_in_list,file);
			fclose(file);
		}//if
	}//if
	free(c_jobs_in_list);

}//save_joblist

/*****************************************************************************/
/*                               LOAD JOBLIST                                */
/*****************************************************************************/

void DestinationVideoImpl::load_joblist(){
	load_joblist_intern();
}//load_joblist

/*****************************************************************************/
/*                            LOAD JOBLIST INTERN                            */
/*****************************************************************************/

bool DestinationVideoImpl::load_joblist_intern(){
	VideoJob* videojob;
	JobList* localloadjoblist;
	int i=0;
	
	localloadjoblist=new JobList();

	QString name = QFileDialog::getOpenFileName("/home", 0,this,"open file dialog","Choose a joblist" );
	if(!name){
		return false;
	}//if
	else{
		assert(name);
		FILE* file = fopen(name,"r");
		if(!file){
			return false;
		}//if
		else{
			assert(file);	
			char* buf = (char*)malloc(MAX_FILE_LEN);
			fread( (void*) buf, 1, MAX_FILE_LEN, file);
			if(localloadjoblist->fromStringJobs(buf)){
				for(i=0;(videojob=(VideoJob*)localloadjoblist->getJobByIndex(i))!=NULL;i++){	
					//Rename job	
					QString q_jobname(videojob->getName());
					QString add_jobname=get_name_of_input();
					q_jobname.insert(0,"-");
					q_jobname.insert(0,add_jobname);
					videojob->setName(q_jobname);

					//Insert job to list and joblist
					joblist->insertItem(q_jobname,-1);
					localjoblist->addJob(videojob);
				}//for	
				fclose(file);			
				return true;
			}//if
			else{
				fclose(file);
				return false;
			}//else
		}//else
	}//else

	return false;

}//load_joblist_intern

/*****************************************************************************/
/*                              JOBLIST SELECTED                             */
/*****************************************************************************/

void DestinationVideoImpl::joblist_selected(){
	QString jobname;
	
	char* c_name;
	
	int i;
	int i_bitsPerSecond;
	int i_bitrate_min;
	int i_bitrate_max;
	int i_width;
	int i_height;
	int i_macro_min;
	int i_macro_max;
	int i_quanti_min;
	int i_quanti_max;
	int i_I_frame;
	int i_P_frame;
	int i_B_frame;
	int i_gop_size;
	int i_b_frame_slide;
	int i_fps;
	int i_dec_buffer_size;
	int i_video_pac_size;
	int i_video_s_sec;
	int i_video_s_ms;
	int i_video_e_sec;
	int i_video_e_ms;
	unsigned int ui_number;
	
	bool b_color;
	bool b_var_gop;
	bool b_only_i_frame;
	bool b_var_frame_rate;
	bool b_en_video_pac;
	
	float f_video_s;
	float f_video_e;
	float f_aspect_ratio; 

	CodecID codec_id;
	VideoJob* videojob;
	
	ui_number=joblist->count();
	if(ui_number>0){
		jobname=joblist->currentText();
		c_name=(char*)jobname.ascii();
		videojob=(VideoJob*)localjoblist->getJob(c_name);

		i_bitsPerSecond=(int)videojob->getBitRate(); 	
		b_var_gop=videojob->getVariableGop(); 
		i_gop_size=videojob->getGopSize();
		i_bitrate_min=videojob->getMinBitrate();
		i_bitrate_max=videojob->getMaxBitrate();
		this->current_aspect_ratio=videojob->getAspectRatio();
		
		//QString q_size
		i_width=videojob->getWidth();
		i_height=videojob->getHeight();
		
		for(i=1;((i<MAX_ARRAY) && (i < this->w_size_array[0]));i++){
			if(this->w_size_array[i]==i_width){
				if(h_size_array[i]==i_height){
					this->current_index_size=i;
				}//if
			}//if
		}//for	

		i_fps=(int)videojob->getFPS();
		b_var_frame_rate=videojob->getVariableFR();
		b_color=videojob->hasColor();
		codec_id=videojob->getCodec();
		f_aspect_ratio=videojob->getAspectRatio();
		i_macro_min=videojob->getMinMacroBlock();
		i_macro_max=videojob->getMaxMacroBlock();
		i_quanti_min=videojob->getMinQuantization();
		i_quanti_max=videojob->getMaxQuantization();
		i_I_frame=videojob->getQuantI();
		i_B_frame=videojob->getQuantB();
		i_P_frame=videojob->getQuantP();
		i_b_frame_slide=videojob->getNumBFrames();
		b_only_i_frame=videojob->getIFramesOnly();
		i_dec_buffer_size=videojob->getDecBufferSize();
		b_en_video_pac=videojob->getEnableVideoPackets();
		i_video_pac_size=videojob->getVideoPacketSize();

		f_video_s=videojob->getStartPosition();
		f_video_e=videojob->getEndPosition();

		i_video_s_sec=(int)f_video_s;
		i_video_e_sec=(int)f_video_e;

		i_video_s_ms=(int)(((float)i_video_s_sec-f_video_s)*1000);
		i_video_e_ms=(int)(((float)i_video_e_sec-f_video_e)*1000);


	
		fill_in_form(c_name,
		             i_bitsPerSecond,
			     i_bitrate_min, 
			     i_bitrate_max,
			     i_width,
			     i_height,
			     codec_id,
			     f_aspect_ratio,
			     b_color,
			     i_macro_min,
			     i_macro_max,
			     i_quanti_min,
			     i_quanti_max,
			     i_I_frame,
			     i_P_frame,
			     i_B_frame,
			     b_var_gop,
			     b_only_i_frame,
			     b_var_frame_rate,
			     i_gop_size,
			     i_b_frame_slide,
			     i_fps,
			     i_dec_buffer_size,
			     b_en_video_pac,
			     i_video_pac_size,
			     i_video_s_sec,
			     i_video_s_ms,
			     i_video_e_sec,
			     i_video_e_ms
			     );
	}//if
}//joblist_selected

/*****************************************************************************/
/*                                  CANCEL                                   */
/*****************************************************************************/

void DestinationVideoImpl::cancel(){
	//free(w_size_array);
	//free(h_size_array);

	this->done(-1);
}//cancel

/*****************************************************************************/
/*                                   SAVE                                    */
/*****************************************************************************/

void DestinationVideoImpl::save(){

	int i;
	VideoJob* videojob;
	VideoESInfo* e_esinfo;

	if(this->newJob){
		if((joblist->currentItem())>=0){
			update_job();
		}//if
		else{
			new_job();
		}//else

		
		for(i=0;(videojob=(VideoJob*)globaljoblist->getJobByIndex(i))!=NULL;i++){
			Source* source;
			QString q_jobname(videojob->getName());
			source=(Source*)(videojob->getSource());
			if(source!=NULL){
				e_esinfo=(VideoESInfo*)source->getESInfo();
				if(memcmp(e_esinfo,this->esinfo,sizeof(e_esinfo))==0){
				
					globaljoblist->delJob(videojob);
					i--;
				}//if		
			}//if
		}//for

		for(i=0;(videojob=(VideoJob*)localjoblist->getJobByIndex(i))!=NULL;i++){
			globaljoblist->addJob((Job*)videojob);
			
		}//for

		if(videojob!=NULL){
			videojob->clean();
		
		}//if	
	}//if
	
	//free(w_size_array);
	//free(h_size_array);


	this->done(0);
}//save

/*****************************************************************************/
/*                               FILL IN FORM                                */
/*****************************************************************************/

void DestinationVideoImpl::fill_in_form(const char* c_name,
                                        int i_bitsPerSecond,
					int i_bitrate_min, 
					int i_bitrate_max,
					int i_width,
					int i_height,
					CodecID codec_id,
					float f_aspect_ratio,
					bool b_color,
					int i_macro_min,
					int i_macro_max,
					int i_quanti_min,
					int i_quanti_max,
					int i_I_frame,
					int i_P_frame,
					int i_B_frame,
					bool b_var_gop,
					bool b_only_i_frame,
					bool b_var_frame_rate,
					int i_gop_size,
					int i_b_frame_slide,
					int i_fps, 	
					int i_dec_buffer_size,
					bool b_en_video_pac,
					int i_video_pac_size,
					int i_video_s_sec,
					int i_video_s_ms,
					int i_video_e_sec,
					int i_video_e_ms
					){
	
	int i=0;
	//NAME
	QString q_name(c_name);
	outputfile->setText(q_name);
	char c_size[MAX_STR_LEN];

	
	//BITRATE
	if(i_bitsPerSecond<=(bitrate->maxValue()*1024)){
		bitrate_kbit->setChecked(true);
		bitrate->setValue(i_bitsPerSecond/1024);
		bitrate_min->setValue(i_bitrate_min/1024);
		bitrate_max->setValue(i_bitrate_max/1024);
	}//if
	else{
		bitrate_mbit->setChecked(true);
		bitrate->setValue(i_bitsPerSecond/1024/1024);
		bitrate_min->setValue(i_bitrate_min/1024/1024);
		bitrate_max->setValue(i_bitrate_max/1024/1024);
	}//else

	//SIZE
	bool flag=false;
	for(i=1;((i<MAX_ARRAY) && (i<w_size_array[0]));i++){
		if(w_size_array[i]==i_width){
			if(h_size_array[i]==i_height){
				size->setCurrentItem(i);
				flag=true;
			}//if
		}//if
	}//for
	
	if(flag==false){
		memset(c_size,0,MAX_STR_LEN);
		sprintf(c_size,"%i x %i",i_width,i_height);
		if(i<MAX_ARRAY){
			size->insertItem((QString)c_size,i);
			size->setCurrentItem(i);
			w_size_array[i]=i_width;
			h_size_array[i]=i_height;
		}//if
		w_size_array[0]+=1;
	}//if


	//CODEC
	//codec->setCurrentItem((int)codec_id);
	set_Codec_ID(codec_id);
	//codec_name=MP4Decoder::codecToString((CodecID)i_count);


	//COLOR
	color->setChecked(b_color);
	
	
	//MACRO BLOCK
	macro_min->setValue(i_macro_min);
	macro_max->setValue(i_macro_max);

	//QUANTIZATION
	quanti_min->setValue(i_quanti_min);
	quanti_max->setValue(i_quanti_max);

	//FRAME
	i_frame->setValue(i_I_frame);
	p_frame->setValue(i_P_frame);
	b_frame->setValue(i_B_frame);

	//GOP
	var_gop->setChecked(b_var_gop);
	only_i_frame->setChecked(b_only_i_frame);
	var_frame_rate->setChecked(b_var_frame_rate);
	
	gop_size->setValue(i_gop_size);
	b_frame_slide->setValue(i_b_frame_slide);
	fps->setValue(i_fps);
	set_gop_pattern(i_b_frame_slide);


	if(b_en_video_pac){
		video_pac_size->setValue(i_video_pac_size);
	}//if
	en_video_pac->setChecked(b_en_video_pac);
	dec_buffer_size->setValue(i_dec_buffer_size);

	//VIDEO PACKETS
	var_frame_rate_clicked();
	en_video_pac_clicked();
	b_frame_slide_sliderMoved(i_b_frame_slide);
	only_i_frame_clicked();
	var_gop_clicked();

	//VIDEO
	video_s_sec->setValue(i_video_s_sec);
	video_s_ms->setValue(i_video_s_ms);
	video_e_sec->setValue(i_video_e_sec);
	video_e_ms->setValue(i_video_e_ms);
	
	//free(c_size);
}//fill_in_form

/*****************************************************************************/
/*                               COLOR CLICKED                               */
/*****************************************************************************/

void DestinationVideoImpl::color_clicked(){
	if(color->isChecked()){
		if(!esinfo->getHasColor()){
			//Extension Color
			color->setChecked(false);
		}//if
	}//if
}//color_clicked

/*****************************************************************************/
/*                              VAR GOP CLICKED                              */
/*****************************************************************************/

void DestinationVideoImpl::var_gop_clicked(){
	if(var_gop->isChecked()){
		gop_size->setValue(gop_size->minValue());
		gop_size->setDisabled(true);
		t_gop_size->setDisabled(true);
		only_i_frame->setChecked(false);
		only_i_frame->setDisabled(true);		
	}//if
	else{
		gop_size->setDisabled(false);
		t_gop_size->setDisabled(false);
		only_i_frame->setDisabled(false);	
	}//else

}//var_gop_clicked

/*****************************************************************************/
/*                            ONLY I FRAME CLICKED                           */
/*****************************************************************************/

void DestinationVideoImpl::only_i_frame_clicked(){
	if(only_i_frame->isChecked()){
		only_i_frame->setDisabled(false);
		b_frame_slide->setDisabled(true);
		t_b_frame_slide->setDisabled(true);
		gop_pattern->setText("");
		gop_size->setDisabled(true);
		t_gop_size->setDisabled(true);
		var_gop->setChecked(false);
		var_gop->setDisabled(true);
		t_gop_pattern->setDisabled(true);

	}//if
	else{
		b_frame_slide->setDisabled(false);
		t_b_frame_slide->setDisabled(false);
		gop_size->setDisabled(false);
		t_gop_size->setDisabled(false);
		var_gop->setDisabled(false);
		t_gop_pattern->setDisabled(false);
	}//else
}//only_i_frame_clicked

/*****************************************************************************/
/*                          VAR FRAME RATE CLICKED                           */
/*****************************************************************************/

void DestinationVideoImpl::var_frame_rate_clicked(){
	if(var_frame_rate->isChecked()){
		var_frame_rate->setDisabled(false);
		fps->setValue(fps->minValue());
		fps->setDisabled(true);
		t_fps->setDisabled(true);
	}//if
	else{
		fps->setDisabled(false);
		t_fps->setDisabled(false);
	}//else
}//var_frame_rate_clicked

/*****************************************************************************/
/*                                RESET FORM                                 */
/*****************************************************************************/

void DestinationVideoImpl::reset_form(){
	joblist->clearSelection();
	first_settings();
}//reset_form

/*****************************************************************************/
/*                     ENABLE VIDEO PACKETS CLICKED                          */
/*****************************************************************************/

void DestinationVideoImpl::en_video_pac_clicked(){
	if(en_video_pac->isChecked()){
		en_video_pac->setDisabled(false);
		video_pac_size->setDisabled(false);
		t_video_pac_size->setDisabled(false);
	}//if
	else{
		video_pac_size->setValue(video_pac_size->minValue());
		video_pac_size->setDisabled(true);
		t_video_pac_size->setDisabled(true);
	}//else

}//en_video_pac_clicked

/*****************************************************************************/
/*                       B FRAME SLIDE SLIDER MOVED                          */
/*****************************************************************************/

void DestinationVideoImpl::b_frame_slide_sliderMoved( int i_slider ){
	set_gop_pattern(i_slider);
}//b_frame_slide_sliderMoved

/*****************************************************************************/
/*                           SET GOP PATTERN                                 */
/*****************************************************************************/

void DestinationVideoImpl::set_gop_pattern(int i_slider){
	int i_gop_size;
	int b;
	int num=0;
	char ca_gop_pattern[MAX_STR_LEN];
	char c_frame[MAX_STR_LEN];
	memset(ca_gop_pattern,0,MAX_STR_LEN);
	memset(c_frame,0,MAX_STR_LEN);
	
	i_gop_size=gop_size->value();
	sprintf(c_frame,"%d",i_slider);
	QString q_frame(c_frame);
	b_frame_value->setText(q_frame);


	//First I
	strcat(ca_gop_pattern,"I");
	num++;
	strcat(ca_gop_pattern,"P");
	num++;
	while(num<i_gop_size){
		for(b=1;b<=i_slider;b++){
			if(num<i_gop_size){
			strcat(ca_gop_pattern,"B");
			num++;
			}//if
		}//for
		if(num<i_gop_size){
		strcat(ca_gop_pattern,"P");
		num++;
		}//if

	}//while
	strcat(ca_gop_pattern,"I");
	
	QString q_gop_pattern((char*)ca_gop_pattern);
	gop_pattern->setText(q_gop_pattern);	

	//free(ca_gop_pattern);
}//set_gop_pattern

/*****************************************************************************/
/*                            BITRATE VALUE CHANGED                          */
/*****************************************************************************/

void DestinationVideoImpl::bitrate_valueChanged(int new_avbitrate){
	if(bitrate_min->value()>new_avbitrate){
		bitrate_min->setValue(new_avbitrate);
	}//if
	if(bitrate_max->value()<new_avbitrate){
		bitrate_max->setValue(new_avbitrate);
	}//if

}//bitrate_valueChanged

/*****************************************************************************/
/*                              SIZE ACTIVATED                               */
/*****************************************************************************/

void DestinationVideoImpl::size_activated(int index){
	int width=0;
	int height=0;
	int i;
	bool flag=false;
	char c_size[MAX_STR_LEN];
	float f_help_aspect;
	
	if(index==0){
		if((joblist->currentItem())<0){
			this->current_aspect_ratio=this->esinfo->getAspectRatio();
			width=this->esinfo->getWidth();
			height=this->esinfo->getHeight();
		}//if
		else{
			width=w_size_array[this->current_index_size];
			height=h_size_array[this->current_index_size];
		}//else

		
		f_help_aspect=this->current_aspect_ratio;
		
		UserDefineImpl *dialog = new UserDefineImpl(&f_help_aspect,&width,&height,this, 0, TRUE, TRUE);
		dialog->exec();
	
		if(f_help_aspect>=0){
			this->current_aspect_ratio=f_help_aspect;
		}//if


		if( (width>0) || (height>0) ){
	
			for(i=1;((i<MAX_ARRAY) && (i<w_size_array[0]));i++){
				if(w_size_array[i]==width){
					if(h_size_array[i]==height){
						size->setCurrentItem(i);
						flag=true;
					}//if
				}//if
			}//for
		
			if(flag==false){
				memset(c_size,0,MAX_STR_LEN);
				sprintf(c_size,"%i x %i",width,height);
				if(i<MAX_ARRAY){
					size->insertItem((QString)c_size,i);
					w_size_array[i]=width;
					h_size_array[i]=height;
				}//if
				w_size_array[0]+=1;
				size->setCurrentItem(i);
				//free(c_size);
			}//if
		}//if
		else{
			joblist_selected();
		}//else
	}//if
}//size_activated

/*****************************************************************************/
/*                        OUTPUTFILE RETURN PRESSED                          */
/*****************************************************************************/

void DestinationVideoImpl::outputfile_returnPressed(){
	outputfile->doKeyboardAction(QTextEdit::ActionBackspace);
	new_job();
}//outputfile_returnPressed

/*****************************************************************************/
/*                           OUTPUTFILE TEXT CHANGED                         */
/*****************************************************************************/

void DestinationVideoImpl::outputfile_textChanged(){
	int i=-1;

	QString q_file_name=outputfile->text();
	
	i=q_file_name.find(QRegExp("\t"), 0 );

	if(i>=0){
		q_file_name.remove(i,1);
		outputfile->setText(q_file_name);
		//outputfile->unsetCursor();
	}//if

	i=-1;
	i=q_file_name.find(QRegExp("/"), 0 );
	if(i==0){
		q_file_name.remove(i,1);
		outputfile->setText(q_file_name);
	}//if
	
	//bitrate->setCursor(ArrowCursor);
	
}//outputfile_textChanged

/*****************************************************************************/
/*                              FILL IN CODEC                                */
/*****************************************************************************/

void DestinationVideoImpl::fill_in_codec(){
	int i;	
	
	for(i=0;codec_txt[i]!="UNKNOWN_CODEC";i++){
		codec->insertItem(codec_txt[i],-1);
	}//for    
	//codec->insertItem(codec_txt[i]);
}//fill_in_codec

/*****************************************************************************/
/*                             SET CODEC ID                                  */
/*****************************************************************************/

void DestinationVideoImpl::set_Codec_ID(CodecID cid){
	const char* codec_name;
	int i;
	
	codec_name=MP4Decoder::codecToString(cid);

	for(i=0;codec_txt[i]!="UNKNOWN_CODEC";i++){
		if(strcmp(codec_name,codec_txt[i])==0){
			codec->setCurrentItem(i);
		}//if
	}//for    
}//set_Codec_ID


void DestinationVideoImpl::fixed_quantisation_clicked(){
	if(fixed_quantisation->isChecked()){
		macro_min->setDisabled(false);
		macro_max->setDisabled(false);
		quanti_min->setDisabled(false);
		quanti_max->setDisabled(false);
		i_frame->setDisabled(false);
		p_frame->setDisabled(false);
		b_frame->setDisabled(false);

		macro_min->setValue(12);
		macro_max->setValue(12);
		quanti_min->setValue(12);
		quanti_max->setValue(12);
		i_frame->setValue(12);
		p_frame->setValue(16);
		b_frame->setValue(16);
	}//if
	else{
		macro_min->setDisabled(true);
		macro_max->setDisabled(true);
		quanti_min->setDisabled(true);
		quanti_max->setDisabled(true);
		i_frame->setDisabled(true);
		p_frame->setDisabled(true);
		b_frame->setDisabled(true);

	}//else

}//fixed_quantisation_clicked
