/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   Job.hpp                                                     *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef MUVITRANS_JOB_HPP__
#define MUVITRANS_JOB_HPP__


#include "global.hpp"
#include "assert.h"
class Source;

//#define MAX_LEN 30

class Job {

protected:
	u32 jobID; // unique ID
	char name[MAX_STR_LEN];
	Source* in_es;
	char outFile[MAX_STR_LEN];
	char description[MAX_STR_LEN];
	u32 targetBitRate;
	u32 reference_counter;
	bool is_video;
	float startPosition;
	float endPosition;

public:
	/** constructor */

	Job(const char *name, Source* es, const char* outFile, u32 targetBitRate, float startPosition = 0, float endPosition = 0);
	Job();
	
	/** destructor */
	virtual ~Job();

	/** methods to get params of job */
	const char* getName() const;
	const Source* getSource() const;
	const char* getOutFileName() const;
	u32 getBitRate() const;
	u32 getID() const;
	float getStartPosition() const;
	float getEndPosition() const;
	virtual const char* getDescription() = 0;

	bool isVideo();
	
	/** implement methods of superclass */
	virtual bool toString(char* buf, int max_len) =0;
	virtual char* toString()=0;
	virtual bool fromString(const char* initString) =0;
	
	/** create a unique id to reference job */
	static u32 createJobID();
	
	/** if job is referenced, the counter will be incremented */
	void increment();

	/** decrements reference_counter of job */
	void decrement();

	/** if counter is less or equal 0 it is not a valid job*/
	void checkReferences() const;

	/** decrements counter and checks, if it is 1, then the job can
	 ** be deleted */
	void clean();
	
	/** methods to set job params */
	void setName(const char* name);
	void setSource(Source* es);
	void setOutFileName(const char* outFile);
	void setBitRate(u32 rate);
	void setID(u32 id);
	void setStartPosition(float start);
	void setEndPosition(float end);
	
	/** method has to be implemented in subclasses */
	virtual Job* clone(Source* new_in_es) = 0;
	
}; //class Job
#endif
