/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   JobList.cpp                                                 *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/


#include "JobList.hpp"
#include "Source.hpp"
#include "xml.hpp"
#include "Job.hpp"
#include "VideoJob.hpp"
#include "AudioJob.hpp"

JobList::JobList()
{
	dprintf_full("\033[32mJobList::construct\033[0m\n");
}

/** destructor */
JobList::~JobList()
{
	dprintf_full("\033[32mJobList::destruct\033[0m\n");
	list <Job*>::iterator jobIterator = this->jobs.begin();
	while(!this->jobs.empty())
	{
		jobIterator = this->jobs.begin();
		delJob(*jobIterator);
	}
}

/** @param return is false if a job with the
  * same identifier as @param job already exists
  */

bool JobList::addJob(Job* job)
{
	dprintf_full("\033[32m>> JobList::addJob\033[0m\n");
	assert(job);
	job->checkReferences();
	list <Job*>::iterator li = this->jobs.begin();
	while(li != this->jobs.end())
	{
		if((*li) == job)
		{
			dprintf_err("\033[31mJobList::add : job already exists\033[0m\n");
			return false;
		}
		li++;
	}
	this->jobs.push_back(job);
	job->increment();
	dprintf_full("\033[32m<< JobList::add job ok\033[0m\n");
	return true;
}

bool JobList::delJob(Job* job)
{
	dprintf_full("\033[32m>> JobList::delJob\033[0m\n");
	assert(job);
	job->checkReferences();
	list <Job*>::iterator li;
	if(this->jobs.empty())
	{	
		dprintf_err("\033[31mJobList::delJob : empty list\033[0m\n");
		return false;
	}
	else li=this->jobs.begin();
	while(li!=this->jobs.end())
	{
		if((*li) != job)
		{
			li++;
		}
		else
		{
			job->clean();
			this->jobs.erase(li);
			dprintf_full("\033[32m<< JobList::delJob ok\033[0m\n");
			return true;
		}
	}
	dprintf_err("\033[31mJobList::delJob : element not found\033[0m\n");
	return false;
}

Job* JobList::getJob(u32 jobId)
{
	dprintf_full("\033[32m>> JobList::getJob(%i)\033[0m\n", jobId);
	list<Job*>::iterator li;
	if(this->jobs.empty())
	{
		dprintf_err("\033[31mJobList::getJob : emtpy list\033[0m\n");
		return NULL;
	}
	else li = this->jobs.begin();
	while(li!=this->jobs.end())
	{
		if((*li)->getID() == jobId)
		{
			dprintf_full("\033[32m>> JobList::getJob(jobID) ok\033[0m\n");
			(*li)->increment();
			return (*li);
		}
		li++;
	}
	dprintf_err("\033[31mJobList::getJob : job not found\033[0m\n");
	return NULL;
}

Job* JobList::getJob(const char* jobName)
{
	dprintf_full("\033[32m>>JobList::getJob(jobName)\033[0m\n");
	assert(jobName);
	list<Job*>::const_iterator li;
	if(this->jobs.empty())
	{
		dprintf_err("\033[31mJobList::getJob(jobName) : empty list\033[0m\n");
		return NULL;
	}
	else li = this->jobs.begin();
	while(li != this->jobs.end())
	{
		if(strcmp((*li)->getName(),jobName)==0)
		{
			dprintf_full("\033[32m<< JobList::getJob(jobName) ok\033[0m\n");
			(*li)->increment();
			return (*li);
		}
		li++;
	}
	dprintf_full("\033[31mJobList::getJob(jobName) : job not found\033[0m\n");
	return NULL;
}


Job* JobList::getJobByIndex(u32 index)
{
	dprintf_full("\033[32mJobList::getJobByIndex\033[0m\n");
	list<Job*>::iterator next = this->jobs.begin();
	while( next!=this->jobs.end() && index>0 )
	{
		next++;
		index--;
	}
	if ( index == 0 && next!=this->jobs.end())
	{
		dprintf_full("\033[32mJobList::getJobByIndex : found element\033[0m\n");
		(*next)->increment();
		return *next;
	}
	else
	{
		dprintf_err("\033[31mJobList::getJobByIndex : End of list\033[0m\n");
		return NULL;
	}
}
									

JobList* JobList::clone(Source* in_es)
{
	dprintf_full("\033[32m>> JobList::clone\033[0m\n");
	assert(in_es);
	JobList* clone = new JobList();
	list <Job*>::iterator li=this->jobs.begin();
	while(li != this->jobs.end())
	{
		clone->addJob((*li)->clone(in_es));
		li++;
	}
	dprintf_full("\033[32m<< JobList::clone\033[0m\n");			
	return clone;	
}

bool JobList::toStringJobs(char* output, int max_len)
{
	dprintf_full("\033[32m>> JobList::toStringJobs(output, max_len)\033[0m\n");
	assert(output);
	int len=0;
	list <Job*>::iterator li;
	if(this->jobs.empty())
	{
		dprintf_err("\033[31mJobList::toStringJobs(output, max_len) : empty list\033[0m\n");
		output[0]='\0';
		return false;
	}
	else li=this->jobs.begin();
	strcpy(output, JL_START);
	output=(char*)output+strlen(JL_START);
	while(li!=this->jobs.end() && max_len > 0)
	{
		if(!((*li)->toString(output, max_len)))
		{
			dprintf_err("\033[31mJobList::toStringJobs(output, max_len) : toString of job not ok\033[0m\n");
			return false;
		}
		li++;
		len=strlen(output);
		output=(char*)output+len;
		max_len-=len;
	}
	if(max_len>(int)strlen(JL_END))
	{
		if(NULL == strcpy(output, JL_END))
		{
			dprintf_err("\033[31mJobList::toStringJobs(output, max_len) : JobListEnd not found\033[0m\n");
			return false;
		}
		else
		{
			dprintf_full("\033[32m<< JobList::toStringJobs(output, max_len) ok\033[0m\n");
			return true;
		}
        }
	else 
	{
		dprintf_err("\033[31mJobList::toStringJobs(output, max_len)\033[0m\n");
		return false;
	}
}

char* JobList::toStringJobs()
{
	dprintf_full("\033[32m>> JobList::toStringJobs()\033[0m\n");
	char* output = (char*)malloc(MAX_FILE_LEN);
	if(toStringJobs(output, MAX_FILE_LEN))
	{
		dprintf_full("\033[32m<< JobList::toStringJobs ok\033[0m\n");
		return output;
	}
	else
	{
		dprintf_err("\033[31mJobList::toStringJobs\033[0m\n");
		snprintf(output, MAX_STR_LEN, "%s%s", JL_START, JL_END);
		return output;
	}
}

bool JobList::toStringIds(char* output, int max_len)
{
	dprintf_full("\033[32m>> JobList::toStringIds(output, max_len)\033[0m\n");
	assert(output);
	int len=0;
	strcpy(output, JL_START);
	output=(char*)output+strlen(JL_START);
	max_len -= strlen(JL_START);
	int i=0;
	u32 id;
	Job* job = NULL;
	while((job = this->getJobByIndex(i)) != NULL)
	{
		id=job->getID();
		len = snprintf(output, max_len, JOBID, id);
		output=(char*)output+len;
		i++;
	}
	if(max_len >= (int)strlen(JL_END))
	{
		strcpy(output, JL_END);
		dprintf_full("\033[32mJobList::toStringIds ok\033[0m\n");
		return true;
	}
	else
	{
		dprintf_err("\033[31mJobList::toStringIds\033[0m\n");
		return false;
	}
}

char* JobList::toStringIds()
{
	dprintf_full("\033[32m>> JobList::toStringIds()\033[0m\n");
	char* output = (char*)malloc(MAX_XML_LEN);
	if(toStringIds(output, MAX_XML_LEN))
	{
		dprintf_full("\033[32m<< JobList::toStringIds ok\033[0m\n");
		return output;
	}
	else
	{
		free(output);
		dprintf_err("\033[31mJobList::toStringIds\033[0m\n");
		return NULL;
	}
}

bool JobList::fromStringJobs(const char* initString)
{
	dprintf_full("\033[32m>> JobList::fromStringJobs\033[0m\n");
	assert(initString);
	const char* next = initString;
	char* end = NULL;
	if(next == strstr(next, JL_START))
	{
		next=(const char*)next+strlen(JL_START);
	}
	else
	{
		dprintf_err("\033[31mJobList::fromStringJobs ERROR\033[0m\n");
		return false;
	}
	if(NULL==(end = strstr(next, JL_END)))
	{
		dprintf_err("\033[31mJobList::fromStringJobs : end not found\033[0m\n");
		return false;
	}
	while(next < end)
	{
		if(0 ==	memcmp( (void*)next, (void*)VJOB_START, strlen(VJOB_START)))
		{
			VideoJob* vj = new VideoJob();
			if(vj->fromString(next))
			{
				this->addJob(vj);
				if(NULL!=(next=strstr(next, VJOB_END)))
				{
					next=(const char*)next+strlen(VJOB_END);
				}
				else
				{
					dprintf_err("\033[31mJobList::fromStringJobs : error at finding vjob end\033[0m\n");
					
					return false;
				}
			}
			else
			{
				dprintf_err("\033[31mJobList::fromStringJobs : error at reading vjob\033[0m\n");
				return false;
			}
		}
		else if(0 == memcmp( (void*)next, (void*)AJOB_START, strlen(AJOB_START) ))
		{	
	 	       	AudioJob* aj = new AudioJob();
			if(aj->fromString(next))
			{
				this->addJob(aj);
				if(NULL!=(next=strstr(next, AJOB_END)))
				{       
					next=(const char*)next+strlen(AJOB_END);
				}
				else
				{
					dprintf_err("\033[31mJobList::fromStringJobs : error at reading ajob\033[0m\n");
					return false;
				}
			}
			else
			{
				dprintf_err("\033[31mJobList::fromStringJobs : error at reading ajob\033[0m\n");
				return false;
			}
				
		}
		else 
		{
			dprintf_err("\033[31mJobList::fromStringJobs : error at reading job\033[0m\n");
			return false;
		}
	}
			
	if(next==end)
	{
		dprintf_full("\033[32m<< JobList::fromStringJobs ok\033[0m\n");
		return true;
	}
	else
	{
		dprintf_err("\033[31mJobList::fromStringJobs\033[0m\n");
		return false;
	}

}

JobList* JobList::fromStringIds(const char* initString, JobList* jl, Source* source)
{
	dprintf_full("\033[32m>> JobList::fromStringIds\033[0m\n");
	assert(initString);
	assert(jl);
	Job* job;
	JobList* new_jl = this;
	const char* next = initString;
	char* end = NULL;
	u32 id = 0;
	if(next == strstr(next, JL_START))
	{
		next=(const char*)next+strlen(JL_START);
	
	}
	else
	{
		dprintf_err("\033[31mJobList:fromStringIds : no jl-start found\n\033[0m");
		return NULL;
	}
	if(NULL == (end = strstr(next, JL_END)))
	{
		dprintf_err("\033[31mJobList::fromStringIds : end not found\033[0m\n");
		return NULL;
	}
	while(next < end)
	{
		if(1 == sscanf(next, JOBID, &id))
		{
			if((job = jl->getJob(id)) != NULL)
			{
				job->setSource(source);
				new_jl->addJob(job);
				next=(const char*)strchr(next, '\n')+1;
			}
			else
			{
				dprintf_err("\033[31mJobList::fromStringJobs : no job found!!\033[0m\n");
			}
		}
		else
		{
			dprintf_err("\033[31mJobList::fromStringJobs : no jobid found!!\033[0m\n");
		}
	}
	if(next == end)
	{	
		dprintf_full("\033[32m<< JobList::fromStringIds ok\033[0m\n");
		return new_jl;
	}
	else return NULL;
}
