/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   JobList.hpp                                                 *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 *************************************************************i**********/

#ifndef __JOBLIST_HPP__
#define __JOBLIST_HPP__

class Job;
class Source;

#include <list>
#include <iostream>
#include "global.hpp"


class JobList{

protected:
	std::list<Job*> jobs;

public:
	/** constructor */
	JobList();

	/** destructor */
 	virtual ~JobList();

	/** @param return is false if a job with the
	  * same identifier as @param job already exists */
	bool addJob(Job* job); //generates unique ID
	bool delJob(Job* job);

	/** returns job with given id */
	Job* getJob(u32 jobId); 

	/** returns job with given name */
	Job* getJob(const char* jobName); 
	

	/** returns job corresponding to given index */
	Job* getJobByIndex(u32 index); 

	/** clones full joblist */
	JobList* clone(Source* in_es = NULL);

	/** prints all data of jobs including all params */
	bool toStringJobs(char *buf, int max_len);

	/** prints all data of jobs including all params, the return buf is malloc'd, so free() it! */
	char* toStringJobs();

	/** only prints ids of containd jobs */
	bool toStringIds(char *buf, int max_len);
        char* toStringIds();
		
	/** reads information about jobs and creates them */	
	bool fromStringJobs(const char* initString);

	/** reads ids and searches for existing jobs */
	JobList* fromStringIds(const char* initString, JobList* jl, Source* source);
}; //class
#endif
