 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	MultiTranscoderImpl.cpp					*
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/
#include <qobject.h>
#include <qpushbutton.h>
#include <qbutton.h>
#include <qfiledialog.h>
#include <qlineedit.h>
#include <qgroupbox.h>
#include <qlistview.h>
#include <qpixmap.h>
#include <qapplication.h>
#include <qstring.h>
#include <qlistbox.h>
#include <iostream>
#include <qaction.h>


#include "MultiTranscoderImpl.hpp"

#include "JobList.hpp"
#include "ESInfo.hpp"
#include "Job.hpp"

#include "main.hpp"
#include "Source.hpp"
#include "SourceList.hpp"
#include "SystemFile.hpp"
#include "VideoJob.hpp"
#include "AudioJob.hpp"
#include "EditSystemFileImpl.hpp"
#include "DeleteSourceImpl.hpp"
#include "DeleteJobImpl.hpp"
#include "DeleteProjectImpl.hpp"
#include "ProjectNameImpl.hpp"
#include "DestinationAudioImpl.hpp"
#include "DestinationVideoImpl.hpp"
#include "ProgressImpl.hpp"
#include "WorkingDirectoryImpl.hpp"


#include "ContainerInfo.hpp"
#include "container/ContainerFile.hpp"

#define POS 18

QString durationTime(int d);
void updateSystemFileView(QListView* qlvSys, Source* s, JobList* jl, QString id, QString pfad);
void mt_checkJob(QListView* qlvSys, QListView* qlv, Job* j, QString jobname, QString idJob, QString description, QPixmap* pixmap);
void insertJob(QListView* qlv, Job* j, QString q_jobname, QString id, QString description, QPixmap* pixmap);
void SourceToSystemfile(QListView* qlv, QListViewItem* qlvi, Source* s, QString q_jobname, QString id, QPixmap* pixmap, QString pfad);
void JobToSystemfile(QListView* qlv, QListViewItem* qlvi, Job* j, QString q_jobname, QString id, QString description, QPixmap* pixmap);

MultiTranscoderImpl::MultiTranscoderImpl(QWidget* parent, const char* name, WFlags fl): MultiTranscoder( parent, name, fl )
{

	QString filename = "./images/arrow_down.png";
	QPixmap* p = new QPixmap(filename);
	mt_button_down->setPixmap(*p);
	mt_listView_source->setSorting(-1);
	mt_listView_source->setColumnWidth(6,0);
	mt_listView_jobs->setSorting(-1);
	mt_listView_systemfile->setSorting(-1);
	mt_listView_systemfile->setColumnWidth(6,0);

  //set buttons deactive
  mt_button_delete_source->setDisabled(TRUE);
  mt_button_info_source->setDisabled(TRUE);
  
  mt_button_new_jobs->setDisabled(TRUE);
  mt_button_delete_jobs->setDisabled(TRUE);
  mt_button_edit_jobs->setDisabled(TRUE);
  
  mt_button_down->setDisabled(TRUE);
  
  mt_button_delete_systemfiles->setDisabled(TRUE);
  mt_button_edit_systemfiles->setDisabled(TRUE);

  //set menu items deactive
  delete_Source->setEnabled(FALSE);
  info_Source->setEnabled(FALSE);
  
  new_Job->setEnabled(FALSE);
  delete_Job->setEnabled(FALSE);
  edit_Job->setEnabled(FALSE);
  
  delete_Systemfile->setEnabled(FALSE);
  edit_Systemfile->setEnabled(FALSE);
  
}

MultiTranscoderImpl::~MultiTranscoderImpl()
{
}


void MultiTranscoderImpl::mt_addSource()
{
	Source* s = NULL;
	SourceList* sl = NULL;
	QString filename;
	QPixmap* pixmap = NULL;

	QFileDialog* fd = new QFileDialog(this, "file dialog", TRUE);
	QString types("(*.mp2 *.mp3 *.mp4 *.mpg *.mpeg );;" "DVD (*.vob *.ifo);;" "(*.cmp *.mp4v *.m2v *.mpa *.avi *.mov)");
	QString name = fd->getOpenFileName(NULL, types, this,"open file dialog","Choose a file" );

	if(name.isEmpty()){
		dprintf_err("ERROR:mt_addSource:No file selected\n");
	}
	else{
		ContainerInfo* input = ContainerFile::loadContainerInfo(name.latin1());
		if(!input){
			dprintf_err("ERROR:mt_addSource:Cannot open inputfile\n");
		}
		else{
			std::list<ESInfo*>* eslist = input->getESList();
			if(eslist->empty()){
				dprintf_err("ERROR:mt_addSource:Inputfile is not a ES\n");
			}
			else{
				for( list<ESInfo*>::iterator li=eslist->begin(); li!=eslist->end(); ++li){
					QString in_sysfile = (char*)(*li)->getInput();
					if(in_sysfile.isEmpty()){
						dprintf_err("ERROR:mt_addSource:ES has no name\n");
					}
					else{
						//GUI shows the filename without the path
						char* file = (char*)malloc(MAX_STR_LEN);
						if(!file){
							dprintf_err("ERROR:mt_addSource:Malloc\n");
						}
						else{
							int i = in_sysfile.findRev('/');	//linux only
							if(i < 0){
								dprintf_err("ERROR:mt_addSource:No / in the path\n");
								free(file);
								file = (char*)in_sysfile.latin1();
							}//if
							else{
								if(MAX_STR_LEN < snprintf(file, MAX_STR_LEN,  "%s", (char*)(in_sysfile.latin1() + i + 1 )))
								{
									dprintf_err("ERROR:mt_addSource: snprintf\n");
								}
							}

							s = new Source((*li));
							if( !(sl = project->getSources()) ){
								dprintf_err("ERROR:mt_addSource: getSources()\n");
							}
							else{

								if( !(sl->add(s)) ){
									dprintf_err("ERROR:mt_addSource: add(Source*) project\n");
								}
								else{
									QListViewItem *last = mt_listView_source->lastItem();
									QListViewItem *item = NULL;

									QString es_id = QString::number((int)(*li)->getStreamId());
									QString type = "Unknown";
									if((*li)->isAudioStream()){
										filename = "./images/Agruen.png";
										pixmap = new QPixmap(filename);
									}
									else if((*li)->isVisualStream()){
										filename = "./images/Vgruen.png";
										pixmap = new QPixmap(filename);
									}

									QString start = QString("0:0:0");
									int d = ((int)(*li)->getDurationInMs())/1000;
									QString end = durationTime(d);
									QString description = QString((*li)->getDescriptionText());

									item = new QListViewItem( mt_listView_source, last, QString(""), QString(file), es_id, start, end, description, in_sysfile);
									item->setPixmap(0,*pixmap);
									mt_listView_source->ensureItemVisible(item);
									item->setDragEnabled (TRUE);
									mt_listView_source->setColumnWidth(6,0);

								}//else
							}//else
						}//else
					}//else
				}//for
			}//else
		}//else
	}//else
	//printf("\033[33m%s\033[0m\n",sl->toStringFull());
}//mt_addSource


void MultiTranscoderImpl::mt_delSource()
{
	QListViewItem *item = NULL;
	Source *s = NULL;
	SourceList* sl = NULL;
  JobList*jl = NULL;
	int pos = 0;
  QString id, pfad;

	if( (item = mt_listView_source->selectedItem () ) > 0){
		pos=(item->itemPos()) / POS;
		id = item->text(2);
		pfad = item->text(6);

		DeleteSourceImpl *dialog = new DeleteSourceImpl(this,0,TRUE,TRUE);
		int i =dialog->exec();

		if(dialog != NULL){
			delete dialog;
			dialog = NULL;
		}

		if(i == -1){
			return;
		}
		else{
			if( !(sl=project->getSources()) ){
				dprintf_err("ERROR:mt_delSource: getSources()\n");
			}
			else{
				if( !(s=sl->getSourceByIndex( (u32)pos )) ){
					dprintf_err("ERROR:mt_delSource: getSourceByIndex()\n");
				}
				else{
          jl=s->getJobList();
		
					if( !(sl->remove(s)) ){
						dprintf_err("ERROR:mt_delSource: remove()\n");
					}
					else{
						mt_listView_source->takeItem(item);

						int childs = 0;
						if((childs = mt_listView_jobs->childCount()) > 0){

							for(i=0;i < childs;i++){
								QListViewItem* delItem = mt_listView_jobs->firstChild();
								mt_listView_jobs->takeItem(delItem);
							}//for
						}//if

            //update mt_systemfileView
            updateSystemFileView(mt_listView_systemfile, s, jl, id, pfad);
					}//else
				}//else
				//s->clean();
			}//else
		}//else
	}//if
	else{
		dprintf_err("ERROR:No source selected\n");
	}//else
}//mt_delSource

void MultiTranscoderImpl::mt_infoSource()
{
	QListViewItem *item = NULL;
	QString sname;
	Source *s = NULL;
	SourceList* sl = NULL;
	JobList* jl = NULL;
	ESInfo* es = NULL;
	AudioESInfo* aes = NULL;
	VideoESInfo* ves = NULL;

	QString q_jobname;
	QString id;
	int pos = 0;

	if( (item = mt_listView_source->selectedItem() ) > 0){
		pos=(item->itemPos()) / POS;

		if( !(sl=project->getSources()) ){
			dprintf_err("ERROR:mt_infoSource: getSources()\n");
		}
		else{

			if( !(s=sl->getSourceByIndex( (u32)pos )) ){
				dprintf_err("ERROR:mt_infoSource: getSourceByIndex()\n");
			}
			else{
				if( !(es=s->getESInfo()) ){
					dprintf_err("ERROR:mt_infoSource: getESInfo()\n");
				}
				else{
					if( !(jl=s->getJobList()) ){
						dprintf_err("ERROR:mt_infoSource: getJobList()\n");
					}
					else{
						if(es->isAudioStream()){
							aes = (AudioESInfo*)es;
							DestinationAudioImpl *dialog = new DestinationAudioImpl(aes, jl, false, 0, this, 0, TRUE, TRUE);
							dialog->exec();

						}//if
						else if(es->isVisualStream()){
							ves = (VideoESInfo*)es;
							DestinationVideoImpl *dialog = new DestinationVideoImpl(ves, jl, FALSE, 0, this, 0, TRUE, TRUE);
							dialog->exec();
						}//else if
					}//else
				}//else
			}//else
		}//else
	}//if
	else{
		dprintf_err("No source selected\n");
	}

}//mt_infoSource

void MultiTranscoderImpl::mt_addJob()
{
	QListViewItem *item = NULL;
	QString sname;
	Source *s = NULL;
	SourceList* sl = NULL;
	JobList* jl = NULL;
	JobList* jlProject = NULL;
	ESInfo* es = NULL;
	AudioESInfo* aes = NULL;
	VideoESInfo* ves = NULL;
	VideoJob* videojob = NULL;
	AudioJob* audiojob = NULL;
	QString q_jobname;
	QString id,name;
	QString filename;
	QPixmap* pixmap = NULL;
	int i = 0;
	int pos = 0;

	if( (item = mt_listView_source->selectedItem() ) > 0){
		pos=(item->itemPos()) / POS;

		if( !(sl=project->getSources()) ){
			dprintf_err("ERROR:mt_addJob: getSources()\n");
		}
		else{
			if(! (jlProject = project->getJobList()) ){
				dprintf_err("ERROR:mt_addJob: getJobList()\n");
			}
			else{

				s=sl->getSourceByIndex( (u32) pos);
				es=s->getESInfo();
				if(!(jl=s->getJobList())){
					dprintf_err("ERROR:mt_addJob:getJobList()\n");
				}
				else{
					if(es->isAudioStream()){
						filename = "./images/Arot.png";
						pixmap = new QPixmap(filename);
						aes = (AudioESInfo*)es;
						DestinationAudioImpl *dialog = new DestinationAudioImpl(aes,jl, true, 0, this,0,TRUE,TRUE);
						dialog->exec();
						for(i=0;(audiojob=(AudioJob*)jl->getJobByIndex(i))!=NULL;i++){

							q_jobname = audiojob->getName();
							if(q_jobname.isEmpty()){
								dprintf_err("ERROR:mt_addJob:q_jobname.isEmpty()\n");
							}
							else{
								id = QString::number((int)audiojob->getID());
								if(id.isEmpty()){
									dprintf_err("ERROR:mt_addJob:id.isEmpty()\n");
								}
								else{
									if( !(jlProject->addJob(audiojob)) ){
										dprintf_err("ERROR:mt_addJob:jlProject->addJob\n");
									}
									else{
                    if(mt_listView_jobs->findItem(id,2)==0){
											QString description = audiojob->getDescription();
											insertJob(mt_listView_jobs,(Job*)audiojob,q_jobname,id, description, pixmap);
										}
                  }//else
								}//else
							}//else
						}//for

					}
					else if(es->isVisualStream()){
						filename = "./images/Vrot.png";
						pixmap = new QPixmap(filename);
						ves = (VideoESInfo*)es;
						DestinationVideoImpl *dialog = new DestinationVideoImpl(ves,jl,TRUE,0,this,0,TRUE,TRUE);
						dialog->exec();
						for(i=0;(videojob=(VideoJob*)jl->getJobByIndex(i))!=NULL;i++){

							q_jobname = videojob->getName();
							if(q_jobname.isEmpty()){
								dprintf_err("ERROR:mt_addJob:q_jobname.isEmpty()\n");
							}
							else{
								id = QString::number((int)videojob->getID());
								if(id.isEmpty()){
									dprintf_err("ERROR:mt_addJob:id.isEmpty()\n");
								}
								else{
									if( !(jlProject->addJob(videojob)) ){
										dprintf_err("ERROR:mt_addJob:jlProject->addJob\n");
									}
									else{
										if(mt_listView_jobs->findItem(id,2)==0){
											QString description = videojob->getDescription();
											insertJob(mt_listView_jobs,(Job*)videojob,q_jobname,id, description, pixmap);

										}//if
									}//else
								}//else

							}//else
						}//for
					}//else if
				}//else
			}//else
		}//else
	}//if
	else{
		dprintf_err("No source selected\n");
	}//else


}//mt_addJob

void MultiTranscoderImpl::mt_delJob()
{
	QListViewItem *item = NULL;
	QListViewItem *itemS = NULL;
	QString jobID;
	Job* j = NULL;
	Source *s = NULL;
	SourceList* sl = NULL;
	JobList* jl = NULL;
  JobList* pjl = NULL;
	int pos = 0;

	SystemFileList* sysLi = NULL;
	SystemFile* sys = NULL;

	QListViewItem* sysItem = NULL;
	QListViewItem* delItem = NULL;
	char* name;
	int childs;


	if( (item = mt_listView_jobs->selectedItem () ) > 0){
		jobID=item->text(2);

		DeleteJobImpl *dialog = new DeleteJobImpl(this,0,TRUE,TRUE);
		int i =dialog->exec();

		if(dialog != NULL){
			delete dialog;
			dialog = NULL;
		}

		if(i == -1){
			return;
		}
		else{
			if( (itemS = mt_listView_source->selectedItem() ) > 0){
				pos=(itemS->itemPos()) / POS;

				if( !(sl=project->getSources()) ){
					dprintf_err("ERROR:mt_delJob: getSources()\n");
				}
				else{
					if( !(s=sl->getSourceByIndex( (u32)pos )) ){
						dprintf_err("ERROR:mt_delJob: getSourceByIndex()\n");
					}
					else{
						if( !(jl = s->getJobList()) ){
							dprintf_err("ERROR:mt_delJob: getJobList()\n");
						}
						else{
							if( !(j=jl->getJob( (u32)jobID.toInt() )) ){
								dprintf_err("ERROR:mt_delJob: getJob()\n");
							}
							else{
								if( !(jl->delJob(j)) ){
									dprintf_err("ERROR:mt_delJob: jl->delJob()\n");
								}
								else{
                  if( !(pjl = project->getJobList()) ){
                    dprintf_err("ERROR:mt_delJob: getJobList()\n");
                  }
                  else{
                    if( !(pjl->delJob(j)) ){
									    dprintf_err("ERROR:mt_delJob: pjl->delJob()\n");
								    }
								    else{
                    
      								mt_listView_jobs->takeItem(item);

      								//update mt_systemfileView
      								if( !(sysLi = project->getSystemFileList()) ){
      									dprintf_err("ERROR:mt_checkJob: getSystemFileList\n");
      								}
      								else{
      									for(int i=0;(sys=sysLi->getSysFileByIndex(i)) !=NULL ;i++){
      										if( !(name = (char*)sys->getName()) ){
      											dprintf_err("ERROR:mt_checkJob: sys->getName()\n");
      										}
      										else{
      											if( (sysItem = mt_listView_systemfile->findItem((QString)name,1)) == 0){
      												dprintf_err("ERROR:mt_checkJob: Systemfile not found\n");
      											}
      											else{
      												if( (childs = sysItem->childCount()) > 0){
      													delItem = sysItem->firstChild();
      													for(int k=0;k < childs;k++){
      														if(delItem != 0){
      															if( ((QString::compare(delItem->text(2),jobID))) == 0){
      																//File found, maybe a job, which was edit

      																if( (delItem->text(6)).isEmpty() ){
      																if( !(sys->removeJob(j)) ){
      																		dprintf_err("ERROR:mt_delSys: removeJob\n");
      }
      else{																		sysItem->takeItem(delItem);
      }//else
      }//if
      															}//if
      															delItem = delItem->nextSibling();
      														}//if
      													}//for

      												}//if

      											}//else
      										}//else
      									}//for

      								}//else
                    }//else
                  }//else
								}//else
							}//else
						}//else
					}//else
				}//else
			}//if
		}//else
	}//if
	else{
		dprintf_err("ERROR:No source selected\n");
	}//else
}//mt_delJob

void MultiTranscoderImpl::mt_editJob()
{
	QListViewItem *item = NULL;
	QListViewItem *itemS = NULL;
	ESInfo* es = NULL;
	AudioESInfo* aes = NULL;
	VideoESInfo* ves = NULL;
	VideoJob* videojob = NULL;
	AudioJob* audiojob = NULL;
	QPixmap* pixmap = NULL;
	QString q_jobname,filename, id, jobID;
	Source *s = NULL;
	SourceList* sl = NULL;
	JobList* jl = NULL;
	int pos = 0;


	if( (item = mt_listView_jobs->selectedItem () ) > 0){
		jobID=item->text(2);

		if( (itemS = mt_listView_source->selectedItem() ) > 0){
			pos=(itemS->itemPos()) / POS;

			if( !(sl=project->getSources()) ){
				dprintf_err("ERROR:mt_editJob: getSources()\n");
			}
			else{

				if( !(s=sl->getSourceByIndex( (u32)pos )) ){
					dprintf_err("ERROR:mt_editJob: getSourceByIndex()\n");
				}
				else{
					if(!(jl=s->getJobList())){
						dprintf_err("ERROR:mt_editJob:getJobList()\n");
					}
					else{
						es=s->getESInfo();
						if(es->isAudioStream()){

							filename = "./images/Arot.png";
							pixmap = new QPixmap(filename);
							aes = (AudioESInfo*)es;
							DestinationAudioImpl *dialog = new DestinationAudioImpl(aes,jl, false, (u32)jobID.toInt(), this,0,TRUE,TRUE);
							int i = dialog->exec();
							if(i == -1){
								//cancel
								return;
							}
							else{
								mt_listView_jobs->takeItem(item);

								for(i=0;(audiojob=(AudioJob*)jl->getJobByIndex(i))!=NULL;i++){

									q_jobname = audiojob->getName();
									if(q_jobname.isEmpty()){
										dprintf_err("ERROR:mt_editJob:q_jobname.isEmpty()\n");
									}
									else{
										id = QString::number((int)audiojob->getID());
										if(id.isEmpty()){
											dprintf_err("ERROR:mt_editJob:id.isEmpty()\n");
										}
										else{
											if(mt_listView_jobs->findItem(id,2)==0){
												QString description = audiojob->getDescription();
												insertJob(mt_listView_jobs,(Job*)audiojob,q_jobname,id,description,pixmap);
												mt_checkJob(mt_listView_systemfile, mt_listView_jobs,(Job*)audiojob,q_jobname,id,description,pixmap);
											}
										}//else
									}//else
								}//for
							}//else
						}//if
						else if(es->isVisualStream()){

							filename = "./images/Vrot.png";
							pixmap = new QPixmap(filename);
							ves = (VideoESInfo*)es;
							DestinationVideoImpl *dialog = new DestinationVideoImpl(ves,jl,false,(u32)jobID.toInt(),this,0,TRUE,TRUE);
							int i = dialog->exec();
							if(i == -1){
								//cancel
								return;
							}
							else{
								mt_listView_jobs->takeItem(item);

								for(i=0;(videojob=(VideoJob*)jl->getJobByIndex(i))!=NULL;i++){
									q_jobname = videojob->getName();
									if(q_jobname.isEmpty()){
										dprintf_err("ERROR:mt_editJob:q_jobname.isEmpty()\n");
									}
									else{
										id = QString::number((int)videojob->getID());
										if(id.isEmpty()){
											dprintf_err("ERROR:mt_editJob:id.isEmpty()\n");
										}
										else{
											if(mt_listView_jobs->findItem(id,2)==0){
												QString description = videojob->getDescription();
												insertJob(mt_listView_jobs,(Job*)videojob,q_jobname,id,description,pixmap);
												mt_checkJob(mt_listView_systemfile, mt_listView_jobs,(Job*)videojob,q_jobname,id,description,pixmap);
											}//if
										}//else

									}//else
								}//for
							}//else
						}//else if
					}//else
				}//else
			}//else
		}//if
	}//if
	else{
		dprintf_err("ERROR:No source selected\n");
	}//else

}//mt_editJob

void MultiTranscoderImpl::mt_addSys()
{
	dialog = new EditSystemFileImpl(this,0,TRUE,TRUE, mt_listView_systemfile, 0);
	dialog->exec();

	if(dialog != NULL){
		delete dialog;
		dialog = NULL;
	}
}//mt_addSys

void MultiTranscoderImpl::mt_delSys()
{
	SystemFileList *sysLi = NULL;
	JobList *jl = NULL;
	SourceList *sl = NULL;
	SystemFile *sys = NULL;

	QString name, pfad, id;
	QListViewItem *qlvi = NULL;
	QListViewItem *parent = NULL;
	Source* src = NULL;
	Job *j = NULL;

	if( (qlvi = mt_listView_systemfile->selectedItem() ) > 0){
		name=qlvi->text(1);

		if(name.isEmpty()){
			dprintf_err("ERROR:mt_delSys: name.isEmpty()\n");
		}
		else{
			//is a systemfile
			if(qlvi->parent() == 0){
				//remove Item from the SystemFileList of the project
				if( !(sysLi = project->getSystemFileList()) ){
					dprintf_err("ERROR:mt_delSys: getSystemFileList\n");
				}
				else{
					if( !(sys = sysLi->get(name.latin1())) ){
						dprintf_err("ERROR:mt_delSys: get\n");
					}
					else{
						if( !(sysLi->del(sys)) ){
							dprintf_err("ERROR:mt_delSys: del\n");
						}
						else{
							mt_listView_systemfile->takeItem(qlvi);
						}
					}//else
				}//else
			}//if
			//is a job or source
			else{
				parent = qlvi->parent();
				name=parent->text(1);
				//Name of the systemfile of
				if(name.isEmpty()){
					dprintf_err("ERROR:mt_delSys: name.isEmpty()\n");
				}
				else{
					if( !(sysLi = project->getSystemFileList()) ){
						dprintf_err("ERROR:mt_delSys: getSystemFileList\n");
					}
					else{
						if( !(sys = sysLi->get(name.latin1())) ){
							dprintf_err("ERROR:mt_delSys: get\n");
						}
						else{
							pfad = qlvi->text(6);
							if((pfad.isEmpty())){
								if( !(jl = project->getJobList()) ){
									dprintf_err("ERROR:mt_delSys: getJobList\n");
								}
								else{
									id = qlvi->text(2);
									if(id.isEmpty()){
											dprintf_err("ERROR:mt_delSys:id.isEmpty()\n");
									}
									else{
										if( !(j = jl->getJob((u32)id.toInt())) ){
											dprintf_err("ERROR:mt_delSys: jl->get\n");
										}
										else{
											if( !(sys->removeJob(j)) ){
												dprintf_err("ERROR:mt_delSys: removeJob\n");
											}
											else{
												parent->takeItem(qlvi);
											}
										}//else
									}//else
								}//else

							}//if
							else{
								if( !(sl = project->getSources()) ){
									dprintf_err("ERROR:mt_delSys: getSources\n");
								}
								else{
									id = qlvi->text(2);
									if(id.isEmpty()){
										dprintf_err("ERROR:mt_delSys:id.isEmpty()\n");
									}
									else{
										if( !(src = sl->get(pfad.latin1(), (u32)id.toInt())) ){
											dprintf_err("ERROR:mt_delSys: sl->get\n");
										}
										else{
											if( !(sys->removeSource(src)) ){
												dprintf_err("ERROR:mt_delSys: removeSource\n");
											}
											else{
												parent->takeItem(qlvi);
											}

										}//else
									}//else

								}//else
							}//else
						}//else
					}//else
				}//else
			}//else

		}//else
	}//if
}//mt_delSys

void MultiTranscoderImpl::mt_editSys()
{
	QListViewItem *qlvi = NULL;

	if( (qlvi = mt_listView_systemfile->selectedItem() ) > 0){

		if(qlvi->parent() == 0){
			dialog = new EditSystemFileImpl(this,0,TRUE,TRUE, mt_listView_systemfile, qlvi);
			dialog->exec();

			if(dialog != NULL){
				delete dialog;
				dialog = NULL;
			}
		}//if
		else{
			dprintf_err("ERROR:mt_editSys: Is not a systemfile\n");
		}

	}//if
}//mt_editSys


void MultiTranscoderImpl::mt_loadProject()
{
  SourceList* sl = NULL;
  Source* s = NULL;
  JobList* jl = NULL;
  Job* j = NULL;
  SystemFileList* sysLi = NULL;
  SystemFile* sys = NULL;
  ESInfo* es = NULL;

  QPixmap* pixmap = NULL;
  QString filename;

	QFileDialog* fd = new QFileDialog(this, "file dialog", TRUE);
	QString name = fd->getOpenFileName(NULL, "Project (*.pro)", this,"Open file dialog","Open a project" );
	if(name.isEmpty()){
		dprintf_err("no file selected\n");
	}
	else{
		if( !(project->load(name.latin1())) ){
			dprintf_err("ERROR:mt_loadProject: load\n");
		}
    else{
      
      // ---------- Begin: load Sources--------------------------//
      if( !(sl = project->getSources()) ){
        dprintf_err("ERROR:mt_loadProject: getSources()\n");
      }//if
      else{
        for(int i=0;(s=sl->getSourceByIndex(i)) !=NULL ;i++){
          if( !(es = s->getESInfo()) ){
            dprintf_err("ERROR:mt_loadProject: getESInfo()\n");
          }
          else{
            
            QListViewItem *last = mt_listView_source->lastItem();
    				QListViewItem *item = NULL;

    				QString es_id = QString::number((int)es->getStreamId());
    				QString type = "Unknown";
    				if(es->isAudioStream()){
    					filename = "./images/Agruen.png";
    					pixmap = new QPixmap(filename);
    				}
    				else if(es->isVisualStream()){
    					filename = "./images/Vgruen.png";
    					pixmap = new QPixmap(filename);
    				}//else if

            char* file = (char*)malloc(MAX_STR_LEN);
            QString in_sysfile = (char*)s->getFileName();
            int i = in_sysfile.findRev('/');	//linux only
						if(i < 0){
 							dprintf_err("ERROR:mt_addSource:No / in the path\n");
 							free(file);
 							file = (char*)in_sysfile.latin1();
						}//if
            else{
								if(MAX_STR_LEN < snprintf(file, MAX_STR_LEN,  "%s", (char*)(in_sysfile.latin1() + i + 1 )))
								{
									dprintf_err("ERROR:mt_addSource: snprintf\n");
								}//if
						}//else

    				QString start = QString("0:0:0");
    				int d = ((int)es->getDurationInMs())/1000;
    				QString end = durationTime(d);
    				QString description = QString(es->getDescriptionText());
    
    				item = new QListViewItem( mt_listView_source, last, QString(""), QString(file), es_id, start, end, description, in_sysfile);
    				item->setPixmap(0,*pixmap);
    				mt_listView_source->ensureItemVisible(item);
    				item->setDragEnabled (TRUE);
    				mt_listView_source->setColumnWidth(6,0);
        
          }//else
        }//for
      }//else
      // ---------- End: load Sources--------------------------//

      // ---------- Begin: load Jobs--------------------------//
      //same functionality as checkSource
      // ---------- End: load Jobs--------------------------//

      // ---------- Begin: load Systemfile--------------------------//
      if( !(sysLi = project->getSystemFileList()) ){
        dprintf_err("ERROR:mt_loadProject: getSystemFileList()\n");
      }//if
      else{
        for(int i=0;(sys=sysLi->getSysFileByIndex(i)) !=NULL ;i++){
          
          QString sysname = (QString)sys->getName();
          QString filename = "./images/systemfile.png";
					QPixmap* pixmap = new QPixmap(filename);
					QListViewItem* last = mt_listView_systemfile->lastItem();
					QListViewItem* it = new QListViewItem(mt_listView_systemfile,last,QString(""),sysname);
					it->setPixmap(0,*pixmap);
					mt_listView_systemfile->ensureItemVisible(it);
					it->setDragEnabled (TRUE);

          //filling the jobs of the systemfile
          if(! (jl = sys->getJobList()) ){
            dprintf_err("ERROR:mt_loadProject: getSystemFileList()\n");
          }
          else{
            for(int i=0;(j=jl->getJobByIndex(i)) !=NULL ;i++){
              QString jobName = (QString)j->getName();
              QString idJob = QString::number((int)j->getID());
              
              if( !(s = (Source*)j->getSource()) ){
					      dprintf_err("ERROR:mt_delJob: getSource()\n");
					    }
					    else{
					      QString description;
					      es=s->getESInfo();
 								if(es->isAudioStream()){
 									filename = "./images/Arot.png";
 									pixmap = new QPixmap(filename);
 									description = ((AudioJob*)j)->getDescription();
 								}
 								else if(es->isVisualStream()){
 									filename = "./images/Vrot.png";
 									pixmap = new QPixmap(filename);
 									description = ((VideoJob*)j)->getDescription();
 								}
 								JobToSystemfile(mt_listView_systemfile, it, j, jobName, idJob, description, pixmap);
 							}//else
            }//for
          }//else

          //filling the sources of the systemfile
          if(! (sl = sys->getSourceList()) ){
            dprintf_err("ERROR:mt_loadProject: getSystemFileList()\n");
          }
          else{
            for(int i=0;(s=sl->getSourceByIndex(i)) !=NULL ;i++){
              QString pfad = (QString)s->getFileName();           

              QString description;
				      es=s->getESInfo();
 							if(es->isAudioStream()){
 								filename = "./images/Agruen.png";
 								pixmap = new QPixmap(filename);
 							}
 							else if(es->isVisualStream()){
 								filename = "./images/Vgruen.png";
 								pixmap = new QPixmap(filename);
 							}
              QString idSource = QString::number((int)s->getID());
              SourceToSystemfile(mt_listView_systemfile,it, s, pfad, idSource, pixmap, pfad);
        
		SourceToSystemfile(mt_listView_systemfile,it, s, pfad, idSource, pixmap, pfad);
            }//for
          }//else
        }//for
      }//else
      // ---------- End: load Systemfile--------------------------//


      
    }//else
	}//else
}//mt_loadProject

void MultiTranscoderImpl::mt_saveProject()
{

  QString cap = this->caption();
  int j = cap.findRev(':');
    if(j < 0){
    	QFileDialog* fd = new QFileDialog(this, "file dialog", TRUE);
    	QString name = fd->getSaveFileName(NULL, "Project (*.pro)", this,"Save file dialog","Save the file" );
    	if(!name){
    		dprintf_err("no file selected\n");
    	}
    	else{
    		int n = name.find(".pro");
    		if( n < 0){
    			name.append(".pro");
    		}
    		if( !(project->save(name.latin1())) ){
    			dprintf_err("ERROR:mt_saveProject: save\n");
    		}
    		else{
    			project->setName(name.latin1());

          QString file;
          int i = name.findRev('/');
    			if( i > 0){
            file =  (name.right(name.length() - i - 1)).latin1();
            QString m = "MultiTranscoder: ";
            file = m.append(file);
            this->setCaption(file);
    			}//if
          else{
            this->setCaption(name);
          }//else

    		}//else
    	}//else
  }//if
  else{
    if( !(project->save(project->getName())) ){
    			dprintf_err("ERROR:mt_saveProject: save\n");
    		}

    }

}//mt_saveProject

void MultiTranscoderImpl::mt_renameProject()
{
	ProjectNameImpl *dialog = NULL;
	dialog = new ProjectNameImpl(this,0,TRUE,TRUE);
	dialog->exec();

	if(dialog != NULL){
		delete dialog;
		dialog = NULL;
	}//if
}//mt_renameProject

void MultiTranscoderImpl::mt_workingDir()
{
	WorkingDirectoryImpl *dialog = NULL;
	dialog = new WorkingDirectoryImpl(this,0,TRUE,TRUE);
	dialog->exec();

	if(dialog != NULL){
		delete dialog;
		dialog = NULL;
	}//if
}//mt_renameProject

void MultiTranscoderImpl::mt_transcode()
{
	ProgressImpl *dialog = NULL;
	dialog = new ProgressImpl(0,project,this,0,TRUE,TRUE);
	dialog->exec();

	if(dialog != NULL){
		delete dialog;
		dialog = NULL;
	}
}//mt_transcode

void MultiTranscoderImpl::mt_saveQuitProject()
{
	mt_saveProject();
}

void MultiTranscoderImpl::mt_cancel()
{
    DeleteProjectImpl *dialog = new DeleteProjectImpl(this,0,TRUE,TRUE);
		int i =dialog->exec();

		if(dialog != NULL){
			delete dialog;
			dialog = NULL;
		}

		if(i == -1){
			return;
		}
		else{
      this->close();
    }
}

void MultiTranscoderImpl::mt_downFile()
{
	SystemFileList *sysLi = NULL;
	SystemFile *sys = NULL;
	Source *s = NULL;
	SourceList* sl = NULL;
	Job* j = NULL;
	JobList* jl = NULL;
	ESInfo* es = NULL;

	QListViewItem *itemS = NULL;	//Source
	QListViewItem *itemD = NULL;	//Destination
	QListViewItem* parent = NULL;
	QListViewItem *help = NULL;
	QPixmap *pixmap = NULL;
	QString name, sys_name, filename;
	QString idJob, id, pfad;
	int idSource = 0;
	int pos = 0;
	int flag = 0;	//default for job


	if( (itemD = mt_listView_systemfile->selectedItem()  ) > 0){
		//Subitems of subitems shouldn?t be possible. Only one hierarchy
		if( (parent = itemD->parent()) != 0){
			itemD = parent;
			if(itemD == 0){
				dprintf_err("ERROR:mt_downFile: parent\n");
			}
		}

		//for selected Job
		if( (itemS = mt_listView_jobs->selectedItem()  ) > 0){
			name = itemS->text(1);
			idJob = itemS->text(2);
			flag = 0;
		}//if
		//for selected Source
		else if( (itemS = mt_listView_source->selectedItem()  ) > 0){
			name = itemS->text(1);
			id = itemS->text(2);
			pfad = itemS->text(6);
			idSource=(itemS->itemPos())/POS;
			flag = 1;	//for source
		}//else if

		sys_name=itemD->text(1);
		if(sys_name.isEmpty() || name.isEmpty()){
			dprintf_err("ERROR:mt_downFile:isEmpty()\n");
		}
		else{
			if( !(sysLi = project->getSystemFileList()) ){
				dprintf_err("ERROR:mt_downFile: getSystemFileList\n");
			}
			else{
				if( !(sys = sysLi->get(sys_name.latin1())) ){
					dprintf_err("ERROR:mt_downFile: get\n");
				}
				else{
					if( !(sl=project->getSources()) ){
						dprintf_err("ERROR:mt_downFile: getSources()\n");
					}
					else{
						if(flag == 1){
						//is a source

							//check if file is already included
							if( (help = itemD->firstChild()) == 0 ){
							//systemfile hasn`t source or job included
								if( !(s = sl->getSourceByIndex((u32)idSource) )){
									dprintf_err("ERROR:mt_downFile: getSourceByIndex\n");
								}
								else{
									if( !(sys->addSource(s)) ){
										dprintf_err("ERROR:mt_downFile: addSource\n");
									}
									else{
										es=s->getESInfo();
										if(es->isAudioStream()){
											filename = "./images/Agruen.png";
											pixmap = new QPixmap(filename);
										}
										else if(es->isVisualStream()){
											filename = "./images/Vgruen.png";
											pixmap = new QPixmap(filename);
										}


										SourceToSystemfile(mt_listView_systemfile,itemD, s, name, id, pixmap, pfad);
									}//else
								}//else
							}//if
							else if( ((QString::compare(help->text(1),name)) == 0) && ((QString::compare(help->text(2),id)) == 0) ){
								dprintf_err("ERROR:mt_downFile:file already exists\n");
							}
							else{
							//if Systemfile has more then one source
								int exist = 0;
								while( ((help = help->nextSibling()) != 0) && (exist == 0)){
									if( ((QString::compare(help->text(1),name)) == 0) && ((QString::compare(help->text(2),QString::number(idSource)) == 0) )){
										exist=1;
										dprintf_err("ERROR:mt_downFile:file already exists!\n");
									}
								}//while
								if(help == 0 && exist == 0){
									if( !(s = sl->getSourceByIndex((u32)idSource)) ){
										dprintf_err("ERROR:mt_downFile: getSourceByIndex THIS!!!!\n");
									}
									else{
										if( !(sys->addSource(s)) ){
											dprintf_err("ERROR:mt_downFile: addSource\n");
										}
										else{
											//s = (Source*)j->getSource();
											es=s->getESInfo();
											if(es->isAudioStream()){
												filename = "./images/Agruen.png";
												pixmap = new QPixmap(filename);
											}
											else if(es->isVisualStream()){
												filename = "./images/Vgruen.png";
												pixmap = new QPixmap(filename);
											}

											SourceToSystemfile(mt_listView_systemfile,itemD, s, name, id, pixmap, pfad);
										}//else
									}//else
								}//if
							}//else
						}//if flag = 1
						else{
						//is a job, flag=0

							if( (itemS = mt_listView_source->selectedItem() ) > 0){
								pos=(itemS->itemPos()) / POS;

								if( !(s=sl->getSourceByIndex( (u32)pos )) ){
									dprintf_err("ERROR:mt_delJob: getSourceByIndex()\n");
								}
								else{
									if( !(jl = s->getJobList()) ){
										dprintf_err("ERROR:mt_delJob: getJobList()\n");
									}
									else{

										//check if file is already included
										if( (help = itemD->firstChild()) == 0 ){
										//systemfile hasn`t source or job included

											if( !(j = jl->getJob((u32)idJob.toInt())) ){
												dprintf_err("ERROR:mt_downFile: getJob\n");
											}
											else{
												if( !(sys->addJob(j)) ){
													dprintf_err("ERROR:mt_downFile: addSource\n");
												}
												else{
													if( !(s = (Source*)j->getSource()) ){
														dprintf_err("ERROR:mt_delJob: getSource()\n");
													}
													else{
														QString description;
														es=s->getESInfo();
														if(es->isAudioStream()){
															filename = "./images/Arot.png";
															pixmap = new QPixmap(filename);
															description = ((AudioJob*)j)->getDescription();
														}
														else if(es->isVisualStream()){
															filename = "./images/Vrot.png";
															pixmap = new QPixmap(filename);
															description = ((VideoJob*)j)->getDescription();
														}

														JobToSystemfile(mt_listView_systemfile, itemD, j, name, idJob, description, pixmap);
													}//else
												}//else
											}//else
										}//if
										else if( ((QString::compare(help->text(1),name)) == 0) && ((QString::compare(help->text(2),idJob)) == 0) ){
											dprintf_err("ERROR:mt_downFile:file already exists\n");
										}
										else{
										//if Systemfile has more then one source/job
											int exist = 0;
											while( ((help = help->nextSibling()) != 0) && (exist == 0)){
												if( ((QString::compare(help->text(1),name)) == 0) && ((QString::compare(help->text(2),idJob)) == 0) ){
													exist=1;
													dprintf_err("ERROR:mt_downFile:file already exists!\n");
												}
											}
											if(help == 0 && exist == 0){
												if( !(j = jl->getJob((u32)idJob.toInt())) ){
													dprintf_err("ERROR:mt_downFile: getJob\n");
												}

												else{
													if( !(sys->addJob(j)) ){
														dprintf_err("ERROR:mt_downFile: addSource\n");
													}
													else{
														QString description;
														s = (Source*)j->getSource();
														es=s->getESInfo();
														if(es->isAudioStream()){
															filename = "./images/Arot.png";
															pixmap = new QPixmap(filename);
															description = ((AudioJob*)j)->getDescription();
														}
														else if(es->isVisualStream()){
															filename = "./images/Vrot.png";
															pixmap = new QPixmap(filename);
															description = ((VideoJob*)j)->getDescription();
														}

														JobToSystemfile(mt_listView_systemfile, itemD, j, name, idJob, description, pixmap);
													}//else
												}//else
											}//if
										}//else
									}//else
								}//else
							}//if
						}//else
					}//else
				}//else
        printf("\033[33m%s\033[0m\n",sysLi->toString());
			}//else
		}//else
	}//if
	else{
		dprintf_err("ERROR:No source selected\n");
	}//else
}//mt_downFile


void MultiTranscoderImpl::mt_check_Source(QListViewItem* item)
{

  if(!item){
    //control activation of buttons and menu items
    mt_button_delete_source->setDisabled(TRUE);
    mt_button_info_source->setDisabled(TRUE);
    mt_button_new_jobs->setDisabled(TRUE);

    delete_Source->setEnabled(FALSE);
    info_Source->setEnabled(FALSE);
    new_Job->setEnabled(FALSE);

    if( (mt_listView_jobs->selectedItem()  ) == 0){
      mt_button_delete_jobs->setDisabled(TRUE);
      mt_button_edit_jobs->setDisabled(TRUE);
    }
    else{
      if( (mt_listView_systemfile->selectedItem()  ) > 0){
        mt_button_down->setDisabled(FALSE);
      }
    }

    mt_button_down->setDisabled(TRUE);
    
	  dprintf_err("ERROR:mt_check_Source:item\n");
		return;
	}
	else{
    //control activation of buttons and menu items
    mt_button_delete_source->setDisabled(FALSE);
    mt_button_info_source->setDisabled(FALSE);
    mt_button_new_jobs->setDisabled(FALSE);

    delete_Source->setEnabled(TRUE);
    info_Source->setEnabled(TRUE);
    new_Job->setEnabled(TRUE);

    if( (mt_listView_systemfile->selectedItem()  ) > 0){
      mt_button_down->setDisabled(FALSE);
    }
    
    mt_button_delete_jobs->setDisabled(TRUE);
    mt_button_edit_jobs->setDisabled(TRUE);
    delete_Job->setEnabled(FALSE);
    edit_Job->setEnabled(FALSE);

		int pos,i, childs;
		JobList* jl = NULL;
		Job* j = NULL;
		Source* s = NULL;
		SourceList* sl = NULL;
		QString jID,filename;
		QListViewItem* delItem = NULL;
		ESInfo* es = NULL;
		QPixmap* pixmap = NULL;

		if( !(sl=project->getSources()) ){
			dprintf_err("ERROR:mt_check_Source: getSources()\n");
		}
		else{

			pos=(item->itemPos()) / POS;
			if( !(s=sl->getSourceByIndex( (u32)pos )) ){
				dprintf_err("ERROR:mt_check_Source: getSourceByIndex()\n");
			}
			else{
				if( !(jl=s->getJobList())){
					dprintf_err("ERROR:mt_check_Source: getJobList()\n");
				}
				else{
					if((childs = mt_listView_jobs->childCount()) > 0){

						for(i=0;i < childs;i++){
							delItem = mt_listView_jobs->firstChild();
							mt_listView_jobs->takeItem(delItem);
						}//for
					}//if
;
					for(i=0;(j=jl->getJobByIndex(i)) !=NULL ;i++){

						QString q_jobname = (QString)j->getName();
						QString id = QString::number((int)j->getID());
						QString description;

						es=s->getESInfo();
						if(es->isAudioStream()){
							filename = "./images/Arot.png";
							pixmap = new QPixmap(filename);
							description = ((AudioJob*)j)->getDescription();
						}
						else if(es->isVisualStream()){
							filename = "./images/Vrot.png";
							pixmap = new QPixmap(filename);
							description = ((VideoJob*)j)->getDescription();
						}

						insertJob(mt_listView_jobs, j, q_jobname, id, description, pixmap);
					}//for
				}//else
			}//else
		}//else
	}//else
}//mt_check_Source


void mt_checkJob(QListView* qlvSys, QListView* qlv, Job* j, QString jobname, QString idJob, QString description, QPixmap* pixmap)
{
	SystemFileList* sysLi = NULL;
	SystemFile* sys = NULL;

	QListViewItem* sysItem = NULL;
	QListViewItem* delItem = NULL;
	char* name;
	int childs;


	if( !(sysLi = project->getSystemFileList()) ){
		dprintf_err("ERROR:mt_checkJob: getSystemFileList\n");
	}
	else{
		for(int i=0;(sys=sysLi->getSysFileByIndex(i)) !=NULL ;i++){
			if( !(name = (char*)sys->getName()) ){
				dprintf_err("ERROR:mt_checkJob: sys->getName()\n");
			}
			else{
				if( (sysItem = qlvSys->findItem((QString)name,1)) == 0){
					dprintf_err("ERROR:mt_checkJob: Systemfile not found\n");
				}
				else{
					if( (childs = sysItem->childCount()) > 0){
						delItem = sysItem->firstChild();
						for(int k=0;k < childs;k++){;
							if(delItem != 0){
								if( ((QString::compare(delItem->text(2),idJob))) == 0){
									//File found, maybe a job, which was edit
									if( (delItem->text(6)).isEmpty() ){
										//is a job
										sysItem->takeItem(delItem);
										JobToSystemfile(qlv, sysItem, j, jobname, idJob, description, pixmap);
									}//if
								}//if
								delItem = delItem->nextSibling();
							}//if
						}//for

					}//if

				}//else
			}//else
		}//for

	}//else
}//mt_checkJob

void updateSystemFileView(QListView* qlvSys, Source* s, JobList* jl, QString id, QString pfad)
{
  SystemFileList* sysLi = NULL;
	SystemFile* sys = NULL;
  Job* j = NULL;
  QString jobID;

	QListViewItem* sysItem = NULL;
	QListViewItem* delItem = NULL;
	char* name;
  int childs;


  //update mt_systemfileView
  if( !(sysLi = project->getSystemFileList()) ){
 		dprintf_err("ERROR:updateSystemFileView: getSystemFileList\n");
 	}
 	else{
   
    //for sources
 		for(int i=0;(sys=sysLi->getSysFileByIndex(i)) !=NULL ;i++){
 			if( !(name = (char*)sys->getName()) ){
 				dprintf_err("ERROR:updateSystemFileView: sys->getName()\n");
 			}
 			else{
 				if( (sysItem = qlvSys->findItem((QString)name,1)) == 0){
 					dprintf_err("ERROR:updateSystemFileView: Systemfile not found\n");
 				}//if
 				else{
 					if( (childs = sysItem->childCount()) > 0){
 						delItem = sysItem->firstChild();
 						for(int k=0;k < childs;k++){
 							if(delItem != 0){
                
                if( !( (delItem->text(6)).isEmpty()) ){
 								  if( ((QString::compare(delItem->text(2),id))) == 0){
                     if( ((QString::compare(delItem->text(6),pfad))) == 0){
 											if( !(sys->removeSource(s)) ){
 											  dprintf_err("ERROR:updateSystemFileView: removeSource\n");
                       }
                       else{
                         sysItem->takeItem(delItem);
                       }//else
                     }//if
 									}//if
                }//if
                delItem = delItem->nextSibling();
              }//if 
 						}//for
 					}//if
 				}//else
 			}//else
 		}//for

   //for jobs of the deleted source file
   for(int k=0;(sys=sysLi->getSysFileByIndex(k)) !=NULL ;k++){
 			if( !(name = (char*)sys->getName()) ){
 				dprintf_err("ERROR:updateSystemFileView: sys->getName()\n");
 			}
 			else{
 				if( (sysItem = qlvSys->findItem((QString)name,1)) == 0){
 					dprintf_err("ERROR:updateSystemFileView: Systemfile not found\n");
 				}//if
 				else{

          if(!jl){
            dprintf_err("ERROR:updateSystemFileView: No Jobs!\n");
 				  }
 				  else{
            for(int p=0;(j=jl->getJobByIndex(p)) !=NULL ;p++){
              jobID = QString::number((int)j->getID());

              if( (childs = sysItem->childCount()) > 0){
 						    delItem = sysItem->firstChild();
 						    for(int m=0;m < childs;m++){
 							    if(delItem != 0){
            
                    if( ((QString::compare(delItem->text(2),jobID))) == 0){
                       if( (delItem->text(6)).isEmpty() ){
 
                        if( !(sys->removeJob(j)) ){
  										    dprintf_err("ERROR:updateSystemFileView: removeJob\n");
                        }
                        else{
                          sysItem->takeItem(delItem);
                        }//else
                      }//if
                    }//if
                    delItem = delItem->nextSibling();
                  }//if
 						    }//for
 					    }//if
            }//for
 				  }//else
 			  }//else
      }//else
 		}//for 
 	}//else
}//updateSystemFileView

QString durationTime(int d)
{
	int sec1 = (d%3600)%60;
	int min1 = (d%3600)/60;
	int std1 = d/3600;
	QString std = QString::number(std1);
	QString min = QString::number(min1);
	QString sec = QString::number(sec1);
	QString end = std+":"+min+":"+sec;

	return end;
}//durationTime

void insertJob(QListView* qlv, Job* j, QString q_jobname, QString id, QString description, QPixmap* pixmap){

	ESInfo* es = NULL;
	Source* s = NULL;

	s = (Source*)j->getSource();
	es=s->getESInfo();
	QString start = QString("0:0:0");
	int d = ((int)es->getDurationInMs())/1000;
	QString end = durationTime(d);
	QListViewItem *last = qlv->lastItem();
	QListViewItem *item = NULL;
	item = new QListViewItem(qlv, last, QString(""), q_jobname,id, start, end, description);
	item->setPixmap(0,*pixmap);
	qlv->ensureItemVisible(item);
	item->setDragEnabled (TRUE);
}//insertJob

void SourceToSystemfile(QListView* qlv, QListViewItem* qlvi, Source* s, QString q_jobname, QString id, QPixmap* pixmap, QString pfad){

	ESInfo* es = NULL;	//QListViewItem *last = qlvi->lastItem();

	es=s->getESInfo();
	QString start = QString("0:0:0");
	int d = ((int)es->getDurationInMs())/1000;
	QString end = durationTime(d);
	QString description = (QString)es->getDescriptionText();
	QListViewItem *last = qlv->lastItem();
	QListViewItem *item = NULL;
	item = new QListViewItem(qlvi, last, QString(""), q_jobname,id, start, end, description, pfad);
	item->setPixmap(0,*pixmap);
	qlv->ensureItemVisible(item);
	item->setDragEnabled (TRUE);
	qlv->setColumnWidth(6,0);
}//SourceToSystemfile

void JobToSystemfile(QListView* qlv, QListViewItem* qlvi, Job* j, QString q_jobname, QString id, QString description, QPixmap* pixmap){

	ESInfo* es = NULL;
	Source* s = NULL;

	s = (Source*)j->getSource();
	es=s->getESInfo();
	QString start = QString("0:0:0");
	int d = ((int)es->getDurationInMs())/1000;
	QString end = durationTime(d);
	QListViewItem *last = qlv->lastItem();
	QListViewItem *item = NULL;
	item = new QListViewItem(qlvi, last, QString(""), q_jobname,id, start, end, description);
	item->setPixmap(0,*pixmap);
	qlv->ensureItemVisible(item);
	item->setDragEnabled (TRUE);
}//JobToSystemfile


void MultiTranscoderImpl::mt_control_Job_button(QListViewItem* item){
  if(!item){
    mt_button_delete_jobs->setDisabled(TRUE);
    mt_button_edit_jobs->setDisabled(TRUE);
    
    delete_Job->setEnabled(FALSE);
    edit_Job->setEnabled(FALSE);
   
		dprintf_err("ERROR:mt_control_Job_button:item\n");
		return;
	}
	else{
    mt_button_delete_jobs->setDisabled(FALSE);
    mt_button_edit_jobs->setDisabled(FALSE);

    delete_Job->setEnabled(TRUE);
    edit_Job->setEnabled(TRUE);

    if( (mt_listView_systemfile->selectedItem()  ) > 0){
      mt_button_down->setDisabled(FALSE);
    }
  }//else

}//mt_control_Job_button


void MultiTranscoderImpl::mt_control_Systemfile_button(QListViewItem* item){
  if(!item){
    mt_button_delete_systemfiles->setDisabled(TRUE);
    mt_button_edit_systemfiles->setDisabled(TRUE);

    delete_Systemfile->setEnabled(FALSE);
    edit_Systemfile->setEnabled(FALSE);

    mt_button_down->setDisabled(TRUE);
     
		dprintf_err("ERROR:mt_control_Systemfile_button:item\n");
		return;
	}
	else{
    mt_button_delete_systemfiles->setDisabled(FALSE);
    mt_button_edit_systemfiles->setDisabled(FALSE);

    delete_Systemfile->setEnabled(TRUE);
    edit_Systemfile->setEnabled(TRUE);

    if( (mt_listView_source->selectedItem()  ) > 0){
      mt_button_down->setDisabled(FALSE);
    }

  }//else
    

}//mt_control_Systemfile_button
