/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   Project.cpp                                                 *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/


#include "Project.hpp"
#include "SystemFile.hpp"
#include "JobList.hpp"
#include "SourceList.hpp"
#include "Source.hpp"
#include "Job.hpp"

/** constructor */
Project::Project(const char* name)
{
	dprintf_full("\033[32mProject::construct\033[0m\n");
	assert(name);
	strncpy(this->name, name, MAX_STR_LEN-1);
	this->name[MAX_STR_LEN-1] = '\0';
	this->workingDirectory = (const char*)malloc(MAX_STR_LEN);
	memset((void*)this->workingDirectory, 0, MAX_STR_LEN);
}
	
/** destructor */
Project::~Project()
{
	dprintf_full("\033[32mProject::destruct\033[0m\n");
}
	
bool Project::load(const char* name)
{
	dprintf_full("\033[32mProject::load\033[0m\n");
	assert(name);
	FILE *file = fopen(name, "r");
	assert(file);
	
	char* buf = (char*)malloc(MAX_FILE_LEN);
	while( !feof (file))
	{
		fread( (void*) buf, 1, MAX_FILE_LEN, file);
		const char* next = buf;
		char* end = NULL;
		if(next==strstr(next, PROJECT_START))
		{
			next=(const char*)next + strlen(PROJECT_START);
		}
		else
		{
			fclose(file);
			free(buf);
			dprintf("\033[31mProject::load : no projectstart found\033[0m\n");
			return false;
		}
		if(next==strstr(next, NAME_SCAN))
		{
			if(NULL!= (end=strchr(next, '\n')))
			{
				next=(const char*)next+strlen(NAME_SCAN);
				memset(this->name, 0, MAX_STR_LEN);
				if(NULL == memcpy(this->name, next, end-next))
				{
					fclose(file);
					free(buf);
					dprintf_err("\033[31mProject::fromString\033[0m\n");
					return false;
				}
				else
				{
					next = (const char*) end+1;
					if(next == strstr(next, WORK_DIR))
					{
						if(NULL!= (end=strchr(next, '\n')))
						{
						next=(const char*)next + strlen(WORK_DIR);
						strncpy((char*)this->workingDirectory, next, end-next);
						next=(const char*) end+1;
						}
						else
						{
							fclose(file);
						        free(buf);
							dprintf_err("\033[31mProject::fromString\033[0m\n");
							return false;
						}
						if(this->sourceList.fromString(next, NULL, &this->jobList, true))
						{
						next = (const char*)strstr(next, SL_END)+strlen(SL_END);
						if(this->jobList.fromStringJobs(next))
						{
							next = (const char*)strstr(next, JL_END)+strlen(JL_END);
							if(this->sysFileList.fromString(next, &(this->sourceList), &(this->jobList)))
							{
							next =(const char*)strstr(next, SYSFILELIST_END)+strlen(SYSFILELIST_END);
							
							if(next == strstr(next, PROJECT_END))
							{
								fclose(file);	
								free(buf);
								dprintf_full("\033[32mProject::fromString ok\033[0m\n");
								return true;
							}
							else
							{
								fclose(file);
								free(buf);
								dprintf_err("\033[31mProject::fromString\033[0m\n");
							 	return false;
							}
						}
						else
						{
							fclose(file);
							free(buf);
							dprintf_err("\033[31mProject::fromString\033[0m\n");
							return false;
						}
						}
						else
						{
							fclose(file);
							free(buf);
							dprintf_err("\033[31mProject::fromString\033[0m\n");
							return false;
						}
					}
					else
					{
						fclose(file);
						free(buf);
						dprintf_err("\033[31mProject::fromString\033[0m\n");
						return false;
					}
				}
			}
			}
			else
			{
				fclose(file);
				free(buf);
				dprintf_err("\033[31mProject::fromString\033[0m\n");
				return false;
			}
		}
		else
		{
			fclose(file);
			free(buf);
			dprintf_err("\033[31mProject::fromString\033[0m\n");
			return false;
		}
	}
	fclose(file);
	free(buf);
	dprintf_err("\033[31mProject::load ERROR\033[0m\n"); 
	return false;
	
}

bool Project::save(const char* name)
{
	dprintf_full("\033[32mProject::save\033[0m\n");
	assert(name);
	FILE* file = fopen(name, "w+");
	assert(file);
	char* srcList = NULL;
	char* jobL = NULL;
	char* sysFL = NULL;
	fputs(PROJECT_START, file);
	fprintf(file, NAME_PRINT, this->name);
	fprintf(file, WORK_DIR_PRINT, this->workingDirectory);
	srcList = this->sourceList.toStringESInfo();
	if(EOF != fputs(srcList, file))
		free(srcList);
	jobL = this->jobList.toStringJobs();
	printf("(%s)\n",jobL);
	if(EOF != fputs(jobL, file))
		free(jobL);
	sysFL = this->sysFileList.toString();
	if(EOF != fputs(sysFL, file))
		free(sysFL);
	if(EOF != fputs(PROJECT_END, file))
	
	fclose(file);
	dprintf_full("\033[32mProject::save ok\033[0m\n");
	return true;
}

void Project::setName(const char* name)
{
	dprintf_full("\033[32mProject::setName\033[0m\n");
	assert(name);
	strncpy(this->name, name, MAX_STR_LEN-1);
}

const char* Project::getName() const
{
	dprintf_full("\033[32mProject::getName\033[0m\n");
	return this->name;
}

void Project::setWorkingDirectory(const char* dir)
{
	dprintf_full("\033[32mProject::setWorkingDirectory\033[0m\n");
	assert(dir);
        strncpy((char*)this->workingDirectory, dir, MAX_STR_LEN-1);
}

const char* Project::getWorkingDirectory() const
{
	dprintf_full("\033[32mProject::getWorkingDirectory\033[0m\n");
        return this->workingDirectory;
}


/** returns a pointer to Source list */
SourceList* Project::getSources()
{
	dprintf_full("\033[32mProject::getSources\033[0m\n");
	return &this->sourceList;
}
	
JobList* Project::getJobList()
{
	dprintf_full("\033[32mProject::getJobList\033[0m\n");
	return &this->jobList;

}

SystemFileList* Project::getSystemFileList()
{
	dprintf_full("\033[32mProject::getSystemFileList\033[0m\n");
	return &this->sysFileList;
}

bool Project::checkConsistency()
{
	dprintf_full("\033[32m>> Project::checkConsistency\033[0m\n");
	if(!this->sysFileList.checkConsistency())
	{
		return false;
	}
	SourceList* srclist = NULL;
	SourceList* dummy_sl = new SourceList();
	Source* source = NULL;
	SystemFile* sysFile = NULL;
	JobList* src_jl = NULL;
	JobList* dummy_jl = new JobList();
	Job* dummy = NULL;
	int i= 0;
	int j= 0;
	int l= 0;
	int k=0;
	ESInfo* pro_sl_es = NULL;
	const char* pro_sl_name;
	u32 pro_id;
	Source* pro_src = NULL;
	Source* dummy_src = NULL;
	int m = 0;
	u32 id1;
	Job* job_in_projl = NULL;
	Job* job_in_dummy = NULL;

	while(NULL != (sysFile = this->sysFileList.getSysFileByIndex(i)))
	{
		i++;
		j=0;
		if(NULL != (srclist = sysFile->getSourceList()))
		{
			while(NULL!=(source = srclist->getSourceByIndex(j)))
			{
				if(!dummy_sl->add(source))
				return false;
				if(NULL == (src_jl = source->getJobList()))
				return false;
				l=0;
				while(NULL != (dummy = src_jl->getJobByIndex(l)))
				{
					if(!dummy_jl->addJob(dummy))
					return false;
					else
					l++;
				}
				j++;
			}
		}
		else
		{
			dprintf_err("\033[31mProject::cC : error in getting sourcelist\033[0m\n");
			return false;
		}
	}
	source->clean();
	while(NULL != (pro_src = this->sourceList.getSourceByIndex(k)))
	{
		if(NULL != (pro_sl_es = pro_src->getESInfo()))
		{
			if(NULL != (pro_sl_name = pro_sl_es->getInput()))
			{
				if(0 <= (pro_id = pro_sl_es->getStreamId()))
				{
					if(NULL != (dummy_src = this->sourceList.get(pro_sl_name, pro_id)))
					{
						if(!dummy_sl->remove(dummy_src))
						return false;
						else k++;
					}
				}
			}
		}
		else
		{
			dprintf_err("\033[31mProject::cC : could not find source\033[0m\n");
			return false;
		}
	}
	pro_src->clean();
	dummy_src->clean();
	if(NULL == dummy_sl->getSourceByIndex(0))
	{
		delete(dummy_sl);
		m=0;
		while(NULL != (job_in_projl = this->jobList.getJobByIndex(m)))
		{
			if(0 <= (id1 = job_in_projl->getID()))
			{
				if(NULL != (job_in_dummy = dummy_jl->getJob(id1)))
				{
					if(dummy_jl->delJob(job_in_dummy))
					{
						m++;
					}
					else return false;
				}
				else
				{	
					dprintf_err("\033[31mProject::cC : job not found\033[0m\n");
					return false;
				}
			}
			else return false;
		}
			
		if(NULL == dummy_jl->getJobByIndex(0))
		{
			dprintf_full("\033[32m<< Project::checkConsistency ok\033[0m\n");
			return true;
		}
		else
		{
			dprintf_err("\033[31mProject::cC : joblist not empty\033[0m\n");
			return false;
		}
	}
	else
	{
		dprintf_err("\033[31mProject::cC : sourcelist not empty\033[0m\n");
		return false;

	}
}
