/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   Project.hpp                                                 *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/


#ifndef __PROJECT_HPP__
#define __PROJECT_HPP__

#include "SourceList.hpp"
#include "SystemFileList.hpp"
#include "JobList.hpp"
#include "global.hpp"
#include "xml.hpp"
#include "assert.h"

class Project{

protected:
	char name[MAX_STR_LEN];
	const char* workingDirectory;
	SourceList sourceList;
	JobList jobList;
	SystemFileList sysFileList;

public:
	/** constructor */
	Project(const char* name);
	
	/** destructor */
	virtual ~Project();
	
	/** method to load an existing project from a config-file */
	bool load(const char* name);

	/** method to save a project to file */
	bool save(const char* name);
	
	/** methods to set and project params */
	void setName(const char* name);
	const char* getName() const;

	void setWorkingDirectory(const char* dir);
	const char* getWorkingDirectory() const;

	/** checks, if all jobs are in the project-joblist and if all 
	 ** sources are in the sourcelist, otherwise sth. has gone wrong */
	bool checkConsistency();
	
	/** methods to get lists of the project */
	SourceList* getSources(); 
	JobList* getJobList();
	SystemFileList* getSystemFileList();

}; //Project

#endif
