/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   Source.cpp                                                  *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "Source.hpp"
#include "ESInfo.hpp"
#include <list>
#include "ContainerInfo.hpp"
#include "avcodec.h"
#include "container/ContainerFile.hpp"
#include "xml.hpp"
#include "SourceList.hpp"

Source::Source(ESInfo* es)
{
	dprintf_full("\033[32mSource::construct %s\033[0m\n",es->getInput());
	//no assert(es) because of fromString in SourceList
	this->reference_counter = 1;
	u32 id = createSourceID();
	this->sourceID = id;
	this->es=es;
}

/** destructor */
Source::~Source()
{
	dprintf_full("\033[32mSource::destruct\033[0m\n");
	if(this->reference_counter != 1)
		assert(1==0);
}

ESInfo* Source::getESInfo() 
{
	dprintf_full("\033[32mSource::getESInfo\033[0m\n");
	this->checkReferences();
	return this->es;
}


u32 Source::getID() const
{
        dprintf_full("\033[32mSource::getID\033[0m\n");
        this->checkReferences();
        return this->sourceID;
}
			
JobList* Source::getJobList()
{
	dprintf_full("\033[32mSource::getJobList\033[0m\n");
	this->checkReferences();
	return &(this->jl);
}


const char* Source::getFileName()
{
	dprintf_full("\033[32mSource::getFileName\033[0m\n");
	this->checkReferences();
 	assert(this->es);
	return this->es->getInput();
}

bool Source::toStringFull(char *buf, int max_len)
{
	dprintf_full("\033[32m>> Source::toStringFull(buf, max_len)\033[0m\n");
	assert(buf && max_len>0);
	this->checkReferences();
	char* format = (char*)malloc(max_len);
	memset(format, 0, max_len);
	memset(buf, 0, max_len);

	if(max_len < snprintf(format, max_len, "%s%s%s%s%s", SOURCE_START, 
	NAME_PRINT, STREAM_ID, "%s", SOURCE_END))
	{
		free(format);
		dprintf_err("\033[31mSource::toStringFull(buf, len)\033[0m\n");
		return false;
	}
	if(max_len < snprintf(buf, max_len, format, this->es->getInput(), this->es->getStreamId(), this->jl.toStringIds()))
	{
		free(format);
		dprintf_err("\033[31mSource::toStringFull(buf, len)\033[0m\n");
		return false;
	}
	free(format);
	dprintf_full("\033[32mSource::toStringFull(buf, len) ok\033[0m\n");
	return true;	
}

char* Source::toStringFull()
{
	dprintf_full("\033[32m>> Source::toStringFull()\033[0m\n");
	this->checkReferences();
	char* output = (char*)malloc(MAX_XML_LEN);
	if(toStringFull(output, MAX_XML_LEN))
	{
		dprintf_full("\033[32m<< Source::toStringFull ok\033[0m\n");
		return output;
	}
	else
	{
		dprintf_err("\033[31mSource::toStringFull\033[0m\n");
		free(output);
		return NULL;
	}
}


bool Source::toStringES(char *buf, int max_len)
{
	dprintf_full("\033[32m>> Source::toStringES(buf, max_len)\033[0m\n");
	assert(buf && max_len>0);
	this->checkReferences();
	char* format = (char*)malloc(max_len);
	memset(format, 0, max_len);
	memset(buf, 0, max_len);

	if(max_len < snprintf(format, max_len, "%s%s%s%s", SOURCE_START, 
	NAME_PRINT, STREAM_ID, SOURCE_END))
	{
		free(format);
		dprintf_err("\033[31mSource::toStringEs(buf, len)\033[0m\n");
		return false;
	}
	if(max_len < snprintf(buf, max_len, format, this->es->getInput(), this->es->getStreamId()))
	{
		free(format);
		dprintf_err("\033[31mSource::toStringES(buf, len)\033[0m\n");
		return false;
	}
	free(format);
	dprintf_full("\033[32mSource::toStringES(buf, len) ok\033[0m\n");
	return true;	
}

char* Source::toStringES()
{
	dprintf_full("\033[32m>> Source::toStringES()\033[0m\n");
	this->checkReferences();
	char* output = (char*)malloc(MAX_XML_LEN);
	if(toStringES(output, MAX_XML_LEN))
	{
		dprintf_full("\033[32m<< Source::toStringES ok\033[0m\n");
		return output;
	}
	else
	{
		dprintf_err("\033[31mSource::toStringES\033[0m\n");
		free(output);
		return NULL;
	}
}


Source* Source::fromString(const char* initString, SourceList* sl, JobList* jl, bool create) //create is true if sources have to be created
{

	dprintf_full("\033[32mSource::fromString\033[0m\n");
	assert(initString);
	const char* next = initString;
	char* end = NULL;
	char* name = (char*)malloc(MAX_STR_LEN);
	memset(name, 0, MAX_STR_LEN);
	Source* source = NULL;
	ESInfo* es = NULL;
	if(next==strstr(next, SOURCE_START))
	{
		next=(const char*)next+strlen(SOURCE_START);
	}
	else
	{
		free(name);
		dprintf_err("\033[31mSource::fromString\033[0m\n");
		return NULL;
	}	
	if(next==strstr(next, NAME_SCAN))
	{
		next=(const char*)next+strlen(NAME_SCAN);
		if(NULL!=(end=strchr(next, '\n')))
		{
			memcpy(name, next, end-next);
			next =(const char*)end+1;
			u32 id;
			
			if(1 ==	sscanf(next, STREAM_ID, &id))
			{
				next =(const char*)strchr(next, '\n')+1;
				if(create == true)
				{
					ContainerInfo* infile = ContainerFile::loadContainerInfo(name);
					if(!infile)				
					{
						free(name);
						dprintf_err("\033[31mSource::fromString : cannot open inputfile\033[0m\n");
						return NULL;
					}
					if(NULL != (es=infile->getES(id)))
					{
						source = new Source(es);
					}
					else
					{
						free(name);
						dprintf_err("\033[31mSource::fromString : no esinfo\033[0m\n");
						return NULL;
					}
				}
				else if(create == false)
				{
					assert(sl);
					JobList* jobl;
					if((source = sl->get((const char*)name, id)) != NULL)
					{
						jobl = source->getJobList();
						jobl->fromStringIds(next, jl, source); 
						next=(const char*)strstr(next, SOURCE_END);
					}
					else return NULL;
				}
				else return NULL;
			
				if(next == strstr(next, SOURCE_END))
	                        {
					free(name);
		                        dprintf_full("\033[32mSource::fromString ok\033[0m\n");
					return source;
				}
				else
				dprintf_err("\033[31mSource::fromString : source end not found\033[0m\n");
			}
			else
			dprintf_err("\033[31mSource::fromString : error at id\033[0m\n");
		}
		else
		dprintf_err("\033[31mSource::fromString:no end found\033[0m\n");
	}
	else
	dprintf_err("\033[31mSource::fromString : no name found\033[0m\n");

	free(name);
	return NULL;
}	

u32 Source::createSourceID()
{
	dprintf_full("\033[32mSource::createSourceID\033[0m\n");
	static u32 i = 0;
	return ++i;
}


void Source::checkReferences() const
{
	dprintf_full("Source::checkReferences\n");
	
	if(this->reference_counter <= 0)
	{
		dprintf_err("\033[31mSource::checkReferences : error\033[0m\n");
		assert(1==0);
	}
}

void Source::increment()
{
	dprintf_full("\033[32mSource::increment\033[0m\n");
	this->checkReferences();
        this->reference_counter++;
}


void Source::decrement()
{
	dprintf_full("\033[32mSource::decrement\033[0m\n");
	this->checkReferences();
        this->reference_counter--;
	
}
	 
void Source::clean()
{
	dprintf_full("\033[32mSource::clean\033[0m\n");
	this->decrement();
	if(this->reference_counter == 1)
	{
		delete(this);
		dprintf_full("\033[32mSource::clean : deleted Job\033[0m\n");
	}
	else
	dprintf_err("\033[31mSource::clean : source is still referenced\033[0m\n");
}



