/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   Source.hpp                                                  *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef __SOURCE_HPP__
#define __SOURCE_HPP__

#include "global.hpp"

#include "assert.h"
#include <list>
#include "JobList.hpp"

class SourceList;
class ESInfo;

class Source {


protected:
	ESInfo* es;
	JobList jl;
	u32 reference_counter;
	u32 sourceID;
	
public:
	/** constructor */
	Source(ESInfo *es);

	/** destructor */
	virtual ~Source();
	
	/** methods to get params and lists of Source */
	ESInfo* getESInfo();
	u32 getID() const;
	JobList* getJobList();
	const char* getFileName();
	
	/** prints all data of source including ESInfo and Joblist with jobids */
	bool toStringFull(char *buf, int max_len);
	char* toStringFull();

	/** only prints information about the ESInfo of this source, i.e.
	 ** name and streamID */
	bool toStringES(char *buf, int max_len);
	char* toStringES();

	/** methods to correct reference_counter */
	void increment();
	void decrement();
	
	/** create unique source id */
	static u32 createSourceID();
		
	/** checks if reference_counter is less or equal 0 */
	void checkReferences() const;
	
	/** method to check if reference_counter is 1 and to delete object */
	void clean();
		
	/** reads one source and if create is true, it will be created and
	 ** added to the sourcelist of the project, if create is false 
	 ** it searches for the existing source in the given sourcelist */
	Source* fromString(const char* initString, SourceList* sl, JobList* jl, bool create);
};
#endif
