/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   SourceList.cpp                                              *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/


#include "SourceList.hpp"
#include "Source.hpp"
#include "xml.hpp"
	
/** constructor */
SourceList::SourceList()
{
	dprintf_full("\033[32mSourceList::construct\033[0m\n");
}

/** destructor */
SourceList::~SourceList()
{
	dprintf_full("\033[32mSourceList::destruct\033[0m\n");
	list <Source*>::iterator sourceIterator = this->sources.begin();
	while(!this->sources.empty())
	{
		sourceIterator = this->sources.begin();
		remove(*sourceIterator);
	}
}
		
/** return is false if a source with the same identifier already exists */
bool SourceList::add(Source* s)
{	
	dprintf_full("\033[32m>> SourceList::add\033[0m\n");
	assert(s);
	list <Source*>::iterator li = this->sources.begin();
	ESInfo* new_es = s->getESInfo();
	const char* new_name = new_es->getInput();
	const char* list_name;
	u32 new_id = new_es->getStreamId();
	u32 list_id;
	ESInfo* list_es = NULL;
	while(li != this->sources.end())
	{
		list_es = (*li)->getESInfo();
		list_name = list_es->getInput();
		list_id = list_es->getStreamId();
		if((strcmp(list_name, new_name)==0) && (list_id == new_id))
		{
			dprintf_err("\033[31mSourceList::add : source exits\033[0m\n");
			return false;
		}
		li++;
		
	}
	this->sources.insert(this->sources.end(),s);
	s->increment();
	dprintf_full("\033[32mSourceList::add ok\033[0m\n");
	return true;
}
	
bool SourceList::remove(Source* s) //only unlinks from list, dont free!
{
	dprintf_full("\033[32mSourceList::remove\033[0m\n");
	assert(s);
	list<Source*>::iterator li;
	if(this->sources.empty())
	{
		dprintf_err("\033[31mSourceList::remove : empty list\033[0m\n");
		return false;
	}
	else li=this->sources.begin();
	while(li!= this->sources.end())
	{
		if((*li) != s)
		{
			li++;
		}
		else
		{
			s->clean();
			this->sources.erase(li);
			dprintf_full("\033[32mSourceList::remove ok\033[0m\n");
			return true;
		}
	}
	dprintf_err("\033[31mSourceList::delJob : element not found\033[0m\n");
	return false;
		
}

Source* SourceList::get(const char* fileName, u32 streamId)
{
	dprintf_full("\033[32mSourceList::get\033[0m\n");
	assert(fileName);
	list<Source*>::iterator li;
	if(this->sources.empty())
	{
		dprintf_err("\033[32mSourceList::get : empty list\033[0m\n");
		return NULL;
	}
	else li = this->sources.begin();
	while(li!=this->sources.end())
	{
		if(strcmp((*li)->getFileName(), fileName)==0)
		{
			ESInfo* es = (*li)->getESInfo();
			if(streamId == es->getStreamId())
			{
				dprintf_full("\033[32mSourceList::get ok\033[0m\n");
				(*li)->increment();
				return (*li);
			}
		}
		li++;
	}
	dprintf_err("\033[31mSourceList::get filename %s not found\033[0m\n",fileName);
	_exit(1);
	return NULL;
}

Source* SourceList::getSourceByIndex(u32 index)
{
	dprintf_full("\033[32mSourceList::getSourceByIndex\033[0m\n");
	list<Source*>::iterator next = this->sources.begin();
	while( next!=this->sources.end() && index>0 )
	{
		next++;
		index--;
	}
	if ( index == 0 && next!=this->sources.end())
	{
		dprintf_full("\033[32mSourceList::getSourceByIndex : found element\033[0m\n");
		(*next)->increment();
		return *next;
	}
	else
	{
		dprintf_err("\033[31mSourceList::getSourceByIndex : End of list\033[0m\n");
		return NULL;
	}
	

}

bool SourceList::toStringFull(char* output, int max_len)
{
	dprintf_full("\033[32m>> SourceList::toStringFull(buf, len)\033[0m\n");
	assert(output);
	int len=0;
	list <Source*>::iterator li;
	if(this->sources.empty())
	{
		dprintf_err("\033[31mSourceList::toStringFull(output, max_len) : empty list\033[0m\n");
		return false;
	}
	else li=this->sources.begin();
	strcpy(output, SL_START);
	output=(char*)output+strlen(SL_START);
	while(li!=this->sources.end() && max_len > 0)
	{
		if(!((*li)->toStringFull(output, max_len)))
		{
			dprintf_err("\033[31mSourceList::toStringFull(output, max_len) : toString of job not ok\033[0m\n");
			return false;
		}
		li++;
		len=strlen(output);
		output=(char*)output+len;
		max_len-=len;
	}
	if(max_len>(int)strlen(SL_END))
	{
		if(NULL == strcpy(output, SL_END))
		{
			dprintf_err("\033[31mSourceList::toStringFull(output, max_len) : SourceListEnd not found\033[0m\n");
			return false;
		}
		else
		{
			dprintf_full("\033[32m<< SourceList::toStringFull(output, max_len) ok\033[0m\n");
			return true;
		}
	}	
	else
	{
		dprintf_err("\033[31mSourceList::toStringFull(output, max_len)\033[0m\n");
		return false;
	}
}
	

char* SourceList::toStringFull()
{
	dprintf_full("\033[32m>> SourceList::toStringFull\033[0m\n");
	char* output = (char*)malloc(MAX_FILE_LEN);
	if(toStringFull(output, MAX_FILE_LEN))
	{
		dprintf_full("\033[32m<< SourceList::toStringFull ok\033[0m\n");
		return output;
	}
	else
	{
		dprintf_err("\033[31mSourceList::toStringFull\033[0m\n");
		snprintf(output, strlen(SL_START)+strlen(SL_END), "%s%s", SL_START, SL_END);
		return output;
	}
}

bool SourceList::toStringESInfo(char* output, int max_len)
{
	dprintf_full("\033[32m>> SourceList::toStringESInfo(buf, len)\033[0m\n");
	assert(output);
	int len=0;
	list <Source*>::iterator li;
	if(this->sources.empty())
	{
		dprintf_err("\033[31mSourceList::toStringESInfo(output, max_len) : empty list\033[0m\n");
		return false;
	}
	else li=this->sources.begin();
	strcpy(output, SL_START);
	output=(char*)output+strlen(SL_START);
	while(li!=this->sources.end() && max_len > 0)
	{
		if(!((*li)->toStringES(output, max_len)))
		{
			dprintf_err("\033[31mSourceList::toStringESInfo(output, max_len) : toString of job not ok\033[0m\n");
			return false;
		}
		li++;
		len=strlen(output);
		output=(char*)output+len;
		max_len-=len;
	}
	if(max_len>(int)strlen(SL_END))
	{
		if(NULL == strcpy(output, SL_END))
		{
			dprintf_err("\033[31mSourceList::toStringESInfo(output, max_len) : SourceListEnd not found\033[0m\n");
			return false;
		}
		else
		{
			dprintf_full("\033[32m<< SourceList::toStringESInfo(output, max_len) ok\033[0m\n");
			return true;
		}
	}	
	else
	{
		dprintf_err("\033[31mSourceList::toStringESInfo(output, max_len)\033[0m\n");
		return false;
	}

}

char* SourceList::toStringESInfo()
{
	dprintf_full("\033[32m>> SourceList::toStringESInfo\033[0m\n");
	char* output = (char*)malloc(MAX_FILE_LEN);
	if(toStringESInfo(output, MAX_FILE_LEN))
	{
		dprintf_full("\033[32m<< SourceList::toStringESInfo ok\033[0m\n");
		return output;
	}
	else
	{
		dprintf_err("\033[31mSourceList::toStringESInfo\033[0m\n");
		snprintf(output, MAX_STR_LEN, "%s%s", SL_START, SL_END);
		return output;
	}

}
// flag is true, if sources have to be built
bool SourceList::fromString(const char* initString, SourceList* sl, JobList* jl, bool flag)
{
	dprintf_full("\033[32mSourceList::fromString\033[0m\n");
	assert(initString);
	const char* next = initString;
	char* end = NULL;
	Source* src = NULL;
	if(next == strstr(next, SL_START))
	{
		next=(const char*)next+strlen(SL_START);
	}
	else
	{
		dprintf_err("\033[31mSourceList::fromString\033[0m\n");
		return false;
	}
	if(NULL==(end = strstr(next, SL_END)))
	{
		dprintf_err("\033[31mSourceList::fromString : end not found\033[0m\n");
		return false;
	}
	while(next < end)
	{
		if(0 == memcmp( (void*)next, (void*)SOURCE_START, strlen(SOURCE_START)))
		{
			Source* src2 = NULL;
			if(flag == true)
			{
				if((src = src2->fromString(next, NULL, NULL, true)) != NULL)
				{
					this->add(src);
					if(NULL!=(next=strstr(next, SOURCE_END)))
					{
						next=(const char*)next+strlen(SOURCE_END);
					}
					else
					{
						dprintf_err("\033[31mSourceList::fromString : error with true\033[0m\n");
						return false;
					}
				}
				else return false;
			}
			else if(flag == false)
			{
				if((src = src2->fromString(next, sl, jl, false))!=NULL)
				{
					this->add(src);
					if(NULL!=(next=strstr(next, SOURCE_END)))
					{
						next=(const char*)next+strlen(SOURCE_END);
					}
					else
					{
						dprintf_err("\033[31mSourceList::fromString : error with false\033[0m\n");
						return false;
					}
				}
			}
			else
			{
				dprintf_err("\033[31mSourceList::fromString : invalid flag\033[0m\n");
				return false;
			}
		}					
	}
	if(next==end)
	{
		dprintf_full("\033[32m<< SourceList::fromString ok\033[0m\n");
		return true;
	}
	else
	{
		dprintf_err("\033[31mSourceList::fromString\033[0m\n");
		return false;
	}
}
		
