/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   SystemFile.cpp                                              *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "SystemFile.hpp"
#include "ContainerInfo.hpp"
#include "container/ffMP4ContainerFile.hpp"
#include "ESInfo.hpp"
#include "xml.hpp"
#include "Source.hpp"
#include "Job.hpp"

/** constructor */
SystemFile::SystemFile()
{
	dprintf_full("\033[32mSystemFile::constructor\033[0m\n");
	name[0] = '\0';
}

/** destructor */
SystemFile::~SystemFile()
{
	dprintf_full("\033[32mSystemFile::destructor\033[0m\n");
}

const char* SystemFile::getName() const
{
	dprintf_full("\033[32mSystemFile::getName\033[0m\n");
	return this->name;
}

bool SystemFile::getInterleave() const
{
	dprintf_full("\033[32mSystemFile::getInterleave\033[0m\n");
	return this->interleave;
}

bool SystemFile::getStreamable() const
{
	dprintf_full("\033[32mSystemFile::getStreamable\033[0m\n");
	return this->streamable;
}
		
bool SystemFile::addJob(Job *j)
{
	dprintf_full("\033[32mSystemFile::addJob\033[0m\n");
	assert(j);
	return this->jl.addJob(j);
}

bool SystemFile::addSource(Source* s)
{
	dprintf_full("\033[32mSystemFile::addSource\033[0m\n");
	assert(s);
	return this->sl.add(s);
}

bool SystemFile::removeJob(Job *j) //only unlink, not free
{
	dprintf_full("\033[32mSystemFile::removeJob\033[0m\n");
	assert(j);
	return this->jl.delJob(j);
}

bool SystemFile::removeSource(Source* s) //only unlink, not free
{
	dprintf_full("\033[32mSystemFile::removeSource\033[0m\n");
	assert(s);
	return this->sl.remove(s);
}

JobList* SystemFile::getJobList()
{
	dprintf_full("\033[32mSystemFile::getJobList\033[0m\n");
	return &this->jl;
}

SourceList* SystemFile::getSourceList()
{
	dprintf_full("\033[32mSystemFile::getSourceList\033[0m\n");
	return &this->sl;
}

bool SystemFile::toString(char *buf, int max_len)
{
	dprintf_full("\033[32m>> SystemFile::toString(buf, max_len)\033[0m\n");
	assert(buf && max_len>0);
	char* format = (char*)malloc(max_len);
	memset(format, 0, max_len);
	memset(buf, 0, max_len);
	
	if(max_len < snprintf(format, max_len, "%s%s%s%s%s%s", SYSFILE_START, NAME_PRINT, INTERLEAVE, STREAMABLE, "%s", SYSFILE_END))
	{
		free(format);
		dprintf_err("\033[31mSystemFile::toString(buf, len)\033[0m\n");
		return false;
	}
	if(max_len < snprintf(buf, max_len, format, this->name, (int)this->interleave, (int)streamable, this->sl.toStringFull(), this->jl.toStringIds()))
	{
		free(format);
		dprintf_err("\033[31mSystemFile::toString(buf, len)\033[0m\n");
		return false;
	}
	free(format);
	dprintf_full("\033[32mSystemFile::toString(buf, len) ok\033[0m\n");
	return true;

}

char* SystemFile::toString()
{
	dprintf_full("\033[32m>> SystemFile::toString()\033[0m\n");
	char* output = (char*)malloc(MAX_XML_LEN);
	if(toString(output, MAX_XML_LEN))
	{
		dprintf_full("\033[32m<< SystemFile::toString ok\033[0m\n");
		return output;
	}
	else
	{
		free(output);
		dprintf_err("\033[31mSystemFile::toString\033[0m\n");
		return NULL;
	}
}
	
bool SystemFile::updateJobList(SourceList* sl, JobList* jl)
{
	dprintf_full("\033[32m>> SystemFile::updateJobList\033[0m\n");
	assert(sl);
	assert(jl);
	JobList* new_jl = NULL;
	Job* job = NULL;
	Source* source = NULL;
	int i = 0;
	int j = 0;
	while(NULL != (source = sl->getSourceByIndex(i)))
	{
		if(NULL != (new_jl = source->getJobList()))
		{
			while(NULL != (job = new_jl->getJobByIndex(j)))
			{
				if(!jl->addJob(job))
				{
					dprintf_err("\033[31mSystemFile::updateJobList : adding job failed\033[0m\n");
					//source->clean();
					//job->clean();
					return false;
				}
				else
				j++;
			}
		}
		else
		{
			dprintf_err("\033[31mSystemFile::updateJobList\033[0m\n");
			//source->clean();
			//job->clean();
			return false;
		}
		i++;
	}
	dprintf_full("\033[32m<< SystemFile::updateJobList ok\033[0m\n");
	return true;
}


SystemFile* SystemFile::fromString(const char* initString, SourceList* src, JobList* projectjl)
{
	dprintf_full("\033[32mSystemFile::fromString\033[0m\n");
	assert(initString);
	const char* next = initString;
	char* end = NULL;
	if(next==strstr(next, SYSFILE_START))
	{
		next=(const char*)next+strlen(SYSFILE_START);
	}
	else
	{
		dprintf_err("\033[31mSystemFile::fromString\033[0m\n");
		return NULL;
	}
	if(next==strstr(next, NAME_SCAN))
	{	
		next=(const char*)next+strlen(NAME_SCAN);
		if(NULL != (end=strchr(next, '\n')))
		{
			memset(this->name, 0, MAX_STR_LEN);
			if(NULL == memcpy(this->name, next, end-next))
			{
				dprintf_err("\033[31mSystemFile::fromString\033[0m\n");
				return NULL;
			}
			else
			{
				next=(const char*) end+1;
				int i = 0;
				if(1 == sscanf(next, INTERLEAVE, &i))
				{
					if(i == 0)
						this->interleave = false;
					else this->interleave = true;
					
					next=(const char*)strchr(next, '\n')+1;
				if(1==sscanf(next, STREAMABLE, &i))
				{
					if(i == 0)
						this->streamable = false;
					else this->streamable = true;
					
					next=(const char*)strchr(next, '\n')+1;
					if(this->sl.fromString(next, src, projectjl, false))
					{
						next=(const char*)strstr(next, SL_END)+strlen(SL_END);
						if(strstr(next, SYSFILE_END))
						{
							if(!updateJobList(&(this->sl), &(this->jl)))
							{
								dprintf_err("\033[31mSystemFile::fromString : error at update\033[0m\n");
								return NULL;
							}
							else
							{
							return this;
							}
						}
						else
						{
							dprintf_err("\033[31mSystemFile::fromString : no sysfileend found\033[0m\n");
							return NULL;
						}
					}
					else
					{
						dprintf_err("\033[31mSystemFile::fromString : error at sourcelist-fromString\033[0m\n");
                                                return NULL;
					}
					
				}
				else
				dprintf_err("\033[31mSystemFile::fromString : error at reading param streamable\033[0m\n");
				
				}
				else
				dprintf_err("\033[31mSystemFile::fromString : error at reading param interleave\033[0m\n");
			}
			
		}
		else
		dprintf_err("\033[31mSystemFile::fromString : no end of name found\033[0m\n");
	}
	else
	dprintf_err("\033[31mSystemFile::fromString : no name found\033[0m\n");
	return NULL;
}

bool SystemFile::checkConsistency()
{
	dprintf_full("\033[32mSystemFile::checkConsistency\033[0m\n");
	Source* source = NULL;
	JobList* source_jobl = NULL;
	JobList* compare_jl = new JobList();
	Job* job = NULL;
	int i=0;
	int j=0;
	while(NULL != (source = this->sl.getSourceByIndex(i)))
	{
		i++;
		if(NULL != (source_jobl = source->getJobList()))
		{
			j=0;
			while(NULL != (job = source_jobl->getJobByIndex(j)))
			{
				if(compare_jl->addJob(job))
				j++;
				else 
				{
					job->clean();
					source->clean();
					return false;
				}
			}
		}
		else
		{
			dprintf_err("\033[31mSystemFile::cC : error in getting joblist\033[0m\n");
			source->clean();
			job->clean();
			return false;
		}
	}
	
	int k=0;
	u32 id1;
	Job* job_in_sysjl = NULL;
	Job* job_in_extralist = NULL;
	while(NULL!=(job_in_sysjl = this->jl.getJobByIndex(k)))
	{
		if(0<=(id1 = job_in_sysjl->getID()))
		{
			if(NULL != (job_in_extralist = compare_jl->getJob(id1)))
			{
				if(!compare_jl->delJob(job_in_extralist))
				{
					job_in_sysjl->clean();
					return false;
				}
				else
				k++;
			}
			else
			{
				dprintf_err("\033[31mSystemFile::cC ERROR -> not consistent\033[0m\n");
				job_in_sysjl->clean();
				return false;
			}
		}
		else
		{
			job_in_sysjl->clean();
			return false;
		}
	}
	if(NULL == compare_jl->getJobByIndex(0))
	{
		job_in_sysjl->clean();
		//delete(compare_jl);
		dprintf_full("\033[32mSystemFile::cC was ok\033[0m\n");
		return true;
	}
	else
	{
		job_in_sysjl->clean();
		//delete(compare_jl);
		dprintf_err("\033[31mSystemFile::cC dummy list not empty!\033[0m\n");
		return false;
	}
}
	
void SystemFile::setName(const char* name)
{
	dprintf_full("\033[32mSystemFile::setName\033[0m\n");
	assert(name);
	strncpy(this->name, name, MAX_STR_LEN-1);
}

void SystemFile::setStreamable(bool streamable)
{
	dprintf_full("\033[32mSystemFile::setStreamable\033[0m\n");
	this->streamable = streamable;
}

void SystemFile::setInterleave(bool interleave)
{
	dprintf_full("\033[32mSystemFile::setInterleave\033[0m\n");
	this->interleave = interleave;
}


