/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   SystemFile.hpp                                              *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef __SYSTEMFILE_HPP__
#define __SYSTEMFILE_HPP__

#include <list>
#include "JobList.hpp"
#include "SourceList.hpp"
#include "global.hpp"
#include "assert.h"
#include <iostream>

class SystemFile {

protected:
	char name[MAX_STR_LEN];
	bool interleave;
	bool streamable;

	JobList jl;
	SourceList sl;

public:
	/** constructor */
	SystemFile();

	/** destructor */
	virtual ~SystemFile();
	
	/** get params of SystemFile */
	const char* getName() const;
	bool getInterleave() const;
	bool getStreamable() const;
	
	/** adds a job to the joblist of the systemfile */
	bool addJob(Job *j);

	/** adds source to sourcelist of systemfile */
	bool addSource(Source* s);

	bool removeJob(Job *j); //only unlink, not free
	bool removeSource(Source* s); //only unlink, not free

	/** get lists of this systemfile */
	JobList* getJobList();
	SourceList* getSourceList();
	
	bool toString(char *buf, int max_len);
	char* toString();

	/** reads params of systemfile and calls fromStrings of Sourcelist
	 ** and Joblist */
	SystemFile* fromString(const char* initString, SourceList* src, JobList* jl);

	/** checks if all jobs in sourcelists are in systemfile-joblist */
	bool checkConsistency();

	/** set params of systemfile */
	void setName(const char* name);
	void setStreamable(bool streamable);
	void setInterleave(bool interleave);

	/** after fromString jobs have to be added to joblist of systemfile */
	bool updateJobList(SourceList* sl, JobList* jl);
}; //class

#endif
