/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   SystemFileList.cpp                                          *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "SystemFileList.hpp"
#include "SystemFile.hpp"
#include "JobList.hpp"
#include "SourceList.hpp"
#include "xml.hpp"

/** constructor */
SystemFileList::SystemFileList()
{
	dprintf_full("\033[32mSystemFileList::construct\033[0m\n");
}

/** destructor */
SystemFileList::~SystemFileList()
{
	dprintf_full("\033[32mSystemFileList::destruct\033[0m\n");
	list <SystemFile*>::iterator fileListIterator = this->sysFiles.begin();
	while(!this->sysFiles.empty())
	{
		fileListIterator = this->sysFiles.begin();
		delete(*fileListIterator);
		this->sysFiles.pop_front();
	}
}
		
/** return is false if a sysfile with the same identifier already exists */
bool SystemFileList::add(SystemFile* sysFile)
{
	dprintf_full("\033[32mSystemFileList::add\033[0m\n");
	assert(sysFile);
	list <SystemFile*>::iterator li = this->sysFiles.begin();
	while(li != this->sysFiles.end())
	{
		if((*li) == sysFile)
		{
			dprintf_err("\033[31mSystemFileList::add : elem already in list\033[0m\n");
			return false;
		}
		li++;
	}
	this->sysFiles.push_back(sysFile);
	return true;
}
	
bool SystemFileList::del(SystemFile* sysFile)
{
	dprintf_full("\033[32mSystemFileList::del\033[0m\n");
	assert(sysFile);
	list <SystemFile*>::iterator li;
	if(this->sysFiles.empty())
	{
		dprintf_err("\033[32mSystemFileList::del : empty list\033[0m\n");
		return false;
	}
	else li = this->sysFiles.begin();
	while(li != this->sysFiles.end())
	{
		if((*li) != sysFile)
		{
			li++;
		}
		else
		{
			delete(*li);
			this->sysFiles.erase(li);
			dprintf_full("\033[32mSystemFileList::del ok\033[0m\n");
			return true;
		}
	}
	dprintf_err("\033[31mSystemFileList::del : element not found\033[0m\n");
	return false;
		
}

SystemFile* SystemFileList::get(const char* outFileName)
{
	dprintf_full("\033[32mSystemFileList::get\033[0m\n");
	assert(outFileName);
	list <SystemFile*>::const_iterator li;
	if(this->sysFiles.empty())
	{
		dprintf_err("\033[31mSystemFileList::get : empty list\033[0m\n");
		return NULL;
	}
	else li = this->sysFiles.begin();
	while (li != this->sysFiles.end())
	{
		if(strcmp((*li)->getName(), outFileName)==0)
		{
			dprintf_full("\033[32mSystemFileList::get ok\033[0m\n");
			return (*li);
		}
		li++;
	}
	dprintf_err("\033[31mSystemFileList::get\033[0m\n");
	return NULL;
}

SystemFile* SystemFileList::getSysFileByIndex(u32 index)
{
        dprintf_full("\033[32mSystemFileList::getSysFileByIndex\033[0m\n");
        list<SystemFile*>::iterator next = this->sysFiles.begin();
        while( next!=this->sysFiles.end() && index>0 )
        {
                next++;
                index--;
        }
        if ( index == 0 && next!=this->sysFiles.end())
        {
                dprintf_full("\033[32mSystemFileList::getSysFileByIndex : found element\033[0m\n");
                return *next;
        }
        else
        {
                dprintf_err("\033[31mSystemFileList::getSysFileByIndex : End of list\033[0m\n");
                return NULL;
        }
}

bool SystemFileList::checkConsistency()
{
	dprintf_full("\033[32m>> SystemFileList::cC\033[0m\n");
	list <SystemFile*>::const_iterator li;
	if(this->sysFiles.empty())
	{
		dprintf_err("\033[31mSystemFileList::cC empty list\033[0m\n");
		return false;
	}
	else li = this->sysFiles.begin();
	while (li != this->sysFiles.end())
	{
		if((*li)->checkConsistency())
		{
			li++;
		}
		else
		{
			dprintf_err("\033[31mSystemFileList::cC error in sysfilecheck\033[0m\n");
			return false;
		}
	}
	dprintf_full("\033[32mSystemFileList::cC ok\033[0m\n");
	return true;
		
}

bool SystemFileList::toString(char* output, int max_len)
{
	dprintf_full("\033[32m>> SystemFileList::toString(buf, len)\033[0m\n");
	assert(output);
	int len=0;
	list <SystemFile*>::iterator li;
	if(this->sysFiles.empty())
	{
		dprintf_err("\033[31mSystemFileList::toString(output, max_len) : empty list\033[0m\n");
		return false;
	}
	else li=this->sysFiles.begin();
	strcpy(output, SYSFILELIST_START);
	output=(char*)output + strlen(SYSFILELIST_START);
	while(li!=this->sysFiles.end() && max_len > 0)
	{
		if(!((*li)->toString(output, max_len)))
		{
			dprintf_err("\033[31mSystemFileList::toString(output, max_len) : toString of sysfile not ok\033[0m\n");
			return false;
		}
		li++;
		len=strlen(output);
		output=(char*) output + len;
		max_len = max_len - len;
	}
	if(max_len > (int)strlen(SYSFILELIST_END))
	{
		if(NULL == strcpy(output, SYSFILELIST_END))
		{
	        	dprintf_err("\033[31mSystemFileList::toString(output, max_len) : SystemFileListEnd not found\033[0m\n");
			return false;
		}
		else
		{
			dprintf_full("\033[32m<< SystemFileList::toString(output, max_len) ok\033[0m\n");
			return true;
		}
	}
        else
	{
		dprintf_err("\033[31mSystemFileList::toString(output, max_len)\033[0m\n");
		return false;
	}
}

char* SystemFileList::toString()
{
	dprintf_full("\033[32m>> SystemFileList::toString\033[0m\n");
	char* output = (char*)malloc(MAX_FILE_LEN);
        if(toString(output, MAX_FILE_LEN))
	{
		dprintf_full("\033[32m<< SystemFileList::toString ok\033[0m\n");
		return output;
	}
	else
	{
		dprintf_err("\033[31mSystemFileList::toString\033[0m\n");
		snprintf(output, MAX_STR_LEN, "%s%s", SYSFILELIST_START, SYSFILELIST_END);
		return output;
	}
}
	        
bool SystemFileList::fromString(const char* initString, SourceList* src, JobList* jl)
{
	dprintf_full("\033[32mSystemFileList::fromString\033[0m\n");
	assert(initString);
	const char* next = initString;
	char* end = NULL;
	if(next == strstr(next, SYSFILELIST_START))
	{
		next=(const char*)next+strlen(SYSFILELIST_START);
	}
	else
	{
		dprintf_err("\033[31mSystemFileList::fromString\033[0m\n");
		return false;
	}
	if(NULL==(end = strstr(next, SYSFILELIST_END)))
	{
		dprintf_err("\033[31mSystemFileList::fromString : end not found\033[0m\n");
		return false;
	}
	while(next < end)
	{
		if(0 == memcmp( (void*)next, (void*)SYSFILE_START, strlen(SYSFILE_START)))
		{
			SystemFile* sysFile = new SystemFile();
			if(sysFile->fromString(next, src, jl)!= NULL)
			{
				this->add(sysFile); //TODO check
				if(NULL!=(next=strstr(next, SYSFILE_END)))
				{
					next=(const char*)next+strlen(SYSFILE_END);
				}
				else
				{
					dprintf_err("\033[31mSystemFileList::fromString\033[0m\n");
					return false;
				}
			}
			else
			{
				dprintf_err("\033[31mSystemFileList::fromString\033[0m\n");
				return false;
			}
		}					
	}
	if(next==end)
	{
		dprintf_full("\033[32m<< SystemFileList::fromString ok\033[0m\n");
		return true;
	}
	else
	{
		dprintf_err("\033[31mSystemFileList::fromString\033[0m\n");
		return false;
	}
}
		
