/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   SystemFileList.hpp                                          *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef __SYSTEMFILELIST_HPP__
#define __SYSTEMFILELIST_HPP__

#include <list>
#include "global.hpp"

class SourceList;
class SystemFile;
class JobList;

class SystemFileList {

protected:
	std::list<SystemFile*> sysFiles;

public:
	/** constructor */
	SystemFileList();

	/** destructor */
	virtual ~SystemFileList();
	
	/** adds systemfile to list */
	bool add(SystemFile* sysFile);
	
	/** deletes systemfile from list */
	bool del(SystemFile* sysFile);

	/*returns systemfile with given name */
	SystemFile* get(const char* outFileName);

	/** returns systemfile at position with given index */
	SystemFile* getSysFileByIndex(u32 index);
	
	/** checks if all systemfiles are consistent */
	bool checkConsistency();

	/** prints systemfilelist */
	bool toString(char* output, int max_len);
	char* toString();

	/** reads systemfilelist and calls methods for sources and joblists */
	bool fromString(const char* initString, SourceList* src, JobList* jl);
	

}; //class

#endif
