/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   Transcoder.hpp                                              *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "Transcoder.hpp"

#include "Project.hpp"
#include "AudioESInfo.hpp"

#include "ContainerInfo.hpp"

#ifndef ISOMP4
#include "avformat.h"
#else
#include "avcodec.h"
#endif

#include "container/ContainerFile.hpp"
#include "container/ContainerDemux.hpp"
#include "VideoAdaptorChain.hpp"
#include "global.hpp"
#include "VideoESInfo.hpp"
#include "adaptors/YUVColorReductionAdaptor.hpp"
#include "adaptors/YUVSpatialReductionAdaptor.hpp"
#include "adaptors/MP4Encoder.hpp"
#include "adaptors/ColorReductionAdaptor.hpp"
#include "adaptors/MP4Decoder.hpp"
#include "adaptors/Forwarder.hpp"
#include "container/ContainerDemux.hpp"
#include "VideoJob.hpp"
#include "AudioJob.hpp"
#include "Job.hpp"
#include "Source.hpp"

#define MP4CREATOR "mp4creator"
#define CREATE "-create="


Transcoder::Transcoder(Project* proj)
{
	
	dprintf_full("\033[32m>>Transcoder::constructor\033[0m\n");
	this->pro = proj;
}

void Transcoder::muxFiles()
{
	dprintf_full("\033[32m>>Transcoder::muxFiles()\033[0m\n");
	sleep(2);
	int i = 0;
	int j = 0;
	SystemFile* sf = NULL;
	JobList* jl = NULL;
	SourceList* sl = NULL;
	char instructions[MAX_STR_LEN];
	char jobcmp[MAX_STR_LEN];
	memset(instructions, 0, MAX_STR_LEN);
	memset(jobcmp, 0, MAX_STR_LEN);
	Job* job = NULL;
	Source* source = NULL;
	const char* jobname = NULL;
	const char* sourcename = NULL;
	char* outname = (char*)malloc(MAX_STR_LEN);
	memset(outname, 0, MAX_STR_LEN);
	VideoESInfo* ves = NULL;
	VideoESInfo* videosource = NULL;
	ContainerInfo* ci = NULL;
	Adaptor* forwarder = NULL;
	SystemFileList* sysfiles = this->pro->getSystemFileList();
	while(NULL != (sf = sysfiles->getSysFileByIndex(i)))
	{
		const char* name = sf->getName();
		bool interleave = sf->getInterleave();
		//bool streamable = sf->getStreamable();
		float fps = 0.0;
		jl = sf->getJobList();
		sl = sf->getSourceList();
		// if output file exists --> delete it
		remove(name);
		
	
		while(NULL != (job = jl->getJobByIndex(j)))
		{
			
			jobname = job->getOutFileName();
			snprintf(jobcmp, MAX_STR_LEN, "mv %s %s.cmp", jobname, jobname);
			system(jobcmp);
			if(job->isVideo())
				fps = ((VideoJob*)job)->getFPS();
			if(interleave) 				
				snprintf(instructions,MAX_STR_LEN,"%s %s%s.cmp -rate=%f -timescale=90000 -interleave %s",MP4CREATOR, CREATE, jobname, fps, name);
			else	
				snprintf(instructions,MAX_STR_LEN,"%s %s%s.cmp -rate=%f -timescale=90000 %s",MP4CREATOR, CREATE, jobname, fps, name);
			system(instructions);
			j++;
		}
		j = 0;
		
		while(NULL != (source = sl->getSourceByIndex(j)))
		{
			sourcename = source->getFileName();
			ves = (VideoESInfo*)source->getESInfo();
			fps = ves->getFPS();
			ci = ContainerFile::loadContainerInfo(sourcename);
			assert(ci);
			videosource=(VideoESInfo*)ci->getES(ves->getStreamId());
			snprintf(outname, MAX_STR_LEN, "mv %s %s_%i.strm.cmp", sourcename, sourcename, ves->getStreamId());
			forwarder = new Forwarder();
			ContainerDemux::adaptESInfo(videosource, outname, forwarder, 0u, 4294967295u, false, false);
			delete ci;
			if(interleave)
                                snprintf(instructions,MAX_STR_LEN,"%s %s%s_%i.strm.cmp -rate=%f -timescale=90000 -interleave %s",MP4CREATOR, CREATE, sourcename, ves->getStreamId(), fps, outname);
	                else
	                        snprintf(instructions,MAX_STR_LEN,"%s %s%s_%i.strm.cmp -rate=%f -timescale=90000 %s",MP4CREATOR, CREATE, sourcename, ves->getStreamId(), fps, name);
			
			j++;
		}
		i++;
	}


	
	
}

void Transcoder::run()
{
	dprintf_full("\033[32m>>Transcoder::run\033[0m\n");
	//	vitooki_init_codecs();
	this->currentJob = 0;
	int i = 0;
	int j = 0;
	JobList* jl = this->pro->getJobList();
	//SystemFileList* sysl = this->pro->getSystemFileList();
	const char* workingDir = this->pro->getWorkingDirectory(); 
	Job* job = NULL;
	while(jl->getJobByIndex(i) != NULL)
		i++;
	this->numberJobs = i;
	printf("numberjobs: %i, current: %i\n", this->numberJobs, this->currentJob);
	while(NULL != (job =jl->getJobByIndex(j)))
	{
	
	u32 bitRate = job->getBitRate();
	bool transcode = false;
	const char* jobname = job->getOutFileName();
	char outfile[MAX_STR_LEN];
	const char* name = NULL;
	Source* source = NULL;
	if(job->isVideo()) //if is videojob
	{
		dprintf_full("\033[32mTranscoder:: is videojob\033[0m\n");
		VideoAdaptorChain* adaptor = NULL;
		/* get info of original esinfo */
		source = (Source*)job->getSource();
		assert(source);
		name = source->getFileName();
		VideoESInfo* ves = (VideoESInfo*)source->getESInfo();
		assert(ves);
		u32 ves_bitRate = ves->getAvgBandwidth();
		u32 ves_width =ves->getWidth();
		u32 ves_height = ves->getHeight();
		bool ves_color = ves->getHasColor();
		

		/* read params to be changed */
		u32 width = ((VideoJob*)job)->getWidth();
		u32 height = ((VideoJob*)job)->getHeight();
		bool col = ((VideoJob*)job)->hasColor();
		float startPos = ((VideoJob*)job)->getStartPosition();
		float endPos = ((VideoJob*)job)->getEndPosition();

#ifdef ENABLE_FFMPEG
		u32 minBitRate = ((VideoJob*)job)->getMinBitrate();
		u32 maxBitRate = ((VideoJob*)job)->getMaxBitrate();
		float fps = ((VideoJob*)job)->getFPS();
		bool varFR = ((VideoJob*)job)->getVariableFR();
		CodecID codec = ((VideoJob*)job)->getCodec();
		float aspectR = ((VideoJob*)job)->getAspectRatio();
		u32 minMacro = ((VideoJob*)job)->getMinMacroBlock();
		u32 maxMacro = ((VideoJob*)job)->getMaxMacroBlock();
		u32 minQuant = ((VideoJob*)job)->getMinQuantization();
		u32 maxQuant = ((VideoJob*)job)->getMaxQuantization();
		//u32 quantI = ((VideoJob*)job)->getQuantI();
		//u32 quantB = ((VideoJob*)job)->getQuantB();
		//u32 quantP = ((VideoJob*)job)->getQuantP();
		//bool varGop = ((VideoJob*)job)->getVariableGop();
		u32 gop = ((VideoJob*)job)->getGopSize();
		bool videoPackets = ((VideoJob*)job)->getEnableVideoPackets();
		u32 numB = ((VideoJob*)job)->getNumBFrames();
		bool ionly = ((VideoJob*)job)->getIFramesOnly();
		int bufSize = ((VideoJob*)job)->getDecBufferSize();
		float ves_aspect_ratio = ves->getAspectRatio();
#endif

		u32 startFrame = ves->getFrameNumber(startPos);
		u32 endFrame = ves->getFrameNumber(endPos);

		if(ves_width < width)
	                width = ves_width;
	        if(ves_height < height)
	                height = ves_height;
		int countAdaptors = 0;
		if( (ves_width/8) != (width/8) || (ves_height/8) != (height/8)
			|| !ves_color || bitRate < ves_bitRate) 
		{
			transcode = true;
			adaptor = new VideoAdaptorChain();
			dprintf_full("\033[32mTranscoder::built new AdaptorChain\033[0m\n");
		}
		if(transcode)
		{
			adaptor->addAdaptor(new MP4Decoder(ves,false));
			dprintf_full("\033[32mTranscoder::added new MP4Decoder\033[0m\n");
	                countAdaptors++;
	        }
		
		if( (ves->getWidth()/8) != (width/8) || (ves->getHeight()/8) != (height/8) )
		{
			adaptor->addAdaptor(new YUVSpatialReductionAdaptor(ves, &width, &height));
			dprintf_full("\033[32mTranscoder::added new YUVSpatialReductionAdaptor\033[0m\n");
			countAdaptors++;
		}

		if(ves_color && !col )
		{
			adaptor->addAdaptor(new YUVColorReductionAdaptor(ves));
			dprintf_full("\033[32mTranscoder::added new YUVColorReductionAdaptor\033[0m\n");
		        countAdaptors++;
		}

		if (bitRate > ves_bitRate)
		{
			bitRate=ves_bitRate;
		}
		if(transcode) {
#ifdef ENABLE_FFMPEG
			AVCodec *acodec = NULL;
			acodec = avcodec_find_encoder(codec);
			if(!acodec) {
				dprintf_err("\033[31mTranscoder::Error in finding encoder\033[0m\n");
				//this->exit();
				//return sth.
			}
			AVCodecContext* acc = avcodec_alloc_context();
			
			acc->bit_rate = bitRate;
			if(videoPackets)
			{
				acc->rtp_payload_size = 1480;
			}
			// acc->bit_rate_tolerance = acc->bit_rate * 10;
			//acc->frame_rate = (int)fps;

			acc->width = width;
			acc->height = height;
			
			if(ionly)
			{
				acc->gop_size = 0;
			}
			else acc->gop_size = gop;

			acc->qmin = minQuant;
			acc->qmax = maxQuant;
			acc->codec_id = codec;
			acc->max_b_frames=numB;
			
			if (ves_aspect_ratio != aspectR)
			{
				// only used by MPEG-2, stolen from mpeg4data.h
				if(aspectR==4.0/3.0 && acc->width<=525) 
				{
					acc->sample_aspect_ratio.num=10; 
					acc->sample_aspect_ratio.den=11; 
				}
				else if(aspectR==4.0/3.0 && acc->width<=625) 
				{
					acc->sample_aspect_ratio.num=12; 
					acc->sample_aspect_ratio.den=11; 
				}
				else if(aspectR==16.0/9.0 && acc->width<=525) 
				{
					acc->sample_aspect_ratio.num=40; 
					acc->sample_aspect_ratio.den=33; 
				}	
				else if(aspectR==16.0/9.0 && acc->width<=625) 
				{
					acc->sample_aspect_ratio.num=16; 
					acc->sample_aspect_ratio.den=11; 
				}						
				else if(acc->width == acc->height) 
				{
					acc->sample_aspect_ratio.num=1; 
					acc->sample_aspect_ratio.den=1; 
				}	
			}
			
			acc->rc_max_rate = maxBitRate;
			acc->rc_min_rate = minBitRate;
			acc->rc_buffer_size = bufSize;
			acc->mb_qmin = minMacro;
			acc->mb_qmax = maxMacro;
			/*
			if(varFR)	{
				acc->frame_rate_base = 1; //for var fps value is 1
			}
			*/

			adaptor->addAdaptor(new MP4Encoder(ves, bitRate, (int)ves->getFPS(), 300,  numB,
																				 UncompressedVideoFrame::ColorSpaceI420, MP4Decoder::FFMPEG, codec, (char*)acc));
			dprintf_full("\033[32mTranscoder::added new MP4Encoder\033[0m\n");
#else
			dprintf_err("Transcoder::run  compiled without ffmpeg!\n");
			::exit(1);
#endif
			countAdaptors++;
			snprintf(outfile, MAX_STR_LEN, "%s%s", workingDir, jobname);
			printf("workingDir: %s\n",workingDir);
			job->setOutFileName(outfile);
			printf("name outputfile: (%s)\n", outfile);
			ContainerInfo* ci = ContainerFile::loadContainerInfo(name);
			assert(ci);
			VideoESInfo* transcodingSource=(VideoESInfo*)ci->getES(ves->getStreamId());
			//omitHeader true for YUV output
			ContainerDemux::adaptESInfo(transcodingSource, outfile, adaptor, startFrame, endFrame, false, false);
			delete ci;
			dprintf_full("\033[32m<<Transcoder::Transcoder\033[0m\n");
		}
		else {
			char copy[MAX_STR_LEN];
			snprintf(copy, MAX_STR_LEN, "cp %s %s%s.cmp", name, workingDir, job->getOutFileName());
			system(copy);
		}
		
		printf("did job %i\n\n\n", this->currentJob);
		this->currentJob++;
	}
	else //is audiojob
	{
		printf("\033[32mTranscoder::is audio job\033[0m\n");
		//sleep(2);
		//FUTURE USE
		/* params of original esinfo */
		
		source = (Source*)job->getSource();
		AudioESInfo* aes = (AudioESInfo*)source->getESInfo();
		name = source->getFileName();
		/*u32 aes_sampleRate = aes->getSampleRate();
		u32 aes_bitsPerSample = aes->getBitsPerSample();
		u32 aes_bitRate = aes->getAvgBandwidth();
		CodecID aes_codec = aes->getCodecID();
		*/
		
		/* new params of audiojob */
		
		/*u32 sampleRate = ((AudioJob*)job)->getSampleRate();
		u32 numChannels = ((AudioJob*)job)->getNumChannels(); 
		u32 bitsPerS = ((AudioJob*)job)->getBitsPerSample();
		CodecID codec = ((AudioJob*)job)->getCodec();
		
		transcode = true;
		AVCodecContext* acc = new AVCodecContext();
		if(bitRate != aes_bitRate)
		{
			acc->bit_rate = bitRate;
		}
	
		if(aes_sampleRate != sampleRate)
		{
			acc->sample_rate = sampleRate;
		}
		
		if(aes_codec != codec)
		{
			acc->codec_id = codec;
		}
		acc->channels = numChannels;
		if( aes_bitsPerSample != bitsPerS )
		{
			acc->bits_per_sample = bitsPerS;
		}
		*/
		ContainerInfo* ci = ContainerFile::loadContainerInfo(name);
		assert(ci);
		AudioESInfo* transcodingSource=(AudioESInfo*)ci->getES(aes->getStreamId());
		//omitHeader true for YUV output
		Adaptor* adapt = new Forwarder();
		ContainerDemux::adaptESInfo(transcodingSource, jobname, adapt, 0u, 4294967295u, false, false);
		delete ci;
		this->currentJob++;
	}

	j++;
	printf("j :%i\n", j);
	printf("in list at job: %i\n", this->currentJob);
	}//while

	muxFiles();
 	this->exit();

}

float Transcoder::getProgress() const
{
	dprintf_full("\033[32m>>Transcoder::getProgress\033[0m\n");
	printf("transcoding job %i of %i (%f %c)\n", this->currentJob+1, this->numberJobs, (float)((this->currentJob*1.0)/(this->numberJobs*1.0)*100.0), '%');
	fflush(stdout);
	return ((this->currentJob*1.0)/(this->numberJobs*1.0)*100);
}

