#include "UserDefineImpl.hpp"

#include <stdio.h>
#include <stdlib.h>
#include <qwidget.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qspinbox.h>
#include <qtextedit.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qstring.h>
#include <qslider.h>
#include <qfiledialog.h>
#include <qlistview.h>

#include "avformat.h"
#include "global.hpp"
#include "JobList.hpp"
#include "Job.hpp"
#include "Project.hpp"
#include "Source.hpp"
#include "SourceList.hpp"
#include "SystemFile.hpp"
#include "VideoJob.hpp"
#include "ESInfo.hpp"
#include "VideoESInfo.hpp"
#include "adaptors/MP4Decoder.hpp"
#include "ContainerInfo.hpp"
#include "container/ffMP4ContainerFile.hpp"
#include "UserDefineImpl.hpp"



//------------------------ Konstructor ----------------------------

UserDefineImpl::UserDefineImpl(float* f_aspect_ratio,			       
			       int* i_width,
			       int* i_height,
			       QWidget* parent, 
			       const char* name,
			       bool modal, 
			       WFlags fl)
:UserDefine(parent,name,modal,fl){
	this->i_width=i_width;
	this->i_height=i_height;
	this->f_aspect_ratio=f_aspect_ratio;

	init();
}//UserDefine

//-------------------------- Destructor ---------------------------

UserDefineImpl::~UserDefineImpl(){

}//DestinationVideoImpl

//--------------------------- INIT --------------------------------

void UserDefineImpl::init(){
	int i_count;
	char c_aspect_ratio[MAX_STR_LEN];

	width->setMinValue(0);
	width->setMaxValue(10000);

	height->setMinValue(0);
	height->setMaxValue(10000);
		
	//ASPECT RATIO
	aspect_ratio->clear();

	f_aspect->setChecked(true);

	memset(xarray,0,MAX_ASPECT);
	memset(yarray,0,MAX_ASPECT);
	
	//Extension Aspect Ratio 1
	xarray[0]=16;
	xarray[1]=11;
	xarray[2]=4;
	xarray[3]=1;

	yarray[0]=9;
	yarray[1]=9;
	yarray[2]=3;
	yarray[3]=1;
	

	//SET ASPECT RATIOS
	for(i_count=0;i_count<MAX_ASPECT;i_count++){
		memset(c_aspect_ratio,0,MAX_STR_LEN);
		sprintf(c_aspect_ratio,"%i : %i",xarray[i_count],yarray[i_count]);
		aspect_ratio->insertItem((QString)c_aspect_ratio,i_count);
	}//for
	aspect_ratio->insertItem("unknown",MAX_ASPECT);
	
	//Extension Aspect Ratio 2
	if( (*this->f_aspect_ratio>1.7) && (*this->f_aspect_ratio<1.8) ){
		aspect_ratio->setCurrentItem(0);
		
	}//if
	else if( (*this->f_aspect_ratio>1.3) && (*this->f_aspect_ratio<1.4) ){
		aspect_ratio->setCurrentItem(2);
		
	}//if
	else if( (*this->f_aspect_ratio>1.2) && (*this->f_aspect_ratio<1.3) ){
		aspect_ratio->setCurrentItem(1);
		
	}//if
	else if( (*this->f_aspect_ratio==1.0) ){
		aspect_ratio->setCurrentItem(3);
		
	}//if
	else{
		aspect_ratio->setCurrentItem(MAX_ASPECT);
	}//if

	//SET WIDTH AND HEIGHT INITIAL
	width->setValue(*this->i_width);
	height->setValue(*this->i_height);

}//init

//--------------------------- CANCEL --------------------------------

void UserDefineImpl::cancel(){
	*this->f_aspect_ratio=-1;
	*this->i_height=0;
	*this->i_width=0;
	
	this->done(-1);
}//cancel

//--------------------------- OK --------------------------------

void UserDefineImpl::ok(){
	this->done(0);
}//ok

//----------------------- KEEP ASPECT RATIO -------------------------

void UserDefineImpl::k_aspect_clicked(){
	int i=-1;
	if(f_aspect->isChecked()){
		i=aspect_values();
		if(i<MAX_ASPECT){
			height->setValue(width->value()*yarray[i]/xarray[i]);
			*this->f_aspect_ratio=(float)xarray[i]/(float)yarray[i];
		}//if
	}//if
	else{
		aspect_ratio->setDisabled(true);
		*this->f_aspect_ratio=0;
	}//else
	*this->i_height=height->value();
	*this->i_width=width->value();
}//k_aspect_clicked

//----------------------- WIDTH VALUE CHANGED --------------------------

void UserDefineImpl::width_valueChanged(int i_width){
	int i=-1;
	if(f_aspect->isChecked()){
		i=aspect_values();
		if( (i<MAX_ASPECT) && (i >= 0) ){
			height->setValue(i_width*yarray[i]/xarray[i]);
			*this->f_aspect_ratio=(float)xarray[i]/(float)yarray[i];
		}//if
	}//if
	*this->i_height=height->value();
	*this->i_width=width->value();
}//width_valueChanged

//----------------------- HEIGHT VALUE CHANGED -------------------------

void UserDefineImpl::height_valueChanged(int i_height){
	int i=-1;
	if(f_aspect->isChecked()){
		i=aspect_values();
		if( (i<MAX_ASPECT) && (i >= 0) ){
			width->setValue(i_height*xarray[i]/yarray[i]);
			*this->f_aspect_ratio=(float)xarray[i]/(float)yarray[i];
		}//if
	}//if
	*this->i_height=height->value();
	*this->i_width=width->value();

}//height_valueChanged

//----------------------- ASPECT VALUES -------------------------

int UserDefineImpl::aspect_values(){
	char* c_aspect_ratio;
	int x;
	int y;
	int i;

	c_aspect_ratio=(char*)(aspect_ratio->currentText()).ascii();
		
	if(strcmp(c_aspect_ratio,"unknown")==0){
		return -1;
	}//if
	sscanf(c_aspect_ratio,"%i : %i",&x,&y);
	
	for(i=0;i<MAX_ASPECT;i++){		
		if( (x==xarray[i]) && (y==yarray[i]) ){
			return i;			
		}//if		
	}//for
	return MAX_ASPECT;
}//if

//----------------------- ASPECT RATIO ACTIVATED -------------------------

void UserDefineImpl::aspect_ratio_activated(int index){
	
	if(f_aspect->isChecked()){
		
		if(index<MAX_ASPECT){
			height->setValue(width->value()*yarray[index]/xarray[index]);	
		}//if
	}//if
	*this->f_aspect_ratio=(float)xarray[index]/(float)yarray[index];
	*this->i_height=height->value();
	*this->i_width=width->value();

}//aspect_ratio_activated

