/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   VideoJob.cpp                                                *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "VideoJob.hpp"
class Source;
#include "Job.hpp"
#include "assert.h"

/** constructor */
VideoJob::VideoJob():Job()
{
	dprintf_full("\033[32mVideoJob::empty construct\033[0m\n");
	this->name[0] = '\0';
        this->in_es = NULL;
	this->outFile[0] = '\0';
	this->is_video = true;
}

VideoJob::VideoJob(const char *name, Source* es, const char* outFile,
u32 targetBitRate, u32 minBitrate, u32 maxBitrate, int width, int height,
float FPS, bool variableFR, bool color, CodecID codec, float aspectRatio,
int minMacroBlock, int maxMacroBlock, int minQuantization, int maxQuantization,
int quantI, int quantB, int quantP, bool variableGop, int gopSize, 
bool enableVideoPackets, int videoPacketSize, int numBFrames, bool iFramesOnly, int decBufferSize, float startPosition, float endPosition):Job(name, es, outFile, targetBitRate, startPosition, endPosition)
{
	dprintf_full("\033[32mVideoJob::construct\033[0m\n");
	assert(name);
	assert(outFile);
	strncpy(this->name, name, MAX_STR_LEN-1);
	this->name[MAX_STR_LEN-1] = '\0';
	this->in_es = es;
	strncpy(this->outFile, outFile, MAX_STR_LEN-1);
	this->outFile[MAX_STR_LEN-1] = '\0';
	this->targetBitRate = targetBitRate;
	this->startPosition = startPosition;
	this->endPosition = endPosition;
	
	this->minBitrate = minBitrate;
	this->maxBitrate = maxBitrate;
	this->width = width;
	this->height = height;
	this->FPS = FPS;
	this->variableFR = variableFR;
	this->color = color;
	this->codec = codec;
	this->aspectRatio = aspectRatio;
	this->minMacroBlock = minMacroBlock;
	this->maxMacroBlock = maxMacroBlock;
	this->minQuantization = minQuantization;
	this->maxQuantization = maxQuantization;
	this->quantI = quantI;
	this->quantB = quantB;
	this->quantP = quantP;
	this->variableGop = variableGop;
	this->gopSize = gopSize;
	this->enableVideoPackets = enableVideoPackets;
	this->videoPacketSize = videoPacketSize;
	this->numBFrames = numBFrames;
	this->iFramesOnly = iFramesOnly;
	this->decBufferSize = decBufferSize;

	this->is_video = true;
}
	
/** destructor */
VideoJob::~VideoJob()
{
	dprintf_full("\033[32mVideoJob::destruct\033[0m\n");
}

u32 VideoJob::getMinBitrate() const
{
	dprintf_full("\033[32mVideoJob::getMinBitrate\033[0m\n");
	this->checkReferences();
	return this->minBitrate;
};

u32 VideoJob::getMaxBitrate() const
{
	dprintf_full("\033[32mVideoJob::getMaxBitrate\033[0m\n");
	this->checkReferences();
	return this->maxBitrate;
};

int VideoJob::getWidth() const
{
	dprintf_full("\033[32mVideoJob::getWidth\033[0m\n");
	this->checkReferences();
	return this->width;
};

int VideoJob::getHeight() const
{
	dprintf_full("\033[32mVideoJob::getHeight\033[0m\n");
	this->checkReferences();
	return this->height;
};

float VideoJob::getFPS() const
{
	dprintf_full("\033[32mVideoJob::getFPS\033[0m\n");
	this->checkReferences();
	return this->FPS;
};

bool VideoJob::getVariableFR() const
{
	dprintf_full("\033[32mVideoJob::getVariable\033[0m\n");
	this->checkReferences();
	return this->variableFR;
};

bool VideoJob::hasColor() const
{
	dprintf_full("\033[32mVideoJob::hasColor\033[0m\n");
	this->checkReferences();
	return this->color;
};

CodecID VideoJob::getCodec() const
{
	dprintf_full("\033[32mVideoJob::getCodec\033[0m\n");
	this->checkReferences();
	return this->codec;
};

float VideoJob::getAspectRatio() const
{
	dprintf_full("\033[32mVideoJob::getAspectRatio\033[0m\n");
	this->checkReferences();
	return this->aspectRatio;
}

int VideoJob::getMinMacroBlock() const
{
	dprintf_full("\033[32mVideoJob::getMinMacroBlock\033[0m\n");
	this->checkReferences();
	return this->minMacroBlock;
};

int VideoJob::getMaxMacroBlock() const
{
	dprintf_full("\033[32mVideoJob::getMaxMacroBlock\033[0m\n");
	this->checkReferences();
	return this->maxMacroBlock;
};

int VideoJob::getMinQuantization() const
{
	dprintf_full("\033[32mVideoJob::getMinQuantization\033[0m\n");
	this->checkReferences();
	return this->minQuantization;
};

int VideoJob::getMaxQuantization() const
{
	dprintf_full("\033[32mVideoJob::getMaxQuantization\033[0m\n");
	this->checkReferences();
	return this->maxQuantization;
};

int VideoJob::getQuantI() const
{
	dprintf_full("\033[32mVideoJob::getQuantI\033[0m\n");
	this->checkReferences();
	return this->quantI;
};

int VideoJob::getQuantB() const
{
	dprintf_full("\033[32mVideoJob::getQuantB\033[0m\n");
	this->checkReferences();
	return this->quantB;
};

int VideoJob::getQuantP() const
{
	dprintf_full("\033[32mVideoJob::getQuantP\033[0m\n");
	this->checkReferences();
	return this->quantP;
};

bool VideoJob::getVariableGop() const
{
	dprintf_full("\033[32mVideoJob::getVariableGop\033[0m\n");
	this->checkReferences();
	return this->variableGop;
}

int VideoJob::getGopSize() const
{
	dprintf_full("\033[32mVideoJob::getGopSize\033[0m\n");
	this->checkReferences();
	return this->gopSize;
}

bool VideoJob::getEnableVideoPackets() const
{
	dprintf_full("\033[32mVideoJob::getEnableVideoPackets\033[0m\n");
	this->checkReferences();
	return this->enableVideoPackets;
};

int VideoJob::getVideoPacketSize() const
{
	dprintf_full("\033[32mVideoJob::getVideoPacketSize\033[0m\n");
	this->checkReferences();
	return this->videoPacketSize;
};

int VideoJob::getNumBFrames() const
{
	dprintf_full("\033[32mVideoJob::getNumBFrames\033[0m\n");
	this->checkReferences();
	return this->numBFrames;
};

bool VideoJob::getIFramesOnly() const
{
	dprintf_full("\033[32mVideoJob::getIFramesOnly\033[0m\n");
	this->checkReferences();
	return this->iFramesOnly;
};

int VideoJob::getDecBufferSize() const
{
	dprintf_full("\033[32mVideoJob::getDecBufferSize\033[0m\n");
	this->checkReferences();
	return this->decBufferSize;
};

	
/** Implementation of virtual methods of Job */
bool VideoJob::toString(char* output, int max_len)
{
	dprintf_full("\033[32m>> VideoJob::toString(output, len)\033[0m\n");
	assert(output && max_len>0);
	this->checkReferences();
	char* format = (char*)malloc(max_len);
	memset(format, 0, max_len);
	memset(output, 0, max_len);
	
	if(max_len < snprintf(format, max_len, "%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s", VJOB_START, JOBID, NAME_PRINT, FILENAME_PRINT,
	TARGET_BR, START_POS, END_POS, MIN_BR, MAX_BR, WIDTH, HEIGHT, VIDEO_FPS,
	VAR_FR, COLOR, CODECID, ASPECTRATIO, MINMBLOCK, MAXMBLOCK, MINQUANT, 
	MAXQUANT, QUANTI, QUANTB, QUANTP, VAR_GOP, GOPSIZE, ENABLE_VP, 
	VIDEO_PACK_SIZE, BFRAMES, IFRAMES_ONLY, DEC_BUF_SIZE, VJOB_END))
	{
		free(format);
		dprintf_err("\033[31mVideoJob::toString : len too short\033[0m\n");
		return false;
	}
	if(max_len < snprintf(output, max_len, format, this->jobID, this->name,
	this->outFile, this->targetBitRate, this->startPosition, 
	this->endPosition, this->minBitrate, this->maxBitrate,
	this->width, this->height, this->FPS, (int)this->variableFR, 
	(int)this->color, this->codec, this->aspectRatio, this->minMacroBlock,
	this->maxMacroBlock, this->minQuantization, this->maxQuantization, 
	this->quantI, this->quantB, this->quantP, (int)this->variableGop, 
	this->gopSize, (int)this->enableVideoPackets, this->videoPacketSize, 
	this->numBFrames, (int)this->iFramesOnly, this->decBufferSize))
	{
		free(format);
		dprintf_err("\033[31mVideoJob::toString : len too short\033[0m\n");
		return false;
	}
	free(format);
	dprintf_full("\033[32m<< VideoJob::toString\033[0m\n");
	return true;
}


char* VideoJob::toString()
{
	dprintf_full("\033[32m>> VideoJob::toString\033[0m\n");
	this->checkReferences();
	char* output = (char*)malloc(MAX_XML_LEN);
	if(toString(output, MAX_XML_LEN))
	{
		dprintf_full("\033[32m<< VideoJob::toString ok\033[0m\n");
		return output;
	}
	else
	{
		free(output);
		dprintf_err("\033[31mVideoJob::toString\033[0m\n");
		return NULL;
	}
}

bool VideoJob::fromString(const char* initString)
{
	dprintf_full("\033[32m>> VideoJob::fromString\033[0m\n");
	assert(initString);
	//this->checkReferences();
	const char* next = initString;
	char* end = NULL;
	if( next == strstr(next, VJOB_START))
	{
		next = (const char*) next+strlen(VJOB_START);
	}
	else
	{
		dprintf_err("\033[31mVideoJob::fromString\033[0m\n");
		return false;
	}
	if(1==sscanf(next, JOBID, &this->jobID))
	{
		next = (const char*)strchr(next, '\n')+1;
	}
	else
	{
		dprintf_err("\033[31mVideoJob::fromString\033[0m\n");
		return false;
	}
	if(next == strstr(next, NAME_SCAN))
	{	
		next=(const char*)next+strlen(NAME_SCAN);
		if(NULL!=(end=strchr(next, '\n')))
		{
			memset(this->name, 0, MAX_STR_LEN);
			if(NULL==memcpy(this->name, next, end-next))
			{
				dprintf_err("\033[31mVideoJob::fromString\033[0m\n");
				return false;
	
			}
			else
			{
				next = (const char*)end+1;
			}
		}
		else 
		{
			dprintf_err("\033[31mVideoJob::fromString\033[0m\n");
			return false;
		}
	}
	else 
	{
		dprintf_err("\033[31mVideoJob::fromString\033[0m\n");
		return false;
	}
	if(next == strstr(next, FILENAME_SCAN))
        {
		next=(const char*)next+strlen(FILENAME_SCAN);
		if(NULL!=(end=strchr(next, '\n')))
	        {
			memset(this->outFile, 0, MAX_STR_LEN);
			if(NULL==memcpy(this->outFile, next, end-next))
			{
				dprintf_err("\033[31mVideoJob::fromString\033[0m\n");
				return false;
			}
			else 
			{
				next = (const char*)end+1;
			}
		}
		else 
		{
			dprintf_err("\033[31mVideoJob::fromString\033[0m\n");
			return false;
		}
	}
	else 
	{
		dprintf_err("\033[31mVideoJob::fromString\033[0m\n");
		return false;
	}
	
	char* format = (char*)malloc(MAX_XML_LEN);
	memset(format, 0, MAX_XML_LEN);

	snprintf(format, MAX_XML_LEN, "%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s", TARGET_BR, START_POS, END_POS, MIN_BR, MAX_BR, WIDTH, HEIGHT,
	VIDEO_FPS, VAR_FR, COLOR, CODECID, ASPECTRATIO, MINMBLOCK, MAXMBLOCK, 
	MINQUANT, MAXQUANT, QUANTI, QUANTB, QUANTP, VAR_GOP, GOPSIZE, ENABLE_VP, 	VIDEO_PACK_SIZE, BFRAMES, IFRAMES_ONLY, DEC_BUF_SIZE, VJOB_END);

	if( 26 == sscanf(next, format, &this->targetBitRate, 
	&this->startPosition, &this->endPosition, &this->minBitrate,
	&this->maxBitrate, &this->width, &this->height, &this->FPS, 
	&this->variableFR, &this->color, &this->codec, &this->aspectRatio,
	&this->minMacroBlock, &this->maxMacroBlock, &this->minQuantization, 
	&this->maxQuantization, &this->quantI, &this->quantB, &this->quantP, 
	&this->variableGop, &this->gopSize, &this->enableVideoPackets, 
	&this->videoPacketSize, &this->numBFrames, &this->iFramesOnly, 
	&this->decBufferSize))
	{
		free(format);
		dprintf_full("\033[32m<< VideoJob::fromString\033[0m\n");
		return true;
	}
	else
	{
		free(format);
		dprintf_err("\033[31mVideoJob::fromString\033[0m\n");
		return false;
	}						
}


void VideoJob::setMinBitrate(u32 minBitrate)
{
	dprintf_full("\033[32mVideoJob::setMinBitrate\033[0m\n");
	this->checkReferences();
	this->minBitrate = minBitrate;
}

void VideoJob::setMaxBitrate(u32 maxBitrate)
{
	dprintf_full("\033[32mVideoJob::setMaxBitrate\033[0m\n");
	this->checkReferences();
	this->maxBitrate = maxBitrate;
}

void VideoJob::setWidth(int width)
{
	dprintf_full("\033[32mVideoJob::setWidth\033[0m\n");
	this->checkReferences();
	this->width = width;
}

void VideoJob::setHeight(int height)
{
	dprintf_full("\033[32mVideoJob::setHeigth\033[0m\n");
	this->checkReferences();
	this->height = height;
}

void VideoJob::setFPS(float FPS)
{
	dprintf_full("\033[32mVideoJob::setFPS\033[0m\n");
	this->checkReferences();
	this->FPS = FPS;
}

void VideoJob::setColor(bool color)
{
	dprintf_full("\033[32mVideoJob::setColor\033[0m\n");
	this->checkReferences();
	this->color = color;
}
void VideoJob::setVariableFR(bool variableFR)
{
	dprintf_full("\033[32mVideoJob::setVariableFR\033[0m\n");
	this->checkReferences();
	this->variableFR = variableFR;
}

void VideoJob::setCodec(CodecID codec)
{
	dprintf_full("\033[32mVideoJob::setCodec\033[0m\n");
	this->checkReferences();
	this->codec = codec;
}

void VideoJob::setAspectRatio(float aspectRatio)
{
	dprintf_full("\033[32mVideoJob::setAspectRatio\033[0m\n");
	this->checkReferences();
	this->aspectRatio = aspectRatio;
}

void VideoJob::setMinMacroBlock(int minMacroBlock)
{
	dprintf_full("\033[32mVideoJob::setMinMacroBlock\033[0m\n");
	this->checkReferences();
	this->minMacroBlock = minMacroBlock;
}

void VideoJob::setMaxMacroBlock(int maxMacroBlock)
{
	dprintf_full("\033[32mVideoJob::setMaxMacroBlock\033[0m\n");
	this->checkReferences();
	this->maxMacroBlock = maxMacroBlock;
}

void VideoJob::setMinQuantization(int minQuantization)
{
	dprintf_full("\033[32mVideoJob::setMinQuantization\033[0m\n");
	this->checkReferences();
	this->minQuantization = minQuantization;
}

void VideoJob::setMaxQuantization(int maxQuantization)
{
	dprintf_full("\033[32mVideoJob::setMaxQuantization\033[0m\n");
	this->checkReferences();
	this->maxQuantization = maxQuantization;
}

void VideoJob::setQuantI(int quantI)
{
	dprintf_full("\033[32mVideoJob::setQuantI\033[0m\n");
	this->checkReferences();
	this->quantI = quantI;
}

void VideoJob::setQuantB(int quantB)
{
	dprintf_full("\033[32mVideoJob::setQuantB\033[0m\n");
	this->checkReferences();
	this->quantB = quantB;
}

void VideoJob::setQuantP(int quantP)
{
	dprintf_full("\033[32mVideoJob::setQuantP\033[0m\n");
	this->checkReferences();
	this->quantP = quantP;
}

void VideoJob::setVariableGop(bool variableGop)
{
	dprintf_full("\033[32mVideoJob::setVariableGop\033[0m\n");
	this->checkReferences();
	this->variableGop = variableGop;
}

void VideoJob::setGopSize(int gopSize)
{
	dprintf_full("\033[32mVideoJob::setGopSize\033[0m\n");
	this->checkReferences();
	this->gopSize = gopSize;
}
	 


void VideoJob::setEnableVideoPackets(bool enableVideoPackets)
{
	dprintf_full("\033[32mVideoJob::setEnableVideoPackets\033[0m\n");
	this->checkReferences();
	this->enableVideoPackets = enableVideoPackets;
}

void VideoJob::setVideoPacketSize(int videoPacketSize)
{
	dprintf_full("\033[32mVideoJob::setVideoPacketSize\033[0m\n");
	this->checkReferences();
	this->videoPacketSize = videoPacketSize;
}

void VideoJob::setNumBFrames(int numBFrames)
{
	dprintf_full("\033[32mVideoJob::setNumBFrames\033[0m\n");
	this->checkReferences();
	this->numBFrames = numBFrames;
}

void VideoJob::setIFramesOnly(bool iFramesOnly)
{
	dprintf_full("\033[32mVideoJob::setIFramesOnly\033[0m\n");
	this->checkReferences();
	this->iFramesOnly = iFramesOnly;
}

void VideoJob::setDecBufferSize(int decBufferSize)
{
	dprintf_full("\033[32mVideoJob::setDecBufferSize\033[0m\n");
	this->checkReferences();
	this->decBufferSize = decBufferSize;
}

Job* VideoJob::clone(Source* new_in_es)
{
	dprintf_full("\033[32m>> VideoJob::clone\033[0m\n");
	//assert(new_in_es);
	this->checkReferences();
	VideoJob* clone = new VideoJob(this->name, this->in_es, this->outFile,
	this->targetBitRate, this->minBitrate, this->maxBitrate,
	this->width, this->height, this->FPS, this->variableFR, this->color,
	this->codec, this->aspectRatio, this->minMacroBlock,
	this->maxMacroBlock, this->minQuantization, this->maxQuantization, 
	this->quantI, this->quantB, this->quantP, this->variableGop, 
	this->gopSize, this->enableVideoPackets, this->videoPacketSize, 
	this->numBFrames, this->iFramesOnly, this->decBufferSize, 
	this->startPosition, this->endPosition);
	if(new_in_es != NULL)
	{
	clone->setSource(new_in_es);
	}
	dprintf_full("\033[32m<< VideoJob::clone\033[0m\n");
	return clone;
}


const char* VideoJob::getDescription() {
	dprintf_full("\033[32mVideoJob::generateDescription\033[0m\n");
	char codecName[MAX_STR_LEN];
	strcpy(codecName,"Unknown");

#ifdef ENABLE_FFMPEG
	AVCodec* av=avcodec_find_decoder(this->codec);
	if(av)
		strcpy(codecName,av->name);
#endif

	snprintf((char*)description, MAX_STR_LEN, "%10s: %4ukBps; %4ux%4u; %2.1f/%1uB", 
					 codecName, this->targetBitRate/1024, this->width, this->height, 
					 this->FPS, this->numBFrames);
	return description;
}
