/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   VideoJob.hpp                                                *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef __VIDEOJOB_HPP__
#define __VIDEOJOB_HPP__

#include "Job.hpp"
#include "avformat.h"  //for CodecID
#include "xml.hpp"
#include "global.hpp"

class VideoJob:public Job
{

protected:
	/** params of the destination file*/
	u32 minBitrate;
	u32 maxBitrate;
	int width;
	int height;
	float FPS;
	bool variableFR;
	bool color;
	CodecID codec;
	float aspectRatio;
	int minMacroBlock;
	int maxMacroBlock;
	int minQuantization;
	int maxQuantization;
	int quantI;
	int quantB;
	int quantP;
	bool variableGop;
	int gopSize;
	bool enableVideoPackets;
	int videoPacketSize;
	// FIXME
	int numBFrames;
	bool iFramesOnly;
	int decBufferSize;

public:
	/** constructor */
	VideoJob();
	VideoJob(const char *name, Source* es, const char* outFile,
	u32 targetBitRate,u32 minBitrate, u32 maxBitrate, int width,
	int height, float FPS, bool variableFR, bool color, CodecID codec, 
	float aspectRatio, int minMacroBlock, int maxMacroBlock, 
	int minQuantization, int maxQuantization, int quantI, int quantB,
	int quantP, bool variableGop, int gopSize, bool enableVideoPackets,
	int videoPacketSize, int numBFrames, bool iFramesOnly, 
	int decBufferSize, float startPosition = 0, float endPosition = 0);
	
	/** destructor */
	virtual ~VideoJob();
	
	/** methods to get params of VideoJob */
	u32 getMinBitrate() const;
	u32 getMaxBitrate() const;
	int getWidth() const;
	int getHeight() const;
	float getFPS() const;
	bool getVariableFR() const;
	bool hasColor() const;
	CodecID getCodec() const;
	float getAspectRatio() const;
	int getMinMacroBlock() const;
	int getMaxMacroBlock() const;
	int getMinQuantization() const;
	int getMaxQuantization() const;
	int getQuantI() const;
	int getQuantB() const;
	int getQuantP() const;
	bool getVariableGop() const;
	int getGopSize() const;
	bool getEnableVideoPackets() const;
	int getVideoPacketSize() const;
	int getNumBFrames() const;
	bool getIFramesOnly() const;
	int getDecBufferSize() const;
	const char* getDescription();
	
	/** Implementation of virtual methods of Job */
	bool toString(char* buf, int max_len);
	char* toString();
	bool fromString(const char* initString);

	/** methods to set params of VideoJob */
	void setMinBitrate(u32 minBitrate);
	void setMaxBitrate(u32 maxBitrate);
	void setWidth(int width);
	void setHeight(int height);
	void setFPS(float FPS);
	void setVariableFR(bool variableFR);
	void setColor(bool color);
	void setCodec(CodecID codec);
	void setAspectRatio(float aspectRatio);
	void setMinMacroBlock(int minMacroBlock);
	void setMaxMacroBlock(int maxMacroBlock);
	void setMinQuantization(int minQuantization);
	void setMaxQuantization(int maxQuantization);
	void setQuantI(int quantI);
	void setQuantB(int quantB);
	void setQuantP(int quantP);
	void setVariableGop(bool variableGop);
	void setGopSize(int gopSize);
	void setEnableVideoPackets(bool enableVideoPackets);
	void setVideoPacketSize(int videoPacketSize);
	void setNumBFrames(int numBFrames);
	void setIFramesOnly(bool iFramesOnly);
	void setDecBufferSize(int decBufferSize);

	/** implementation of superclass to clone VideoJob */
	Job* clone(Source* new_in_es);

}; //class VideoJob
#endif
