 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	PerformanceTest.cpp                                           *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "PerformanceTest.hpp"
#include "container/ContainerDemux.hpp"
#include "DataChannel.hpp"

PerformanceTest::PerformanceTest(VideoESInfo *pESInfo, ContainerInfo *pContainerFile, int iTest,MP4Decoder::meCoderType codec)
{
	assert(pESInfo!=NULL);
	mpESInfo = pESInfo;
	mpContainerFile = pContainerFile;
	if(codec==MP4Decoder::XVID) {
		dprintf_full("PerformanceTest::PerformanceTest XVID iTest: %i\n",iTest);
	}
	else if(codec==MP4Decoder::FFMPEG) {
		dprintf_full("PerformanceTest::PerformanceTest FFMPEG  iTest: %i\n",iTest);
	}
	else {
		dprintf_err("PerformanceTest::PerformanceTest %i(unknown). Defaulting to FFMPEG  iTest: %i\n",(int)codec,iTest);
		codec=MP4Decoder::FFMPEG;
	}
	muiDecodeTime = 0;
	muiAdaptTime = 0;
	muiEncodeTime = 0;
	
	u32 width = pESInfo->getWidth()/2;
	u32 height = pESInfo->getHeight()/2;
	
	mpAdaptorChain = new VideoAdaptorChain();
	
	// iTest: 1 = spatial
	//		  2 = color
	//		  3 = quality
	//		  4 = temporal
	//		  5 = strong temporal
	switch(iTest)
	{
	case 1:
		{
			// Spatial Reduction Adaptor
			mpAdaptorChain->addAdaptor(new TimeMeasureAdaptor(new MP4Decoder(pESInfo,false,
																																			 UncompressedVideoFrame::ColorSpaceI420,codec),
																												stdout));
			mpAdaptorChain->addAdaptor(new TimeMeasureAdaptor(new YUVSpatialReductionAdaptor(pESInfo, &width, &height),stdout));
			pESInfo->setWidth(width);
			pESInfo->setHeight(height);
			mpAdaptorChain->addAdaptor(new TimeMeasureAdaptor(new MP4Encoder(pESInfo, 3000000,(int)pESInfo->getFPS(),
																																			 300 /*GopSize*/ ,0 /*BVOPs*/,
																																			 UncompressedVideoFrame::ColorSpaceI420,codec)
																												,stdout));
			break;
		}


	case 2:
		{
			// Color Reducton Adaptor
			mpAdaptorChain->addAdaptor(new TimeMeasureAdaptor(new MP4Decoder(pESInfo,false,
																																			 UncompressedVideoFrame::ColorSpaceI420,codec),
																												stdout));
			mpAdaptorChain->addAdaptor(new TimeMeasureAdaptor(new YUVColorReductionAdaptor(pESInfo),stdout));
			mpAdaptorChain->addAdaptor(new TimeMeasureAdaptor(new MP4Encoder(pESInfo, pESInfo->getAvgBandwidth() * 80 / 100,
																																			 (int)pESInfo->getFPS(), 300,0 /*BVOPs*/,
																																			 UncompressedVideoFrame::ColorSpaceI420,codec),
																												stdout));
			break;
		}


	case 3:
		{
			// Quality Reducton Adaptor
			mpAdaptorChain->addAdaptor(new TimeMeasureAdaptor(new MP4Decoder(pESInfo,false,UncompressedVideoFrame::ColorSpaceI420,codec),stdout));
			mpAdaptorChain->addAdaptor(new TimeMeasureAdaptor(new MP4Encoder(pESInfo, 150000,(int)pESInfo->getFPS(), 300,0 /*BVOPs*/,UncompressedVideoFrame::ColorSpaceI420,codec),stdout));
			break;
		}


	case 4:
		{
			// Temporal Reduction Adaptor
			mpAdaptorChain->addAdaptor(new TimeMeasureAdaptor(new TemporalAdaptor(pESInfo),stdout));
			break;
		}


	case 5:
		{
			// Strong Temporal Reduction Adaptor
			mpAdaptorChain->addAdaptor(new TimeMeasureAdaptor(new StrongTemporalAdaptor(pESInfo),stdout));
			break;
		}
	default:
		{
			dprintf_err("Invalid option to PerformanceTest::PerformanceTest()\n");
			break;
		}
	}
}

PerformanceTest::~PerformanceTest()
{
	if(mpAdaptorChain)
		delete mpAdaptorChain;
}

void PerformanceTest::run()
{
	// for MPEG4 output
	DataChannel* dc=ContainerDemux::createDataChannel(mpESInfo,NULL, mpAdaptorChain,0u,0xffffffffu);
	if(dc) {
		dc->start();
		dc->wait();
	}
	else {
		dprintf_err("PerformanceTest::run() failed to create DataChannel\n");
	}
	
	std::list<Adaptor*> *ada= mpAdaptorChain->getAdaptors();
	std::list<Adaptor*>::iterator pAdIt=ada->begin();
	TimeMeasureAdaptor* t=(TimeMeasureAdaptor*)(*pAdIt);
	if(ada->size()==1) { // temporal adaptation
		muiAdaptTime=t->getTimeSpent();
	}
	else if(ada->size()==2) { //quality reduction
		muiDecodeTime=t->getTimeSpent();
		++pAdIt;
		t=(TimeMeasureAdaptor*)(*pAdIt);
		muiEncodeTime=t->getTimeSpent();	
	}
	else if(ada->size()==3) { // rest
		muiDecodeTime=t->getTimeSpent();
		++pAdIt;
		t=(TimeMeasureAdaptor*)(*pAdIt);
		muiAdaptTime=t->getTimeSpent();	
		++pAdIt;
		t=(TimeMeasureAdaptor*)(*pAdIt);
		muiEncodeTime=t->getTimeSpent();		
	}
	delete dc;	
	// also deleted adaptor
	mpAdaptorChain=NULL;
 	this->exit();

}
