 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	main.cpp                                                      *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

//#include "adaptors/xvidInterface.h"
#include "vitooki.hpp"

#ifndef ISOMP4
#include "io/ffMP4IO.hpp"
#include "container/ffMP4ContainerFile.hpp"
#include "avformat.h"
#else
#include "io/isoMP4IO.hpp"
#include "container/ContainerFile.hpp"
#include "avcodec.h"
#endif

#include "global.hpp"
#include "PerformanceTest.hpp"

#ifdef WIN32
#include <sys/timeb.h>
#endif

#include "container/ContainerDemux.hpp"
#include "adaptors/Forwarder.hpp"


#define MYUSE_FILE "../../media/alien_2b_IP16q_B28q_25fps_30I.mp4"


static long getdifftime(struct timeval *t1, struct timeval *t2)
{
	long ret = 0;

	ret = t2->tv_usec - t1->tv_usec;
	ret += (t2->tv_sec - t1->tv_sec) * 1000000;

	return ret;
};


int main(int argc, char* argv[])
{
	system("rm -rf demux\n");
	system("mkdir demux\n");
	int testruns=1;
	vitooki_init_codecs();
	
	// Coder Chain test
	ContainerInfo *mp4File=NULL;
	
	if(argc==3) {
		if(sscanf(argv[2],"%i",&testruns)!=1)
			testruns=1;
		if(testruns<0)
			testruns=1;
		if(testruns>100) {
			printf("Not more than 100 test runs allowed\n");
			testruns=100;
		}
		
	}
	if(argc>=2) {
		mp4File= ContainerFile::loadContainerInfo(argv[1]);
	}
	else {
		printf("Usage: adaptorbench <mp4file> <testruns>\nDefaulting to %s\n",MYUSE_FILE);
		mp4File= ContainerFile::loadContainerInfo(MYUSE_FILE);
	}
	if(!mp4File) {
		if(argc!=2)
			printf("Couldn't find %s\r\n",MYUSE_FILE);
		else
			printf("Couldn't find %s\r\n",argv[1]);
		printf("Usage: adaptorbench <mp4file> <testruns>\n");	
		exit(0);
	}

	VideoESInfo* vis=mp4File->getFirstVisualES();
	if(vis==NULL) {
		printf("%s doesn't contain any visual stream. Exitting...\r\n",mp4File->getLocalFile());
		exit(0);
	}

	PerformanceTest *pt1=NULL;
	FILE* fp=fopen("bench.res","wb");
	if(fp==NULL)
		fp=stdout;
	struct timeval t1;
	struct timeval t2;
	Forwarder* f=new Forwarder();
	ContainerDemux::demultiplexAndUpdateIO(mp4File, "demux",f, false);
	// 1st index: xvid ffmpeg, 2nd: test type, 3rd index: decode-adapt-encode, total elapsed time
	u32 testResults[4];
	u32 overhead=0;
	const char* tmp[6]={"INVALID         ","SPATIALREDUCTION","COLORREDUCTION  ",
				"QUALITYREDUCTION","TEMPORAL        ","STRONGTEMPORAL  "};
	const char* codc[3]={"UNKNOWN","XVID","FFMPEG"};
	delete f;
	fprintf(fp,"#  id\t\t\t\ttestrun\t\tdecode\t\tadapt\t\tencode\t  elapsedtime  overhead (all values microsecs!)\n");
	for(int j=2;j>=1;j--) { // FFMPEG then XVID
		for (int i=1;i<6;i++) {  	// iTest: 1 = spatial,2 = color,3 = quality,4 = temporal,5 = strong temporal

			for(int h=0;h<testruns;h++) {
				ContainerInfo* mp4Clone=mp4File->clone();
				VideoESInfo* visClone=mp4Clone->getFirstVisualES();
				assert(visClone);
				pt1 = new PerformanceTest(visClone, mp4Clone, i, (MP4Decoder::eCoderType)j);
				gettimeofday(&t1, NULL);
	
				if(pt1)
				{
					pt1->start();
					pt1->wait();
				}
	
				gettimeofday(&t2, NULL);
	
				testResults[0]=pt1->getDecodeTimeInMicroSecs();
				testResults[1]=pt1->getAdaptTimeInMicroSecs();
				testResults[2]=pt1->getEncodeTimeInMicroSecs();
				testResults[3]=(u64)(getdifftime(&t1, &t2));
				overhead=testResults[3]-testResults[2]-testResults[1]-testResults[0];
				fprintf(fp,"%6s::%s\t%3i\t%12u\t%12u\t%12u\t%12u\t%12u\n",codc[j],tmp[i],h,
					testResults[0],testResults[1],testResults[2],testResults[3],overhead);
				fflush(fp);
				delete pt1;
				delete mp4Clone;
			}
		}
	}
	fclose(fp);
	delete mp4File;
	return 0;
};
