/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: dvdleech.cpp                                                  *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include <stdio.h>

#include "container/DVDContainerFile.hpp"
#include "io/DVDIO.hpp"
#include "ContainerInfo.hpp"
#include "Frame.hpp"
#include "Adaptor.hpp"
#include "container/ContainerDemux.hpp"
#include "adaptors/Forwarder.hpp"
#include "adaptors/MP4Decoder.hpp"
#define MAXFRAMES 50 //frames to deliver
#define TORIP 5 //percentage to rip (1-99 % other values mean 100%)
#define PLAYFRAMES 2600u //frames to visualize

void usage(){
	printf("USAGE:\n\n");
	printf("dvdleech <path_to_dvd-device> <temporary_target>\n");
	printf("-> e.g. dvdleech /mnt/dvd /home/foo/bar.vob\n");
	exit(0);
}

int main(int argc, char *argv[]){
	
	if(argc < 2 || argv[1]=="--help" || argv[1]=="-h")
		usage();
	
	printf("\n	Welcome to dvdleech\n\n");
	printf("a small test tool for vitooki dvd integration\n");
	
	/* PHASE 1 information gathering */
	printf("\n\n############################################################\n");
	printf("dvdleech: PHASE 1\n");
	printf("dvdleech: INFORMATION GATHERING\n");
	printf("############################################################\n\n");
	
	/* open dvd */
	printf("dvdleech: Open DVD\n");
	dvd_handle_t *dvd_handle = DVDContainerFile::openDVD(argv[1]);
	if(!dvd_handle)
		usage();
	
	/* get the filename again (this is only for testing) */
	char *filename = DVDContainerFile::getFileName(dvd_handle);
	printf("dvdleech: Filename has been set to:\n%s\n\n",filename);	
	
	/* get number of vts */
	u32 nr_vts = DVDContainerFile::getNrOfVTS(dvd_handle);
	printf("dvdleech: DVD contains %i video title set(s)\n",nr_vts);
	
	/* get suggested main title */
	u32 main_title_no = DVDContainerFile::getMainTitleNo(dvd_handle);
	printf("dvdleech: Suggested main title: %i\n", main_title_no);
	
	/* load the containerinfo for the main video title set */
	printf("dvdleech: Loading ContainerInfo for VTS %i\n",main_title_no);
	ContainerInfo *vts = DVDContainerFile::loadContainerInfo(dvd_handle,main_title_no);
	
	printf("dvdleech: Phase 1 finished!\n");
		
	/* PHASE 2 ripping */
	printf("\n\n\n\n\n############################################################\n");
	printf("dvdleech: PHASE 2\n");
	printf("dvdleech: RIPPING\n");
	printf("############################################################\n\n");
	
	printf("\n...press enter to continue...\n");
	int c=getchar();
	
	/* set temporary target name to copy vobfile */
	printf("dvdleech: Set targetname to:\n%s\n\n",argv[2]);
	DVDContainerFile::setTargetName(dvd_handle,argv[2]);
	
	/* copy main title */
	printf("dvdleech: Rip main title to target location\n");
	if ( !DVDContainerFile::copyMain(dvd_handle,TORIP) )
		printf("dvdleech: Rip successful\n");
	
	/* close dvd */
	printf("dvdleech: Close	DVD\n");
	DVDContainerFile::closeDVD(dvd_handle);
		
	/* PHASE 3 data gathering */
	printf("\n\n\n\n\n############################################################\n");
	printf("dvdleech: PHASE 3\n");
	printf("dvdleech: DATA GATHERING\n");
	printf("############################################################\n\n");

	printf("\n...press enter to continue...\n");
	c=getchar();	
	
	printf("dvdleech: Get visual ESInfo for main title\n");
	
	/* get first visual esinfo */
	VideoESInfo *esinfo = vts->getFirstVisualES();
		
	///* open VOB file from dvd*/
	//printf("getting frames from dvd!\n");
	//char filename2[255];
	//sprintf(filename2, "%s/VIDEO_TS/VTS_%02i_1.VOB", filename, main_title_no);
	//DVDIO *dvdio = new DVDIO((ESInfo*)esinfo, filename2);
	//u32 error1 = dvdio->open();
	//printf("dvdleech: Open VOB file success status: %i\n",error);
	
	///* get frames for given esinfo in IO */
	//printf("dvdleech: getting %i video frames...\n",MAXFRAMES);
	//Frame *frame1;
	//int i1;
	//for (i1=0;i1<MAXFRAMES;i1++)
	//	frame = dvdio->getFrame();
	//printf("dvdleech: %i videoframes read!\n",MAXFRAMES);

	/* open VOB file from target */
	printf("dvdleech: Open IO for given ESInfo\n");
	DVDIO *dvdio2 = new DVDIO((ESInfo*)esinfo, argv[2]);
	u32 error2 = dvdio2->open();
	printf("dvdleech: Open VOB file success status: %i\n",error2);
	
	/*get frames for given es info in IO */
	printf("dvdleech: Get %i video frames ...\n",MAXFRAMES);
	int i2;
	Frame *frame2;
	for (i2=0; i2 < MAXFRAMES; i2 ++)
		frame2 = dvdio2->getFrame();
	printf("dvdleech: %i videoframes read!\n",MAXFRAMES);
	
	printf("\n...press enter to continue...\n");
	c=getchar();
		
	/* visualization test part */
	printf("dvdleech: VISUALIZATION DEMO!\n");
		
	/* set containerinfo to new filename of vobfile! */
	vts->setLocalFile(argv[2]);
	const char *vobfile = vts->getLocalFile();
	
	/* play video from ripped vob file */
	Adaptor* adaptor=new MP4Decoder(esinfo, true);
	
	// ContainerDemux::demultiplexAndUpdateIO(vts, NULL, adaptor, false, false, true);	
	// write YUV to disk
	//ContainerDemux::adaptESInfo(esinfo, "test.yuv", adaptor, 0u, 1000u, true, false);
	// write YUV to screen
	printf("dvdleech: Visualizing %i video frames...\n",PLAYFRAMES);
	ContainerDemux::adaptESInfo(esinfo, NULL, NULL, 0u, PLAYFRAMES, true, true);
	
	/* END */
	printf("\ndvdleech: DVDLEECH FINISHED -> exit program!\n");

	return 0;
}
