 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	muvicut.cpp                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include <assert.h>
#include "DataChannel.hpp"
#include "DataSink.hpp"
#include "io/MPGStreamIO.hpp"
#include "ContainerInfo.hpp"
#include "ESInfo.hpp"
#include "adaptors/CuttingAdaptor.hpp"

#ifdef WIN32
#include <crtdbg.h>
#endif
#include "vitooki.hpp"
#include "container/ContainerFile.hpp"
#include "io/ffMP4IO.hpp"


void printHelp(const char* name)
{
	printf("%s <.mp4> <out-prefix> <start-sec> <end-sec>\r\n",name);
	printf("cuts all elementary streams within the given bounds in seconds (eg. 3.5 - 13.75)\n");
	printf("to make decoding work flawlessly, it is started at the first possible I-frame, so the\n");
  printf("start point will be a little after the given start-sec!\n");
	printf("if more than one elementary stream is in the .mp4 container, all will be extracted\n");
  printf("             and numbered according to their stream-IDs\n");
  printf("note, that the output filenames will look like:  out-prefix-01.cmp .. out-prefix-04.mp3\n");
}

int main(int argc, char* argv[]) 
{

#ifdef WIN32
	int tmpFlag = _CrtSetDbgFlag( _CRTDBG_REPORT_FLAG );
	tmpFlag |= _CRTDBG_LEAK_CHECK_DF;
	_CrtSetDbgFlag( tmpFlag );
#endif

	if(argc < 5) {
		printHelp(argv[0]);
		exit(1);
	}

	vitooki_init_codecs();

	ContainerInfo* mp4=ContainerFile::loadContainerInfo(argv[1]);
	if(!mp4) {
		printf("Inputfile %s not found\r\n",argv[1]);
		exit(1);
	}

	//FIXME: video is cut from next video-I-frame, but audio is cut exactly --> no lip sync!

	list < ESInfo * >* streams = mp4->getESList();

	list < ESInfo * >::const_iterator li = streams->begin();
	while (li != streams->end()) {
			char outfile[255];
			double startSec = atof(argv[3]);
			double endSec   = atof(argv[4]);

			ESInfo* es=(*li);
			//FIXME: quick hack to determine file extension, audio is always .mp3
			if (es->isVisualStream())
				sprintf(outfile,"%s-%09llu.cmp",argv[2],es->getStreamId());
			else if (es->isAudioStream())
				sprintf(outfile,"%s-%09llu.mp3",argv[2],es->getStreamId());
			else
				sprintf(outfile,"%s-%09llu.unknown",argv[2],es->getStreamId());

			IO* input=NULL;
#ifdef ENABLE_FFMPEG
			input=new ffMP4IO(es, argv[1], false);
#else
			dprintf_err("muvicut  compiled without ffmpeg!\n");
			::exit(1);
#endif
			assert(es && input);

			MPGStreamIO* output=new MPGStreamIO(es, outfile, true);
			CuttingAdaptor *cutter = new CuttingAdaptor(es,startSec,endSec);
			
			DataChannel* dc=new DataChannel(input, es, cutter);
			
			dc->insert(new DataSink(0, output, "127.0.0.1", 0));
			dc->play(0, 0.0);
			dc->start();
			dc->wait();
			delete dc;

		++li;
	}

	delete mp4;
	return 0;
};
