 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	calcpsnr.cpp                                                  *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include <stdio.h>
#include "vitooki.hpp"
#include "io/YUVStreamIO.hpp"
#include "io/DevNull.hpp"
#include "container/ContainerFile.hpp"

#ifndef ISOMP4
#include "io/ffMP4IO.hpp"
#else
#include "io/isoMP4IO.hpp"
#endif

#include "VideoESInfo.hpp"
#include "ContainerInfo.hpp"
#include "DataChannel.hpp"
#include "DataSink.hpp"
#include "adaptors/PSNR.hpp"
#include "adaptors/MP4Decoder.hpp"
#include "adaptors/StatisticsUpdater.hpp"
#include "adaptors/FrameCounter.hpp"
#include "adaptors/YUVScalingAdaptor.hpp"
#include "adaptors/TemporalAdaptor.hpp"
#include "Statistics.hpp"
#include "VideoAdaptorChain.hpp"

void printHelp(const char* executable)
{
	printf("%s <compressedvideo> <original.yuv> <outputfile> [width  height]\n",executable);
	printf("if width and height is given, the .mp4 is decoded & scaled (eg. from SD to 352x288 or vice versa)!\n");
}	

int main(int argc, char* argv[])
{
	int new_width = -1;
	int new_height = -1;
	YUVScalingAdaptor *ysr;

	if(argc < 4) {
		printHelp(argv[0]);
		::exit(0);
	}
	vitooki_init_codecs();
	FILE* fp=fopen(argv[2],"rb");
	if(!fp) {
		printf("YUV file %s not found\n",argv[2]);
	}
	else
		fclose(fp);
	
	FILE* outFile=fopen(argv[3],"wb");
	if(!outFile) {
		printf("Failed to create output file %s\n",argv[3]);
		::exit(1);
	}

	ContainerInfo* mp4=ContainerFile::loadContainerInfo(argv[1]);
	if(!mp4) {
		printf("Inputfile %s not found\r\n",argv[1]);
		::exit(1);
	}
	VideoESInfo* es=mp4->getFirstVisualES();
	if(!es) {
		printf("No visual stream found in %s\n",argv[1]);
			::exit(1);
	}

	if (argc>4) {
		new_width=atoi(argv[4]);
	  new_height=atoi(argv[5]);
		ysr = new YUVScalingAdaptor(es, new_width, new_height); //also changes ESInfo!!!
	}


	// create YUV reader
	YUVStreamIO* input=new YUVStreamIO(es,argv[2],false);
	IO* chainInput=NULL;
#ifdef ENABLE_FFMPEG
			chainInput=new ffMP4IO(es, argv[1], false);
#else
			dprintf_err("muvicut  compiled without ffmpeg!\n");
			::exit(1);
#endif
	IO* output=new DevNull();
	VideoAdaptorChain* vac=new VideoAdaptorChain();


	//add this to drop every B-frame out of the stream
	//	TemporalAdaptor *bdrop = new TemporalAdaptor(es);
	//vac->addAdaptor(bdrop);

	MP4Decoder *mp4d = new MP4Decoder(es,false, UncompressedVideoFrame::ColorSpaceI420);
	vac->addAdaptor(mp4d);

	if (new_width != -1) {
		vac->addAdaptor(ysr); //had to be declared at top, since ESInfo was re-written for input-YUVStreamIO!
	}

	Statistics* stat=new Statistics(es,true);
	FrameCounter* fc=new FrameCounter(es,stat);
	vac->addAdaptor(fc);

	StatisticsUpdater* statUpd=new StatisticsUpdater(es,stat);
	vac->addAdaptor(statUpd);

	PSNR *psnr=new PSNR(es,input,stat,outFile);
	vac->addAdaptor(psnr);

	DataChannel* dc=new DataChannel(chainInput,es, vac);
	dc->insert(new DataSink(0, output, "127.0.0.1", 0));
	dc->play(0, 0.0);
	dc->start();
	dc->wait();
	delete dc;
	delete mp4;
	delete stat;
	return 0;
};

	
