 /***********************************************************************
 *                                                                      *
 * QtPlayer - a simple video player using the ViTooKi library           *
 *                                                                      *
 * title:	main.cpp                                                *
 *                                                                      *
 * $Id: main.cpp,v 1.4 2005/01/20 09:47:51 mkropfbe Exp $
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <qapplication.h>
#include <qfont.h>
#include <qstring.h>
#include <qtextcodec.h>
#include <qtranslator.h>

#include "QtPlayer.hpp"

void printUsage(char *progname)
{
  fprintf(stderr, "Usage: %s [-h] [--gray] <mp4file>\n"
	  "Load first visual stream of <mp4file> as video.\n"
	  "Demuxed MP4 elementary streams will be stored in the current directory.\n"
	  "--gray      render to gray-scale color space\n",
	  progname);
}

int main(int argc, char *argv[])
{
  char *progname = argv[0];
  bool arg_invalid = false;
  char *video_file = NULL;
  const char *curr_dir = ".";
  bool opt_gray = false;
  QApplication a(argc, argv);

  /* parse command-line arguments */
  while (!arg_invalid && --argc > 0) {
    argv++;
    if (strcmp(*argv, "-h") == 0) {
      printUsage(progname);
      return 0;
    } else if (strcmp(*argv, "--gray") == 0) {
      opt_gray = true;
    } else if ((*argv)[0] != '-' && !video_file) {
      video_file = *argv;
    } else {
      arg_invalid = true;
      break;
    }
  }
  if (arg_invalid || !video_file) {
    printUsage(progname);
    return 1;
  }

//  a.setFont(QFont("helvetica", 12));
  QtPlayer *player=new QtPlayer();
  a.setMainWidget(player);

  if (!player->loadVideo(video_file, curr_dir,
			 opt_gray ? UncompressedVideoFrame::ColorSpaceGRAY8
			 : UncompressedVideoFrame::ColorSpaceRGB24)) {
    fprintf(stderr, "Loading video %s FAILED!\n", video_file);
    delete player;
    return 1;
  }
  player->playerPlay();

  player->show();
  return a.exec();
}


