 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:    SimplePlayer                                               *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *  $Id: main.cpp,v 1.60 2006/11/09 15:14:50 mkropfbe Exp $
 *                                                                      *
 ***********************************************************************/


#ifndef WINCE


#include "vitooki.hpp"
#include "global.hpp"
#include "simpleplayer.hpp"

#ifdef BUILD_ARCH_ARM
#include <sys/time.h> 
#include <sys/resource.h> 
#include "unistd.h"
#include "errno.h"
#endif

void printUsage(const char *progName) {
	printf("\n%s [%s] [%s] [%s] [%s] [%s YUVFILE] <filename>\n\n",progName,
				 NOAUDIO, WITHPHONES, ROTATE, FULLSCREEN, SPLASHSCREEN);
	printf("  %s assumes YUVFILE to be same-sized as the real video!\n", SPLASHSCREEN);
	printf("  NOTE:  %s, %s   are only availabe on ipaq ARM devices!\n\n", WITHPHONES, ROTATE);
	exit(1);
}


int main(int argc, char *argv[]){

	bool enableVideo4Linux = false;
	bool enableHeadphones = false;
	bool enableAudio = true;
	bool enableVideo = true;
	bool enableRotate = false;
	bool enableFullscreen = false;
	char *filename = NULL;
	char *splashscreen_filename = NULL;
	M3SMetaInfo *m3s = NULL;
	int PARAM_SIZE = 1024;

	Simpleplayer *sp = NULL;
	
	if(argc<2) {
		printUsage(argv[0]);
	}

	vitooki_init_all();


	int i = 1;

	while (i < argc) {
		// video for linux support
		if (strcmp(argv[i], VIDEO4LINUX) == 0) {
			dprintf_full("taking Video 4 Linux as source\n");
			enableVideo4Linux = true;
		}	
		// only on iPAQ: enable headphones
		else if (strcmp(argv[i], WITHPHONES) == 0) {
			dprintf_full("enabling WITHPHONES\n");
			enableHeadphones = true;
		}
		else if (strcmp(argv[i], NOAUDIO) == 0) {
			dprintf_full("disabling AUDIO\n");
			enableAudio = false;
		}
		else if (strcmp(argv[i], NOVIDEO) == 0) {
			dprintf_full("disabling VIDEO\n");
			enableVideo = false;
		}
		else if (strcmp(argv[i], ROTATE) == 0) {
			dprintf_full("enabling ROTATE\n");
			enableRotate = true;
		}
		else if (strcmp(argv[i], FULLSCREEN) == 0) {
			dprintf_full("enabling FULLSCREEN\n");
			enableFullscreen = true;
		}
		else if (strcmp(argv[i], SPLASHSCREEN) == 0) {
			splashscreen_filename = argv[i+1];
			i++; //skip over to next param....
			dprintf_full("enabling SPLASHSCREEN %s\n",splashscreen_filename);
		}
		else if (strstr(argv[i], ".m3s") != NULL || strstr(argv[i], ".M3S") != NULL) {
			dprintf_full("\nfound m3s file");
			m3s = new M3SMetaInfo();
			if (strstr(argv[i], "http://") == NULL) {
				if (!m3s->loadFromFile(argv[i])) {
					dprintf_err("\nmain(): cannot open %s\n", argv[i]);
					return -1;
				}
			} else {
				if (!m3s->loadFromURL(argv[i])) {
					dprintf_err("\nmain(): cannot load url!");
					return -1;
				}
			}


			int k = strlen(argv[i]);
			while (--k >= 0) {
#ifndef WIN32					
				if (argv[i][k] == '/') break;
#else
				if (argv[i][k] == '\\') break;
#endif
			}

			if (strstr(argv[i], "http://") != NULL) {

				char *slashPos = strstr(argv[i] + strlen("http://") + 1, "/");

				if (slashPos != NULL) {
					char *lastSlashPos = slashPos;
					while ((slashPos = strstr(slashPos + 1, "/")) != NULL) lastSlashPos = slashPos;

					if (strstr(m3s->getVideoFilepath(), "http://") == NULL) {
						m3s->appendVideoFilepath(PARAM_SIZE, argv[i], (lastSlashPos+1)-argv[i]);
					}
					
					if (strstr(m3s->getAudioFilepath(), "http://") == NULL) {
						m3s->appendAudioFilepath(PARAM_SIZE, argv[i], (lastSlashPos+1)-argv[i]);
					}
				}
			}
			else if (k > 0) {
				
				if (m3s->getVideoFilepath()[0] != '\\' && strstr(m3s->getVideoFilepath(), "http://") == NULL) {
					m3s->appendVideoFilepath(PARAM_SIZE, argv[i], k+1);
				}
				
				if (m3s->getAudioFilepath()[0] != '\\' && strstr(m3s->getAudioFilepath(), "http://") == NULL) {
					m3s->appendAudioFilepath(PARAM_SIZE, argv[i], k+1);
				}
			}

		}
		else {
			if (filename == NULL) {
				filename = argv[i];
				dprintf_full("used file: %s\n", filename);
			} else {
				dprintf_err("found second file: \"%s\"\nyou can only specify one file!", argv[i]);
				exit(0);
			}
		}
		i++;
	}

#ifdef BUILD_ARCH_ARM
	int ret;
	int set=-20;
	if ((ret=nice(set)) == set) {
	//  if (setpriority(PRIO_PROCESS,0,set) == 0) {
		dprintf_err("switching to nice level %i\n",set);
	} else {
		dprintf_err("couldnt nice level, maybe you are not root? (errno %i, EPERM: %i ret: %i\n",errno,EPERM,ret);
	}
#endif


	sp = new Simpleplayer(filename, enableVideo4Linux, enableAudio, enableVideo, enableHeadphones, enableFullscreen,
												false, true, true, false, enableRotate, 0, 0, m3s, splashscreen_filename);
	sp->startPlayer();

	return 0;
}


#endif
