/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: simpleplayer.cpp                                              *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/


#include "simpleplayer.hpp"
#include "adaptors/YUVinYUVoverlay.hpp"
#include "io/MKIO.hpp"
#include "io/VITMuxDemuxIO.hpp"
#include "io/OGGMuxDemuxIO.hpp"

Simpleplayer::Simpleplayer(char *filename,
													 bool enableVideo4Linux,
													 bool enableAudio,
													 bool enableVideo,
													 bool enableHeadphones,
													 bool enableFullscreen,
													 bool pollEvents,
													 bool labelResizable,
													 bool fixAspectRatio, 
													 bool allowUpscaleX,
													 bool enableRotate,
													 int width,
													 int height,
													 M3SMetaInfo *m3s,
													 char *splashscreen_filename)
{
	this->filename = filename;
	this->enableVideo4Linux  = enableVideo4Linux;
	this->enableAudio = enableAudio;
	this->enableVideo = enableVideo;
	this->enableHeadphones = enableHeadphones;
	this->enableFullscreen = enableFullscreen;
	this->pollEvents = pollEvents, 
	this->labelResizable = labelResizable, 
	this->fixAspectRatio = fixAspectRatio, 
	this->allowUpscaleX = allowUpscaleX, 
	this->enableRotate = enableRotate;
	this->splashscreen_filename = splashscreen_filename;
	

	this->width = width;
	this->height = height;

	this->quitting = false;

	this->m3s = m3s;

	this->vdc = NULL;
	this->adc = NULL;


	this->ref = NULL;
	this->gt = NULL;
	this->esync = NULL;
	this->vio = NULL;
	this->aio = NULL;
}

Simpleplayer::~Simpleplayer()
{
}


unsigned int Simpleplayer::getFilesize(char *filename) {
	unsigned int len = 0;

	if (strstr(filename, "http://") != NULL) {
		len = 0;
		dprintf_err("\nSimpleplayer::getFilesize(%s) cannot determine file size for an url!\n",filename);
	} else {
		FILE *file = fopen(filename, "rb");
		if (file) {
			fseek(file, 0, SEEK_END);
			len = ftell(file);
			fclose(file);
			dprintf_full("\nSimpleplayer::getFilesize(%s) = %d\n", filename, len);
		} else {
			dprintf_err("\nSimpleplayer::getFilesize(%s) cannot open!\n", filename);
		}
	}
	return len;
}



int Simpleplayer::startPlayer() {
	SDL_Event event;
	AdaptorChain *vidChain = NULL, *audChain = NULL;
	Adaptor *videoDecoder = NULL, *audioDecoder = NULL;
	Adaptor *sharedAdapt = NULL, *sharedAdapt2 = NULL;
	DataSink *vds = NULL, *ads = NULL;
	ContainerInfo *input = NULL;
	
	char *VIDEOFILENAME, *AUDIOFILENAME;
	int FPS, VIDKBPS, AUDIOFRAMESIZE;
	int AVGBANDWIDTH, BITSPERSAMPLE, NUMCHANNELS, SAMPLERATE;
	int WIDTH, HEIGHT, LENGTH;

	bool useM3Sfile = false;
	if (m3s != NULL) useM3Sfile = true;
//#ifdef WINCE
//	useM3Sfile = true;
//#endif	


	//open the container file (eg. mp4) and extract all ElemStream info
	if (useM3Sfile == false) {		
//#ifndef WINCE
		dprintf_err("main: opening %s\n", filename);
		if(!enableVideo4Linux)
			input = ContainerFile::loadContainerInfo(filename);
#ifndef WINCE
		else { //Video4Linux
			dprintf("unsing video4linux device as input...\n");
			input = Video4LinuxContainerFile::loadContainerInfo();
		}
#endif
		if(!input) {
			dprintf_err("main: Cannot open inputfile\n");
			return -1;
		}
//#endif
	} else {
		LENGTH = m3s->getLength();
		
		VIDEOFILENAME = m3s->getVideoFilepath();
		FPS = m3s->getFPS();
		VIDKBPS = m3s->getVideoKBPS() * 1000;
		WIDTH = m3s->getWidth();
		HEIGHT = m3s->getHeight();
		
		AUDIOFILENAME = m3s->getAudioFilepath();
		AUDIOFRAMESIZE = 1250; 
		AVGBANDWIDTH = m3s->getAudioKBPS() * 1000;
		BITSPERSAMPLE = m3s->getAudioKBPS();
		NUMCHANNELS = m3s->getNumAudioChannels();
		SAMPLERATE = m3s->getSampleRate();
	}
	
	ref = new ReferenceCounter(); //used for video+audio stream
	gt = new GlobalTimer(); //used for video+audio stream


	VideoESInfo *ves = NULL;
	IO *videoinput = NULL; 
	
	if (useM3Sfile == false) {
//#ifndef WINCE
		ves=input->getFirstVisualES();
//#endif
	} else if (m3s->isVideoEnabled() && enableVideo) {
		//hacky code for winCE elemStream testing....
		dprintf_full("main: opening %s\n", VIDEOFILENAME);
		ves = new VideoESInfo(1, 
			CODEC_TYPE_VIDEO,
			NULL, 
			90000/FPS,
			NULL,
			0,
			0,
			VIDKBPS,
			0,
			90000,
			90000 * LENGTH, 
			0, //size
			WIDTH,
			HEIGHT,
			true,
			true,
			CODEC_ID_MPEG4,
			0.0,
			(float)WIDTH/(float)HEIGHT,
			true,
			50,
			0,
			0);
	   
		if (strstr(VIDEOFILENAME, "http://") != NULL) {
			//FIXME: http only works with mpeg-streams!
		  videoinput = new BufferedHttpMPGStreamReader(ves, VIDEOFILENAME, false, false, 
							       BufferedHttpMPGStreamReader::getDefaultBufferSize());
		} else {
			videoinput = new BufferedMPGStreamReader(ves, VIDEOFILENAME, false, false);
			if (!((BufferedMPGStreamReader*)videoinput)->testFileopen()) {
				dprintf_err_fatal("main: error opening video file: %s\n", VIDEOFILENAME);
				return -1;
			}
		}
	}


	if (ves && enableVideo) { //if there was a video stream, get it going
		vidChain=new VideoAdaptorChain();
		UncompressedVideoFrame::ColorSpace cs;
#ifdef WINCE
		cs = UncompressedVideoFrame::ColorSpaceI420;
#else
#ifdef BUILD_ARCH_ARM
		cs = UncompressedVideoFrame::ColorSpaceRGB565;
#else
		if(enableVideo4Linux)
			cs = UncompressedVideoFrame::ColorSpaceI420;
		else
			//			cs = UncompressedVideoFrame::ColorSpaceRGB565;
			cs = UncompressedVideoFrame::ColorSpaceYV12;
		//			cs = UncompressedVideoFrame::ColorSpaceRGB32;
#endif
#endif
		dprintf_full("simpleplayer::startPlayer(): checking CodecID()\n");
		if(ves->getCodecID() == CODEC_ID_THEORA) {
		  dprintf_err("SimplePlayer: detected THEORA, adding decoder\n");
#ifdef ENABLE_OGG
		  videoDecoder = new TheoraDecoder(ves);
		  vidChain->addAdaptor(videoDecoder);
#endif
		}
		else if (ves->getCodecID() != CODEC_ID_RAWVIDEO) {
		  dprintf_err("simpleplayer::startPlayer(): using ffmpeg/xvid MP4 decoder for all other video formats!!!\n");
#ifndef WINCE  //used on all flavors of linux (PC/ipaq) or winPC
			videoDecoder = new MP4Decoder(ves,false, cs, MP4Decoder::FFMPEG);
#else 
			//on winCE, there is no ffmpeg lib, so fallback to xvid
			videoDecoder = new MP4Decoder(ves,false,cs, MP4Decoder::XVID);
#endif
			vidChain->addAdaptor(videoDecoder);
		}
		
		sharedAdapt = new SharedAdaptor(ref, (Adaptor*&)gt);
		vidChain->addAdaptor(sharedAdapt);
		esync = new ESSynchronizer(ves, gt);
		vidChain->addAdaptor(esync);

		if (useM3Sfile == false) {
			vdc = new DataChannel(IOCreator::createInput(ves, false), ves, vidChain);
		} else {
			vdc = new DataChannel(videoinput, ves, vidChain);
		}


		vio = new SDLRenderer(ves,cs,width,height,enableFullscreen,
				pollEvents,labelResizable,fixAspectRatio,allowUpscaleX,enableRotate);
		//		vio = new X11Renderer(ves,cs,width,height);

		/**************************
		 * big _FEATURE_ for hourglass splash screen, before the real video is played.... */
		if (splashscreen_filename) {
			dprintf_full("SimplePlayer: setting hourglass splashscreen %s\n",splashscreen_filename);
			ContainerInfo* hg_ci=new ContainerInfo(NULL, splashscreen_filename , ContainerInfo::MP4);
			VideoESInfo *hg_ves = new VideoESInfo(1,0,hg_ci,3600,NULL,0,0,
																						0,0,
																						90000,3600,0,ves->getWidth(),ves->getHeight());
			IO *hg_io = IOCreator::createInput((ESInfo*)hg_ves);
			
			vio->open(); //init the SDL display
			hg_io->open();
			UncompressedVideoFrame* hg_frm = (UncompressedVideoFrame*)hg_io->getFrame();
			
			//adapt to correct colorspace!
			UncompressedVideoFrame* conv_frm = hg_frm->convertToColorSpace(cs);
			if (conv_frm) {
				delete hg_frm;
				vio->writeFrame(conv_frm, hg_ves);
				delete conv_frm;
			} else { //assume bypass from yuv to yuv
				vio->writeFrame(hg_frm, hg_ves);
				delete hg_frm;
			}

			delete hg_io;
			delete hg_ci;
			delete hg_ves;
		}
		/************   end of big _FEATURE_ */

		vds = new DataSink(VID_ID, vio, "127.0.0.1");
		vdc->insert(vds);  //insert into paused list
		vdc->play(VID_ID, 0.0); //open the io stream (if not yet open) and move first entry of paused list to active list
	} //has video
	


	AudioESInfo *aes = NULL;
	MPGStreamIO *audioinput = NULL;
	
	if (useM3Sfile == false) {
//#ifndef WINCE
		aes=input->getFirstAudioES();
//#endif
	} else if (m3s->isAudioEnabled() && enableAudio) {
		dprintf_full("main: opening %s\n", AUDIOFILENAME);
		aes = new AudioESInfo(2, //streamid
				CODEC_TYPE_AUDIO, //streamtype
				NULL, //vob
				0, //vop-timeincrement (WILL BE SET LATER)
				NULL, //encodeddecconf
				0, //objtype
				0, //bufsize
				AVGBANDWIDTH, //avgBW
				0, //maxBW
				90000, //timeScale
				90000 * LENGTH, //duration
				0, //size
				SAMPLERATE, //samplerate
				NUMCHANNELS, //numaudiochannels
				true, //complete
				CODEC_ID_MP3, //codec_id
				0.0, //quality
				BITSPERSAMPLE); //bitsPerSample
		
		if (strstr(VIDEOFILENAME, "http://") != NULL) {
			audioinput = new BufferedHttpMPGStreamReader(aes, AUDIOFILENAME, false, false, BufferedHttpMPGStreamReader::getDefaultBufferSize(), AUDIOFRAMESIZE);
			((BufferedHttpMPGStreamReader*)audioinput)->open();
			aes->setVOPTimeIncrement(90000*LENGTH/(((BufferedHttpMPGStreamReader*)audioinput)->getContentLength()/AUDIOFRAMESIZE));
		} else {
			audioinput = new BufferedMPGStreamReader(aes, AUDIOFILENAME, false, false, BufferedMPGStreamReader::getDefaultBufferSize(), AUDIOFRAMESIZE);
			if (!((BufferedMPGStreamReader*)audioinput)->testFileopen()) {
				dprintf_err_fatal("main: error opening audio file: %s\n", AUDIOFILENAME);
				if(vdc) {
					vdc->teardown(VID_ID, true);
				}
				return -1;
			}
			aes->setVOPTimeIncrement(90000*LENGTH/(getFilesize(AUDIOFILENAME)/AUDIOFRAMESIZE));
		}
	}

	
	if (aes && enableAudio) { //if there was an audio stream, get it going
		audChain=new AudioAdaptorChain();
		switch (aes->getCodecID()) {
#ifdef WINCE
			//FIXME: why not include avcodec.h????
	  #define CODEC_ID_MP3 86017
#endif
		case CODEC_ID_MP3:	
		case CODEC_ID_AAC:
		case CODEC_ID_MPEG4AAC:
			dprintf_err("SimplePlayer: detected MP3 audio, adding decoder\n");
			//			audChain->addAdaptor(new MP4audioDecoder(aes, false));
			audioDecoder = new MadMP3Decoder(aes);
			audChain->addAdaptor(audioDecoder);
			break;
		case CODEC_ID_MP2:
			dprintf_err("SimplePlayer: detected MP2 audio, adding decoder\n");
			//audChain->addAdaptor(new MP4audioDecoder(aes, false));
			audioDecoder = new MadMP3Decoder(aes);
			audChain->addAdaptor(audioDecoder);
			break;
		case CODEC_ID_PCM_S16LE:
		case CODEC_ID_PCM_S16BE:
		case CODEC_ID_PCM_U16LE:
		case CODEC_ID_PCM_U16BE:
			dprintf_err("SimplePlayer: detected raw PCM audio, passing directly\n");
			//use this for testing the encoding/decoding facility:
			//audChain->addAdaptor(new MP3Encoder(aes,128*1000));
			//audChain->addAdaptor(new MadMP3Decoder(aes));
			break;
		case CODEC_ID_VORBIS:
		  dprintf_err("SimplePlayer: detected VORBIS, adding decoder\n");
#ifdef ENABLE_OGG
		  audioDecoder = new VorbisDecoder(aes);
		  audChain->addAdaptor(audioDecoder);
#endif
		  break;

		default:
			dprintf_err("SimplePlayer: unknown audio stream format codec ID %i!\n",aes->getCodecID());
			adc = NULL;
			aes = NULL;
		}

		if (aes != NULL) {
			sharedAdapt2 = new SharedAdaptor(ref, (Adaptor*&)gt);
			audChain->addAdaptor(sharedAdapt2);
			
			if (useM3Sfile == false) {
				adc = new DataChannel(IOCreator::createInput(aes, false), aes, audChain);
			} else {
				adc = new DataChannel(audioinput, aes, audChain);
			}

	#ifdef WIN32
			//aio =  new DevNull(); //no audible output
			aio =  new SDLaudioIO(aes, esync); //esync may be NULL, if there is no video!
	#else
			aio = new DSPaudioIO(aes, esync, enableHeadphones); // only on iPAQ: enable headphones
	#endif

			ads = new DataSink(AUD_ID, aio, "127.0.0.1");
			adc->insert(ads); //insert into paused list
			adc->play(AUD_ID, 0.0); //open the io stream (if not yet open) and move first entry of paused list to active list
		}
	}//has audio


//#ifdef WINCE
	if (esync != NULL) esync->setPaused(false);
//#endif

	if (vdc)
		vdc->start();

	if (adc)
		adc->start();

	

	
	while ( !quitting  ) {
		
		if (SDL_PollEvent( &event ) && event.type != SDL_MOUSEMOTION) {
			SDL_KeyboardEvent *key = &event.key;
			char *buf = NULL;
			char c = 0;

#ifndef WINCE
			if (key != NULL)
				buf = SDL_GetKeyName( key->keysym.sym );
			if (buf[1]=='\0') //only single character (a-zA-Z...)
				c = toupper(buf[0]);
#endif

			//if (event != NULL) {
				switch( event.type ){
				case SDL_KEYUP:
					if ( ((key->keysym.mod & KMOD_CTRL)  && (c == 'C')) ||
							 (buf != NULL && strstr(buf, "escape") != NULL)) {
						dprintf_err("SimplePlayer: SDL_PollEvent   STREAMERR because of CTRL-C/ESC!...so teardown!\n");
						quitting = true;
						if(vdc) {
							vdc->teardown(VID_ID, true);
						}
						if (adc) {
							adc->teardown(AUD_ID, true);
						}
					}
					break;
					
				case SDL_KEYDOWN: //ignore
					break;
					
				case SDL_MOUSEBUTTONDOWN: //click in window
				case SDL_QUIT: //window close
					dprintf_err("SimplePlayer: SDL_PollEvent   STREAMERR because of window close!...so teardown!\n");
					quitting = true;
					if(vdc) {
						vdc->teardown(VID_ID, true);
					}
					if (adc) {
						adc->teardown(AUD_ID, true);
					}
					break;
					
				default:
					break;
				}
			//}
			
		}
		
		if (!quitting) {
			//catch if one stream fails, then finish the other, too


			if (vdc && (!vdc->running() || (vio->getState() == IO::STREAMERR) ) ) { //video stops
				dprintf_full("SimplePlayer: video stopped with state %d!\n", vio->getState());
				if (adc) {
					/* disabled!!! , so audio takes the lead! this is NEEDED for multimundus-audioguides with only start-image!
					adc->teardown(AUD_ID, true);
					break;
					*/
				} else { //there is no audio anyway...
					break;
				}
			}
			
			
			if (adc && (!adc->running() || (aio->getState() == IO::STREAMERR)) ) { //audio stops
				dprintf_full("SimplePlayer: audio stopped with state %d!\n", aio->getState());
				if (vdc) 
					vdc->teardown(VID_ID, true);
				break;
			}


			msleep(15);
		}
				
	}

	dprintf_err("SimplePlayer: after big while loop...\n");

	if (adc) {
		if (adc->running()) {
			dprintf_full("SimplePlayer: waiting for adc to exit...\n");
#ifdef WINCE
			adc->exit(); 
#else
			adc->wait();
#endif
			dprintf_full("done\n");
		}
		dprintf_full("SimplePlayer: deleting adc!\n");
		delete adc;
		adc = NULL;
	}
	
	if (vdc) {
		if (vdc->running()) {
			dprintf_full("SimplePlayer: waiting for adc to exit...\n");
#ifdef WINCE
			vdc->exit();
#else
			vdc->wait();
#endif
			dprintf_full("SimplePlayer done\n");
		}
		dprintf_full("SimplePlayer: deleting vdc!\n");
		delete vdc;
		vdc = NULL;
	}	

	dprintf_full("SimplePlayer: deleting aes and ves..\n");

	if (aes != NULL) delete aes;
	if (ves != NULL) delete ves;

	return 0;
}
