/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	simpleplayer.hpp                                            *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef SIMPLEPLAYER_HPP
#define SIMPLEPLAYER_HPP


#ifndef WINCE
#include <signal.h>
#endif
#include "ContainerInfo.hpp"


#ifndef ISOMP4
#include "avformat.h"
#else
#include "avcodec.h"
#endif

#include "container/ContainerFile.hpp"
#ifndef WINCE
#include "container/Video4LinuxContainerFile.hpp"
#endif
#include "DataChannel.hpp"
#include "DataSink.hpp"
#include "VideoAdaptorChain.hpp"
#include "VideoESInfo.hpp"
#include "AudioAdaptorChain.hpp"
#include "AudioESInfo.hpp"
#include "adaptors/MP4Decoder.hpp"
#include "adaptors/MP3Encoder.hpp"
#include "adaptors/MP4audioDecoder.hpp"
#include "adaptors/MadMP3Decoder.hpp"
#include "adaptors/ESSynchronizer.hpp"
#include "adaptors/TheoraDecoder.hpp"
#include "adaptors/VorbisDecoder.hpp"
#include "io/IOCreator.hpp"
#include "io/SDLaudioIO.hpp"
#include "io/DSPaudioIO.hpp"
#include "io/DevNull.hpp"
#include "io/SDLRenderer.hpp"
#include "io/BufferedMPGStreamReader.hpp"
#include "io/BufferedHttpMPGStreamReader.hpp"
#include "ReferenceCounter.hpp"
#include "adaptors/SharedAdaptor.hpp"
#include "adaptors/ESSynchronizer.hpp"
#include "adaptors/GlobalTimer.hpp"
#include "io/YUVStreamIO.hpp"
#include "io/OGGMuxDemuxIO.hpp"
#include "adaptors/DataDump.hpp"
#include "metadata/M3SMetaInfo.hpp"


#define VID_ID 55
#define AUD_ID 99
#define VIDEO4LINUX "--v4l"
#define WITHPHONES "--enableHeadphones"
#define NOAUDIO "--no-audio"
#define NOVIDEO "--no-video"
#define ROTATE "--rotate"
#define FULLSCREEN "--fullscreen"
#define SPLASHSCREEN "--splashscreen"


class Simpleplayer
{
public:
	Simpleplayer (char *filename = NULL, 
								bool enableVideo4Linux = false,
								bool enableAudio = true, 
								bool enableVideo = true, 
								bool enableHeadphones = false, 
								bool enableFullscreen = false,
								bool pollEvents = false,  
								bool labelResizable = true,
								bool fixAspectRatio = true, 
								bool allowUpscaleX = false, 
								bool enableRotate = false,
								int width = 0,
								int height = 0, 
								M3SMetaInfo *m3s = NULL,
								char *splashscreen_file = NULL);

	~Simpleplayer();

	int startPlayer();

protected:
	DataChannel *vdc; //video
	DataChannel *adc; //audio
	ReferenceCounter *ref; //used for video+audio stream
	GlobalTimer *gt; //used for video+audio stream
	ESSynchronizer *esync;
	IO *vio;
	IO *aio;
	M3SMetaInfo *m3s;

	bool enableVideo4Linux;
	bool enableHeadphones;
	bool enableAudio;
	bool enableVideo;
	bool enableRotate;
	bool enableFullscreen;
	bool pollEvents;
	bool labelResizable;
	bool fixAspectRatio;
	bool allowUpscaleX;
	bool quitting;

	char *filename;
	char *splashscreen_filename;

	int width, height;
	
	
	unsigned int getFilesize(char *filename);
};


#endif
