 /***********************************************************************
 *                                                                      *
 * MuViNo - ViTooKi Video Annotation Tool                               *
 *                                                                      *
 * title: testMP7Time.cpp                                               *
 *                                                                      *
 * $Id: testMP7Time.cpp,v 1.1 2004/12/29 16:16:56 mtaschwer Exp $
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

#include <stdio.h>
#include <string.h>
#include <metadata/MP7Time.hpp>

bool testMP7Time()
{
  /* input time points */
  char *tpoints[] = { "2004-12-31T10:59:09:238679F1000000+02:00",
		      "T08:12F100",
		      "T13:26+01:00",
		      "-T02:14:59",
		      "1999-09-28",
		      "2001-01-01T00:09",
		      0 };
  /* expected output time points (MPEG-7) */
  char *tpoints_out_mp7[] = { "2004-12-31T10:59:09:238679F1000000+02:00",
			      "T08:00:00:12F100",
			      "T13:26:00+01:00",
			      "-T02:14:59",
			      "1999-09-28",
			      "2001-01-01T00:09:00",
			      0 };
  /* expected output time points (pretty) */
  char *tpoints_out_pretty[] = { "2004-12-31 10:59:09.23+02:00",
				 "08:00:00.12",
				 "13:26:00+01:00",
				 "-02:14:59",
				 "1999-09-28",
				 "2001-01-01 00:09:00",
				 0 };
  /* input durations */
  char *dur[] = { "P2DT18H59M09S475N1000F-01:00Z",
		  "-PT124H8S",
		  "P3650D",
		  "-P01D-02:00Z",
		  "PT4S12345N100000F",
		  "PT18H+02:00Z",
		  "PT014N100F",
		  0 };
  /* expected output durations (MPEG-7) */
  char *dur_out_mp7[] = { "P2DT18H59M9S475N1000F-01:00Z",
			  "-PT124H8S",
			  "P3650D",
			  "-P1D-02:00Z",
			  "PT4S12345N100000F",
			  "PT18H+02:00Z",
			  "PT14N100F",
			  0 };
  /* expected output durations (pretty) */
  char *dur_out_pretty[] = { "2 days 18:59:09.47-01:00",
			     "-124:00:08",
			     "3650 days",
			     "-1 days -02:00",
			     "00:00:04.12",
			     "18:00:00+02:00",
			     "00:00:00.14",
			     0 };
  /* time points and durations for endTimePoint tests
   * (date and time zone parts not supported)
   */
  char *tp_dur[] = { "T09:59:59:4F10", "PT1S5N100F",
		     "T24:00:01", "-PT2S",
		     "T00:00:03", "PT1D60S",
		     "T10:3F10", "-PT3N5F" };
  /* expected end time points (MPEG-7) */
  char *tp_end[] = { "T10:00:00:45F100",
		     "T23:59:59",
		     "T24:01:03",
		     "T09:59:59:7F10" };
  /* expected end time points (pretty) */
  char *tp_end_pretty[] = { "10:00:00.45",
			    "23:59:59",
			    "24:01:03",
			    "09:59:59.70" };
  /* expected toString() output for tp_dur[] */
  char *tp_dur_str[] = { "09:59:59.40 [00:00:01.05]",
			 "24:00:01 [-00:00:02]",
			 "00:00:03 [1 days 00:01:00]",
			 "10:00:00.30 [-00:00:00.60]" };

  printf("***** Testing MP7Time Class *****\n");
  MP7Time tm;
  const char *result;
  bool ok;
  bool status = true;
  printf("Testing timePoint methods:\n");
  int arrsize = sizeof(tpoints) / sizeof(char *) - 1;
  for (int i=0; i < arrsize; i++) {
    ok = tm.setTimePoint(tpoints[i]);
    status &= ok;
    printf("  setTimePoint       (%-50s)    %s\n", tpoints[i], ok ? "OK" : "FAILED");
    result = tm.timePointMP7();
    ok = (strcmp(result, tpoints_out_mp7[i]) == 0);
    printf("  timePointMP7() =    %-50s     %s\n", result, ok ? "OK" : "FAILED");
    status &= ok;
    result = tm.timePointPretty();
    ok = (strcmp(result, tpoints_out_pretty[i]) == 0);
    printf("  timePointPretty() = %-50s     %s\n", result, ok ? "OK" : "FAILED");
    status &= ok;
  }
  printf("Testing duration methods:\n");
  arrsize = sizeof(dur) / sizeof(char *) - 1;
  for (int i=0; i < arrsize; i++) {
    ok = tm.setDuration(dur[i]);
    status &= ok;
    printf("  setDuration        (%-50s)    %s\n", dur[i], ok ? "OK" : "FAILED");
    result = tm.durationMP7();
    ok = (strcmp(result, dur_out_mp7[i]) == 0);
    printf("  durationMP7() =     %-50s     %s\n", result, ok ? "OK" : "FAILED");
    status &= ok;
    result = tm.durationPretty();
    ok = (strcmp(result, dur_out_pretty[i]) == 0);
    printf("  durationPretty() =  %-50s     %s\n", result, ok ? "OK" : "FAILED");
    status &= ok;
  }
  printf("Testing endTimePoint and toString:\n");
  arrsize = sizeof(tp_dur) / sizeof(char *);
  for (int i=0; i < arrsize; i += 2) {
    ok = tm.setTimePoint(tp_dur[i]);
    status &= ok;
    printf("  setTimePoint       (%-50s)    %s\n", tp_dur[i], ok ? "OK" : "FAILED");
    ok = tm.setDuration(tp_dur[i+1]);
    status &= ok;
    printf("  setDuration        (%-50s)    %s\n", tp_dur[i+1], ok ? "OK" : "FAILED");
    result = tm.endTimePoint().timePointMP7();
    ok = (strcmp(result, tp_end[i/2]) == 0);
    printf("  endTimePoint =      %-50s     %s\n", result, ok ? "OK" : "FAILED");
    status &= ok;
    result = tm.endTimePointPretty();
    ok = (strcmp(result, tp_end_pretty[i/2]) == 0);
    printf("  endTimePointPretty= %-50s     %s\n", result, ok ? "OK" : "FAILED");
    status &= ok;
    result = tm.toString();
    ok = (strcmp(result, tp_dur_str[i/2]) == 0);
    printf("  toString() =        %-50s     %s\n", result, ok ? "OK" : "FAILED");
    status &= ok;
  }
  return status;
}
