#include "testSimpleRtp.hpp"
#include "io/SimpleRtp.hpp"
#include "io/MPGStreamIO.hpp"
#include "io/IOCreator.hpp"
#include "ContainerInfo.hpp"
#include "DataSink.hpp"
#include "net/RFC3016.hpp"
#include "DataChannel.hpp"
#include "container/ContainerDemux.hpp"
#include "container/ContainerFile.hpp"
#include "io/Rtp.hpp"
#include "global.hpp"


#define VIDEOFILE "alien_2b_IP16q_B28q_25fps_30I.mp4"
#define VIDEOFILE2 "coastguard_9_6.mp4"
#define RTSP_URL "rtsp://127.0.0.1/"VIDEOFILE

#ifdef WIN32
#define TESTFILE "..\\..\\media\\"VIDEOFILE
#define TESTFILE2 "..\\..\\..\\media\\"VIDEOFILE2
#else
#define TESTFILE "../../media/"VIDEOFILE
#define TESTFILE2 "../../../media/"VIDEOFILE2
#endif

void doSimpleRtpTest()
{
	int portForSender=portGen.getNextFreePortPair();
	int portForReader=portGen.getNextFreePortPair();
	PacketizationLayer * pl=new RFC3016();
	PacketizationLayer * pl2=new RFC3016();
	Url* tmp=new Url(RTSP_URL);
	ContainerInfo* videoSender=ContainerFile::loadContainerInfo(TESTFILE,tmp);
	
	if(!videoSender)
		videoSender=ContainerFile::loadContainerInfo(TESTFILE2,tmp);
	if(!videoSender){
		fprintf(stderr,"SIMPLERTPTEST ABORTED! FILE %s or %s NOT FOUND",TESTFILE,TESTFILE2);
		exit(1);
	}
	delete tmp;tmp=NULL;
	ContainerInfo* videoReader=videoSender->clone();
	ESInfo* esSend=(ESInfo*)videoSender->getFirstVisualES();
	ESInfo* esRead=(ESInfo*)videoReader->getFirstVisualES();

//	ContainerDemux::demultiplexAndUpdateIO(videoSender,".");

	IO* inputReader=new SimpleRtp(RTSP_URL,portForSender,portForReader,"127.0.0.1",
			esRead,pl,false);
	IO* outputReader=new MPGStreamIO( esRead,"streamreadfromrtp.cmp",true);
	IO* outputSender=new SimpleRtp(RTSP_URL,portForReader,portForSender,"127.0.0.1",esSend,pl2,true);
	outputSender->open();
	inputReader->open();
	IO* inputSender=IOCreator::createInput(esSend, true);

	DataSink* datasinkSender = new DataSink(0, outputSender, "127.0.0.1", 0, NULL);
	DataSink* datasinkReader = new DataSink(0, outputReader, "127.0.0.1", 0, NULL);
	DataChannel* dcSender = new DataChannel(inputSender, esSend);
	DataChannel* dcReader= new DataChannel(inputReader,esRead);

	dcSender->insert(datasinkSender);
	dcReader->insert(datasinkReader);
	dcSender->play(0, 0.0);
	dcReader->play(0, 3.0);
	dcReader->start();

	msleep(1000);

	dcSender->start();
	
	if(dcSender->running())
	{
		dcSender->wait();
	}
	videoSender->saveToCfgFile("streamsender.conf");
	delete dcSender;
	if(dcReader->running())
		dcReader->wait();
	dprintf_full("doSimpleRtpTest Saving .conf file for receiver\n");
	videoReader->saveToCfgFile("streamreadfromrtp.conf");
	
	delete dcReader;
	delete videoSender; delete videoReader;
};
