#include "vitooki.hpp"
#include "net/Url.hpp"
#include "net/RTSP.hpp"
#include "BitField.hpp"
#include "metadata/UserPreferences.hpp"
#include "RingBuffer.hpp"
#include "PointerRingBuffer.hpp"
#include "PointerHashTable.hpp"
#include "testSimpleRtp.hpp"

bool testMP7Time();		// defined in testMP7Time.cpp

static void testRingBuffer()
{
	PointerRingBuffer<int*> pIntRing(1024);
	RingBuffer<int> intRing(1024);
	int *t=new int[10];
	int i=0;
	for(i=0;i<10;i++) {
		t[i]=i*i;
		int* d=&t[i];
		pIntRing.put(d,i);
		intRing.put(t[i],i);
	}
	int* tmp;
	int tmp2;
	bool valid=false;
	for(i=9;i>=0;i--) {
		tmp=pIntRing.get(i);
		assert(tmp);
		tmp2=intRing.get(i,&valid);
		assert(valid);
		assert(*tmp==tmp2);
		printf("%i ",*tmp);
	}
	delete t;

}

void usage()
{
  printf("Usage: testlibrary TARGET ...\n"
	 "where TARGET is one of:\n"
	 "    SimpleRtp\n"
	 "    RingBuffer\n"
	 "    UserPrefs\n"
	 "    Url\n"
	 "    BitField\n"
	 "    RTSP\n"
	 "    MP7Time\n"
	 "    ALL              all of the above targets\n");
}

int main(int argc, char* argv[])
{
  enum targets { simpleRtp = 0, ringBuffer, userPrefs, url, bitField, rtsp,
		 mp7Time };
  char *targets_str[] = { "SimpleRtp", "RingBuffer", "UserPrefs", "Url",
			  "BitField", "RTSP", "MP7Time" };
  bool tests[7] = { false, };
  int numtests = sizeof(tests)/sizeof(bool);

  if (argc < 2) {
    usage();
    return 0;
  }
  /* parse command line */
  for (--argc, ++argv; argc > 0; --argc, ++argv) {
    if (strcmp(*argv, "ALL") == 0) {
      for (int i = numtests-1; i >= 0; i--)
	tests[i] = true;
      break;
    } else
      for (int i = numtests-1; i >= 0; i--) {
	if (strcmp(*argv, targets_str[i]) == 0)
	  tests[i] = true;
      }
  }
  /* perform tests */
  bool ok = true;
  vitooki_init_all();
  if (tests[simpleRtp])
    doSimpleRtpTest();
  if (tests[ringBuffer])
    testRingBuffer();
  if (tests[userPrefs])
    UserPreferences::doTest();
  if (tests[url])
    Url::doTest();
  if (tests[bitField])
    BitField::doTest();
  if (tests[rtsp])
    RTSP::doTest();
  if (tests[mp7Time])
    ok &= testMP7Time();
  printf("\nOverall test status is %s\n", ok ? "OK" : "FAILED");
  return ok ? 0 : 1;
}
