 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	YALT.cpp                                                      *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/


//#include "ESInfo.hpp"
#include "vitooki.hpp"
#include "metadata/TerminalCapabilities.hpp"
#include "net/PacketizationLayer.hpp"
#include "adaptors/GlobalTimer.hpp"
#include "net/RFC3016.hpp"
#include "io/Rtp.hpp"
#include "adaptors/MP4Decoder.hpp"
#include "DataChannel.hpp"
#include "net/RTSP.hpp"
#include "ContainerInfo.hpp"
#include "VideoESInfo.hpp"
#include "Statistics.hpp"
#include <stdio.h>
#ifdef WIN32
#include <crtdbg.h>
#endif

#define STD_URL "rtsp://dude.itec-e.uni-klu.ac.at:554/coastguard_9_6.mp4"
#define STD_ADDR "143.205.140.140"

#define STD_PLAYERID "dude"
#define MAX_LENGTH 1500
#define MUCH 100000

/********************************************************************
 * print_usage
 *-------------------------------------------------------------------
 * Prints the usage message
 *-------------------------------------------------------------------
 ********************************************************************/
void print_usage()
{
	printf("Usage: yalt <url> <localport> <start> <end> [options]\n");
	printf("=================================\n");
	printf("-h\tprint this message\n");
	printf("\n");
	exit(0);
}

void destroy(DataChannel* dc,TerminalCapabilities* tc, ContainerInfo* strc, RTSP* rtsp, MP4Decoder* mp4d)
{
	if (dc)
	{
		printf("\033[32m< delete dc; >\033[0m\n");
		delete dc;
		dc = NULL;
	}

	if (tc)
	{
		printf("\033[32m< delete tc; >\033[0m\n");
		delete tc;
		tc = NULL;
	}

	if (strc)
	{
		printf("\033[32m< delete strc; >\033[0m\n");
		delete strc;
		strc = NULL;
	}

	if (rtsp)
	{
		printf("\033[32m< delete rtsp; >\033[0m\n");
		delete rtsp;
		rtsp = NULL;
	}

	if (mp4d)
	{
		printf("\033[32m< delete mp4d; >\033[0m\n");
		delete mp4d;
		mp4d = NULL;
	}

	exit(1);
}

int main(int argc, char* argv[])
{

#ifdef WIN32
	int tmpFlag = _CrtSetDbgFlag( _CRTDBG_REPORT_FLAG );
	tmpFlag |= _CRTDBG_LEAK_CHECK_DF;
	_CrtSetDbgFlag( tmpFlag );
#endif
	int i;
	/*----------------------------------------------------------
	  Parameter checking
	  ----------------------------------------------------------*/
	if (argc < 5 ){
		print_usage();
	}
	
	int localport=1338;
	int start=0;
	int end=0;
//	bool visualize=false;
//	bool cache=false;
	
	if ( 1 != sscanf( argv[2], "%i", &localport ) ) print_usage();
	if ( 1 != sscanf( argv[3], "%i", &start ) ) print_usage();
	if ( 1 != sscanf( argv[4], "%i", &end ) ) print_usage();
	
	// TODO parsing for -v -c
	
	
	for (i = 0 ; i < argc ; i++){
		if (strcmp(argv[i],"-h") == 0){
			print_usage();
		}
	}
	
	char sUrl[MAX_STR_LEN];
	strcpy(sUrl,argv[1]);
	Url uUrl (sUrl);
	printf("Passed URL: %s   local port: %i\n",uUrl.getServerName(), localport);
	//exit(1);

	std::string url = STD_URL;
	char* tmpcp = (char*)malloc(1024);
	char** tmpcpp = &tmpcp;
	
	vitooki_init_all();
	
	u32 width=352;				// vars for TernminalCapabilities
	u32 height=228;
	
	u32 maxdecbufsize=0;
	u32 maxdecbitrate=0;
	u32 displaybits=24;
	u32 refreshrate=100;
	float mem=32;
	u32 netcap=0;
	u32 minnetcap=0;
	u32 netdelay=0;
	u32 bufdelay=0;
	u8 numaudiochann=2;
	u8 audiobitspersample=16;
	u16 samplefreq=44100;

	int remoteport = ((uUrl.hasPort())?uUrl.getPort():554);			// port of RTSP-server
//	int localport = 1338;			// local port for data
 	std::string addr = ((uUrl.getServerName())?uUrl.getServerName():"localhost");
	bool w=false;
	
	std::string playerid = STD_PLAYERID;
	
	bool switching=false;
	
	struct sockaddr_in sin;
	
	struct hostent* hp;
	
	bool color = true;

	char* rtspcmdbuf;
	int csock;
	int len;
	int sessionID;
	char buf[MUCH];				// buffer for incomming RTSP packets

	RTSP* rtsp = new RTSP();
	MP4Decoder* mp4d=NULL;
	Rtp* rtp=NULL;
	GlobalTimer* gt=NULL;
	DataChannel* dc=NULL;
	PacketizationLayer* p=NULL;
	TerminalCapabilities* tc=NULL ;
	VideoESInfo* esi=NULL ;
	ContainerInfo* strc=NULL;
	Statistics* stats=NULL;
		
	printf("\033[32m< hp = gethostbyname( addr %s); >\033[0m\n",addr.c_str());

	hp = gethostbyname(addr.c_str());
	if (hp)
	{
		printf("\033[32m< resolved host >\033[0m\n");
	}
	else
	{
		printf("\033[31m< error, can't resolv host >\033[0m\n");
	}
	
	memset( (char*)&sin,0,sizeof(struct sockaddr_in) );
	sin.sin_family = AF_INET;
	bcopy( hp->h_addr,(char*)&sin.sin_addr,hp->h_length );
	sin.sin_port = htons(remoteport);
	
	printf("\033[032m< int csock = socket( ... ); >\033[0m\n");
	
	csock = socket(PF_INET, SOCK_STREAM, 0);
	if ( csock > 0 )
	{
		printf("\033[32m< csocket is open >\033[0m\n");
	}
	else
	{
		printf("\033[31m< error, can't csocket >\033[0m\n");
		destroy(dc, tc, strc, rtsp, mp4d);	
	}
	
	printf("\033[32m< connect( csock, (...)&sin, sizeof(...) ); >\033[0m\n");

	if (  0 > connect( csock, (struct sockaddr*)&sin, sizeof(struct sockaddr) )  )
	{
		printf("\033[31m< error, can't connect >\033[0m\n");
		destroy(dc, tc, strc, rtsp, mp4d);
	}
	else
	{
		printf("\033[32m< connected to server >\033[0m\n");
	}	
	
	printf("\033[32m< RTSP::generateDescripeForURL >\033[0m");
	rtsp->generateDescribeForUrl( sUrl,playerid.c_str() );
	
	printf("\033[32m< RTSP::getbuffer >\033[0m\n");
	rtspcmdbuf = (char*)rtsp->getBuffer();

	printf("\033[32m< DESCRIBE is: [%s] >\033[0m\n",rtspcmdbuf);

	printf("\033[32m< Sending ...\033[0m");
	fflush(stdout);
	len = send( csock, rtspcmdbuf, strlen(rtspcmdbuf), 0 );
	printf("\033[32m ... %i Bytes\033[0m\n",len);
	
	printf("\033[32m< recv ...\033[0m");
	fflush(stdout);
	memset(buf,'\0',MUCH);
	len = recv( csock, &buf[0], MAX_LENGTH, 0);
	buf[len+1] = '\0';
	printf("\033[32m ... %i Bytes\033[0m\n",len);
	
	printf("\033[32mBUF is: [%s]\033[0m\n",&buf[0]);
	
	
	printf("\033[35mparseResponseDescribe\033[0m\n");	
	strc = rtsp->parseResponseDescribe( &buf[0], sUrl, tmpcpp );

	if (strc != NULL)
		esi = strc->getFirstVisualES();
	else
		{
		printf("\033[31m< ERROR Describe failed >\033[0m\n");
		destroy(dc, tc, strc, rtsp, mp4d);
		}

	if (esi != NULL)
		printf("\033[32m< ESI OK >\033[0m\n");	
	else
		{
		printf("\033[31m< ERROR ESI failed >\033[0m\n");
		destroy(dc, tc, strc, rtsp, mp4d);
		}

	

	printf("\033[32m< TerminalCapabilities* tc = new Terminalcapabilities ( ... ); >\033[0m\n");
	tc = new TerminalCapabilities (maxdecbufsize, maxdecbitrate, displaybits,
			color, width, height, refreshrate, mem,
			netcap, minnetcap, netdelay, bufdelay,
			numaudiochann, audiobitspersample, samplefreq);

	printf("\033[32m< Packetizationlayer* p = new RFC3016(); >\033[0m\n");
	p = new RFC3016();

	printf("\033[32m< GlobalTimer* g = new GlobalTimer(); >\033[0m\n");
	gt = new GlobalTimer();

	printf("\033[32m< Statistics* stats = new Statistics(...); >\033[0m\n");
	stats = new Statistics(esi,w);

	printf("\033[32m< RTP* rtp = new RTP( ... esi, p, tc, gt, stats ... ); >\033[0m\n");
	rtp = new Rtp(sUrl, remoteport, addr.c_str(), localport, esi, p, w, tc, gt, stats);
	rtp->open();
	
	printf("\033[32m< DataChannel dc* = new DataChannel( rtp, esi ); >\033[0m\n");
	dc = new DataChannel ( rtp, esi );
	
	mp4d = new MP4Decoder(esi,true);
	
	printf("\033[35mgenerateSetup\033[0m\n");
	rtsp->generateSetup(sUrl, strc->getFirstVisualES()->getStreamId(), localport, true, localport);
	rtspcmdbuf = (char*)rtsp->getBuffer();
	
	printf("\033[35mSending ...\033[0m");
	fflush(stdout);
	len = send( csock, rtspcmdbuf, strlen(rtspcmdbuf), 0 );
	printf("\033[35m... %i Bytes\033[0m\n",len);
	
	printf("\033[35mrecv ...\033[0m");
	fflush(stdout);
	memset(buf,'\0',MUCH);
	len = recv( csock, &buf[0], MAX_LENGTH, 0 );
	buf[len+1] = '\0';
	printf("\033[35m... %i Bytes\033[0m\n",len);

	printf("\03332mBuf is: [%s]\033[0m\n",&buf[0]);

	rtsp->parseResponseSetup(&buf[0],&sessionID, &localport,&remoteport,&switching,tmpcpp);
	
	printf("\033[35mgeneratePlay\033[0m\n");
	rtsp->generatePlay(sUrl, (u32)sessionID, start, end, 1);
	rtspcmdbuf = (char*)rtsp->getBuffer();

	printf("\033[25mSending ...\033[0m");
	fflush(stdout);
	len = send( csock, rtspcmdbuf, strlen(rtspcmdbuf), 0 );
	printf("\033[35m... %i Bytes\033[0m\n",len);
	rtp->play(10);

	printf("\033[25mrecv ...\033[0m");
	fflush(stdout);
	memset(buf,'\0',MUCH);
	len = recv( csock, &buf[0], MAX_LENGTH, 0 );
	buf[len+1]='\0';
	printf("\033[35m... %i Bytes\033[0m\n",len);
	printf("\033[32mBuf is: [%s]\033[0m\n",&buf[0]);

	char* tehvideo;

	tehvideo = dc->enableCaching(true,".");
	//dc->visualizeCaching();
	
	printf("\033[35m< < STARTING DataChannel > >\033[0m\n");
	dc->start();
	printf("\033[35m< < DataChannel STARTED > >\033[0m\n");

	printf("\033[35m< < DataChannel WAIT > >\033[0m\n");
	dc->wait();
	printf("\033[35m< < DataChannel FINISHED > >\033[0m\n");
	
	//printf("\033[31m<<<<<<<<<<<<<<<%s>>>>>>>>>>>>>>>>\033[0m\n",tehvideo);

	printf("\n\033[32m< < < < SHUT DOWN > > >\033[0m\n\n");
	
	destroy(dc, tc, strc, rtsp, mp4d);

	printf("\033[32m< return 0; >\033[0m\n");
	return 0;
}
