#globally used settings...

#choose build-ARCHITECTURE: intel32, intel64, arm
VITOOKI_BUILD_ARCH	?=	intel32

#where to install the ViTooKi libs and apps
#PREFIX ?= /usr/local
#PREFIX ?= $(HOME)
PREFIX ?= $(shell [ -d /itec/apps ] && echo "/itec/apps" || echo "/usr/local")

#where to look for various libs (SDL, freetype2)
TOOLPREFIX     = /usr
EXTRALIBS		= 3rdparty


##################################
ifeq (intel,$(findstring intel,$(VITOOKI_BUILD_ARCH)))    # intel arch
#enables write support for matroska container
#also links (and builds) libmatroska and libebml
ENABLE_MKV=yes
ENABLE_MKV_WRITE=yes
ENABLE_OGG=yes
ENABLE_ISOMP4=no
ENABLE_ISOMP4=no
ENABLE_SDL=yes
ENABLE_QT=yes
ENABLE_X11=yes
ENABLE_XVID=yes
ENABLE_FFMPEG=yes
#DVDREAD uses ffmpeg, so both have to be yes for dvdread!
ENABLE_DVDREAD=yes
QT_LIB_INC=-lqt-mt

SDLDIR       = $(TOOLPREFIX)
SDL_LIBS     = -L$(SDLDIR)/lib
SDL_INCLUDES = -I$(SDLDIR)/include/SDL
##SDL_FLAGS    = $(SDL_LIBS) -lSDL
SDL_FLAGS    = $(SDL_LIBS) -lSDL -lSDL_image

LAMEDIR      = $(BASEDIR)/$(EXTRALIBS)/lame
LAMELIB      = -lmp3lame
FFMPEG      = $(BASEDIR)/$(EXTRALIBS)/ffmpeg
FFMPEGINC    = -I$(FFMPEG)/libavformat -I$(FFMPEG)/libavcodec -I$(FFMPEG)/libavutil
#FFMPEGLIB    = $(FFMPEG)/libavformat/libavformat.a  $(FFMPEG)/libavcodec/libavcodec.a $(FFMPEG)/libavutil/libavutil.a $(LAMELIB)
FFMPEGLIB    = -lavformat -lavcodec -lavutil $(LAMELIB)

CPP       = g++
GCC				= gcc
AR  	= ar

##################################
else                    # ARM ipaq
CONFIG_ARM=--build=i686-pc-linux-gnu --host=arm-unknown-linux-gnu

#overwrite oe INCLUDES!
CPPFLAGS=
LDFLAGS=
CFLAGS=

ENABLE_MKV=no
ENABLE_MKV_WRITE=no
ENABLE_OGG=yes
ENABLE_SDL=yes
ENABLE_QT=no
ENABLE_X11=yes
ENABLE_DVDREAD=no
ENABLE_XVID=yes
ENABLE_FFMPEG=yes
ENABLE_ISOMP4=no

QT_LIB_INC=-lqte
TOOLPREFIX	=	$(QPEDIR)

#X11DIR = $(TOOLPREFIX)
#X11_INCLUDES = -I/usr/X11R6/include

SDLDIR       = $(TOOLPREFIX)
SDL_LIBS     = -L$(SDLDIR)/lib
SDL_INCLUDES = -I$(SDLDIR)/include/SDL
#SDL_FLAGS    = $(SDL_LIBS) -L$(QPEDIR)/qt2/lib -lopiecore2 -lqpe -lqte -lSDL -lasound  -lts -lpng12 -lz -ljpeg
SDL_FLAGS    = $(SDL_LIBS) -lSDL -lasound

LAMEDIR      = $(TOOLPREFIX)
LAMELIB      = -L$(LAMEDIR) -lmp3lame
FFMPEG      = $(BASEDIR)/$(EXTRALIBS)/ffmpeg
FFMPEGINC    = -I$(FFMPEG)/libavformat -I$(FFMPEG)/libavcodec -I$(FFMPEG)/libavutil
#FFMPEGLIB    = -lavformat -lavcodec $(LAMELIB) -la52 -lfaac -lpostproc -lvorbisenc -lz
FFMPEGLIB    = $(FFMPEG)/libavformat/libavformat.a  $(FFMPEG)/libavcodec/libavcodec.a $(FFMPEG)/libavutil/libavutil.a $(LAMELIB)  -lvorbisenc -lz


CPP       = arm-linux-g++
GCC       = arm-linux-gcc
AR 	= arm-linux-ar
endif


######always disable DVDREAD when ffmpeg is disabled!
ifeq ($(ENABLE_FFMPEG),no)
ENABLE_DVDREAD=no
endif

#uncomment, if you want!
# use QThread instead of our pthread-wrapper VThread
# (only works of course, if ENABLE_QT is enabled)
ifeq ($(ENABLE_QT),yes)
###CPPFLAGS += -DQT_THREAD_SUPPORT
endif


#add build_architecture as define
ifeq ($(VITOOKI_BUILD_ARCH),intel32)
CPPFLAGS += -DBUILD_ARCH_INTEL -DARCH_IS_32BIT -DARCH_IS_IA32
else ifeq ($(VITOOKI_BUILD_ARCH),intel64)
CPPFLAGS += -DBUILD_ARCH_INTEL -DARCH_IS_64BIT -DARCH_IS_IA64
else
CPPFLAGS += -DBUILD_ARCH_ARM   -DARCH_IS_32BIT -DARCH_IS_GENERIC
endif

#if you want, turn on network benches, turn off feedback/RTX or adaptation:
###CPPFLAGS += -DNETBENCH -DWITHOUT_RTP_EXT -DNO_ADJUST

#X11 X-Windows
X11DIR ?= $(TOOLPREFIX)/X11R6
X11_INCLUDES ?= -I$(X11DIR)/include
X11_LIBS     ?= -L$(X11DIR)/lib
X11_FLAGS    = $(X11_LIBS) -lX11 -lXext -lXau -lXrender -lXrandr -lXdmcp

# qt... QTDIR is only set if not predefined!   ?=
QTDIR        ?= $(TOOLPREFIX)/lib/qt3
QT_LIBS      ?= -L$(QTDIR)/lib
UIC          ?= $(QTDIR)/bin/uic
MOC          ?= $(QTDIR)/bin/moc
QT_INCLUDES  ?= -I$(QTDIR)/include
QT_MKSPECS   = $(QTDIR)/mkspecs/linux-g++
QT_FLAGS     = $(QT_LIBS) $(QT_LIB_INC)

EXPATDIR       = $(TOOLPREFIX)
EXPAT_LIBS     = -L$(EXPATDIR)/lib
EXPAT_INCLUDES = -I$(EXPATDIR)/include
EXPAT_FLAGS		= $(EXPAT_LIBS) -lexpat

LIBMADDIR       = $(TOOLPREFIX)
LIBMAD_LIBS     = -L$(LIBMADDIR)/lib
LIBMAD_INCLUDES = -I$(LIBMADDIR)/include
LIBMAD_FLAGS		= $(LIBMAD_LIBS) -lmad

# sord
SORDDIR      = $(BASEDIR)/$(EXTRALIBS)/sord
SORDLIB      = $(SORDDIR)/src/libsord.a
SORDINC      = -I$(SORDDIR)/include

# common ucl
COMMONUCL    = $(BASEDIR)/$(EXTRALIBS)/commonucl
UCLLIB       = $(COMMONUCL)/.libs/libuclmmbase.a
UCLINC       = -I$(COMMONUCL)

# xvidcore
XVID	     = $(BASEDIR)/$(EXTRALIBS)/xvidcore-1.1.0-beta2
XVIDCORE     = $(XVID)/src
XVIDLIB      = $(XVID)/build/generic/=build/libxvidcore.a
XVIDINC      = -I$(XVIDCORE)

LIBEBML	     = $(BASEDIR)/$(EXTRALIBS)/libebml
EBMLLIB	     = -lebml
EBMLINC	     = -I$(LIBEBML)

LIBMATROSKA  = $(BASEDIR)/$(EXTRALIBS)/libmatroska
MATROSKALIB  = -lmatroska
MATROSKAINC  = -I$(LIBMATROSKA)

PARADIR      = $(BASEDIR)/$(EXTRALIBS)/paragui
PARALIB      = $(PARADIR)/src/.libs/libparagui.a
PARAINC      = -I$(PARADIR)/include

# dvdread
DVDREAD      = $(BASEDIR)/$(EXTRALIBS)/libdvdread
DVDREAD_INCLUDES   = -I$(DVDREAD)
DVDREAD_LDFLAGS    = -ldvdread -ldvdcss

#no debugging output (make release)
RELEASE_CPPFLAGS = -O3 -funroll-loops -DVITOOKI_DEBUG_LEVEL=0 -fPIC

# debug level: see src/debug.hpp
# DEBUG may be set on the make command-line! (make debug)
DEBUG_CPPFLAGS = -g -fPIC -DVITOOKI_DEBUG=$(DEBUG) -DVITOOKI_DEBUG_LEVEL=$(DEBUG)
DEBUG_LDFLAGS  = -g -fPIC

# profiling flags
# set PROF to 1 on the command-line! (make prof)
PROF_CPPFLAGS = -pg
PROF_LDFLAGS  = -pg

##INCLUDES  = -I($TOOLPREFIX)/include
INCLUDES += -I$(BASEDIR)/$(EXTRALIBS)
INCLUDES += $(UCLINC) $(SORDINC)
INCLUDES += $(EXPAT_INCLUDES) $(LIBMAD_INCLUDES)
INCLUDES += -I. -I$(TOOLPREFIX)/include/freetype2
#incuded as default, since CODEC_ID and other defs are always used
INCLUDES += $(FFMPEGINC)


LDFLAGS  += -L$(BASEDIR)/src/.libs -Wl,-rpath-link,$(BASEDIR)/src/.libs $(EXPAT_FLAGS) $(LIBMAD_FLAGS) $(SORDLIB) $(UCLLIB)
CPPFLAGS += $(EXTRA_FLAGS)
CPPFLAGS += -Wall
CPPFLAGS += -D_FILE_OFFSET_BITS=64 -D_LARGEFILE_SOURCE
#for easier user-based development
CPPFLAGS += -DUSER_$(USER)

ifeq ($(ENABLE_QT),yes)
CPPFLAGS += -DENABLE_QT
LDFLAGS  += $(QT_FLAGS)
INCLUDES += $(QT_INCLUDES)
endif

ifeq ($(ENABLE_SDL),yes)
CPPFLAGS += -DENABLE_SDL
LDFLAGS  += $(SDL_FLAGS)
INCLUDES += $(SDL_INCLUDES)
endif

ifeq ($(ENABLE_X11),yes)
CPPFLAGS += -DENABLE_X11
LDFLAGS  += $(X11_FLAGS)
INCLUDES += $(X11_INCLUDES)
endif

ifeq ($(ENABLE_DVDREAD),yes)
CPPFLAGS += -DENABLE_DVDREAD
LDFLAGS  += $(DVDREAD_LDFLAGS)
INCLUDES += $(DVDREAD_INCLUDES)
endif

ifeq ($(ENABLE_XVID),yes)
CPPFLAGS += -DENABLE_XVID
LDFLAGS  += $(XVIDLIB)
INCLUDES += $(XVIDINC)
endif

ifeq ($(ENABLE_ISOMP4),yes)
ISOMP4BASEDIR   = $(BASEDIR)/$(EXTRALIBS)/isomp4/libisomediafile
ISOMP4DIR    = $(MP4BASEDIR)/linux/libisomediafile
ISOMP4LIB    = $(shell ls $(ISOMP4DIR)/libisomediafile.a)
ISOMP4INC  = -I$(MP4BASEDIR)/src -I$(MP4BASEDIR)/linux
#CPPFLAGS += -DENABLE_ISOMP4
CPPFLAGS += -DISOMP4
LDFLAGS  += $(ISOMP4LIB)
INCLUDES += $(ISOMP4INC)
endif

ifeq ($(ENABLE_FFMPEG),yes)
CPPFLAGS += -DENABLE_FFMPEG
LDFLAGS  += $(FFMPEGLIB)
#already incuded as default
#INCLUDES += $(FFMPEGINC)
endif

ifdef DEBUG
CPPFLAGS += $(DEBUG_CPPFLAGS)
LDFLAGS  += $(DEBUG_LDFLAGS)
endif

ifdef PROF
CPPFLAGS += $(PROF_CPPFLAGS)
LDFLAGS  += $(PROF_LDFLAGS)
endif

# Matroska
ifeq ($(ENABLE_MKV),yes)
CPPFLAGS += -DENABLE_MKV
endif

ifeq ($(ENABLE_MKV_WRITE),yes)
CPPFLAGS += -DENABLE_MKV_WRITE
LDFLAGS  += $(MATROSKALIB)
INCLUDES += $(MATROSKAINC) 
LDFLAGS  += $(EBMLLIB)
INCLUDES += $(EBMLINC)
endif

# OGG + Theora + Vorbis
ifeq ($(ENABLE_OGG),yes)
CPPFLAGS += -DENABLE_OGG
LDFLAGS += -logg -ltheora -lvorbis -lvorbisenc -lvorbisfile
endif

%.o: %.c
	$(GCC) -c $(CPPFLAGS) $(INCLUDES) $<

%.c:	%.h

%.cpp:	%.hpp

%.o: %.cpp
	$(CPP) -c $(CPPFLAGS) $(INCLUDES) -o $@ $<

%.hpp: %.ui
	$(UIC) -o $(subst .ui,.hpp,$<) $<

%.cpp: %.ui
	$(UIC) -i $(subst .ui,.hpp,$<) $< |	sed 's/\(static const .* \)const/\1/'  > $@

moc_%.cpp: %.hpp
		$(MOC) $< -o $@


# make 'all' the default target
all:

all_recursive clean_recursive distclean_recursive install_recursive uninstall_recursive doc_recursive:
	if [ -n "$(SUBDIRS)" ]; then \
	    for dir in $(SUBDIRS); do $(MAKE) -C $$dir $(patsubst %_recursive,%,$@) || exit 1; done \
	fi

.PHONY: all all_recursive clean_recursive distclean_recursive install_recursive uninstall_recursive doc_recursive
