 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	Adaptor.hpp                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PS_ADAPTOR_HPP_
#define _PS_ADAPTOR_HPP_
class ESInfo;

#include "global.hpp"
#include "Frame.hpp"
#include "ESInfo.hpp"

#ifdef WIN32
#include <winsock2.h>
#endif



/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: Adaptor.hpp,v 1.14 2006/01/20 15:37:17 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class Adaptor 
{
public:
	
	/** 
		Accepts as input a frame, and checks if it can adapt the frame,
		If the frame is modified, a new Frame object is created and inserted into the return list.
		This new frame is returned only, if the size of the payload is
		larger than zero. If the frame was not modified, it is also inserted
		into the list.
		For more complex adaptors, it will happen, that the adaptor caches up one complete GOP
		and returns the full GOP in network order. A caching adaptor has to create
		deep-copies for each frames.
		Never directly modify the payload of the input frm, always create copies!!!

		@param frm object to adapt. An Adaptor is never allowed to free
		the input frame. This has to be done in the caller.
		@return List of adapted Frame objects
	*/
	virtual list<Frame*> adapt(Frame * frm) { 
		list<Frame*> emptyList; 
		return emptyList; 
	};

  Adaptor() {};
	virtual ~Adaptor(){};

	/**
		Close and destroy an Adaptor.

		Releases all internally buffered frame objects, and deallocate 
		all allocated memory. The Adaptor must not be used after calling
		the close method!!!

		@return List of adapted Frame objects.
	*/
	virtual list<Frame*> close() {
		list < Frame * >emptyList;
		return emptyList;
	}

	/**
		Create a shallow copy of the Adaptor.

		Creates an Adaptor with the same setup (without copying the current status).
		@return An Adaptor object.
	*/
	virtual Adaptor *clone() = 0;

	/**
		Initialize internal data structures.
	*/
	virtual void initialize() = 0;

	/**
		Return the name of the Adaptor.

		Do use the same name as the classname (workaround for 
		broken isInstanceOf in most compilers).
		
		@return String containing class name.
	*/
	virtual const char *getName() { return name; };
	
	/** Sets a reference to an ESInfo object. Adaptors are
		allowed to change ESInfo objects. If you don't want that,
		pass an es->clone(). NEVER delete this es in the destructor of 
		your Adaptor!
	*/ 
	virtual void setESInfo(ESInfo * esi) {};

	virtual ESInfo * getESInfo() { return es; };

	/** returns adaptation costs (CPU only) per second! 
	  * For visual streams they are expressed in pixels/seconds.
		* Call this only on INITIALIZED adaptors!!!! Otherwise costs will
		* be way too high.
	*/
	virtual u32 getTranscodingCosts() const = 0;

protected:
	bool initialized;
	/** the name of the Adaptor, Workaround for broken instanceof in compilers 
	  * and used for pretty-printing. Do not forget to initialize this value
		* in your subclassed
		*/
	char name[128];
  ESInfo *es;
	
};
#endif
