 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	AdaptorChain.hpp                                              *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _ADAPTOR_CHAIN_HPP_
#define _ADAPTOR_CHAIN_HPP_

#include "Adaptor.hpp"

/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: AdaptorChain.hpp,v 1.9 2005/06/19 14:47:22 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class AdaptorChain : public Adaptor {
public:
	/**
		 Constructor.
	*/
	AdaptorChain();
	
	/**
		 Destructor.
	*/
	virtual ~ AdaptorChain();
	
	/**
		 Virtual method, used to adapt a input Frame object to
		 a list of output Frame objects.
	*/
	virtual list <Frame*> adapt(Frame * frm);
	
	/**
		 Add a new Adaptor to the adaptor chain.
		 
		 FIXME: check for valid chain not implemented!
		 
		 @param pAdaptor Pointer to Adaptor which should be added.
		 @return true if adding pAdaptor creates a valid adaptor chain,
		 false otherwise
	*/
	virtual bool addAdaptor(Adaptor * pAdaptor);
	
	/** adds an adaptor to the front of the adaptor list! */
	bool pushFrontAdaptor(Adaptor * pAdaptor);
	
	list<Adaptor*> *getAdaptors() { return &mAdaptorList;};
	/**
		 Close the adaptor chain, and flush all buffered Frame objects
		 (if any).
		 
		 @return List of Frame objects.
	*/
	virtual list <Frame *> close();
	
	virtual void initialize();
	
	/** returns adaptation costs (CPU only) */
	virtual u32 getTranscodingCosts() const;
	
protected:
	
	/**
		 List of Adaptor objects. 
	*/
	list < Adaptor * >mAdaptorList;
};


#endif				/// _ADAPTOR_CHAIN_HPP_
