/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   AudioESInfo.hpp                                             *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/
 
#ifndef PS_AUDIOELEMENTARYSTREAMINFO_HPP
#define PS_AUDIOELEMENTARYSTREAMINFO_HPP

#include "ESInfo.hpp"

class AudioESInfo: public ESInfo
{
public:
	/** creates an AudioESInfo object
	  * @param encodeddecconf is only shallow copied and
	  * freed in the destructor
	  * @param avgBW and @param maxBW specify the bandwidth of the given ES
	  * and are in Bits/sec!
	*/
	AudioESInfo(u32 streamId, u32 streamtype, ContainerInfo * vob, u32 timeIncrement,
				 u8 * encodeddecconf, u32 objtype, u32 bufsize,
				 u32 avgBW, u32 maxBW, u32 timeScale,
				 u64 duration, u32 size,  
				 u32 samplerate, u32 numAudioChannels,
				 bool complete=true, 
				 CodecID codec_id = CODEC_ID_MP3, float quality = 1.0, 
				 u32 bitsPerSample = 16);
	
	virtual ~AudioESInfo(){};

	u32 getHandlerType() const;

	u32 getSampleRate() const;

	void setSampleRate(u32 sampleRate);

	u32 getAudioChannels() const;

	void setAudioChannels(u32 channels);

	void setLanguage(const char *lang) { strncpy(language, lang, MAX_STR_LEN); };

	const char *getLanguage() { return language; };
	
	void generateDescriptionText();

	u32 getBitsPerSample() const;

  /** returns type of audio -> stereo, mono etc */
	u32 getChannelMod() const;

	void setChannelMod(u32 uiChannelMod);
	
	bool isVisualStream() const;

	bool isAudioStream() const;

	bool isODStream() const;

	bool isSceneDescriptionStream() const;

  /** creates a deep copy of the ES */
	ESInfo *clone(ContainerInfo * mp4);

protected:
	u32 channelMod;
	u32 sample_rate;
	u32 audio_channels;	
	/* bits per sample, when the frame is uncompressed i.e. WAV */
	u32 bitsPerSample;
	char language[MAX_STR_LEN];
};

#endif

 
