 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	CompressedVideoFrame.cpp                                      *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "CompressedVideoFrame.hpp"
#include <math.h>
/**
 * Constructor
 * -----------
 */

CompressedVideoFrame::CompressedVideoFrame(FrameType t, int w, int h) : VideoFrame(t,w,h) {
}


CompressedVideoFrame::~CompressedVideoFrame() {
}


Frame::FrameType CompressedVideoFrame::detectFrameType() {
  if (!(accessUnit && accessUnit->payload && accessUnit->size>3)) {
    return ERROR_VOP;
  }
  
  
  
  bool typeSet = false;
  bool found = false;
  uint i = 0;

  if(!found) {
    // is it THEORA???
    if(memcmp(&accessUnit->payload[1],"theora",6) == 0) {
			//dprintf_full("CompressedVideoFrame::detectFrameType THEORA!\n");
      // YES IT IS!
      typeSet = true;
      found = true;
      // now check, if it is a keyframe!
      /*
       * 07.03.06: [www.xiph.org/theora/doc/Theora_I_spec.pdf], page 18:
       * "Theora only supports intra frames (I frames in MPEG) and
       * inter frames (P frames in MPEG). There is no equivalent
       * to the bi-predictive frames (B frames) found in MPEG
       * codecs."
       *
       * For ViTooKi we classify inter frames as B frames in order
       * to profit from frame dropping strategies used in ViTooKi.
       */

      /*
       * from libtheora-1.0alpha5/lib/toplevel.c
       *
       * if (op->packet[0] & 0x80) return -1; // not a data packet 
       * return (op->packet[0] & 0x40) ? 0 : 1; // inter or intra 
       */
      if(accessUnit->payload[0] & 0x80) // it is not a valid THEORA data packet, maybe HEADER frame... don't care...
				setType(NN_VOP);
      else if(accessUnit->payload[0] & 0x40)
				setType(B_VOP);
      else
				setType(I_VOP);
      
      // all done... so return!
      return getType();
    }
  }
  
  
  while((i+3) < accessUnit->size && !found) {
		
    if(accessUnit->payload[i] == 0x00)
      if(accessUnit->payload[i+1] == 0x00)
				if(accessUnit->payload[i+2] == 0x01)
					if( (accessUnit->payload[i+3] >= 0xb0) || (accessUnit->payload[i+3] == 0x00)) { //MPEG4 or 2/1
						found = true;
						i--;
					}
    i++;
  }
  
  //MPEG-4 video frame startcode
  
  if (found && accessUnit->size > i+3) { 
		//dprintf_full("CompressedVideoFrame::detectFrameType MPEG4!\n");
		
    // start code for MPEG-4 frame header
    if(accessUnit->payload[i+3] == 0xb6) {
      // detect MPEG-4 frame type
      if ( (accessUnit->size > i+4) &&
					 ( ((accessUnit->payload[i+4]) >> 6) < 4) ) { 
				//set MPEG-4 frame type
				setType( (FrameType) ((accessUnit->payload[i+4]) >> 6) );
				typeSet = true;
      }
      // frame type not found
      else {
				dprintf_full("CompressedVideoFrame::detectFrameType --> INVALID VOP Header %i at AU found --> MPEG-4\n",
										 (accessUnit->payload[i+4]) >> 6);
				setType( ERROR_VOP );
      }
    } else 
      if(accessUnit->payload[i+3] >= 0xb0) {
				setType( HEADER_VOP );	
				typeSet = true;
      }
  }
  
  
  if(!typeSet && (accessUnit->size>(i+5))) {
    // MPEG-1/2 video frame startcode
		//dprintf_full("CompressedVideoFrame::detectFrameType MPEG1/2!\n");

    if (accessUnit->payload[i+3] == 0x00) {
      int type = accessUnit->payload[i+5] & 0x38;
      if (type == 0x08) {
				setType( I_VOP);
				typeSet = true;
      }
      if (type == 0x10){
				setType( P_VOP);
				typeSet = true;
      }
      if (type == 0x18){
				setType( B_VOP);
				typeSet = true;
      }
    }
    else {
      dprintf_full("CompressedVideoFrame::detectFrameType --> INVALID VOP Header at AU found --> MPEG-1/2\n");
      
      setType( ERROR_VOP );
    }
    /*
      printf("Sequence: %0x %0x %0x %0x %0x %0x\n",
      accessUnit->payload[i],
      accessUnit->payload[i+1],
      accessUnit->payload[i+2],
      accessUnit->payload[i+3],
      accessUnit->payload[i+4],
      accessUnit->payload[i+5]);
    */
  }
  
  /*	int ftype = getType();
    if(ftype == 0) {
    cout << " I ";
    }
    if(ftype == 1) {
    cout << " P ";
    }
    if(ftype == 2) {
    cout << " B ";
    }
    cout << "-";
  */
  return getType();
};

