 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	DataSink.hpp                                                  *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PS_CLIENT_HPP
#define _PS_CLIENT_HPP

#include "global.hpp"
#include "IO.hpp"
class Adaptor;


/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: DataSink.hpp,v 1.4 2006/01/20 15:37:17 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class DataSink {
public:
	/** ideally address should be a stringPresentation of the form "127.0.0.1". If the 
	 * address is a name ("hurra.itec.uni-klu.ac.at"), a dns lookup has to be performed.
	 * FIXME: implement this lookup
	 * @param id: use a binary representation of the ip address
	 * @param address is deep-copied
	 * @param out is shallow-copied and freed in the destructor
	 * @param adaptor is shallow-copied and freed in the destructor
	 */
	DataSink(u32 id, IO * out, const char *address, int port = 0, Adaptor * adaptor = NULL);

	virtual ~ DataSink();

	const char *getAddress() const;

	IO *getOutput() const {
		return output;
	};

	int getPort() const {
		return port;
	};
	
	/** inserts the frame into the sendqueue of the client. blocking
	 * the local adaptor is called and then the result is sent
	 * Do not forget to delete the input-frm :-)
	 */
	bool send(Frame * frm);
	
	/**closes the output, sends buffered frames if @param immediate is false */
	bool close(bool immediate=false);
	Adaptor *getAdaptor() {
		return localAdaptor;
	};

	u32 getDataSinkId() const { return clientId;};
	void pauseOutput();
	void muteOutput();
	void playOutput(double prefetch);
	
protected:
	char *adr;
	/** use the binary ip address */
	u32 clientId;
	IO *output;
	int port;
	list < Frame * >sendQueue;
	/** a client can have a local Adaptor/-Chain that adapts
	 * the result from the global common Adaptor (in DataChannel)
	 * to its specific needs
	 */
	Adaptor *localAdaptor;
	Frame *lastFrm;
};
#endif
