 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	Frame.hpp                                                     *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef __Frame_hpp__
#define __Frame_hpp__

#include "global.hpp"

#include "IO.hpp"


/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: Frame.hpp,v 1.11 2006/03/01 14:34:33 tkuglits Exp $
 *
 *@short <short description>
 *
 */
class Frame {

public:
	
// the frame types
  typedef enum { I_VOP, P_VOP, B_VOP, S_VOP, NN_VOP, YUV_VOP, RGB_VOP,
		 MP3_AUDIO, AAC_AUDIO, RAW_AUDIO, VORBIS_AUDIO,
		 HEADER_VOP, ERROR_VOP, UNKNOWN_VOP} FrameType, VOP_TYPE;

  static const char *VOPTypeToChar(VOP_TYPE t) {
    static const char *VOP_CHAR[] = {"I_VOP", "P_VOP", "B_VOP", "S_VOP", "NN_VOP", "YUV_VOP", "RGB_VOP",
				     "MP3_AUDIO", "AAC_AUDIO", "RAW_AUDIO", "VORBIS_AUDIO", 
				     "HEADER_VOP", "ERROR_VOP", "UNKNOWN_VOP"};
    return VOP_CHAR[t];
  }

	Frame(FrameType t);

	// concrete methods
	void setType(FrameType t) {
		if(accessUnit) accessUnit->frameType = (u32)t;
	}

	FrameType getType() const {
		return (accessUnit? (FrameType)accessUnit->frameType: ERROR_VOP);
	}

  /** returns the timescale of the stream measured in ticks/second */
	u32 getMediaTimeScale() const {
		return mediaTimeScale;
	}

  /** set the mediaTimeScale */
	void setMediaTimeScale(u32 uiMediaTimeScale) {
		mediaTimeScale = uiMediaTimeScale;
	}

  /** fills the buffer with an AU. au != NULL must be true.
	 * Problem: calling resize on the buffer, is not advised.
	 */
	bool setAU(AU * au, bool detectType = true);

	/** removes the previously set AU, so it will not be freed in Frame destructor.
	  * You must delete the AU outside the class, otherwise you'll have a massive
	  * memory leak!
	*/
	bool unsetAU();


	AU *getAU() const {	return accessUnit; };

	void setAUy(u8 *y) { accessUnit->y = y; };
	void setAUu(u8 *u) { accessUnit->u = u; };
	void setAUv(u8 *v) { accessUnit->v = v; };

	/* this codecID should correlate to the according ESInfo->getStreamID() */
	void setCodecID(u32 id) { codecID = id; };

	/* this track ID should correlate to the according ESInfo->getStreamID() */
	u32 getCodecID() { return codecID; };

	/* this streamID should correlate to the according ESInfo->getStreamID() */
	void setStreamID(u32 id) { streamID = id; };

	/* this track ID should correlate to the according ESInfo->getStreamID() */
	u32 getStreamID() { return streamID; };

	/* used by ESSynchronizer, so DataChannel will drop this frame after returning from adaptor chain.
	 * further, DataChannel will set this for every frame before it is passed to the DataSinks, 
	 * after being sent to all outputs, it's deleted by the DataChannel.
	 *so if you have some buffering output, make sure to unmark it and delete it on your own! (eg. network writers, muxers,...) */
  void markForDelete(bool yes=true) { markedForDelete = yes; };

	/* DataChannel will drop this frame after returning from adaptor chain, or after passing to the DataSinks */
	bool isMarkedForDelete() { return markedForDelete; };

	// virtual methods
	virtual ~ Frame();
	
	/** Determines the correctness of the buffer
	  * and extracts the frametype of the frame
		* On any problem: FrameType ERROR_VOP is returned
	*/
	virtual FrameType detectFrameType() { return UNKNOWN_VOP; };

protected:

	AU *accessUnit;

	u32 codecID;

	u32 streamID;

	u32 mediaTimeScale;

	bool markedForDelete;
};

#endif				// __Frame_hpp__
