 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	IO.cpp                                                        *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "IO.hpp"
char* sRTXState[]={"none","ack","nack","all"};

IO::IO() {
	mutexStateChanged.initialize("mutexStateChanged");
//	pthread_cond_init(&condStateChanged,NULL);
	state=CLOSED;
	framesSeen=0;
	currentFrameNumber=0;
	endFrameNumber=0;
}


IO::~IO() {
	mutexStateChanged.destroy();
//	pthread_cond_destroy(&condStateChanged);
}


IO::State IO::setState(IO::State s) {
	mutexStateChanged.lock();
	state=s;
//	pthread_cond_signal(&condStateChanged);
	mutexStateChanged.release();
	return state;
}

IO::State IO::play(double prefetchTime) {

	dprintf_full("IO::play prefetch %f (old state %i)\n",prefetchTime,state);
	
	if (state==CLOSED) {
		open();
		return state;
	} else
		if ( (state==PAUSED) || (state==MUTED) ) 
			setState(OPEN);
		else {
			dprintf_err("IO::play tried to OPEN from state %i\n",state);
			return state;
		}
	return state;
}


IO::State IO::pause() {
	
	dprintf_err("IO::pause (old state %i)\n",state);
	if (getState() == IO::OPEN || 
      getState()  == IO::OPENING || 
      getState()  == IO::CLOSING || 
      getState()  == IO::STREAMEOF || 
      getState()  == IO::MUTED)
    return setState(IO::PAUSED);
  else
    return getState();
	//	return (state==OPEN)?setState(PAUSED):state;
}


IO::State IO::mute() {
	dprintf_err("IO::mute (old state %i)\n",state);
	return (state==OPEN)?setState(MUTED):state;
}

