 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	IO.hpp                                                        *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PS_PROXY_IO_HPP
#define _PS_PROXY_IO_HPP

#include "global.hpp"
#include <assert.h>
#ifdef ISOMP4
#include "ISOMovies.h"
#endif
//#include "VCondition.hpp"
#include "VMutex.hpp"

class ESInfo;
class Frame;
class VThread;


enum RTXState { NONE,ACK,NACK,NACKACK};
#define LAST_RTX_STATE NACKACK
extern char* sRTXState[];

	typedef struct rtx_info {
		//address is taken from RTP data stream
		u16 remotePort;
		u16 localPort;
		struct rtp *session;
		u16 payloadType;
		u32 ticks;
		u32 mId;
		u32 rtxTimeInMs;
		RTXState state;
		rtx_info() {
			rtxTimeInMs=0;
			state=NONE;
			ticks=payloadType=remotePort=localPort=mId=0;
			session=NULL;
		};
		rtx_info(RTXState type,u32 maxTime) {
			rtxTimeInMs=maxTime;
			state=type;
			ticks=payloadType=remotePort=localPort=mId=0;
			session=NULL;
		};
		/** copies everything except session! */
		rtx_info(const rtx_info* r) {
			assert(r);
			remotePort  = r->remotePort; 
			localPort   = r->localPort;
			payloadType = r->payloadType;
			ticks       = r->ticks;
			mId         = r->mId;
			rtxTimeInMs = r->rtxTimeInMs;
			state       = r->state;
			session     = NULL;
		}   
		~rtx_info()
		{
		};
	} rtx_info;

	typedef struct rtx_group {
		u32 midESInfo;
		rtx_info* rtx;
		const ESInfo* es;

		rtx_group(RTXState type,u32 maxTime) {
			rtx=new rtx_info(type,maxTime);
			es=NULL;
			midESInfo=0;
		};
		~rtx_group() {
			if(rtx)
				delete rtx;
		};
	} rtx_group;

enum Error { SA_OK, SA_Error, SA_EOF };
struct AU {
	/** the actual frame data including VOP header, must be the first entry in AU! */
	u8 *payload;
	u8 *y,*u,*v; // on linux point to the components in the payload, on winCE it uses three diff. buffers
	/** size of the payload in bytes, must be the 2nd entry in AU! */
	u32 size;
	/** composition timestamp (ticks/secs) */
	u32 cts;
	/** decoding timestamp */
	u32 dts;
	u32 duration;
	u32 sampleFlags;
	u8 prio;
	u32 frameType;
	Error err;

	/** serializes the AU without payload. */
	char *toString() {
		int auSize = sizeof(AU) - sizeof(u8 *);
		char *res = new char[auSize];
		memcpy(res, &size, auSize);
		return res;
	};
	/** deserializes the string, inits the AU and sets payload to NULL */
	bool fromString(char *in) {
		memcpy(&size, in, sizeof(AU) - sizeof(u8 *));
		payload = NULL;
		return true;
	};
	/** two au are identical if the have the same size,cts,dts,prio and duration.
	    payload/flags/err are ignored */
	bool operator == (const AU & a1) {
		return a1.size == size && a1.cts == cts && a1.dts == dts
	    && a1.duration == duration && a1.prio == prio;
	};
	/** two au are identical if the have the same size,cts,dts,prio and duration.
	    payload/flags/err are ignored */
	bool operator == (const AU * &a1) {
		return a1->size == size && a1->cts == cts && a1->dts == dts
	    && a1->duration == duration && a1->prio == prio;
	};
	AU() {
		payload=NULL;
		size = 0;
		cts = 0;
		dts = 0;
		duration = 0;
		sampleFlags = 0;
		prio = 255;
		frameType=0;
		err = SA_OK;
	};
	/** copies everything but payload, size is set to 0 */
	AU(const AU & in) {
		payload=NULL;
		size = 0;
		cts = in.cts;
		dts = in.dts;
		duration = in.duration;
		sampleFlags = in.sampleFlags;
		prio = in.prio;
		frameType=in.frameType;
		err = in.err;
	};
	AU(const AU * in) {
		payload=NULL;
		size = 0;
		cts = in->cts;
		dts = in->dts;
		duration = in->duration;
		sampleFlags = in->sampleFlags;
		prio = in->prio;
		frameType=in->frameType;
		err = in->err;
	};
	~AU() {
		if(payload) {
			delete[] payload;
			payload=NULL;
		}
	};

};

/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: IO.hpp,v 1.25 2006/03/17 17:14:52 tkuglits Exp $
 *
 *@short <short description>
 *
 */
class IO: public VThread {
	
public:
  /* OPEN and MUTED means playing and streaming, adapting in the DC and in the DataSinks,
	 *      but MUTED does all except adaptation in the DataSinks, where the decoding normally
	 *      takes place...still, the I/O output writeFrame is called with the unadapted frame!
   *      by this, global DC adaptors might still be used for eg. statistics
	 * PAUSED is kinda like CLOSED, not playing and streaming, but it could be continued
	 * PREFETCHING is used at beginning or on buffer underrun
	 * STREAMERR is set, if eg. no header was received or, for the Rtp class, 
	 *                                                  the rtp udp ports where already in use 
	 *
	 * StateInNumbers: 0,      1,          2,         3,      4,     5,      6,       7,           8,         9
	 */
	enum    State { CLOSED, CLOSING, FORCE_CLOSING, OPENING, OPEN, PAUSED, MUTED, PREFETCHING, STREAMEOF, STREAMERR };

#define IO_NETWORK_HIGHEST_PRIORITY 0

	IO();

	virtual ~IO();

	/** returns a frame if one complete frame is available, otherwise null 
		is returned. This function is typically blocking. Don't use a NULL
		return value to conclude STREAMEOF, always check with getState()!
	*/
	virtual Frame * getFrame() = 0;

	/** returns the number of packets sent. Returns 0 on error */
	virtual int writeFrame(Frame * frm, ESInfo *es = NULL) = 0;

	/** opens the IO connection. State is set to OPENING. Depending on the
			underlying QIODevice, a network connection or a file connection
			is established. When the connection is ready for use, State is OPEN */
	virtual bool open() = 0;

	/** closes the IO class. @param immediate specifies, if (optional)
	  * buffered data should be read/sent to the client (==false), or immediately
	  * dumped (==true)
	  */
	virtual bool close(bool immediate=false) = 0;

	/** deletes the data source for this IO class. Will only work with
	 * local files, remote files can not be deleted and will always return true.
	 */
	virtual bool destroy() = 0;

	virtual State getState() const { return state;};

	/** returns the newly set state */
	virtual State setState(State s);

	/** returns a value  from 0..100 indicating buffer usage */
	virtual int getBufferFillLevel() const = 0;

	/** flushes the buffer from starting TimeStamp to end TS
	 * @param from_ts: 0 is beginning, everything else is somewhere in the stream
	 * @param to_ts: -1 is ending, everything else is somewhere in the stream
	 * @returns the number of freed bytes
	 */
	virtual int flushBuffer(long from_ts = 0, long to_ts = -1)
	{
		from_ts = from_ts;	// 4Wall
		to_ts = to_ts;		// 4Wall
		return 0;
	};

	/** returns a pointer to the local file name or an URL */
	virtual const char *getURL() const = 0;

	/** repositions the IO class to the given frame.
	 * Will return false in the following cases:
	 *  - an illegal frame number (too large) was specified
	 *  - the stream is not seekable, because the underlying 
	 *    device is a network device and the requested frame 
	 *    is already out of range. 
	 *  - Seeking doesn't work on IO classes opened for writing
	 */
	virtual bool setToFrameNumber(u32 frameNumber) {
		dprintf_full("IO::setToFrameNumber dummy implementation\n");
		return false;
	};

	/** repositions the IO class to @param frameNumber, if this
	  * Frame is an I-frame. Otherwise, according to @param backwards,
		* a frameNumber lower or higher than @param frameNumber is searched and returned.
		* @return the chosen frameNumber or -1 in case of an error
		* will only work with IO classes opened for reading.
		*/
	virtual long setToClosestIFrame(u32 frameNumber, bool backwards = true) {
		dprintf_full("IO::setToClosestIFrame dummy implementation\n");
		setToFrameNumber(frameNumber);
		backwards = backwards;
		return 0;
	};

	/** sets the last frame that should be sent. Allows to specify
	  * in combination with setToFrameNumber a range of frames 
		* that should be sent. Set @param stopNumber to 0, to specify
		* the last frame.
		* @returns true on success
		*/
	virtual bool setEndFrameNumber(u32 stopNumber) {
		endFrameNumber=stopNumber;
		return true;
	}

	virtual u32 getEndFrameNumber() const {	return endFrameNumber; }

	virtual u32 getCurrentFrameNumber() const {	return currentFrameNumber; };

	/** indicates if the next call to getFrame will send the
	 * frame header. */
	virtual void setResendFrameHeader(bool set) { resendFrameHeader=set;};

	virtual void run() { ; }; //immediately stop unused thread

	virtual void setESInfo(ESInfo * es) =0;

	virtual ESInfo* getESInfo() =0;

	/** returns the number of frames received or sent. 
	 * counter will be resetted when open is called */
	virtual u32 getFramesSeen() const { return framesSeen;};

	/** sets a shallow copy of @param rtx at the ESInfo object */
	virtual void setRtxInfo(const rtx_info* rtx) { rtx = rtx; };

	//stupid default implementations...
	virtual State play(double prefetchTime=0.0); //implemented in IO.cpp

	virtual State pause();

	virtual State mute();

protected:

	State state;
	/** indicates if the next call to getFrame will send the
	 * frame header. This is a fix for Quicktime
	 * which gets confused if we sent a header
	 * a second time, even though npt=0.0
	 */
	bool resendFrameHeader;
	u32 currentFrameNumber;
	/** specifies the last frame number that should be sent */
	u32 endFrameNumber;
	VMutex mutexStateChanged;
//	VCondition condStateChanged;
	u32 framesSeen;
};
#endif
