/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   ObjectDescriptorESInfo.cpp                                              *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/
#include "ObjectDescriptorESInfo.hpp"
#include "ReferenceCounter.hpp"
#include "BitField.hpp"
/** creates a ObjectDescriptorESInfo object
	* @param encodeddecconf is only shallow copied and
	* freed in the destructor
	* @param avgBW and @param maxBW specify the bandwidth of the given ES
	* and are in Bits/sec !
*/
ObjectDescriptorESInfo::ObjectDescriptorESInfo(u32 streamId, u32 streamtype, 
			 ContainerInfo * vob, u32 timeIncrement,
			 u8 * encodeddecconf, u32 objtype, u32 bufsize,
			 u32 avgBW, u32 maxBW, u32 timeScale,
			 u64 duration, u32 size,  bool complete, 
			 CodecID codec_id, float quality):ESInfo(streamId,
						streamtype,vob,timeIncrement,encodeddecconf,
						objtype,bufsize,avgBW,maxBW,
						timeScale,duration,size,complete,codec_id,quality)
{
	generateDescriptionText();
};

ObjectDescriptorESInfo::~ObjectDescriptorESInfo()
{
	
};

u32 ObjectDescriptorESInfo::getHandlerType() const
{
	return MP4ObjectDescriptorHandlerType;
};

bool ObjectDescriptorESInfo::isVisualStream() const
{
	return false;
};
bool ObjectDescriptorESInfo::isAudioStream() const
{
	return false;
};
bool ObjectDescriptorESInfo::isODStream() const
{
	return true;
};
bool ObjectDescriptorESInfo::isSceneDescriptionStream() const
{
	return false;
};

/** creates a deep copy of the ES */
ESInfo *ObjectDescriptorESInfo::clone(ContainerInfo * mp4)
{
	dprintf_full("ESInfo::clone id %llu\n",streamId);
	u8 *decConf = new u8[strlen((char *) encodedDecoderConfig) + 1];
	strcpy((char *) decConf, (const char *) encodedDecoderConfig);
	ESInfo *es = new ObjectDescriptorESInfo(streamId, streamType, mp4, vopTimeIncrement, decConf, objectType,
													bufferSize, avgBandwidth, maxBandwidth,
													mediaTimeScale, duration, mediaSize,
													complete, codec_id, this->quality);
	es->setInput(input);
	if(!existingClones) {
		existingClones=new ReferenceCounter();
		existingClones->increase(); // for the orig es
	}
	existingClones->increase(); // for the first clone
	es->setCloneCounter(existingClones);
	BitField* fs=getFrameStatistic();
	if(fs) {
		es->setFrameStatistic(fs->createWithNewSize(fs->getSize(),false));
	}
	return es;
};

void ObjectDescriptorESInfo::generateDescriptionText()
{
	snprintf(description,MAX_STR_LEN,"Unknown Streamtype");
}
