 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	HashTable.hpp                                          *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/
#ifndef VITOOKI_PS_POINTERHASHTABLE_HPP
#define VITOOKI_PS_POINTERHASHTABLE_HPP
#include <assert.h>
/**
 *
 * A HashTable for pointers only. Hashing is done with a simple MODULO
 * operation. If a collision occurs, the element is inserted into an
 * overflow list but only if it has the same key than all the other 
 * elements at that position.
 *
 *@author Peter Schojer
 *
 *@version $Id: PointerHashTable.hpp,v 1.8 2006/01/20 15:37:17 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
template<class T> class PointerHashTable 
{
	/** always empty list */
	std::list<T> invalid;
	std::list<T>* data;
	/** the maximum size of the hashtable */
	int size;
	unsigned int* keys;

public:
	PointerHashTable(int siz) {
		size=siz;
		data=new std::list<T>[size];
		keys=new unsigned int[size];
	};
	~PointerHashTable() {
		for(unsigned int i=0;i<size;i++) {
			freeElementsForPos(i);
		}
		delete[] data; delete[] keys;
	};

	void put(T& elem,unsigned int key) {
		unsigned int pos=key%size;
		assert(elem != NULL);
		// if we receive another element that maps to the same pos but has
		// a different key then we delete it!
		if(!data[pos].empty() && keys[pos]!=key)
			freeElementsForKey(pos);
		
		data[pos].push_back(elem);
	};

	std::list<T>& get(unsigned int key) {
		unsigned int pos=key%size;
		if(keys[pos]==key)
			return data[pos];
		return invalid;
	};
	
	void unlinkElementsForKey(const unsigned int key) {
		unsigned int pos=key%size;
		if(!data[pos].empty()) {
			list<T>::iterator li;
			while(!data[pos].empty()) {
				data[pos].pop_front();
			}
		}
	}
	void freeElementsForKey(const unsigned int key) {
		unsigned int pos=key%size;
		if(keys[pos]==key)
			freeElementsForPos(pos);
	}

protected:
	/** calls delete for every element assigned to @param pos */
	void freeElementsForPos(const unsigned int p) {
		unsigned int pos=p%size;
		if(!data[pos].empty()) {
			list<T>::iterator li;
			while(!data[pos].empty()) {
				li=data[pos].begin();
				if(*li)
					delete (*li);
				data[pos].pop_front();
			}
		}
	};

};
#endif
