 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	ReferenceCounter.cpp                                          *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "ReferenceCounter.hpp"
#include "global.hpp"
#include "debug.hpp"
#include <stdio.h>

ReferenceCounter::ReferenceCounter()
{
	usageCnt=0; 
	mutex.initialize("ReferenceCounterMutex");
	valid=true;
};

ReferenceCounter::~ReferenceCounter()
{
	//dprintf_full("ReferenceCounter::~ReferenceCounter()\n");
	mutex.lock();
	mutex.destroy();
};

void ReferenceCounter::reset()
{
  lock();
  usageCnt = 0;
  valid = true;
  unlock();
}

/** returns false only when the reference counter was deactivated. */
bool ReferenceCounter::increase()
{
	bool ret=false;
	lock();
	if(valid) {
		usageCnt++;
		ret=true;
	}
	unlock();
	return ret;
};

/** returns false only when release is more often called than use */
bool ReferenceCounter::decrease()
{
	bool ret=false;
	
	lock();
	if(valid && usageCnt>0) {
		usageCnt--;
		ret=true;
	}
	unlock();
	return ret;
};

/** deactivates the RefCounter. Useful, when preparing to destroy the refCounter. */
void ReferenceCounter::deactivate() 
{
	lock();
	valid=false; usageCnt=0;
	unlock();
};

unsigned int ReferenceCounter::getUsage() const
{
	unsigned int res=0;
	lock();
	res=usageCnt;
	unlock();
	return res;
};

void ReferenceCounter::lock() const
{
	mutex.lock();
};

void ReferenceCounter::unlock() const
{
	mutex.release();
};

bool ReferenceCounter::deactivateWhenUsage(unsigned int i)
{
	lock();
	if(usageCnt<=i) {
		valid=false;
		usageCnt=0;
	}
	unlock();
	return !valid;
};

bool ReferenceCounter::decreaseAndDeactivateWhenUsage(unsigned int i)
{
	lock();
	if(valid && usageCnt>0) {
		usageCnt--;
	}
	if(usageCnt<=i) {
		valid=false;
	}
	unlock();
	return !valid;
};
