 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	ReferenceCounter.hpp                                          *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PS_MPEG4ITEC_REFERENCECOUNTER_HPP_
#define _PS_MPEG4ITEC_REFERENCECOUNTER_HPP_

/** thread safe wrapper for increasing/decreasing integer values */
#include "VMutex.hpp"

/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: ReferenceCounter.hpp,v 1.5 2005/04/27 10:21:50 klschoef Exp $
 *
 *@short <short description>
 *
 */
class ReferenceCounter
{
public:
	ReferenceCounter();
	virtual ~ReferenceCounter();

        /** Set counter to 0 and activate it. */
        void reset();

	/** returns false only when the reference counter was deactivated. */
	bool increase();

	/** returns false only when release is more often called than use */
	bool decrease();

	/** deactivates the RefCounter. Useful, when preparing to destroy the refCounter. */
	void deactivate();

	/** deactivates the RefCounter. when usage is lower equal @param i 
		* returns true when refcounter was deactivated
		*/
	bool deactivateWhenUsage(unsigned int i);

	/** first decreases the refCounter then 
	  * deactivates the RefCounter. when usage is lower equal @param i 
		* @param return is true when refcounter was deactivated
		*/
	bool decreaseAndDeactivateWhenUsage(unsigned int i);

	unsigned int getUsage() const;

private:
	void lock() const;
	void unlock() const;

private:
	mutable VMutex mutex;
	unsigned int usageCnt;
	bool valid;
};
#endif
