 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	RingBuffer.hpp                                          *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/
#ifndef VITOOKI_PS_RINGBUFFER_HPP
#define VITOOKI_PS_RINGBUFFER_HPP

/**
 *
 * A RingBuffer for non pointers only
 *
 *@author Peter Schojer
 *
 *@version $Id: RingBuffer.hpp,v 1.4 2004/06/02 14:17:22 pschojer Exp $
 *
 *@short <short description>
 *
 */
template<class T> class RingBuffer{
	T invalid;
	T* data;
	int* keys;
	int startPos;
	int endPos;
	int size;
public:
	RingBuffer(int siz) {
		size=siz;
		data=new T[size];
		keys=new int[size];
		startPos=endPos=0;
		// set to NULL
		memset(data,0,size*sizeof(T));
	};

	/** doesn't free any memory. NEVER use this for pointers! Use PointerRingBuffer */
	~RingBuffer() {
		delete[] keys;delete[] data;
	};

	void put(T& elem,int key){
		if(startPos==0 && 0==endPos) {
			// this happens exactly only once! During the first insert.
			// in all other cases endPos will be always larger than startPos
			data[startPos]=elem;
			keys[startPos]=key;
			endPos++;
		}
		else {
			assert(endPos-startPos<=size); // > should never happen
			if(endPos-startPos==size) {
				// we are full, overwrite object
				startPos++;
			}
			// now insert the element	
			data[endPos%size]=elem;
			keys[endPos%size]=key;
			endPos++;
		}
	};
	
	/** returns the element and removes it from the RingBuffer.
	Searches form the oldest element to the newest. For optimal performance
	get Elements in the order you inserted them! If no element is found,
	@param found is set to false and an invalid element is returned */
	T get(int key, bool* found) {			
		assert(found);
		// endpos always points to the next insert pos
		for(int i=startPos;i<endPos;i++) {
			if(keys[i%size]==key) {
				keys[i%size]=-1;
				*found=true;
				return data[i%size];
			}
		}
		*found=false;
		return invalid;
	};
};	

#endif
