#ifndef WINCE

#ifndef VITOOKI_SIMPLE_STATISTICS_HPP
#define VITOOKI_SIMPLE_STATISTICS_HPP
#include "global.hpp"

class ESInfo;

#define VITOOKI_SIMPLESTATISTICS_MAXSECONDS 128
class SimpleStatistics
{
public:
	typedef struct StatEntry{
		// streamoutsec in absolute time!
		u32 second;
		u32 numPacketsIn;
		u32 numBytesIn;
		u32 numFramesIn;
		u32 numPacketsOut;
		u32 numBytesOut;
		u32 numFramesOut;
		u32 packetsDropped;
		u32 packetsLost;
		/** millisecs buffered at the client side */
		u32 msBuffered;
		u32 packetsBuffered;
		u32 bytesBuffered;
		u32 firstTS;
		u32 lastTS;
		u32 firstCTS;
		u32 lastCTS;

		StatEntry() {
			reset(0);
		}
		StatEntry(u32 sec) {
			reset(sec);
		};
		void reset(u32 sec) {
			second=sec;
			numPacketsIn=numBytesIn=numFramesIn=0;
			numPacketsOut=numBytesOut=numFramesOut=0;
			msBuffered=packetsBuffered=bytesBuffered=0;
			firstTS=0xffffffff; lastTS=0;
			firstCTS=0xffffffff; lastCTS=0;
			packetsDropped=packetsLost=0;
		};
		/** adds two statentries, keeps the smaller second value! */
		void operator+=(const StatEntry &t) {
			if(second>t.second) {
				second=t.second;
			}
			numPacketsIn+= t.numPacketsIn;
			numBytesIn+= t.numBytesIn;
			numFramesIn+= t.numFramesIn;
			numPacketsOut+= t.numPacketsOut;
			numBytesOut+= t.numBytesOut;
			numFramesOut+= t.numFramesOut;
			packetsDropped+= t.packetsDropped;
			packetsLost+= t.packetsLost;
			msBuffered+= t.msBuffered;
			packetsBuffered+= t.packetsBuffered;
			bytesBuffered+= t.bytesBuffered;
			if(firstTS>t.firstTS)
				firstTS=t.firstTS;
			if(lastTS<t.lastTS)
				lastTS=t.lastTS;
			if(firstCTS>t.firstCTS)
				firstCTS=t.firstCTS;
			if(lastCTS<t.lastCTS)
				lastCTS=t.lastCTS;	
		};
		/** creates a StatEntry from a single line of text */
		static StatEntry *createFromString(const char* tmp) {
			StatEntry* s=new StatEntry();
			int res=sscanf(tmp,"%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u",
				&s->second,&s->numPacketsIn,&s->numBytesIn,&s->numFramesIn,
				&s->numPacketsOut,&s->numBytesOut,&s->numFramesOut,
				&s->packetsBuffered,&s->bytesBuffered,&s->msBuffered,
				&s->packetsDropped,&s->firstTS,&s->lastTS,&s->packetsLost,&s->firstCTS,&s->lastCTS);
			if(res!=16) {
				delete s;
				s=NULL;
			}
			return s;
		};
		/** creates a StatEntry from a single line of text, in case of an error, false is returned 
		  * and the content of @param s is not defined! 
		  */
		static bool createFromString(const char* tmp, StatEntry& s) {
			int res=sscanf(tmp,"%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u",
				&s.second,&s.numPacketsIn,&s.numBytesIn,&s.numFramesIn,
				&s.numPacketsOut,&s.numBytesOut,&s.numFramesOut,
				&s.packetsBuffered,&s.bytesBuffered,&s.msBuffered,
				&s.packetsDropped,&s.firstTS,&s.lastTS,&s.packetsLost,&s.firstCTS,&s.lastCTS);
			return (res==16);
		};
		/** converts to string without \n at end */
		bool convertToString(char* tmp,int stringSize) const{
			int res=snprintf(tmp,stringSize,"%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u\t%u",
			second,numPacketsIn,numBytesIn,numFramesIn,
			numPacketsOut,numBytesOut,numFramesOut,
			packetsBuffered,bytesBuffered,msBuffered,
			packetsDropped,firstTS,lastTS,packetsLost,firstCTS,lastCTS);
			return (res<stringSize);
		};
	} StatEntry;

	/** @param overwriteFileIfExists: overwrites filename if set to true, if set to false
	  * appends to fileName */
	SimpleStatistics(const char* saveToFileName,const ESInfo* es, bool serverMode,
		bool overwriteFileIfExists=true);
	~SimpleStatistics();

	void insertFrame(u32 cts, u32 bytes,u32 seqNrStart,u32 seqNrEnd);
	void insertPacket(u32 cts, u32 bytes, u32 seqNr, bool firstPacketOfFrame);
	void getFrame(u32 cts,u32 bytes,u32 seqNrStart,u32 seqNrEnd);
	/** whenever a frame is corrupted and its associated, already inserted,
	  * packets have to be deleted, call this method */
	void dropInsertedPacket(u32 bytes, u32 seqNr, u32 cts,bool noOtherPacketsWereDroppedForThisFrame);

	/** if a packet was lost, i.e the serevr sent it and it never arrived at the client,, call this method */
	void notifyLostPacket(u32 seqNr);
	/** if @param clearBuffers is set to true, the client has flushed its buffers,
	  * thus resetting statistics. YOU MUST call play at least once before insert or get! */
	void play(u32 cts,bool clearBuffers=true);
	void pause();
	void teardown();
	u32 getStartUpDelayInMS() const { return startUpDelayInMS;};
	/** max value for n is 30, values lower equal than 1 check for the current second */
	u32 getNumBytesOutLastNSeconds(int n);
	/** max value for n is 30, values lower equal than 1 check for the current second.
	  * The second value of the returned StatEntry contains the first second of the range */
	StatEntry getSumOfLastNSeconds(int n);
	static u32 getCurrentSecond();
	static u32 getCurrentMilliSecond();
	static s32 calcDiffInMS(const struct timeval &oldTime, const struct timeval &newTime);

	/** scans the directory @param dir for all log files that match the key(s) and combines
	  * these files into one global statisticfile @param outFile due to the time information.
	  * For example, your directory contains 500 logs that are named log_<number>_server.out
	  * and log_<number>_player.in. If you want to combine the server files, pass "log_"
	  * as first key and "_server.out" as second key. At least one key must be defined!
	  * @param return is false, if no key is specified, if no file was found that matches
	  * the key(s), or if the output file could not be opened for writing
	  */
	static bool createGlobalLog(const char* outFile, const char* srcDir, const char* key1, 
		const char* key2=NULL, const char* key3=NULL);

	struct timeval* getstartTime()
	{
		return &startTime;
	}
	
	char* getfileName()
	{
		return fileName;
	}
	
protected:
	/** called whenever currentPos reaches the last position in stats
	  * or when teardown is invoked */
	void saveToFile(int startPos,int endPos);
	static void saveOneStatEntry(FILE* fp, const StatEntry& stat);
	void detectNewSecond();
	

	/** currentPos in stats */
	int currentPos;
	/** the last not saved position */
	int lastNotSavedPos;
	/** contains the time of the first play */
	struct timeval startTime;
	/** contains the cts value during the last play */
	u32 lastCTS;
	/** highest CTS value seen in a packet */
	u32 highestCTSSeen;
	/** contains the last getFrame CTS */
	u32 lastGetCTS;
	/** contains the time of the last pause */
	struct timeval pauseTime;
	/** measures the time from first play to first insert */
	u32 startUpDelayInMS;
	const ESInfo* es;
	u32 lastSecSaved;
	StatEntry stats[VITOOKI_SIMPLESTATISTICS_MAXSECONDS];
	char fileName[MAX_STR_LEN];
	/** stores if play was at least called once */
	bool playCalled;
	bool serverMode;
	bool teardownCalled;
};
#endif

#endif //wince
