/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	Statistics.cpp                                                *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "Statistics.hpp"
#include "ESInfo.hpp"



/*****************************************************************/
Statistics::Statistics(ESInfo *esi, bool channelIsWriter) {
  es = esi;


  //GENERIC STATISTICS
  first_packet_dtime=0;
  last_packet_dtime=0;
  first_packet_ts=0;
  highest_packet_ts=0;
  playout_sec=0.0;
  memset(playsec_stat,0,sizeof(playsec_stat));


  //NETWORK RELATED STUFF
  streamout_sec=0;
  streamout_float_sec=0.0;
  avg_interarrival_time=0;
  avg_rtt=0;
  stream_bw=0;
  memset(streamsec_stat,0,sizeof(streamsec_stat));

  //BUFFER RELATED STUFF
  client_preQ_max_size=0;
  buf_ahead_sec=0.0;
  prefetched_secs=0.0;
  stillToPrefetchSecs=0.0;

  //ADAPTATION RELATED STUFF
  adapt_secs=1.0;
  adapt_rate=100;

  /* counts the number of opened Stat instances
   * this is used for statistics output filenames
   */
  static int globalStatsCounter;
  char *dummy = new char[MAX_STR_LEN];
  char *path = new char[MAX_STR_LEN];
  int tmp=globalStatsCounter++;
#define STAT_FILE "rtp_stat"

#ifdef VITOOKI_DEBUG

  sprintf(path,".");
  if (!channelIsWriter) {
#ifndef WIN32
		//  sprintf(path,"../muviserver/");
#endif
    sprintf(dummy,"%s/%s-playout-client-%i",path,STAT_FILE,tmp);
  } else
    sprintf(dummy,"%s/%s-playout-server-%i",path,STAT_FILE,tmp);
  if (!(playout_stat_fp=fopen(dummy,"w+")) ) {
#ifndef WINCE
		perror("Statistics-playout file");
#endif
		dprintf_full("Statistics: playout file was %s\n",dummy);
		assert(playout_stat_fp);
	}

  sprintf(path,".");
  if (!channelIsWriter) {
#ifndef WIN32
		//  sprintf(path,"../muviserver/");
#endif
    sprintf(dummy,"%s/%s-streamout-client-%i",path,STAT_FILE,tmp);
  } else
    sprintf(dummy,"%s/%s-streamout-server-%i",path,STAT_FILE,tmp);

  if (!(streamout_stat_fp=fopen(dummy,"w+")) ) {
#ifndef WINCE
		perror("Statistics-streamout file");
#endif
		dprintf_full("Statistics: streamout file was %s\n",dummy);
		assert(streamout_stat_fp);
	}

  dprintf_full("Statistics::Statistics created stat files number %i for file %s\n",tmp,dummy);

#endif

  delete [] dummy;
  delete [] path;
}

/*****************************************************************/
Statistics::~Statistics() {
  fclose(streamout_stat_fp);
  fclose(playout_stat_fp);
}

/*****************************************************************/
void Statistics::setESInfo(ESInfo *new_es) {
	es = new_es;
	streamsec_stat[getStreamoutSec() % RTP_STAT_SECS].stream_switched = true;
	playsec_stat[(int)floor((float)highest_packet_ts / (float)es->getMediaTimeScale()) % RTP_STAT_SECS].stream_switched = true;
}


/*****************************************************************/
int Statistics::setAvgRTT(int curr_rtt) {
  if (getAvgRTT() == 0)
    avg_rtt = curr_rtt;
  else
    avg_rtt = (int) ((float)getAvgRTT() 
										 * RTP_AVG_RTT_ALPHA + curr_rtt
										 * (1-RTP_AVG_RTT_ALPHA) );

  return avg_rtt;
}

/*****************************************************************/
int Statistics::setAvgInterarrivalTime(int msecs) {
  if (getAvgInterarrivalTime() == 0)
    avg_interarrival_time = msecs;
  else
    avg_interarrival_time = (int) ((float)getAvgInterarrivalTime() 
																	 * RTP_AVG_RTT_ALPHA + msecs
																	 * (1-RTP_AVG_RTT_ALPHA) );
  return avg_interarrival_time;
}


/*****************************************************************/
long Statistics::getBWfromStreamoutSec(u32 sec) {
  if ((sec > getStreamoutFullSec())
      || ((signed)sec < MAX(0,(signed)getStreamoutFullSec() - RTP_STAT_SECS)) ) {
    return -1;
  }

  //dprintf_full("Statistics::getBWfromStreamoutSec %i  data|rtx|nacked   %6i:%6i:%6i\n",sec,
  //getStreamoutSecDataBW(sec), getStreamoutSecRtxBW(sec), getStreamoutSecNackedBW(sec));
  return getStreamoutSecDataBW(sec) + getStreamoutSecRtxBW(sec) - getStreamoutSecNackedBW(sec);
}

/*****************************************************************/
long Statistics::getBWfromPlayoutSec(u32 sec) {
  if ((sec > (unsigned)ceil(getPlayoutSec()))
      || ((signed)sec < MAX(0,(signed)ceil(getPlayoutSec()) - RTP_STAT_SECS)) ) {
    return -1;
  }

  //dprintf_full("Statistics::getBWfromStreamoutSec %i  data|rtx|nacked   %6i:%6i:%6i\n",sec,
  //getStreamoutSecDataBW(sec), getStreamoutSecRtxBW(sec), getStreamoutSecNackedBW(sec));
  return getPlayoutSecDataBW(sec) + getPlayoutSecRtxBW(sec) - getPlayoutSecNackedBW(sec);
}


/*****************************************************************/
/** @returns the number of stored BW secs **/
u32 Statistics::getNumStreamoutSecs() {
  return RTP_STAT_SECS; 
}


/*****************************************************************/
bool Statistics::incPlayoutSecPSNR(int sec, double psnr) { 
  playsec_stat[sec	% RTP_STAT_SECS].sum_psnr += psnr;
  playsec_stat[sec	% RTP_STAT_SECS].num_psnr++; 
  return true;
};


/*****************************************************************/
double Statistics::getPlayoutSecPSNR(int sec) {

  return playsec_stat[sec % RTP_STAT_SECS].sum_psnr / 
    playsec_stat[sec % RTP_STAT_SECS].num_psnr; 

};


/*****************************************************************/
float Statistics::getStreamoutSecLossPercent(int sec) { 
	float sentBW = (float) (getStreamoutSecDataBW(sec) 
													+ getStreamoutSecRtxBW(sec));
	if (sentBW > 0)
		return getStreamoutSecNackedBW(sec) / (sentBW / 100);
	else
		return -1.0;
}

/***************************************************************/
bool Statistics::checkForNewPlayoutSec(int this_frame_sec, int highest_ts_sec) {

  if (highest_ts_sec < 0) //initial reader value is -1  !!!
    return true;

  //new client buf second
  if ( highest_ts_sec < this_frame_sec ) {
    dprintf_full("Statistics(%p)::checkForNewPlayoutSec: this %i highest %i\n",
								 this,highest_ts_sec,this_frame_sec);

    if (this_frame_sec - highest_ts_sec > RTP_STAT_SECS) {
      memset(playsec_stat,0,sizeof(playsec_stat));
      highest_ts_sec = this_frame_sec;
    } else
      while (highest_ts_sec < this_frame_sec ) {
				highest_ts_sec++;
				memset(&playsec_stat[highest_ts_sec % RTP_STAT_SECS],0,sizeof(OnePlayoutSecStat));
      }
		
#ifdef VITOOKI_DEBUG 
    if (this_frame_sec >= 1) {
      dprintf_full("Statistics(%p)::checkForNewPlayoutSec: netStat highest_ts_sec %i (last PSNR %3.3f): [data|rtx|pkts|fps]", this,	highest_ts_sec,	getPlayoutSecPSNR(this_frame_sec - 1));
      for (int i=this_frame_sec - 1;
					 i >= MAX(0,this_frame_sec +1 - RTP_STAT_SECS);
					 i--)
				printf("#[%2i: %3i|%2i|%2i|%2i]",i,
							 getPlayoutSecDataBW(i)/128,
							 getPlayoutSecRtxBW(i)/128,
							 getPlayoutSecDataPkts(i),
							 getPlayoutSecNetFPS(i));
      printf("\n");
    }
#endif
    writePlayoutSecStats(this_frame_sec);
		
    return true;
  }
  return false;
}


/**************************************************************************/
void Statistics::writePlayoutSecStats(int this_frame_sec) {
  int oldSec = this_frame_sec - 2;

#ifdef VITOOKI_DEBUG
		
  if (this_frame_sec >= 2) {

    fprintf(playout_stat_fp,"%i\t",	oldSec);
    fprintf(playout_stat_fp,"%i\t",	getPlayoutSecNetFPS(oldSec));
    fprintf(playout_stat_fp,"%i\t",	getPlayoutSecFPS(oldSec)); //eg. after ESSync dropping
    fprintf(playout_stat_fp,"%i\t",	getPlayoutSecDataBW(oldSec)/128);
    fprintf(playout_stat_fp,"%i\t",	getPlayoutSecRtxBW(oldSec)/128);
    fprintf(playout_stat_fp,"%i\t",	getPlayoutSecNackedBW(oldSec)/128);
    fprintf(playout_stat_fp,"%i\t",	(getPlayoutSecDataBW(oldSec)
																		 + getPlayoutSecRtxBW(oldSec)
																		 - getPlayoutSecNackedBW(oldSec)) /128);
    fprintf(playout_stat_fp,"%3.3f\t",	getPlayoutSecPSNR(oldSec));


		fprintf(playout_stat_fp,"%i\t",	getPlayoutSecIFrame(oldSec));
		fprintf(playout_stat_fp,"%i\t",	getPlayoutSecPFrame(oldSec));
		fprintf(playout_stat_fp,"%i\t",	getPlayoutSecBFrame(oldSec));
		fprintf(playout_stat_fp,"%i\t", playsec_stat[oldSec % RTP_STAT_SECS].stream_switched);
		fprintf(playout_stat_fp,"%i\t",playsec_stat[oldSec % RTP_STAT_SECS].stream_bw/128);

    fprintf(playout_stat_fp,"\n");
    fflush(playout_stat_fp);	
  }
#endif
}


/****************************************************************/
bool Statistics::checkForNewStreamoutSec(double now_dtime) {
  double this_sec;
  u32 old_streamout_sec;
  int i=0;

  if (getFirstPacketTime() == 0) {
    dprintf_err("Statistics::checkForNewStreamoutSec no firstPacketTime for calculations....\n");
    return false;
  }

  if (now_dtime < 0.0) { //no time given, so get real now time...
    struct  timeval now_tv;
    gettimeofday(&now_tv,NULL);
    now_dtime = now_tv.tv_sec + now_tv.tv_usec/1000000.0;
  }

  this_sec = now_dtime - getFirstPacketTime();
  setStreamoutFloatSec(this_sec);

  //negative playout sec for initial prefetching!
  setPlayoutSec(getStreamoutFloatSec() - getPrefetchedSecs());

  if (getStreamoutFloatSec() >= getPrefetchedSecs()) {
    setBufAheadSec(((double)(getHighestPacketTS()-getFirstPacketTS())/(double)es->getMediaTimeScale())
		   - (getStreamoutFloatSec() - getPrefetchedSecs())
		   - (float)getAvgRTT()/1000.0); //is always zero on client side!
    // conservative approach: we use RTT instead of latency, since RTT/2 might be wrong on
    // unevenly loaded links
  } else
    setBufAheadSec( ((double)(getHighestPacketTS()-getFirstPacketTS()) / (double)es->getMediaTimeScale()) );

	
  old_streamout_sec = getStreamoutFullSec();
  setStreamoutFullSec((int)float(this_sec));

  if (old_streamout_sec < getStreamoutFullSec()) { //NEW STREAMOUT SEC
    if (getStreamoutFullSec() - old_streamout_sec > RTP_STAT_SECS) {
			//eg. paused for over 15 secs
			old_streamout_sec = getStreamoutFullSec();
			memset(&streamsec_stat, 0, sizeof(OneStreamoutSecStat)*RTP_STAT_SECS);
		} else {	
			//reset all skipped-over secs
			while(old_streamout_sec < getStreamoutFullSec()) {
				old_streamout_sec++; //advance to next skipped one
				memset(&streamsec_stat[old_streamout_sec	% RTP_STAT_SECS], 0, sizeof(OneStreamoutSecStat));
			}
		}

		streamsec_stat[(getStreamoutFullSec()-1) % RTP_STAT_SECS].stream_bw = getStreamBW();
		if (getPlayoutSec() > 1)
			playsec_stat[((int)floor(getPlayoutSec()-1)) % RTP_STAT_SECS].stream_bw = getStreamBW();

    // calc client buffer fill rate
    //fake float part by non-exact percentage of according full sec
    int playout_floor_sec = MAX(0,(int)floor(getPlayoutSec()));
    //		int buf_ahead_floor_sec = (int)floor(getBufAheadSec());
    setStreamoutSecClientBufFill(getStreamoutFullSec()-1, 0);
    setStreamoutSecBufAhead(getStreamoutFullSec()-1, getBufAheadSec());
    setStreamoutSecPlayoutSec(getStreamoutFullSec()-1, getPlayoutSec());

    // left-over buffers from actual playout sec
    double playout_fract = (double)playout_floor_sec + 1 - MAX(0.0, getPlayoutSec());
    //ignore case that buf_ahead could be < 1.0, since the rest wont be in our playsec_stat anyway...
    incStreamoutSecClientBufFill(getStreamoutFullSec()-1,
																 (int)( (double)(getPlayoutSecDataBW(playout_floor_sec)
																								 + getPlayoutSecRtxBW(playout_floor_sec) )
																				* playout_fract ) );

    int buf_ahead_floor_sec_left = (int)floor(getBufAheadSec() - playout_fract);
    for (i=1; i <= buf_ahead_floor_sec_left; i++)
      incStreamoutSecClientBufFill(getStreamoutFullSec()-1, 
																	 getPlayoutSecDataBW(playout_floor_sec + i) 
																	 + getPlayoutSecRtxBW(playout_floor_sec + i) );
		
    // and now the tail of the last sec
    playout_fract = getBufAheadSec() - playout_fract - buf_ahead_floor_sec_left;
    if (playout_fract > 0) //still more ahead
      incStreamoutSecClientBufFill(getStreamoutFullSec()-1,
																	 (int)((double) (getPlayoutSecDataBW(
																																			 (int)floor(MAX(0,(int)getPlayoutSec()) 
																																									+ getBufAheadSec()))
																									 + getPlayoutSecRtxBW((int)floor(MAX(0,(int)getPlayoutSec()) 
																																									 + getBufAheadSec())) )
																				 * playout_fract ) );

#ifdef VITOOKI_DEBUG
    dprintf_full("Statistics(%p)::checkForNewStreamoutSec: netStat this_sec %6.3f"
								 " (playout %6.3f): [data|rtx|nacked|ClBuf]",
								 this, getStreamoutFloatSec(),getPlayoutSec());
    for (i=getStreamoutFullSec() - 1;
				 i >= MAX(0,(int)getStreamoutFullSec() + 1 - RTP_STAT_SECS);
				 i--)
      printf("#[%2i: %3i|%2i|%2i|%i]",i,
						 getStreamoutSecDataBW(i)/128,
						 getStreamoutSecRtxBW(i)/128,
						 getStreamoutSecNackedBW(i)/128,
						 getStreamoutSecClientBufFill(i)/128);
    printf("\n");
#endif

    return true;
  }

  return false;
}


/**************************************************************************/
void Statistics::writeStreamoutSecClientStats(long preQsize) {
  int oldSec = getStreamoutFullSec()-1;

#ifdef VITOOKI_DEBUG
  // client-side streamout sec
  fprintf(streamout_stat_fp,"%i\t",oldSec);
  fprintf(streamout_stat_fp,"%i\t",getStreamoutSecDataBW(oldSec)/128);
  fprintf(streamout_stat_fp,"%i\t",getStreamoutSecRtxBW(oldSec)/128);
  fprintf(streamout_stat_fp,"%i\t",(getStreamoutSecDataBW(oldSec)
																		+ getStreamoutSecRtxBW(oldSec))/128);
  fprintf(streamout_stat_fp,"%5.3f\t",getPlayoutSec());
  fprintf(streamout_stat_fp,"%5.3f\t",getBufAheadSec());
  fprintf(streamout_stat_fp,"%i\t",getStreamoutSecClientBufFill(oldSec)/128);
  fprintf(streamout_stat_fp,"%li\t",preQsize/128);
  fprintf(streamout_stat_fp,"\n");
  fflush(streamout_stat_fp);
#endif

}


/**************************************************************************/
void Statistics::writeStreamoutSecServerStats() {
  int oldSec = getStreamoutFullSec()-2;


#ifdef VITOOKI_DEBUG

  //sender-side streamout sec
  fprintf(streamout_stat_fp,"%i\t",oldSec);
  fprintf(streamout_stat_fp,"%i\t",getStreamoutSecNetBW(oldSec)/128);
  fprintf(streamout_stat_fp,"%i\t",getStreamoutSecDataBW(oldSec)/128);
  fprintf(streamout_stat_fp,"%i\t",getStreamoutSecRtxBW(oldSec)/128);
  fprintf(streamout_stat_fp,"%i\t",getStreamoutSecNackedBW(oldSec)/128);
  fprintf(streamout_stat_fp,"%i\t",(getStreamoutSecDataBW(oldSec)
																		+ getStreamoutSecRtxBW(oldSec)
																		- getStreamoutSecNackedBW(oldSec) ) /128);
  fprintf(streamout_stat_fp,"%5.3f\t",getStreamoutSecNackedBW(oldSec) /      // calc pkt_loss
					(float)((getStreamoutSecDataBW(oldSec) + getStreamoutSecRtxBW(oldSec))/100));
  fprintf(streamout_stat_fp,"%5.3f\t",getPlayoutSec());
  fprintf(streamout_stat_fp,"%5.3f\t",getBufAheadSec());
  fprintf(streamout_stat_fp,"%i\t",getAdaptRate());
  fprintf(streamout_stat_fp,"%5.0f\t",getAdaptSecs()*100);
  fprintf(streamout_stat_fp,"%i\t",getStreamoutSecClientBufFill(oldSec)/128);
  fprintf(streamout_stat_fp,"%i\t",streamsec_stat[oldSec % RTP_STAT_SECS].stream_switched);
  fprintf(streamout_stat_fp,"%i\t",streamsec_stat[oldSec % RTP_STAT_SECS].stream_bw/128);
  fprintf(streamout_stat_fp,"\n");
  fflush(streamout_stat_fp);

#endif	
}

