 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	UncompressedVideoFrame.hpp                                    *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef __UncompressedVideoFrame_hpp__
#define __UncompressedVideoFrame_hpp__

#include "VideoFrame.hpp"


/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: UncompressedVideoFrame.hpp,v 1.7 2006/06/22 13:31:17 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class UncompressedVideoFrame : public VideoFrame {

public:

	UncompressedVideoFrame(FrameType t, int width, int height);

	virtual ~UncompressedVideoFrame();

	static Frame::FrameType detectFrameType(AU* accessUnit);

	/** Determines the correctness of the buffer
	  * and extracts the frametype of the frame
		* On any problem: FrameType ERROR_VOP is returned
	*/
	Frame::FrameType detectFrameType();
	
	/**
		 Colorspace enumeration of colorspaces available. This is an excerpt
		 of the colorspaces available in XviD (xvid.h) and/or FFmpeg (avcodec.h).
		 so those colorSpaces are also avail in MP4Decoder
		 - ColorSpaceDefault: Default is set to YUV420P.
		 - ColorSpaceYV12: YV12 format, U and V swapped here!
		 - ColorSpaceI420: I420 format, 
		 - ColorSpaceRGB24: Standard RGB colorspace 24 bit, but 4 byte
		 - ColorSpaceRGB32: RGB Colorspace with alpha channel, so 4 byte
		 - ColorSpaceRGB565: RGB with 16 bit with 2 byte
	*/
	typedef enum ColorSpace {
		ColorSpaceDefault = 4,
		ColorSpaceRGB24 = 0, 
		ColorSpaceYV12 = 1, 
		ColorSpaceI420 = 4, 
		ColorSpaceRGB32 = 1000,
		ColorSpaceRGB565, // implemented in interface to FFMPEG only
		ColorSpaceGRAY8,	// implemented in interface to FFMPEG only
		ColorSpaceUNKNOWN
	} ColorSpace;


	/**
	 * @returns eg. for ColorSpaceGRAY8 the value 8, RGB32 and RGB24 will be 32,
	 * whereas RGB565 will be 16
	 */
	static int mapColorSpaceToBitDepth(ColorSpace colorspace);

	void setColorSpace(ColorSpace c) { colorSpace = c; };

	ColorSpace getColorSpace() { return colorSpace; };

	/* should transcode a single frame to another colorspace 
	* @returns a newly allocated frame, dont forget to delete the old one! */
	UncompressedVideoFrame *convertToColorSpace(ColorSpace cs);

  /** Returns the buffer with Y component of *YUVI420* of Uncompressed Video
    
    @param buf - This is the buffer which will ultimately contain the Y component
    @return Size of the buffer
  */
  int getYBuffer(u8 **buf);
  /** Returns the buffer with U component of *YUVI420* of Uncompressed Video
    
    @param buf - This is the buffer which will ultimately contain the Y component
    @return Size of the buffer
  */
  int getUBuffer(u8 **buf);
  /** Returns the buffer with V component of *YUVI420* of Uncompressed Video
    
    @param buf - This is the buffer which will ultimately contain the Y component
    @return Size of the buffer
  */
  int getVBuffer(u8 **buf);
  
  double computePSNRy(UncompressedVideoFrame * f);
  double computePSNRu(UncompressedVideoFrame * f);
  double computePSNRv(UncompressedVideoFrame * f);
  double computePSNR(UncompressedVideoFrame * f);

  double getPSNR() { return this->psnr; };

	
protected:
  double computePSNRwithOffset(UncompressedVideoFrame * f, long offset);

  double psnr;

	ColorSpace colorSpace;

};

#endif				// __UncompressedVideoFrame_hpp__
