 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: VCondition.cpp                                                *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/


#include "VCondition.hpp"


/***********************************************************************
Constructor-1
***********************************************************************/
VCondition::VCondition()
{
#ifdef WINCE
#ifndef DISABLE_CONDITIONS
	cond = CreateEvent(NULL, 
						true, //manual control
						true, //initial state is signaled
						NULL);
#endif
#endif
}


/***********************************************************************
Destructor
***********************************************************************/
VCondition::~VCondition()
{
	destroy();
}


/***********************************************************************
Initialize condition variable
***********************************************************************/
void VCondition::initialize()
{
#ifndef DISABLE_CONDITIONS
#ifndef WINCE
	pthread_cond_init(&cond, NULL);
#else
	ResetEvent(cond); //set event state to nonsignaled
#endif
#endif
}


/***********************************************************************
Send a signal to all waiting threads
***********************************************************************/
void VCondition::signal()
{
#ifndef DISABLE_CONDITIONS
#ifndef WINCE
	pthread_cond_signal(&cond);
#else
	SetEvent(cond); //set event state to signaled
#endif
#endif
}

/***********************************************************************
Broadcast signal to all waiting threads
***********************************************************************/
void VCondition::broadcast()
{
#ifndef DISABLE_CONDITIONS
#ifndef WINCE
	pthread_cond_broadcast(&cond);
#endif
#endif
}


/***********************************************************************
Wait for condition variable
***********************************************************************/
void VCondition::condWait()
{
#ifndef DISABLE_CONDITIONS
#ifndef WINCE
	pthread_cond_wait(&cond, NULL);
#else
	//wait for condition variable to be signaled
	WaitForSingleObject(cond, INFINITE);
#endif
#endif
}

/***********************************************************************
Wait for condition variable (2nd version)
***********************************************************************/
#ifndef WINCE
void VCondition::condWait(pthread_mutex_t *mutex)
{
#ifndef DISABLE_CONDITIONS
	pthread_cond_wait(&cond, mutex);
#endif
}
#else
void VCondition::condWait(HANDLE mutex)
{
	condWait();
}
#endif


/***********************************************************************
Destroy lock object
***********************************************************************/
void VCondition::destroy()
{
#ifndef DISABLE_CONDITIONS
#ifndef WINCE
	//pthread_cond_destroy(&cond);
#else
	if (cond != NULL) {
		CloseHandle(cond);
		cond = NULL;
	}
#endif
#endif
}


