 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: VMutex.cpp                                                    *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/


#include "VMutex.hpp"


/***********************************************************************
Constructor-1
***********************************************************************/
VMutex::VMutex()
{
	initialize(NULL);
}

/***********************************************************************
Constructor-2
***********************************************************************/
VMutex::VMutex(char *name)
{
	initialize(name);
}


/***********************************************************************
Destructor
***********************************************************************/
VMutex::~VMutex()
{
	destroy();
#ifdef WINCE
	delete mutexname;
#endif
}


/***********************************************************************
Initialize mutex variable
***********************************************************************/
void VMutex::initialize(char *name)
{
#ifndef DISABLE_MUTEXES
#ifndef WINCE
	if (pthread_mutex_init(&mutex,0) != 0) {
		dprintf_err("\nVMutex::initialize: cannot create mutex_ \n");
	}
#else
	mutexname = new char[MAX_PATH];
	sprintf(mutexname, "%s %d %d %d %d %d", name,	Random(), Random(), Random(), Random(), Random());

	mutex = CreateMutex(NULL, false, (LPCTSTR) mutexname);
	if (mutex == NULL) {
		dprintf_err("\nVMutex::initialize: cannot create mutex %s : %d\n", mutexname, mutex);
	}
#endif
#endif
}


/***********************************************************************
Wait for lock on mutex variable
***********************************************************************/
void VMutex::lock()
{
#ifndef DISABLE_MUTEXES
#ifndef WINCE
	pthread_mutex_lock(&mutex);
#else
	WaitForSingleObject(mutex, INFINITE);
#endif
#endif
}

/***********************************************************************
Try to get lock on mutex variable (if locked, return immediately)
***********************************************************************/
void VMutex::tryLock()
{
#ifndef DISABLE_MUTEXES
#ifndef WINCE
	pthread_mutex_trylock(&mutex);
#else
	WaitForSingleObject(mutex, 0);
#endif
#endif
}


/***********************************************************************
Release lock on mutex variable
***********************************************************************/
void VMutex::release()
{
#ifndef DISABLE_MUTEXES
#ifndef WINCE
	pthread_mutex_unlock(&mutex);
#else
	ReleaseMutex(mutex);
#endif
#endif
}


/***********************************************************************
Destroy lock object
***********************************************************************/
void VMutex::destroy()
{
#ifndef DISABLE_MUTEXES
#ifndef WINCE
	pthread_mutex_destroy(&mutex);
#else
	if (mutex != NULL) {
		CloseHandle(mutex);
		mutex = NULL;
	}
#endif
#endif
}

/***********************************************************************
Return internally used mutex variable
***********************************************************************/
#ifndef WINCE
//pthread_mutex_t &VMutex::getMutexObject()
pthread_mutex_t *VMutex::getMutexObject()
#else
HANDLE VMutex::getMutexObject()
#endif
{
#ifndef WINCE
	return &mutex;
#else
	return mutex;
#endif
}
