 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: VMutex.hpp                                                    *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/


/** @author Klaus Schoeffmann */


#ifndef VMUTEX_HPP
#define VMUTEX_HPP


#include "VThread.hpp"

#ifdef WINCE
#include <winbase.h>
#endif


class VMutex
{
public:

	/** Creates a mutex variable */
	VMutex();

	/** Creates and initializes mutex variable with name @param name */
	VMutex (char *name);

	~VMutex ();

	/** Initialize mutex variable */
	void initialize(char *name);

	/** Wait for lock on mutex variable */
	void lock();

	/** Try to get lock on mutex variable (if locked, return immediately) */
	void tryLock();

	/** Release lock on mutex variable */
	void release();

	/** Destroy lock object */
	void destroy();

	/** Return internally used mutex variable */
#ifndef WINCE
	pthread_mutex_t *getMutexObject();
#else
	HANDLE getMutexObject();
#endif


private:

#ifndef WINCE
	pthread_mutex_t mutex;
#else
	HANDLE mutex;
	char *mutexname;
#endif

};


#endif
