 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:  Vthread.cpp                                                  *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef QT_THREAD_SUPPORT



#include "VThread.hpp"


VThread::VThread() {
	is_running = is_finished = false;
}

VThread::~VThread() {
}

/* @PARAM time defines waittime
 */
bool VThread::wait( unsigned long time) {
	if (!is_running || is_finished)
		return true;
	else {
#ifndef WINCE
		dprintf_full(" VThread::wait for pthread join\n");
		bool ret=pthread_join(my_thread, NULL);
		//		dprintf_full(" VThread::wait for pthread join DONE\n");
#else
		while (is_running && !is_finished) Sleep(10);
		bool ret = true;
#endif
		is_finished = true;
		is_running = false;
		return ret;
	}
}

/* @returns if this thread's execution is done */
bool VThread::finished() const {
	return is_finished;
}

/* @returns if this thread's execution is still proceeding */
bool VThread::running() const {
	return is_running;
}

/* exits the currently running thread and informs all other waiting threads */
void VThread::exit() {
	if (is_running) {
		is_finished = true;
		is_running = false;
#ifndef WINCE
		pthread_exit(NULL);
#else
		if (my_thread != NULL) 
			ExitThread(*my_thread);
#endif
	}
}

/** set the scheduling strategy to round-robin */
int VThread::setRoundRobinScheduling(int val)
{
#ifndef WINCE
	struct sched_param sp;
	sp.sched_priority = val;
	return pthread_setschedparam(pthread_self(), SCHED_RR, &sp);
#else
	return 0;
#endif
}

/* starts the overwritten run() method
 */
void VThread::start() {
#ifndef WINCE
	dprintf_full(" VThread::start new thread\n");
#endif

	if (running()) {
		wait(0);
	}

#ifndef WINCE
	if (pthread_create(&my_thread, NULL, VThread::HelperFunction, this) != 0)
		perror("pthread start failed:");
#else
	CreateThread(NULL, 
				0, 
				(LPTHREAD_START_ROUTINE) VThread::HelperFunction, 
				this, 
				0, //CREATE_SUSPENDED, 
				my_thread);
	//SetThreadPriority(my_thread, THREAD_PRIORITY_TIME_CRITICAL);
#endif //wince
	is_running = true;
	is_finished = false;
}


#ifndef WINCE
void * VThread::HelperFunction (void *args)
#else
DWORD WINAPI VThread::HelperFunction (void *args)
#endif
 {
 VThread *pThis = static_cast<VThread *> (args);
 
 pThis->setRunning(true);
 pThis->setFinished(false);
 
 pThis->run();
 
 pThis->setRunning(false);
 pThis->setFinished(true);
 
#ifndef WINCE
 return NULL;
#else
 return 0;
#endif
 }

 
#endif  //enable pthread threads

 

