 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: VThread.hpp                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef VTHREAD_HPP
#define VTHREAD_HPP

#ifdef WINCE
#include <windef.h>
#endif

#include "global.hpp"
#include <assert.h>
#ifndef WINCE
#include <pthread.h>
#endif //wince

#ifdef QT_THREAD_SUPPORT
#include <qthread.h>
#define VThread QThread
#else  //enable pthread threads

/**
 *
 * OO-wrapper class to PThread lib
 *
 *@author Michael Kropfberger
 *
 *@version $Id: VThread.hpp,v 1.9 2005/08/10 07:04:12 klschoef Exp $
 *
 *@short <short description>
 *
 */
class VThread {

public:
    VThread();
		virtual ~VThread() ;

	/* @PARAM time defines waittime
	 */
    bool wait( unsigned long time = ULONG_MAX );

	/* starts the overwritten run() method
	 */
    void start();

	/* @returns if this thread's execution is done */
    bool finished() const;
	/* @returns if this thread's execution is still proceeding */
    bool running() const;

	/* exits the currently running thread and informs all other waiting threads */
   void exit();

   /* set the scheduling strategy to round-robin */
   int setRoundRobinScheduling(int val);
   
protected:
	/* overwrite this with the real work method!
	 * it is VERY important to finish this function with a call to this->exit()  !!!!
	 * if this->exit() is omitted, the wait() functionality may not work correctly! */
	virtual void run() = 0;


	/* internally needed, just ignore it! */
#ifndef WINCE
	static void * HelperFunction (void *args);
	pthread_t my_thread;
#else
	static DWORD WINAPI HelperFunction (void *args);
	LPDWORD my_thread;
#endif //wince 

	void setRunning(bool running) {this->is_running = running;};
	void setFinished(bool finished) {this->is_finished = finished;}; 


	bool is_running;
	bool is_finished;
};
#endif //enable qt_thread

#endif //vthread_hpp

