/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   VideoESInfo.hpp                                             *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/
 
#ifndef PS_VIDEOELEMENTARYSTREAMINFO_HPP
#define PS_VIDEOELEMENTARYSTREAMINFO_HPP
#include "ESInfo.hpp"

class VideoESInfo: public ESInfo
{
public:
	/** creates an VideoESInfo object
		* @param encodeddecconf is only shallow copied and
		* freed in the destructor
		* @param avgBW and @param maxBW specify the bandwidth of the given ES
		* and are in Bits/sec !
		* @param w specifies the width of the video
		* @param h specifies the height of the video
	*/
  VideoESInfo(u32 streamId, u32 streamtype, ContainerInfo * vob, u32 timeIncrement,
	      u8 * encodeddecconf, u32 objtype, u32 bufsize,
	      u32 avgBW, u32 maxBW, u32 timeScale,
	      u64 duration, u32 size,  
	      u32 w, u32 h, bool colorVideo=true,
	      bool complete=true, 
	      CodecID codec_id = CODEC_ID_MPEG4, float quality = 1.0,
	      float aspect_ratio = 352.0/288.0, bool staticFramePattern=true, u32 GOP_size = 50, 
	      u32 num_B_frames = 0, float avgBFrameSize=-1.0);

	virtual ~VideoESInfo(){};

	u32 getHandlerType() const;

	u32 getWidth() const;

	u32 getHeight() const;

	void setWidth(u32 uiWidth);

	void setHeight(u32 uiHeight);

  /** sets the number of frames per second, also adjusts the VopTimeIncrement! */
	void setFPS(float fps);

  /** returns the number of frames per second for this ES */
	float getFPS() const;

	bool getHasColor() const;
	
	void setHasColor(bool col);

	bool isVisualStream() const;

	bool isAudioStream() const;

	bool isODStream() const;

	bool isSceneDescriptionStream() const;

	void setEncodedDecoderConfig(u8 * enc);
	
	float getAspectRatio() const;
	
	void setAspectRatio(float AspectRatio);
	
	/** returns the number of frames one GOP consists off,
	  * including the I-frame. The returned value is only valid if hasStaticFramePattern
		* returns true!
		*/
	u32 getGOP_size() const;

	void setGOP_size(u32 gop_size);
	
	/** returns the number of B-frames between I-P or P-P.
	  * The returned value is only valid in hasStaticFramePattern
		* returns true! */
	u32 getNum_B_frames() const;

	void setNum_B_frames(u32 num_B_Frames);

	/** a frame pattern is static iff the GOP size is constant and the frame pattern
	  * constantly repeats itself (with the exception of the first GOP which is always
	  * different).
	  */
	bool hasStaticFramePattern() const;

	void setStaticFramePattern(bool pattern);

  /** creates a deep copy of the ES */
	ESInfo *clone(ContainerInfo * mp4);
	
	void setAvgBFrameSize(float avg);
	/** A Value between 0.0 and 1.0.
	  * Returns the total amount of the size of B-frames in relation to the total stream size */
	float getAvgBFrameSize() const;
	void generateDescriptionText();

protected:
	u32 width;
	u32 height;
	float aspect_ratio;
	bool hasColor;
	u32 GOP_size;
	u32 num_B_frames;
	bool staticFramePattern;
	/** returns the total amount of the size of B-frames DIV total stream size */
	float avgBFrameSize;
};

#endif
