 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	ColorReductionAdaptor.cpp                                     *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "ColorReductionAdaptor.hpp"

ColorReductionAdaptor::ColorReductionAdaptor(VideoESInfo * pESInfo, u32 key,
																						 u32 bitrate,MP4Decoder::meCoderType coderType)
{
    es = pESInfo;
    initialized = false;
    keyFrameInterval = key;
	targetBitrate=bitrate;
	strcpy(name,"ColorReductionAdaptor");
	codec=coderType;
	dprintf_full("ColorReductionAdaptor::ColorReductionAdaptor Codec %i\n",(int)coderType);
}

void ColorReductionAdaptor::initialize()
{
	if(initialized)
		return;

  addAdaptor(new MP4Decoder(es, false, UncompressedVideoFrame::ColorSpaceI420, codec));
  addAdaptor(new YUVColorReductionAdaptor(es));
	if(targetBitrate==0 || targetBitrate>es->getAvgBandwidth())
		targetBitrate=es->getAvgBandwidth() * 80 / 100;
  addAdaptor(new MP4Encoder(es, targetBitrate, (int)es->getFPS(), keyFrameInterval, 0 /*BVOPs*/, UncompressedVideoFrame::ColorSpaceI420, codec));
  initialized = true;
};

ColorReductionAdaptor::~ColorReductionAdaptor()
{
}

Adaptor *ColorReductionAdaptor::clone()
{
    return new ColorReductionAdaptor(es,keyFrameInterval,targetBitrate,codec);
}

