 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	ColorReductionAdaptor.hpp                                     *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef __COLORREDUCTION_ADAPTOR_HPP__
#define __COLORREDUCTION_ADAPTOR_HPP__

#include "VideoAdaptor.hpp"
#include "VideoAdaptorChain.hpp"
#include "MP4Encoder.hpp"
#include "MP4Decoder.hpp"
#include "YUVColorReductionAdaptor.hpp"
#include "VideoESInfo.hpp"
#include "Frame.hpp"

/**
 * Implementation of Adaptor that removes U and V component in a MPEG4 VES.
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: ColorReductionAdaptor.hpp,v 1.5 2004/05/27 17:56:53 pschojer Exp $
 *
 *@short <short description>
 *
 */
class ColorReductionAdaptor:public VideoAdaptorChain {
  public:
	/**
		Constructor.
		Average Bitrate of ESInfo object will be reduced by 20%, which 
		is approximately the proportion of color in the VES.

		@param pESInfo Pointer to ESInfo object.
	*/
    ColorReductionAdaptor(VideoESInfo * pESInfo, u32 keyFrameInterval = 30, u32 bitrate=0,
			MP4Decoder::meCoderType coderType = MP4Decoder::FFMPEG);

	/**
		Destructor.
	*/
    virtual ~ ColorReductionAdaptor();

    Adaptor *clone();

    void initialize();

  protected:
     u32 keyFrameInterval;
		 u32 targetBitrate;
		 MP4Decoder::meCoderType codec;
};

#endif				// __COLORREDUCTION_ADAPTOR_HPP__
