 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	CuttingAdaptor.cpp                                            *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "CuttingAdaptor.hpp"


CuttingAdaptor::CuttingAdaptor(ESInfo *esi, double start, double end) {

	es = esi;
	startSec = start;
	endSec = end;
	startTS = (u32)(startSec * es->getMediaTimeScale());
	endTS = (u32)(endSec * es->getMediaTimeScale());
	IVOPseen = false;
	strcpy(name,"CuttingAdaptor");
}


CuttingAdaptor::~CuttingAdaptor() {
}


void CuttingAdaptor::initialize() {
}


/***************************************************************/
list < Frame * > CuttingAdaptor::adapt(Frame * frm) {
	list < Frame * >tmp;
	u32 thisCTS;
	int thisSec;

	thisCTS = frm->getAU()->cts;
	thisSec = (int)floor((double)thisCTS / es->getMediaTimeScale());
	dprintf_full("CuttingAdaptor::adapt frame CTS %i, VOP increment %i, playoutSec: %i\n",
							 thisCTS, es->getVOPTimeIncrement(), thisSec );

	if ( (thisCTS >= startTS) && (thisCTS <= endTS) ) { //in range, so keep frame!
		if (!IVOPseen) {//search for first I-frame
			if ( ((es->isVisualStream() &&  (frm->getType() == Frame::I_VOP))) ||
					 es->isAudioStream() ||
					 (frm->getType() == Frame::UNKNOWN_VOP) ) 
				IVOPseen=true;
		}
		//standard forwarding adaptor behaviour
		if (IVOPseen)
			tmp.push_front(frm);
	}

	return tmp;
}
