 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	DataDump.cpp                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "DataDump.hpp"
#include "Frame.hpp"
#include "UncompressedVideoFrame.hpp"


DataDump::DataDump(ESInfo* esi, IO *outStream, bool autoFix) {

	es = esi;
	out = outStream;
	this->autoFix = autoFix;
	dummyFrame = NULL;
	numFrames = 0;
	lastAU.payload = NULL;
	strcpy(name,"DataDump");
}


DataDump::~DataDump() {
	delete out;
	if (dummyFrame) {
		dummyFrame->unsetAU();
		delete dummyFrame;
	}
}


void DataDump::initialize() {
}

void DataDump::setESInfo(ESInfo * esi) {
    es = esi;
}

ESInfo *DataDump::getESInfo() {
    return es;
}


/***************************************************************/
list < Frame * > DataDump::adapt(Frame * frm) {
	list < Frame * >tmp;
	u32 thisCTS;
	int thisSec;
	int done=0;

	//call open on first frame
	if (numFrames == 0)
		if (!out->open()) {
			dprintf_err("DataDump: FATAL: not writing, since another IO class is accessing file %s\n",out->getURL());
			//standard forwarding adaptor behaviour
			tmp.push_front(frm);
			return tmp;
		}

	UncompressedVideoFrame *thisFrame = (UncompressedVideoFrame*)frm;
	thisCTS = frm->getAU()->cts;
	thisSec = (int)floor((double)thisCTS / es->getMediaTimeScale());
	dprintf_full("DataDump::adapt frame %i CTS %i, VOP increment %i\n", numFrames, thisCTS,es->getVOPTimeIncrement());

	//FIXME: assert(isVideoFrame)

	//first frame, malloc payload, size is always the same for YUV frames!
	if (!lastAU.payload) {
		dprintf_full("DataDump::adapt creating initial buffers\n");
		lastPayload = (u8*)malloc(thisFrame->getAU()->size);
		lastAU.payload = lastPayload;
		lastAU.size = thisFrame->getAU()->size;
		dummyFrame = new UncompressedVideoFrame(Frame::NN_VOP, thisFrame->getWidth(), thisFrame->getHeight());
		dummyFrame->setMediaTimeScale(thisFrame->getMediaTimeScale());
		dummyFrame->setAU(&lastAU,false); //disable detectFrameType() for efficiency!
	}

	while(!done) {
		if (numFrames * es->getVOPTimeIncrement() == thisCTS) { //NO FRAME LOSS
			out->writeFrame(thisFrame);
			done=1;
		} else if (autoFix) { //catch up BECAUSE OF FRAME LOSS
			dprintf_full("DataDump::adapt %i dummyADAPT!\n",numFrames);
			out->writeFrame(dummyFrame);
		}
		numFrames++;
	}

	
	//	assert(lastAU.size == thisFrame->getAU()->size);
	dummyFrame->unsetAU();
	lastAU = thisFrame->getAU();
  lastAU.payload = lastPayload;
	lastAU.size = thisFrame->getAU()->size;
	memcpy(lastAU.payload, thisFrame->getAU()->payload, thisFrame->getAU()->size);
	dummyFrame->setAU(&lastAU,false); //disable detectFrameType() for efficiency!
 
	//standard forwarding adaptor behaviour
	tmp.push_front(frm);
	return tmp;
}
