 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: DataDump.hpp                                                  *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _DataDump_HPP
#define _DataDump_HPP
#include "Adaptor.hpp"
#include "ESInfo.hpp"
#include "IO.hpp"
#include "io/DevNull.hpp"


/**
 * Adaptor class DataDump stores each passing frame to disk (any IO), but also
 * inserts old frames if one was missing, to adhere to frame dropping adaptation
 * if this automatic insertion feature is not wanted, set the corresponding constructor flag!
 *
 * @author Michael Kropfberger
 *
 * @version $Id: DataDump.hpp,v 1.5 2006/01/20 15:37:17 mkropfbe Exp $
 *
 * @short <short description>
 *
 */
class DataDump : public Adaptor {
public:

	/* Constructor
	 * @param outStream defines th IO class for writing the output
	 * @param autoFix if set to <code>false</code>, automatic "missing frame" insertion feature turned off
	 *
	 */
	DataDump(ESInfo* esi, IO *outStream, bool autoFix = true);
	
	virtual ~DataDump();
	
	list < Frame * >adapt(Frame * frm);
	
 	Adaptor* clone() { return new DataDump(es, new DevNull(), autoFix); };

	void initialize();
	
	list < Frame * >close() {
		list < Frame * >tmp;
		return tmp;
	};

	/** Sets a reference to an ESInfo object. Adaptors are
		allowed to change ESInfo objects. If you don't want that,
		pass an es->clone() 
	*/ 
	void setESInfo(ESInfo * esi);

	ESInfo * getESInfo();

	/** returns adaptation costs (CPU only). This adaptor causes hd costs, which are 
	  * currently ignored. as CPU side effect we can only guess but it should depend
		* on the bytes of the video
	*/
	u32 getTranscodingCosts() const {
		return es->getAvgBandwidth()/8;
	}

private:
protected:
	ESInfo *es;
  IO *out;
	bool autoFix;
	AU lastAU;
	u8* lastPayload;
	Frame *dummyFrame;
	int numFrames;
};
#endif

