 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: FrameRateAdjust.cpp                                           *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "FrameRateAdjust.hpp"
#include "UncompressedVideoFrame.hpp"


/***************************************************************/
FrameRateAdjust::FrameRateAdjust(ESInfo *esi, u32 newTimeIncrement) {

	es = esi;
	timeInc = newTimeIncrement;
	oldTimeInc = es->getVOPTimeIncrement(); 
	//	es->setVOPTimeIncrement(timeInc);
	strcpy(name,"FrameRateAdjust");
	currentFrameNumber = 0;
	currentTicks = 0;

	dprintf_full("FrameRateAdjust (from %i to %i timeInc)\n", oldTimeInc, timeInc);
}


/***************************************************************/
FrameRateAdjust::~FrameRateAdjust() {
}


/***************************************************************/
void FrameRateAdjust::initialize() {
}


/***************************************************************/
list < Frame * > FrameRateAdjust::adapt(Frame * frm) {
	list < Frame * >tmp;
	u32 thisCTS;
	int thisSec;

	thisCTS = frm->getAU()->cts;
	thisSec = (int)floor((double)thisCTS / es->getMediaTimeScale());
	dprintf_full("FrameRateAdjust::adapt frame CTS %i (dropped until %i), VOP increment %i, playoutSec: %i \n",
							 thisCTS, currentTicks, oldTimeInc, thisSec );

	currentFrameNumber++;
	if (oldTimeInc <= timeInc) { // remove some frames
		if (thisCTS >= currentTicks) {
			//standard forwarding adaptor behaviour
			tmp.push_front(frm);
			currentTicks += timeInc;
		} else {
			//this frame is dropped....
			dprintf_full("FrameRateAdjust::adapt: this frame %p is dropped (will be deleted by AdaptorChain)....\n",frm);
		}
	} else {  // add some frames
		dprintf_err("FrameRateAdjust::adapt should add frames now (from %i to %i timeInc)... NOT SUPPORTED!\n",
								oldTimeInc,timeInc);
		::exit(1);
	}

	return tmp;
}
