 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	FrameRateAdjust.hpp                                           *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _MK_FRAMERATEADJUST_HPP
#define _MK_FRAMERATEADJUST_HPP
#include "Adaptor.hpp"
#include "Frame.hpp"
#include "ESInfo.hpp"

class Statistics;

/**
 * Adaptor class FrameRateAdjust adjusts the incoming framerate to a 
 * predefined outgoing framerate (both expressed by a timeIncrement)
 * so either by-passing frames are dropped (eg. 25fps -> 15fps) 
 * or doubly inserted (eg. 15 fps -:> 25fps)
 * with the mpeg4-used ticksPerSecond, the base is always 90000 ticks
 * 30 fps use a timeincrement of 3000 ticks/frame
 * 25 fps use a timeincrement of 3600 ticks/frame
 * 20 fps means 4500 ticks/frame
 * 15 fps means 6000 ticks/frame
 * 10 fps means 9000 ticks/frame
 * 
 *
 *@author Michael Kropfberger
 *
 *@version $Id: FrameRateAdjust.hpp,v 1.4 2006/01/20 15:37:17 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class FrameRateAdjust : public Adaptor {
public:

	/* Constructor
	 * @param esi is the ESInfo of the incoming stream, so it's timeIncrement is used
	 * @param newTimeIncrement sets the new <code>esi</code> timeIncrement
	 * 
	*/
	FrameRateAdjust(ESInfo* esi, u32 newTimeIncrement);
	
	virtual ~FrameRateAdjust();
	
	list < Frame * >adapt(Frame * frm);
	
	Adaptor *clone() { return new FrameRateAdjust(es, timeInc);	};

	void initialize();
	
	list < Frame * >close() {
		list < Frame * >tmp;
		return tmp;
	};

	/** Sets a reference to an ESInfo object. Adaptors are
		allowed to change ESInfo objects. If you don't want that,
		pass an es->clone() 
	*/ 
	void setESInfo(ESInfo * esi) {es=esi;};

	ESInfo * getESInfo() { return es;}

	/** returns adaptation costs (CPU only) */
	u32 getTranscodingCosts() const{
		return 1000;
	}

private:
protected:
	ESInfo* es;
	u32 timeInc;
	u32 oldTimeInc;
	u32	currentFrameNumber;
	u32 currentTicks;

};
#endif

