 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: FrameRotate.hpp                                               *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef __FRAMEROTATE_HPP__
#define __FRAMEROTATE_HPP__

#include "VideoAdaptor.hpp"
#include "UncompressedVideoFrame.hpp"
#include "ESInfo.hpp"

/**
 *
 *@author Michael Kropfberger
 *
 *@version $Id: FrameRotate.hpp,v 1.3 2005/06/19 14:47:41 mkropfbe Exp $
 *
 *@short Implementation of Adaptor that 90degree rotates every incoming frame
 *
 */
class FrameRotate:public VideoAdaptor {
public:
	/**
	 * Constructor.
	 *
	 * Takes input frame size from pESInfo, and 90degree rotates every
	 * incoming frame, after that, the pESInfo is set to the new size (x,y toggled)
	 * @param pESInfo Pointer to ESInfo object, containing valid size 
	 * information about the input Frame objects.
	 * rotation works for all RGB and YUV formats!
	 * @param degrees: is a number out of 90,180, 270
	 */
	FrameRotate(VideoESInfo * pESInfo, u32 degrees);

	virtual ~ FrameRotate();

	/**
	 * Frame frm in - list of Frame objects out.
	 *
	 * Output list will contain one Frame object only.
	 * If the incoming frm has already the correct size, no scaling is done
	 *
	 * @param frm Pointer to uncompressed Frame object
	 * @return List of Frame objects
	*/
	list < Frame * >adapt(Frame * frm);
		
	virtual void setESInfo(ESInfo * esi);

	virtual ESInfo * getESInfo();

	/**
	 *Not implemented.
	 *
	 *@return Defaults to NULL.
	 */
	Adaptor *clone();
	
	void initialize();

	/** returns adaptation costs (CPU only) 
	  * rotating is expensive */
	u32 getTranscodingCosts() const {
		return (u32)(((VideoESInfo*)es)->getWidth() * ((VideoESInfo*)es)->getHeight()
			* ((VideoESInfo*)es)->getFPS()*2);
	}

private:
	/* Width of input YUVFrame objects.	 */
	u32 muiWidth;

	/* Height of input YUVFrame objects.	 */
	u32 muiHeight;

	/* Width of output YUVFrame objects.	 */
	u32 muiTargetWidth;

	/* Height of output YUVFrame objects.	 */
	u32 muiTargetHeight;

	u32 degrees;

};

#endif				/// __FRAMEROTATE_HPP__
