 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	FrameTypeCounter.cpp                                          *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "FrameTypeCounter.hpp"
#include "UncompressedVideoFrame.hpp"
#include "Statistics.hpp"


FrameTypeCounter::FrameTypeCounter(ESInfo *esi, Statistics *statistics) {

	es = esi;
	stats = statistics;
	strcpy(name,"FrameTypeCounter");
}


FrameTypeCounter::~FrameTypeCounter() {
}


void FrameTypeCounter::initialize() {
}


/***************************************************************/
list < Frame * > FrameTypeCounter::adapt(Frame * frm) {
	list < Frame * >tmp;
	u32 thisCTS;
	int thisSec;

	thisCTS = frm->getAU()->cts;
	thisSec = (int)floor((double)thisCTS / es->getMediaTimeScale());
	dprintf_full("FrameTypeCounter::adapt frame CTS %i, VOP increment %i, playoutSec: %i FPS: %i\n",
							 thisCTS, es->getVOPTimeIncrement(), thisSec, stats->getPlayoutSecFPS(thisSec) );

	//	if (frm->getType() != Frame::HEADER_VOP)
	if(frm->getType() == Frame::I_VOP)
			stats->incPlayoutSecIFrame(thisSec);
	if(frm->getType() == Frame::P_VOP)
			stats->incPlayoutSecPFrame(thisSec);
	if(frm->getType() == Frame::B_VOP)
			stats->incPlayoutSecBFrame(thisSec);

	

	//standard forwarding adaptor behaviour
	tmp.push_front(frm);
	return tmp;

}
