/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	MP3Encoder.hpp                                                *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef AFX_MP3ENCODER_HPP
#define AFX_MP3ENCODER_HPP

#if _MSC_VER > 1000
#pragma once
#endif				// _MSC_VER > 1000

#include "AudioAdaptor.hpp"
#include "UncompressedAudioFrame.hpp"
#include "CompressedAudioFrame.hpp"
#include "AudioESInfo.hpp"
#include "MP4Decoder.hpp"

/**
 * Implementation of Adaptor that takes UncompressedAudioFrame (raw PCM) as input
 * and returns an CompressedAudioFrame (MPEG-1 layer 3 aka mp3) as output.
 *
 *@author Michael Kropfberger
 *
 *@version $Id: MP3Encoder.hpp,v 1.2 2005/05/13 18:18:02 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class MP3Encoder:public AudioAdaptor {
public:
	/**
	 * Constructor.
	 * Creates a MP3Encoder instance. Do set attributes of pESInfo object carefully.
	 * @param pESInfo Pointer to ESInfo object with proper encoding parameters.
	 * @param targetBitRate is set in bits/second
	 * @param outputFormat which video format should be created as output, will be ignored
	 * Used attributes of ESInfo: avgBandwidth
	 */
	MP3Encoder(AudioESInfo * pESInfo, 
						 u32 targetBitRate,
						 enum CodecID outputFormat=CODEC_ID_MP3LAME,
						 char* mp4EncoderConfig=NULL);

	/** Destructor.	*/
	virtual ~ MP3Encoder();

	/**
	 * raw PCM Frame in - MP3Frame out.
	 * Input raw PCM Frame object will be deleted in this method!
	 * Adaptor implementation for raw PCM Frame objects.
	 * @param *frm Pointer to raw PCM Frame object.
	 * @return List containing MP3Frame objects. Will contain in general
	 * only one MPGFrame object
	 */
	list < Frame * >adapt(Frame * frm);
	
	/**
	 * Flush all buffered MPGFrame objects (if any).
	 *	 @return List of MPGFrame objects
	 */
	list < Frame * >close();

	/** Not implemented.
	 * @return Defaults to NULL.
	 */
	Adaptor *clone();
	
	void initialize();
	
	u32 getTranscodingCosts() const { return 1000; };

private:
	/** Handle to XviD encoder instance.*/
	char *mpEncoderHandle;
	
	/** Size of decoded payload of Frame object in bytes.	*/
	u32 mFrameSize;
	
	/** Internal output buffer for encoder. */
	char mpOutBuf[MAX_BUFFER_SIZE];

	/** Internal input buffer for encoder. */
	char mpInBuf[MAX_BUFFER_SIZE];
	u32 inBufSize;

	/** Contains the number of frames encoded.	*/
	u32 muiFrameNumber;

	bool firstFrame;

	/** The target bitrate for the encoder */
	u32 muiTargetBitrate;

	enum CodecID outputFormat;
};

#endif

