 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	MP4audioDecoder.hpp                                           *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#if !defined(AFX_MP4AUDIODECODER_HPP__E08F3107_12A3_4E12_8A7F_63B15BE916FF__INCLUDED_)     
#define AFX_MP4AUDIODECODER_HPP__E08F3107_12A3_4E12_8A7F_63B15BE916FF__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif				// _MSC_VER > 1000



#include "ffmpegInterface.h"

#include "AudioAdaptor.hpp"
#ifdef ISOMP4
#include "ISOMovies.h"
#endif
#include "IO.hpp"

class AudioESInfo;
class YUVFrame;
class Frame;


/**
 * 
 *
 *@author Mithilesh Kumar
 *
 *@version $Id: MP4audioDecoder.hpp,v 1.10 2005/04/28 08:55:47 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class MP4audioDecoder:public AudioAdaptor {
  public:
	enum MpegAudioVersionID {
		MPEG_VESION_2_5,
		RESERVED,
		MPEG_VERSION_2,
		MPEG_VERSION_1,
	};

	enum MpegAudioLayerNo {
		RESERVed,
		LAYER_III,
		LAYER_II,
		LAYER_I,
	};

	enum ChannelMod {
		STEREO,
		JOINT_STEREO,
		DUAL_CHANNEL,
		SINGLE_CHANNEL,
	};

	enum Emphasis {
		NONE_,
		MS5015,
		RESERVEd,
		CCIT_J_17,
	};


	/**
		Codec selection enumeration, currently available is FFMPEG only
	*/
	typedef enum eCoderType{XVID = 1, FFMPEG} meCoderType;

	/**
		Creates an uninitialized MP4audioDecoder instance.
		@param pESInfo Pointer to ESInfo containing VOS/VO/VOL headers
		@param enableSwitching specifies if the decoder should reinit itself, when
		a new header is detected. set to false in the server, set to true in the client
		@param coderType audio codec to use.
		@see setCoderType
	*/
	MP4audioDecoder(AudioESInfo * pESInfo, bool enableSwitching, meCoderType coderType= FFMPEG); 
		

	/**
		Destroys the MP4audioDecoder
	*/
    virtual ~ MP4audioDecoder();
		
	static bool isValidAudioFrame(const u8* payload);
	static bool isValidDecoderConfig(const u8* payload);

	/**
		Compressed frm in - uncompressed PCM frame out.

		@param frm Pointer to compressed Frame
		@return List of YUVFrame objects
	*/
    list < Frame * >adapt(Frame * frm);

	/**
		Flush all buffered uncompressed Frame objects (if any).

		@return List of YUVFrame objects
	*/
    list < Frame * >close();

	/**
		Not implemented.

		@return Defaults to NULL.
	*/
    Adaptor *clone();

	/**
		Initializes the Decoder.
	*/
    void initialize();

	/**
		Codec selection

		@param coderType Type of codec to use. Defaults to FFMPEG
		@see meCoderType
	*/
	void setCoderType(eCoderType coderType = FFMPEG);

	/** returns adaptation costs (CPU only). How high are audio decoding costs?
	  * assume 25% of CIF video with 25 fps (FIXME: measure it)*/
	u32 getTranscodingCosts() const{
		return 352*288*25/4;
	}
  private:
	/**
		Contains the selected codec type

		@see setCoderType
	*/
	eCoderType meCoder;

	/**
		Decoder Initialization method. The decoder chosen with setCoderType is initialized.

		@param pDecoderHandle Handle to decoder instance
	*/
	void DecoderInit(char **pDecoderHandle, CodecID codecID);
	/**
		Close Decoder.

		@param pDecoderHandle Handle to decoder instance created with DecoderInit
		@see DecoderInit
	*/
	void DecoderClose(char **pDecoderHandle);
	/**
		Decode a single Frame.

		@param pDecoderHandle Handle to valid decoder instance
		@see DecoderInit
		@param pCompressedFrame Pointer to memory area where a complete compressed frame is stored.
		@param iCFSize Size of frame stored in pCompressedFrame
		@param PCMFrame Pointer to allocated memory area where the uncompressed frame will
				be written to.
	*/
	int DecodeFrame(char *pDecoderHandle, char *pCompressedFrame, 
		int iCFSize, char* PCMFrame, u32 *mFrameSize);
	/**
		Handle to FFMPEG decoder instance.
	*/
    char *mpDecoder;

	/**
		Size of decoded payload of Frame object in bytes.
	*/
	u32 mFrameSize;

	/**
	   Used by initialize() and adapt().
	*/
    char mpBuffer[MAX_FRAME_SIZE];
	/**
		For internal use only
	*/
    int lenDecoded;
	/**
		For internal use only
	*/
    uint muiLastFrameStart;
	/**
		For internal use only
	*/
    uint muiLastFrameEnd;
	/**
		Contains the number of frames decoded.
	*/
    u32 muiFrameNumber;

		/** to support switching the decoder must be able
				to detect when a new decConfig was set at the 
				ESInfo object. Thus, buffer the old decCOnfig
				and also provide temporary storage for the new one
		*/
		u8* decConfig;
		u8* newDecConfig;
		u32 decConfigSize;
		/** specifies if the decoder should check if a new ESHeader
		was set. Set to false in the server, set to true in the client */
		bool enableSwitching;


	/**
		DecoderInit will store parsed header information in this struct
		The Header for MPEG audio is the first 32 bits.
	*/
    typedef struct {
	/* Frame sync is the Mpeg audio Frame Header Startcode*/
	int frame_sync;
	/* MPEG Audio version ID
		00 - MPEG Version 2.5
		01 - reserved
		10 - MPEG Version 2 (ISO/IEC 13818-3)
		11 - MPEG Version 1 (ISO/IEC 11172-3) */
	enum MpegAudioVersionID mpeg_audio_version_ID;
	/* Layer description
		00 - reserved
		01 - Layer III
		10 - Layer II
		11 - Layer I */
	enum MpegAudioLayerNo mpeg_audio_layer_no;
	/*Protection bit
		0 - Protected by CRC (16bit crc follows header)
		1 - Not protected */
	bool protection;
	/*Bitrate index 
		NOTE: bitrate in kbps
		see Bitrate index Table */
	unsigned int bitrate;
	/* Bitrate index Table 
		bitrate is obtainded by mapping into this table */
	unsigned int mpeg_audio_bitrate_index_table[80];
	/* Sampling rate frequency index (values are in Hz)
		see Sampling rate frequency Table*/
	unsigned int sampling_rate;
	/* Sampling rate frequency Table
	*/
	unsigned int mpeg_audio_sampling_rate_index_table[12];
	/* Padding bit
		0 - frame is not padded
		1 - frame is padded with one extra slot */
	bool padding;
	/* calculate frame size
	*/
	unsigned int frame_size_in_bytes;
	/* Private bit. It may be freely used for specific needs of an application, 
		i.e. if it has to trigger some application specific events.*/
	bool private_bit;
	/* Channel Mode
		00 - Stereo
		01 - Joint stereo (Stereo)
		10 - Dual channel (Stereo)
		11 - Single channel (Mono) */
	enum ChannelMod channel_mod;
	/* Mode extension (Only if Joint stereo) 
	*/
	unsigned int mode_extension;
	/* Copyright info 
		0 - Audio is not copyrighted
		1 - Audio is copyrighted*/
	bool copyright;
	/* Original
		0 - Copy of original media
		1 - Original media */
	bool original;
	/* Emphasis
		00 - none
		01 - 50/15 ms
		10 - reserved
		11 - CCIT J.17 */
	enum Emphasis emphasis;
	
    } structMPEGaudioHeader;

	/**
		If the decoder was successfully created, this variable 
		contains the information stored in MPEGaudio header.

		@see DecoderInit()
	*/
    structMPEGaudioHeader mp4AudioHeader;

	/**
		For internal use of parseXYZ methods only.

		This variable contains the bitbuffer for getBit(), getBits() and nextBits() methods.
		@see getBit()
		@see getBits(int iNumber)
		@see nextBits(int iNumber)
	*/
    u8 mcByte;
	/**
		For internal use of parseXYZ methods only.

		Byte buffer for nextByte() method.
		@see nextByte()
		@see mpcData
	*/
    u8 *mpcData;
	/**
		For internal use of parseXYZ methods only.

		Size of mpcData buffer for nextByte() method.
		@see nextByte()
	*/
    int miSize;
	/**
		For internal use of parseXYZ methods only.

		Is set in nextByte() to true when end of mpcData buffer is reached.
		@see nextByte()
		@see mpcData
	*/
    bool mbEOD;
	/**
		For internal use of parseXYZ methods only.

		Current byte index withing mpcData. Maintained by nextByte().
		@see nextByte()
	*/
    int miCurrentNoBytes;
	/**
		For internal use of parseXYZ methods only.

		Current bit index within CurrentNoBytes byte. 
		Maintained by getBit and getBits().
		@see getBit()
		@see getBits(int iNumber)
		@see mpcData
	*/
    int miCurrentBit;
	/**
		For internal use of parseXYZ methods only.

		Used by bookmark() and restore().
		@see bookmark()
		@see restore()
		@see mcByte
	*/
    int miBookmarkedCurrentBit;
	/**
		For internal use of parseXYZ methods only.

		Used by bookmark() and restore().
		@see bookmark()
		@see restore()
	*/
    int miBookmarkedByte;
	/**
		For internal use of parseXYZ methods only.

		Used by bookmark() and restore().
		@see bookmark()
		@see restore()
	*/
    int miBookmarkedCurrentNoBytes;

	/**
		Initialize internal buffer structures. 

		Must be called before any call to parseXYZ functions.

		@see parseMPEGaudioHeader()
		@see mpcData
	*/
    void initializeParser(u8 * pFramem, u32 iSize);
	/** 
		Initialize a structMPEGaudioHeader structure.

		Sets everything in a structMPEGaudioHeader to 0.
		@see structMPEGaudioHeader
	*/
    void initializeMPEGaudioHeaderStruct();
	/**
		Fetch next byte from mpcData.

		@see mpcData
		@see getBit()
		@see getBits(int iNumber)
		@see nextBits(int iNumber)
	*/
    void nextByte();
	/**
		Check if current bit is byte aligned, if not then seek for next byte-aligned position.

		@see nextStartCode()
		@return Always true
	*/
    bool bytealigned();
	/**
		Fetch next bit from bitstream and advance.

		@see getBits(int iNumber))
		@see nextBits(int iNumber)
		@see mcByte
		@see mpcData
		@return Value of next bit in bitstream.
	*/
    int getBit();
	/**
		Fetch next iNumber bits in bitstream and advance.

		@see getBit()
		@see nextBits(int iNumber)
		@see mcByte
		@see mpcData
		@return Value of next iNumber bits in bitstream.
	*/
    int getBits(int iNumber);
	/**
		Fetch next iNumber bits in bitstream. Do not advance.

		@see getBit()
		@see getBits(int iNumber)
		@see mcByte
		@see mpcData
		@return Value of next iNumber bits in bitstream.
	*/
    int nextBits(int iNumber);
	/**
		Locate next startcode.
	*/
    void nextStartCode();

	/**
		Read quanzizer matrix values from the bitstream.
	*/
    void loadQuantizerMatrix(unsigned int puiMatrix[]);

	/**
		Check for correct set marker bit.

		@iLine Codeline where method is called.
	*/
    void readMarkerBitFunc(int iLine);
	/**
		Save current status of the bitstream.
		@see restore()
	*/
    void bookmark();
	/**
		Restore the last bookmark()-ed status of the bitstream.
		@see bookmark()
	*/
    void restore();

    bool video_signal_type_func();
	/**
	  	Parse an Audio Frame  header. (currently MPEG1&2 audio only)
		@return True on success, false on error.
	  */
    bool parseMPEGaudioHeader(u8 * pFramem, u32 iSize);
    
};

#endif				// !defined(AFX_MP4DECODER_HPP__E08F3107_12A3_4E12_8A7F_63B15BE916FF__INCLUDED_)
