 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	MadMP3Decoder.hpp                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _MadMP3Decoder_HPP
#define _MadMP3Decoder_HPP

#include <math.h> /* for pow() and log10() */
#include <mad.h>

#include "AudioAdaptor.hpp"
#include "AudioESInfo.hpp"
#include "IO.hpp"
#include "io/DevNull.hpp"


/**
 * Adaptor class MadMP3Decoder decodes bypassing MPEG-1/2 layer 3 audio packets 
 * to raw PCM sample blocks using the libmad
 *
 * @author Michael Kropfberger
 *
 * @version $Id: MadMP3Decoder.hpp,v 1.3 2006/01/20 15:37:17 mkropfbe Exp $
 *
 * @short <short description>
 *
 */
class MadMP3Decoder : public AudioAdaptor {
public:

	/* Constructor
	 * @param outStream defines th IO class for writing the output
	 * @param autoFix if set to <code>false</code>, automatic insertion feature turned off
	 *
	 */
	MadMP3Decoder(AudioESInfo* es);
	
	virtual ~MadMP3Decoder();
	
	list < Frame * >adapt(Frame * frm);
	
 	Adaptor* clone() { return new MadMP3Decoder(es); };

	void initialize();
	
	list < Frame * >close() {
		list < Frame * >tmp;
		return tmp;
	};


	/** returns adaptation costs (CPU only). This adaptor causes hd costs, which are 
	  * currently ignored. as CPU side effect we can only guess but it should depend
		* on the bytes of the video
	*/
	u32 getTranscodingCosts() const {
		return es->getAvgBandwidth()/8;
	}

private:
protected:
	static signed short MadFixedToSshort(mad_fixed_t Fixed);
	static void PrintFrameInfo(struct mad_header *Header);



#define MAD_INPUT_BUFFER_SIZE	(5*8192)

	struct mad_stream	Stream;
	struct mad_frame	madFrame;
	struct mad_synth	Synth;
	mad_timer_t			Timer;
	unsigned char		*inputBuffer;
	unsigned char	*guardPtr;
	int					Status;
	unsigned long		FrameCount;

	u32 numInFrames;
	bool initialized;
	u32 lastTS;
	AU * pAU;
};

#endif

