 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	PSNR.hpp                                                      *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PSNR_HPP
#define _PSNR_HPP
#include "VideoAdaptor.hpp"
#include "VideoESInfo.hpp"
#include "../io/YUVStreamIO.hpp"
#include "UncompressedVideoFrame.hpp"

class Statistics;

/**
 * Adaptor class PSNR calculates psnr for each frame and provides
 * overall statistics
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: PSNR.hpp,v 1.11 2006/01/20 15:37:17 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class PSNR : public VideoAdaptor {
public:

	/* Constructor 
	 * @param: original yuvstream, which will be opened when first used
	 * @param: statistics class, to which the avgPSNR per second will be stored
	 * @param: psnrFP, a file pointer to an already opened text file, where per frame PSNR is stored
	*/
	PSNR(VideoESInfo* esi, YUVStreamIO *OrigYUVStream, Statistics *statistics, FILE* psnrFP=NULL);
	
	virtual ~PSNR();
	
	list < Frame * >adapt(Frame * frm);
	
	Adaptor *clone() { return new PSNR(es, yuvOrig,stats,NULL);	};
	
	void initialize();
	
	list < Frame * >close() {
		list < Frame * >tmp;
		if(psnrInfo) {
			fclose(psnrInfo);
			psnrInfo=NULL;
		}
		return tmp;
	};

	/** returns adaptation costs (CPU only) */
	u32 getTranscodingCosts() const {
		return 10000u;
	}
private:
protected:
  YUVStreamIO *yuvOrig;
	AU lastAU;
	u8* lastPayload;
	UncompressedVideoFrame *dummyFrame;
	Statistics *stats;

	double sumPSNR;
	int numFrames;
	int numFramesNotInf;
	FILE* psnrInfo;
};
#endif

