 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	QualityReductionAdaptor.hpp                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef __QUALITY_ADAPTOR_HPP__
#define __QUALITY_ADAPTOR_HPP__

#include "Adaptor.hpp"
#include "VideoAdaptorChain.hpp"
#include "MP4Encoder.hpp"
#include "MP4Decoder.hpp"
#include "VideoESInfo.hpp"
#include "Frame.hpp"

/**
 * Implementation of Adaptor that reduces average bitrate of a MP4 VES.
 * Encoding scheme used is CBR, and might vary.
 *
 *@author Bernhard Penz
 *
 *@version $Id: QualityReductionAdaptor.hpp,v 1.4 2004/05/27 17:56:53 pschojer Exp $
 *
 *@short <short description>
 *
 */
class QualityReductionAdaptor:public VideoAdaptorChain {
  public:
	/**
		Constructor.

		@param pESInfo Pointer to ESInfo object.
		@param uiTargetBitrate Desired target bitrate.
	*/
    QualityReductionAdaptor(VideoESInfo * pESInfo, u32 uiTargetBitrate,
			    u32 keyFrameInterval = 30,MP4Decoder::meCoderType codec = MP4Decoder::FFMPEG);
	/**
		Destructor.
	*/
    virtual ~ QualityReductionAdaptor();


    Adaptor *clone();

    void initialize();
  protected:
     u32 targetBitrate;
    u32 keyFrameInterval;
		MP4Decoder::meCoderType codec;
};

#endif				/// __QUALITY_ADAPTOR_HPP__
