 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	SharedAdaptor.hpp                                             *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

/**
 *
 * Wrapper for an adaptor so that the shared Adaptor can be used amongst several
 * DataChannels/DataSinks. Create one adaptor, one ReferenceCounter and N SharedAdaptor wrappers.
 * When deleting the SharedAdaptor, the last one that is deleted will destroy the
 * Adaptor
 *
 *@author Peter Schojer
 *
 *@version $Id: SharedAdaptor.hpp,v 1.4 2004/06/14 09:02:46 pschojer Exp $
 *
 *@short adaptor shareable amongst several DataSinks/DataChannels
 *
*/

#ifndef SharedAdaptor_hpp
#define  SharedAdaptor_hpp

#include "Adaptor.hpp"
#include "ReferenceCounter.hpp"

class SharedAdaptor: public Adaptor
{
public:
	/* Wrapper for an adaptor so that the shared Adaptor can be used amongst several
	* DataChannels/DataSinks. Create one adaptor, one ReferenceCounter and N SharedAdaptor wrappers.
	* When deleting the SharedAdaptor, the last one that is deleted will destroy the
	* Adaptor. Do not increase the ReferenceCounter outside of the class!!!!
	* Increasing and decreasing must happen inside the SharedAdaptor
	*/
	SharedAdaptor(ReferenceCounter*& ref, Adaptor* &a);

	virtual ~SharedAdaptor();
	
	ReferenceCounter* getReferenceCounter();

	Adaptor* getAdaptor();	

	list < Frame * >adapt(Frame * frm) { return adaptor->adapt(frm); };
	
	Adaptor *clone() { return new SharedAdaptor(usageCnt, adaptor);	};
	
	void initialize() {};
	
	/** the following is only for virtual method implementation
	*/ 
	void setESInfo(ESInfo * esi) {};

	ESInfo * getESInfo() { return NULL; };
	
	/** returns adaptation costs (CPU only) */
	u32 getTranscodingCosts() const {
		return 0u;
	}

	
protected:
	/** how many objects are currently using this adaptor */
	ReferenceCounter* usageCnt;
	/** the adaptor protected by this wrapper class */
	Adaptor* adaptor;
};


#endif
