 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	SpatialReductionAdaptor.hpp                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef __SPATIALREDUCTION_ADAPTOR_HPP__
#define __SPATIALREDUCTION_ADAPTOR_HPP__

#include "VideoAdaptor.hpp"
#include "VideoAdaptorChain.hpp"
#include "MP4Encoder.hpp"
#include "MP4Decoder.hpp"
#include "YUVSpatialReductionAdaptor.hpp"
#include "ESInfo.hpp"
#include "Frame.hpp"

/**
 * Implementation of Adaptor that implements spatial adaptation
 *
 *@author Bernhard Penz
 *
 *@version $Id: SpatialReductionAdaptor.hpp,v 1.4 2004/05/27 17:56:53 pschojer Exp $
 *
 *@short <short description>
 *
 */
class SpatialReductionAdaptor : public VideoAdaptorChain {
  public:
	/**
		Constructor.

		@param pESInfo Pointer to ESInfo object. Width, height, and bitrate information
			about the input stream will be fetched from this object.
		@param puiWidth Desired width of the output stream.
		@param puiHeight Desired height of the output stream.
		@param uiTargetBitrate Target bitrate of the output stream.
	*/
    SpatialReductionAdaptor(VideoESInfo * pESInfo, u32 * puiWidth,
			    u32 * puiHeight, u32 uiTargetBitrate,
			    u32 keyFrameInterval = 300,MP4Decoder::meCoderType codec = MP4Decoder::FFMPEG);
	/**
		Destructor.
	*/
    virtual ~ SpatialReductionAdaptor();

	/**
		implemented. 
		@return SpatialReductionAdaptor
	*/
    Adaptor *clone();

    void initialize();
  protected:
     u32 width;
    u32 height;
    u32 targetBitrate;
    u32 keyFrameInterval;
		MP4Decoder::meCoderType codec;
};

#endif				/// __SPATIALREDUCTION_ADAPTOR_HPP__
