 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	StatisticsUpdater.cpp                                         *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "StatisticsUpdater.hpp"
#include "Statistics.hpp"


StatisticsUpdater::StatisticsUpdater(ESInfo *esi, Statistics *statistics) {
	dprintf_full("StatisticsUpdater::StatisticsUpdater up and ready\n");
	es = esi;
	stats = statistics;
	strcpy(name,"StatisticsUpdater");
}


StatisticsUpdater::~StatisticsUpdater() {
}


void StatisticsUpdater::initialize() {
}


/***************************************************************/
list < Frame * > StatisticsUpdater::adapt(Frame * frm) {
	list < Frame * >tmp;
	u32 thisCTS;

	thisCTS = frm->getAU()->cts;
	dprintf_full("StatisticsUpdater::adapt frame CTS %i, VOP increment %i\n",
							 thisCTS,es->getVOPTimeIncrement() );

	//STATS BLOCK
	struct  timeval now_tv;
	double now_dtime;
	double thisFloatSec = (double)frm->getAU()->cts / es->getMediaTimeScale();
	int thisSec = (int)floor(thisFloatSec);

	gettimeofday(&now_tv,NULL);
	now_dtime = now_tv.tv_sec + now_tv.tv_usec/1000000.0;

	if (stats->getFirstPacketTime() == 0) {
	  stats->setFirstPacketTime(now_dtime);
	  stats->setFirstPacketTS(thisCTS);
	}
	stats->setLastPacketTime(now_dtime);

	stats->checkForNewPlayoutSec(thisSec,(int)floor((double)stats->getHighestPacketTS() 
							/ es->getMediaTimeScale()));
	stats->setHighestPacketTS(frm->getAU()->cts);
	if (frm->getType() != Frame::HEADER_VOP)
		stats->incPlayoutSecNetFPS(thisSec);

	stats->checkForNewStreamoutSec(now_dtime);
	stats->incPlayoutSecDataBW(thisSec,frm->getAU()->size);
	stats->incStreamoutSecDataBW(thisSec,frm->getAU()->size);

	//standard forwarding adaptor behaviour
	tmp.push_front(frm);
	return tmp;

}
