 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	TemporalAdaptor.cpp                                           *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "TemporalAdaptor.hpp"
#include "Frame.hpp"
#include "IO.hpp"
#include "BitField.hpp"
	/** Temporaladaptor updates @param esi to reflect the change in
	* the framerate
	*/
TemporalAdaptor::TemporalAdaptor(VideoESInfo * esi) {
	es = esi;
	inFrames = 0;
	outFrames = 0;
	lastDTS = 0;
	initialized = false;
	strcpy(name,"TemporalAdaptor");
};
TemporalAdaptor::TemporalAdaptor() {
	es = NULL;
	inFrames = 0;
	outFrames = 0;
	lastDTS = 0;
	initialized = true;
	strcpy(name,"TemporalAdaptor");
};

void TemporalAdaptor::initialize()
{
	if(initialized)
		return;
	initialized = true;
	if(es) {
		u32 oldBW=es->getAvgBandwidth();
		float bFrame=es->getAvgBFrameSize();		
		u32 newBW=(u32)(oldBW*bFrame);
		dprintf_full("TemporalAdaptor::initialize BW %u -%f-> %u\n",oldBW,bFrame,newBW);
		es->setAvgBFrameSize(0.0);
		es->setNum_B_frames(0);
		es->setAvgBandwidth(newBW);
	}
}
/** accepts as input a frame, the frame is never modified.
  * If the frame was a B-frame, the returned list will contain no elements.
  * If no adaptation happened, the unmodified frame will be in the list.
  */
list < Frame * >TemporalAdaptor::adapt(Frame * frm)
{
	// a temporaladaptor is not allowed to delete the input frame
	// because other DataSinks might need it as input
	if(!initialized)
		initialize();
	inFrames++;
	list < Frame * >tmp;
	if (!doAdapt(frm)) {
		tmp.push_front(frm);
		outFrames++;
	}

	return tmp;
};

list < Frame * >TemporalAdaptor::close()
{
	list < Frame * >tmp;

	// set the fps
	// make sure it's a multiple of origVopTimeIncr
	if (es) {
		origVopTimeIncr=es->getVOPTimeIncrement();
		u32 vInc=es->getDuration()/this->outFrames;
		u32 t=vInc%origVopTimeIncr;
		vInc-=t;
		es->setVOPTimeIncrement(vInc);
		u64 fs=es->getCurrentDemuxedMediaSize();
		fs*=8000;
		fs/=es->getDurationInMs();
		es->setAvgBandwidth((u32)fs);
	}
	// correct the bitfield, which now shows a too small complete status
	if(es && es->getFrameStatistic()) {
		BitField* b=new BitField(es->getFrameStatistic()->getSize(),true);
		es->setFrameStatistic(b);
	}
	return tmp;
};
