 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	TemporalAdaptor.hpp                                           *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PS_TEMPORALADAPTOR_HPP
#define _PS_TEMPORALADAPTOR_HPP
#include "VideoAdaptor.hpp"
#include "global.hpp"
#include "Frame.hpp"
#include "ESInfo.hpp"

/**
 *
 * drops all bypassing B-frames
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: TemporalAdaptor.hpp,v 1.7 2005/01/19 13:41:41 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class TemporalAdaptor : public VideoAdaptor {
  public:
	TemporalAdaptor(VideoESInfo * esi);
	TemporalAdaptor();

	/** accepts as input a frame, and directly modifies this frame. After
	* adaptation the payload of the frame might be null
	*/
	list < Frame * >adapt(Frame * frm);
	Adaptor *clone() {
		return new TemporalAdaptor(es);
	};

	list < Frame * >close();
	void initialize();
	virtual ~ TemporalAdaptor() {
	};

	/** returns adaptation costs (CPU only) */
	u32 getTranscodingCosts() const {
		return 1000u;
	}
protected:
	bool doAdapt(Frame * &frm) {
		return frm->getType() == Frame::B_VOP;
	};
	u32 inFrames;
	u32 outFrames;
	u32 lastDTS;
	u32 origVopTimeIncr;
};
#endif
