/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	TheoraEncoder.hpp                                       *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "TheoraEncoder.hpp"
#include "io/OGGMuxDemuxIO.hpp"

/**
 * Implementation of Adaptor that takes UncompressedVideoFrame as input
 * and returns an CompressedVideoFrame as output.
 *
 *@author Thomas Kuglitsch
 *
 *@version $Id: TheoraEncoder.cpp,v 1.5 2006/10/30 15:34:51 tkuglits Exp $
 *
 *@short <short description>
 *
 */

class OGGMuxDemuxIO;

/*
  VideoESInfo(u32 streamId, u32 streamtype, ContainerInfo * vob, u32 timeIncrement,
              u8 * encodeddecconf, u32 objtype, u32 bufsize,
	      u32 avgBW, u32 maxBW, u32 timeScale,
	      u64 duration, u32 size,  
	      u32 w, u32 h, bool colorVideo=true,
	      bool complete=true, 
	      CodecID codec_id = CODEC_ID_MPEG4, float quality = 1.0,
	      float aspect_ratio = 352.0/288.0, bool staticFramePattern=true, u32 GOP_size = 50, 
	      u32 num_B_frames = 0, float avgBFrameSize=-1.0);
*/


TheoraEncoder::TheoraEncoder(VideoESInfo *ves){
  dprintf_full("TheoraEncoder::TheoraEncoder\n");

  OGGMuxDemuxIO::initTheora(ves);

}

TheoraEncoder::~TheoraEncoder(){}

list <Frame *>TheoraEncoder::adapt(Frame * frm){
  list <Frame *>frmList;
  ogg_packet *op;
  yuv_buffer *yuv;
  theora_info *ti;
  CompressedVideoFrame *theoraFrame;
  AU *pAU;

  AU *au = frm->getAU();
  op = new ogg_packet;
  yuv = new yuv_buffer;
  ti = OGGMuxDemuxIO::getTheoraInfo();

  dprintf_full("TheoraEncoder::adapt(%p)\n",frm);
  {
    yuv->y_width = ti->width;
    yuv->y_height = ti->height;
    yuv->y_stride = ti->width;

    yuv->uv_width = ti->width / 2;
    yuv->uv_height = ti->height / 2;
    yuv->uv_stride = ti->width / 2;

    yuv->y = au->payload;
    yuv->u = au->payload + ti->width * ti->height;
    yuv->v = au->payload + ti->width * ti->height * 5/4;
  }

  double start_dtime, now_dtime;
  struct timeval now_tv;
  gettimeofday(&now_tv,NULL);
  start_dtime = now_tv.tv_sec + now_tv.tv_usec/1000000.0;

  theora_encode_YUVin(    OGGMuxDemuxIO::getTheoraState(), yuv);
  theora_encode_packetout(OGGMuxDemuxIO::getTheoraState(), 0, op);

  int op_size = sizeof(ogg_packet);
  u8* op_copy = new u8[op_size + op->bytes];

  {
    ogg_packet *tmp = (ogg_packet*)op_copy;

    // op_copy = (u8*)&op;
    // tmp->packet = op->packet;

    memcpy(op_copy, op, op_size);
    tmp->packet = op_copy + op_size;
    memcpy(tmp->packet, op->packet, op->bytes);
  }

  gettimeofday(&now_tv,NULL);
  now_dtime = now_tv.tv_sec + now_tv.tv_usec/1000000.0;
	   
  pAU = new AU();
  pAU->payload = op_copy;
  pAU->size = op->bytes;
  
  theoraFrame = new CompressedVideoFrame(Frame::UNKNOWN_VOP, ti->width, ti->height);
  theoraFrame->detectFrameType();
  theoraFrame->setAU(pAU);
  theoraFrame->setMediaTimeScale(frm->getMediaTimeScale());
  frmList.push_back(theoraFrame);

  dprintf_full("TheoraEncoder::adapt(%p) %li bytes ... %u bytes in %2.0f sec\n",frm,op->bytes,au->size, (now_dtime-start_dtime)*1000.0);
  
  return frmList;
}

void TheoraEncoder::initialize(){

  /*
    theora_encode_init(td, ti);
    theora_info_clear(ti);
    
    // first, encode theora header
    //theora_encode_header(td, &op);

    //VideoFrame *vf = new CompressedVideoFrame(FrameType t, int width, int height);
    VideoFrame *vf;
    vf = new CompressedVideoFrame(Frame::NN_VOP, frame_x, frame_y);
    // OGGMuxDemuxIO::writeFrame(Frame *frm, ESInfo *out_es = NULL);
    //    OGGMuxDemuxIO::writeFrame(vf);
    initialized = true;
  */
}
