/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	TheoraEncoder.hpp                                       *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _THEORA_ENCODER_H
#define _THEORA_ENCODER_H

#include "VideoAdaptor.hpp"
#include "UncompressedVideoFrame.hpp"
#include "CompressedVideoFrame.hpp"
#include "VideoESInfo.hpp"
#include "io/OGGMuxDemuxIO.hpp"
#include <theora/theora.h>

class OGGMuxDemuxIO;

/**
 * Implementation of Adaptor that takes UncompressedVideoFrame as input
 * and returns an CompressedVideoFrame as output.
 *
 *@author Thomas Kuglitsch
 *
 *@version $Id: TheoraEncoder.hpp,v 1.3 2006/10/25 07:29:10 tkuglits Exp $
 *
 *@short <short description>
 *
 */
class TheoraEncoder:public VideoAdaptor {
public:

  TheoraEncoder(VideoESInfo *ves);
  ~TheoraEncoder();
  
  /**
   * YUVFrame in - TheoraFrame out.
   * Input YUVFrame object will be deleted in this method!
   * Adaptor implementation for YUVFrame objects.
   * @param *frm Pointer to YUVFrame object.
   * @return List containing TheoraFrame objects. 
   */
  list < Frame * >adapt(Frame * frm);

  TheoraEncoder* clone() { return NULL; };
  void initialize();
  u32 getTranscodingCosts() const{
    return 0;
  };

private:
  theora_info *ti;
  theora_state *td;
  theora_comment *tc;
  int frame_x, frame_y;
};

#endif
