 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	TimeMeasureAdaptor.hpp                                  *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PS_TIMEMEASUREADAPTOR_HPP_
#define _PS_TIMEMEASUREADAPTOR_HPP_
//class Frame;
class ESInfo;

#include "global.hpp"
#include "Frame.hpp"
#include "Adaptor.hpp"

#ifdef WIN32
#include <winsock2.h>
#endif


class TimeMeasureAdaptor : public Adaptor
{
public:
	TimeMeasureAdaptor(Adaptor* a,FILE* fp=NULL);
	virtual ~TimeMeasureAdaptor();

	/** 
		Accepts as input a frame, and checks if it can adapt the frame,
		If the frame is modified, a new Frame object is created and inserted into the return list.
		This new frame is returned only, if the size of the payload is
		larger than zero. If the frame was not modified, it is also inserted
		into the list.
		For more complex adaptors, it will happen, that the adaptor caches up one complete GOP
		and returns the full GOP in network order. A caching adaptor has to create
		deep-copies for each frames.
		Never directly modify the payload of the input frm, always create copies!!!

		@param frm object to adapt. An Adaptor is never allowed to free
		the input frame. This has to be done in the caller.
		@return List of adapted Frame objects
	*/
	list<Frame*> adapt(Frame * frm);
	/**
		Close and destroy an Adaptor.

		Releases all internally buffered frame objects, and deallocate 
		all allocated memory. The Adaptor must not be used after calling
		the close method!!!

		@return List of adapted Frame objects.
	*/
	list<Frame*> close();
	/**
		Create a shallow copy of the Adaptor.

		Creates an Adaptor with the same setup (without copying the current status).
		@return An Adaptor object.
	*/
	Adaptor *clone();
	/**
		Initialize internal data structures.
	*/
	void initialize();
	
	/** Sets a reference to an ESInfo object. Adaptors are
		allowed to change ESInfo objects. If you don't want that,
		pass an es->clone() 
	*/ 
	void setESInfo(ESInfo * esi);
	ESInfo * getESInfo();

	/**
		Retrieve the time measured with startTimer 
		and stopTimer methods.

		@return Time in microseconds.
	*/
	unsigned int getTimeSpent();
	unsigned int getTimeSpentForLastFrame();

	/** returns adaptation costs (CPU only) */
	u32 getTranscodingCosts() const {
		return 1000u;
	}
private:

	/**
		Start an internal timer.

		This function shall be used internally only, e.g. at the
		first line of the adapt method. If used otherwise, correct
		function is not guaranteed.
	*/
	void startTimer();
	/**
		Stop an internal timer.

		This function shall be used internally only, e.g. at the
		last line of the adapt method. If used otherwise, correct
		function is not guaranteed.
	*/
	void endTimer();

	static long getdifftime(struct timeval *t1, struct timeval *t2);
	
protected:
	Adaptor* adaptor;
	unsigned int muiTimeSpent;
	unsigned int muiTimeSpentLastFrame;
	struct timeval mtStart;
	struct timeval mtEnd; 
	FILE* output;
	
};
#endif
