 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	Visualizer.cpp                                                *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "Visualizer.hpp"
#ifdef ENABLE_SDL
#include "io/SDLRenderer.hpp"
#endif
#ifdef ENABLE_QT
#include "io/QLabelRenderer.hpp"
#endif
#ifdef ENABLE_X11
#include "io/X11Renderer.hpp"
#endif

Visualizer::Visualizer(const VideoESInfo * esi, OutputMethod out, UncompressedVideoFrame::ColorSpace colorSpace) {
	firstFrame = true;
	initialized = false;
	this->colorSpace = colorSpace;
	output = out;
	assert(esi);
	es = esi;
	switch (output) {
	case SDL:
#ifdef ENABLE_SDL
		//this renderer automatically creates an output-SDL-surface (window)
		renderer = new SDLRenderer(es, colorSpace, 0,0, 
#ifdef BUILD_ARCH_ARM
															 true  //enable fullScreen
#else
															 false //no fullScreen
#endif
															 ,true /* pollEvents */, true /* isLabelResizable */,
															 true /* fixAspectRatio */ , false /* allowUpscaleX */,
															 false /* rotateXDisplay */ );
#else
		dprintf_err("Visualizer: ViTooKi was compiled without SDL support!\n");
		::exit(1);		
#endif
		break;

	case Qt:
#ifdef ENABLE_QT
#else
		dprintf_err("Visualizer: ViTooKi was compiled without Qt support!\n");
		::exit(1);		
#endif

	case X11:
#ifdef ENABLE_X11
		//this renderer automatically creates an output-X11-window on the default display
		renderer = new X11Renderer(es, colorSpace, 0,0, true, true, true, false );
#else
		dprintf_err("Visualizer: ViTooKi was compiled without X11 support!\n");
		::exit(1);		
#endif
		break;

	default:
		//make compiler happy
		break;
	}

	dprintf_full("Visualizer created (%ix%i) for outputIO: %s\n", es->getWidth(), es->getHeight(), renderer->getURL());
	strcpy(name,"Visualizer");
}


void Visualizer::initialize() {
	firstFrame = true;
	
	width = es->getWidth();
	height = es->getHeight();

	initialized = true;
	dprintf_full("Visualizer initialized\n");
}


list < Frame * >Visualizer::adapt(Frame * frm) {	
	list < Frame * >f;
	if (!initialized)
		initialize();
	
	if (!(frm && frm->getAU() && frm->getAU()->payload))
		return f; //don't forward frame
	
	if (firstFrame) {
		if (!renderer->open()) {
				dprintf_err("Visualizer: FATAL: could not open outputIO %s\n",renderer->getURL());
		} else
			firstFrame = false;
	}

	if (renderer->writeFrame(frm) < 1) { //couldn't be displayed!
		//don't forward frame
		if (renderer->getState() == IO::STREAMERR)
			::exit(0);
	} else
		f.push_front(frm);


	return f;
}


list < Frame * >Visualizer::close() {
	list < Frame * >f;
	return f;
}
