 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	Visualizer.hpp                                                *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#if !defined(_VISUALIZER_HPP_)
#define _VISUALIZER_HPP_

#if _MSC_VER > 1000
#pragma once
#endif				// _MSC_VER > 1000

#include "VideoAdaptor.hpp"
#include "VideoESInfo.hpp"
#include "MP4Decoder.hpp"
#include "io/VideoRenderer.hpp"

/**
 *
 * The Visualizer Adaptor allows access to VideoRenderer IO graphical output,
 * so from within an adaptor chain, the intermediate results can be displayed
 * either on SDL, Qt, or X11
 *
 *@author Michael Kropfberger
 *
 *@version $Id: Visualizer.hpp,v 1.3 2005/01/20 09:47:52 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class Visualizer:public VideoAdaptor {
public:
	typedef enum { Qt, SDL, X11	} OutputMethod;


	Visualizer(const VideoESInfo * es, OutputMethod out, UncompressedVideoFrame::ColorSpace colorSpace);

	void initialize();

	list < Frame * >adapt(Frame * frm);

	Adaptor *clone() {
		return new Visualizer(es, output, colorSpace);
	};

	list < Frame * >close();
	
	virtual ~ Visualizer() { dprintf_full("~Visualizer\n");
		close();
	};

	/** returns adaptation costs (CPU only). Displaying costs are a 10th of decoding costs*/
	u32 getTranscodingCosts() const {
		return (u32)(es->getWidth()*es->getHeight()*es->getFPS()/10);
	}

private:
	const VideoESInfo *es;
	OutputMethod output;
	int width;
	int height;
	int video_bpp;
	int ySize;
	int uvSize;
	bool firstFrame;

	VideoRenderer *renderer;
	UncompressedVideoFrame::ColorSpace colorSpace;
};

#endif				// !defined(_VISUALIZER_HPP_)
