/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	VorbisDecoder.hpp                                       *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _VORBIS_DECODER_HPP
#define _VORBIS_DECODER_HPP

#ifdef ENABLE_OGG

#include "AudioAdaptor.hpp"
#include "AudioESInfo.hpp"
#include "Frame.hpp"
#include "UncompressedAudioFrame.hpp"
#include "container/OGGContainerFile.hpp"
#include <vorbis/codec.h>
#include <vorbis/vorbisfile.h>
#include <vorbis/vorbisenc.h>

class VorbisDecoder : public AudioAdaptor {
public:

	/* Constructor
	 * @param outStream defines th IO class for writing the output
	 * @param autoFix if set to <code>false</code>, automatic insertion feature turned off
	 *
	 */
	VorbisDecoder(AudioESInfo* es);
	
	virtual ~VorbisDecoder();
	
	list < Frame * >adapt(Frame * frm);
	
 	Adaptor* clone() { return new VorbisDecoder(es); };

	void initialize();
	
	list < Frame * >close() {
		list < Frame * >tmp;
		return tmp;
	};


	/** returns adaptation costs (CPU only). This adaptor causes hd costs, which are 
	  * currently ignored. as CPU side effect we can only guess but it should depend
		* on the bytes of the video
	*/
	u32 getTranscodingCosts() const {
		return es->getAvgBandwidth()/8;
	}

private:
protected:

#define VORBIS_INPUT_BUFFER_SIZE	(5*8192)

  OGGStreamInfo *osi;
  vorbis_dsp_state *vd;

  unsigned char	*inputBuffer;
	unsigned char	*guardPtr;
	int		Status;
	unsigned long	FrameCount;

	u32 numInFrames;
	bool initialized;
	u32 lastTS;
	AU * pAU;
};

#endif
#endif
