/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	VorbisEncoder.hpp                                       *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef AFX_VOBISENCODER_HPP
#define AFX_VORBISENCODER_HPP

#include "AudioAdaptor.hpp"
#include "UncompressedAudioFrame.hpp"
#include "CompressedAudioFrame.hpp"
#include "AudioESInfo.hpp"
#include "VorbisDecoder.hpp"
#include "io/OGGMuxDemuxIO.hpp"

/**
 * Implementation of Adaptor that takes UncompressedAudioFrame (raw PCM) as input
 * and returns an CompressedAudioFrame (VORBIS aka OGG-VORBIS) as output.
 *
 *@author Thomas Kuglitsch
 *
 *@version $Id: VorbisEncoder.hpp,v 1.2 2006/10/30 15:34:51 tkuglits Exp $
 *
 *@short <short description>
 *
 */
class VorbisEncoder:public AudioAdaptor {
public:
  /**
   * Constructor.
   * Creates a VorbisEncoder instance. Do set attributes of pESInfo object carefully.
   * @param pESInfo Pointer to ESInfo object with proper encoding parameters.
   * @param targetBitRate is set in bits/second
   * Used attributes of ESInfo: avgBandwidth
   */
  VorbisEncoder(AudioESInfo * pESInfo, u32 targetBitRate);
  
  /** Destructor.	*/
  virtual ~ VorbisEncoder();

	/**
	 * raw PCM Frame in - VorbisFrame out.
	 * Input raw PCM Frame object will be deleted in this method!
	 * Adaptor implementation for raw PCM Frame objects.
	 * @param *frm Pointer to raw PCM Frame object.
	 * @return List containing VorbisFrame objects. Will contain in general
	 * only one VorbisFrame object
	 */
	list < Frame * >adapt(Frame * frm);
	
	/**
	 * Flush all buffered VorbisFrame objects (if any).
	 *	 @return List of VorbisFrame objects
	 */
	list < Frame * >close();

	void initialize();

  /**
   * Does nothing exept returning NULL
   */
  Adaptor* clone();
	
	u32 getTranscodingCosts() const { return 1000; };

private:
  vorbis_info*      vi;
  vorbis_comment*   vc;
  vorbis_dsp_state* vd;
  vorbis_block*     vb;



	/** Handle to XviD encoder instance.*/
	char *mpEncoderHandle;
	
	/** Size of decoded payload of Frame object in bytes.	*/
	u32 mFrameSize;
	
	/** Internal output buffer for encoder. */
	char mpOutBuf[MAX_BUFFER_SIZE];

	/** Internal input buffer for encoder. */
	char mpInBuf[MAX_BUFFER_SIZE];
	u32 inBufSize;

	/** Contains the number of frames encoded.	*/
	u32 muiFrameNumber;

	bool firstFrame;

	/** The target bitrate for the encoder */
	u32 muiTargetBitrate;

	enum CodecID outputFormat;
};

#endif

