 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	YUVColorReductionAdaptor.hpp                                  *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef __YUV_COLORREDUCTION_ADAPTOR_HPP__
#define __YUV_COLORREDUCTION_ADAPTOR_HPP__

#include "VideoAdaptor.hpp"
#include "UncompressedVideoFrame.hpp"
#include "VideoESInfo.hpp"

/**
 * Implementation of Adaptor that applies color reduction on YUVFrame objects.
 * In addition average bitrate of the ESInfo object is reduced by 20%.
 *
 *@author Bernhard Penz
 *
 *@version $Id: YUVColorReductionAdaptor.hpp,v 1.6 2005/06/19 14:47:41 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class YUVColorReductionAdaptor:public VideoAdaptor {
  public:
	/**
		Constructor.

		Takes input frame size from pESInfo, and reduces average bitrate setting of ESInfo by 20%.
		@param pESInfo Pointer to ESInfo object, containing valid size information
			about the input YUVFrame objects.
	*/
    YUVColorReductionAdaptor(VideoESInfo * pESInfo);
	/**
		Destructor.
	*/
    virtual ~ YUVColorReductionAdaptor();

	/**
		YUVFrame frm in - list of YUVFrame objects out.

		Output list will contain one YUVFrame object only.

		@param frm Pointer to uncompressed YUVFrame object
		@return List of YUVFrame objects
	*/
    list < Frame * >adapt(Frame * frm);
	/**
		Flush all buffered YUVFrame objects (if any).

		@return List of YUVFrame objects
	*/
    list < Frame * >close();
	/**
		Not implemented.

		@return Defaults to NULL.
	*/
    Adaptor *clone();

    void initialize();
		/** returns adaptation costs (CPU only).
		Throwing away color is extremely cheap.. memset per frame 
		0.4% of decoding costs*/
	u32 getTranscodingCosts() const {
			return (u32)(((VideoESInfo*)es)->getWidth() * ((VideoESInfo*)es)->getHeight()
			* ((VideoESInfo*)es)->getFPS() * 0.008);
	}
  private:
	/**
		Variable showing the starting point of U and V component in 
		YUVFrame object payload, i.e. size of Y component.
	*/
     u32 muiUVStart;
	/**
		Size of U and V component (together) of YUVFrame object payload.
	*/
    u32 muiUVSize;
};

#endif				// __YUV_COLORREDUCTION_ADAPTOR_HPP__
