 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	YUVScalingAdaptor.hpp                                         *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef __YUV_SCALING_ADAPTOR_HPP__
#define __YUV_SCALING_ADAPTOR_HPP__

#include "VideoAdaptor.hpp"
#include "UncompressedVideoFrame.hpp"
#include "ESInfo.hpp"

/**
 *
 *@author Bernhard Penz and Michael Kropfberger
 *
 *@version $Id: YUVScalingAdaptor.hpp,v 1.4 2004/09/06 08:48:09 pschojer Exp $
 *
 *@short Implementation of Adaptor that applies spatial transformation on YUVFrame objects.
 * Up- and downscaling is implemented using SDL 
 *
 */
class YUVScalingAdaptor:public VideoAdaptor {
  public:
	/**
	 * Constructor.
	 *
	 * Takes input frame size from pESInfo, and sets arbitrary output
	 * frame size according to puiWidth/puiHeight. 
	 * @param pESInfo Pointer to ESInfo object, containing valid size 
	 * information about the input YUVFrame objects.  
	 * @param width Pointer to target width of output YUVFrame objects. 
	 * @param height Pointer to target height of output YUVFrame objects.
	 */
    YUVScalingAdaptor(VideoESInfo * pESInfo, u32 width, u32 height);

    virtual ~ YUVScalingAdaptor();

	/**
	 * YUVFrame frm in - list of YUVFrame objects out.
	 *
	 * Output list will contain one YUVFrame object only.
	 * If the incoming frm has already the correct size, no scaling is done
	 *
	 * @param frm Pointer to uncompressed YUVFrame object
	 * @return List of YUVFrame objects
	*/
	list < Frame * >adapt(Frame * frm);
	
	/**
	 *Flush all buffered YUVFrame objects (if any).
	 *
	 *@return List of YUVFrame objects
	 */
	list < Frame * >close();
	
	/**
	 *Not implemented.
	 *
	 *@return Defaults to NULL.
	 */
	Adaptor *clone();
	
	void initialize();
	/** returns adaptation costs (CPU only) 
	  * YUVscaling is expensive, nearly twice as much as encoding! */
	u32 getTranscodingCosts() const {
		return (u32)(es->getWidth()*es->getHeight()*es->getFPS()*3.72);
	}
private:
	/**
	 *Width of input YUVFrame objects.
	 */
	u32 muiWidth;

	/**
	 *Height of input YUVFrame objects.
	 */
	u32 muiHeight;

	/**
	 *Width of output YUVFrame objects.
	 */
	u32 muiTargetWidth;

	/**
	 *Height of output YUVFrame objects.
	 */
	u32 muiTargetHeight;

	
	/**
	 *Scalefactor which will be applied on width of YUVFrame objects.
	 *Divide by 1000000 to obtain the real scalefactor!
	 */
	u32 muiScaleFactorWidth;

	/**
	 *Scalefactor which will be applied on height of YUVFrame objects.
	 *Divide by 1000000 to obtain the real scalefactor!
	 */
	u32 muiScaleFactorHeight;
};

#endif				/// __YUV_SPATIALREDUCTION_ADAPTOR_HPP__
