 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	YUVSpatialReductionAdaptor.hpp                                *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef __YUV_SPATIALREDUCTION_ADAPTOR_HPP__
#define __YUV_SPATIALREDUCTION_ADAPTOR_HPP__

#include "VideoAdaptor.hpp"
#include "UncompressedVideoFrame.hpp"
#include "ESInfo.hpp"

/**
 * Implementation of Adaptor that applies spatial transformation on YUVFrame objects.
 * Currently only downscaling is implemented using nearest neighbor hood search.
 *
 *@author Bernhard Penz
 *
 *@version $Id: YUVSpatialReductionAdaptor.hpp,v 1.4 2004/06/14 09:02:46 pschojer Exp $
 *
 *@short <short description>
 *
 */
class YUVSpatialReductionAdaptor:public VideoAdaptor {
  public:
	/**
		Constructor.

		Takes input frame size from pESInfo, and sets output frame size according to
		puiWidth/puiHeight. 
		@param pESInfo Pointer to ESInfo object, containing valid size information
			about the input YUVFrame objects.
		@param puiWidth Pointer to target width of output YUVFrame objects.
			Must be divisible by 2!!!!
		@param puiHeight Pointer to target height of output YUVFrame objects.
			Must be divisible by 2!!!!
	*/
    YUVSpatialReductionAdaptor(VideoESInfo * pESInfo, u32 * puiWidth,
			       u32 * puiHeight);
    virtual ~ YUVSpatialReductionAdaptor();

	/**
		YUVFrame frm in - list of YUVFrame objects out.

		Output list will contain one YUVFrame object only.

		@param frm Pointer to uncompressed YUVFrame object
		@return List of YUVFrame objects
	*/
    list < Frame * >adapt(Frame * frm);
	/**
		Flush all buffered YUVFrame objects (if any).

		@return List of YUVFrame objects
	*/
    list < Frame * >close();

	/**
		Not implemented.

		@return Defaults to NULL.
	*/
    Adaptor *clone();

    void initialize();
	/** returns adaptation costs (CPU only) 
	  * YUVscaling is expensive, this is an optimized impl., 30% more expensive than decoding */
	u32 getTranscodingCosts() const {
		return (u32)(es->getWidth()*es->getHeight()*es->getFPS()*1.3);
	}
  private:
	/**
		Width of input YUVFrame objects.
	*/
     u32 muiWidth;
	/**
		Height of input YUVFrame objects.
	*/
    u32 muiHeight;
	/**
		Width of output YUVFrame objects.
	*/
    u32 muiTargetWidth;
	/**
		Height of output YUVFrame objects.
	*/
    u32 muiTargetHeight;

	/**
		Scalefactor which will be applied on width of YUVFrame objects.
		Divide by 1000000 to obtain the real scalefactor!
	*/
    u32 muiScaleFactorWidth;
	/**
		Scalefactor which will be applied on height of YUVFrame objects.
		Divide by 1000000 to obtain the real scalefactor!
	*/
    u32 muiScaleFactorHeight;
};

#endif				/// __YUV_SPATIALREDUCTION_ADAPTOR_HPP__
