 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	YUVinYUVoverlay.cpp                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "YUVinYUVoverlay.hpp"
#include "UncompressedVideoFrame.hpp"


YUVinYUVoverlay::YUVinYUVoverlay(VideoESInfo* esi, YUVStreamIO *watermarkStream, int x, int  y, bool loopWatermark) {

	es = esi;
	waterStream = watermarkStream;
	posX = x;
  posY = y;
	this->loopWatermark = loopWatermark;
	waterFrame = NULL;
	numFrames = 0;
	lastAU.payload = NULL;
	strcpy(name,"YUVinYUVoverlay");
}


YUVinYUVoverlay::~YUVinYUVoverlay() {
	delete waterStream;
	if (waterFrame) {
		waterFrame->unsetAU();
		delete waterFrame;
	}
}


void YUVinYUVoverlay::initialize() {
}

void YUVinYUVoverlay::setESInfo(ESInfo * esi) {
	es = (VideoESInfo*)esi;
}

ESInfo *YUVinYUVoverlay::getESInfo() {
    return es;
}


/***************************************************************/
list < Frame * > YUVinYUVoverlay::adapt(Frame * frm) {
	list < Frame * >tmp;
	u32 thisCTS;
	int thisSec;
	int inW,inH,waterW,waterH;
  u8 *inPay, *waterPay;
  UncompressedVideoFrame *inFrame = (UncompressedVideoFrame*)frm;

	//call open on first frame
	if (numFrames == 0)
		if (!waterStream->open()) {
			dprintf_err("YUVinYUVoverlay: FATAL: not writing, since another IO class is accessing file %s\n",waterStream->getURL());
			//standard forwarding adaptor behaviour
			tmp.push_front(frm);
			return tmp;
		}

	thisCTS = inFrame->getAU()->cts;
	thisSec = (int)floor((double)thisCTS / es->getMediaTimeScale());
	dprintf_full("YUVinYUVoverlay::adapt frame %i CTS %i, VOP increment %i\n", numFrames, thisCTS,es->getVOPTimeIncrement());

	//FIXME: assert(isVideoFrame)

	waterFrame = (UncompressedVideoFrame*)waterStream->getFrame();

	if (!waterFrame && loopWatermark) {	
		//loop & reget first frame
		//		waterStream->setToFrameNumber(0);
		waterStream->close();
		waterStream->open();
		waterFrame = (UncompressedVideoFrame*)waterStream->getFrame();
		assert(waterFrame);
	}

	if (waterFrame) {
		inW = inFrame->getWidth();
		inH = inFrame->getHeight();
		waterW = waterFrame->getWidth();
		waterH = waterFrame->getHeight();
		int inCromU = inW*inH;
		int waterCromU = waterW*waterH;
		int inCromV = inCromU + (inW*inH)/4;
		int waterCromV = waterCromU + (waterW*waterH)/4;
		
		inPay = inFrame->getAU()->payload;
		waterPay = waterFrame->getAU()->payload;
		for (int i=0; i < waterW; i++)
			for (int j=0; j < waterH; j++)
				memcpy(&inPay[(posY+j)*inW+posX], &waterPay[j*waterW], waterW);
		
		for (int i=0; i < waterW/2; i++)
			for (int j=0; j < waterH/2; j++) {
				memcpy(&inPay[posX/2+inCromU+(posY/2+j)*(inW/2)], &waterPay[waterCromU+j*(waterW/2)], waterW/2);
				memcpy(&inPay[posX/2+inCromV+(posY/2+j)*(inW/2)], &waterPay[waterCromV+j*(waterW/2)], waterW/2);
			}
	}

	//standard forwarding adaptor behaviour
	tmp.push_front(frm);
	return tmp;
}
