 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: YUVinYUVoverlay.hpp                                           *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _YUVinYUVoverlay_HPP
#define _YUVinYUVoverlay_HPP
#include "VideoAdaptor.hpp"
#include "VideoESInfo.hpp"
#include "io/YUVStreamIO.hpp"
#include "io/DevNull.hpp"


/**
 * Adaptor class YUVinYUVoverlay provides a graphical overlaying (destructive multiplexing)
 * from one YUV stream (eg. a watermark or a small video preview) into a larger main YUV stream.
 *
 * NOTE: there is no alpha channel support, yet! but the feature should be fairly easy to add:
 * eg. assume the color black (Y:0,U:0,V:0) to be the alpha, so don't copy it...
 * but this will drastically slow down the whole overlaying process!
 *
 * @author Michael Kropfberger
 *
 * @version $Id: YUVinYUVoverlay.hpp,v 1.2 2006/01/20 15:37:17 mkropfbe Exp $
 *
 * @short <short description>
 *
 */
class YUVinYUVoverlay : public VideoAdaptor {
public:

	/* Constructor
	 * @param watermarkStream defines th IO class for writing the output
	 * @param loopWatermark if set to <code>true</code>, the watermarkStream is replayed forever
	 * @param x and y depict the coordiates, where the watermarkStream should be overlayed
	 *
	 */
	YUVinYUVoverlay(VideoESInfo* esi, YUVStreamIO *watermarkStream, int x, int  y, bool loopWatermark = true);
	
	virtual ~YUVinYUVoverlay();
	
	list < Frame * >adapt(Frame * frm);
	
 	Adaptor* clone() { return new YUVinYUVoverlay(es, (YUVStreamIO*)waterStream->clone(), posX, posY, loopWatermark); };

	void initialize();
	
	list < Frame * >close() {
		list < Frame * >tmp;
		return tmp;
	};

	/** Sets a reference to an ESInfo object. Adaptors are
		allowed to change ESInfo objects. If you don't want that,
		pass an es->clone() 
	*/ 
	void setESInfo(ESInfo * esi);

	ESInfo * getESInfo();

	/** returns adaptation costs (CPU only). This adaptor causes hd costs, which are 
	  * currently ignored. as CPU side effect we can only guess but it should depend
		* on the bytes of the video
	*/
	u32 getTranscodingCosts() const {
		return es->getAvgBandwidth()/8;
	}

private:
protected:
	VideoESInfo *es;
  YUVStreamIO *waterStream;
	int posX, posY;
	bool loopWatermark;
	AU lastAU;
	u8* lastPayload;
	UncompressedVideoFrame *waterFrame;
	int numFrames;
};
#endif

