 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	ffmpegInterface.h                                             *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/



#ifndef FFMPEG_INTERFACE_H
#define FFMPEG_INTERFACE_H



//take care! copied from global.hpp !!!
#define MAX_FRAME_SIZE 65536*8 //512KByte per frame
#define ERR_OK 0
#define ERR -1
#define ERR_FILE -500

#include "avcodec.h"

#ifdef USE_FASTMEMCPY
#include "fastmemcpy.h"
#endif

#ifdef HAVE_MMX
#include "i386/mmx.h"
#endif
#ifdef ISOMP4
#include "ISOMovies.h" //for the sake of u8 only
#else
typedef unsigned char u8;
#endif


	/**
		Available Colorspaces via interface, excerpt from available colorspaces in ffMPEG.
		- ColorSpaceDefault: Default is set to YVV420P.
		- ColorSpaceYV12: YV12 format, U and V swapped here!
		- ColorSpaceI420: I420 format, can be played e.g. via Dicas MPEGable player.
		- ColorSpaceRGB24: Standard RGB colorspace
		- ColorSpaceRGB32: RGB Colorspace with alpha channel
		- ColorSpaceRGB565: RGB with 16 bit
	
	*/
	typedef enum FFMPEGenumColorSpace{
		FFMPEGColorSpaceUnknown = -1, /*@PIX_FMT_ANY, */
		FFMPEGColorSpaceDefault = PIX_FMT_YUV420P,
		FFMPEGColorSpaceRGB24 =   PIX_FMT_RGB24,
		FFMPEGColorSpaceYV12 =    100,  /* vitooki does U/V swapping now, ffmpeg seems to output in YV12 */
		FFMPEGColorSpaceI420 =    PIX_FMT_YUV420P, /* vitooki just copies the YVU swapped ffmpeg result */
		FFMPEGColorSpaceRGB32 =   PIX_FMT_RGBA32,
		FFMPEGColorSpaceRGB565 =   PIX_FMT_RGB565,
		FFMPEGColorSpaceGRAY8 =   PIX_FMT_GRAY8
	} ffmpegColorSpace;

#ifdef __cplusplus
extern "C" {
#endif


#ifndef WINCE

/**
	Initialize the decoder for a specific video size.

	A handle to	a decoder will be created for the speficied video size

	@see DecoderClose()
	@param pDecoderHandle Handle to decoder.
	@param uiWidth Width of video.
	@param uiHeight Height of video.
	@return ERR_OK on success, ERR otherwise.
*/
int FFMPEGDecoderInit(char **pDecoderHandle, enum CodecID codecID, unsigned int uiWidth, unsigned int uiHeight);

/**
	Close Decoder.

	Frees occupied resources and destroys the decoder.

	@see DecoderInit()
	@param pDecoderHandle Handle to decoder.
*/
void FFMPEGDecoderClose(char **pDecoderHandle);

/**
	Query width of the frame.

	Call DecoderInit before use.

	@param pDecoderHandle Handle to decoder.
	@see DecoderInit().
	@return 0 if width of frame is unknown, width of decoded frame otherwise.
*/
int FFMPEGDecoderGetWidth(char *pDecoderHandle);

/**
	Query height of the frame.

	Call DecoderInit before use.
	
	@param pDecoderHandle Handle to decoder.
	@see DecoderInit().
	@return 0 if height of frame is unknown, height of decoded frame otherwise.
*/
int FFMPEGDecoderGetHeight(char *pDecoderHandle);

/**
	Decode a single frame.

	Call DecoderInit before, otherwise decoding will fail.

	@param pDecoderHandle Handle to decoder.
	@param pCompressedFrame Pointer to raw data of compressed frame.
	@param iCFSize Size of pFrame data.
	@param pYUVFrame Pointer to decoded raw data.
	@param eColorSpace Colorspace of decoded data.
	@see xvidColorSpace
	@return bytes read from pCompressedFrame in case of success, 0 on failure.
*/
int FFMPEGDecodeFrame(char *pDecoderHandle, char *pCompressedFrame, 
				int iCFSize, char* pYUVFrame, ffmpegColorSpace eColorSpace);

/**
	Initialize the encoder.

	Initialize the encoder with height and width of the YUV input and with the desired bitrate.
	@param pEncoderHandle Handle to encoder instance.
	@param iWidth Width of input/output frames
	@param iHeight Height of input/output frames
	@param iBitrate Desired target bitrate, may vary from defined value.
	@param id wished output format
	@param iFramerate Desired target framerate, must match frame rate of input Frame objects.
	@param keyFrameInterval eg. every 30th frame has to be an I_VOP
	@return ERR_OK on success, ERR otherwise
*/
	int FFMPEGEncoderInit( char **pEncoderHandle, enum CodecID id, int iWidth, int iHeight, int iBitrate,  int iFramerate, int numBFrames, int keyFrameInterval);

/**
	Frees occupied resources and destroys the encoder.

	@see EncoderInit()
	@param pEncoderHandle Handle to encoder.
*/
void FFMPEGEncoderClose(char **pEncoderHandle);

/**
	Pass a single frame to the encoder.

	The encoder does not force keyframes.

	@param pEncoderHandle Hande to Encoder instance
	@param pYUVFrame Pointer to raw data
	@param pCompressed Pointer to memory area where compressed
		output frames shall be written to.
	@see EncoderInit()
	@param eColorSpace Colorspace of input stream.
	@param forceIFrame Set this t0 1 to signal the encoder to write a I frame, useful for key frame generation. 
		WARNING_ Release build crashes if first frame is not forced to be an I-frame.
	@see xvidColorSpace
	@return Size of frame written to outBuf.
*/
int FFMPEGEncodeFrame(char *pEncoderHandle, char *pYUVFrame, 
				char *pCompressed, ffmpegColorSpace eColorSpace, int forceIFrame);

/**
	Read bitstream stream from disk and store encoded result on disk also.

	@param in Filename pointing to input stream.
	@param width Width of input frames.
	@param height Height of input frames.
	@param bitrate Desired target bitrate of the encoded stream
	@param out Filename where MP4 VES shall go to.
	@param eColorSpace Colorspace of input stream.
	@see xvidColorSpace
	@return o on successm != 0 on error.
*/
int FFMPEGEncodeFile(char *in, int width, int height, int bitrate, ffmpegColorSpace eColorSpace, char *out);






//AUDIO STUFF
/**
        Initialize the decoder for a audio (currently MP3 only).

        A handle to     a decoder will be created for the speficied audio file

        @see DecoderClose()
        @param pDecoderHandle Handle to decoder.
        @return ERR_OK on success, ERR otherwise.
*/
int FFMPEGaudioDecoderInit(char **pDecoderHandle, enum CodecID codecID);

/**
        Close Decoder.

        Frees occupied resources and destroys the decoder.

        @param pDecoderHandle Handle to decoder.
*/
void FFMPEGaudioDecoderClose(char **pDecoderHandle);

/**
        Decode a single audio frame.

        Call FFMPEGaudioDecoderInit before, otherwise decoding will fail.

        @param pDecoderHandle Handle to decoder.
        @param pCompressedFrame Pointer to raw data of compressed frame.
        @param iCFSize Size of pFrame data.
        @param PCMFrame Pointer to decoded raw data.
        @return bytes read from pCompressedFrame in case of success, 0 on failure.
*/
int FFMPEGaudioDecodeFrame(char *pDecoderHandle, char *pCompressedFrame,
                                int iCFSize, char* PCMFrame, int* mFrameSize);

/**
        Initialize the audio encoder.

        @param pEncoderHandle Handle to encoder instance.
        @param iBitrate Desired target bitrate, may vary from defined value.
        @param iSampleRate Desired target sample rate, must match sample rate of input Frame objects.
				@param iChannels, either 1 (mono) or 2 (Stereo)
				@param iBits, either 8 or 16
        @return ERR_OK on success, ERR otherwise
*/
	int FFMPEGaudioEncoderInit( char **pEncoderHandle, int iBitrate,  int iSampleRate, int iChannels, int iBits);

/**
        Frees occupied resources and destroys the encoder.

        @see EncoderInit()
        @param pEncoderHandle Handle to encoder.
*/
void FFMPEGaudioEncoderClose(char **pEncoderHandle);

/**
        Pass a single audio frame to the encoder.

        @param pEncoderHandle Hande to Encoder instance
        @param PCMFrame Pointer to raw data
        @param pCompressed Pointer to memory area where compressed
                output frames shall be written to.
        @see AudioEncoderInit()
        @return Size of frame written to outBuf.
*/
	int FFMPEGaudioEncodeFrame(char *pEncoderHandle, char *PCMFrame, char *pCompressed);


#endif //wince



#ifdef __cplusplus
}
#endif

#endif


