 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	xvidInterface.h                                               *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef XVID_INTERFACE_H
#define XVID_INTERFACE_H



#ifdef WINCE
//for xvid on windows ce
#define EDGE_SIZE	16
#endif


#ifdef __cplusplus
extern "C" {
#endif

//take care! copied from global.hpp !!!
#define MAX_FRAME_SIZE 65536*8 //512KByte per frame
#define ERR_OK 0
#define ERR -1
#define ERR_FILE -500


	/**
		Available Colorspaces via interface, excerpt from available colorspaces in XviD.
		- ColorSpaceDefault: Default is set to YV12, XviDs native colorspace.
		- ColorSpaceYV12: YV12 format, U and V swapped here!
		- ColorSpaceI420: I420 format, can be played e.g. via Dicas MPEGable player.
		- ColorSpaceRGB24: Standard RGB colorspace
		- ColorSpaceRGB32: RGB Colorspace with alpha channel
	
	*/

/*	
defined colorspaces in XVID:
#define XVID_CSP_PLANAR   (1<< 0) //4:2:0 planar (==I420, except for pointers/strides) 
#define XVID_CSP_USER	  XVID_CSP_PLANAR
#define XVID_CSP_I420     (1<< 1)  //4:2:0 planar 
#define XVID_CSP_YV12     (1<< 2)  //4:2:0 planar 
#define XVID_CSP_YUY2     (1<< 3)  //4:2:2 packed 
#define XVID_CSP_UYVY     (1<< 4)  //4:2:2 packed 
#define XVID_CSP_YVYU     (1<< 5)  //4:2:2 packed 
#define XVID_CSP_BGRA     (1<< 6)  //32-bit bgra packed 
#define XVID_CSP_ABGR     (1<< 7)  //32-bit abgr packed 
#define XVID_CSP_RGBA     (1<< 8)  //32-bit rgba packed 
#define XVID_CSP_ARGB     (1<<15)  //32-bit argb packed 
#define XVID_CSP_BGR      (1<< 9)  //24-bit bgr packed 
#define XVID_CSP_RGB555   (1<<10)  //16-bit rgb555 packed 
#define XVID_CSP_RGB565   (1<<11)  //16-bit rgb565 packed 
#define XVID_CSP_SLICE    (1<<12)  //decoder only: 4:2:0 planar, per slice rendering 
#define XVID_CSP_INTERNAL (1<<13)  //decoder only: 4:2:0 planar, returns ptrs to internal buffers 
#define XVID_CSP_NULL     (1<<14)  //decoder only: dont output anything 
#define XVID_CSP_VFLIP    (1<<31)  //vertical flip mask 
*/
	
	typedef enum XVIDenumColorSpace {
		XVIDColorSpaceUnknown = -1,
		XVIDColorSpaceRGB24 = 512 /*0*/, 
		XVIDColorSpaceYV12 = 4 /*1*/,
		XVIDColorSpaceI420 = 2 /*4*/, 
		XVIDColorSpaceRGB32 = 256 /*1000*/} xvidColorSpace;
		


/**
	Initialize the decoder for a specific video size.

	A handle to	a decoder will be created for the speficied video size

	@see DecoderClose()
	@param pDecoderHandle Handle to decoder.
	@param uiWidth Width of video.
	@param uiHeight Height of video.
	@return ERR_OK on success, ERR otherwise.
*/
int XVIDDecoderInit2(char **pDecoderHandle, unsigned int uiWidth, unsigned int uiHeight);

/**
	Close Decoder.

	Frees occupied resources and destroys the decoder.

	@see DecoderInit()
	@param pDecoderHandle Handle to decoder.
*/
void XVIDDecoderClose(char **pDecoderHandle);

/**
	Query width of the frame.

	Call DecoderInit before use.

	@param pDecoderHandle Handle to decoder.
	@see DecoderInit().
	@return 0 if width of frame is unknown, width of decoded frame otherwise.
*/
int XVIDDecoderGetWidth(char *pDecoderHandle);

/**
	Query height of the frame.

	Call DecoderInit before use.
	
	@param pDecoderHandle Handle to decoder.
	@see DecoderInit().
	@return 0 if height of frame is unknown, height of decoded frame otherwise.
*/
int XVIDDecoderGetHeight(char *pDecoderHandle);

/**
	Decode a single frame.

	Call DecoderInit before, otherwise decoding will fail.

	@param pDecoderHandle Handle to decoder.
	@param pCompressedFrame Pointer to raw data of compressed frame.
	@param iCFSize Size of pFrame data.
	@param pYUVFrame Pointer to decoded raw data.
	@param eColorSpace Colorspace of decoded data.
	@see xvidColorSpace
	@return bytes read from pCompressedFrame in case of success, 0 on failure.
*/
int XVIDDecodeFrame(char *pDecoderHandle, char *pCompressedFrame, 
				int iCFSize, char* pYUVFrame, xvidColorSpace eColorSpace);




#ifdef WINCE
int XVIDDecodeFrameCE(char *pCompressedFrame, int iCFSize, unsigned char * &y, unsigned char * &u, unsigned char * &v);
#endif


/**
	Read a mp4 VES stream with given size from disk and store decoded result also
	on disk.

	@param in Filename pointing to mp4 VES file.
	@param out Filename where output shall be written.
	@param uiWidth Width of MPEG4 frames.
	@param uiHeight Height of MPEG4 frames.
	@return Size of bytes successful written to pYUVFrame.
	@param eColorSpace Colorspace of decoded data.
	@see xvidColorSpace
	@see DecoderInit()
*/
int XVIDDecodeFile2(char *in, char *out, unsigned int uiWidth, 
				unsigned int uiHeight, xvidColorSpace eColorSpace);

/**
	Initialize the encoder.

	Initialize the encoder with height and width of the YUV input and with the desired bitrate.
	@param pEncoderHandle Handle to encoder instance.
	@param iWidth Width of input/output frames
	@param iHeight Height of input/output frames
	@param iBitrate Desired target bitrate, may vary from defined value.
	@param iFramerate Desired target framerate, must match frame rate of input Frame objects.
	@return ERR_OK on success, ERR otherwise
*/
int XVIDEncoderInit( char **pEncoderHandle, int iWidth, int iHeight, int iBitrate,  int iFramerate, int numBFrames);

/**
	Frees occupied resources and destroys the encoder.

	@see EncoderInit()
	@param pEncoderHandle Handle to encoder.
*/
void XVIDEncoderClose(char **pEncoderHandle);

/**
	Pass a single frame to the encoder.

	The encoder does not force keyframes.

	@param pEncoderHandle Hande to Encoder instance
	@param pYUVFrame Pointer to raw data
	@param pCompressed Pointer to memory area where compressed
		output frames shall be written to.
	@see EncoderInit()
	@param eColorSpace Colorspace of input stream.
	@param forceIFrame Set this t0 1 to signal the encoder to write a I frame, useful for key frame generation. 
		WARNING_ Release build crashes if first frame is not forced to be an I-frame.
	@see xvidColorSpace
	@return Size of frame written to outBuf.
*/
int XVIDEncodeFrame(char *pEncoderHandle, char *pYUVFrame, 
				char *pCompressed, xvidColorSpace eColorSpace, int forceIFrame);

/**
	Read bitstream stream from disk and store encoded result on disk also.

	@param in Filename pointing to input stream.
	@param width Width of input frames.
	@param height Height of input frames.
	@param bitrate Desired target bitrate of the encoded stream
	@param out Filename where MP4 VES shall go to.
	@param eColorSpace Colorspace of input stream.
	@see xvidColorSpace
	@return o on successm != 0 on error.
*/
int XVIDEncodeFile(char *in, int width, int height, int bitrate, xvidColorSpace eColorSpace, char *out);


#ifdef WINCE
void XvidSetWidth(int width);
void XvidSetHeight(int height);
#endif

#ifdef __cplusplus
}
#endif

#endif
