 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	CacheManager.hpp                                              *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

// Definition: Abstract Cache Manager Class //

#ifndef _CACHEMANAGER_HPP_
#define _CACHEMANAGER_HPP_

#include "global.hpp"
#ifdef ISOMP4
#include "ISOMovies.h"
#endif
#include "VMutex.hpp"
#include "net/ProxySession.hpp"
class ContainerInfo;
class Url;
class MetaObject;

/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: CacheManager.hpp,v 1.11 2005/04/27 10:21:50 klschoef Exp $
 *
 *@short <short description>
 *
 */
class CacheManager {
protected:
    u32 cacheSize;
    u32 actSize;
    u32 requests;
    u32 hits;
    u64 byteRequested;
    u64 byteHits;
    double qualityHits;
    VMutex lock;
		/** set to true if the cachemanager
		  * is used in a server and you don't want the cache manager
			* to compare the IP address of the request with it's local IP
			*/
		bool serverMode;
	/** @param testPriorDelete tests if enough disk space can be freed
	 * so that the actSize is below the cacheSize or if too many objects are locked
	 * If this test fails, no files are removed form the cache and false is returned */
    virtual bool replace(bool testPriorDelete) = 0;
public:
	virtual ~ CacheManager() {};
	virtual void putVideo(ContainerInfo * video,u32 dskSpaceReserved=0) = 0;
	virtual ContainerInfo *getVideo(const Url *URL) = 0;
	virtual MetaObject* findMetaObject(const Url* url)=0;
	virtual void deleteMetaObject(const Url* url)=0 ;
	virtual u32 getCacheSize() {
		return cacheSize;
	};
	virtual u32 getMaxObjectSize() {
		return cacheSize;
	}
	void lockCache() {lock.lock();};
	void unlockCache() {lock.release();};
	/** doesn't lock cache, when calling you must do this outside
	  * this function lockCache and unlockCache
	  */
	bool reserveDiskSpace(u32 dskSpace) {
		prxStat.insertsIntoCM++;
		if(dskSpace<getMaxObjectSize()) {
			actSize+=dskSpace;
			if(replace(true)) {
				return true;
			}
			prxStat.rejectedDueToLocking++;
			actSize-=dskSpace;
		}
		else {
			prxStat.rejectedDueToSize++;
		}
		return false;
	};
	/** call this when a reservation has to be cancelled */
	void freeDiskSpace(u32 dskSpace) {
		assert(actSize>=dskSpace);
		actSize-=dskSpace;
		// no need to update prxStats, this happens in ProxySession
	}
	virtual u32 getActSize() {
		return actSize;
	};
	virtual u32 getRequests() {
		return requests;
	};
	virtual u32 getHits() {
		return hits;
	};
	virtual u64 getByteRequested() {
		return byteRequested;
	};
	virtual u64 getByteHits() {
		return byteHits;
	};
	virtual double getQualityHits() {
		return qualityHits;
	}

	virtual void getContent() = 0;

	virtual void saveCacheIndexToDir(const char* dir) = 0;

	void saveStatisticTo(FILE* fp) {		
	dprintf_full("CacheManager::saveStatisticTo\n");
		if(fp) {
			fprintf(fp,"Size: %u\nRequests: %u\nHits: %u\nBytesReq(KB): %u\n BytesHit(KB); %u\n",
							cacheSize, requests, hits, (u32)(byteRequested/1024), (u32)(byteHits/1024));
		}
	}
};

#endif				/* _CACHEMANAGER_HPP_ */
