 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	CacheManagerLRU.hpp                                           *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

// Definition: LRU Cache Manager Class //

#ifndef _CACHEMANAGERLRU_HPP_
#define _CACHEMANAGERLRU_HPP_

#include "CacheManager.hpp"
#include "MetaObject.hpp"
class Url;

/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: CacheManagerLRU.hpp,v 1.6 2006/01/20 15:37:17 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class CacheManagerLRU:public CacheManager {
protected:
	/** list of video objects, which will be replaced */
	list < ContainerInfo * >lrulist;
	/** stores metaobjects */
	list < MetaObject * > metaObjects;
	/** @param testPriorDelete tests if enough disk space can be freed
	 * so that the actSize is below the cacheSize or if too many objects are locked
	 * If this test fails, no files are removed form the cache and false is returned */
	bool replace(bool testPriorDelete);
	void internalConsistencyCheck();
	MetaObject* findMetaObject(const Url* url);
	void deleteMetaObject(const Url* url) ;
	/** requires mutex to be locked */
	u32 getSizeOfNotLockedObjects();
public:
	CacheManagerLRU(u32 size, bool serverMode);
	~CacheManagerLRU();
	/** inserts the video, if the video is smaller than
	 * the total cache size. If it is bigger, the video
	 * will be destroyed and its memory freed!
	 * LRU doesn't take advantage of adaptation. We just
	 * have full or zero quality
	 */
	void putVideo(ContainerInfo * video,u32 dskSpaceAlreadyReserved=0);
	/** returns an ContainerInfo for the given url,
	 * if the URL is cached. lock count is increased
	 * by one. If the client/session object is finished with the ContainerInfo
	 * object it has to call info->unlock()!!!!
	 */
	ContainerInfo *getVideo(const Url *URL);
	u32 getCacheSize() const;
	u32 getActSize() const;
	u32 getRequests() const;
	u32 getHits() const;
	u64 getByteRequested() const;
	u64 getByteHits() const;
	double getQualityHits() const;
	void getContent();
	
	void saveCacheIndexToDir(const char* dir);
};

#endif				/* _CACHEMANAGERLRU_HPP_ */
