 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	ContainerDemux.hpp                                            *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PS_VITOOKI_CONTAINERDEMUX_HPP_
#define _PS_VITOOKI_CONTAINERDEMUX_HPP_

class ContainerInfo;
class Url;
class Adaptor;
class DataChannel;
#include "ESInfo.hpp"

#ifndef ISOMP4
#include "global.hpp"
#else
#include "ISOMovies.h"
#endif

/**
 *
 * is used for simple demultiplexing of mp4 streams, so adaptors can be
 * easily applied for eg. transcoding
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: ContainerDemux.hpp,v 1.9 2005/01/25 14:10:42 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class ContainerDemux {
public:
	/** takes the info object and demultiplexes every ES into
	 * a separate file. A copy of the info object with updated
	 * input descriptors is returned.
	 * if outputDir is set to NULL, no output will be generated
	 * Only call this method with an .mp4 file as input.
	 * If an adaptor is set, each visualstream will be adapted with this adaptor
	 */
	static ContainerInfo *demultiplex(ContainerInfo * &info, const char* outputDir,
																		const Adaptor * visualAdaptor =
																		NULL, bool omitHeader =
																		false, bool deleteMP4 =
																		false, bool visualizeAdaptation =
																		false);
	
	/** same as demultiplex(..) but the input descriptors are updated 
	 * to point to the new input. Per default the MP4FILE is NOT DELETED!
	 * if outputDir is set to NULL, no output will be generated
	 * If an adaptor is set, each visualstream will be adapted with this adaptor    
	 */
	static void demultiplexAndUpdateIO(ContainerInfo * &mp4Info, const char* outputDir,
																		 const Adaptor * visualAdaptor = NULL, bool omitHeader = false, 
																		 bool deleteMP4 = false, bool visualizeAdaptation = false);
	
	
	/** adapts single ESInfo @param es with adaptor @param adaptor.
	 * If @param outputFile is set, the created output will be written to the file,
	 * otherwise DevNull will be used as output. @param startFrame and @param endFrame will
	 * not work with system files but only with demultiplexed es! */	
	static void adaptESInfo(ESInfo* es, const char* outputFile, Adaptor* adaptor = NULL, 
													u32 startFrame = 0u, u32 endFrame = 4294967295u, bool omitHeader = false,
													bool visualize = false);
	
	/** adapts single ESInfo @param es with adaptor @param adaptor.
	 * If @param outputFile is set, the created output will be written to the file,
	 * otherwise DevNull will be used as output. @param startSecond and @param endSecond will
	 * not work with system files but only with demultiplexed es! */	
	static void adaptESInfo(ESInfo* es, const char* outputFile, Adaptor* adaptor = NULL, 
													double startSecond = 0.0, double endSecond = -1.0, bool omitHeader = false, 
													bool visualize = false);
	
	/** creates a DataChannel which is ready for immediate use. A DataSink has been inserted and play
	 * was called for this DataSink. Access to the inserted DataSink is via id 0.
	 * Just start the returned DataChannel with dc->start().
	 * Will return NULL if @param es is NULL.
	 */
	static DataChannel* createDataChannel(ESInfo* es, const char* outputFile, Adaptor* adaptor = NULL, 
																				u32 startFrame = 0u, u32 endFrame = 4294967295u, bool omitHeader = false, 
																				bool visualize = false);
	

};

#endif

