/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: OGGContainerFile.hpp                                             *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/
#ifndef VITOOKI_OGGCONTAINERFILE_HPP_
#define VITOOKI_OGGCONTAINERFILE_HPP_

#ifdef ENABLE_OGG

/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Thomas Kuglitsch
 *
 *@version $Id: OGGContainerFile.hpp,v 1.15 2006/07/03 13:19:46 tkuglits Exp $
 *
 *@short <short description>
 *
 */



#include "global.hpp"
#include "VideoESInfo.hpp"
#include "AudioESInfo.hpp"
#include "io/ByteStream.hpp"
#include "VMutex.hpp"
#include <ogg/ogg.h>
#include <theora/theora.h>
#include <vorbis/codec.h>
#include <Frame.hpp>

class ContainerInfo;
class Url;

//The OGG Stream Type
typedef enum OGGStreamType { OGG_VORBIS, OGG_THEORA, OGG_UNKNOWN } OGGStreamType;

class OGGStreamInfo {
public:
  OGGStreamInfo();
  ~OGGStreamInfo();
  
  void  append(OGGStreamInfo* osi);
  
  void  setNext(OGGStreamInfo* osi);
  OGGStreamInfo* getNext();
  
  void  setStreamID(long id);
  long  getStreamID();
  
  ogg_stream_state* getStreamState();
  ogg_stream_state* getStreamState(int serial);
  void setStreamState(ogg_stream_state *oss);
  
  // vorbis_info / theora_info
  void  setCodecInfo(void* ci);
  void* getCodecInfo();
  
  // vorbis_comment / theora_comment
  void  setCodecComment(void* cc);
  void* getCodecComment();
  
  void  setStreamType(OGGStreamType ost);
  OGGStreamType getStreamType();
  
private:
  OGGStreamInfo *next;
  long streamid;
  ogg_stream_state *oss;
  void *codec_info;
  void *codec_comment;
  OGGStreamType streamtype;
};
  

typedef struct OGGHandler{
  ogg_sync_state oy;
  FILE *io;
  // list of known OGG_Streams
  OGGStreamInfo *osi;
}OGGHandler;

class OGGContainerFile {
public:
  
  /** init. a video object from a pre-stored ContainerFile or via an open ByteStream.  */
	static ContainerInfo *loadContainerInfo(const Url* uri=NULL, ByteStream *stream = NULL);
	
  /** 
   * FIXME: right now done in MuxDemuxIO...
   */
  static bool saveToContainerFile(const char *fileName, ContainerInfo *ci) {
    return false;
  };
  
  static OGGHandler* getOGGHandler();

  /**
   * Helper to grab some data and sync it for
   * page extaction
   * @param in File from which to read
   * @param oy State which is responsible for OGG syncronisation
   */
  static int bufferData(ByteStream *stream, ogg_sync_state *oy);

  static void oggMutexLock(const char* s);
  static void oggMutexRelease(const char* s);

  static void printOggData(const char* c, const Frame* f);
  
private:
  
  /**
   * parseBOS
   * parse BeginningOfStream of OGG-Container searching 
   * for available Streams in this Container
   * @param io File to parse
   * @param oss OggSyncState Structure
   * @return 0 on failure, >0 on success
   */
  static int parseBOS(FILE *io, ogg_sync_state *oss);

  /**
   * Mutex to grant seperated access to ogg-file
   */

  static void initialize();

};//OGGContainerFile

#endif
#endif
