/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: VITContainerFile.hpp                                             *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/
#ifndef VITOOKI_VITCONTAINERFILE_HPP_
#define VITOOKI_VITCONTAINERFILE_HPP_

/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger
 *
 *@version $Id: VITContainerFile.hpp,v 1.4 2006/03/20 17:45:37 mkropfbe Exp $
 *
 *@short <short description>
 *
 */


#include "global.hpp"
#include "VideoESInfo.hpp"
#include "AudioESInfo.hpp"
#include "io/ByteStream.hpp"

class ContainerInfo;
class Url;

//The VIT Header format for the TrackList
typedef enum VITStreamType { VIT_VIDEO, VIT_AUDIO, VIT_SUBTITLE } VITStreamType;

//one track is coded in this struct, union'd for video and audio information
typedef struct VITTrackInfo {
	u32 streamID;     
	u32 streamType;  //chose from enumeration above
	u32 codecID;     //maps to ffmpeg codec ids
	u32 kbps;        //avgBandwidth in kilobit/sec
	u32 vopinc;      //VOPTimeIncrement, for Video, it is 90000/fps, but for MP3 it's special (eg. 2348)
	union {
		struct videoInfo {
			u32 fps;
			u32 width;
			u32 height;
			float aspectRatio;
		} vi;
		struct audioInfo {
			u32 sampleRate;
			u32 channels;
			char language[2]; //use clear-text acronyms like en,de,fr,it
		} ai;
		u32 padding[12];
	} esi;
} VITTrackInfo;

//header used for each frame
typedef struct VITFrameInfo {
	u32 streamID; //for mapping to corresponding ESInfo streams
	u32 cts; //presentation timestamp of this frame
	u32 size; //bytes following in the payload
} VITFrameInfo;




class VITContainerFile {

public:
	
	/** init. a video object from a pre-stored ContainerFile or via an open ByteStream
	 * No data is read, just metainfo
	 */
	static ContainerInfo *loadContainerInfo(const Url* uri=NULL, ByteStream *stream = NULL);
	
	/** 
	 * FIXME: right now done in MuxDemuxIO...
	 */
	static bool saveToContainerFile(const char *fileName, ContainerInfo *ci) {
		return false;
	};


};//VITContainerFile

#endif





