 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: debug.cpp                                                     *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "debug.hpp"

#ifdef WIN32

#include <stdarg.h>

#ifdef WINCE
#include <winbase.h>
#include <afxwin.h>
#endif

static char buf[2048];


#ifdef WINCE
static void * logfile = NULL;
static CFile *myfile = NULL;
static bool createLogfile = true;
static int errno = 0;
#endif


void logFile()
{
#ifdef WINCE
	if (createLogfile) {
	 logfile = CreateFile(_T("vitookilog.txt"),GENERIC_WRITE, FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	 myfile = new CFile((int)logfile);
	 createLogfile = false;
	} else {
	  /*logfile = CreateFile(_T("\\vitookilog.txt"),GENERIC_WRITE, FILE_SHARE_WRITE, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	  myfile = new CFile((int)logfile);
	  myfile->SeekToEnd();*/
	}

 if (logfile != NULL) {
	char threadId[50];
	DWORD curId = GetCurrentThreadId();
	sprintf(threadId, "[%d] ", curId);
	myfile->Write(threadId, strlen(threadId));
	myfile->Write(buf, strlen(buf));
	myfile->Write("\r\n", 2);
	//myfile->Close();
 }

#endif
}



void dprintf(const char *format, ...)
{
#ifdef VITOOKI_DEBUG	
    va_list args;

    va_start(args, format);
    vsprintf(buf, format, args);
#ifndef WINCE
    printf("[%d] %s",(int)pthread_self(), buf);
#else
	logFile();
#endif

    va_end(args);
#endif    
}

void dfprintf(FILE * F, const char *format, ...)
{
#ifdef VITOOKI_DEBUG	
    va_list args;

    va_start(args, format);
    vsprintf(buf, format, args);
#ifndef WINCE
    printf("[%d] %s",(int)pthread_self(), buf);
#else
	logFile();
#endif

    va_end(args);
#endif    
}

void dprintf_err(const char *format, ...)
{
#if VITOOKI_DEBUG_LEVEL >= VITOOKI_DEBUG_ERR
    va_list args;

    va_start(args, format);
    vsprintf(buf, format, args);
#ifndef WINCE
    printf("Log_E: [%d] %s",(int)pthread_self(), buf);
#else
	logFile();
#endif

    va_end(args);
#endif
}

void dprintf_err_fatal(const char *format, ...)
{
#if VITOOKI_DEBUG_LEVEL >= VITOOKI_DEBUG_ERR
    va_list args;

    va_start(args, format);
    vsprintf(buf, format, args);
#ifndef WINCE
    printf("Log_E: [%d] %s",(int)pthread_self(), buf);
#else
	logFile();
	CString serr (buf);
	MessageBox(NULL, serr, _T("Error"), MB_ICONERROR);
#endif

    va_end(args);
#endif
}

void dfprintf_err(FILE * F, const char *format, ...)
{
#if VITOOKI_DEBUG_LEVEL >= VITOOKI_DEBUG_ERR
    va_list args;

    va_start(args, format);
    vsprintf(buf, format, args);
#ifndef WINCE
    fprintf(F, "Log_E:  [%d] %s",(int)pthread_self(), buf);
#else
	logFile();
#endif

    va_end(args);
#endif
}

void dprintf_small(const char *format, ...)
{
#if VITOOKI_DEBUG_LEVEL >= VITOOKI_DEBUG_SMALL
    va_list args;

    va_start(args, format);
    vsprintf(buf, format, args);
#ifndef WINCE
    printf("Log_S: [%d] %s",(int)pthread_self(), buf);
#else
	logFile();
#endif

    va_end(args);
#endif
}

void dfprintf_small(FILE * F, const char *format, ...)
{
#if VITOOKI_DEBUG_LEVEL >= VITOOKI_DEBUG_SMALL
    va_list args;

    va_start(args, format);
    vsprintf(buf, format, args);
#ifndef WINCE
    fprintf(F, "Log_S: [%d] %s",(int)pthread_self(), buf);
#else
	logFile();
#endif

    va_end(args);
#endif
}

void dprintf_full(const char *format, ...)
{
#if VITOOKI_DEBUG_LEVEL >= VITOOKI_DEBUG_FULL
    va_list args;

    va_start(args, format);
    vsprintf(buf, format, args);
#ifndef WINCE
    printf("Log_F: [%d] %s",(int)pthread_self(), buf);
#else
	logFile();
#endif

    va_end(args);
#endif
}

void dfprintf_full(FILE * F, const char *format, ...)
{
#if VITOOKI_DEBUG_LEVEL >= VITOOKI_DEBUG_FULL
    va_list args;

    va_start(args, format);
    vsprintf(buf, format, args);
#ifndef WINCE
    fprintf(F, "[%d] %s",(int)pthread_self(), buf);
#else
	logFile();
#endif

    va_end(args);
#endif
}

#endif
