 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: debug.hpp                                                     *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
 *
 * Purpose:	support printf() debugging
 * Author:	Matthias Ohlenroth
 * Version:	$Id: debug.hpp,v 1.12 2005/07/18 11:08:27 klschoef Exp $
 * Notes:	
 *
 ***********************************************************************/

#ifndef __DEBUG_H__
#define __DEBUG_H__

#include <stdio.h>
#ifndef WINCE
#include <pthread.h>
#endif

#define VITOOKI_DEBUG_ERR	1
#define	VITOOKI_DEBUG_SMALL	2
#define VITOOKI_DEBUG_FULL	3

#if VITOOKI_DEBUG_LEVEL >= VITOOKI_DEBUG_ERR

/* adds the filepointer F to a class/method */
#define dfadd(F) FILE* F;
/* closes the filepointer F */
#define dfclose(F) { if(F) fclose(F);}
#define dfinit0(F) { F = stdout; }
/* initializes FILE* F, with a file named S */
#define dfinit(F, S) { F = fopen(S,"wb"); }
/* creates a fileName by converting the integer value I to a string and
   concatening it to the string S. */
#define dfinit2(F, S, I) { char* _fileN_= (char*)malloc(strlen(S)+21);_fileN_[0]=0;strcpy(_fileN_,S);char _number[20];sprintf(_number,"%i",I);strcat(_fileN_, _number); F = fopen(_fileN_,"wb");free(_fileN_); }
#define dfinitlog(F, S, I) { char* _fileN_= (char*)malloc(strlen(S)+25);_fileN_[0]=0;strcpy(_fileN_,S);char _number[20];sprintf(_number,"%i",I);strcat(_fileN_, _number);strcat(_fileN_,".log");F = fopen(_fileN_,"wb");free(_fileN_); }
#else

#define dfadd(F)
#define dfclose(F) {;}
#define dfinit0(F) {;}
#define dfinit(F, S) {;}
#define dfinit2(F, S, I) {;}
#define dfinitlog(F, S, I) {;}

#endif

#ifdef WIN32
#include <cstdio>

#ifndef VITOOKI_DEBUG_LEVEL
#undef dfadd
#define dfadd(F) FILE* F;
#undef dfinit0
#define dfinit0(F) { F = stdout; };
#endif  //vitooki_debug_level

#if VITOOKI_DEBUG_LEVEL < VITOOKI_DEBUG_ERR
#undef dfadd
#define dfadd(F) FILE* F;
#undef dfinit0
#define dfinit0(F) { F = stdout; };
#endif

void dprintf(const char *format, ...);
void dfprintf(FILE * F, const char *format, ...);
void dprintf_err(const char *format, ...);
void dprintf_err_fatal(const char *format, ...);
void dfprintf_err(FILE * F, const char *format, ...);
void dprintf_small(const char *format, ...);
void dfprintf_small(FILE * F, const char *format, ...);
void dprintf_full(const char *format, ...);
void dfprintf_full(FILE * F, const char *format, ...);

#else				// !WIN32, so LINUX

#define dprintf(A, B...) { printf(A, ## B); fflush(stdout);}
#define dfprintf(F, A, B...) { fprintf(F, A, ## B);fflush(F); }

#if VITOOKI_DEBUG_LEVEL >= VITOOKI_DEBUG_ERR

#define dprintf_err(A, B...) { fprintf(stderr,"Log_E: [%d]",(int)pthread_self()); fprintf(stderr,A, ## B); 	fflush(stderr);}
#define dfprintf_err(F, A, B...) { if(F) { fprintf(F, "Log_E: [%d]",(int)pthread_self()); fprintf(F, A, ## B); fflush(F);} }
#define dprintf_err_fatal(A, B...) {fprintf(stderr,"Log_E: [%d]", (int)pthread_self()); fprintf(stderr,A, ##B); fflush(stderr);}
#else
#define dprintf_err(A, B...) { ; }
#define dfprintf_err(F, A, B...) { ; }
#define dprintf_err_fatal(A, B...) {;}

#endif

#if VITOOKI_DEBUG_LEVEL >= VITOOKI_DEBUG_SMALL

#define dprintf_small(A, B...) { printf("Log_S: [%d]", (int)pthread_self()); printf(A, ## B); fflush(stdout);}
#define dfprintf_small(F, A, B...) { if(F) { fprintf(F,"Log_S: [%d]",(int)pthread_self()); fprintf(F, A, ## B);fflush(F);} }
#else
#define dprintf_small(A, B...) { ; }
#define dfprintf_small(F, A, B...) { ; }
#endif

#if VITOOKI_DEBUG_LEVEL >= VITOOKI_DEBUG_FULL

#define dprintf_full(A, B...) { printf("Log_F: [%d]",(int)pthread_self()); printf(A, ## B); fflush(stdout);}
#define dfprintf_full(F, A, B...) { if (F) { fprintf(F, "Log_F: [%d]",(int)pthread_self()); fprintf(F, A, ## B);fflush(F);} }
#else
#define dprintf_full(A, B...) { ; }
#define dfprintf_full(F, A, B...) { ; }
#endif

#endif

#endif				// WIN32
