 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	global.hpp                                                    *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef __global_hpp__
#define __global_hpp__

// c++ includes

#ifndef WINCE
#include <iostream>
#endif

#include <cmath>
#include <cstdio>
#include <cstdlib>
#include <queue>
#include <list>
#include <map>
#include <string>
#include "debug.hpp"
using namespace std;

// qt includes
#ifdef USE_QT
#include <qstring.h>
#include <qcstring.h>
#include <qfile.h>
#include <qglobal.h>
#if ((WIN32) && ((QT_VERSION >> 16) < 3))
#define QPtrList QList
#define QMemArray QArray
#include <qlist.h>
#define QPtrVector QVector
#define QPtrCollection QCollection
#else
#include <qptrlist.h>
#endif
#else
typedef unsigned int uint;
#endif // USE_QT

#include "VThread.hpp"

#include <time.h>
#ifdef WIN32
	#ifndef WINCE
	#include <sys/timeb.h>
	#endif /* wince */
#include <winsock2.h> // timeval type
#define snprintf _snprintf
#else
#include <sys/time.h>
#include <unistd.h>
#endif

#ifdef linux
#define LINUX
#endif

// some constants for convenience
#define NEW_LINE "\n"
#define NEW_LINE1 "\n"
#define NEW_LINE2 "\n\n"
#define NEW_LINE3 "\n\n"
#define TAB_CHAR "\t"

#define MAX_BUFFER_SIZE 65536*8
#define MSG_BUFFER_SIZE 65536
#define MAX_FRAME_SIZE 65536*8 //512KByte per frame
#define MAX_STR_LEN 1024
#define MAX_IOD_SIZE 1024
#define MAX_HEADERS_SIZE 8196

#ifdef WIN32
#define DEMUX_PATH "demux\\"
#define PATHSEPARATOR "\\"
#define PATHSEPARATORCHAR '\\'
#else
#define DEMUX_PATH "demux/"
#define PATHSEPARATOR "/"
#define PATHSEPARATORCHAR '/'
#endif

#define ASSERT_ALREADY_DEF
#define VERSION_ALREADY_DEF
#define PACKAGE_ALREADY_DEF

#ifndef MAX
#define MAX(A,B) ( ((A)>(B)) ? (A) : (B) )
#endif
#ifndef MIN
#define MIN(A,B) ( ((A)<(B)) ? (A) : (B) )
#endif

// error codes
#define ERR_OK 0
#define ERR -1
#define ERR_FILE -500
#define ERR_PSNR -600


/* Prints a debug message.
 * @param L Debug level
 * @param X File
 */
#ifdef ISOMP4
#include "ISOMovies.h"
#endif

//////////////////////////////////////////////////////////////////////
//  definition of Data types 
//////////////////////////////////////////////////////////////////////
#ifndef ISOMP4
#if defined(_MSC_VER) && !defined(__MWERKS__) 
typedef _int64 u64;
typedef _int64 s64;
#else
typedef unsigned long long u64;
typedef signed long long s64;
#endif
typedef unsigned int u32;
typedef unsigned short u16;
typedef unsigned char u8;
typedef int s32;
typedef short s16;
typedef char s8;
#define MP4VisualHandlerType (('v'<<24)|('i'<<16)|('d'<<8)| 'e')
#define MP4AudioHandlerType (('s'<<24)|('o'<<16)|('u'<<8)| 'n')
#define MP4SceneDescriptionHandlerType (('s'<<24)|('d'<<16)|('s'<<8)| 'm')
#define MP4ObjectDescriptorHandlerType (('o'<<24)|('d'<<16)|('s'<<8)| 'm')
#endif

#define SERVER_ID "ItecMP4Server"
#define PLAYER_ID "ItecMP4Player"
#define QT_PLAYER_ID "QTS"
#define OLD_BFRAMES_DISABLED_QT_PLAYER "qtver=6"
#define REAL_PLAYER_ID "RealMedia Player"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

#ifdef WIN32

#include <windows.h>
inline void msleep(long ms) { Sleep(ms);};
inline void sleep(long sec) { Sleep(1000*sec);};

#ifdef WINCE
static long gettimeofday(struct timeval *tp, void *tz)
{
	DWORD timemillis = timeGetTime();

	tp->tv_sec  = timemillis/1000;
	tp->tv_usec = (timemillis - (tp->tv_sec*1000)) * 1000;
	
	return 0;
};
static char *getMonthName(int nr)
{
	switch(nr) {
	case 1: return "Jan"; break;
	case 2: return "Feb"; break;
	case 3: return "Mar"; break;
	case 4: return "Apr"; break;
	case 5: return "May"; break;
	case 6: return "Jun"; break;
	case 7: return "Jul"; break;
	case 8: return "Aug"; break;
	case 9: return "Sep"; break;
	case 10: return "Oct"; break;
	case 11: return "Nov"; break;
	case 12: return "Dec"; break;
	default: return "";
	}
}
static char *addLeadingZeros(int nr)
{
	char *snr = new char[3];
	
	if (nr >= 10)
		sprintf(snr, "%d", nr);
	else 
		sprintf(snr, "0%d", nr);

	return snr;
}
#else //WINCE
static long gettimeofday(struct timeval *tp, void *tz)
{
	struct _timeb timebuffer;   

	_ftime( &timebuffer );
	tp->tv_sec  = timebuffer.time;
	tp->tv_usec = timebuffer.millitm * 1000;
	return 0;
};
#endif //WINCE



#else //win32
inline void msleep(long ms) { usleep(1000*ms);};

#endif //win32



bool getTimeString(char *timestr, int len);

unsigned int getFilesize(const char *filename);

struct intCompare {
	bool operator() (const int a, const int b) const {
		return (a<b);
	};
};

struct stringCompare {
	bool operator() (const char* a, const char* b) const {
		return (strcmp(a,b)<0);
	};
};

#endif				// __global_hpp__



//******************************************
// Windows CE definitions
//******************************************

#ifdef WINCE
#include <mstimeid.h>
#define srand48 srand
#define lrand48 rand
#endif //WINCE

