 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: AudioIO.hpp                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

// AudioIO.hpp: Class for Audio output.
//
//////////////////////////////////////////////////////////////////////

#ifdef ISOMP4
#include "ISOMovies.h" //XXX for data types definition
#endif

#ifndef _AUDIO_HPP_
#define _AUDIO_HPP_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "IO.hpp"
#include "adaptors/ESSynchronizer.hpp"

#include "AudioESInfo.hpp"
#include "Frame.hpp"
#include "adaptors/GlobalTimer.hpp"
#include "Rtp.hpp"

class Frame;
class AudioESInfo;

/**
 *
 * <Class Description goes here>
 *
 * @author Mithlesh Kumar
 *
 * @version $Id: AudioIO.hpp,v 1.13 2006/01/26 11:42:00 mkropfbe Exp $
 *
 * @short <short description>
 *
 */
class AudioIO : public IO {
public:

	AudioIO(AudioESInfo *es, ESSynchronizer *ess = NULL);

	virtual ~AudioIO();

	void setESInfo(ESInfo * es);

	ESInfo* getESInfo() { return (ESInfo*)es;};

	void setTicksPerSecond(u32 uiTicksPerSeconds);
  
	/** deletes the data source for this IO class. 
	 *          * not implemented, senseless for audio device!  */
	bool destroy();
	
	/** returns a value  from 0..100 indicating buffer usage */
	int getBufferFillLevel() const;
	
	/** returns a pointer to the local file name or an URL */
	const char *getURL() const;
	
	/** repositions the IO class to the given frame.
	 * Will return false in the following cases:
	 *  - an illegal frame number (too large) was specified
	 *  - the stream is not seekable, because the underlying 
	 *    device is a network device and the requested frame 
	 *    is already out of range: FIXME: implement
	 */
	bool setToFrameNumber(u32 frameNumber) {
		return false;
	};
	
	void setDimension();
	
	ESSynchronizer *getESSync() { return essync; };
	
	long getLastPTS() { return lastPTSseen; };
	
	void setLastPTS(long pts) { lastPTSseen = pts; };
	
	
protected:
	ESSynchronizer *essync;
	const AudioESInfo *es;
	u32 muiTicksPerSecond;
	/** stores the CTS of the last rendered frame  */
	long lastPTSseen;
	bool firstFrame;
	long num_frames;
};
#endif
