/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:  BufferedFileByteStream.hpp                                           *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/
#ifndef VITOOKI_BUFFEREDFILEBYTESTREAM_HPP_
#define VITOOKI_BUFFEREDFILEBYTESTREAM_HPP_
 
#include "ByteStream.hpp"


/**
 *
 * wraps simple file access (open, read bytes, close)
 * is used from diverse ContainerFile and diverse MuxDemuxIO classes
 *
 *@author Michael Kropfberger
 *
 *@version $Id: BufferedFileByteStream.hpp,v 1.1 2006/06/19 15:22:38 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class BufferedFileByteStream : public ByteStream {
public:
	BufferedFileByteStream(const char* url, bool forReading = true);

	bool open();

	bool close();

	/* reads the next character from stream  and @returns it as an unsigned char cast to an int,
		 or EOF (-1) on end of file or error
	*/
	int getChar();

	/* reads @param size number of bytes into the buffer @param ptr
	 * @returns the number of bytes read (which might be less than @param size) or EOF (-1)
	 */
	int read(void *ptr, int size);

	/* writes @param size number of bytes from the given buffer @param ptr
	 * @returns the number of bytes successfully written, or -1 if an error occurs
	 */
	int write(void *ptr, int size);

	/* sets the file pointer to the given byte position in the byteStream */
	int seek(u64 pos);

	/* @returns the actual byte position in the byteStream */
	u64 tell();

	
protected:
	FILE *io;
#define BUF_SIZE 4096
	char buf[BUF_SIZE*3]; //so its' maximum fill level is BUF_SIZE + pub_pos * curr buf_size
	int buf_pos, buf_size;

};


#endif
