/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: BufferedHttpMPGStreamReader.hpp                               *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PS_BUFFEREDHTTPMPGSTREAMREADER_HPP_
#define _PS_BUFFEREDHTTPMPGSTREAMREADER_HPP_

#include "MPGStreamIO.hpp"
#include "global.hpp"
#include "net/HttpRequest.hpp"

#ifndef WIN32
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/socket.h>
#include <sys/types.h> 
#endif

class ESInfo;
class Frame;
class QIODevice;

#define SAFETYDIST 4
#define STREAMREADER_DEF_BUFFER_SIZE 4096


/**
 *
 *@author Klaus Schoeffmann
 *
 */

class BufferedHttpMPGStreamReader:public MPGStreamIO {
public:

	BufferedHttpMPGStreamReader(ESInfo * es, const char *file, bool writeOnly, bool omitHeader = false, 
							int bufferSize =  STREAMREADER_DEF_BUFFER_SIZE, int maxFrameSize = MAX_FRAME_SIZE);

	virtual ~BufferedHttpMPGStreamReader();

	bool testFileopen();

	Frame *getFrame();

	static int getDefaultBufferSize() { return STREAMREADER_DEF_BUFFER_SIZE; }; //assume one kernel page for efficient reading

	int writeFrame(const Frame * frm) { return 0; }

	bool open();

	bool close(bool immediate=false);

	bool destroy() { return false; };

	bool setToFrameNumber(u32 frameNumber) { return false; };

	long setToClosestIFrame(u32 frameNumber, bool backwards) { return 0; };

	u32 getContentLength() { return contentLength; };


private:

	bool readBuffer();
	int buffered_getc();
	void goBack(int nrBytes);
	int buffered_read(u8 *buf, int nrBytes);
	int buffered_copy(u8 *buf, int nrBytes);
	void initBuffer();

	u8 *buffer;
	int bufferSize;
	int bytesBuffered, lastBytesBuffered;
	int bytesRead;
	int bytesReadTotal;
	bool bufferAtEnd;

	HttpRequest *http;

	u32 contentLength;
	char *contentStart;

	int maxFrameSize;
	int DTS;
};

#endif
