 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	BufferedIO.hpp                                               *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _MK_BUFFEREDIO_HPP_
#define _MK_BUFFEREDIO_HPP_

#include "IO.hpp"
#include "io/MuxDemuxIO.hpp"
#include "Frame.hpp"
#include "global.hpp"

//Forward declaration
class MuxDemuxIO;

/**
 *
 * stores writeFrame() written frames in an internal list-buffer, to be fetchted FIFO-style with getFrame()
 *
 *@author Michael Kropfberger
 *
 *@version $Id: BufferedIO.hpp,v 1.7 2006/02/11 16:35:26 klschoef Exp $
 *
 *@short <short description>
 *
 */
class BufferedIO:public IO {
public:
	/* If writeonly is true only write access
	 * is permitted, otherwise the file is read-only, url is deep-copied
	 */
	BufferedIO(ESInfo * es, MuxDemuxIO *muxio, bool writeOnly);

	virtual ~ BufferedIO();


	/** returns a frame if one complete frame is available, otherwise null 
	 * is returned. This function is typically blocking. Don't use a NULL
	 * return value to conclude STREAMEOF, always check with getState()!
	 */
	virtual Frame *getFrame();
	
  /** returns the number of packets sent. Returns 0 on error */
	virtual int writeFrame(Frame * frm, ESInfo *out_es = NULL);
	
  /** opens the IO connection. State is set to OPENING. 
	 * When the connection is ready for use, State is OPEN */
	bool open();

	bool close(bool immediate=false);

  /** deletes the elementary stream and its extra hint-file */
	bool destroy();
	
  /** returns the number of elems in the buffer */
	int getBufferFillLevel() const {
		return buffer.size();
	};
	
	IO *clone() const { return new BufferedIO(es,muxio,writeOnly);};
	
  /** not implemented/needed */
	const char *getURL() const {
		return NULL;
	};

	void setESInfo(ESInfo * esi) {es=esi; };

	ESInfo* getESInfo() { return es; };		

	void addToBuffer(Frame *frm) { 	buffer.push_back(frm); };

	/* finds out, what would be the next CTS in Buffer, if getFrame() _would_ be called */
	//long long int nextCTSinBuf();
	s64 nextCTSinBuf();

private:


protected:
	bool writeOnly;
	ESInfo *es;
	MuxDemuxIO *muxio;
	bool atFileStart;
	long lastCTSSeen;
	

	list<Frame*> buffer;
};

#endif
