/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   ByteStream.hpp                                              *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/
#ifndef VITOOKI_BYTESTREAM_HPP_
#define VITOOKI_BYTESTREAM_HPP_
 
#include "global.hpp"


/**
 *
 * wraps simple file access (open, read bytes, close), should be subclassed by eg. Http reading access
 * is used from diverse ContainerFile and diverse MuxDemuxIO classes
 *
 *@author Michael Kropfberger
 *
 *@version $Id: ByteStream.hpp,v 1.6 2006/05/12 11:32:50 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class ByteStream {
public:
	ByteStream(const char* uri, bool forReading = true);

	virtual ~ByteStream() {};

	const char *getUrl();

	bool isReadOnly() {return readOnly; };

	virtual bool open() = 0;

	virtual bool close() = 0;

	/* reads the next character from stream  and @returns it as an unsigned char cast to an int,
		 or EOF (-1) on end of file or error
	*/
	virtual int getChar() = 0;

	/* reads @param size number of bytes into the buffer @param ptr
	 * @returns the number of bytes read (which might be less than @param size) or EOF (-1)
	 */
	virtual int read(void *ptr, int size) = 0;

	/* writes @param size number of bytes from the given buffer @param ptr
	 * @returns the number of bytes successfully written, or -1 if an error occurs
	 */
	virtual int write(void *ptr, int size) = 0;

	/* sets the file pointer to the given byte position in the byteStream */
	virtual int seek(u64 pos) = 0;

	/* @returns the actual byte position in the byteStream */
	virtual u64 tell() = 0;

	
protected:
	char url[MAX_STR_LEN];
	bool readOnly;

};


#endif
