 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: DSPaudioIO.hpp                                                *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

// DSPaudioIO.hpp: Class for audio output.
//
//////////////////////////////////////////////////////////////////////
#ifndef WIN32


#ifndef _DSPAUDIOIO_HPP_
#define _DSPAUDIOIO_HPP_

//#ifndef WIN32 //audio via /dev/dsp is Linux-only

#ifdef ISOMP4
#include "ISOMovies.h" //XXX for data types definition
#endif

#include "AudioIO.hpp"
#include "AudioESInfo.hpp"
#include "adaptors/ESSynchronizer.hpp"


class Frame;
class AudioESInfo;

/**
 *
 * audio output to linux pseudo-file /dev/dsp
 *
 *@author Mithlesh Kumar
 *
 *@version $Id: DSPaudioIO.hpp,v 1.10 2006/01/20 15:37:18 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class DSPaudioIO : public AudioIO {
public:

	/* @param es points to the raw audio stream's ES, which incl. sample rate, stereo/mono flags...
	 * @param ess points to the ESSync, if needed to sync with a video output
	 * @param enableHeadphones only works on iPAQs to turn off speaker output and enable headphones
	 * (see the file SLASH_dev_SLASH_audio.c)
	 */
	DSPaudioIO(AudioESInfo *es, ESSynchronizer *ess = NULL, bool enableHeadphones = false);

	virtual ~DSPaudioIO();

	int initialize();
	
	// IO Methods
	Frame * getFrame() ;

	/** returns the number of packets sent. Returns 0 on error */
	int writeFrame(Frame * frm, ESInfo *out_es=NULL);

	/** opens the IO connection. State is set to OPENING.
			When the connection is ready for use, State is OPEN */
	bool open();

	bool close(bool immediate=false);

private:
	bool requestUnpauseESS;
	bool enableHeadphones;


};


#endif

#endif /*WIN32*/
