/************************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: DVDIO.cpp                                                     *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/
#include "DVDIO.hpp"
#include "Frame.hpp"
#include "VideoESInfo.hpp"
#include "AudioESInfo.hpp"
#include "CompressedVideoFrame.hpp"
#include "CompressedAudioFrame.hpp"

/* construktor */
DVDIO::DVDIO(ESInfo *es_info, const char *file_name) {

	assert(file_name);
	esinfo = es_info;
	filename = new char[strlen(file_name)+1];
	strcpy(filename,file_name);
	av_input_format = NULL; // has to be set to use av_open_input_file !!
	DTS = 0;
	currentFrameNumber=0;
};

/* open vob file */
bool DVDIO::open() {
	if(state!=OPEN) {
		int error;
		printf("DVDIO::open:\n");
		printf("Opening VOB file \n");
		av_register_all();
		error = av_open_input_file(&av_format_context, filename, av_input_format, 0, av_format_parameters);
		state = OPEN;
		return (error==0);
		currentFrameNumber=0;
	}
	return true;
	
};

/* get frame */
Frame * DVDIO::getFrame() {
	if(state!=OPEN)
		return NULL;
		
	printf("============================================================\n");
	printf("DVDIO::getFrame: \n");
	if (( (this->currentFrameNumber > this->endFrameNumber) && (endFrameNumber!=0) )) {
		close(true);
		state=STREAMEOF;
		return NULL;
	}
	
	Frame *frame = NULL;
  AU *access_unit = new AU();
  AVPacket avpacket_1, *avpacket = &avpacket_1;
	streamID = esinfo->getStreamId();
		
	printf("Getting new Frame from streamID: %i\n",streamID);
	printf("IDs:");
	// to get correct stream, brute @ moment, maybe improove!
	do {
		if(av_read_frame(av_format_context, avpacket)){
			printf("Error reading avpacket!\n");
			return NULL;
		}
		printf(" %i ",av_format_context->streams[avpacket->stream_index]->id );
		
		if(av_format_context->streams[avpacket->stream_index]->id != streamID) {
			//printf("DVDIO::getFrame: Wrong ID -> dropping packet\n");
			av_free_packet(avpacket);
		}
		
	} while (av_format_context->streams[avpacket->stream_index]->id != streamID);
	printf("\nCorrect ID found! -> ");	
	
	// access_unit stuff copied from ffmp4io
	access_unit->size = avpacket->size;
	
	access_unit->cts = DTS;
	access_unit->dts = DTS;
	DTS+=((VideoESInfo*)esinfo)->getVOPTimeIncrement();
	access_unit->payload = new u8[access_unit->size];
  	memcpy(access_unit->payload, avpacket->data, access_unit->size);
	
	// packet no longer needed!
	av_free_packet(avpacket);
	
	// create frame copied from ffmp4io
	if(esinfo->isVisualStream())		
    frame = new CompressedVideoFrame(Frame::NN_VOP, ((VideoESInfo*)esinfo)->getWidth(), ((VideoESInfo*)esinfo)->getHeight());
  else if(esinfo->isAudioStream())
    frame = new CompressedAudioFrame(Frame::NN_VOP);
  else //hack for any other stream type maybe subpic in the future!
		frame = new CompressedVideoFrame(Frame::NN_VOP,0,0);
  
  if (!frame->setAU(access_unit)) { //automatically detects frame type
    printf("Failed to set access unit at frame?\n");
  }
  frame->setMediaTimeScale(esinfo->getMediaTimeScale());
	
	// frame type output
	Frame::FrameType ft = frame->getType();
	printf("FrameType = %s\n\n", Frame::VOPTypeToChar(ft));
	currentFrameNumber++;
	return frame;
}
