/************************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: DVDIO.hpp                                                     *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/
#ifndef _DVDIO_HPP
#define _DVDIO_HPP

#include "global.hpp"
#include "IO.hpp"
#include "avformat.h"

class Frame;
class ESInfo;

/**                         **/
/** class description       **/
/** and function prototypes **/
/**                         **/

/* Container for DVDs */
class DVDIO: public IO {
public:
	
	/* construktor 
	 * @ param es info
	 * @ param vobfile
	 */
	DVDIO(ESInfo *es_info, const char *file_name);
	
	/* get frame for given esinfo
	 * @ return Frame
	 */
	Frame *getFrame();
	
	/* opens VOB file 
	 * @return open status, 0 ok
	 */
	bool open();

	/* set esinfo */
	void setESInfo(ESInfo * es) {esinfo=es;};
	
	/*  get esinfo */
	ESInfo* getESInfo() { return esinfo;};
	
	int writeFrame(Frame*, ESInfo *out_es = NULL) { assert(false); return 1;};
	

	bool close(bool immediate= false) { state=CLOSED; /*assert("IMPLEMENT ME");*/ return true;};
	/** delete the vob file */

	bool destroy() {assert(false); return true;};

	int getBufferFillLevel() const { return 50;};

	const char* getURL() const { return filename;};

	
protected:
	ESInfo *esinfo;
	char *filename;
	AVFormatContext *av_format_context;	
	AVInputFormat *av_input_format;
	AVFormatParameters *av_format_parameters;
	int streamID; 
	u64 DTS;	
};

#endif
