 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	DevNull.hpp                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PS_DEVNULL_HPP
#define _PS_DEVNULL_HPP

#include "../IO.hpp"

/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: DevNull.hpp,v 1.13 2006/01/26 11:42:00 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class DevNull:public IO {

public:
	DevNull():IO() {
		strcpy(url, "/dev/null");
	};
	
	/** returns a frame if one complete frame is available, otherwise null 
		is returned. This function is typically blocking. Don't use a NULL
		return value to conclude STREAMEOF, always check with getState()!
	*/
	virtual Frame *getFrame() {
#ifdef _POSIX_PRIORITY_SCHEDULING
		sched_yield(); //this is necessary to give parallel getFrames a chance
#endif
		return NULL;
	};
	
	/** returns the number of packets sent. Returns 0 on error */
	virtual int writeFrame(Frame * frm, ESInfo *out_es=NULL) {
#ifdef _POSIX_PRIORITY_SCHEDULING
		sched_yield(); //this is necessary to give parallel getFrames a chance
#endif
		return 1;
	};

	/** opens the IO connection. State is set to OPENING. Depending on the
			underlying QIODevice, a network connection or a file connection
			is established. When the connection is ready for use, State is OPEN */
	virtual bool open() {
		state=OPEN;
		return true;
	};

	virtual bool close(bool immediate=false) {
		state=CLOSED;
		immediate = immediate;
		return true;
	};

	/** deletes the data source for this IO class. Will only work with
	 * local files, remote files can not be deleted and will always return true.
	 */
	virtual bool destroy() {
		return true;
	};
 
	/** returns a value  from 0..100 indicating buffer usage */
	virtual int getBufferFillLevel() const {
		return 50;
	};
  
  	/** for compatibility to IO class **/
	void setESInfo(ESInfo * es) { es = es; };

	ESInfo* getESInfo() { return NULL;};
  	
  	/** returns a local file name or an URL */
	virtual const char *getURL() const {
		return url;
	};
	
	/** repositions the IO class to the given frame.
	 * Will return false in the following cases:
	 *  - an illegal frame number (too large) was specified
	 *  - the stream is not seekable, because the underlying 
	 *    device is a network device and the requested frame 
	 *    is already out of range: FIXME: implement
	 */
	virtual bool setToFrameNumber(u32 frameNumber) {
		frameNumber = frameNumber;
		return false;
	};
  
protected:
	char url[12];
};

#endif
