/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:   HttpByteStream.hpp                                              *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "HttpByteStream.hpp"

HttpByteStream::HttpByteStream(const char* url, bool forReading) : ByteStream(url,forReading) {
	http = NULL;
	offset=0;
}


bool HttpByteStream::open() {
	if (http) { //already open
		dprintf_err("HttpByteStream::open: already connected to %s!\n",url);
		return true;
	}

	if (readOnly) {
		http = new HttpRequest(url);
		if (!http->makeHttpRequest()) {
			dprintf_err("HttpByteStream::open: server/URI not found: %s!\n",url);
			return false;
		} else {
			dprintf_err("HttpByteStream::open: URI %s is open, mime-type: %s, %i content bytes will follow!\n",
									url,http->getMimeType(),http->getContentLength());
			return true;
		}
	}
	else //not impl
		return false;
}


bool HttpByteStream::close() {
	if (!http)
		return false;
	else
		http->closeSock();
	return true;
}


int HttpByteStream::getChar() {
	if (!http)
		return false;

	char c;
	int ret = this->read(&c,1);
	if (ret == 1)
		return (int)c;
	else
	return ret;
}


int HttpByteStream::read(void *ptr, int size) {
	if (!http || !readOnly)
		return -1;
	int pos=0;
	int len=0;

	do {
		len = http->continueReceive((char*)ptr+pos,size-pos);
		if (len>0)
			pos += len;
	} while ((len >= 0) && (pos<size));

	/*
	for (int i=0; i < size; i++)
		printf("%c",*((char*)ptr+i));
	printf("%c\n");

	dprintf_full("HttpByteStream::read: wanted %i bytes, really got %i bytes, last return %i\n",size,pos,len);
	*/

	offset+=pos;
	return pos;

}


int HttpByteStream::write(void *ptr, int size){
	if (!http|| readOnly)
		return -1;

	/* NOT IMPLEMENTED */
	return -1;
}


int HttpByteStream::seek(u64 pos) {
	if (!http)
		return -1;

	return -1;
}


u64 HttpByteStream::tell() {
	if (!http)
		return 0;

	return offset;
}
