#ifdef ENABLE_MKV

/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	MKIO.cpp                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "io/MKIO.hpp"
#include "CompressedVideoFrame.hpp"
#include "CompressedAudioFrame.hpp"
#include "Frame.hpp"
#include "VideoESInfo.hpp"
#include "AudioESInfo.hpp"


#ifdef _MSC_VER
#include <windows.h>      // for min/max
#endif // _MSC_VER

#include <iostream>



using namespace LIBMATROSKA_NAMESPACE;
using namespace std;


unsigned int BIN_FILE_SIZE = 15000;
unsigned int TXT_FILE_SIZE = 3000;
const unsigned int BIN_FRAME_SIZE = 1500;
const unsigned int TXT_FRAME_SIZE = 200;
const u64  TIMECODE_SCALE = 1000000; 

const bool bWriteDefaultValues = false;
/*
#define	CACHESIZE     65536
struct StdIoStream {
  		struct InputStream  base;
  		FILE		      *fp;
  		int		      error;
	};
	


typedef struct StdIoStream StdIoStream;
*/
/* StdIoStream methods */

/* read count bytes into buffer starting at file position pos
 * return the number of bytes read, -1 on error or 0 on EOF
 */
int   StdIoRead(StdIoStream *st, ulonglong pos, void *buffer, int count) {
  size_t  rd;
  if (fseek(st->fp, pos, SEEK_SET)) {
    st->error = errno;
    return -1;
  }
  rd = fread(buffer, 1, count, st->fp);
  if (rd == 0) {
    if (feof(st->fp))
      return 0;
    st->error = errno;
    return -1;
  }
  return rd;
}

/* scan for a signature sig(big-endian) starting at file position pos
 * return position of the first byte of signature or -1 if error/not found
 */
longlong StdIoScan(StdIoStream *st, ulonglong start, unsigned signature) {
  int	      c;
  unsigned    cmp = 0;
  FILE	      *fp = st->fp;

  if (fseek(fp, start, SEEK_SET))
    return -1;

  while ((c = getc(fp)) != EOF) {
    cmp = ((cmp << 8) | c) & 0xffffffff;
    if (cmp == signature)
      return ftell(fp) - 4;
  }

  return -1;
}

/* return cache size, this is used to limit readahead */
unsigned StdIoGetCacheSize(StdIoStream *st) {
  return CACHESIZE;
}

/* return last error message */
const char *StdIoGetLastError(StdIoStream *st) {
  return strerror(st->error);
}

/* memory allocation, this is done via stdlib */
void  *StdIoMalloc(StdIoStream *st, size_t size) {
  return malloc(size);
}

void  *StdIoRealloc(StdIoStream *st, void *mem, size_t size) {
  return realloc(mem,size);
}

void  StdIoFree(StdIoStream *st, void *mem) {
  free(mem);
}

/* progress report handler for lengthy operations
 * returns 0 to abort operation, nonzero to continue
 */
int   StdIoProgress(StdIoStream *st, ulonglong cur, ulonglong max) {
  return 1;
}

StdIoStream	st;
MatroskaFile	*mf;

//KaxSegment FileSegment;


/***************************************************************/
MKIO::MKIO(ESInfo *esi, const char *mkvFile, bool write) {

	es = esi;
	state = CLOSED;
	input = new char[strlen(mkvFile) + 1];
	strcpy(input, mkvFile);
	writeOnly = write;
	DTS = 0;
//	file_iformat = NULL;
	currentFrameNumber = 0;
	endFrameNumber = 0;
	priofp = NULL;
//  	ic=NULL;
//  	file_iformat=NULL;
	headerWritten = false;
  	
  	char		err_msg[256];

//   	ap = &ap1;
	if (es->isVisualStream()) { //set up timely distribution priority table
		tableSize = (int)ceil(((VideoESInfo*)es)->getFPS());
		int maxdepth=(int)((log((double)tableSize)/log(2.0)) + 1);

		dprintf_full("MKIO:  timelyDistTable: calculating maxdepth %i for patSize %i (=fps)\n", maxdepth,tableSize);
		int actpos=tableSize;  //pseudo-global helper var
		for (int i=0; i<maxdepth; i++){
			patSplit(prioTable,tableSize,i,&actpos);
		}

#ifdef VITOOKI_DEBUG
		dprintf_full("MKIO:  timelyDistTable:   ");
		printPat(prioTable,tableSize);
#endif
	}
	
	if(!write) {
		/* fill in I/O object */
		memset(&st,0,sizeof(st));
		st.base.read = reinterpret_cast<int(*)(InputStream*, long long unsigned int,void*,int)>(StdIoRead);
		st.base.scan = reinterpret_cast<long long(*)(InputStream*,long long unsigned int, unsigned int)>(StdIoScan);
		st.base.getcachesize = reinterpret_cast<unsigned int(*)(InputStream*)>(StdIoGetCacheSize);
		st.base.geterror = reinterpret_cast<const char*(*)(InputStream*)>(StdIoGetLastError);
		st.base.memalloc = reinterpret_cast<void*(*)(InputStream*,size_t)>(StdIoMalloc);
		st.base.memrealloc = reinterpret_cast<void*(*)(InputStream*,void*,size_t)>(StdIoRealloc);
		st.base.memfree = reinterpret_cast<void(*)(InputStream*,void*)>(StdIoFree);
		st.base.progress = reinterpret_cast<int(*)(InputStream*,long long unsigned int,long long unsigned int)>(StdIoProgress);
	
		/* open source file */
		st.fp = fopen(input,"rb");
		if (st.fp == NULL) {
			fprintf(stderr,"Can't open '%s': %s\n",input,strerror(errno));
			
		}
		setvbuf(st.fp, NULL, _IOFBF, CACHESIZE);
	
		/* initialize matroska parser */
		mf = mkv_OpenEx(&st.base, /* pointer to I/O object */
			0, /* starting position in the file */
			0, /* flags, you can set MKVF_AVOID_SEEKS if this is a non-seekable stream */
			err_msg, sizeof(err_msg)); /* error message is returned here */
	
		if (mf == NULL) {
			fclose(st.fp);
			fprintf(stderr,"Can't parse Matroska file: %s\n", err_msg);
			
		}
		dprintf_full("Matroska parser initialized!\n");
	}

	
}


/***************************************************************/
MKIO::~MKIO() {
	delete[] input;
	close();
	//	dfclose(fp);
	if(priofp)
		fclose(priofp);

	if (out_file)
		delete out_file;
}




/***************************************************************/
Frame *MKIO::getFrame() {
   
	if (state != OPEN || writeOnly) {	// FIXME: change this when adding buffering
    		return NULL;
  	}

	if( (this->currentFrameNumber > this->endFrameNumber) && (endFrameNumber != 0)) {
    		close(true);
    		state=STREAMEOF;
    		return NULL;
  	}
 
	Frame *frm = NULL;
	Frame::FrameType type = Frame::NN_VOP;
	AU *au = new AU();

	//AVPacket pkt1, *pkt = &pkt1;	//necessary?
  	//int err;

	
  	//unsigned	i;
  	int		track;
  	//FILE		*out = stdout;

	


	// list available tracks
	/*for (i = 0; i < mkv_GetNumTracks(mf); ++i) {
      		TrackInfo	*ti = mkv_GetTrackInfo(mf, i);
      		printf("Track %u: %s\n", i, ti->CodecID);
    	}*/

	dprintf("\n--->READING FRAME<---\n");

	
	


	ulonglong	StartTime, EndTime, FilePos;
    	unsigned	rt, FrameSize, FrameFlags;
    	unsigned	fb = 0;
    	void	      	*frame = NULL;
    	CompressedStream  *cs = NULL;

	// find out, what the track number of the current track is
	//for (i = 0; i < mkv_GetNumTracks(mf); ++i) {
	//	TrackInfo	*ti = mkv_GetTrackInfo(mf, i);
	//	dprintf("\n--->Track %s<---\n",ti->CodecID);
	//	// let's look if the IDs match, then we get the track number
	//	if(es->getStreamId()==ti->UID) {
	//		track = ti->Number - 1; //stupid mkv starts with zero...
	//		dprintf("\n--->Track %d<---\n",track);
	//	}
	//}
	
	track = es->getStreamId();
	
	if(es->isVisualStream()) {
		dprintf("\n--->Visual stream detected<---\n");
		
		au->prio = IO_NETWORK_HIGHEST_PRIORITY;
		au->duration = es->getVOPTimeIncrement();
		au->cts = currentFrameNumber * es->getVOPTimeIncrement();
		au->dts = currentFrameNumber * es->getVOPTimeIncrement();
		au->sampleFlags = 0;

		DTS += es->getVOPTimeIncrement();
		
		dprintf("MATROSKA: TRACK %d selected!",track);
		dprintf("\n--->Tracks parsed<---\n");
		
		/* mask other tracks because we don't need them */
		mkv_SetTrackMask(mf, ~(1 << track));
		
		/* init zlib decompressor if needed */
		if (mkv_GetTrackInfo(mf,track)->CompEnabled) {
			cs = cs_Create(mf, track, err_msg, sizeof(err_msg));
			if (cs == NULL) {
				fprintf(stderr,"Can't create decompressor: %s\n",err_msg);
				return NULL;
			}
		}	
		
		if(mkv_ReadFrame(mf, 0, &rt, &StartTime, &EndTime, &FilePos, &FrameSize, &FrameFlags) == 0) {
			
			if(cs) {
				dprintf("MATROSKA: compressed frame\n");
				/* CODE TO HANDLE COMPRESSED FRAME */
			}
			else {
				dprintf("MATROSKA: uncompressed frame\n");
				size_t	  rd;
				
				if (fseek(st.fp, FilePos, SEEK_SET)) {
					dprintf("fseek(): %s\n", strerror(errno));
					//return NULL;
				}
				
				dprintf("MATROSKA: Allocating memory\n");
				/* allocate memory for frame/payload */
				if (fb < FrameSize) {
	  				dprintf("MATROSKA: fb < FrameSize\n");
					fb = FrameSize;
	  				frame = realloc(frame, fb);
	  				if (frame == NULL) {
	    					dprintf("Out of memory\n");
	    					//return NULL;
	  				}
				}
				dprintf("MATROSKA: memory (%u) for frame %d allocated\n",FrameSize, currentFrameNumber);
					
				dprintf("MATROSKA: reading frame\n");
				rd = fread(frame,1,FrameSize,st.fp);
				dprintf("MATROSKA: read frame %d from file, rd = %d\n",currentFrameNumber, rd);
				
				if (rd != FrameSize) {
	  				dprintf("MATROSKA: rd!= FrameSize\n");
					if (rd == 0) {
	    					if (feof(st.fp)){
	      						dprintf("Unexpected EOF while reading frame\n");}
	    					else
	      						dprintf("Error reading frame: %s\n",strerror(errno));
	  				} else
	    					dprintf("Short read while reading frame\n");
	  					//return NULL;
				}
				dprintf("MATROSKA: preparing to copy payload ...\n");
				//fwrite(frame,1,FrameSize,out);
				// has to do deep-copy, otherwise collision with MP4lib mem handler
  				
				au->size = FrameSize;
				au->payload = new u8[au->size];
				dprintf("MATROSKA: initializing payload-memory (deep-copy) ...\n");
				memcpy(au->payload,frame,FrameSize);
				dprintf("MATROSKA: payload copied!\n");
			}
		}//readframe
		else{

			state = STREAMEOF;
			return NULL;

		}



		
//		dprintf("MATROSKA: Writing Frame %d, size=%u\n",currentFrameNumberFrameSize);

	}
	else if(es->isAudioStream()) {
		dprintf("\n--->Audio stream detected<---\n");
		au->prio = IO_NETWORK_HIGHEST_PRIORITY;
		//au->duration = es->getVOPTimeIncrement();
		au->cts = DTS;
		au->dts = DTS;
		au->duration = 0;
		au->sampleFlags = 0;
		au->err = SA_OK;
	
		DTS += es->getVOPTimeIncrement();
		
			
		dprintf("MATROSKA: TRACK %d selected!",track);
		dprintf("\n--->Tracks parsed<---\n");

		/* mask other tracks because we don't need them */
    		mkv_SetTrackMask(mf, ~(1 << track));

		/* init zlib decompressor if needed */
    		if (mkv_GetTrackInfo(mf,track)->CompEnabled) {
      			cs = cs_Create(mf, track, err_msg, sizeof(err_msg));
      			if (cs == NULL) {
				fprintf(stderr,"Can't create decompressor: %s\n",err_msg);
				return NULL;
      			}
    		}	

		if(mkv_ReadFrame(mf, 0, &rt, &StartTime, &EndTime, &FilePos, &FrameSize, &FrameFlags) == 0){

			if(cs) {
				dprintf("MATROSKA: compressed frame\n");
			/* CODE TO HANDLE COMPRESSED FRAME */
			}
			else {
				dprintf("MATROSKA: uncompressed frame\n");
				size_t	  rd;

				if (fseek(st.fp, FilePos, SEEK_SET)) {
	  				dprintf("fseek(): %s\n", strerror(errno));
	  				//return NULL;
				}
				
				dprintf("MATROSKA: Allocating memory\n");
				/* allocate memory for frame/payload */
				if (fb < FrameSize) {
	  				dprintf("MATROSKA: fb < FrameSize\n");
					fb = FrameSize;
	  				frame = realloc(frame, fb);
	  				if (frame == NULL) {
	    					dprintf("Out of memory\n");
	    					//return NULL;
	  				}
				}
				dprintf("MATROSKA: memory (%u)for frame %d allocated\n",FrameSize, currentFrameNumber);
				dprintf("MATROSKA: reading frame\n");
				rd = fread(frame,1,FrameSize,st.fp);
				dprintf("MATROSKA: read frame %d from file, rd = %d\n",currentFrameNumber, rd);
				
				if (rd != FrameSize) {
	  				dprintf("MATROSKA: rd!= FrameSize\n");
					if (rd == 0) {
	    					if (feof(st.fp)){
	      						dprintf("Unexpected EOF while reading frame\n");}
	    					else
	      						dprintf("Error reading frame: %s\n",strerror(errno));
	  				} else
	    					dprintf("Short read while reading frame\n");
	  					//return NULL;
				}
				dprintf("MATROSKA: preparing to copy payload ...\n");
				//fwrite(frame,1,FrameSize,out);
				// has to do deep-copy, otherwise collision with MP4lib mem handler
  				
				au->size = FrameSize;
				au->payload = new u8[au->size];
				dprintf("MATROSKA: initializing payload-memory (deep-copy) ...\n");
				memcpy(au->payload,frame,FrameSize);
				dprintf("MATROSKA: payload copied!\n");
					


			}
		}//readframe
		else{

			state = STREAMEOF;
			return NULL;

		}
		

	}
	else{
		dprintf("\n--->Other stream detected<---\n");
	}

	// frame types will be detected in frm->setAU()!
	if(es->isVisualStream())
		frm = new CompressedVideoFrame(type, ((VideoESInfo*)es)->getWidth(), ((VideoESInfo*)es)->getHeight());
	else
		frm = new CompressedVideoFrame(type, 0, 0);

	frm->setAU(au);
	frm->setMediaTimeScale(es->getMediaTimeScale());
	framesSeen++;
	currentFrameNumber++;
#ifdef _POSIX_PRIORITY_SCHEDULING
	sched_yield();
#else
	// for windows pthread lib!
#ifdef WIN32
#ifndef WINCE
	sched_yield();
#else
	Sleep(1);
#endif
#endif
#endif

  return frm;
}


/***************************************************************/
bool MKIO::open() {
	dprintf_full("MKIO::open()\r\n");
	if(!writeOnly)
		return openForReading();
	else
		return openForWriting();
}


/***************************************************************/
IO::State MKIO::play(double prefetchTime) {
	dprintf_full("MKIO::play prefetch %f (old state %i)\n",prefetchTime,state);

	if (state==CLOSED) {
		open();
		return state;
	} else
		if ( (state==PAUSED) || (state==MUTED) ) 
			setState(OPEN);
		else {
			dprintf_err("MKIO::play tried to OPEN from state %i\n",state);
			return state;
		}
	return state;
	
}


/***************************************************************/
bool MKIO::openForReading() {
	state = OPENING;
	//int err;
	/*
	dprintf_full("MKIO::openForReading : input is %s\n", input);
	if (strlen(input)==0)
	{
		
		memset(ap, 0, sizeof(*ap));
		if (es->isVisualStream())
		{
			dprintf_full("MKIO::openForReading : using video4linux as input: VIDEO\n"); 
			ap->frame_rate = 25;
			ap->width = 720;
			ap->height = 576;

			file_iformat = av_find_input_format("video4linux");
			ap->device  = NULL;
        		ap->channel = 0;
			ap->standard = "ntsc";
		}
		else if (es->isAudioStream())
		{
			dprintf_full("MKIO::openForReading : using audio grabbing as input: AUDIO\n"); 
			ap->sample_rate = 44100;
			ap->channels = 2;
			file_iformat = av_find_input_format("audio_device");
		}
		else 
			dprintf_err("MKIO::openForReading : Video 4 Linux: Unknown stream type!\n"); 
	}
	
	err = av_open_input_file(&ic, input, file_iformat, 0, ap);
	if(err < 0) {
		dprintf_full("MKIO::openForReading : Error in opening file!! ffmpeg error %i\n",err);
		state = CLOSED;
		return false;
	}
	dprintf_full("MKIO::openForReading : av_open_input_file is ok\r\n");

	//If not enough info to get the stream parameters, we decode the
	//first frames to get it. (used in cases when file has no headers eg. mpeg1)
#ifndef WIN32   //This gives error on windows in case of audio
	err = av_find_stream_info(ic);
	if (err < 0) {
		fprintf(stderr, "%s: could not find codec parameters\n", input);
		state = CLOSED;
		return false;
	}
#endif


	streamID = es->getStreamId();
	char *tmp = new char[MAX_STR_LEN];
  strncpy(tmp,input,MAX_STR_LEN);
  if (es->isVisualStream())
		strcat(tmp,".video");
	else if (es->isAudioStream())
		strcat(tmp,".audio");
  else {
    dprintf_err("ffMP4IO::openForReading requesting streamID neither video nor audio!\n");
		delete [] tmp;
		return false;
	}
  sprintf(tmp,"%s.%i.prio",tmp,streamID);

  if ((priofp=fopen(tmp,"r")) != NULL) {
		dprintf_full("ffMP4IO::openForReading: successfully opened prio-File %s\n",tmp);
	} else {
		dprintf_full("ffMP4IO::openForReading: prio-File %s not found. using timely distribution!\n",tmp);
	}
	*/
	state = OPEN;
	dprintf_full("MKIO::openForReading :  State is open\n");
//	delete [] tmp;
	return true;
}


/***************************************************************/
bool MKIO::openForWriting() {
  
	state = OPEN;
	dprintf_full("MKIO::openForWriting :  State is open\n");

	

	return true;
}


/***************************************************************/
bool MKIO::close(bool immediate) {
	
	dprintf_full("MKIO::close: closing MKIO\n");

	if (priofp)
		fclose(priofp);
	priofp = NULL;
	if (state == CLOSED)
		return true;

	state = CLOSED;
	int err;
	/*
	if (ic)
		{
			av_close_input_file(ic);	
			ic = NULL;
			
		}
	*/

	err = 0; 			//ASSUMING no error has occured because 
	//av_close_.... does not return anything; ISONoErr = 0 

	//out_file.close();
	// send a NULL-frame to announce the end of the file
	
#ifdef ENABLE_MKV_WRITE
	MKIO::writeFrame(NULL);

	assert(out_file);

	out_file->close();
#endif

	return err;	
}


/***************************************************************/
int MKIO::writeFrame(Frame * frm, ESInfo *out_es) {
#ifdef _POSIX_PRIORITY_SCHEDULING
  sched_yield(); //this is necessary to give parallel getFrames a chance
#endif
	
#ifdef ENABLE_MKV_WRITE
	dprintf_full("MKIO::writeFrame: writing frame (%i bytes) to input %s\n", frm->getAU()->size, input);	
	
	if (!headerWritten) {
	// write the head of the file (with everything already configured)
		out_file = new StdIOCallback (input, MODE_CREATE);
		
		///// Writing EBML test
			EbmlHead FileHead;
			
			EDocType & MyDocType = GetChild<EDocType>(FileHead);
			*static_cast<EbmlString *>(&MyDocType) = "matroska";
			
			EDocTypeVersion & MyDocTypeVer = GetChild<EDocTypeVersion>(FileHead);
			*(static_cast<EbmlUInteger *>(&MyDocTypeVer)) = MATROSKA_VERSION;
			
			EDocTypeReadVersion & MyDocTypeReadVer = GetChild<EDocTypeReadVersion>(FileHead);
			*(static_cast<EbmlUInteger *>(&MyDocTypeReadVer)) = 1;
			
			FileHead.Render(*out_file, bWriteDefaultValues);
			/* EBML Header rendered to file */
			
			//KaxSegment FileSegment;
			
			// size is unknown and will always be, we can render it right away
			uint64 SegmentSize = FileSegment.WriteHead(*out_file, 5, bWriteDefaultValues);
			/* Segment Header rendered to file */
			
			
			KaxTracks & MyTracks = GetChild<KaxTracks>(FileSegment);
			
			// reserve some space for the Meta Seek writen at the end		
			EbmlVoid Dummy;
			Dummy.SetSize(300); // 300 octets
			Dummy.Render(*out_file, bWriteDefaultValues);
			/* Dummy Header rendered to file */
			
			//KaxSeekHead MetaSeek;
			
			// fill the mandatory Info section
			KaxInfo & MyInfos = GetChild<KaxInfo>(FileSegment);
			KaxTimecodeScale & TimeScale = GetChild<KaxTimecodeScale>(MyInfos);
			*(static_cast<EbmlUInteger *>(&TimeScale)) = TIMECODE_SCALE;
			
			KaxDuration & SegDuration = GetChild<KaxDuration>(MyInfos);
			*(static_cast<EbmlFloat *>(&SegDuration)) =  es->getDuration();
			
			*((EbmlUnicodeString *)&GetChild<KaxMuxingApp>(MyInfos))  = L"ViTooKi MKV Muxer";
			*((EbmlUnicodeString *)&GetChild<KaxWritingApp>(MyInfos)) = L"ViTooKi MKV MKIO";
			GetChild<KaxWritingApp>(MyInfos).SetDefaultSize(25);
			
			uint32 InfoSize = MyInfos.Render(*out_file);
			MetaSeek.IndexThis(MyInfos, FileSegment);
			
			
			if (es->isVisualStream()) {


			}
			else if (es->isAudioStream()) {


			}

			// fill track 1 params
			KaxTrackEntry & MyTrack1 = GetChild<KaxTrackEntry>(MyTracks);
			MyTrack1.SetGlobalTimecodeScale(TIMECODE_SCALE);
			
			/* fill in tracknumber from ESInfo */
			KaxTrackNumber & MyTrack1Number = GetChild<KaxTrackNumber>(MyTrack1);
			*(static_cast<EbmlUInteger *>(&MyTrack1Number)) = 1;
			
			/* fill in UID, can/should be equal Tracknumber*/
			KaxTrackUID & MyTrack1UID = GetChild<KaxTrackUID>(MyTrack1);
			*(static_cast<EbmlUInteger *>(&MyTrack1UID)) = 7;
			
			/* set track to audio or video, according to ES track type */
			/* if ES->isvisual */
			*(static_cast<EbmlUInteger *>(&GetChild<KaxTrackType>(MyTrack1))) = track_audio;
			
			/* Set CODEC in MATROSKA-ID-Style, read from ES in FFMPEG-style */
			KaxCodecID & MyTrack1CodecID = GetChild<KaxCodecID>(MyTrack1);
			*static_cast<EbmlString *>(&MyTrack1CodecID) = "Dummy Audio Codec";
			
			MyTrack1.EnableLacing(true);
			
			// Test the new ContentEncoding elements
			KaxContentEncodings &cencodings = GetChild<KaxContentEncodings>(MyTrack1);
			KaxContentEncoding &cencoding = GetChild<KaxContentEncoding>(cencodings);
			*(static_cast<EbmlUInteger *>(&GetChild<KaxContentEncodingOrder>(cencoding))) = 10;
			*(static_cast<EbmlUInteger *>(&GetChild<KaxContentEncodingScope>(cencoding))) = 11;
			*(static_cast<EbmlUInteger *>(&GetChild<KaxContentEncodingType>(cencoding))) = 12;
			
			KaxContentCompression &ccompression = GetChild<KaxContentCompression>(cencoding);
			*(static_cast<EbmlUInteger *>(&GetChild<KaxContentCompAlgo>(ccompression))) = 13;
			GetChild<KaxContentCompSettings>(ccompression).CopyBuffer((const binary *)"hello1", 6);
			
			KaxContentEncryption &cencryption = GetChild<KaxContentEncryption>(cencoding);
			*(static_cast<EbmlUInteger *>(&GetChild<KaxContentEncAlgo>(cencryption))) = 14;
			GetChild<KaxContentEncKeyID>(cencryption).CopyBuffer((const binary *)"hello2", 6);
			*(static_cast<EbmlUInteger *>(&GetChild<KaxContentSigAlgo>(cencryption))) = 15;
			*(static_cast<EbmlUInteger *>(&GetChild<KaxContentSigHashAlgo>(cencryption))) = 16;
			GetChild<KaxContentSigKeyID>(cencryption).CopyBuffer((const binary *)"hello3", 6);
			GetChild<KaxContentSignature>(cencryption).CopyBuffer((const binary *)"hello4", 6);
			/* if audio */
			// audio specific params
			KaxTrackAudio & MyTrack1Audio = GetChild<KaxTrackAudio>(MyTrack1);
			
			KaxAudioSamplingFreq & MyTrack1Freq = GetChild<KaxAudioSamplingFreq>(MyTrack1Audio);
			*(static_cast<EbmlFloat *>(&MyTrack1Freq)) = 44100.0;
			MyTrack1Freq.ValidateSize();
			
#if MATROSKA_VERSION >= 2
			KaxAudioPosition & MyTrack1Pos = GetChild<KaxAudioPosition>(MyTrack1Audio);
			binary *_Pos = new binary[5];
			_Pos[0] = '0';
			_Pos[1] = '1';
			_Pos[2] = '2';
			_Pos[3] = '3';
			_Pos[4] = '\0';
			MyTrack1Pos.SetBuffer(_Pos, 5);
#endif // MATROSKA_VERSION
			
			KaxAudioChannels & MyTrack1Channels = GetChild<KaxAudioChannels>(MyTrack1Audio);
			*(static_cast<EbmlUInteger *>(&MyTrack1Channels)) = 2;
			
			// fill track 2 params
			KaxTrackEntry & MyTrack2 = GetNextChild<KaxTrackEntry>(MyTracks, MyTrack1);
			MyTrack2.SetGlobalTimecodeScale(TIMECODE_SCALE);
			
			KaxTrackNumber & MyTrack2Number = GetChild<KaxTrackNumber>(MyTrack2);
			*(static_cast<EbmlUInteger *>(&MyTrack2Number)) = 200;
			
			KaxTrackUID & MyTrack2UID = GetChild<KaxTrackUID>(MyTrack2);
			*(static_cast<EbmlUInteger *>(&MyTrack2UID)) = 13;
			
			*(static_cast<EbmlUInteger *>(&GetChild<KaxTrackType>(MyTrack2))) = track_video;
			
			KaxCodecID & MyTrack2CodecID = GetChild<KaxCodecID>(MyTrack2);
			*static_cast<EbmlString *>(&MyTrack2CodecID) = "Dummy Video Codec";
			
			KaxTrackDefaultDuration & MyTrack2DefaultDuration = GetChild<KaxTrackDefaultDuration>(MyTrack2);
			//*(static_cast<EbmlUInteger *>(&MyTrack2DefaultDuration)) = 40000000;
			*(static_cast<EbmlUInteger *>(&MyTrack2DefaultDuration)) = es->getDuration();
			
			/* if video */
			MyTrack2.EnableLacing(false);
						
			// video specific params
			
			VideoESInfo *esVideo = (VideoESInfo *) es;
			
			KaxTrackVideo & MyTrack2Video = GetChild<KaxTrackVideo>(MyTrack2);
			
			KaxVideoPixelHeight & MyTrack2PHeight = GetChild<KaxVideoPixelHeight>(MyTrack2Video);
			*(static_cast<EbmlUInteger *>(&MyTrack2PHeight)) = esVideo->getHeight();
			
			KaxVideoPixelWidth & MyTrack2PWidth = GetChild<KaxVideoPixelWidth>(MyTrack2Video);
			*(static_cast<EbmlUInteger *>(&MyTrack2PWidth)) = esVideo->getWidth();
			
			
			uint64 TrackSize = MyTracks.Render(*out_file, bWriteDefaultValues);
			
			KaxTracks * pMyTracks2 = static_cast<KaxTracks *>(MyTracks.Clone());
			
			MetaSeek.IndexThis(MyTracks, FileSegment);
			
			/* first header stops here
			 * the passing of the payload takes place here*/

			// "manual" filling of a cluster"
			
			KaxCues AllCues;
			AllCues.SetGlobalTimecodeScale(TIMECODE_SCALE);
			
			KaxCluster Clust1;
			Clust1.SetParent(FileSegment); // mandatory to store references in this Cluster
			Clust1.SetPreviousTimecode(0, TIMECODE_SCALE); // the first timecode here
			Clust1.EnableChecksum();

			// automatic filling of a Cluster
			// simple frame
			// the payload of vitooki frame objects can be used here for writing to file -> DataBuffer
			KaxBlockGroup *MyNewBlock, *MyLastBlockTrk1 = NULL, *MyLastBlockTrk2 = NULL, *MyNewBlock2;
			
			DataBuffer *data7 = new DataBuffer((binary *)"tototototo", countof("tototototo"));
			Clust1.AddFrame(MyTrack1, 250 * TIMECODE_SCALE, *data7, MyNewBlock, LACING_EBML);
			if (MyNewBlock != NULL)
				MyLastBlockTrk1 = MyNewBlock;

			/* #### */

			DataBuffer *data0 = new DataBuffer((binary *)"TOTOTOTO", countof("TOTOTOTO"));
			Clust1.AddFrame(MyTrack1, 260 * TIMECODE_SCALE, *data0, MyNewBlock); // to test EBML lacing
			if (MyNewBlock != NULL)
				MyLastBlockTrk1 = MyNewBlock;

			DataBuffer *data6 = new DataBuffer((binary *)"tototototo", countof("tototototo"));
			Clust1.AddFrame(MyTrack1, 270 * TIMECODE_SCALE, *data6, MyNewBlock); // to test lacing
			if (MyNewBlock != NULL) {
				MyLastBlockTrk1 = MyNewBlock;
			} else {
				MyLastBlockTrk1->SetBlockDuration(50 * TIMECODE_SCALE);
			}
			
			DataBuffer *data5 = new DataBuffer((binary *)"tototototo", countof("tototototo"));
			Clust1.AddFrame(MyTrack2, 23 * TIMECODE_SCALE, *data5, MyNewBlock); // to test with another track
			
			// add the "real" block to the cue entries
			AllCues.AddBlockGroup(*MyLastBlockTrk1);
			
			// frame for Track 2
			DataBuffer *data8 = new DataBuffer((binary *)"tttyyy", countof("tttyyy"));
			Clust1.AddFrame(MyTrack2, 107 * TIMECODE_SCALE, *data8, MyNewBlock, *MyLastBlockTrk2);
			
			AllCues.AddBlockGroup(*MyNewBlock);
			
			// frame with a past reference
			DataBuffer *data4 = new DataBuffer((binary *)"tttyyy", countof("tttyyy"));
			Clust1.AddFrame(MyTrack1, 300 * TIMECODE_SCALE, *data4, MyNewBlock, *MyLastBlockTrk1);
			
			// frame with a past & future reference
			if (MyNewBlock != NULL) {
				DataBuffer *data3 = new DataBuffer((binary *)"tttyyy", countof("tttyyy"));
				if (Clust1.AddFrame(MyTrack1, 280 * TIMECODE_SCALE, *data3, MyNewBlock2, *MyLastBlockTrk1, *MyNewBlock)) {
					MyNewBlock2->SetBlockDuration(20 * TIMECODE_SCALE);
					MyLastBlockTrk1 = MyNewBlock2;
				} else {
					printf("Error adding a frame !!!");
				}
			}
			
			AllCues.AddBlockGroup(*MyLastBlockTrk1);
			//AllCues.UpdateSize();
			
			// simulate the writing of the stream :
			// - write an empty element with enough size for the cue entry
			// - write the cluster(s)
			// - seek back in the file and write the cue entry over the empty element
			
			uint64 ClusterSize = Clust1.Render(*out_file, AllCues, bWriteDefaultValues);
			Clust1.ReleaseFrames();
			MetaSeek.IndexThis(Clust1, FileSegment);
			
			KaxCluster Clust2;
			Clust2.SetParent(FileSegment); // mandatory to store references in this Cluster
			Clust2.SetPreviousTimecode(300 * TIMECODE_SCALE, TIMECODE_SCALE); // the first timecode here
			Clust2.EnableChecksum();
			
			DataBuffer *data2 = new DataBuffer((binary *)"tttyyy", countof("tttyyy"));
			Clust2.AddFrame(MyTrack1, 350 * TIMECODE_SCALE, *data2, MyNewBlock, *MyLastBlockTrk1);
			
			AllCues.AddBlockGroup(*MyNewBlock);
			
			ClusterSize += Clust2.Render(*out_file, AllCues, bWriteDefaultValues);
			Clust2.ReleaseFrames();
			
			
			uint32 CueSize = AllCues.Render(*out_file, bWriteDefaultValues);
			MetaSeek.IndexThis(AllCues, FileSegment);
			
			// Chapters
			KaxChapters Chapters;
			Chapters.EnableChecksum();
			KaxEditionEntry & aEdition = GetChild<KaxEditionEntry>(Chapters);
			KaxChapterAtom & aAtom = GetChild<KaxChapterAtom>(aEdition);
			KaxChapterUID & aUID = GetChild<KaxChapterUID>(aAtom);
			*static_cast<EbmlUInteger *>(&aUID) = 0x67890;
			
			KaxChapterTimeStart & aChapStart = GetChild<KaxChapterTimeStart>(aAtom);
			*static_cast<EbmlUInteger *>(&aChapStart) = 0;
			
			KaxChapterTimeEnd & aChapEnd = GetChild<KaxChapterTimeEnd>(aAtom);
			*static_cast<EbmlUInteger *>(&aChapEnd) = 300 * TIMECODE_SCALE;
			
			KaxChapterDisplay & aDisplay = GetChild<KaxChapterDisplay>(aAtom);
			KaxChapterString & aChapString = GetChild<KaxChapterString>(aDisplay);
			*static_cast<EbmlUnicodeString *>(&aChapString) = L"der film in einem kapitel";
			
			KaxChapterLanguage & aChapLang = GetChild<KaxChapterLanguage>(aDisplay);
			*static_cast<EbmlString *>(&aChapLang) = "de";
			
			KaxChapterDisplay & aDisplay2 = GetNextChild<KaxChapterDisplay>(aAtom, aDisplay);
			KaxChapterString & aChapString2 = GetChild<KaxChapterString>(aDisplay2);
			*static_cast<EbmlUnicodeString *>(&aChapString2) = L"The movie in one chapter";
			
			KaxChapterLanguage & aChapLang2 = GetChild<KaxChapterLanguage>(aDisplay2);
			*static_cast<EbmlString *>(&aChapLang2) = "eng";
			
			uint32 ChapterSize = Chapters.Render(*out_file, bWriteDefaultValues);
			MetaSeek.IndexThis(Chapters, FileSegment);

			// Write some tags
			KaxTags AllTags;
			AllTags.EnableChecksum();
			KaxTag & aTag = GetChild<KaxTag>(AllTags);
			KaxTagTargets & Targets = GetChild<KaxTagTargets>(aTag);
			KaxTagGeneral & TagGeneral = GetChild<KaxTagGeneral>(aTag);
			
			KaxTagTrackUID & TrackUID = GetChild<KaxTagTrackUID>(Targets);
			*static_cast<EbmlUInteger *>(&TrackUID) = 0x12345;
			
			KaxTagChapterUID & ChapterUID = GetChild<KaxTagChapterUID>(Targets);
			*static_cast<EbmlUInteger *>(&ChapterUID) = 0x67890;
			
			KaxTagSubject & Subject = GetChild<KaxTagSubject>(TagGeneral);
			*static_cast<EbmlUnicodeString *>(&Subject) = L"Test123";
			
			KaxTagBibliography & Biblio = GetChild<KaxTagBibliography>(TagGeneral);
			*static_cast<EbmlUnicodeString *>(&Biblio) = L"Biblio";
			
			KaxTagFile & File = GetChild<KaxTagFile>(TagGeneral);
			*static_cast<EbmlUnicodeString *>(&File) = L"general tag";
			
			KaxTagLanguage & Lang = GetChild<KaxTagLanguage>(TagGeneral);
			*static_cast<EbmlString *>(&Lang) = "de";
			
			uint32 TagsSize = AllTags.Render(*out_file, bWriteDefaultValues);
			MetaSeek.IndexThis(AllTags, FileSegment);
			
			TrackSize += pMyTracks2->Render(*out_file, bWriteDefaultValues);
			MetaSeek.IndexThis(*pMyTracks2, FileSegment);
			
			uint32 MetaSeekSize = Dummy.ReplaceWith(MetaSeek, *out_file, bWriteDefaultValues);
			
			
			// let's assume we know the size of the Segment element
			// the size of the FileSegment is also computed because mandatory elements exist
			if (FileSegment.ForceSize(SegmentSize - FileSegment.HeadSize() + MetaSeekSize
																+ TrackSize + ClusterSize + CueSize + InfoSize + TagsSize + ChapterSize)) {
				FileSegment.OverwriteHead(*out_file, true);

			headerWritten = true;
		}
	}

	//write normal frame to mkv container here

#endif	

	return 1;
}


/***************************************************************/
/** returns -1, when at end of File, returns 0 when file is closed
 * returns 50 otherwise */
int MKIO::getBufferFillLevel() const {
	if (state == OPEN)
		return 50;
	else if (state == STREAMEOF)
		return -1;
	else
		return 0;
}


/***************************************************************/
bool MKIO::destroy() {
	close();
	return remove(input) == 0;
}


/** returns false if file format does not support file seeking
 * else returns true
 */
bool MKIO::setToFrameNumber(u32 frameNumber) {
	
	return false;
}

/***************************************************************/
int MKIO::patSplitRec(int *pat, int len, int actdepth, int maxdepth, int gowhere, int *actpos) {
	int half=len/2;
	int *middle = &pat[half];
	int ret;
	
	//	printf("depth %i (of %i) pos %i len %i left %i right %i --> ",actdepth,maxdepth,*actpos,len,half,len-half);
	//printPat(pat,len,-1,0);

	if (actdepth < maxdepth) {
		if (gowhere) {
			ret = patSplitRec(pat, half, actdepth+1, maxdepth, gowhere,actpos);
			if (ret == 0)
				ret = patSplitRec(pat+half,len-half, actdepth+1, maxdepth, gowhere,actpos);
			return ret;
		} else {
			ret = patSplitRec(pat+half,len-half, actdepth+1, maxdepth, gowhere,actpos);
			if (ret == 0)
				ret = patSplitRec(pat, half, actdepth+1, maxdepth, gowhere,actpos);
			return ret;
		}
	} else {
		if (*middle == -1) {
			*middle = (*actpos)--;
			return 1;
		} else {
			//	printf("FULL\n"); 
			return 0;
		}
	}

}


/***************************************************************/
void MKIO::patSplit(int *pat, int len, int maxdepth, int *actpos) {
	int i=1, j=1;

	if (*actpos == len) { //init
		for (int k=0; k<len;k++)
			pat[k]=-1;
		patSplitRec(pat, len, 0, 0, 0, actpos);
	}

	while ((i!=0) || (j!=0))  {
		//		printf(">>>next node RIGHT\n");
		//printPat(pat,len);
		i = patSplitRec(pat, len, 0, maxdepth, 1, actpos);
		//		printf(">>>next node LEFT \n");
		//printPat(pat,len);
		j = patSplitRec(pat, len, 0, maxdepth, 0, actpos);
	}
}


/***************************************************************/
void MKIO::printPat(int *pat, int len) {
	int i;
	for (i=0; i<len; i++) {
		if (pat[i] > -1)
			printf("%2i-", pat[i]);
		else 
			printf("..-");
	}
	printf("\n");
}

#endif
