#ifdef ENABLE_MKV 

 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	MKIO.hpp                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _MK_SERVER_MKIO_HPP
#define _MK_SERVER_MKIO_HPP

#include "IO.hpp"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef ENABLE_MKV_WRITE
#include <fcntl.h>
#include <errno.h>
#include "ebml/StdIOCallback.h"
#include "ebml/EbmlHead.h"
#include "ebml/EbmlSubHead.h"
#include "ebml/EbmlVoid.h"
#include "matroska/FileKax.h"
#include "matroska/KaxSegment.h"
#include "matroska/KaxTracks.h"
#include "matroska/KaxTrackEntryData.h"
#include "matroska/KaxTrackAudio.h"
#include "matroska/KaxTrackVideo.h"
#include "matroska/KaxCluster.h"
#include "matroska/KaxClusterData.h"
#include "matroska/KaxSeekHead.h"
#include "matroska/KaxCues.h"
#include "matroska/KaxInfo.h"
#include "matroska/KaxInfoData.h"
#include "matroska/KaxTags.h"
#include "matroska/KaxTag.h"
#include "matroska/KaxChapters.h"
#include "matroska/KaxContentEncoding.h"
#endif


#ifdef __cplusplus
extern "C" {
#include "../container/MatroskaParser.h"
}
#endif

#define	CACHESIZE     65536
#define SAFETYDIST 4
#define MAXVOPSIZE 64000
#define MAXMP3VOPSIZE 64000
#define STREAMREADER_DEF_BUFFER_SIZE 4096


struct StdIoStream {
  		struct InputStream  base;
  		FILE		      *fp;
  		int		      error;
	};
	


typedef struct StdIoStream StdIoStream;


class Frame;
class ESInfo;

/**
 *
 * reads mkv-containers via MatroskaParser
 *
 *@author Armin Mueller
 *
 *@version $Id: MKIO.hpp,v 1.14 2006/03/24 15:09:30 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class MKIO:public IO {
public:
	/** reading from an MKV file is only possible from a file.
	 * mkvFile is deep-copied. If writeonly is true only write access
	 * is permitted, otherwise the file is read-only
	 */
	MKIO(ESInfo * es, const char *mkvFile, bool writeOnly);
	
	/** frees everything except es */
	virtual ~ MKIO();

 	/** returns a frame if one complete frame is available, otherwise null 
			is returned. This function is typically blocking. Don't use a NULL
			return value to conclude STREAMEOF, always check with getState()!
	*/
	Frame *getFrame();

	/** returns the number of packets sent. Returns 0 on error */
	int writeFrame(Frame * frm, ESInfo *out_es = NULL);

	/** opens the IO connection. State is set to OPENING. 
			When the file is ready for use, State is OPEN */
	bool open();

	IO::State play(double prefetchTime);

	bool close(bool immediate=false);
  
  /** be vary careful, when destroying MKIO files, because many MKIO objects will
	 * refer to the same file. It is suggested that one deletes MKV videos with the
	 * ContainerInfo::destroy method, which will take care of that
	 */
	bool destroy();
  
	bool setToFrameNumber(u32 frameNumber);
  
	/** returns -1, when at end of File, returns 0 when file is closed
	 * returns 50 otherwise */
	int getBufferFillLevel() const;

	/** returns a local file name or an URL */
	const char *getURL() const { return input; };

	void setESInfo(ESInfo * esi) { es=esi; };

	ESInfo* getESInfo() { return es; };
	


protected:
	
//	AVFormatContext *ic;
//	AVInputFormat *file_iformat;
	//AVFormatParameters *ap;
//	AVFormatParameters ap1, *ap;
	int streamID;
	
	bool writeOnly;
	dfadd(fp);

private:
	void printPat(int *pat, int len);
	void patSplit(int *pat, int len, int maxdepth, int *actpos);
	int patSplitRec(int *pat, int len, int actdepth, int maxdepth, int gowhere, int *actpos);
	
	u64 DTS;
	bool openForReading();
	bool openForWriting();
	
	StdIoStream	st;
	MatroskaFile	*mf;
	char		err_msg[256];

	char *input;
	ESInfo * es;
	
	bool headerWritten; //Matroska header
	StdIOCallback *out_file;
	LIBMATROSKA_NAMESPACE::KaxSegment FileSegment;
	LIBMATROSKA_NAMESPACE::KaxSeekHead MetaSeek;
	//LIBMATROSKA_NAMESPACE::KaxTracks & MyTracks;

  	FILE *priofp;
	int prioTable[256]; //highest possible framerate of video
	int tableSize;
};

#endif


#endif

