 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	MP4IO.hpp                                                     *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _MK_MP4IO_HPP
#define _MK_MP4IO_HPP

#include "IO.hpp"
#include "global.hpp"

class Frame;
class ESInfo;

/**
 *
 * Base class for accessing mp4-containers
 *
 *@author Michael Kropfberger
 *
 *@version $Id: MP4IO.hpp,v 1.5 2006/01/26 11:42:00 mkropfbe Exp $
 *
 *@short <short description>
 *
 */

class MP4IO : public IO {
public:
	/** reading from an MP4 file is only possible from a file.
	 * mp4File is deep-copied. If writeonly is true only write access
	 * is permitted, otherwise the file is read-only
	 */
	//MP4IO(ESInfo * es, const char *mp4File, bool writeOnly);
	MP4IO();
	
	/** frees everything except es */
	~MP4IO();

 	/** returns a frame if one complete frame is available, otherwise null 
			is returned. This function is typically blocking. Don't use a NULL
			return value to conclude STREAMEOF, always check with getState()!
	*/
	virtual Frame *getFrame() = 0;

	/** returns the number of packets sent. Returns 0 on error */
	virtual int writeFrame(Frame * frm, ESInfo *out_es=NULL) = 0;

	/** opens the IO connection. State is set to OPENING. 
			When the file is ready for use, State is OPEN */
	virtual bool open() = 0;

	virtual IO::State play(double prefetchTime) = 0;

	virtual bool close(bool immediate=false) = 0;
  
  /** be vary careful, when destroying MP4IO files, because many MP4IO objects will
	 * refer to the same file. It is suggested that one deletes MP4 videos with the
	 * ContainerInfo::destroy method, which will take care of that
	 */
	virtual bool destroy() = 0;
  
	virtual bool setToFrameNumber(u32 frameNumber) = 0;
  
	/** returns -1, when at end of File, returns 0 when file is closed
	 * returns 50 otherwise */
	virtual int getBufferFillLevel() const = 0;

	/** returns a local file name or an URL */
	const char *getURL() { return input; };
    
	void setESInfo(ESInfo * esi) { es=esi; };

	ESInfo* getESInfo() { return es; };
	
protected:
	ESInfo * es;
	char *input;
};

#endif
