 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	MuxDemuxIO.hpp                                               *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _MK_MUXDEMUXIO_HPP_
#define _MK_MUXDEMUXIO_HPP_

#include "IO.hpp"
#include "global.hpp"
#include "io/BufferedIO.hpp"
#include "io/ByteStream.hpp"
#include "VMutex.hpp"

//Forward declaration
class BufferedIO;

/**
 *
 * muxes/demuxes from/to containers to elementary-stream BufferIOs
 *
 *@author Michael Kropfberger
 *
 *@version $Id: MuxDemuxIO.hpp,v 1.9 2006/02/24 13:47:42 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class MuxDemuxIO:public IO {
public:
	/* 
	 */
	MuxDemuxIO(ByteStream *stream);

	/** used by writeFrame to add the correct streamID for each frame */
	virtual bool addBufferedIO(int streamID, BufferedIO *bufIO);

	/** used by BufferedIO::close to remove the correct streamID when finishing */
	virtual bool removeBufferedIO(int streamID, BufferedIO *bufIO);
	
	virtual int getNumberOfConnectedBufferedIO();

	virtual ~ MuxDemuxIO();


	/** returns a frame and forwards it to the correct BufferIO
	 * This function is typically blocking. Don't use a NULL
	 * return valuse to conclude STREAMEOF, always check with getState()!
	 */
	virtual Frame *getFrame();
	
  /** muxes a frame to the container */
	virtual int writeFrame(Frame * frm,ESInfo *out_es=NULL);

	/** give the MuxDemuxIO the chance to do mux/demux (called by fferedIOs) */
	virtual void processFrame();
	
  /** opens the IO connection. State is set to OPENING. 
	 * When the connection is ready for use, State is OPEN */
	bool open();

	bool close(bool immediate=false);

  /** deletes the elementary stream and its extra hint-file */
	bool destroy();
	
  /** returns a value  from 0..100 indicating buffer usage. Fixme: returns a constant value of 50 */
	int getBufferFillLevel() const { return 50;	};
	
	IO *clone() const { return new MuxDemuxIO(stream);};
	
  /** returns a local file name or an URL */
	const char *getURL() const { return stream->getUrl();	};

	void setESInfo(ESInfo * esi) { assert(1==0); };

	ESInfo* getESInfo() { assert(1==0); return NULL; };		

private:
protected:
	bool readOnly;
	map < int, BufferedIO* > ioList;
	VMutex *mutex;
	bool firstFrame;
	ByteStream *stream;

	/** give the MuxDemuxIO the chance to query the BufferIOs for new muxing data */
	virtual void processWriteFrame();


	/** let the MuxDemuxIO read frames and demux/pass them to the correct BufferIOs */
	virtual void processReadFrame();

};

#endif
