 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	OGGMuxDemuxIO.hpp                                               *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _MK_OGGMUXDEMUXIO_HPP_
#define _MK_OGGMUXDEMUXIO_HPP_

#ifdef ENABLE_OGG

#include "MuxDemuxIO.hpp"
#include "container/OGGContainerFile.hpp"
#include "adaptors/TheoraDecoder.hpp"
#include "adaptors/VorbisDecoder.hpp"

class MuxDemuxIO;

/**
 *
 * muxes/demuxes from/to OGG container to elementary-stream BufferIOs
 *
 *@author Thomas Kuglitsch
 *
 *@version $Id: OGGMuxDemuxIO.hpp,v 1.13 2006/10/30 15:34:54 tkuglits Exp $
 *
 *@short <short description>
 *
 */
class OGGMuxDemuxIO:public MuxDemuxIO {

public:

  /* If writeOnly is true only write access
   * is permitted, otherwise the file is read-only, url is deep-copied
   */
  OGGMuxDemuxIO(ByteStream *stream);
  
  virtual ~OGGMuxDemuxIO();
  
  
  /** returns a frame and forwards it to the correct BufferIO
   * This function is typically blocking. Don't use a NULL
   * return value to conclude STREAMEOF, always check with getState()!
   */
  virtual Frame *getFrame();
  
  /** muxes a frame to the container */
  virtual int writeFrame(Frame * frm, ESInfo *out_es=NULL);
  
  static void initTheora(ESInfo* esi);
  static void initVorbis(ESInfo* esi);
  
  static theora_info* getTheoraInfo() ;
  static theora_state* getTheoraState();

  static vorbis_info* getVorbisInfo();
  static vorbis_dsp_state* getVorbisDspState();
  static vorbis_block* getVorbisBlock();

private:
  //  ogg_page og;
  static theora_info* ti;
  static theora_state* ts;
  static int theorainitialized;
  static int vorbisinitialized;

OGGHandler *osh, *osh_w;
//ogg_page  og;// = new ogg_page;
int serialtheora, serialvorbis;
//theora_state *ts;
//theora_info *ti;

static vorbis_info *vi;
static vorbis_comment *vc;
static vorbis_dsp_state *vd;
static vorbis_block *vb;

OGGStreamInfo *osi;
//int theorainitialized;
//int vorbisinitialized;

//theora_info* ti;
//theora_state* ts;

int vorbisframesSeen;
int theoraframesSeen;

  //int numStreams;
  //long* theStream;
  //int streamp;

long packetsSeen;

protected:


};

#endif
#endif
