 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:    QFrameBuffer.cpp                                           *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *  $Id: QFrameBuffer.cpp,v 1.4 2005/02/22 18:26:10 mkropfbe Exp $
 *                                                                      *
 ***********************************************************************/

#include <qapplication.h>
#include <qevent.h>
#include <qimage.h>
#include <qlabel.h>
#include <qpixmap.h>
#include "global.hpp"
#include "IO.hpp"
#include "Frame.hpp"
#include "QFrameBuffer.hpp"
#include "VideoFrameBuffer.hpp"



/*------------ QFrameBuffer methods --------------------------------------- */

QFrameBuffer::QFrameBuffer(QLabel *qlabel, int depth, int numColors, bool gray,
			   QEvent::Type eventType)
  : QWidget(qlabel), VideoFrameBuffer(gray) {

  this->qlabel = qlabel;
  this->eventType = eventType;
  int real_depth = (depth == 24) ? 32 : depth;
  image = new QImage(qlabel->width(), qlabel->height(), real_depth, numColors);
  if (depth == 24)
    image->setAlphaBuffer(false);
  if (numColors > 0 && gray) {
    // initialize the color table to gray values
    int i;
    QRgb *ct = image->colorTable();
    for (i=0; i < numColors; i++)
      ct[i] = qRgb(i, i, i);
  }
  hide();			// this widget should never be visible!
}

QFrameBuffer::~QFrameBuffer()
{
  delete image;
}

/* 
 * QApplication::postEvent() is thread-safe.
 */
void QFrameBuffer::postPaintEvent()
{
  /* request repainting of qlabel without erasing it before */
  QApplication::postEvent(this, new QCustomEvent(eventType));
}

void QFrameBuffer::customEvent(QCustomEvent *event)
{
  int rval;
  if (event->type() != eventType)
    return;
  lock();
	// bitblt into display
  rval = qlabel->pixmap()->convertFromImage(*image, 0);
  unlock();
  if (rval) {
    dprintf_full("QFrameBuffer::customEvent() repainting qlabel\n");
    /* repaint qlabel immediately, without erasing it before */
    QPaintEvent paintEvent(qlabel->visibleRect(), false);
    QApplication::sendEvent(qlabel, &paintEvent);
  } else {
    dprintf_err("QFrameBuffer::paintEvent(): convertFromImage() failed!\n");
  }
}
