 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:    QLabelRenderer.cpp                                         *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *  $Id: QLabelRenderer.cpp,v 1.17 2006/01/20 15:37:18 mkropfbe Exp $
 *                                                                      *
 ***********************************************************************/

#include <qapplication.h>
#include <qevent.h>
#include <qimage.h>
#include <qlabel.h>
#include <qpixmap.h>
#include "global.hpp"
#include "IO.hpp"
#include "Frame.hpp"
#include "QLabelRenderer.hpp"
#include "VideoRenderer.hpp"


QLabelRenderer::QLabelRenderer(const VideoESInfo *_es, UncompressedVideoFrame::ColorSpace colorSpace,
															 QLabel *qlabel, bool allowUpscaleX, QEvent::Type eventType)
	: VideoRenderer(_es, colorSpace, qlabel->width(), qlabel->height(), false, true, allowUpscaleX)
{
	dprintf_full("QLabelRenderer Construct %s ES %p\n",this->url, this->es);
  this->qlabel = qlabel;
  if (!qlabel->pixmap()) {
    QPixmap pixmap(qlabel->width(), qlabel->height());
    pixmap.fill(Qt::black);
    qlabel->setPixmap(pixmap);
  }
  qlabel->pixmap()->setOptimization(QPixmap::BestOptim);
  this->eventType = eventType;
	strcpy(this->url,"QLabelRenderer");
}

QLabelRenderer::~QLabelRenderer()
{
}

bool QLabelRenderer::updateDisplayRegion() {
	
  if (!es || !es->isVisualStream() 
      || !qlabel || !qlabel->width() || !qlabel->height()) {
    dprintf_err("QLabelRenderer::updateDisplayRegion(): invalid members: es = %p, qlabel = %p\n",
		es, qlabel);
		::exit(1);
    return false;
  }

	return VideoRenderer::updateDisplayRegion();

}

bool QLabelRenderer::open()
{
  if (!updateDisplayRegion()) {
    state = STREAMERR;
    return false;
  }
  /* initialize frameBuffer */
  if (frameBuffer) {
    delete frameBuffer;
    frameBuffer = NULL;
  }
  switch (colorSpace) {
  case UncompressedVideoFrame::ColorSpaceGRAY8:
    frame_info.size = frame_info.width * frame_info.height;
    // construct a frame buffer with 8 bpp and a color table of size 256
    frameBuffer = new QFrameBuffer(qlabel, 8, 256, true, eventType);
    break;
  case UncompressedVideoFrame::ColorSpaceRGB24:
    frame_info.size = frame_info.width * frame_info.height * 3;
    frameBuffer = new QFrameBuffer(qlabel, 24, 0, false, eventType);
    break;
  case UncompressedVideoFrame::ColorSpaceRGB32:
    frame_info.size = frame_info.width * frame_info.height * 4;
    frameBuffer = new QFrameBuffer(qlabel, 32, 0, false, eventType);
    break;
  default:
    dprintf_err("QLabelRenderer::open(): invalid color space: %d\n", colorSpace);
    state = STREAMERR;
    return false;
  }

  if (!frameBuffer) {
    dprintf_err("QLabelRenderer::open(): QFrameBuffer creation FAILED! (w = %d, h = %d)\n",
		qlabel->width(), qlabel->height());
    state = STREAMERR;
    return false;
  }

	//setVideoFrameBuffer(frameBuffer);
  state = OPEN;
  framesSeen = 0;
  currentFrameNumber = 0;
  dprintf_small("QLabelRenderer::open(): initialized frame and display regions:\n"
								"    frame: width = %d, height = %d, size = %d, scaleX = %5.3f, scaleY = %5.3f\n"
								"    display: x(%d:%d), y(%d:%d), width = %d, height = %d\n",
								frame_info.width, frame_info.height, frame_info.size,
								FIXED2DOUBLE(frame_info.scaleX), FIXED2DOUBLE(frame_info.scaleY),
								display_region.left, display_region.left + display_region.width,
								display_region.top, display_region.top + display_region.height,
								display_region.width, display_region.height);
  return true;
}


int QLabelRenderer::writeFrame(Frame * frm, ESInfo *out_es)
{
	return VideoRenderer::writeFrame(frm);
}
