 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	RawStreamIO.hpp                                               *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _MK_RAWSTREAMIO_HPP_
#define _MK_RAWSTREAMIO_HPP_

#include "IO.hpp"
#include "global.hpp"

/**
 *
 * writes raw frames (eg. YUV, audio-pcm or any other raw content)
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: RawStreamIO.hpp,v 1.4 2006/01/26 11:42:00 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class RawStreamIO:public IO {
public:
	/* If writeonly is true only write access
	 * is permitted, otherwise the file is read-only, url is deep-copied
	 */
	RawStreamIO(ESInfo * es, const char *file, bool writeOnly);

	virtual ~ RawStreamIO();


	/* absolutely needed for <code>getFrame</code>
	 * @param size gives the bytesize of one frame
	 */
	void setFrameSize(u32 size);
	
	/** returns a frame if one complete frame is available, otherwise null 
	 * is returned. This function is typically blocking. Don't use a NULL
	 * return valuse to conclude STREAMEOF, always check with getState()!
	 */
	virtual Frame *getFrame();
	
  /** returns the number of packets sent. Returns 0 on error */
	virtual int writeFrame(Frame * frm, ESInfo *out_es =  NULL);
	
  /** opens the IO connection. State is set to OPENING. Depending on the
	 *  underlying IODevice, a network connection is stablished or a file
	 *  connection. When the connection is ready for use, State is OPEN */
	bool open();

	bool close(bool immediate=false);

  /** deletes the elementary stream and its extra hint-file */
	bool destroy();
	
  /** returns a value  from 0..100 indicating buffer usage. Fixme: returns a constant value of 50 */
	int getBufferFillLevel() const {
		return 50;
	};
	
	IO *clone() const { return new RawStreamIO(es,url,writeOnly);};
	
  /** returns a local file name or an URL */
	const char *getURL() const {
		return url;
	};

	/** repositions the IO class to the given frame.
	 * Will return false in the following cases:
	 *  - an illegal frame number (too large) was specified
	 *  - the stream is not seekable, because the underlying 
	 *    device is a network device and the requested frame 
	 *    is already out of range: FIXME: implement
	 */
	bool setToFrameNumber(u32 frameNumber);

	void setESInfo(ESInfo * esi) {es=esi; };

	ESInfo* getESInfo() { return es; };		

private:
protected:
	FILE *io;
	char *url;
	bool writeOnly;
	ESInfo *es;
	bool atFileStart;
	long lastCTSSeen;
	u64 frameSize;
};

#endif
