 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 *  title:   SDLFrameBuffer.hpp                                         *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *  $Id: SDLFrameBuffer.hpp,v 1.6 2005/01/16 16:35:12 mkropfbe Exp $
 *                                                                      *
 ***********************************************************************/

#ifndef _SDLFRAMEBUFFER_HPP
#define _SDLFRAMEBUFFER_HPP

#include "IO.hpp"
#include "VideoESInfo.hpp"
#include "adaptors/MP4Decoder.hpp"
#include "VideoFrameBuffer.hpp"
#include "SDL.h"


/**
 * @short This class serves as a frame buffer and event handler of a SDL RGB Surface
 * widget for displaying video frames.
 *
 * @author Michael Kropfberger
 * @version $Id: SDLFrameBuffer.hpp,v 1.6 2005/01/16 16:35:12 mkropfbe Exp $
 */
class SDLFrameBuffer :public VideoFrameBuffer
{
public:
  /** Constructor. 
	 * @param surface     the SDL_Surface of the actual SDL window on the display
   * @param depth       the color depth of the frame buffer (8, 24, 32).
   * @param numColors   the color table size of the frame buffer
   *                    (if <code>depth == 8</code>).
   * @param gray        if <code>true</code> and <code>numColors > 0</code>, set the
   *                    color table entries to all gray values.
   */
  SDLFrameBuffer(SDL_Surface *surface, int depth, bool gray, int width, int height);

  /** Destructor. */
  virtual ~SDLFrameBuffer();

  /** Set a pixel in the frame buffer.
   * the frame buffer should be locked before setPixel()
   * by calling the {@link #lock()} method.
   */
  inline void setPixel(int x, int y, uint value);

	/** does the fast bitblitting of the full (and unscaled) Frame
	 */
	void bitBlt(AU *au);

  /** Lock the mutex protecting the frame buffer.
   * This function will block the calling thread until the mutex can be obtained.
   * @see #unlock()
   */
  inline void lock() {
		if ( SDL_MUSTLOCK(surface) )
			{
        if ( SDL_LockSurface(surface) < 0 ) {
					::exit(4);
        }
			}
  }

  /** Unlock the mutex locked by {@link #lock()}. */
  inline void unlock() {
		if ( SDL_MUSTLOCK(surface) )
			{
        SDL_UnlockSurface(surface);
			}
  }

  /** Post a custom event for <code>this</code> object
   * to display the frame buffer on the SDL surface.
   * This is thread-safe and will cause the GUI thread to invoke
   * {@link #customEvent()}. We need to use custom events, as
   * hidden widgets will not receive paint events.
   */
  void postPaintEvent();

protected:
	SDL_Surface *image;
	SDL_Surface *surface;
	int width;
	int height;
};

#endif /* _SDLFRAMEBUFFER_HPP */
