 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	SLASH_dev_SLASH_audio.c                                       *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef WIN32

#include "SLASH_dev_SLASH_audio.h"

//static
int output(int  fd, void const *buf, unsigned int len) {
  char const *ptr = buf;
  int wrote;
	
  while (len) {
    wrote = write(fd, ptr, len);
    if (wrote == -1) {
      if (errno == EINTR)
				continue;
      else
				return -1;
    }

    ptr += wrote;
    len -= wrote;
  }
	
  return 0;
}

//static
int audio_buffer(int  fd, void const *buf, unsigned int len) {
  char const *ptr = buf;
  static char hold[AUDIO_FILLSZ];
  static unsigned int held;
  unsigned int left, grab;
  if (len == 0) {
    if (held) {
      memset(&hold[held], 0, &hold[AUDIO_FILLSZ] - &hold[held]);
      held = 0;
			
      return output(fd, hold, AUDIO_FILLSZ);
    }
		
    return 0;
  }
	
  if (held == 0 && len == AUDIO_FILLSZ)
    return output(fd, ptr, len);
	
  left = AUDIO_FILLSZ - held;
	
  while (len) {
    grab = len < left ? len : left;
		
    memcpy(&hold[held], ptr, grab);
    held += grab;
    left -= grab;
		
    ptr  += grab;
    len  -= grab;
		
    if (left == 0) {
      if (output(fd, hold, AUDIO_FILLSZ) == -1)
				return -1;
			
      held = 0;
      left = AUDIO_FILLSZ;
    }
  }
	
  return 0;
}


//static
int dsp_audio_init(int fd, unsigned int muiTicksPerSecond, unsigned int muiChannelMod, int enableHeadphones) {
  int format, stereo, speed;

	/*	
	if (ioctl(fd,SNDCTL_DSP_RESET,NULL) == -1) {
    perror("ioctl(SNDCTL_DSP_RESET)");
    return -1;
	}
	*/

	
#ifdef BUILD_ARCH_ARM
	int mask;
	if (enableHeadphones == 1)
		mask = SOUND_MASK_LINE;
 	else
		mask = ~SOUND_MASK_LINE;

	if (ioctl(fd, SOUND_MIXER_WRITE_RECSRC, &mask) == -1) { 
		perror("ioctl: switch to headphones out");
		return -1;
	}
#endif

  //format = AFMT_S16_BE;  // big endian
 	format = AFMT_S16_LE;  // little endian
	if (ioctl(fd, SNDCTL_DSP_SETFMT, &format) == -1) {
    perror("ioctl(SNDCTL_DSP_SETFMT)");
    return -1;
  }
  if (format != AFMT_S16_LE) {
    fprintf(stderr, "AFMT_S16_LE not available\n");
    return -1;
  }
	
	stereo = muiChannelMod;
  if (ioctl(fd, SNDCTL_DSP_CHANNELS, &muiChannelMod) == -1) {
    perror("ioctl(SNDCTL_DSP_CHANNELS)");
    return -1;
  }
	
  if (muiChannelMod != stereo) {
    fprintf(stderr, "mono/stereo selection failed\n");
    return -1;
  }
	
  speed = muiTicksPerSecond;
  if (ioctl(fd, SNDCTL_DSP_SPEED, &speed) == -1) {
    perror("ioctl(SNDCTL_DSP_SPEED)");
    return -1;
  }
  if (speed != muiTicksPerSecond) {
    fprintf(stderr, "sample speed %u not available (closest %u)\n", muiTicksPerSecond, speed);
    return -1;
  }

  return 0;
}


#endif /* WIN32 */

