/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: UnbufferedFileByteStream.hpp                                  *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "UnbufferedFileByteStream.hpp"

#ifndef WIN32
#include <sys/file.h> 
#endif
 
#ifndef WINCE
#include <errno.h>
#endif

UnbufferedFileByteStream::UnbufferedFileByteStream(const char* url, bool forReading) : ByteStream(url,forReading) {
	io = NULL;
}


bool UnbufferedFileByteStream::open() {
	if (io) { //already open....
		dprintf_err("UnbufferedFileByteStream::open: %s already open!\n",url);
		return true;
	}

	if (readOnly)
		io = fopen(url,"rb");
	else {
		io = fopen(url,"w");
#ifndef WIN32
		if (io) {
			//make sure that we are the only writer system-wide!
			int val = flock(fileno(io),LOCK_EX | LOCK_NB);
			if (val == -1)
				perror("UnbufferedFileByteStream::open flock out-file");
			//ret = ((val == 0) || (errno == ENOLCK)); //eg. NFS might not support flocks
			if (errno == EWOULDBLOCK) {
				dprintf_err("MuxDemuxIO::open(): FATAL: lock on %s is held by other process! \n", url);
				::exit(1);
			}
		}
#endif
	}

	if (!io) {
		dprintf_err("UnbufferedFileByteStream::open: file not found: %s!\n",url);
		return false;
	} else {
		return true;
	}
}


bool UnbufferedFileByteStream::close() {
	if (!io)
		return false;
	else
		return fclose(io);
}

int UnbufferedFileByteStream::getChar() {
	if (!io)
		return false;

	return fgetc(io);
}


int UnbufferedFileByteStream::read(void *ptr, int size) {
	if (!io || !readOnly)
		return -1;

	int ret = fread(ptr,size,1,io);

	//NOTE: ignores any ferrors, just assume eof...
	if ( (ret == 0) && (feof(io) || ferror(io)) )
		//return -1;
		return 0;

	return size;
}


int UnbufferedFileByteStream::write(void *ptr, int size){
	if (!io|| readOnly)
		return -1;

	int ret = fwrite(ptr,size,1,io);
	fflush(io);

	//assure that all bytes where written!
	if ( (ret < size) || feof(io) || ferror(io) )
		return -1;

	return size;
}


int UnbufferedFileByteStream::seek(u64 pos) {
	if (!io)
		return -1;

	return fseek(io, pos, SEEK_SET);
}


u64 UnbufferedFileByteStream::tell() {
	if (!io)
		return 0;

	return ftell(io);
}
