 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 *  title: VideoFrameBuffer.hpp                                         *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *  $Id: VideoFrameBuffer.hpp,v 1.5 2005/04/27 10:21:51 klschoef Exp $
 *                                                                      *
 ***********************************************************************/

#ifndef _VIDEOFRAMEBUFFER_HPP
#define _VIDEOFRAMEBUFFER_HPP

#include "IO.hpp"
#include "VideoESInfo.hpp"
#include "adaptors/MP4Decoder.hpp"
#include "VMutex.hpp"


/**
 * @short This class serves as a frame buffer and event handler of a graphical
 * widget/label for displaying video frames.
 * <p>
 * It manages an pixmap/image object as a frame buffer whose pixels may be set using
 * the {@link #setPixel()} method. For painting the frame buffer's contents into
 * the label/widget's pixmap, a {@link #customEvent()} handler is implemented.
 * </p>
 *
 * @author Mario Taschwer and Michael Kropfberger
 * @version $Id: VideoFrameBuffer.hpp,v 1.5 2005/04/27 10:21:51 klschoef Exp $
 */
class VideoFrameBuffer
{
public:

  /** Constructor. 
   */
  VideoFrameBuffer(bool gray);

  /** Destructor. */
  virtual ~VideoFrameBuffer();

  /** Set a pixel in the frame buffer.
   * If this method is called from a thread other than the Qt
   * GUI thread, the frame buffer should be locked before
   * by calling the {@link #lock()} method.
   */
  virtual inline void setPixel(int x, int y, uint index_or_rgb) {};

	/** does the fast bitblitting of the full (and unscaled) Frame
	 */
	virtual inline void bitBlt(AU *au) {};

  /** Lock the mutex protecting the frame buffer.
   * This function will block the calling thread until the mutex can be obtained.
   * @see #unlock()
   */
  inline void lock() {
    mutex.lock(); 
  }

  /** Unlock the mutex locked by {@link #lock()}. */
  inline void unlock() {
    mutex.release();
  }

  /** Post a custom event for <code>this</code> object
   * to display the frame buffer on the label/widget.
   */
  virtual void postPaintEvent() = 0;

protected:
	bool gray;

private:
  /** The mutex protecting the frame buffer. */
  VMutex mutex;


};

#endif /* _VIDEOFRAMEBUFFER_HPP */
