 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:    X11FrameBuffer.cpp                                         *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *  $Id: X11FrameBuffer.cpp,v 1.7 2005/04/19 12:51:55 mkropfbe Exp $
 *                                                                      *
 ***********************************************************************/

#ifndef WIN32

#include "global.hpp"
#include "IO.hpp"
#include "Frame.hpp"
#include "X11FrameBuffer.hpp"
#include "VideoFrameBuffer.hpp"



/*------------ X11FrameBuffer methods --------------------------------------- */

X11FrameBuffer::X11FrameBuffer(Display *display, Window *xwin, int depth, bool gray, int width, int height)
  : VideoFrameBuffer(gray)
{
	this->display = display;
	this->xwin = xwin;
	this->width = width;
	this->height = height;
	this->depth = depth;
	assert(depth % 8 == 0);

	gc=XCreateGC(display, *xwin, 0, NULL);
	pixmap = (char*)calloc(width*height, sizeof(depth/8));
	image = XCreateImage(display, CopyFromParent,
											 depth, ZPixmap, 0, pixmap,
											 width, height, 8 /*don't pad scanline*/, 0); //BitmapPad(display), 0);
	//	row_size = width + (width % (BitmapPad(display)/8) != 0);
	row_size = width;
	
	if (!image || !pixmap) {
		dprintf_err("Couldn't initialize blitbuffer at depth %i with dimension %ix%i\n",depth,width,height);
		::exit(1);
	}
	dprintf_err("X11FrameBuffer: depth %i bit, row_size %i\n",depth, row_size);
}


X11FrameBuffer::~X11FrameBuffer()
{
	if(image)
		XDestroyImage(image);
	image = NULL;
	//should be freed automatically by XDestroyImage
	if (pixmap)
		free(pixmap);
	pixmap = NULL;
}


void X11FrameBuffer::setPixel(int x, int y, uint value) {
	if (gray)
		value = value << 16 | value << 8 | value;
	
	if (depth >=24) {
		pixel32 = (uint *)pixmap + y*row_size + x;
 		*pixel32 = value; 
	} else {
		pixel16 = (unsigned short int *)pixmap + y*row_size + x;
		*pixel16 = (unsigned short int) value;
	}
}


void X11FrameBuffer::bitBlt(AU *au) {
	dprintf_full("X11FrameBuffer::bitBlt   AU->size %i\n", au->size);
 	memcpy(pixmap, au->payload, au->size);
}


void X11FrameBuffer::postPaintEvent()
{
	XPutImage(display, *xwin, gc, image, 0,0,0,0, width, height); 
  XSync(display, 0);
  XFlush(display);
}


#endif /* WIN32 */
