 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 *  title:   X11FrameBuffer.hpp                                         *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *  $Id: X11FrameBuffer.hpp,v 1.8 2005/04/19 12:51:55 mkropfbe Exp $
 *                                                                      *
 ***********************************************************************/


#ifndef WIN32

#ifndef _X11FRAMEBUFFER_HPP
#define _X11FRAMEBUFFER_HPP

#include <X11/Xatom.h>
#include <unistd.h>

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>

#include "IO.hpp"
#include "VideoESInfo.hpp"
#include "adaptors/MP4Decoder.hpp"
#include "VideoFrameBuffer.hpp"


/**
 * @short This class serves as a frame buffer and event handler of a SDL RGB Surface
 * widget for displaying video frames.
 *
 * @author Michael Kropfberger
 * @version $Id: X11FrameBuffer.hpp,v 1.8 2005/04/19 12:51:55 mkropfbe Exp $
 */
class X11FrameBuffer :public VideoFrameBuffer
{
public:
  /** Constructor. 
	 * @param display     the X11 display handle
	 * @param xwin        the X11 window handle
   * @param depth       the color depth of the frame buffer (8, 24, 32).
   * @param numColors   the color table size of the frame buffer
   *                    (if <code>depth == 8</code>).
   * @param gray        if <code>true</code> and <code>numColors > 0</code>, set the
   *                    color table entries to all gray values.
   */
  X11FrameBuffer(Display *display, Window *xwin, int depth, bool gray, int width, int height);

  /** Destructor. */
  virtual ~X11FrameBuffer();

  /** Set a pixel in the frame buffer.
   * the frame buffer should be locked before setPixel()
   * by calling the {@link #lock()} method.
   */
  inline void setPixel(int x, int y, uint value);

	/** does the fast bitblitting of the full (and unscaled) Frame
	 */
	void	bitBlt(AU *au);
	
  /** Post a custom event for <code>this</code> object
   * to display the frame buffer on the SDL surface.
   * This is thread-safe and will cause the GUI thread to invoke
   * {@link #customEvent()}. We need to use custom events, as
   * hidden widgets will not receive paint events.
   */
  void postPaintEvent();

protected:
	Display *display;
	Window *xwin;
  GC gc;
	XImage *image;
	char *pixmap;

	int width;
	int height;
	int row_size;
	uint *pixel32;
	unsigned short int *pixel16;
	int depth;
};


#endif /* _X11FRAMEBUFFER_HPP */

#endif /* WIN32 */
