 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	YUVStreamIO.cpp                                               *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "YUVStreamIO.hpp"
#include "VideoESInfo.hpp"
#ifndef WINCE
#include <errno.h>
#endif
#ifndef WIN32
#include <sys/file.h> 
#endif
 
YUVStreamIO::YUVStreamIO(VideoESInfo * esi, const char *file, bool write): RawStreamIO(esi,file,write) {
	setFrameSize((u64)(esi->getWidth() * esi->getHeight() * 1.5));
}



/** returns a frame if one complete frame is available, otherwise null 
		is returned. This function is typically blocking. Don't use a NULL
		return value to conclude STREAMEOF, always check with getState()!
*/
Frame *YUVStreamIO::getFrame() {
	UncompressedVideoFrame *frm;

  Frame *inFrm = RawStreamIO::getFrame();

	if (!inFrm)
		return NULL;
	else {
		frm = new UncompressedVideoFrame(Frame::YUV_VOP, ((VideoESInfo*)(es))->getWidth(), ((VideoESInfo*)(es))->getHeight());
		frm->setColorSpace(UncompressedVideoFrame::ColorSpaceYV12);
		frm->setMediaTimeScale(inFrm->getMediaTimeScale());
		frm->setAU(inFrm->getAU(),false);
		inFrm->unsetAU(); //so it will not be freed...
		delete inFrm;
		return frm;
	}
}

