 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	isoMP4IO.hpp                                                  *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PS_SERVER_ISOMP4IO_HPP
#define _PS_SERVER_ISOMP4IO_HPP


#include "MP4IO.hpp"


#ifdef ISOMP4
#include "ISOMovies.h"

class Frame;
class ESInfo;

/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: isoMP4IO.hpp,v 1.7 2006/01/26 11:42:00 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class isoMP4IO:public MP4IO {
public:
	/** reading from an MP4 file is only possible from a file.
	 * mp4File is deep-copied. If writeonly is true only write access
	 * is permitted, otherwise the file is read-only
	 */
	isoMP4IO(ESInfo * es, const char *mp4File, bool writeOnly);
	
	/** frees everything except es */
	virtual ~ isoMP4IO();

	/** returns a frame if one complete frame is available, otherwise null 
			is returned. This function is typically blocking. Don't use a NULL
			return value to conclude STREAMEOF, always check with getState()!
	*/
	Frame *getFrame();
	
	/** returns the number of packets sent. Returns 0 on error */
	int writeFrame(Frame * frm, ESInfo *out_es=NULL);
  
  /** opens the IO connection. State is set to OPENING. Depending on the
			underlying QIODevice, a network connection is stablished or a file
			connection. When the connection is ready for use, State is OPEN */
	bool open();

	bool close(bool immediate=false);
	
	/** be vary careful, when destroying MP4IO files, because many MP4IO objects will
	 * refer to the same file. It is suggested that one deletes MP4 videos with the
	 * ContainerInfo::destroy method, which will take care of that
	 */
	bool destroy();
    
	/** returns -1, when at end of File, returns 0 when file is closed
	 * returns 50 otherwise */
	int getBufferFillLevel() const;
  
	void setESInfo(ESInfo * esi) {streamInfo=esi;};

	ESInfo* getESInfo() { return streamInfo;};

protected:
	ESInfo * streamInfo;
	ISOMovie moov;
	ISOTrack trak;
	ISOHandle au;
	ISOTrackReader reader;
	bool writeOnly;
	dfadd(fp);

private:
	bool openForReading();
	bool openForWriting();
};

#endif

#endif
