/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: M3SMetaInfo.hpp                                               *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/
#ifndef M3SMETAINFO_HPP
#define M3SMETAINFO_HPP
 
#include "global.hpp"
#include "net/HttpRequest.hpp"


#define LINE_SIZE	1024


class M3SMetaInfo
{
public:
	typedef enum {none, general, video, audio} SECTION;

	M3SMetaInfo();
	~M3SMetaInfo();
	
	void initialize();
	
	bool loadFromFile(const char *filename); 

	bool loadFromURL(const char *url);
	
	int getLength() { return length; };
	
	bool isVideoEnabled() { return videoEnabled; };
	char *getVideoFilepath() { return videoFilepath; };
	short getFPS() { return fps; };
	short getVideoKBPS() { return videoKbps; };
	short getWidth() { return width; };
	short getHeight() { return height; };
	
	bool isAudioEnabled() { return audioEnabled; };
	char *getAudioFilepath() { return audioFilepath; };
	short getAudioKBPS() { return audioKbps; };
	short getNumAudioChannels() { return numAudioChannels; };
	unsigned int getSampleRate() { return sampleRate; };
	
	
	void setVideoFilepath(char *videoFilepath);
	void setAudioFilepath(char *audioFilepath);
	void appendVideoFilepath(int paramSize, char *prefix, int len);
	void appendAudioFilepath(int paramSize, char *prefix, int len);


private:
	int readLine(char **unallocatedLine, char *content, u32 &refOffset, u32 contentLength);
	void parseFileContent(char *content, u32 contentLength);
	
protected:
	int length;
	
	bool videoEnabled;
	char *videoFilepath;
	short fps;
	short videoKbps;
	short width;
	short height;
	
	bool audioEnabled;
	char *audioFilepath;
	short audioKbps;
	short numAudioChannels;
	unsigned int sampleRate;

	HttpRequest *http;
	
	char *findParam(char *line, char *name, int *len);
};

#endif


