 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	MP7File.hpp                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PS_MP7FILE_
#define _PS_MP7FILE_

#include <cstdio>
#include <cstdlib>

#include "Adaptor.hpp"
#include "../adaptors/TemporalAdaptor.hpp"
#include "../adaptors/StrongTemporalAdaptor.hpp"

#ifdef ISOMP4
#include "ISOMovies.h"
#endif

class ContainerInfo;
class ESInfo;

/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: MP7File.hpp,v 1.5 2004/03/25 13:38:17 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class MP7File {
  public:
	/** generates an MPEG-7 description for a locally stored ContainerInfo object.
	  * This method will apply different Adaptors to the video 
	  * and use this information to generate the mp7file. 
	  * Note: this method can take very long.
	  * In case of an error NULL is returned, otherwise the name of the generated MPEG-7 file	  
	  */ 
    static char *generateMP7Description(ContainerInfo * mp4Info,
					char *mp7File,
					bool showVisualResult = false);
    
	/** sets the MetaItems (incl. Adaptors) in the QualityInfo object of the ContainerInfo object
	  * by parsing the mp7File. The .mp7 file must be in ASCII format, BIM is not supported.
	  * If sth. went wrong false is returned, otherwise true.
	  */ 
    static bool setAdaptors(ContainerInfo * mp4Info, char *mp7File);
    
	/** convert the Meta-Info stored in the mp4Info object to an MPEG-7 description.
	  * Already applied adaptors will be exclude form the MPEG-7 description.
	  */ 
    static bool convertMetaInfoToMP7(const ContainerInfo * mp4Info,
				     char *mp7File);
    
	/** opens a FILE and writes an MPEG-7 header to the file. In case of an error
	  * NULL is returne, otherwise the file pointer
	  */ 
    static FILE *openMP7File(const char *mp7File);
    
	/** closes an mpeg-7 file, writes the closing header, returns false if fp is NULL */ 
    static bool closeMP7File(FILE * fp);
    
	/** writes a variationset entry to a file, applies all Adaptors specified in @param adaptor to the mp4Info
	  * object, which is not modified! 
	  * @param showVisualResults creates a Renderer that displays adaptation results.
	  */ 
    static bool writeVariationSet(FILE * fp, Adaptor ** adaptor,
				  int numAdaptors, ContainerInfo * mp4Info,
				  bool showVisualResult = false);
  private:
	/** writes the follwing : @param close is false: <key>
	  * @param close is true </key>
	  */ 
    static void writeEntry(FILE * fp, u32 indent, const char *const key,
			   bool close);
    
	/** writes the following MPEG-7 entries: <key/> */ 
    static void writeSimpleEntry(FILE * fp, u32 indent,
				 const char *const key);
    
	/** write one ComponentMediaProfile entry. Returns the size of the ES */ 
    static u32 writeCMP(FILE * fp, u32 indent, ESInfo * es);
    
	/** writes the media information and returns the total size of the video */ 
    static u32 writeMediaInformation(FILE * fp, u32 indent,
				     ContainerInfo * mp4);
    static void writeAdaptation(FILE * fp, u32 indent,
				 ContainerInfo * mp4, const char* a,
				 u32 variationId, u32 priority,
				 float fidelity);
    static void writeVideoDescriptionEntry(FILE * fp, u32 indent,
					    const char *const key,
					    u32 intVal);
    static void writeVideoDescriptionEntry(FILE * fp, u32 indent,
					    const char *const key,
					    const char *stringVal);
  protected:
		static const char *const NL;
    static const char *const content;
    static const char *const name;
    static const char *const fileFormat;
    static const char *const variationSet;
    static const char *const fileSize;
    static const char *const mediaProfile;
    static const char *const componentMediaProfile;
    static const char *const bitRate;
    static const char *const mediaLocator;
    static const char *const preHeader;
    static const char *const closeHeader;
};


#endif
