/***********************************************************************
*                                                                      *
* ViTooKi                                                              *
*                                                                      *
* title:  MP7Parser.cpp                                                *
*                                                                      *
*                                                                      *
*                                                                      *
* ITEC institute of the University of Klagenfurt (Austria)             *
* http://www.itec.uni-klu.ac.at                                        *
*                                                                      *
*                                                                      *
* For more information visit the ViTooKi homepage:                     *
* http://ViTooKi.sourceforge.net                                       *
* vitooki-user@lists.sourceforge.net                                   *
* vitooki-devel@lists.sourceforge.net                                  *
*                                                                      *
* This file is part of ViTooKi, a free video toolkit.                  *
* ViTooKi is free software; you can redistribute it and/or             *
* modify it under the terms of the GNU General Public License          *
* as published by the Free Software Foundation; either version 2       *
* of the License, or (at your option) any later version.               *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
* MA  02111-1307, USA.                                                 *
*                                                                      *
***********************************************************************/

/***********************************************************************
*                                                                      *
*  REVISION HISTORY:                                                   *
*                                                                      *
*                                                                      *
*                                                                      *
***********************************************************************/ 

#include "MP7Parser.hpp"
#include <string>
#include "MP7Time.hpp"

int depth;

// class for MP7GetVarDescrType data exchange
class MP7_VarDescrType_Userdata {
public:
	int error;
	int varset_counter;
	bool inside_vardescrtype;
	bool vardescr;
	bool source;
	bool variation;
	// constructor
	MP7_VarDescrType_Userdata();
};

// class for GetVarSet data exchange
class MP7_VarSet_Userdata: public MP7_VarDescrType_Userdata {
public:
	int which_varset;
	int variation_counter;
	bool get_visualcoding_name;
	bool get_audiocoding_name;
	bool get_fileformat_name;
	bool get_mediauri;
	bool get_filesize;
	bool get_bitrate;
	bool get_title;
	bool get_titleimage;
	bool get_freetextannotation;
	bool get_keyword;
	bool get_name;
	bool get_lo_name;
	bool get_lo_region;
	bool get_lo_date;
	bool get_copyright;
	bool get_media_duration;
	bool get_media_timepoint;
	bool get_semantic_name;
	bool get_semantic_region;
	bool get_semantic_time;
	bool inside_semantic;
	bool inside_semantic_place;
	bool inside_semantic_time;
	bool inside_semantic_place_place;
	bool inside_varset;
	bool inside_source;
	bool inside_visualcoding;
	bool inside_audiocoding;
	bool inside_fileformat;
	bool inside_mediainformation;
	bool inside_creationinformation;
	bool inside_titlemedia;
	bool inside_textannotation;
	bool inside_creator;
	bool inside_coordinates;
	bool inside_mediatime;
	MP7Parser::MP7_Sourcedata source_data;
	MP7Parser::MP7_Metadata meta_data;
	MP7Parser::MP7_MediaTime media_time;
	// constructor
	MP7_VarSet_Userdata();
};

// class for MP7GetVariation data exchange
class MP7_Variation_Userdata: public MP7_VarSet_Userdata {
public:
	int which_variation;
	bool inside_variation;
	bool get_varrelationship;
	MP7Parser::MP7_Variationdata var_data;
	// constructor
	MP7_Variation_Userdata();
};

// open a given filename and return the handle
FILE *MP7Parser::MP7Open(const char *filename)
{
	FILE *fp  = fopen(filename, "r");
	if (!fp) {
		return NULL;
	}
	return fp;
}

// close a handle
bool MP7Parser::MP7Close(FILE *fp)
{
	if (!fp) {
		return false;
	}
	fclose(fp);
	return true;
}

double MP7Parser::MP7_MediaTime::tpAsDouble()
{
	MP7Time mp7time;

	mp7time.setTimePoint(mediatimepoint_alpha.c_str());
	return mp7time.timeToSec();
}

double MP7Parser::MP7_MediaTime::durAsDouble()
{
	MP7Time mp7duration;
	
	mp7duration.setDuration(mediaduration_alpha.c_str());
	return mp7duration.durationToSec();
}

// constructor for MP7_VarDescrData
MP7Parser::MP7_VarDescrData::MP7_VarDescrData()
{
	errorcode = ERR__OK;
	nr_varsets = 0;
	vsl_pointer = NULL;
}

// destructor for MP7_VarDescrData
MP7Parser::MP7_VarDescrData::~MP7_VarDescrData()
{
	if (vsl_pointer != NULL) {
		delete vsl_pointer;
		vsl_pointer = NULL;
	}
}

// assignment operator for MP7_VarDescrData
MP7Parser::MP7_VarDescrData & MP7Parser::MP7_VarDescrData::operator= (const MP7_VarDescrData & other)
{
	MP7_VarSetList *newvsl;

	if (&other != this) {
		errorcode = other.errorcode;
		nr_varsets = other.nr_varsets;
		if (other.vsl_pointer != NULL) {
			newvsl = new MP7_VarSetList;
			newvsl->sdata = other.vsl_pointer->sdata;
			newvsl->vdata = other.vsl_pointer->vdata;
			newvsl->mdata = other.vsl_pointer->mdata;
			newvsl->mtime = other.vsl_pointer->mtime;
			newvsl->tmpdec = other.vsl_pointer->tmpdec;
			newvsl->nr_variations = other.vsl_pointer->nr_variations;
			newvsl->next = other.vsl_pointer->next;
			vsl_pointer = newvsl;
		}
	}
	return *this;
}

// copy constructor for MP7_Sourcedata
MP7Parser::MP7_VarDescrData::MP7_VarDescrData(const MP7_VarDescrData & other)
{
	*this = other;
}

// constructor for MP7_VarSetList
MP7Parser::MP7_VarSetList::MP7_VarSetList()
{
	nr_variations = 0;
	next = NULL;
}

// constructor for MP7_VarList
MP7Parser::MP7_VarList::MP7_VarList()
{
	next = NULL;
}


// constructor for MP7_Sourcedata
MP7Parser::MP7_Sourcedata::MP7_Sourcedata()
{
	has_audio = false;
	video_height = 0;
	video_width = 0;
	video_rate = 0.0;
	filesize = 0;
	bitrate = 0;
	id = "";
	video_codec = "";
	audio_codec = "";
	fileformat = "";
	path = "";
}


// constructor for MP7_Variationdata
MP7Parser::MP7_Variationdata::MP7_Variationdata()
{
	fidelity = 0.0;
	priority = 0;
	variation_id = "";
	variation_relationship = "";
}

// constructor for MP7_MediaTime
MP7Parser::MP7_MediaTime::MP7_MediaTime()
{
	mediatimepoint_alpha = "";
	mediaduration_alpha = "";
}

// constructor for MP7_Metadata
MP7Parser::MP7_Metadata::MP7_Metadata()
{
	title = "";
	title_image = "";
	free_text_annotations = "";
	keywords = "";
	creator_name = "";
	location_name = "";
	location_region = "";
	date = "";
	semantic_place = "";
	semantic_region = "";
	semantic_time = "";

	copyright = "";
}

MP7Parser::MP7_VideoSegmentData::MP7_VideoSegmentData()
{
	title = "";
	title_image = "";
	free_text_annotations = "";
	keywords = "";
	semantic_place = "";
	semantic_region = "";
	semantic_time = "";
}

MP7Parser::MP7_TemporalDecomposition::MP7_TemporalDecomposition()
{
	errorcode = 0;
	vseg_list = NULL;
}

MP7Parser::MP7_VideoSegmentList::MP7_VideoSegmentList()
{
	next = NULL;
	children = NULL;
}


MP7Parser::MP7_Variations::MP7_Variations()
{
	errorcode = 0;
	var_list = NULL;
}
// constructor for MP7_Variation_Userdata
MP7_Variation_Userdata::MP7_Variation_Userdata()
{
	which_variation = 1;
	inside_variation = false;
	get_varrelationship = false;
}

// constructor for MP7_VarDescrType_Userdata
MP7_VarDescrType_Userdata::MP7_VarDescrType_Userdata()
{
	error = ERR__OK; 
	varset_counter = 0;
	inside_vardescrtype = false;
	vardescr = false;
	source = false;
	variation = false;
}

// constructor for MP7_VarSet_Userdata
MP7_VarSet_Userdata::MP7_VarSet_Userdata()
{
	which_varset = 1;
	variation_counter = 0;
	get_visualcoding_name = false;
	get_audiocoding_name = false;
	get_fileformat_name = false;
	get_filesize = false;
	get_bitrate = false;
	get_mediauri = false;
	get_title = false;
	get_titleimage = false;
	get_freetextannotation = false;
	get_keyword = false;
	get_name = false;
	get_lo_name = false;
	get_lo_region = false;
	get_lo_date = false;
	get_copyright = false;
	get_media_duration = false;
	get_media_timepoint = false;
	get_semantic_name = false;
	get_semantic_region = false;
	get_semantic_time = false;
	inside_varset = false;
	inside_source = false;
	inside_fileformat = false;
	inside_visualcoding = false;
	inside_audiocoding = false;
	inside_mediainformation = false;
	inside_creationinformation = false;
	inside_titlemedia = false;
	inside_textannotation = false;
	inside_creator = false;
	inside_coordinates = false;
	inside_mediatime = false;
	inside_semantic = false;
	inside_semantic_place = false;
	inside_semantic_time = false;
	inside_semantic_place_place = false;
}

class MP7_CheckVideoSegments_Userdata {
public:
	int error;
	bool inside_varset;
	bool inside_source;
	bool inside_tempdecomp;
	bool inside_segment;
	int segment_counter;
	int varset_counter;
	int which_varset;
	// constructor
	MP7_CheckVideoSegments_Userdata();
};

MP7_CheckVideoSegments_Userdata::MP7_CheckVideoSegments_Userdata() {
	error = 0;
	inside_varset = false;
	inside_source = false;
	inside_tempdecomp = false;
	inside_segment = false;
	segment_counter = 0;
	varset_counter = 0;
	which_varset = 1;
}

class MP7_GetVideoSegment_Userdata {
public:
	int error;
	bool inside_varset;
	bool inside_source;
	bool inside_tempdecomp;
	bool inside_segment;
	bool inside_creationinformation;
	bool inside_mediatime;
	bool inside_textannotation;
	bool inside_semantic;
	bool inside_titlemedia;
	bool inside_semantic_place;
	bool inside_semantic_place_place;
	bool inside_semantic_time;
	bool get_title;
	bool get_titleimage;
	bool get_media_timepoint;
	bool get_media_duration;
	bool get_freetextannotation;
	bool get_keyword;
	bool get_name;
	bool get_region;
	bool get_time;
	int segment_counter;
	int varset_counter;
	int which_varset;
	int which_segment;
	int segment_depth;
	MP7Parser::MP7_VideoSegmentData vseg_data;
	// constructor
	MP7_GetVideoSegment_Userdata();
};


MP7_GetVideoSegment_Userdata::MP7_GetVideoSegment_Userdata() 
{
	error = 0;
	inside_varset = false;
	inside_source = false;
	inside_tempdecomp = false;
	inside_segment = false;
	inside_creationinformation = false;
	inside_mediatime = false;
	inside_textannotation = false;
	inside_semantic = false;
	inside_titlemedia = false;
	inside_semantic_place = false;
	inside_semantic_place_place = false;
	inside_semantic_time = false;
	get_title = false;
	get_titleimage = false;
	get_media_timepoint = false;
	get_media_duration = false;
	get_freetextannotation = false;
	get_keyword = false;
	get_name = false;
	get_region = false;
	get_time = false;
	segment_counter = 0;
	varset_counter = 0;
	which_varset = 1;
	which_segment = 1;
	segment_depth = 0;
}

static void XMLHandler_CheckVideoSegments_start(void *data, const char *el, const char **attr)
{
	MP7_CheckVideoSegments_Userdata *usr_data_ptr = (MP7_CheckVideoSegments_Userdata *)data;

	switch (depth) {
	// VariationSet
	case VARDESCR_STARTLEVEL+1:
		// if the right VariationSet has not been already found
		if (!(*usr_data_ptr).inside_varset) {
			// check if it's a VariationSet
			if (strcmp("VariationSet", el) == 0) {
				// increase the VariationSet counter and check if it's the VariationSet we are looking for
				(*usr_data_ptr).varset_counter++;
				if ((*usr_data_ptr).varset_counter == (*usr_data_ptr).which_varset) {
					(*usr_data_ptr).inside_varset = true;
				}
			}
		}
		break;
		// Source + Variations
	case VARDESCR_STARTLEVEL+2:
		// if we are in the right VariationSet ..
		if ((*usr_data_ptr).inside_varset) {
			// and in it's source definition, we set the boolean
			if (strcmp("Source", el) == 0) {
				(*usr_data_ptr).inside_source = true;
			}
		}
		break;
		// check where we are inside of Source/Video definition
	case VARDESCR_STARTLEVEL+4:		
		// only if in the right VariationSet and in it's Source definition
		if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source) {
			if (strcmp("TemporalDecomposition", el) == 0) {
				(*usr_data_ptr).inside_tempdecomp = true;
			}
		}
		break;
	default:
		// only if in the right VariationSet and in it's Source definition and inside TemporalDecomposition
		if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source && (*usr_data_ptr).inside_tempdecomp) {
			if (strcmp("VideoSegment", el) == 0) {
				(*usr_data_ptr).inside_segment = true;
				(*usr_data_ptr).segment_counter++;
			}
		}
	}

	depth++;
}

static void XMLHandler_CheckVideoSegments_end(void *data, const char *el)
{
	MP7_CheckVideoSegments_Userdata *usr_data_ptr = (MP7_CheckVideoSegments_Userdata *)data;

	if (depth == VARDESCR_STARTLEVEL+2) {
		if (strcmp("VariationSet", el) == 0) {
			(*usr_data_ptr).inside_varset = false;
			(*usr_data_ptr).inside_source = false;
		}
	}

	depth--;
}

static void XMLHandler_GetVideoSegment_data(void *userData, const XML_Char *s, int len)
{
	MP7_GetVideoSegment_Userdata *usr_data_ptr = (MP7_GetVideoSegment_Userdata *)userData;

	// CreationInformation - Title 
	if ((*usr_data_ptr).get_title) {
		(*usr_data_ptr).vseg_data.title.append(s,len);
		(*usr_data_ptr).get_title = false;
	}

	// CreationInformation - Title Image
	else if ((*usr_data_ptr).get_titleimage) {
		(*usr_data_ptr).vseg_data.title_image.append(s,len);
		(*usr_data_ptr).get_titleimage = false;
	}

	// MediaTime - MediaTimePoint
	else if ((*usr_data_ptr).get_media_timepoint) {
		(*usr_data_ptr).vseg_data.mediatime.mediatimepoint_alpha.append(s,len);
		(*usr_data_ptr).get_media_timepoint = false;
	}
		
	// MediaTime - MediaDuration
	else if ((*usr_data_ptr).get_media_duration) {
		(*usr_data_ptr).vseg_data.mediatime.mediaduration_alpha.append(s,len);
		(*usr_data_ptr).get_media_duration = false;
	}
	// TextAnnotation - FreeTextAnnotation
	else if ((*usr_data_ptr).get_freetextannotation) {
		if ((*usr_data_ptr).vseg_data.free_text_annotations.empty()) {
			(*usr_data_ptr).vseg_data.free_text_annotations.append(s,len);
		}
		else {
			(*usr_data_ptr).vseg_data.free_text_annotations.append(", ");
			(*usr_data_ptr).vseg_data.free_text_annotations.append(s,len);
		}
		(*usr_data_ptr).get_freetextannotation = false;
	}
	// TextAnnotation - Keyword(s)
	else if ((*usr_data_ptr).get_keyword) {
		if ((*usr_data_ptr).vseg_data.keywords.empty()) {
			(*usr_data_ptr).vseg_data.keywords.append(s,len);
		}
		else {
			(*usr_data_ptr).vseg_data.keywords.append(", ");
			(*usr_data_ptr).vseg_data.keywords.append(s,len);
		}
		(*usr_data_ptr).get_keyword = false;
	}
	// Semantic - SemanticPlace - Place - Name
	else if ((*usr_data_ptr).get_name) {
		(*usr_data_ptr).vseg_data.semantic_place.append(s,len);
		(*usr_data_ptr).get_name = false;
	}
	// Semantic - SemanticPlace - Place - Region
	else if ((*usr_data_ptr).get_region) {
		(*usr_data_ptr).vseg_data.semantic_region.append(s,len);
		(*usr_data_ptr).get_region = false;
	}
	// Semantic - SemanticTime - TimePoint
	else if ((*usr_data_ptr).get_time) {
		(*usr_data_ptr).vseg_data.semantic_time.append(s,len);
		(*usr_data_ptr).get_time = false;
	}
}


static void XMLHandler_GetVideoSegment_start(void *data, const char *el, const char **attr)
{
	MP7_GetVideoSegment_Userdata *usr_data_ptr = (MP7_GetVideoSegment_Userdata *)data;

	if (!(*usr_data_ptr).inside_segment) {
		switch (depth) {
		// VariationSet
		case VARDESCR_STARTLEVEL+1:
			// if the right VariationSet has not been already found
			if (!(*usr_data_ptr).inside_varset) {
				// check if it's a VariationSet
				if (strcmp("VariationSet", el) == 0) {
					// increase the VariationSet counter and check if it's the VariationSet we are looking for
					(*usr_data_ptr).varset_counter++;
					if ((*usr_data_ptr).varset_counter == (*usr_data_ptr).which_varset) {
						(*usr_data_ptr).inside_varset = true;
					}
				}
			}
			break;
			// Source + Variations
		case VARDESCR_STARTLEVEL+2:
			// if we are in the right VariationSet ..
			if ((*usr_data_ptr).inside_varset) {
				// and in it's source definition, we set the boolean
				if (strcmp("Source", el) == 0) {
					(*usr_data_ptr).inside_source = true;
				}
			}
			break;
			// check where we are inside of Source/Video definition
		case VARDESCR_STARTLEVEL+4:     
			// only if in the right VariationSet and in it's Source definition
			if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source) {
				if (strcmp("TemporalDecomposition", el) == 0) {
					(*usr_data_ptr).inside_tempdecomp = true;
				}
			}
			break;
		default:
			// only if in the right VariationSet and in it's Source definition and inside TemporalDecomposition
			if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source && (*usr_data_ptr).inside_tempdecomp) {
				if (strcmp("VideoSegment", el) == 0) {
		  			(*usr_data_ptr).segment_counter++;
					if ((*usr_data_ptr).segment_counter == (*usr_data_ptr).which_segment) {
						(*usr_data_ptr).inside_segment = true;
						(*usr_data_ptr).segment_depth = depth;
					}
				}
			}
		}
	} 
	else {
		// CreationInformation, MediaTime, TextAnnotation, Semantic
		if (depth == (*usr_data_ptr).segment_depth+1) {
			if (strcmp("CreationInformation", el) == 0) {
				(*usr_data_ptr).inside_creationinformation = true;
			} else if (strcmp("MediaTime", el) == 0) {
				(*usr_data_ptr).inside_mediatime = true;
			} else if (strcmp("TextAnnotation", el) == 0) {
				(*usr_data_ptr).inside_textannotation = true;
			} else if (strcmp("Semantic", el) == 0) {
				(*usr_data_ptr).inside_semantic = true;
			}
		}
		// MediaTime - TimePoint, Duration
		else if ((*usr_data_ptr).inside_mediatime && (depth == (*usr_data_ptr).segment_depth+2)) {
			if (strcmp("MediaTimePoint", el) == 0) {
				(*usr_data_ptr).get_media_timepoint = true;
			}
			else if (strcmp("MediaDuration", el) == 0) {
				(*usr_data_ptr).get_media_duration = true;
			}
		}
		// TextAnnotation or Abstract - FreeTextAnnotation
		else if ((*usr_data_ptr).inside_textannotation && 
				((depth == (*usr_data_ptr).segment_depth+2) || (depth == (*usr_data_ptr).segment_depth+4)))  {
			if (strcmp("FreeTextAnnotation", el) == 0) {
				(*usr_data_ptr).get_freetextannotation = true;
			}
		}
		// Semantic - SemanticPlace
		else if ((*usr_data_ptr).inside_semantic && (depth == (*usr_data_ptr).segment_depth+2)) {
			if ((strcmp("SemanticBase", el) == 0) && (strcmp("SemanticPlaceType", attr[1]) == 0)) {
				(*usr_data_ptr).inside_semantic_place = true;
			}
			if ((strcmp("SemanticBase", el) == 0) && (strcmp("SemanticTimeType", attr[1]) == 0)) {
				(*usr_data_ptr).inside_semantic_time = true;
			}
		}
		// Semantic - SemanticPlace - Place
		else if ((*usr_data_ptr).inside_semantic_place && (depth == (*usr_data_ptr).segment_depth+3)) {
			if (strcmp("Place", el) == 0) {
				(*usr_data_ptr).inside_semantic_place_place = true;
			}
		}
		// TextAnnotation or Abstract - Keyword
		else if ((*usr_data_ptr).inside_textannotation && 
				((depth == (*usr_data_ptr).segment_depth+3) || (depth == (*usr_data_ptr).segment_depth+5))){
			if (strcmp("Keyword", el) == 0) {
				(*usr_data_ptr).get_keyword = true;
			}
		}
		// CreationInformation - Title, Titlemedia, Abstract
		else if ((*usr_data_ptr).inside_creationinformation && (depth == (*usr_data_ptr).segment_depth+3)) {
			if (strcmp("Title", el) == 0) {
				(*usr_data_ptr).get_title = true;
			}
			else if (strcmp("TitleMedia", el) == 0) {
				(*usr_data_ptr).inside_titlemedia = true;
			}
			// FreeTextAnnotations and/or Keywords can be after a TextAnnotation or in an Abstract
			// So we say inside_textannotation, though we are technically inside the Abstract
			else if (strcmp("Abstract", el) == 0) {
				(*usr_data_ptr).inside_textannotation = true;
			}
		}
		// Semantic - SemanticPlace - Place - Name, Region
		else if ((*usr_data_ptr).inside_semantic_place_place && (depth == (*usr_data_ptr).segment_depth+4)) {
			if (strcmp("Name", el) == 0) {
				(*usr_data_ptr).get_name = true;
			}
			else if (strcmp("Region", el) == 0) {
				(*usr_data_ptr).get_region = true;
			}
		}
		else if ((*usr_data_ptr).inside_semantic_time && (depth == (*usr_data_ptr).segment_depth+4)) {
			if (strcmp("TimePoint", el) == 0) {
				(*usr_data_ptr).get_time = true;
			}
		}
		// CreationInformation - TitleMedia
		else if ((*usr_data_ptr).inside_titlemedia && (depth == (*usr_data_ptr).segment_depth+5)) {
			if (strcmp("MediaTimePoint", el) == 0) {
				(*usr_data_ptr).get_titleimage = true;
			}
		}
	}

	depth++;
}

static void XMLHandler_GetVideoSegment_end(void *data, const char *el)
{
	MP7_GetVideoSegment_Userdata *usr_data_ptr = (MP7_GetVideoSegment_Userdata *)data;
	

	if ((*usr_data_ptr).inside_segment) {
		if (strcmp("VideoSegment", el) == 0) {
			(*usr_data_ptr).inside_segment = false;
		}
		else if (strcmp("CreationInformation", el) == 0) {
			(*usr_data_ptr).inside_creationinformation = false;
		}
		else if (strcmp("MediaTime", el) == 0) {
			(*usr_data_ptr).inside_mediatime = false;
		}
		else if ((strcmp("TextAnnotation", el) == 0) || (strcmp("Abstract", el) == 0)) {
			(*usr_data_ptr).inside_textannotation = false;
		}
		else if (strcmp("Semantic", el) == 0) {
			(*usr_data_ptr).inside_semantic = false;
		}
		else if (strcmp("TitleMedia", el) == 0) {
			(*usr_data_ptr).inside_titlemedia = false;
		}
		else if (strcmp("SemanticBase", el) == 0) {
			(*usr_data_ptr).inside_semantic_place = false;
			(*usr_data_ptr).inside_semantic_time = false;
		}
		else if (strcmp("Place", el) == 0) {
			(*usr_data_ptr).inside_semantic_place_place = false;
		}
	}


	if (depth == VARDESCR_STARTLEVEL+2) {
		if (strcmp("VariationSet", el) == 0) {
			(*usr_data_ptr).inside_varset = false;
			(*usr_data_ptr).inside_source = false;
		}
	}
	else if (depth == VARDESCR_STARTLEVEL+5) {
		if (strcmp("TemporalDecomposition", el) == 0) {
			(*usr_data_ptr).inside_tempdecomp = false;
		}
	}


	depth--;
}

// data handler for MP7GetVarSet
static void XMLHandler_mp7getvarset_data(void *userData, const XML_Char *s, int len)
{
	MP7_VarSet_Userdata *usr_data_ptr = (MP7_VarSet_Userdata *)userData;
	int rv;

	// get source - visualcodec name
	if ((*usr_data_ptr).get_visualcoding_name) {
		(*usr_data_ptr).source_data.video_codec.append(s,len);
		(*usr_data_ptr).get_visualcoding_name = false;
	}

	// get source - audiocodec name
	else if ((*usr_data_ptr).get_audiocoding_name) {
		(*usr_data_ptr).source_data.audio_codec.append(s,len);
		(*usr_data_ptr).get_audiocoding_name = false;
	}

	// get source - fileformat name
	else if ((*usr_data_ptr).get_fileformat_name) {
		(*usr_data_ptr).source_data.fileformat.append(s,len);
		(*usr_data_ptr).get_fileformat_name = false;
	}

	// get source - filesize
	else if ((*usr_data_ptr).get_filesize) {
		rv = sscanf(s, "%ld", &((*usr_data_ptr).source_data.filesize));
		if (rv != 1) {
			(*usr_data_ptr).error = ERR__INVALID_DATA;
		}
		(*usr_data_ptr).get_filesize = false;
	}

	// get source - bitrate
	else if ((*usr_data_ptr).get_bitrate) {
		rv = sscanf(s, "%ld", &((*usr_data_ptr).source_data.bitrate));
		if (rv != 1) {
			(*usr_data_ptr).error = ERR__INVALID_DATA;
		}
		(*usr_data_ptr).get_bitrate = false;
	}

	// get source - media uri
	else if ((*usr_data_ptr).get_mediauri) {
		(*usr_data_ptr).source_data.path.append(s,len);
		(*usr_data_ptr).get_mediauri = false;
	}

	// get source - creation - title 
	else if ((*usr_data_ptr).get_title) {
		(*usr_data_ptr).meta_data.title.append(s,len);
		(*usr_data_ptr).get_title = false;
	}

	// get source - creation - title image
	else if ((*usr_data_ptr).get_titleimage) {
		(*usr_data_ptr).meta_data.title_image.append(s,len);
		(*usr_data_ptr).get_titleimage = false;
	}

	// get source - creation - free text annotations
	else if ((*usr_data_ptr).get_freetextannotation) {
		if ((*usr_data_ptr).meta_data.free_text_annotations.empty()) {
			(*usr_data_ptr).meta_data.free_text_annotations.append(s,len);
		}
		else {
			(*usr_data_ptr).meta_data.free_text_annotations.append(" - ");
			(*usr_data_ptr).meta_data.free_text_annotations.append(s,len);
		}
		(*usr_data_ptr).get_freetextannotation = false;
	}

	// get source - creation - keywords
	else if ((*usr_data_ptr).get_keyword) {
		if ((*usr_data_ptr).meta_data.keywords.empty()) {
			(*usr_data_ptr).meta_data.keywords.append(s,len);
		}
		else {
			(*usr_data_ptr).meta_data.keywords.append(", ");
			(*usr_data_ptr).meta_data.keywords.append(s,len);
		}
		(*usr_data_ptr).get_keyword = false;
	}

	// get source - creation - creator name
	else if ((*usr_data_ptr).get_name) {
		if ((*usr_data_ptr).meta_data.creator_name.empty()) {
			(*usr_data_ptr).meta_data.creator_name.append(s,len);
		}
		else {
			(*usr_data_ptr).meta_data.creator_name.append(" ");
			(*usr_data_ptr).meta_data.creator_name.append(s,len);
		}
		(*usr_data_ptr).get_name = false;
	}

	// get source - creation - location name
	else if ((*usr_data_ptr).get_lo_name) {
		(*usr_data_ptr).meta_data.location_name.append(s,len);
		(*usr_data_ptr).get_lo_name = false;
	}

	// get source - creation - location region
	else if ((*usr_data_ptr).get_lo_region) {
		(*usr_data_ptr).meta_data.location_region.append(s,len);
		(*usr_data_ptr).get_lo_region = false;
	}

	// get source - creation - location timepoint
	else if ((*usr_data_ptr).get_lo_date) {
		(*usr_data_ptr).meta_data.date.append(s,len);
		(*usr_data_ptr).get_lo_date = false;
	}

	// get source - creation - copyright
	else if ((*usr_data_ptr).get_copyright) {
		(*usr_data_ptr).meta_data.copyright.append(s,len);
		(*usr_data_ptr).get_copyright = false;
	}

	// get source - mediatime - mediatimepoint
	else if ((*usr_data_ptr).get_media_timepoint) {
		(*usr_data_ptr).media_time.mediatimepoint_alpha.append(s,len);
		(*usr_data_ptr).get_media_timepoint = false;
	}
		
	// get source - mediatime - mediaduration
	else if ((*usr_data_ptr).get_media_duration) {
		(*usr_data_ptr).media_time.mediaduration_alpha.append(s,len);
		(*usr_data_ptr).get_media_duration = false;
	}
	else if ((*usr_data_ptr).get_semantic_name) {
		(*usr_data_ptr).meta_data.semantic_place.append(s,len);
		(*usr_data_ptr).get_semantic_name = false;
	}
	else if ((*usr_data_ptr).get_semantic_region) {
		(*usr_data_ptr).meta_data.semantic_region.append(s,len);
		(*usr_data_ptr).get_semantic_region = false;
	}
	else if ((*usr_data_ptr).get_semantic_time) {
		(*usr_data_ptr).meta_data.semantic_time.append(s,len);
		(*usr_data_ptr).get_semantic_time = false;
	}

}

// element handler for MP7GetVarSet - start of element
static void XMLHandler_mp7getvarset_start(void *data, const char *el, const char **attr)
{
	MP7_VarSet_Userdata *usr_data_ptr = (MP7_VarSet_Userdata *)data;
	int rv;

	switch (depth) {
	// VariationSet
	case VARDESCR_STARTLEVEL+1:
		// if the right VariationSet has not been already found
		if (!(*usr_data_ptr).inside_varset) {
			// check if it's a VariationSet
			if (strcmp("VariationSet", el) == 0) {
				// increase the VariationSet counter and check if it's the VariationSet we are looking for
				(*usr_data_ptr).varset_counter++;
				if ((*usr_data_ptr).varset_counter == (*usr_data_ptr).which_varset) {
					(*usr_data_ptr).inside_varset = true;
				}
			}
		}
		break;
		// Source + Variations
	case VARDESCR_STARTLEVEL+2:
		// if we are in the right VariationSet ..
		if ((*usr_data_ptr).inside_varset) {
			// and in it's source definition, we set the boolean
			if (strcmp("Source", el) == 0) {
				(*usr_data_ptr).inside_source = true;
			}
			// count the Variations
			else if (strcmp("Variation", el) == 0) {
				(*usr_data_ptr).variation_counter++;
			}
		}
		break;
		// Source Definition
	case VARDESCR_STARTLEVEL+3:
		// only if in the right VariationSet and in it's Source definition
		if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source) {
			// video id
			if (strcmp("Video", el) == 0) {
				(*usr_data_ptr).source_data.id = attr[1];
			}
		}
		break;
		// check where we are inside of Source/Video definition
	case VARDESCR_STARTLEVEL+4:	
		// only if in the right VariationSet and in it's Source definition
		if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source) {
			if (strcmp("MediaInformation", el) == 0) {
				(*usr_data_ptr).inside_mediainformation = true;
			}
			else if (strcmp("CreationInformation", el) == 0) {
				(*usr_data_ptr).inside_creationinformation = true;
			}
			else if (strcmp("MediaTime", el) == 0) {
				(*usr_data_ptr).inside_mediatime = true;
			}
			else if (strcmp("TextAnnotation", el) == 0) {
				(*usr_data_ptr).inside_textannotation = true;
			}
			else if (strcmp("Semantic", el) == 0) {
				(*usr_data_ptr).inside_semantic = true;
			}
		}
		break;
	case VARDESCR_STARTLEVEL+5:
		if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source && (*usr_data_ptr).inside_mediatime) {
			if (strcmp("MediaTimePoint", el) == 0) {
				(*usr_data_ptr).get_media_timepoint = true;
			}
			else if (strcmp("MediaDuration", el) == 0) {
				(*usr_data_ptr).get_media_duration = true;
			}
		}
		else if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source && (*usr_data_ptr).inside_textannotation) {
            if (strcmp("FreeTextAnnotation", el) == 0) {
				(*usr_data_ptr).get_freetextannotation = true;
			}
		}
		else if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source && (*usr_data_ptr).inside_semantic) {
			if ((strcmp("SemanticBase", el) == 0) && (strcmp("SemanticPlaceType", attr[1]) == 0)) {
				(*usr_data_ptr).inside_semantic_place = true;
			}
			if ((strcmp("SemanticBase", el) == 0) && (strcmp("SemanticTimeType", attr[1]) == 0)) {
				(*usr_data_ptr).inside_semantic_time = true;			
			}		
		}

		break;
		// CreationInformation - Title
	case VARDESCR_STARTLEVEL+6:
		// only if in the right VariationSet and in it's Source definition - CreationInformation
		if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source && (*usr_data_ptr).inside_creationinformation) {
			if (strcmp("Title", el) == 0) {
				(*usr_data_ptr).get_title = true;
			}
			else if (strcmp("CopyrightString", el) == 0) {
				(*usr_data_ptr).get_copyright = true;
			}
			else if (strcmp("TitleMedia", el) == 0) {
				(*usr_data_ptr).inside_titlemedia = true;
			}
			else if (strcmp("Abstract", el) == 0) {
				(*usr_data_ptr).inside_textannotation = true;
			}
			else if (strcmp("Creator", el) == 0) {
				(*usr_data_ptr).inside_creator = true;
			}
			else if (strcmp("CreationCoordinates", el) == 0) {
				(*usr_data_ptr).inside_coordinates = true;
			}
		}
		else if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source && (*usr_data_ptr).inside_textannotation) {
			if (strcmp("Keyword", el) == 0) {
				(*usr_data_ptr).get_keyword = true;
			}
		}
		else if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source && (*usr_data_ptr).inside_semantic_place) {
			if (strcmp("Place", el) == 0) {
				(*usr_data_ptr).inside_semantic_place_place = true;			
			}	
		}
		break;
		// Source - FileSize and BitRate
	case VARDESCR_STARTLEVEL+7:
		// only if in the right VariationSet and in it's Source definition - MediaInformation
		if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source && (*usr_data_ptr).inside_mediainformation) {
			if (strcmp("FileSize", el) == 0) {
				(*usr_data_ptr).get_filesize = true;
			}
			else if (strcmp("BitRate", el) == 0) {
				(*usr_data_ptr).get_bitrate = true;
			}
			else if (strcmp("FileFormat", el) == 0) {
				(*usr_data_ptr).inside_fileformat = true;
			}
		}
		else if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source && (*usr_data_ptr).inside_creationinformation) {
            if ((*usr_data_ptr).inside_textannotation && (strcmp("FreeTextAnnotation", el) == 0)) {
				(*usr_data_ptr).get_freetextannotation = true;
			}
		}
		else if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source && (*usr_data_ptr).inside_semantic_place_place) {
			if (strcmp("Name", el) == 0) {
				(*usr_data_ptr).get_semantic_name = true;
			}
			else if (strcmp("Region", el) == 0) {
				(*usr_data_ptr).get_semantic_region = true;			
			}		
		}
		else if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source && (*usr_data_ptr).inside_semantic_time) {
			if (strcmp("TimePoint", el) == 0) {
				(*usr_data_ptr).get_semantic_time = true;
			}
		}
		break;
		// Source - Visual Coding and MediaUri
	case VARDESCR_STARTLEVEL+8:
		// only if in the right VariationSet and in it's Source definition
		if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source && (*usr_data_ptr).inside_mediainformation) {
			// check for VisualCoding
			if (strcmp("VisualCoding", el) == 0) {
				(*usr_data_ptr).inside_visualcoding = true;
			}
			else if (strcmp("AudioCoding", el) == 0) {
				(*usr_data_ptr).inside_audiocoding = true;
				(*usr_data_ptr).source_data.has_audio = true;
			}
			else if (strcmp("MediaUri", el) == 0) {
				(*usr_data_ptr).get_mediauri = true;        
			}
			else if (((*usr_data_ptr).inside_fileformat) && ((strcmp("Name", el) == 0))) {
				(*usr_data_ptr).get_fileformat_name = true;
			}
		}
		// TitleImage - MediaTimePoint
		if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source && (*usr_data_ptr).inside_creationinformation) {
			if ((*usr_data_ptr).inside_titlemedia && (strcmp("MediaTimePoint", el) == 0)) {
				
			}
			else if ((*usr_data_ptr).inside_textannotation && (strcmp("Keyword", el) == 0)) {
				(*usr_data_ptr).get_keyword = true;
			}
			else if ((*usr_data_ptr).inside_coordinates && (strcmp("Name", el) == 0)) {
				(*usr_data_ptr).get_lo_name = true;
			}
			else if ((*usr_data_ptr).inside_coordinates && (strcmp("Region", el) == 0)) {
				(*usr_data_ptr).get_lo_region = true;
			}
			else if ((*usr_data_ptr).inside_coordinates && (strcmp("TimePoint", el) == 0)) {
				(*usr_data_ptr).get_lo_date = true;
			}
		}
		break;
		// Source - VisualCoding (name, width, height, rate) and Audiocoding (name)
	case VARDESCR_STARTLEVEL+9:
		// only if in the right VariationSet and in it's Source definition
		if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source && (*usr_data_ptr).inside_mediainformation) {
			// VisualCoding
			if ((*usr_data_ptr).inside_visualcoding) {
				// frame data
				if (strcmp("Frame", el) == 0) {
					// height
					rv = sscanf(attr[1], "%i", &((*usr_data_ptr).source_data.video_height));
					if (rv != 1) {
						(*usr_data_ptr).error = ERR__INVALID_DATA;
					}
					// width
					rv = sscanf(attr[3], "%i", &((*usr_data_ptr).source_data.video_width));
					if (rv != 1) {
						(*usr_data_ptr).error = ERR__INVALID_DATA;
					}
					// rate
					rv = sscanf(attr[5], "%f", &((*usr_data_ptr).source_data.video_rate));
					if (rv != 1) {
						(*usr_data_ptr).error = ERR__INVALID_DATA;
					}
				}
			}
		}
		if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source && (*usr_data_ptr).inside_creationinformation) {
			if ((*usr_data_ptr).inside_creator && ((strcmp("GivenName", el) == 0) || (strcmp("FamilyName", el) == 0))) {
				(*usr_data_ptr).get_name = true;
			}
		}
		break;
	case VARDESCR_STARTLEVEL+10:
		if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_source && (*usr_data_ptr).inside_mediainformation) {
			// VisualCoding
			if ((*usr_data_ptr).inside_visualcoding) {
				// codec name
				if (strcmp("Name", el) == 0) {
					(*usr_data_ptr).get_visualcoding_name = true;
				}
			}
			// AudioCoding
			else if ((*usr_data_ptr).inside_audiocoding) {
				// codec name
				if (strcmp("Name", el) == 0) {
					(*usr_data_ptr).get_audiocoding_name = true;
				}
			}
		}
		break;
	}

	depth++;
}

// element handler for MP7GetVarSet - end of element
static void XMLHandler_mp7getvarset_end(void *data, const char *el)
{
	MP7_VarSet_Userdata *usr_data_ptr = (MP7_VarSet_Userdata *)data;


	switch (depth) {
	// VariationSet
	case VARDESCR_STARTLEVEL+2:
		// before leaving a VariationSet, reset the userdata
		if (strcmp("VariationSet", el) == 0) {
			(*usr_data_ptr).inside_varset = false;
			(*usr_data_ptr).inside_source = false;
		}
		break;
	case VARDESCR_STARTLEVEL+5:
		if (strcmp("CreationInformation", el) == 0) {
			(*usr_data_ptr).inside_creationinformation = false;
		}
		else if (strcmp("MediaInformation", el) == 0) {
			(*usr_data_ptr).inside_mediainformation = false;
		}
		else if (strcmp("MediaTime", el) == 0) {
			(*usr_data_ptr).inside_mediatime = false;
		}
		else if (strcmp("TextAnnotation", el) == 0) {
			(*usr_data_ptr).inside_textannotation = false;
		}
		else if (strcmp("Semantic", el) == 0) {
			(*usr_data_ptr).inside_semantic = false;
		}
		break;
	case VARDESCR_STARTLEVEL+6:
		if (strcmp("SemanticBase", el) == 0) {
			(*usr_data_ptr).inside_semantic_place = false;
			(*usr_data_ptr).inside_semantic_time = false;
		}
		break;
	case VARDESCR_STARTLEVEL+7:
		if (strcmp("TitleMedia", el) == 0) {
			(*usr_data_ptr).inside_titlemedia = false;
		}
		else if (strcmp("Abstract", el) == 0) {
			(*usr_data_ptr).inside_textannotation = false;
		}
		else if (strcmp("Creator", el) == 0) {
			(*usr_data_ptr).inside_creator = false;
		}
		else if (strcmp("CreationCoordinates", el) == 0) {
			(*usr_data_ptr).inside_coordinates = false;
		}
		else if (strcmp("Place", el) == 0) {
			(*usr_data_ptr).inside_semantic_place_place = false;
		}
		break;
	case VARDESCR_STARTLEVEL+9:
		// before leaving VisualCoding, reset the userdata
		if (strcmp("VisualCoding", el) == 0) {
			(*usr_data_ptr).inside_visualcoding = false;
		}
		// before leaving AudioCoding, reset the userdata
		else if (strcmp("AudioCoding", el) == 0) {
			(*usr_data_ptr).inside_audiocoding = false;
		}
		break;
	}

	depth--;
}


// data handler for MP7GetVariation
static void XMLHandler_mp7getvariation_data(void *userData, const XML_Char *s, int len)
{
	MP7_Variation_Userdata *usr_data_ptr = (MP7_Variation_Userdata *)userData;
	int rv;

	// get variation relationship
	if ((*usr_data_ptr).get_varrelationship) {
		(*usr_data_ptr).var_data.variation_relationship.append(s,len);
		(*usr_data_ptr).get_varrelationship = false;
	}
	// get visualcodec name
	else if ((*usr_data_ptr).get_visualcoding_name) {
		(*usr_data_ptr).var_data.video_codec.append(s,len);
		(*usr_data_ptr).get_visualcoding_name = false;
	}
	// get fileformat name
	else if ((*usr_data_ptr).get_fileformat_name) {
		(*usr_data_ptr).var_data.fileformat.append(s,len);
		(*usr_data_ptr).get_fileformat_name = false;
	}
	// get filesize
	else if ((*usr_data_ptr).get_filesize) {
		rv = sscanf(s, "%ld", &((*usr_data_ptr).var_data.filesize));
		if (rv != 1) {
			(*usr_data_ptr).error = ERR__INVALID_DATA;
		}
		(*usr_data_ptr).get_filesize = false;
	}
	// get bitrate
	else if ((*usr_data_ptr).get_bitrate) {
		rv = sscanf(s, "%ld", &((*usr_data_ptr).var_data.bitrate));
		if (rv != 1) {
			(*usr_data_ptr).error = ERR__INVALID_DATA;
		}
		(*usr_data_ptr).get_bitrate = false;
	}
	// get source - audiocodec name
	else if ((*usr_data_ptr).get_audiocoding_name) {
		(*usr_data_ptr).var_data.audio_codec.append(s,len);
		(*usr_data_ptr).get_audiocoding_name = false;
	}
	// get source - media uri
	else if ((*usr_data_ptr).get_mediauri) {
		(*usr_data_ptr).var_data.path.append(s,len);
		(*usr_data_ptr).get_mediauri = false;
	}

}

// element handler for MP7GetVariation - start of element
static void XMLHandler_mp7getvariation_start(void *data, const char *el, const char **attr)
{
	MP7_Variation_Userdata *usr_data_ptr = (MP7_Variation_Userdata *)data;
	int rv;

	switch (depth) {
	// VariationSet
	case VARDESCR_STARTLEVEL+1:
		// if the right VariationSet has not been already found
		if (!(*usr_data_ptr).inside_varset) {
			// check if it's a VariationSet
			if (strcmp("VariationSet", el) == 0) {
				// increase the VariationSet counter and check if it's the VariationSet we are looking for
				(*usr_data_ptr).varset_counter++;
				if ((*usr_data_ptr).varset_counter == (*usr_data_ptr).which_varset) {
					(*usr_data_ptr).inside_varset = true;
				}
			}
		}
		break;
		// Variations
	case VARDESCR_STARTLEVEL+2:
		// if we are in the right VariationSet ..
		if ((*usr_data_ptr).inside_varset) {
			// we count to the right Variation
			if (strcmp("Variation", el) == 0) {
				(*usr_data_ptr).variation_counter++;
				// get id, fidelity and priority
				if ((*usr_data_ptr).variation_counter == (*usr_data_ptr).which_variation) {
					(*usr_data_ptr).inside_variation = true;
					// variation_id
					(*usr_data_ptr).var_data.variation_id = attr[1];
					// fidelity
					rv = sscanf(attr[3], "%f", &((*usr_data_ptr).var_data.fidelity));
					if (rv != 1) {
						(*usr_data_ptr).error = ERR__INVALID_DATA;
					}
					// priority
					rv = sscanf(attr[5], "%i", &((*usr_data_ptr).var_data.priority));
					if (rv != 1) {
						(*usr_data_ptr).error = ERR__INVALID_DATA;
					}
				}
			}
		}
		break;
		// VariationRelationship
	case VARDESCR_STARTLEVEL+3:
		// if we are in the right VariationSet and the right Variation ..
		if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_variation) {
			// VariationRelationship
			if (strcmp("VariationRelationship", el) == 0) {
				(*usr_data_ptr).get_varrelationship = true;
			}
		}
		break;
		// video id
	case VARDESCR_STARTLEVEL+4:
		// if we are in the right VariationSet and the right Variation ..
		if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_variation) {
			// id
			if (strcmp("Video", el) == 0) {
				(*usr_data_ptr).var_data.id = attr[1];
			}
		}
		break;
		// Filsize, Bitrate, Fileformat
	case VARDESCR_STARTLEVEL+8:
		// if we are in the right VariationSet and the right Variation ..
		if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_variation) {
			if (strcmp("FileSize", el) == 0) {
				(*usr_data_ptr).get_filesize = true;
			}
			else if (strcmp("BitRate", el) == 0) {
				(*usr_data_ptr).get_bitrate = true;
			}
			else if (strcmp("FileFormat", el) == 0) {
				(*usr_data_ptr).inside_fileformat = true;
			}
		}
		break;
		// VisualCoding, AudioCoding
	case VARDESCR_STARTLEVEL+9:
		// if we are in the right VariationSet and the right Variation ..
		if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_variation) {
			// VisualCoding
			if (strcmp("VisualCoding", el) == 0) {
				(*usr_data_ptr).inside_visualcoding = true;
			}
			// AudioCoding
			else if (strcmp("AudioCoding", el) == 0) {
				(*usr_data_ptr).inside_audiocoding = true;
				(*usr_data_ptr).var_data.has_audio = true;
			}
			else if (strcmp("MediaUri", el) == 0) {
				(*usr_data_ptr).get_mediauri = true;
			}
			else if (((*usr_data_ptr).inside_fileformat) && ((strcmp("Name", el) == 0))) {
				(*usr_data_ptr).get_fileformat_name = true;
			}
		}
		break;
	case VARDESCR_STARTLEVEL+10:
		// if we are in the right VariationSet and the right Variation ..
		if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_variation) {
			// and inside VisualCoding
			if ((*usr_data_ptr).inside_visualcoding) {
				// frame data
				if (strcmp("Frame", el) == 0) {
					// height
					rv = sscanf(attr[1], "%i", &((*usr_data_ptr).var_data.video_height));
					if (rv != 1) {
						(*usr_data_ptr).error = ERR__INVALID_DATA;
					}
					// width
					rv = sscanf(attr[3], "%i", &((*usr_data_ptr).var_data.video_width));
					if (rv != 1) {
						(*usr_data_ptr).error = ERR__INVALID_DATA;
					}
					// rate
					rv = sscanf(attr[5], "%f", &((*usr_data_ptr).var_data.video_rate));
					if (rv != 1) {
						(*usr_data_ptr).error = ERR__INVALID_DATA;
					}
				}
			}
		}
		break;
	case VARDESCR_STARTLEVEL+11:
		// if we are in the right VariationSet and the right Variation ..
		if ((*usr_data_ptr).inside_varset && (*usr_data_ptr).inside_variation) {
			// and inside VisualCoding
			if ((*usr_data_ptr).inside_visualcoding) {
				// codec name
				if (strcmp("Name", el) == 0) {
					(*usr_data_ptr).get_visualcoding_name = true;
				}
			}
			// AudioCoding
			else if ((*usr_data_ptr).inside_audiocoding) {
				// codec name
				if (strcmp("Name", el) == 0) {
					(*usr_data_ptr).get_audiocoding_name = true;
				}
			
			}
		}	
	}

	depth++;
}

// element handler for MP7GetVariation - end of element
static void XMLHandler_mp7getvariation_end(void *data, const char *el)
{
	MP7_Variation_Userdata *usr_data_ptr = (MP7_Variation_Userdata *)data;

	// before leaving a VariationSet, reset the userdata
	if (strcmp("VariationSet", el) == 0) {
		(*usr_data_ptr).inside_varset = false;
	}
	// before leaving a Variation, reset the userdata
	else if (strcmp("Variation", el) == 0) {
		(*usr_data_ptr).inside_variation = false;
	}
	// before leaving VisualCoding, reset the userdata
	else if (strcmp("VisualCoding", el) == 0) {
		(*usr_data_ptr).inside_visualcoding = false;
	}
	// before leaving AudioCoding, reset the userdata
	else if (strcmp("AudioCoding", el) == 0) {
		(*usr_data_ptr).inside_audiocoding = false;
	}

	depth--;
}

// element handler for MP7GetVarDescrType - start of element
static void XMLHandler_mp7getvardescrtype_start(void *data, const char *el, const char **attr)
{
	MP7_VarDescrType_Userdata *usr_data_ptr = (MP7_VarDescrType_Userdata *)data;

	switch (depth) {
	// check if in VariationDescriptionType
	case VARDESCR_STARTLEVEL:
		if (strcmp("Description", el) == 0) {
			if (strcmp("VariationDescriptionType", attr[1]) == 0) {
				(*usr_data_ptr).inside_vardescrtype = true;
				(*usr_data_ptr).vardescr = true;
			}
		}
		break;
		// check for VariationSets and count them
	case VARDESCR_STARTLEVEL+1:
		if ((*usr_data_ptr).inside_vardescrtype) {
			if (strcmp("VariationSet", el) == 0) {
				(*usr_data_ptr).varset_counter++;
			}
			else {
				(*usr_data_ptr).error = ERR__NO_VARIATIONSET_DEF;
			}
		}
		break;
		// check if Source and Variations are defined
	case VARDESCR_STARTLEVEL+2:
		if ((*usr_data_ptr).inside_vardescrtype) {
			if (strcmp("Source",el) == 0) {
				(*usr_data_ptr).source = true;
			}
			else if (strcmp("Variation", el) == 0) {
				(*usr_data_ptr).variation = true;
			}
		}
	}
	depth++;
}


// element handler for MP7GetVarDescrType - end of element
static void XMLHandler_mp7getvardescrtype_end(void *data, const char *el)
{
	MP7_VarDescrType_Userdata *usr_data_ptr = (MP7_VarDescrType_Userdata *)data;

	// Before leaving a VariationSet, if there were at least one
	// Source and one Variation in it
	if (strcmp("VariationSet", el) == 0) {
		if ((!(*usr_data_ptr).source) || (!(*usr_data_ptr).variation)) {
			(*usr_data_ptr).error = ERR__SOURCE_OR_VAR_DEF;
		}
		(*usr_data_ptr).source = false;
		(*usr_data_ptr).variation = false;
	}
	// Before leaving a VariationDescriptionType, reset the userdata
	if ((strcmp("Description", el) == 0) && (depth == VARDESCR_STARTLEVEL)) {
		(*usr_data_ptr).inside_vardescrtype = false;                    
	}

	depth--;
}


MP7Parser::MP7_TemporalDecomposition MP7GetTemporalDecomposition(FILE *fp, int which_variationset)
{
	char buf[BUFFSIZE];
	int numread, done;
	int lastdepth = -1;
	MP7Parser::MP7_VideoSegmentList *current_pointer = NULL;
	MP7Parser::MP7_VideoSegmentList *parent_pointer = NULL;
	MP7_CheckVideoSegments_Userdata checksegment_userdata;
	MP7Parser::MP7_TemporalDecomposition returnval;
	
	// init
	rewind(fp);
	depth = 0;
	checksegment_userdata.which_varset = which_variationset;

	// create parser
	XML_Parser p = XML_ParserCreate(NULL);
	if (!p) {
		returnval.errorcode = ERR__PARSER_CREATE_ERROR;
		return returnval;
	}

	// set parser to get VarDescrData
	XML_SetElementHandler(p, XMLHandler_CheckVideoSegments_start, XMLHandler_CheckVideoSegments_end);
	XML_SetUserData(p, &checksegment_userdata);

	while ((checksegment_userdata.error == ERR__OK) && !(done = feof(fp))) {
		numread = fread(buf, sizeof(char), BUFFSIZE, fp);
		if (ferror(fp)) {
			returnval.errorcode = ERR__READ_ERROR;
			XML_ParserFree(p);
			return returnval;
		}
		else {
			if (XML_Parse(p, buf, numread, done) == XML_STATUS_ERROR) {
				returnval.errorcode = ERR__PARSE_ERROR;
				XML_ParserFree(p);
				return returnval;
			}
		}
	}

	if (checksegment_userdata.varset_counter < which_variationset) {
		returnval.errorcode = ERR__NO_SUCH_VARSET;
		XML_ParserFree(p);
		return returnval;
	}

	if (checksegment_userdata.segment_counter == 0) {
		returnval.errorcode = ERR__NO_VERTDECOMP;
		XML_ParserFree(p);
		return returnval;
	}

	for (int i=1; i<=checksegment_userdata.segment_counter; i++) {
		
		// init
		XML_ParserFree(p);
		depth = 0;
		rewind(fp);
		MP7_GetVideoSegment_Userdata *getsegment_userdata = new MP7_GetVideoSegment_Userdata;
		getsegment_userdata->which_varset = which_variationset;
		getsegment_userdata->which_segment = i;

		// create parser
		p = XML_ParserCreate(NULL);
		if (!p) {
			returnval.errorcode = ERR__PARSER_CREATE_ERROR;
			return returnval;
		}

		// set parser to get VidSegData
		XML_SetElementHandler(p, XMLHandler_GetVideoSegment_start, XMLHandler_GetVideoSegment_end);
		XML_SetCharacterDataHandler(p, XMLHandler_GetVideoSegment_data);
		XML_SetUserData(p, getsegment_userdata);


		while ((getsegment_userdata->error == ERR__OK) && !(done = feof(fp))) {
			numread = fread(buf, sizeof(char), BUFFSIZE, fp);
			if (ferror(fp)) {
				returnval.errorcode = ERR__READ_ERROR;
				XML_ParserFree(p);
				return returnval;
			} else {
				if (XML_Parse(p, buf, numread, done) == XML_STATUS_ERROR) {
					returnval.errorcode = ERR__PARSE_ERROR;
					XML_ParserFree(p);
					return returnval;
				}
			}
		}
		
		MP7Parser::MP7_VideoSegmentList *newvsgl = new MP7Parser::MP7_VideoSegmentList;
		
		//first element
		if (returnval.vseg_list == NULL) {
			newvsgl->vseg_data = getsegment_userdata->vseg_data;
			returnval.vseg_list = newvsgl;
			current_pointer = newvsgl;
		}
		// new element, not first, same depth --> next
		else if (lastdepth == getsegment_userdata->segment_depth) {
			newvsgl->vseg_data = getsegment_userdata->vseg_data;
			current_pointer->next = newvsgl;
			current_pointer = newvsgl;
		}
		// new element, not first, depth is higher --> children
		else if (lastdepth < getsegment_userdata->segment_depth) {
			newvsgl->vseg_data = getsegment_userdata->vseg_data;
			parent_pointer = current_pointer;
			current_pointer->children = newvsgl;
			current_pointer = newvsgl;
		}
		// new element, not first, depth is lower --> parent
		else if (lastdepth > getsegment_userdata->segment_depth) {
			newvsgl->vseg_data = getsegment_userdata->vseg_data;
			current_pointer = parent_pointer;
			current_pointer->next = newvsgl;
			current_pointer = newvsgl;
		}
		lastdepth = getsegment_userdata->segment_depth;
		delete getsegment_userdata;		
	}
	XML_ParserFree(p);
	return returnval;
}

MP7Parser::MP7_Variations MP7GetVariations(FILE *fp, int which_variationset, int nr_variations)
{
	char buf[BUFFSIZE];
	int numread, done;
	MP7Parser::MP7_VarList *currentvl = NULL;
	MP7Parser::MP7_Variations returnval;
	XML_Parser p;
	
	for (int i=1;i<=nr_variations;i++) {

		// init
		depth = 0;
		rewind(fp);
		MP7_Variation_Userdata *udata = new MP7_Variation_Userdata;
		udata->which_varset = which_variationset;
		udata->which_variation = i;
		
		// create parser
		p = XML_ParserCreate(NULL);
		if (!p) {
			returnval.errorcode = ERR__PARSER_CREATE_ERROR;
			return returnval;
		}

		// set parser to get Variations
		XML_SetElementHandler(p, XMLHandler_mp7getvariation_start, XMLHandler_mp7getvariation_end);
		XML_SetCharacterDataHandler(p, XMLHandler_mp7getvariation_data);
		XML_SetUserData(p, udata);


		while ((udata->error == ERR__OK) && !(done = feof(fp))) {
			numread = fread(buf, sizeof(char), BUFFSIZE, fp);
			if (ferror(fp)) {
				returnval.errorcode = ERR__READ_ERROR;
				XML_ParserFree(p);
				return returnval;
			}
			else {
				if (XML_Parse(p, buf, numread, done) == XML_STATUS_ERROR) {
					returnval.errorcode = ERR__PARSE_ERROR;
					XML_ParserFree(p);
					return returnval;
				}
			}
		}
		
		MP7Parser::MP7_VarList *newvl = new MP7Parser::MP7_VarList;
		if (currentvl == NULL) {
			returnval.var_list = newvl;
			currentvl = newvl;
		}
		else {
			currentvl->next = newvl;
			currentvl = newvl;			
		}
		newvl->vdata = udata->var_data;
		returnval.errorcode = udata->error;
		delete udata;
	
		XML_ParserFree(p);
	}
	
	return returnval;
}

// checks if xml file has a valid VariationDescriptionType and returns
// all Variationsets (Sourcedata + Variations on that Source) found
MP7Parser::MP7_VarDescrData MP7Parser::MP7GetVarDescrData(FILE *fp)
{
	char buf[BUFFSIZE];
	int numread, done;
	MP7_VarDescrData returnval;
	MP7_VarDescrType_Userdata vardescr_userdata;
	MP7_VarSet_Userdata varset_userdata;
	MP7_VarSetList *tempvsl, *newvsl;
		
	// init
	depth = 0;
	rewind(fp);

	// create parser
	XML_Parser p = XML_ParserCreate(NULL);
	if (!p) {
		returnval.errorcode = ERR__PARSER_CREATE_ERROR;
		return returnval;
	}

	// set parser to get VarDescrData
	XML_SetElementHandler(p, XMLHandler_mp7getvardescrtype_start, XMLHandler_mp7getvardescrtype_end);
	XML_SetUserData(p, &vardescr_userdata);

	while ((vardescr_userdata.error == ERR__OK) && !(done = feof(fp))) {
		numread = fread(buf, sizeof(char), BUFFSIZE, fp);
		if (ferror(fp)) {
			returnval.errorcode = ERR__READ_ERROR;
			XML_ParserFree(p);
			return returnval;
		}
		else {
			if (XML_Parse(p, buf, numread, done) == XML_STATUS_ERROR) {
				returnval.errorcode = ERR__PARSE_ERROR;
				XML_ParserFree(p);
				return returnval;
			}
		}
	}

	if (!vardescr_userdata.vardescr) {
		returnval.errorcode = ERR__NO_VARDESCRTYPE;
		XML_ParserFree(p);
		return returnval;
	}

	returnval.errorcode = vardescr_userdata.error;
	returnval.nr_varsets = vardescr_userdata.varset_counter;
	
	// loop - get VarSetLists
	for (int i=1; i<=returnval.nr_varsets; i++) {

		// reset data
		XML_ParserFree(p);
		rewind(fp);
		depth = 0;
		MP7_VarSet_Userdata *udata = new MP7_VarSet_Userdata;
		udata->which_varset = i;

		// (re)create parser
		p = XML_ParserCreate(NULL);
		if (!p) {
			returnval.errorcode = ERR__PARSER_CREATE_ERROR;
			return returnval;
		}
		
		// set parser to get VarSets
		XML_SetElementHandler(p, XMLHandler_mp7getvarset_start, XMLHandler_mp7getvarset_end);
		XML_SetCharacterDataHandler(p, XMLHandler_mp7getvarset_data);
		XML_SetUserData(p, udata);
		
		newvsl = new MP7_VarSetList;
		if (returnval.vsl_pointer == NULL) {
			returnval.vsl_pointer = newvsl;
		}
		else {
			tempvsl = returnval.vsl_pointer;
			while (tempvsl->next != NULL) {
				tempvsl = tempvsl->next;
			}
			tempvsl->next = newvsl;
		}
		
		while ((udata->error == ERR__OK) && !(done = feof(fp))) {
			numread = fread(buf, sizeof(char), BUFFSIZE, fp);
			if (ferror(fp)) {
				returnval.errorcode = ERR__READ_ERROR;
				XML_ParserFree(p);
				return returnval;
			}
			else {
				if (XML_Parse(p, buf, numread, done) == XML_STATUS_ERROR) {
					returnval.errorcode = ERR__PARSE_ERROR;
					XML_ParserFree(p);
					return returnval;
				}
			}
		}
			
		// source data
		newvsl->sdata = udata->source_data;
		// variation data
		newvsl->vdata = MP7GetVariations(fp, i, udata->variation_counter);
		// creation information
		newvsl->mdata = udata->meta_data;
		// mediatime
		newvsl->mtime = udata->media_time;
		//get verticaldecomposition
		newvsl->tmpdec = MP7GetTemporalDecomposition(fp, i);
		
		newvsl->nr_variations = udata->variation_counter;
		returnval.errorcode = udata->error;
		delete udata;
	}

	XML_ParserFree(p);
	return returnval;
	
}
