 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	TerminalCapabilities.hpp                                      *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _MPEG4ITEC_TERMINALCAPABILITIES_HPP
#define _MPEG4ITEC_TERMINALCAPABILITIES_HPP
#ifdef ISOMP4
#include "ISOMovies.h"
#else
#include "global.hpp"
#endif

/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: TerminalCapabilities.hpp,v 1.2 2004/02/19 10:22:40 pschojer Exp $
 *
 *@short <short description>
 *
 */
class TerminalCapabilities {
public:
	/** inits all values to zero */
	TerminalCapabilities();
	/** constructor */
	TerminalCapabilities(u32 maxDecBufSize, u32 maxDecBitrate,
						u32 displayBits, bool color, 
						u32 width, u32 height, u32 refreshRate,
						float mem, u32 netCap, u32 minNetCap,
						u32 netDelayVar, u32 buffDelay,
						u8 numAudioChann=0,u8 audioBitsPerSample=16,u32 samplFreq=44100);
	u32 getMaxDecoderBufferSizeInByte() const;
	u32 getMaxDecoderBitRateInBit() const;
	u32 getDisplayBitsPerPixel() const;
	bool getColorDisplay() const;
	u32 getDisplayWidth() const;
	u32 getDisplayHeight() const;
	u32 getDisplayRefreshRate() const;
	float getMemoryStorageMB() const;
	u32 getNetworkCapacityInByte() const;
	u32 getMinGuaranteedNetworkCapacityInByte() const;
	u32 getNetworkDelayVariationInMs() const;
	u32 getBufferingDelayInMs() const;
	u8 getNumAudioChannels() const;
	bool supportsAudio() const;
	u8 getAudioBitsPerSample() const;
	u32 getAudioSamplingFrequency() const;
	
	void setMaxDecoderBufferSizeInByte(const u32);
	void setMaxDecoderBitRateInBit(const u32);
	void setDisplayBitsPerPixel(const u32);
	void setColorDisplay(const bool);
	void setDisplayWidth(const u32);
	void setDisplayHeight(const u32);
	void setDisplayRefreshRate(const u32);
	void setMemoryStorageMB(const float);
	void setNetworkCapacityInByte(const u32);
	void setMinGuaranteedNetworkCapacityInByte(const u32);
	void setNetworkDelayVariationInMs(const u32);
	void setBufferingDelayInMs(const u32);
	void setNumAudioChannels(const u8);
	void setAudioBitsPerSample(const u8);
	void setAudioSamplingFrequency(const u32);

	bool initFromFile(const char* file);
	bool saveToFile(const char* file, bool overWrite) const;
	TerminalCapabilities* clone() const;

protected:
	u32 maxDecoderBufferSizeInByte;
	u32 maxDecoderBitRateInBit;
	u32 displayBitsPerPixel;
	u32 colorDisplay;
	u32 displayWidth;
	u32 displayHeight;
	u32 displayRefreshRate;
	float memoryStorageMB;
	u32 networkCapacityInByte;
	u32 minGuaranteedNetworkCapacityInByte;
	u32 networkDelayVariationInMs;
	u32 bufferingDelayInMs;
	u8 numAudioChannels;
	u8 audioBitsPerSample;
	u32 audioSamplingFrequency;

};
	
#endif
