 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	UserPreferences.cpp                                           *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "UserPreferences.hpp"
#include "debug.hpp"
#include "TerminalCapabilities.hpp"

UserPreferences::UserPreferences(const TerminalCapabilities* tc):
	dimX(0,0,0,0.25),bitrate(0,0,0,0.25),delayInMS(2000,0.0),color(1,0.25),frameRate(0.0,0.0,0.0,0.25)
{
	printf("\033[32mXXXXXXXXXXXXXXXX UserPreferences THIS: %p XXXXXXXXXXX\n\033[0m", this );
	//FIXME.. don't have reliable delay value, so 0 importance
	assert(tc);
	int col= (int)(tc->getColorDisplay()?1:0);
	color.point=col;
	int x=tc->getDisplayWidth();
	dimX.best=x;
	dimX.max=x;
	int fr=tc->getDisplayRefreshRate();
	frameRate.best=(double)fr;
	frameRate.max=(double)fr;
	uint br = tc->getNetworkCapacityInByte()*8;
	if (br > tc->getMaxDecoderBitRateInBit())
		br=tc->getMaxDecoderBitRateInBit();
	bitrate.best=br;
	bitrate.max=br;
 
	//assume we have an incredible rich client
	//	double pay=FLT_MAX;

};

UserPreferences::UserPreferences(const UserPreferences* s):dimX(s->dimX),bitrate(s->bitrate),
	delayInMS(s->delayInMS),color(s->color),frameRate(s->frameRate)
{
	pay=s->pay;
}
/** note that the importance values of all FeatureRanges/-points must sum up to 1.0! */
UserPreferences::UserPreferences(const FeatureRange& dX,const FeatureRange& br, 
	const FeaturePoint& delay, const FeaturePoint& col, const dFeatureRange& fr,const double& pa):
	dimX(dX),bitrate(br),delayInMS(delay),color(col),frameRate(fr)
{
	pay=pa;
	double total=dimX.importance+bitrate.importance+delayInMS.importance+color.importance+frameRate.importance;
	assert(total>0.999 && total<1.001);
};	
	/** note that the importance values of all FeatureRanges/-points must sum up to 1.0! */
UserPreferences::UserPreferences(const FeatureRange* dX, const FeatureRange* br, 
																 const FeaturePoint* delay, const FeaturePoint* col, const dFeatureRange* fr,const double& pa):
	dimX(dX),bitrate(br),delayInMS(delay),color(col),frameRate(fr)
{
	pay=pa;
	double total=dimX.importance+bitrate.importance+delayInMS.importance+color.importance+frameRate.importance;
	assert(total>0.999 && total<1.001);
}
double UserPreferences::calcQuality(const Feature* target, int delay) const
{
	double result=0.0;
	double temp=0.0;
	if(dimX.importance>0.0) {
		temp=dimX.calcQuality(target->dimX);
		if(temp<=0.0)
			return 0.0;
		result+=temp;
	}
	if(color.importance>0.0) {
		temp=color.calcPointQuality(target->color);
		if(temp<=0.0)
			return 0.0;
		result+=temp;
	}
	if(bitrate.importance>0.0) {
		temp=bitrate.calcQuality(target->bitrate);
		if(temp<=0.0)
			return 0.0;
		result+=temp;
	}
	if(frameRate.importance>0.0) {
		temp=frameRate.calcQuality(target->frameRate);
		if(temp<=0.0)
			return 0.0;
		result+=temp;
	}
	if(delayInMS.importance>0.0) {
		temp=delayInMS.calcInverseQuality(delay);
		if(temp<=0.0)
			return 0.0;
		result+=temp;
	}	
	return result;
};	

UserPreferences* UserPreferences::create(FILE* fp)
{
	if(!fp)
		return NULL;
	char *temp = new char[4960];
	if(fp==stdin) {
		printf("\ndimx(min,medium,max)/importance bitrate(m,m,m)/i delay(m)/i color(i)/i framerate(double,d,d)/i pay double: ");
		fflush(stdout);
	}
	fgets(temp,4096,fp);
	
	UserPreferences *up = UserPreferences::create(temp);
	delete [] temp;
	return up;	
};
char* UserPreferences::toString() const{
	char* res=new char[1024];
	char *t1=dimX.toString();
	char *t2=bitrate.toString();
	char *t3=delayInMS.toString();
	char *t4=color.toString();
	char *t5=frameRate.toString();
	sprintf(res,"dimx%s bitrate%s delay%s color%s framerate%s pay%lf",t1,t2,t3,t4,t5,pay);
	delete t1; delete t2; delete t3; delete t4; delete t5;
	return res;
};

UserPreferences* UserPreferences::create(const char* buf)
{
	const char* tmp=NULL;
	FeatureRange *dimX=NULL, *bitrate=NULL;
	dFeatureRange *frameRate=NULL;
	FeaturePoint *delayInMS=NULL, *color=NULL;
	double pay=0.0;
	double sum=0.0;
	bool err=false;
	UserPreferences* up=NULL;
	tmp=strstr(buf,"dimx");
	if(tmp) {
		tmp+=strlen("dimx");
		dimX=FeatureRange::create(tmp);
		sum+=dimX->importance;
	}
	tmp=strstr(buf,"bitrate");
	if(tmp) {
		tmp+=strlen("bitrate");
		bitrate=FeatureRange::create(tmp);
		sum+=bitrate->importance;
	}
	tmp=strstr(buf,"delay");
	if(tmp) {
		tmp+=strlen("delay");
		delayInMS=FeaturePoint::create(tmp);
		sum+=delayInMS->importance;
	}
	tmp=strstr(buf,"color");
	if(tmp) {
		tmp+=strlen("color");
		color=FeaturePoint::create(tmp);
		sum+=color->importance;
	}
	tmp=strstr(buf,"framerate");
	if(tmp) {
		tmp+=strlen("framerate");
		frameRate=dFeatureRange::create(tmp);
		sum+=frameRate->importance;
	}
	tmp=strstr(buf,"pay");
	if(tmp) {
		tmp+=strlen("pay");
		err=(1!=sscanf(tmp,"%lf",&pay));
	}
	if(sum>1.001 || sum < 0.999) {
		dprintf_err("\nWeights in UserPreferences do not sum up to 1.0\n");
		err=true;
	}
	if(dimX && bitrate && frameRate && delayInMS && color && !err) {
		up= new UserPreferences(dimX,bitrate,delayInMS,color,frameRate,pay);	
	}
	if(dimX)
		delete dimX;
	if(bitrate)
		delete bitrate;
	if(delayInMS)
		delete delayInMS;
	if(frameRate)
		delete frameRate;
	if(color)
		delete color;
	return up;
}

void UserPreferences::doTest()
{
	FeatureRange dimX(0,176,352,0.2), bitrate(333,666,999,0.4);
	dFeatureRange frameRate(15.0,17.0,20.0,0.0);
	FeaturePoint delayInMS(1000,0.3), color(1,0.1);
	UserPreferences up1(dimX,bitrate,delayInMS,color,frameRate,99);
	char* up1s=up1.toString();
	assert(up1s);
	UserPreferences* up2=UserPreferences::create(up1s);
	assert(up2);
	char* up2s=up2->toString();
	assert(strcmp(up1s,up2s)==0);
	delete up1s; delete up2; delete up2s;
}


void UserPreferences::correctBitRate(int dimX,int dimY) 
{ 
	FeatureRange* br=&this->bitrate; 
	static const int bitrates[]={1,2,4,8}; 
	static const int numEntries=sizeof(bitrates)/sizeof(int); 
	int keyval[numEntries]; 
	int i;
	for(i=0;i<numEntries;i++)
	{ 
		keyval[i]=dimX*dimY*bitrates[i]; 
	} 
	dprintf_full("UserPreferences::correctBitRate %i,%i\n",dimX,dimY);
// check if one of these bitrates is included in the range 
	for(i=0;i<numEntries;i++)
	{ 
		dprintf_full("UserPreferences::correctBitRate Key %i min%i--max%i\n",keyval[i],br->min,br->max);
		if(keyval[i]>=br->min && keyval[i]<=br->max) 
			return; 
	} 
// we come that far --> invalid bitraterange 
// check if too small 
	if(br->min>keyval[0]) 
		br->min= ((keyval[0]-1000)>0?(keyval[0]-1000):0); 
// too large? 
	if(br->max<keyval[3])
	{ 
		br->max=keyval[3]+1000; 
	}

}

void UserPreferences::correctDimension() 
{ 
	FeatureRange* dimX=&this->dimX; 
// min and max swapped? 
	if(dimX->min>dimX->max)
	{ 
		//		int t=dimX->min; 
		dimX->min=dimX->max; 
		dimX->max=dimX->min; 
	}
	
	int diff=dimX->max-dimX->min; 
	if(diff<44)
	{ 
		diff=44-diff+1; // +1 for div 2 
		diff/=2; 
		dimX->max+=diff; 
		dimX->min-=diff; 
	} 

}
