

 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	ClientSession.hpp                                             *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _MPEG4ITEC_CLIENTSESSION
#define _MPEG4ITEC_CLIENTSESSION

#ifdef ISOMP4
#include "ISOMovies.h"
#endif

#include "global.hpp"
#include "Protocol.hpp"
#include "ContainerInfo.hpp"
#include "PortGenerator.hpp"
#include "../metadata/TerminalCapabilities.hpp"
#include "../metadata/UserPreferences.hpp"
#include "../cache/CacheManager.hpp"

struct rtx_group;
struct rtx_info;
class DataChannel;
class DataSink;
class RTSP;


/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: ClientSession.hpp,v 1.17 2006/01/20 15:37:53 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class ClientSession: public VThread {
  public:
	enum ClientSessionState { SESSION_ERR, SESSION_CLOSED, SESSION_CONNECTED, SESSION_INITIALIZED, SESSION_PLAYING, SESSION_PAUSED };

	/** creates a socket to the given server. If the url is invalid or anyhing
	  * goes wrong, the state is set to SESSION_ERR. Otherwise the state is set
	  * to SESSION_CLOSED */
	ClientSession(const char* uri, IO* videoOutput, IO* audioOutput,
		TerminalCapabilities* tc,bool enableRtx,bool visualize=false);
     
	ClientSession(const char* uri, IO* vout, IO* aout, UserPreferences* up, 
		bool enableRtx,bool visualize=false);
	
	ClientSession(const char* videoUri, const char* proxyUri, IO* vout, IO* aout, 
		UserPreferences* up, bool enableRtx,bool visualize=false);

	/** socket: the raw socket for initializing the session control channel.
		* the server info is deep-copied */   
	ClientSession(int socket, const struct sockaddr_in *server, const char* uri, 
		IO* vout, IO* aout, TerminalCapabilities* c, bool enableRtx,bool visualize=false);

	void construct(const char* uri);

	
  ~ClientSession();
  /** writes the string to the session control channel */
  bool sendResponse(const char *str, int bytes);
	/** reads at most @param MAX_BYTES-1 bytes from a socket and
	sets '\0' at the end of the string.
	This method is blocking, it stores
	the read data in @param str, @param return contains the
	bytes read. returns a negative value in case of error, returns
	0 if the client closed the socket */
	int readRequest(char* str, const int MAX_BYTES);
  bool connect();
  bool play( bool with_caching=false);
  bool pause(u32 sessionKey);
  bool tearDown(u32 sessionKey);
  void run();
  /** deep-copies uri, sets mp4Stream by asking the cacheManager
   * @param makeExactMatch decides on possible server-side transcoding */
	void setUrl(const char *&uri, bool makeExactMatch = true);
  //void setPort(int port);
  const ContainerInfo *getContainerInfo() const {
		return mp4Stream;
  };
  const list < DataChannel * >&getDataChannels() const {
		return channels;
  };
  /** searches an existing DataChannel within this session for the given @param esId.
    * NULL is returned if no DC exists */
  DataChannel *getDataChannel(int esId);
  /** simply appends @param dc to the list of DataChannels. No checking is done,
    * if an entry already exists for the ElementaryStream */
	void setDataChannel(DataChannel * dc) {
		dprintf_full("\033[31mAdded DataChannel !\033[0m\n");
		channels.push_front(dc);
		}
		
  ClientSessionState getState() const {
	return state;
  };
  const char *getUrl() const {
	return url;
  };
  int getControlSocket() const {
	return sessionControlChannel;
  };
  sockaddr_in *getDestInfo() {
	return server;
  };
	/** sets output at the client */
  void setOutput(IO * out);

	/** creates a rtx_group for the given input params */
	bool setRtxInfoForES(u32 esId, rtx_info* rtx);
	/** returns an rtx_group for the esId, or NULL */
	rtx_group* getRtxGroup(u32 esId);

private:
  /** searches for an ES its DataChannel. returns NULL on failure */
  DataChannel * find(int esId);
  /** returns the known protocol or  PROTO_UNKNOWN */
  KnownProtocols determineProtocol(char *request);
	
protected:
	/** the url of the source/destination file */
	char *url;
	
	/** the control channel. Supported protocol is currently RTSP
	 * might add http later if required 
	 */
	int sessionControlChannel;
	/** the List of all DataChannels that this Session manages.
	 * 1 DataChannel = 1 ES 
	 */
	list<DataChannel*> channels;
	/** Information about the video */
	ContainerInfo *mp4Stream;
	/** the current state of the video */
	ClientSessionState state;
	/** The protocol used to parse requests and generate replies.
			FIXME: we hard-code to RTSP, redesign needed for http */
	RTSP *protocol;
	sockaddr_in *server;
	/** buffers the termCap from the response, and then passes
	 * it to the client which is created during SETUP */
	TerminalCapabilities* termCap;

	UserPreferences* usrPref;

	/** the client object needed to render the received data to the screen */
	std::map<u32,DataSink*,struct intCompare> clientObj;
	
	// FIXME: for each DC one output needed!
	IO* videoOut;
	IO* audioOut;
	/** stores for each ES an optional retransmission wish */
	list < rtx_group* > rtxInfo;
	bool rtxEnabled;
	bool decodeData;
};
#endif

