/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title: HttpRequest.hpp                                               *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

/***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PS_HTTPREQUEST_HPP_
#define _PS_HTTPREQUEST_HPP_

/**
 *
 *@author Klaus Schoeffmann
 *
 */


#include "global.hpp"
#include "net/Url.hpp"
#include "net/HttpServer.hpp"


#ifndef WIN32
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/socket.h>
#include <sys/types.h> 
#endif


class HttpRequest 
{
public:
	HttpRequest(const char *url);
	~HttpRequest();

	/* connects to the Http server and parses the header, content length */
	bool makeHttpRequest();

	/* receives data bytes into the given @param recbuffer
	 * take care on your own to call this several times, until it @returns -1 */
	int continueReceive(char *recbuffer, int size);

	int getSock() { return sock; };

	int getContentLength() { return contentLength; };

	char* getMimeType() { return HttpMimeTypeStrings[contentType]; };

	HttpMimeType getMimeTypeEnum() { return contentType; };

	void closeSock();


private:
	Url *uri;

	int sock;
	int contentLength;
	HttpMimeType contentType;
	char *headerbuf;
	int bufend;
	char *contentStart;
	

	bool createSocket(const char *host, int port);

	bool isIPNumber(const char *host);

};



#endif
