/***********************************************************************
*                                                                      *
* ViTooKi                                                              *
*                                                                      *
* title: HttpServer.hpp                                                *
*                                                                      *
*                                                                      *
*                                                                      *
* ITEC institute of the University of Klagenfurt (Austria)             *
* http://www.itec.uni-klu.ac.at                                        *
*                                                                      *
*                                                                      *
* For more information visit the ViTooKi homepage:                     *
* http://ViTooKi.sourceforge.net                                       *
* vitooki-user@lists.sourceforge.net                                   *
* vitooki-devel@lists.sourceforge.net                                  *
*                                                                      *
* This file is part of ViTooKi, a free video toolkit.                  *
* ViTooKi is free software; you can redistribute it and/or             *
* modify it under the terms of the GNU General Public License          *
* as published by the Free Software Foundation; either version 2       *
* of the License, or (at your option) any later version.               *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
* MA  02111-1307, USA.                                                 *
*                                                                      *
***********************************************************************/

/***********************************************************************
*                                                                      *
*  REVISION HISTORY:                                                   *
*                                                                      *
*                                                                      *
*                                                                      *
***********************************************************************/

/***********************************************************************
* Author: Klaus Schoeffmann                                            *
*         original code by Michael Kropfberger (from Mohican :)        *
************************************************************************/


#ifndef __HttpServer_hpp__
#define __HttpServer_hpp__

#ifndef WIN32
#include <stdio.h>
#include <sys/time.h>
#include <time.h>
#include <string.h>
#include <linux/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <strings.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/types.h> 
#include <sys/wait.h> 
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#endif

#include "VThread.hpp"


#define METHOD_GET 0
#define METHOD_POST 1

#define MAX_LEN 4096
#define VERSION "HTTP/1.0"
#define OK "200 OK"
#define NO_CONTENT "204 No Content"
#define NOT_FOUND "404 Not Found"
#define NOT_IMPL "501 Not Implemented"

#define DEFAULT_PAGE "index.html"
#define ERR404_PAGE  "404.html"


	typedef enum {mime_unknown=0,mime_html,mime_plain,mime_gif,mime_jpg,mime_mp4,mime_vit,mime_ogg,mime_mkv} HttpMimeType;
#define NUM_KNOWN_HTTP_MIMETYPES 9
	static char *HttpMimeTypeStrings[NUM_KNOWN_HTTP_MIMETYPES] = {"application/octet-stream","text/html","text/plain",
																												 "image/gif","image/jpg",
																												 "video/mpeg","video/x-vit","video/x-ogg","video/x-mkv"};


class HttpServer : public VThread {
public:


  /** default constructor 
      @param port port on which the HTTP server should be started
      @param rootdir root directory of HTTP server
      @param cgidir directory containing cgi binaries (must be a sub-directory of rootdir)
  */
  HttpServer(int port = 8080, const char *rootdir = ".", const char *cgidir = "/cgi-bin/");
  /** default destructor */
  ~HttpServer();	
  
  
  /** start this thread */
  void run();  
  
  /** stop this thread */
  void stop();
  
private:
  /** socket identifier used by webserver */
  int sock;
  /** port number to use */	
  int port; 
  /** root directory */
  char *rootdir;
  /** directory containing cgi binaries */
  char *cgidir;
  /** flag: true while thread is running */
  bool active;
  /** flag: while this flag is true, the main loop will be repeated*/
  bool shouldBeActive;
  
  /** send specified file to client 
      @param uri file to send
      @param sock socket to use for transmission
  */
  int sendResource(int sock, char *uri);
  
  /** run a CGI program
      @param uri program to start
      @param sock socket to use for I/O
      @param method either GET or POST
      @param body
      @param body_len
  */
  int runCGI(int sock, char *uri, int method, char *body, int body_len);
  
  /** find a specified extension in a URI
      @param uri haystack
      @param ext needle
  */
  char *findExtension(char *uri,char *ext);
  
};


#endif
