 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	PacketizationLayer.cpp                                        *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "PacketizationLayer.hpp"

PacketizationLayer::PacketizationLayer():packetSeqNumber(0) {
}


PacketizationLayer::~PacketizationLayer() {
}


SLPacket * PacketizationLayer::createSLPacket(u8 * payload, size_t size, bool unused1, u32 unused2) {
	return new SLPacket(SLPacketHeader(packetSeqNumber++), payload, size);
};


/** splits an AU up in SLPackets */
SLPacketList PacketizationLayer::createSLPacketList(AU * au) {
	SLPacketList slpl;
	// copied and adapted from MOH
	u32 unitSize = au->size;
	/*frag_offset is RFC2250 specific info 
	 * It is irrelevant for RFC3016 */
	if (au->size <= getMaxPacketPayloadSize()) {
		SLPacket *packet = createSLPacket(au->payload, unitSize, false, 0);
		SLPacketHeader & slph = packet->getSLPacketHeader();
		slph.accessUnitStartFlag = true;
		slph.accessUnitEndFlag = true;
		
		// FIXME: how to know if this is true?
		slph.randomAccessPointFlag = true;
		slph.decodingTimeStampFlag = true;
		slph.decodingTimeStamp = au->dts;
		slph.compositionTimeStamp = au->cts;
		slph.compositionTimeStampFlag = true;
		slpl.push_back(packet);	
		//      pLayer->sendSLPackets(slpl);
	} else {
		u32 maxsize = getMaxPacketPayloadSize();
		u8 * payload = au->payload;
		bool first = true;
		int currentSize = 0;
		while (unitSize) {
			if (unitSize <= maxsize) {	// last packet
				currentSize = unitSize;
			} else {		// first or intermediate packet       
				currentSize = maxsize;
			}
			SLPacket * packet = createSLPacket(payload, currentSize, true, 0);
			SLPacketHeader & slph = packet->getSLPacketHeader();
			if (unitSize <= maxsize) {	// last packet
				slph.accessUnitEndFlag = true;
				slph.accessUnitStartFlag = false;
				slph.randomAccessPointFlag = false;
				slph.decodingTimeStampFlag = false;
				slph.compositionTimeStampFlag = false;
			}
			else { // first or intermediate packet       
				if (first) {
					slph.accessUnitStartFlag = true;
					
					// FIXME: how to know if this is true?
					slph.randomAccessPointFlag = true;
					slph.decodingTimeStampFlag = true;
					slph.compositionTimeStampFlag = true;
					first = false;
				}
				else { // intermediate packet
					slph.accessUnitStartFlag = false;
					slph.randomAccessPointFlag = false;
					slph.decodingTimeStampFlag = false;
					slph.compositionTimeStampFlag = false;
				}
				slph.accessUnitEndFlag = false;
			}
			slph.decodingTimeStamp = au->dts;
			slph.compositionTimeStamp = au->cts;
			slpl.push_back(packet);
			
			// pLayer->sendSLPackets(slpl);
			unitSize -= currentSize;
			payload += currentSize;
		} // while unitSize
	}
	return slpl;
}
