 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	PortGenerator.cpp                                             *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef WINCE
#include <sys/types.h>
#endif
#ifndef WIN32
#include <sys/socket.h>
#include <sys/select.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#endif

#ifdef WIN32
#include <winsock2.h>
#endif


#include <stdlib.h>
#include <string.h>

#include "PortGenerator.hpp"



PortGenerator::PortGenerator() {
	initialize();
}


PortGenerator::~PortGenerator() {
}



/***************************************************************/
void PortGenerator::initialize() {
	struct timeval tv;

	gettimeofday(&tv,0);
	srand(tv.tv_usec);
	memset(UDPsocketOf,0,sizeof(int)*PORT_RANGE);
	memset(TCPsocketOf,0,sizeof(int)*PORT_RANGE);
}


/***************************************************************/
int PortGenerator::setSockAndPort(int port, int proto) {
  struct sockaddr_in sin;
  int s;
  bool isUDP = (proto==PROT_UDP);
	if(port<PORT_OFFS || port>=(PORT_OFFS+PORT_RANGE)) //ignore this
		return 0;

	if ((s = socket(PF_INET, isUDP?SOCK_DGRAM:SOCK_STREAM, 0)) < 0) {
		dprintf_err("PortGenerator::getNextFreePort: socket");
		exit(1);
	}

	/* Build address data structure */
	memset((char *)&sin,0,sizeof(sin));
	
	sin.sin_family = AF_INET;
	sin.sin_addr.s_addr = INADDR_ANY;
	sin.sin_port = htons(port);

	//find a systemwide unassigned port
	if (bind(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
#ifdef WIN32
		::closesocket(s);
#else
		close(s);
#endif
		return 0;
	}

	isUDP?(UDPsocketOf[port-PORT_OFFS] = s):(TCPsocketOf[port-PORT_OFFS] = s);
	
	return s;
}

/***************************************************************/
int PortGenerator::getNextFreePortPair(int proto) {
  int port;
  int s1,s2;
  bool isUDP = (proto==PROT_UDP);
	int ret;


	do {
		//find unassigned port from this PortGenerator
		do {
			port = (rand()%(PORT_RANGE/2)) * 2; //find an even port
		} while ( (isUDP? (UDPsocketOf[port]!=0):(TCPsocketOf[port]!=0)) ||
							(isUDP? (UDPsocketOf[port+1]!=0):(TCPsocketOf[port+1]!=0)));			
		port+=PORT_OFFS;
		if ( (ret = s1 = setSockAndPort(port,proto)) != 0) {
			ret = s2 = setSockAndPort(port+1,proto);
			if (ret == 0) { //odd port failed
				closePort(port,proto); //close even port
			}
		}
		
	} while (ret == 0);

	dprintf_full("PortGenerator::getNextFreePort bound port %i and %i to sock %i and %i\n",port,port+1,s1,s2);


  return port;
}


/***************************************************************/
bool PortGenerator::closePort(int port, int proto) {
  bool isUDP = (proto==PROT_UDP);
	if(port<PORT_OFFS || port>=(PORT_OFFS+PORT_RANGE)) //ignore this
		return true;

	if ((isUDP?UDPsocketOf[port-PORT_OFFS]:TCPsocketOf[port-PORT_OFFS]) != 0) {
#ifdef WIN32
		::closesocket(isUDP?UDPsocketOf[port-PORT_OFFS]:TCPsocketOf[port-PORT_OFFS]);
#else
		close(isUDP?UDPsocketOf[port-PORT_OFFS]:TCPsocketOf[port-PORT_OFFS]);
#endif
		isUDP?UDPsocketOf[port-PORT_OFFS]=0:TCPsocketOf[port-PORT_OFFS]=0;
		return true;
	}

	return false;
}


/***************************************************************/
bool PortGenerator::closePortPair(int port, int proto) {
	return (closePort(port,proto) & closePort(port+1,proto));
}


/***************************************************************/
int PortGenerator::getSockFromPort(int port, int proto) {

 bool isUDP = (proto==PROT_UDP);
	if(port<PORT_OFFS || port>=(PORT_OFFS+PORT_RANGE)) //ignore this
		return 0;

	return (isUDP?UDPsocketOf[port-PORT_OFFS]:TCPsocketOf[port-PORT_OFFS]);
}
