 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	PortGenerator.hpp                                             *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _MK_PORTGENERATOR_HPP_
#define _MK_PORTGENERATOR_HPP_

#include "global.hpp"

#define PROT_UDP 1
#define PROT_TCP 0
#define PORT_OFFS 19998
#define PORT_RANGE 20000

/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: PortGenerator.hpp,v 1.2 2004/03/16 17:15:13 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class PortGenerator {

 public:
	PortGenerator();
	virtual ~ PortGenerator();

	/* randomly finds two consequtive udp ports for eg. RTSP handshake usage.
	 * first port is even, second is first_port+1, so its always odd 
	 * @returns the first and even port number
	 */
	int getNextFreePortPair(int proto=PROT_UDP);


	/* use this to request a certain single port to be socket()ized and bind()
	 * @returns the socket fd on success or zero
	 */
	int setSockAndPort(int port, int proto=PROT_UDP);

	/* @returns the socket fd on success or zero */
	int getSockFromPort(int port, int proto=PROT_UDP);

	/* closes the port and socket
	 * @returns true on success or false else */
	bool closePort(int port, int proto=PROT_UDP);

	/* closes the consequtive port pair and according sockets
	 * @returns true on success or false else */
	bool closePortPair(int port, int proto=PROT_UDP);
  
protected:
	void initialize();

	int UDPsocketOf[PORT_RANGE];
	int TCPsocketOf[PORT_RANGE];
};

#endif
