 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	RFC2250.cpp                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#include "RFC2250.hpp"
#define MAX_RTP_PAYLOAD_SIZE (1488-2-4)	//1488 is maximum, but on RTX, we need 2 bytes for real rtpSeqNo
		                        //and 4 bytes is for RFC2250 audio specific header


RFC2250::RFC2250():PacketizationLayer() {
	payload_buffer = new u8[MAX_RTP_PAYLOAD_SIZE];
	payload_buffer_size = 0;
	buffer_flag = false;
	first = true;
	new_packet = true;
	au_dts = 0;
	au_cts = 0;
}


RFC2250::~RFC2250() {
	if(payload_buffer)
		delete[] payload_buffer;
}


SLPacket * RFC2250::createSLPacket(u8 * payload, size_t size, bool fragment, u32 frag_offset) {
	
	if(first) {
		fragment = true;
		first = false;
		return NULL; // FIXME do we need to send the header in an RTP packet ?? I don't think so!
	}
	if(fragment) {
		if(payload_buffer_size == 0) { //means that buffer is empty, just send act. payload
			buffer_flag = false;
			return new SLPacket(SLPacketHeader(packetSeqNumber++), payload, size, &frag_offset, 4);
		}
		else { //send old payload, then store the act. one
			SLPacket *slp = new SLPacket(SLPacketHeader(packetSeqNumber++), payload_buffer, 
																	 payload_buffer_size, &frag_offset, 4);
			memcpy(payload_buffer, payload, size);
			payload_buffer_size = size;
			buffer_flag = true;
			return slp;
		}
	}
	else {
		if(buffer_flag) {
			SLPacket *slp = new SLPacket(SLPacketHeader(packetSeqNumber++), payload_buffer,
																	 payload_buffer_size, &frag_offset, 4);
			memcpy(payload_buffer, payload, size);
			payload_buffer_size = size;
			buffer_flag = false;
			return slp;
		}
		else {
			if((size+payload_buffer_size) < getMaxPacketPayloadSize()) { //cumulative fill-up of MTU
				memcpy(payload_buffer+payload_buffer_size, payload, size);
				payload_buffer_size += size;
				return NULL;
			}
			else if (payload_buffer_size > 0) {
				SLPacket *slp = new SLPacket(SLPacketHeader(packetSeqNumber++), payload_buffer, 
																		 payload_buffer_size, &frag_offset, 4);
				memcpy(payload_buffer, payload, size);
				payload_buffer_size = size;
				return slp;
			}
		}
	}
	return new SLPacket(SLPacketHeader(packetSeqNumber++), payload, size, &frag_offset, 4);
}
	

/** splits an AU up in SLPackets */
SLPacketList RFC2250::createSLPacketList(AU * au) {
	SLPacketList slpl;
	// copied and adapted from MOH
	u32 unitSize = au->size;
	/*frag_offset is RFC2250 specific info 
	 * It is irrelevant for RFC3016 */
	u32 frag_offset = 0;

	if (au->size <= getMaxPacketPayloadSize()) {
		dprintf_full(" RFC2250::createSLPacketList creates single SLPacket of size %i\n", unitSize);
		SLPacket *packet = createSLPacket(au->payload, unitSize, false, frag_offset);
		if(new_packet){
			au_dts = au->dts;
			au_cts = au->cts;
			new_packet = false;
		} 
		if(packet) {
			SLPacketHeader & slph = packet->getSLPacketHeader();
			slph.accessUnitStartFlag = true;
			slph.accessUnitEndFlag = true;
      
			// FIXME: how to know if this is true?
			slph.randomAccessPointFlag = true;
			slph.decodingTimeStampFlag = true;
			slph.decodingTimeStamp = au_dts;
			slph.compositionTimeStamp = au_cts;
			slph.compositionTimeStampFlag = true;
			slpl.push_back(packet); //add this packet
			new_packet = true;
		}
    
		//      pLayer->sendSLPackets(slpl);
	} else {
		u32 maxsize = getMaxPacketPayloadSize();
		u8 * payload = au->payload;
		bool first = true;
		int currentSize = 0;
		while (unitSize) {
			if (unitSize <= maxsize) {      // last packet
				currentSize = unitSize;
			} else {                // first or intermediate packet       
				currentSize = maxsize;
			}
			SLPacket * packet = createSLPacket(payload, currentSize, true, frag_offset);
			frag_offset += currentSize;
			SLPacketHeader & slph = packet->getSLPacketHeader();
			if (unitSize <= maxsize) {      // last packet
				slph.accessUnitEndFlag = true;
				slph.accessUnitStartFlag = false;
				slph.randomAccessPointFlag = false;
				slph.decodingTimeStampFlag = false;
				slph.compositionTimeStampFlag = false;
			}
			else { // first or intermediate packet       
				if (first) {
					slph.accessUnitStartFlag = true;
          
					// FIXME: how to know if this is true?
					slph.randomAccessPointFlag = true;
					slph.decodingTimeStampFlag = true;
					slph.compositionTimeStampFlag = true;
					first = false;
				}
				else { // intermediate packet
					slph.accessUnitStartFlag = false;
					slph.randomAccessPointFlag = false;
					slph.decodingTimeStampFlag = false;
					slph.compositionTimeStampFlag = false;
				}
				slph.accessUnitEndFlag = false;
			}
			slph.decodingTimeStamp = au->dts;
			slph.compositionTimeStamp = au->cts;
			slpl.push_back(packet);
			
			// pLayer->sendSLPackets(slpl);
			unitSize -= currentSize;
			payload += currentSize;
		} // while unitSize
	}
	return slpl;
}


						

u32 RFC2250::getMaxPacketPayloadSize()
{
	return MAX_RTP_PAYLOAD_SIZE; 
};

u32 RFC2250::getPacketizationType()
{
	return 2250;
}
