 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	RFC2250.hpp                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PS_RFC2250_HPP
#define _PS_RFC2250_HPP

#include "PacketizationLayer.hpp"

/**
 *
 * Implementation of RFC 2250 for packetization of MP3 audio 
 *
 *@author Mithilesh Kumar
 *
 *@version $Id: RFC2250.hpp,v 1.5 2008/11/24 16:34:59 mtaschwer Exp $
 *
 *@short <short description>
 *
 */
class RFC2250:public PacketizationLayer {
public:
	RFC2250();

	~RFC2250();

	SLPacketList createSLPacketList(AU * au);

	/** creates one single SLPacket. Buffering may be done to pack smaller frames
	 * into single RTP payload. In this case NULL is returned
	 * @param payload specified by @param size 
	 * @param fragment is for determining if the payload is to be fragmented
	 * @param frag_offset is for determining the offset into the fragmented audio frame
	 */
	SLPacket * createSLPacket(u8 * payload, size_t size, bool fragment, u32 frag_offset);

	u32 getMaxPacketPayloadSize();

	u32 getPacketizationType();

private:

	/* this is needed for buffering packets when several packet could be packed 
	 * into one RTP payload
	 */     
	u8 * payload_buffer;
	u32 payload_buffer_size;
	bool first;
	bool buffer_flag;

	/** these are for internal use */
	u32 au_dts;
	u32 au_cts;
	bool new_packet;
	
};

#endif
