 
/***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	RTSP.hpp                                                      *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PS_PROXY_RTSP
#define _PS_PROXY_RTSP
#include "Session.hpp"
#include "Protocol.hpp"
#include "../metadata/TerminalCapabilities.hpp"
#include "../metadata/UserPreferences.hpp"

#define INVALID_SESSIONID 0xFFFFFFFF

#ifndef TEARDOWN_ALL
#define TEARDOWN_ALL INVALID_SESSIONID
#endif

class ContainerInfo;
struct rtx_group;
struct rtx_info;
class ESInfo;
class Url;

/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: RTSP.hpp,v 1.15 2006/01/20 15:37:54 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class RTSP:public Protocol {

public:
	RTSP();

	virtual ~RTSP() {};

	bool applyReqToSession(const char *req, u32 bytesRead, Session * ses);
	/**  wrapper method for applyReqToSession() */
	bool applyReqToSession(const char *req, u32 bytesRead, Session * ses, bool optionsCallback);
	
	/** parses the response generated by a DESCRIBE (the SDP description)
	 * and builds a ContainerInfo object from this info.
	 */ 
	ContainerInfo * parseResponseDescribe(const char *response, const char *url, char** serverName,
																				list<rtx_group*> *groups=NULL);
	
	/** parses a reponse to a setup, returns the number of parameters
	 * successfully extracted. Every return value below two should be
	 * considered as an illegal reponse.
	 * @param buffer is an IN-param, rest are OUT-params
	 */
	int parseResponseSetup(const char* buffer, int *sessionID, int *clientPort,
												 int *serverPort,bool *unicast, char** ssrc,
												 rtx_info* info=NULL);
	void generateDescribeForUrl(const char* url, const char* playerId, const TerminalCapabilities* tc=NULL,const UserPreferences* up=NULL);
	/** @param rtxPort specifies the client port used for retransmission, set to 0 to disable retransmisson,
	 * @param sessionId should be  -1 if you want the server to generate a sessionid
	 */
	void generateSetup(const char* url, int trackId, int clientPort,bool unicast,u32 rtxPort=0,int sessionId=-1);
	void generatePlay(const char* url, u32 session, double startTime, double nptEnd,double maxPrefetch);
	void generatePause(const char* url, u32 session);
	void generateTeardown(const char* url, u32 session);
#ifndef WINCE
	/** @param url contains the requested file
	 * @param newUrl contains the new url that the client should use for the request
	 * @param redirectAllowedAtTimePoint contains the time when the client is allowed to 
	 * send the request to the new location. This is an optional parameter.
	 */
	void generateRedirect(const char* url, const char* newUrl, const time_t* redirectAllowedAtTimePoint=NULL);
#endif
	void generateFileNotFound();
	void generateOk();
	void generateSessionNotFound();
	void generateInternalServerError();
	void generateDestUnreachable();
	void generateNotImplemented();
	void generateBadCMD();
	void generateIllegalTransport();
	void generateNotEnoughBandwidth();
	void generateInvalidParameter(const int numParams, const char* const * const param);
	void generateSetParameter(const char* url, u32 session,
														std::map<char*, char*, struct stringCompare>* params);
	void generateGetParameter(const char* url, const u32 session,
														const int numParams, const char** params);
	
	/** parses a GET_PARAMETER @param req and extracts the list of parameters specified in the request
	 * @param numParams contains the number of parameters requested, or 0 on error
	 * @param session contains the session identifier, or 0 on error
	 * @param params contains the strings of parameters requested: (*params)[0] will contain the first
	 */
	void parseGetParameter(const char* req, u32* session,int* numParams, char*** params);
	/** will parse a SET_PARAMETER @param req, writes into @param session the session identifier,
	 * and writes into (*params)[parameterkey] the pointer to a string.
	 * You MUST free the map memory with Rtsp::freeMapMemory
	 */
	void parseSetParameter(const char* req, u32* session,std::map<char*, char*, struct stringCompare>* params);
  bool parseCMDDescribe(Session * ses, const Url *fileName,
												const char *remaining, char** playerId);
  bool parseCMDSetup(Session * ses, const Url *fileName,
										 const char * const remaining, bool* hasSessionKey, int* sesKey,
										 char** protocols, char** unicast,int* esId,int* rPort, int* rPort2,
										 bool* hasRtxPort, u32* clientRtxPort, char **clientAddress = NULL);
	bool parseCMDPlay(Session * ses, const Url *filename,
										const char *remaining, int* sessionId, double* startTime, 
										double* endTime, double* maxPrefetch);
#ifndef WINCE
	/** newUrl and redirectAllowedAtTimePoint are out parameters. Memory for newUrl is reserved
	 * inside the function. Make sure that *newUrl=NULL prior calling
	 */
	void parseCMDRedirect(Session* ses, const Url *fileName, const char* remaining, 
												char** newUrl, time_t* redirectAllowedAtTimePoint);
#endif
	void generateResponseGetParameter(std::map<char*, char*, struct stringCompare>* params);
	bool generateCMDDescribeReply(Session * ses, const Url *fileName, 
																const char *sourceServerName, const char* playerId);
	bool generateSetupReply(Session * ses, const Url *fileName,
													const int sesKey, const char* protocols, const char* unicast, const char* sourceAddress,
													const int esId, const int rPort, const int rPort2, const int lport,
													const u32 clientRtxPort, const u32 serverRtxPort);		
  void generateCMDnotSupportedReply();
  void generateCMDOptionsReply();
	
	
	void incSeqNr() { seqNr++;};
	void decSeqNr() { if(seqNr>1) seqNr--; };
	static bool isSupportedTransport(const char* protocols);
	static bool isCmdOk(const char* buf);
	
	int getLastStatus() const { return lastStatus;};
	const char* getBuffer() const { return buffer;};
	int getDefaultPort() const { return 554;};
	/* parses @param buffer for a sessionid returns INVALID_SESSIONID
	 * if none is found 
	 */
	static u32 extractSessionId(const char* buffer);
	/* searches for the sessionId in @param buffer
	 * if none is found @param return is NULL, otherwise a new string
	 * is created with the new sessionid 
	 */
	static char* replaceSessionIdWith(u32 newId,const char* buffer);
	static void doTest();
	static void freeMapMemory(std::map<char*,char*,struct stringCompare>* params);

	int generateSessionKey();
	/** returns INVALID_SESSIONID if no sessionid was found */
	int extractSessionKeyFromCMD(const char* remaining);
	/** returns last sequence number */
	int getSeqNr() { return seqNr; };
	
	
	/** returns the sessionId which will be terminated */
  //int generateCMDTeardown(Session * ses, const Url *filename, const char *remaining);
	//	void handleCMDPause(Session * ses, const Url *filename, const char *remaining);

	
protected:
	enum MSG_Type
		{ MSG_DESCRIBE, MSG_SETUP, MSG_PLAY, MSG_PAUSE, MSG_OPTIONS,
			MSG_TEARDOWN, GET_PARAMETER, SET_PARAMETER,	MSG_REDIRECT, MSG_UNKNOWN
    };

	int seqNr;
	int lastStatus;
	
	char buffer[MSG_BUFFER_SIZE];
	static const char *const allowedCommands;
	static const int SL_PAYLOAD_TYPE;
	static const int TTL;
	
  bool optionsCallback; //should options-callback be used?
  

private:
	ESInfo* parseSDPMediaBlock(	const char* mediaStart,
															const char* mediaEnd,
															const double durSeconds,
															ContainerInfo * mp4Info,
															list<rtx_group*> *groups=NULL);
	void parseSDP_RTXBlock(const char* media,
												 const char* endOfMedia, 
												 list<rtx_group*> *groups);

  bool parseRequest(const char *request, int len, MSG_Type * msgtype,
										Url **filename, int *cseq, const char **remaining);
	
  bool handleSingleRequest(Session * ses, const char *buffer, int bytesRead);
};

#endif
