 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	SDP.hpp                                                       *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef PS_MPEG4ITEC_SDP
#define PS_MPEG4ITEC_SDP
class ContainerInfo;
class TerminalCapabilities;
struct rtx_group;

#ifndef ISOMP4
#include "global.hpp"
#else
#include "ISOMovies.h"
#endif

/**
 *
 * <Class Description goes here>
 *
 *@author Peter Schojer
 *
 *@version $Id: SDP.hpp,v 1.5 2006/01/20 15:37:54 mkropfbe Exp $
 *
 *@short RTSP helper class for SDP generation needed as response to an RTSP DESCRIBE
 *
 */
class SDP 
{
public:
  struct session {
    std::string name;
    std::string url;
    double nptStart;
    double nptEnd;

    session() {
      nptStart = 0.0;
      nptEnd = 0.0;		// means "end of media"
    }
  };

  /** Default constructor. */
  SDP() {}

  /** Destructor. */
  ~SDP();

  /** Parse an SDP description. The field values may then be
   * retrieved using other instance methods of this class.
   * <p>Currently, only the following SDP fields are understood, 
   * all other occuring fields are ignored:</p>
   * <ul>
   * <li><code>v=0</code> required to start a valid SDP description.</li>
   * <li><code>s=_session_name_</code> session name; required to start a session description.</li>
   * <li><code>a=control:_URL_</code> control URL.</li>
   * <li><code>a=range:npt=_start_-_end_</code> media presentation time range in seconds
   *     (floating point).</li>
   * </ul>
   * <p><b>Warning:</b> This function is <em>not</em> thread-safe!</p>
   * @return <code>false</code> if the given string does not contain a valid
   *         SDP description.
   */
  bool parse(const char *sdp);

  /** Return the fields of the first session of this SDP description
   * instance.
   * @return a pointer to the struct of the first session, or
   *         <code>NULL</code> if there are no sessions.
   * @see #parse(const char *)
   */
  const struct session *firstSession() const;

  /** generates a (S)ession (D)escription (P)rotocol string for the given
   * ContainerInfo object. Will succeed on objects without any elementary
   * streams!
   */
  // MultiSLStreamConfig &mslc replaced by
  // sizeLength + dtsDeltaLength
  static char *generate(ContainerInfo* stream, 	bool perfectHit, const char *filename, const char *localAddr,
			int SL_PAYLOAD_TYPE, int sizeLength,
			int dtsDeltaLength, const TerminalCapabilities* tc=NULL,
			bool includeRtxInfo=false,
			std::list<rtx_group*>* groups=NULL);

  /** encodes the source-decoder config into dest, which must be of size 2n+1 */
  static void encodeDecoderConfig(u8 * dest, const u8 * source, u32 n);

  /** decodes the encoded-decoderConfig stored in source into dest 
   * which must be of size srcSize/2 */
  static void decodeDecoderConfig(u8 * dest, const u8 * source, u32 srcSize);

private:
  /** Clear all instance data. */
  void clear();
  list<struct session *> sessionList;
};	

#endif


