 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	ServerSession.hpp                                             *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _PS_SERVER_SESSION
#define _PS_SERVER_SESSION

#include "global.hpp"

#ifdef WIN32
#pragma warning(disable:4786)
#endif

#include "Session.hpp"
#include "Protocol.hpp"
#include "ContainerInfo.hpp"
#include "../cache/CacheManager.hpp"
#include "../SessionCounter.hpp"
#include "Statistics.hpp"
#ifdef ISOMP4
#include "ISOMovies.h"
#endif

struct rtx_group;
class DataChannel;
class DataSink;
class IO;
class Url;
class AdmissionControl;

#include "PortGenerator.hpp"

#include "../metadata/TerminalCapabilities.hpp"


/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: ServerSession.hpp,v 1.13 2006/01/20 15:37:54 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class ServerSession:public Session {

public:

	/** socket: the raw socket for initializing the session control channel.
		* the client info is deep-copied */
	ServerSession(int socket, const struct sockaddr_in *client, CacheManager * cm,
		AdmissionControl* am,bool enableRtx,SessionCounter* counter=NULL,TerminalCapabilities* tc=NULL);

	~ServerSession();

  bool options(const Url* fileName, const char* remaining);
	bool connect(const Url* fileName, const char* remaining);
	bool setup(const Url* fileName, const char* remaining);
	bool play(const Url* fileName, const char* remaining);
	bool pause(const Url* fileName, const char* remaining);
	bool tearDown(int sessionKey, bool immediate=false, const Url* fileName=NULL, const char* remaining=NULL);
	bool getOptions(const Url* fileName, const char* remaining);
	bool setOptions(const Url* fileName, const char* remaining);
	void run();

	/** deep-copies uri, sets mp4Stream by asking the cacheManager
	 * @param makeExactMatch decides on possible server-side transcoding */
	void setUrl(const Url *uri, bool makeExactMatch = true);
  
	SessionState getState() const {
		return state;
	};

	/** sets output at the client */
//	void setOutput(IO * out);
	
protected:
	char buffer[MSG_BUFFER_SIZE];
//	DataSink * clientObj;
	CacheManager *cm;
	char* localAddress;
	char* playerId;
	//std::map<char*, char*, struct stringCompare> params;
	/** stores to an session id a map of params */
	std::map<u32,std::map<char*, char*, struct stringCompare>*, struct intCompare> params;

	bool startDCs(int sessionId, double startTime, double endTime, 
								double prefetch, bool backwardsClosestIFrame);

	AdmissionControl* admControl;
	bool enableRtx;
	SessionCounter* counter;
};
#endif
