 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	Session.hpp                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _SESSION_HPP_
#define _SESSION_HPP_
#ifdef ISOMP4
#include "ISOMovies.h"
#endif

#include "Protocol.hpp"
#include "adaptors/GlobalTimer.hpp"
#include "adaptors/SharedAdaptor.hpp"
#include "ReferenceCounter.hpp"
#include "cache/CostFunction.hpp"

// Forward declarations
class TerminalCapabilities;
class UserPreferences;
class ContainerInfo;
struct rtx_group;
struct rtx_info;
class DataChannel;
class Adaptor;
class IO;
class Url;
class Rtp;
class RTSP;
/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: Session.hpp,v 1.16 2006/01/20 15:37:54 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class Session: public VThread
{
public:
	enum SessionState {SESSION_ERR, SESSION_NEW, SESSION_INITIALIZED, SESSION_ACTIVE, SESSION_PAUSED, SESSION_CLOSED};
	Session();
	virtual ~Session();
  virtual bool options(const Url* fileName, const char* remaining) = 0;
	virtual bool connect(const Url* fileName, const char* remaining)=0;
	virtual bool setup(const Url* fileName, const char* remaining)=0;
	virtual bool play(const Url* fileName, const char* remaining)=0;
	virtual bool pause(const Url* fileName, const char* remaining)=0;
	virtual bool getOptions(const Url* fileName, const char* remaining)=0;
	virtual bool setOptions(const Url* fileName, const char* remaining)=0;
	/** teardowns the session specified with sessionKey (if no fileName and no reminaing is specified.
	  * Otherwise remaining will be parsed for the sessionkey, and the first param is ignored */
	virtual bool tearDown(int sessionKey, bool immediate=false,const Url* fileName=NULL, const char* remaining=NULL)=0;
	/** deep-copies uri, sets mp4Stream by asking the cacheManager
	 * @param makeExactMatch decides on possible server-side transcoding */
	virtual void setUrl(const Url *uri, bool makeExactMatch = true)=0;
	const Url* getUrl() const;
	/** creates a rtx_group for the given input params */
	bool setRtxInfoForES(u32 esId, rtx_info* rtx);

	/** returns an rtx_group for the esId, or NULL */
	rtx_group* getRtxGroup(u32 esId);

	std::list < rtx_group* >* getAllRtxGroups();

  const struct sockaddr_in *getDataSinkInfo() const;

	/** simply appends @param dc to the list of DataChannels. No checking is done,
		* if an entry already exists for the ElementaryStream */
	void setDataChannel(DataChannel * dc);
	Adaptor* getSuggestedAdaptor();

	const std::list < DataChannel * >& getDataChannels() const;
	
	/** searches an existing DataChannel within this session for the given @param esId.
		* NULL is returned if no DC exists */
	DataChannel *getDataChannel(int esId);

	ContainerInfo *getContainerInfo() const;

	void setTerminalCapabilities(TerminalCapabilities* tc);

	const TerminalCapabilities* getTerminalCapabilities() const;

	int getControlSocket() const;

	/** writes the string to the session control channel */
	bool sendResponse(const char *str,int bytes);

	/** reads at most @param MAX_BYTES-1 bytes from a socket and
	sets '\0' at the end of the string.
	This method is blocking, it stores
	the read data in @param str, @param return contains the
	bytes read. returns a negative value in case of error, returns
	0 if the client closed the socket */
	int readRequest(char* str, const int MAX_BYTES);
	RTSP* getProtocol() {return prot;};
	/** returns the known protocol or  PROTO_UNKNOWN */
	KnownProtocols determineProtocol(const char *request);

protected:
	/** searches for an ES its DataChannel. returns NULL on failure */
	DataChannel * find(int esId);


  GlobalTimer *globalTimer;
	ReferenceCounter *gtRefCount;
	SharedAdaptor *gtSharedAdapt;
	ResourceUsage resUsage;

	int sessionControlChannel;
	SessionState state;
	TerminalCapabilities* termCap;
	UserPreferences* userPref;
	ContainerInfo* mp4Stream;
	/** stores for each ES an optional retransmission wish */
	list < rtx_group* > rtxInfo;
	/** the List of all DataChannels that this Session manages.
		* 1 DataChannel = 1 ES 
		*/
	list < DataChannel * >channels;
	Adaptor* suggestedAdaptor;
	sockaddr_in* client;
	u32 clientId;
	RTSP* prot;
	/** the url of the source/destination file */
	Url* url;
	char* serverID;
	bool canAdmitRequest;
};

#endif
