 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	SyncLayer.hpp                                                 *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

/****************************************************************
 Project: MPEG-4 Environment
 Purpose: Define SL Stream interface
 Author:  M. Ohlenroth
 Date:    $Id: SyncLayer.hpp,v 1.5 2006/01/20 15:37:54 mkropfbe Exp $
 ****************************************************************/


#ifndef _SyncLayer_HPP_
#define _SyncLayer_HPP_

//#include <stdint.h>
#ifndef WINCE
#include <sys/types.h>
#endif

#ifdef ISOMP4
#include "ISOMovies.h"
#else
#include "global.hpp"
#endif


/* SLConfigDescriptor as defined in 14496-1 paragraph 10.2.3.1
 */
struct SLConfigDescriptor {
	u8 predefined;
	bool useAccessUnitStartFlag;
	bool useAccessUnitEndFlag;
	bool useRandomAccessPointFlag;
	bool hasRandomAccessUnitsOnlyFlag;
	bool usePaddingFlag;
	bool useTimeStampsFlag;
	bool useIdleFlag;
	bool durationFlag;
	u32 timeStampResolution;
	u32 OCRResolution;
	u8 timeStampLength;
	u8 OCRLength;
	u8 AU_Length;
	u8 instantBitrateLength;
	u8 degradationPriorityLength;
	u8 AU_seqNumLength;
	u8 packetSeqNumLength;
	u8 reserved;		// = 0b11
	// if (durationFlag)
	u32 timeScale;
	u16 accessUnitDuration;
	u16 compositionUnitDuration;
	// }
	// if (!useTimeStampsFlag) {
	u64 startDecodingTimeStamp;
	u64 startCompositionTimeStamp;
	// }
};

// predefined SLConfigDescriptor for RFC 3016 mode:
extern const SLConfigDescriptor slcd_rfc_3016;

typedef struct SLPacketHeader {
	SLPacketHeader() {};

	SLPacketHeader(const u16 seq) : packetSequenceNumber(seq) {
		accessUnitStartFlag = accessUnitEndFlag =
	    OCRFlag = idleFlag = paddingFlag = false;
		paddingBits = 0;
		packetSequenceNumber = 0;
		DegPrioFlag = false;
		degradationPriority = 0;
		objectClockReference = 0;
		randomAccessPointFlag = false;
		AU_sequenceNumber = 0;
		decodingTimeStampFlag =
	    compositionTimeStampFlag = instantBitrateFlag = false;
		decodingTimeStamp = compositionTimeStamp =
	    accessUnitLength = instantBitrate = false;
	};
	SLPacketHeader(const SLPacketHeader & Aslph) {
		*this = Aslph;
	};
	
	bool accessUnitStartFlag:1;
	bool accessUnitEndFlag:1;
	bool OCRFlag;
	bool idleFlag;
	bool paddingFlag;
	u8 paddingBits;
	u16 packetSequenceNumber;
	bool DegPrioFlag;
	u16 degradationPriority;
	u64 objectClockReference;
	
	bool randomAccessPointFlag;
	u16 AU_sequenceNumber;
	bool decodingTimeStampFlag;
	bool compositionTimeStampFlag;
	bool instantBitrateFlag;
	u64 decodingTimeStamp;
	u64 compositionTimeStamp;
	u32 accessUnitLength;
	u64 instantBitrate;		// could be up to 256 bits!
} SLPacketHeader;


bool operator == (SLPacketHeader const &first,
		  SLPacketHeader const &second);

/**
 *
 * <Class Description goes here>
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: SyncLayer.hpp,v 1.5 2006/01/20 15:37:54 mkropfbe Exp $
 *
 *@short <short description>
 *
 */
class SLPacket {
public:
	SLPacket(SLPacketHeader Aslph, u8 * Apayload = 0, size_t Apl_size = 0,
					 void * extra_data = NULL, u32 extra_data_size = 0);

	virtual ~ SLPacket() {
		if (payload)
	    delete payload;
	};

	size_t getPayloadSize() {
		return pl_size;
	};

	u8 *getPayload() {
		return payload;
	}
	
  /** sets the payload to NULL but doesnt free it! 
   */
	u8 *unsetPayload() {
		u8 *dummy = payload;
		payload = NULL;
		pl_size = 0;
		return dummy;
	}
	
  /** deletes the payload and frees memory
      @returns false if no payload was set
	*/
	bool deletePayload() {
		if (!payload)
			return false;
		delete payload;
		payload=NULL;
		pl_size = 0;
		return true;
	}
	
	SLPacketHeader & getSLPacketHeader() {
		return slph;
	};

protected:
	SLPacketHeader slph;
	u8 *payload;
	size_t pl_size;
};

#define SLPacketList list<SLPacket*>
#define SLPacketHeaderList list<SLPacketHeader>

//
///* receiver class must be implemented by upper layer
// */
//class SLStreamReceiver {
//      protected:
//              // receive callback
//              virtual void receiveSLPackets(SLPacketList slpl) = 0;
//};

#endif
