 /***********************************************************************
 *                                                                      *
 * ViTooKi                                                              *
 *                                                                      *
 * title:	Url.hpp                                                       *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 * ITEC institute of the University of Klagenfurt (Austria)             *
 * http://www.itec.uni-klu.ac.at                                        *
 *                                                                      *
 *                                                                      *
 * For more information visit the ViTooKi homepage:                     *
 * http://ViTooKi.sourceforge.net                                       *
 * vitooki-user@lists.sourceforge.net                                   *
 * vitooki-devel@lists.sourceforge.net                                  *
 *                                                                      *
 * This file is part of ViTooKi, a free video toolkit.                  *
 * ViTooKi is free software; you can redistribute it and/or             *
 * modify it under the terms of the GNU General Public License          *
 * as published by the Free Software Foundation; either version 2       *
 * of the License, or (at your option) any later version.               *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,               *
 * MA  02111-1307, USA.                                                 *
 *                                                                      *
 ***********************************************************************/

 /***********************************************************************
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *                                                                      *
 *                                                                      *
 *                                                                      *
 ***********************************************************************/

#ifndef _MPEG4ITEC_URL_HPP
#define _MPEG4ITEC_URL_HPP
/**
 * URL Class, works with file urls, and also
 * with rtsp or http.
 * Note that NO RELATIVE PATHS are allowed in a file url, this must be
 * a full path in the form of e:\ or e:/ or /path
 * An Url is composed of several fields:
 * protocol://serverName[:port][/][path]
 * port and path are optional. If no protocol is specified, file:// is assumed
 *
 *@author Michael Kropfberger and Peter Schojer
 *
 *@version $Id: Url.hpp,v 1.10 2006/02/24 13:24:39 mkropfbe Exp $
 *
 *@short <short description>
 *
 */

class Url
{
public:	
	Url(const char* url);
	~Url();
	
	int getPort() const;

	void setPort(int port);

	/** returns the path of the url incl. a filename or NULL if none
	was found */
	const char* getPath() const;

	/* returns the filename of the path without any directory
	 * e.g.: will return for file://localhost/test.mp4 --> test.mp4
	 */
	const char* getFileName() const;

	const char* getServerName() const;

	/** returns the protocol without the ://, e.g. "file", "rtsp" */
	const char* getProtocol() const;

	/** builds an url. If the url has no path, it is guaranteed 
	that the url will end with / (e.g.: "rtsp://127.0.0.1:3128/" */
	const char* toString() const;

	/** an url is equal if servername and path are equal 
	  @param ignoreServerName is useful when we have a server */
	bool isEqual(const Url* url, bool ignoreServerName=false) const;

	bool hasServerName() const;

	bool hasPort() const;

	bool hasPath() const;

	bool hasProtocol() const;

	static char* transformUrlToLocalFile(const Url*);

	static char* transformPathToLocalFile(const Url*);

	/** converts an url to an already connected UDP socket */
	static bool convertUrlToSocket(const char* serverName, const int port,
																 struct sockaddr_in *outSockStruct, int &outSocket);

	/** automatically tests Url class for bugs, throws assertion if any is found */
	static void doTest();


private:
	void makeUrl();

	void handleLocalFile(const char* &locFile);

	void toLowerString(char* str);

	char* serverName;
	char* path;
	char* protocol;	
	char* url;
	int port;
};

#endif
